package TelosV2Pkg;
import java.io.*;
import javax.swing.JOptionPane;

public class APR {

	// Declarations
	private static FileReader input;
	public static String arrayRawData[];
	public static String arrayRawDataParsed[][];
	
	private static int stepNumber = 0;
	private static int lineLinesPerStep = 10;
	private static int lineStepType = 1;
	private static int lineLiquid = 2;
	private static int lineLabwareLocation = 3;
	private static int lineAspirationData = 4;
	private static int lineDestinationData = 5;
	private static int lineSourceSeq = 6;
	private static int lineDestSeq = 7;
	private static int lineVolume = 8;
	private static int lineQuadrants = 9;
	
	
	//Method
	public static void main(String[] args) {

	}

	public static String CfgLineRead (int stepNumber, int lineNumber){
		//returns a string equal to the entire line indicated by lineNumber
		String lineReturn;
		int stepLine = stepNumber*lineLinesPerStep+lineNumber;
		lineReturn = arrayRawData[stepLine];
		return lineReturn;
	}
	
	public static String[] CfgLineRead_Parsed (int stepNumber, int lineNumber){
		//returns an array. Each element containing one field parsed from line lineNumber
		String lineReturn[];
		int stepLine = stepNumber*lineLinesPerStep+lineNumber;
		lineReturn = arrayRawDataParsed[stepLine];
		return lineReturn;
	}
	
	public static String[][] Read (String filepath, String delimiter){
		//Reads native .apr file
		//Populates two class public arrays
		//     1..arrayRawData - Single dimension array.  Each element is one line from file
		//     2..arrayRawDataParsed - Multi Dimension array.  Each element is a single parsed record
		//Also returns the parsed data as Multi Dimensional array
		//Also determines the total number of steps (stepNumber) in the .apr
		//
		//Notes:
		//	Step 0 of .apr and associated element 0 of data arrays is reserved for Protocol global data
		//  Therefore, Step 1 and associated element 1 of data arrays if the first actual 'step' in the protocol.
		
		
			File f = new File (filepath);
			String arrayFileData[] = new String[1000];
			String arrayFileDataParsed[][]= new String[1000][100];
			String newline;
			int cntr = 0;
			
			try{
				input = new FileReader(f);
				BufferedReader br = new BufferedReader (input);
				
				while ((newline = br.readLine())!=null){
					arrayFileData[cntr]=newline;
					arrayFileDataParsed[cntr]= newline.split(delimiter);	
					cntr=cntr+1;
				}
				input.close();
			} catch (Exception e){
				JOptionPane.showMessageDialog(null, e);
			} finally {}
			
			arrayRawData = arrayFileData;
			arrayRawDataParsed = arrayFileDataParsed;
			
			//total number of steps.  Can be accessed by StepNumberTotal method
			stepNumber = (cntr / 10) - 1;
			return arrayFileDataParsed;
		}
	
	public static void Write (String filepathOut, String arrayOutput[][], String delimeter){
		
		String arrayOutputLine[] = new String[arrayOutput.length];
		int cntrWriteLn;
		for (int Step = 1; Step < arrayOutput.length; Step++){	
			cntrWriteLn = 0;
			String OutputString = "";
			while (arrayOutput[Step][cntrWriteLn] != null){
	    		OutputString = OutputString + arrayOutput[Step][cntrWriteLn] + delimeter;
	    		cntrWriteLn++;		
			}
			arrayOutputLine[Step] = OutputString;

		}
		
		try {
		      PrintStream out = new PrintStream(new FileOutputStream(filepathOut, true));
		      out.println("VAS");
		      for (int i = 1; i < arrayOutputLine.length; i++){
		    	 if (i==arrayOutputLine.length-1){
		    		 out.print(arrayOutputLine[i]);
		    	 }else {out.println(arrayOutputLine[i]);}
		    	 
		      }
		        
		      out.close();

		    } catch (FileNotFoundException e) {
		      e.printStackTrace();
		    }

	}
	
	  public static String LayoutFile()
	  {
		  int stepLine = 0;
		  String lineReturn = arrayRawData[stepLine];
		  return lineReturn;
	  }
	  
	public static int StepNumberTotal(){
		return stepNumber;
	}
	
	public static String StepType (int stepNumber){
		//returns a string equal to the step type
		String lineReturn;
		int stepLine = stepNumber*lineLinesPerStep+lineStepType;
		lineReturn = arrayRawData[stepLine];
		return lineReturn;
	}
	
	public static String Liquid (int stepNumber){
		//returns a string equal to the Liquid type
		String lineReturn;
		int stepLine = stepNumber*lineLinesPerStep+lineLiquid;
		lineReturn = arrayRawData[stepLine];
		return lineReturn;
	}
	
	public static String[] LiquidParsed (int stepNumber){
		//returns an array. Each element containing one Liquid ([source]  [Destination])
		String lineReturn[];
		int stepLine = stepNumber*lineLinesPerStep+lineLiquid;
		lineReturn = arrayRawDataParsed[stepLine];
		return lineReturn;
	}
	
	public static String LabwareLocation (int stepNumber){
		//returns an string containing the LabwareLocation line
		String lineReturn;
		int stepLine = stepNumber*lineLinesPerStep+lineLabwareLocation;
		lineReturn = arrayRawData[stepLine];
		return lineReturn;
	}
	
	public static String[] LabwareLocationParsed (int stepNumber){
		//returns an array. Each element containing record of the labware location line
		String lineReturn[];
		int stepLine = stepNumber*lineLinesPerStep+lineLabwareLocation;
		lineReturn = arrayRawDataParsed[stepLine];
		return lineReturn;
	}
	
	public static String AspirateData (int stepNumber){
		//returns a string containing the delimited Aspiration Data.
		String lineReturn;
		int stepLine = stepNumber*lineLinesPerStep+lineAspirationData;
		lineReturn = arrayRawData[stepLine];
		return lineReturn;
	}
	
	public static String[] AspirateDataParsed (int stepNumber){
		//returns an array. Each element containing one record of the Aspiration Data line
		String lineReturn[];
		int stepLine = stepNumber*lineLinesPerStep+lineAspirationData;
		lineReturn = arrayRawDataParsed[stepLine];
		return lineReturn;
	}
	
	public static String DispenseData (int stepNumber){
		//returns a string containing the delimited line for the Dispense Data.
		String lineReturn;
		int stepLine = stepNumber*lineLinesPerStep+lineDestinationData;
		lineReturn = arrayRawData[stepLine];
		return lineReturn;
	}
	
	public static String[] DispenseDataParsed (int stepNumber){
		//returns an array. Each element containing one record of the Dispense Data line.
		String lineReturn[];
		int stepLine = stepNumber*lineLinesPerStep+lineDestinationData;
		lineReturn = arrayRawDataParsed[stepLine];
		return lineReturn;
	}
	
	public static String SourcePlateArray(int stepNumber){
		// returns a value indicating the number of wells (24,96,384,1536 etc.)
		String PlateArrayType = "";
		
		if (APR.AspirateDataParsed(stepNumber)[1].endsWith("_96")){
			PlateArrayType = "96";
		}
		
		return PlateArrayType;
	}
	
	public static String SourceSeq (int stepNumber){
		//returns string containing the delimited positions of the source sequence line.
		String seqReturn;
		int stepLine = stepNumber*lineLinesPerStep+lineSourceSeq;
		seqReturn = arrayRawData[stepLine];
		return seqReturn;
	}

	public static String[] SourceSeqParsed (int stepNumber){
		//returns an array. Each element containing one position of the sequence
		String seqReturn[];
		int stepLine = stepNumber*lineLinesPerStep+lineSourceSeq;
		seqReturn = arrayRawDataParsed[stepLine];
		return seqReturn;
	}

	public static String DestSeq (int stepNumber){
		//returns string containing the delimited positions of the source sequence line.
		String seqReturn;
		int stepLine = stepNumber*lineLinesPerStep+lineDestSeq;
		seqReturn = arrayRawData[stepLine];
		return seqReturn;
	}
	
	public static String[] DestSeqParsed (int stepNumber){
		//returns an array. Each element containing one position of the sequence
		String seqReturn[];
		int stepLine = stepNumber*lineLinesPerStep+lineDestSeq;
		seqReturn = arrayRawDataParsed[stepLine];
		return seqReturn;
	}

	public static String Volume (int stepNumber){
		//returns a string containing the delimited records from the Volume line.
		String seqReturn;
		int stepLine = stepNumber*lineLinesPerStep+lineVolume;
		seqReturn = arrayRawData[stepLine];
		return seqReturn;
	}
	
	public static String[] VolumeParsed (int stepNumber){
		//returns an array. Each element containing one record of the volume line.
		String seqReturn[];
		int stepLine = stepNumber*lineLinesPerStep+lineVolume;
		seqReturn = arrayRawDataParsed[stepLine];
		return seqReturn;
	}

	public static String Quadrants (int stepNumber){
		//returns a string containing the delimited records of the Quadrants line.
		String seqReturn;
		int stepLine = stepNumber*lineLinesPerStep+lineQuadrants;
		seqReturn = arrayRawData[stepLine];
		return seqReturn;
	}
	
	public static String[] QuadrantsParsed (int stepNumber){
		//returns an array. Each element containing one record of the Quadrants line.
		String seqReturn[];
		int stepLine = stepNumber*lineLinesPerStep+lineQuadrants;
		seqReturn = arrayRawDataParsed[stepLine];
		return seqReturn;
	}

}

