	/**
	 * Copyright 	- Telos Scientific - 01/10/2010
	 * Author 		- Robin S Campbell
	 * Description	- Dilution window for Serial Step  (Auto Pilot 1.0)
	 * 
	 * Updates - 09/04/13 - JC - SD1.4 -Window Closing Event - Close window button triggers user confirmed close without saving
	 * 
	 */

package TelosV2Pkg;

import java.awt.Color;
import java.awt.GridLayout;
import java.awt.FlowLayout;

import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;

import java.awt.BorderLayout;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JFrame;

import java.awt.GridBagLayout;
import java.awt.GridLayout;

import javax.swing.JCheckBox;
import javax.swing.JButton;

import java.awt.Dimension;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.WindowFocusListener;


public class DilWindow extends JFrame {

	private static final long serialVersionUID = 1L;
    private Icon iconDown = new ImageIcon("C:/Program Files/TELOS/AutoPilot/Config/fill_down.png");
	SerialSchemePanel schemePanel = new SerialSchemePanel();
	private JPanel jContentPane = null;
	private JPanel dilPanel = null;
	private JPanel okPanel = null;
	private JButton okButton = null ;
	private JButton pasteButton = null ;
	public static int windowWidth = 300;
	public static int rowsTotal = 96;

	/**
	 * @param args
	 */
	public static void main(String[] args) {

		try  
		{  
		  //Tell the UIManager to use the platform look and feel  
		  UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());  
		}  
		catch(Exception e)  
		{  
		  //Do nothing  
		}  
		
		// TODO Auto-generated method stub
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				DilWindow thisClass = new DilWindow();
				thisClass.setVisible(true);
			}
		});
	}
	
	//Custom WindowAdapter for Exit Button
	WindowAdapter exitListener = new WindowAdapter() {
        @Override
        
        public void windowClosing(WindowEvent e) {
            int confirm = JOptionPane.showOptionDialog(null, "Exit without saving changes to Scheme?", "Exit Confirmation", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE, null, null, null);
            if (confirm == 0) {
            	hideWindow();
            }
        }
    };
    
	/**
	 * This is the default constructor
	 */
	public DilWindow() {
		super();
		initialize();
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		this.addWindowListener(exitListener);
		this.setSize(windowWidth, 457);
		this.setResizable(false);
		this.setFocusable(true);
		this.setTitle("   Dilution Scheme   ");
		this.setContentPane(getJContentPane());	
	
		this.addWindowFocusListener(new WindowFocusListener(){

			@Override
			public void windowGainedFocus(WindowEvent arg0) {	
			}

			@Override
			public void windowLostFocus(WindowEvent arg0) {
				okButton.grabFocus();
			}
		});
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			jContentPane = new JPanel();
			jContentPane.setLayout(new BorderLayout());
			jContentPane.add(getDilPanel(), BorderLayout.CENTER);
			jContentPane.add(getOKPanel(), BorderLayout.SOUTH);
		}
		return jContentPane;
	}

	private JPanel getDilPanel() {
		if (dilPanel == null) {
			GridLayout gridLayout = new GridLayout();
			gridLayout.setRows(1);
			gridLayout.setColumns(1);
			dilPanel = new JPanel();
			dilPanel.setLayout(gridLayout);
			dilPanel.add(schemePanel);
		 }
		return dilPanel;
	}

	private JPanel getOKPanel() {
		if (okPanel == null) {
			FlowLayout flow = new FlowLayout();
			okPanel = new JPanel();
			okPanel.setLayout(flow);
            okPanel.add(getOKButton());
            okPanel.add(getpasteButton());
		 }
		return okPanel;
	}

	private javax.swing.JButton getOKButton() {
		if (okButton == null) {
			okButton = new javax.swing.JButton();
			okButton.setSize(20, 20);
			okButton.setText("    OK     ");
			okButton.setHorizontalAlignment(SwingConstants.CENTER);
		    okButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(final java.awt.event.ActionEvent e) {
									
			    	boolean inputComplete = true;
			    	
					for (int r = 0; r < schemePanel.table.getRowCount(); r++) {
						if (schemePanel.table.getValueAt(r, 1).equals("") || schemePanel.table.getValueAt(r, 2).equals("")){
							JOptionPane.showMessageDialog(null, "VAS::Information is incomplete!");
							JOptionPane.showMessageDialog(null, "VAS::Please complete volume entries or Close the entry window to Cancel.");
							inputComplete = false;
							break;
						}
					}
					
			    	 if (inputComplete && schemePanel.table.isEditing()){		//SD2.1.1	end editing of any active cell so that info is saved to datamodel and thus can be recorded/saved to .apr
			    		 schemePanel.table.getCellEditor().stopCellEditing();
			    	 }
					
					if (inputComplete){
						getData();
						hideWindow();
					}
					
				}
			});
		    
		}
		return okButton;
	}
	
	private void hideWindow(){
		this.setVisible(false);
	}
	private javax.swing.JButton getpasteButton() {
		if (pasteButton == null) {
			pasteButton = new javax.swing.JButton();
			pasteButton.setSize(20, 20);
			//pasteButton.setText("    paste     ");
			pasteButton.setIcon(iconDown);
			pasteButton.setHorizontalAlignment(SwingConstants.CENTER);
			pasteButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(final java.awt.event.ActionEvent e) {
					int[] rowsSelection = schemePanel.table.getSelectedRows();
					int[] colsSelection = schemePanel.table.getSelectedColumns();
					int row = schemePanel.table.getSelectedRow();
					int col = schemePanel.table.getSelectedColumn();
					for (int c = 0; c < colsSelection.length; c++) {
						for (int i = 0; i < rowsSelection.length; i++) {
//							schemePanel.dataModel.setValueAt(schemePanel.data[rowsSelection[0]][colsSelection[c]], rowsSelection[i], colsSelection[c]);
						schemePanel.table.setValueAt(schemePanel.table.getValueAt(row,  col), rowsSelection[i], colsSelection[c]);

						} 
					}
				}
			});

		}
		return pasteButton;
	}
	
	private void getData(){
		
		TabWindow.arrayList_sdVolumesTransfer.clear();
		TabWindow.arrayList_sdVolumesDiluent.clear();
		
		for (int r = 0; r < schemePanel.table.getRowCount(); r++) {
			if (!schemePanel.table.getValueAt(r, 1).equals("")){
				TabWindow.arrayList_sdVolumesTransfer.add(String.valueOf(schemePanel.table.getValueAt(r, 1)));
				TabWindow.arrayList_sdVolumesDiluent.add(String.valueOf(schemePanel.table.getValueAt(r, 2)));
			}
		}
	}
	
}
