package TelosV2Pkg;
/////////////////////////////
//
//Date:    01.01.11
//Author:  Jesse Campbell
//Telos Scientific
//
//
// VAS Version Info:
// JC 10.23.12 - 	VAS V1.1 - 	Fixed TIP_QUANT() for multi plate with dispensing at surface
//					VAS V1.1 - 	Enhanced Pipetting Logic
//								Single well source aliquoting to use one channel CNU = 1
//								Single well source surface to use DWN as CNU up to CNS.  Instead of single tip back and fourth with ejects.  CNU = DWN
// JC 10.23.12 - 	VAS V1.2 -  Fixed CP Tip type with NTR tips
//
///////////////////////////////

/**
 * AutoPilot PK MODULE
 * Copyright 	- Telos Scientific - 
 * Author 		- Jesse Campbell
 * Description	- PK MODULE 
 * 
 * Version Information
 * V1.0.3 	11.20.12
 *  - First implimentation providing only Dilution plate options
 * V1.0.4	12.04.12
 * 	- Addition of MRD plate functionality
 *  - Addition of Pre-Assay Plate functionality
 *  V1.0.5	12.05.12
 *  - New logic for dilution volumes
 *  - Added "0,0" for MRD_Vol if MRD is not selected to fee Venus a value.
 *  V1.0.6	12.06.12
 *  -Volumes to 1 decimal place
 *  V1.0.7	12.06.12
 *  -Re-wrote dilution logic.  Less than 1:5 not currently functional.  Requires 1000ul tips for sample transfer
 *   *  V1.0.8	12.17.12
 *  -Fixed >10000 pipetting pattern.  
 *  -Added PK icon to step
 *   *  V1.0.9	12.18.12
 *  -MRD volume input added  
 *  -Modified last transfer pattern if less than 8
 *  * 	V1.0.10 12.31.12
 *  -Added tip quantity tracking. Used for correct loading and by Venus to dynamically create tip sequence.  50 and 1000 may be used during a single transfer to accomodate <1:5
 *  - < 1:5 will not be functionaly.  Venus will dynamically create tip sequence before each transfer.  <1:5 will use 1000ul tips.
 *  -Removed all of the "Final Transfer" code
 *  -Pre Assay and Assay volumes are not collected and are passed by PK_Assay_Vols at [17]
 * * V1.0.11 12.31.12 - work performed using Eclipse at PFizer La Jolla
 *  - Made Assay Plate option invisible for Beta Launch
 *  - Changed 1:10 dilution volumes to 30 into 270
 *  * V1.0.12 01.21.13 JC
 *  - Added functionality for transferring with both 50ul and 1000ul tips at same time.  Much work also introduced into ap.hsl for this.
 *  * V1.0.13 01.22.14 JC
 *  - Neat samples diluted onto MRD plate
 *  * V1.0.14 01.30.13 JC
 *  - Fixed so that can run without MRD
 *   * V1.0.15 02.08.13 JC
 *  - Standards and QC always get MRD
 *  - Samples, Standards, and QC can all be Tube or Plate
 *  - * V1.0.16 04.19.13 JC
 *  - Accommodates new interface and line output formats.
 *  - Added Assay Plate functionality added (including addition of 300 tips for assay plate)
 *  - 04.23.13 - Extended V1.0.16 
 *  - Fixes to accommodate V1.0.16 upgrades.
 *  - 05.07.13 - Extended V1.0.17 
 *  - Fixes for tip usage and Pre-assay plate position
 * 	- V1.0.17c indicates change from "Worklist" to "LIMS/LIS FILE:"
 *  - V1.0.18 Implimented Fixes and Modifications as outlined in doc AutoPilot_ToDo_051313.doc
 *  - 07.10.13 JC - AutoPilot 1.2.18   - Finish Serial Dilutin V1.0
 *  - 08.14.13 JC - AutoPilot 1.2.18   - SPE addition.
 *  - 01.15.14 JC - AutoPilot 1.2.18   - Cytomat01 Begin Addition.
 *  - 02.24.14 JC - AutoPilot 1.2.18	- Mods for CherryPicking02 - VASMOD0010
 *  - 03.27.14 JC - AutoPilot 1.2.23	- Created and implimented VAStools.LiquidClassConfirm
 *  - 04.21.14 JC - AutoPilot 1.2.23	- Cyto and MovePlate grip height set to 4 for default plate without lid (or just lid in case of MovePlate).
 *  
 *  - 09/17/14 JC - AutoPilot 1.3.2		- VEF0026  -  Modification to accommodate 24 and 48 well plate pipetting
 *  - 10.24.14 JC - 					- CPF020202 - Barcode pulldown determines file format requirements
 *  - 11/25/14 JC - AutoPilot 1.3.14    - VASMOD0011 - Tubes to Cherry Picking
 *  - 12/01/15 JC 						- VASMOD0012 - Fixes for PK.    Worklist selection when Operator Saving, MRD Volumes calculated earlier in sequence.
 *  - 12/10/15 JC 						- VAS13      - Addition of LICONIC Module
 *  - 12/16/15 JC 						- VAS14		 - Fixed dest multi Dest sequence equalization
 * /

/**
 * AutoPilot Cherry Picking MODULE
 * Copyright 	- Telos Scientific - 
 * Author 		- Jesse Campbell
 * Description	- PK MODULE 
 * 
 * Version Information
 * V1.2.0 	12.04.12
 *  - Fixed tip type selection
 *  - Designed to have Barcode info in worklist.  If no barcodes are present on plates, then operator manually enters barcode on AutoLoad error.
 */
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.EOFException;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintStream;
import java.io.PrintWriter;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.JOptionPane;










import com.itextpdf.text.pdf.PdfReader;
import com.itextpdf.text.pdf.parser.PdfReaderContentParser;
import com.itextpdf.text.pdf.parser.SimpleTextExtractionStrategy;
import com.itextpdf.text.pdf.parser.TextExtractionStrategy;
import com.sun.org.omg.CORBA.Initializer;

public class VASengine {
	
	// SWN - Source Well Number
	// SPA - Source Plate Array (e.g. 96, 384)
	// CNS - Channel Number System
	// P96 - 96 channel head available (0,1)
	// CNU - Channel Number Used
	// ALQ_CHN - Aliquot Channel Number
	
	// Utility Variables

	  public static String vasVersion = "1.1";
	  
	  public static String runID;								//DATE_TIME_RANDOMx4
	  
	  public static ArrayList<String> arrayTipRec_Reagent = new ArrayList();
	  public static ArrayList<String> arrayTipRec_Type = new ArrayList();
	  public static ArrayList<String> arrayTipRec_Pos = new ArrayList();
	  public static ArrayList<String> arrayTipRec_PosEnd = new ArrayList();
	  public static ArrayList<String> arrayTipRec_GotRecycled = new ArrayList();							//indicates if these tips end up getting returned for recycling
	  public static ArrayList<String> arrayPK_SeqSource = new ArrayList<String>();
	  public static ArrayList<String> arrayPK_Dilutions = new ArrayList<String>();
	  public static ArrayList<String> arrayPK_ChannelPattern = new ArrayList<String>();
	  public static ArrayList<String> arraylistPK_SamplesIDAll = new ArrayList<String>();					//Array List containing ID of Each ID from file
	  public static ArrayList<String> arraylistPK_SamplesUnique = new ArrayList<String>();					//Array List containing ID of each Unique Sample
	  public static ArrayList<String> arraylistPK_DilutionFinal = new ArrayList<String>();					//Array List containing Required Final Dilution of each Unique Sample
	  public static ArrayList<String> arraylistPK_VolumeFinal = new ArrayList<String>();					//Array List containing Volume transfered to last well of dilution plate
	  public static ArrayList<String> arraylistPK_VolumeFinal_Diluent = new ArrayList<String>();			//Array List containing Diluent Volume transfered to last well of dilution plate
	  public static ArrayList<String> arraylistPK_VolumeFinal_Total = new ArrayList<String>();				//Array List containing Total Volume (sample+Diluent) in last well of dilution plate
	  public static ArrayList<String> arraylistPK_StdCntrls = new ArrayList<String>();						//Array list containing unique standard/qc/etc
	  public static ArrayList<String> arrayList_SD_VolumesTransfer = new ArrayList<String>();
	  public static ArrayList<String> arrayList_SD_VolumesDiluent = new ArrayList<String>();
      public static ArrayList<String> arrayList_CORDNA_IDs = new ArrayList<String>();
      public static ArrayList<String> arrayList_CORDNA_Conc = new ArrayList<String>();
      public static ArrayList<String> arrayList_CORDNA_VolSource = new ArrayList<String>();
      public static ArrayList<String> arrayList_CORDNA_VolDiluent = new ArrayList<String>();
      public static ArrayList<String> arrayList_CORDNA_TrackingRaw_Position  = new ArrayList<String>();
      public static ArrayList<String> arrayList_CORDNA_TrackingRaw_Vol  = new ArrayList<String>();

	  public static double[] arrayPK_Dilutions_BatchOfEight = new double[8];
	  public static String[] arrayPK_Dilutions_FinalVol = new String[97];
	  public static String[] arraySeqPortrait96 = new String[97];
	  public static String[] arraySeqPortrait = new String[385];
	  public static String[] arraySeqPortrait384 = new String[385];
	  public static String errorMessage = "";							//message that is returned to TabWindow in the GO routine
	  public static double Tip10iccount = 0.0D;
	  public static double Tip1096count = 0.0D;
	  public static double Tip50iccount = 0.0D;
	  public static double Tip5096count = 0.0D;
	  public static double Tip300iccount = 0.0D;
	  public static double Tip30096count = 0.0D;
	  public static double Tip1000iccount = 0.0D;
	  public static int Tip10count_temp;
	  public static int Tip10count96_temp;
	  public static int Tip50count_temp;
	  public static int Tip50count96_temp;
	  public static int Tip300count_temp;
	  public static int Tip300count96_temp;
	  public static int Tip1000count_temp;
	  public static int TipQuant = 0;
	  public static int TipRacksTotal = 0;
	  public static int Step = 0;
	  
	  //positional information from source .apr file
	  public static int LabwareElement = 0;
	  public static int MixElement = 1;
	  public static int TipPositionElement = 2;
	  public static int MultiElement = 3;
	  public static int ShakeElement = 4;
	  public static int ChillElement = 5;
	  public static int TipPickUpElement = 6;
	  public static int TipRecycleElement = 7;
	  public static int ResXElement = 8;
	  public static int SourceLabPosElement = 0;
	  public static int DestLabPosElement = 1;
	  public static int ResXDestElement = 8;
	
	  //File variables
	  public static String FilePath_APR = "";
	  private static FileReader input;
	  
	  //System Specific Variables
	  public static String TIP10;
	  public static String TIP50;
	  public static String TIP300;
	  public static String TIP1000;
	  public static String TIPNTR;
	  public static String ConfigFileAP = "C:\\Program Files\\TELOS\\AutoPilot\\Config\\TelosCFG.cfg";
	  public static String ConfigFileHamilton = "";
	  public static String BitPath = "";
	  public static String LayoutFile = "";
	  public static String LabwareDir = "";
	  
	  //Directly from .apr
	  public static String StepType;
	  public static String ReagentName;
	  public static String SourceLabPos;						//Deck position - Source
	  public static String DestLabPos;							//Deck position - destination
	  public static String LabPosExtra1;						//Deck position - Extra (e.g. Serial)
	  public static String SourceLabID;							//Source labware type (e.g. Cost_96)
	  public static String AspiratePosition;					//Tip positioning for Aspiration
	  public static String SourceMix;							//Mix at source
	  public static String SourceMulti;							//Multi plate aspirating
	  public static String SourceShake;							//Shake the source.  System specific hardware required
	  public static String SourceChill;							//Chill the source.  System specific hardware required
	  public static String SourceTipPickUp;						//Choice for Walk Away tip pickup
	  public static String SourceTipRecycle;					//Choice to recycle the tips or not
	  public static String ResX;
	  public static String DestLabID;							//Destination labware type (e.g. Cost_96)
	  public static String DispensePosition;					//Tip positioning for Dispensing
	  public static String DestMix;								//Mix at destination
	  public static String DestMulti;							//Multi plate dispensing
	  public static String DestShake;							//Shake at destination
	  public static String DestChill;							//Chill at destination
	  public static String DestVariable;						//Detination variable (Reserved)
	  public static String ResXDest;
	  public static String LiquidType;							//Liquid Type
	  public static String LiquidTypeDiluent;					//Liquid Type Diluent (e.g. Serial)
	  public static String CP_Worklist;							//Cherry Picking
	  public static String CP_PlatePosRange;
	  public static String CP_TubePosRange;						//VASMOD0011
	  public static String CP_Vol;
	  public static String CP_MixCycles_Source;					//VASMOD0010
	  public static String CP_MixCycles_Destination;			//VASMOD0010
	  public static String CP_MixVol_Source;					//VASMOD0010
	  public static String CP_MixVol_Destination;				//VASMOD0010
  	  public static String CP_TipID_MixS;
  	  public static String CP_TipID_MixD;
  	  public static String CP_BCselected;						//VASMOD0011
  	  public static String CP_SourceLabwareType = "-1";			//VASMOD0011  0 = plate, 1 = tube.
  	  public static String CP_DestLabwareType = "-1";			//VASMOD0011  0 = plate, 1 = tube.
	  public static String LabwareID_Extra1;					//Extra labware (e.g. Serial Diluent) type (e.g. Cost_96)	
	  public static String PK_Worklist;
	  public static String PK_LabType_MRD;
	  public static String PK_LabType_MRD_Diluent;
	  public static String PK_LabType_DilutionPlate;
	  public static String PK_LabType_Dilution_Diluent;
	  public static String PK_LabType_PreAssay;
	  public static String PK_LabType_Assay;
	  public static String PK_LabType_Sample;
	  public static String PK_LabType_Std;
	  public static String PK_DeckPosition_MRD_Diluent;
	  public static String PK_DeckPosition_Dilution_Diluent;
	  public static String PK_LiquidType_Samples = "";
	  public static String PK_LiquidType_Standards = "";
	  public static String PK_LiquidType_MrdDiluent = "";
	  public static String PK_LiquidType_DilutionDiluent = "";
	  public static String PK_LiquidClass_DilutionDiluent = "";
	  public static String PK_LiquidClass_DilutionSeries = "";
	  public static String PK_LiquidType_DilutionSeries = "";
	  public static String PK_ChannelPatternsString = "";
	  public static String PK_Volume_DilutionSeries_Sample = "";
	  public static String PK_Volume_DilutionSeries_Diluent = "";
	  public static String PK_Volume_DilutionSeries_Mix = "";
	  public static String PK_Volume_DilutionSeries_Final = "";
	  public static String PK_Volume_MRD = "";
	  public static String PK_BatchTracker = "";
	  public static String PK_SourceSeq_Final = "";
	  public static String PK_DestSeq_Final = "73,74,75,76,77,78,79,80,81,82,83,84,85,86,87,88,89,90,91,92,93,94,95,96";
	  public static String PK_SourceSeq = "";
	  public static String PK_DestSeq = "";  //"1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,52,53,54,55,56,57,58,59,60,61,62,63,64,65,66,67,68,69,70,71,72,73,74,75,76,77,78,79,80,81,82,83,84,85,86,87,88,89,90,91,92,93,94,95,96";
	  public static String PK_PreAssay_SourceSeq = "";
	  public static String PK_PreAssay_SourceSeq_DilutionRecovery = "";
	  public static String PK_PreAssay_DestinationSeq = "";
	  public static String PK_PreAssay_StdCntr_SourceSeq = "";
	  public static String PK_PreAssay_StdCntr_DestinationSeq = "";
	  public static String PK_LabwarePos_MRD = "0", PK_LabwarePos_Dilution = "0", PK_LabwarePos_PreAssay = "0", PK_LabwarePos_Assay = "0";
	  public static String PK_LabwarePositions = "";
	  public static String PK_MRD_Read = "";
	  public static String PK_MRD_VolTotal_Read = "";
	  public static String PK_Assay_Vol_Read="";
	  public static String PK_PreAssay_MRD;
	  public static String PK_PreAssay_Vol_SampleTransfer = "";
	  public static String PK_PreAssay_Vol_DiluentTransfer = "0";
	  public static String PK_PreAssay_Vol_Read="";
	  public static String SD_VolumesTransfer;
	  public static String SD_VolumesDiluent;
	  public static String SD_TipsTransfers;
	  public static String SD_DestSeq;
	  public static String SPE_Mode;
	  public static String SPE_FilterLabware;							//Not currently used 081413
	  public static String SPE_CollectionLabware;						//Not currently used 081413
	  public static String SPE_VacDuration;
	  public static String SPE_VacPressure; 
	  public static String Cyto01_ToCyto;							//1=ToCytomat, 0 = from cytomat to deck
	  public static String Cyto01_CytoPositions;					//Position/s of plates in cytomat to be manipulated						
	  public static String Cyto01_DeckPositions;					//Position/s on plates on deck to be manipulated
	  public static String Cyto01_CoverStatus;						//Without Cover = 0.  With Cover = 1
	  public static String Cyto01_LabwareConfig;					//Labware File.  Note: ".rck" added by VAS
	  public static String Cyto01_TransferPosition;					//Deck position for transfer station	  
	  public static String Lico01_ToLico;							//VAS13 1=ToLico, 0 = from Lico to deck
	  public static String Lico01_LicoPositions;					//VAS13 Position/s of plates in Lico to be manipulated						
	  public static String Lico01_DeckPositions;					//VAS13 Position/s on plates on deck to be manipulated
	  public static String Lico01_CoverStatus;						//VAS13 Without Cover = 0.  With Cover = 1
	  public static String Lico01_LabwareConfig;					//VAS13 Labware File.  Note: ".rck" added by VAS
	  public static String Lico01_TransferPosition;					//VAS13 Deck position for transfer station
	  public static String SourceSeqNative;
	  public static String DestSeqNative;
	  public static String MV02_moveType;
	  public static String MV02_labwareConfig;
	  public static String MV02_gripHeight;
	  public static String MV02_sourceRange;
	  public static String MV02_destinationRange;
	  public static String PauseTimed;
	  public static String PauseMinutes;
	  public static String PauseSeconds;
	  public static String PauseMessage;
      
	  public static String CORDNA_Worklist;
	  public static String CORDNA_Procedure;
	  public static String CORDNA_TargetConc;
	  public static String CORDNA_TargetVol;
	  public static String CORDNA_LabwareDiluent;
	  public static String CORDNA_LabwareStock;
	  public static String CORDNA_LabwareNorm;
	  public static String CORDNA_LabwareFinal;
	  public static String CORDNA_LabwareDiluent_Pos;
	  public static String CORDNA_LabwareStock_Pos;
	  public static String CORDNA_LabwareNorm_Pos;
	  public static String CORDNA_LabwareNorm_Type;							//1=Tube, 2=Plate
	  public static String CORDNA_LabwareFinal_Pos;
	  public static String CORDNA_LabwareFinal_Type;						//1=Tube, 2=Plate

	  public static boolean PK_StandardsSample_SourceSeperate;
	  public static boolean CP_Mix;
	  public static boolean CP_MixS;
	  public static boolean CP_MixD;
	  public static boolean CP_BC;											//CPF020202
	  
	  //Determined Here by VAS
	  public static String P96;
	  public static String P96Chk;
	  public static String PipettingTechnique;
	  public static String LiquidClass;
	  public static String LiquidVolatile = "0";						// 0 or 1.  1 indicating that a volatile liquid is being used.
	  public static String TipType;
	  public static String TipID;
	  public static String Tip96StartPos;
	  public static String TipTypeCount = "0,0,0,0";
	  public static String TipTypeCountExt = "0,0,0,0,0,0,0,0";
	  public static String ChannelPattern;
	  public static String SPO;									//Source Plate Orientation Landscape(L) or Portrait(P)
	  public static String DPO;									//Destination Plate Orientation Landscape(L) or Portrait(P)
	  public static String SPS;									//Source Plate Stacked. "0" = NO.  Else "5_15" = stacked 5 high and stack height of 15
	  public static String DPS;									//Source Plate Stacked.
	  public static String SPA;									//Source Plate Array e.g 96, 384
	  public static String DPA;									//Destination Plate Array e.g 96, 384
	  public static String SourceSeqEq;
	  public static String DestSeqEq;
	  public static String DispenseHeight_Jet_Source;
	  public static String DispenseHeight_Jet;
	  public static String CP_TipType_MixS="";
	  public static String CP_TipType_MixD="";
	  public static String CP_MixS_LiquidClass;
	  public static String CP_MixD_LiquidClass;
	  public static String PK_ComponentSelection="";
	  public static String PK_LabType_String="";
	  public static String PK_MRD_VolsSummary="";				
	  public static String PK_Assay_Vols="";
	  public static String PK_TubeNumbers="";
	  public static String PK_Multi_DilutionPlates;
	  public static String PK_Worklist_Dilution;
	  public static String PK_Worklist_SampleID;
	  public static String PK_GoMRD, PK_GoDilution, PK_GoPreAssay, PK_GoAssay;
	  public static String VAS_OperatorInstruction;
	  public static String SD_LC_Diluent;
	  public static String SD_SourceSeq;
	  public static String Cyto01_GripHeight;					//= "4" for no Cover.  Modified if Cyto01_CoverStatus = 1 
	  public static String Lico01_GripHeight;					//VAS13  = "4" for no Cover.  Modified if Lico01_CoverStatus = 1 
	  public static int Volume;
	  public static int SWN;
	  public static int DWN;
	  public static int CNS;
	  public static int CNU;
	  public static int ALQ_Pre;
	  public static int ALQ_Transfers;
	  public static int ALQ_Aliquots;
	  public static int ALQ_AliquotsAspVol;
	  public static int ALQ_Remainders;
	  public static int ALQ_RemaindersAspVol;
	  public static int CP_WorklistTotalRecords = 0;
	  public static int CP_MixVol_Source_Integer = 0;
	  public static int CP_MixVol_Destination_Integer = 0;
	  public static int PK_MRD = 0;
	  public static int PK_MRD_VolTotal = 0;
	  public static int PK_WorklistTotalRecords = 0;
	  public static int PK_WorklistUniqeAll = 0;
	  public static int PK_WorklistUniqeSamples = 0;
	  public static int PK_WorklistAssayWell = 0;
	  public static int PK_StdCntrl_Unique_Total = 0;
	  public static int PK_StdCntrl_ReadLine_Total = 0;
	  public static int PK_Standards_Total = 0;
	  public static int PK_Dilution_SamplesCntr = 0;
	  public static int PK_Tip50iccount;
	  public static int PK_Tip30096count;
	  public static int PK_Tip1000iccount;
	  public static int PK_sourceSeq_SampleStartPos;
	  public static double MRD_SampleVol;
	  public static double MRD_DilutionVol;
	  public static double MRD_SampleVolNeat;
	  public static double MRD_DilutionVolNeat;
	  public static String CORDNA_OutS_Dil_Vol = "";
	  public static String CORDNA_OutS_Dil_SeqD = "";
	  public static String CORDNA_OutS_Dil_MixVol = "";
	  public static String CORDNA_OutS_Dil_ChP_T1000 = "";
	  public static String CORDNA_OutS_Dil_ChP_T300 = "";
	  public static String CORDNA_OutS_Dil_ChP_T50 = "";
	  public static String CORDNA_OutS_Stock_Vol = "";
	  public static String CORDNA_OutS_Stock_SeqS = "";
	  public static String CORDNA_OutS_Stock_SeqD = "";
	  public static String CORDNA_OutS_Stock_ChP_T1000 = "";
	  public static String CORDNA_OutS_Stock_ChP_T300 = "";
	  public static String CORDNA_OutS_Stock_ChP_T50 = "";
	  public static String CORDNA_OutS_Final_SeqD = "";
	  public static int CORDNA_SamplesTotal =0;
	  public static String CORDNA_OutS_Stock_LiquidClass = "";
	  public static String CORDNA_OutS_Diluent_LiquidClass = "";
	  public static String CORDNA_OutS_Aliquot_LiquidClass = "";

	  public static void main(String[] args)  {
		  
	  }

	  public static String GO(String File)    {
		  
		// VASengine.
		// Execution of the Virtual Automation Specialist Engine 

		// Reset run specific variables
		VASPostProcessing.arrayList_ProcessList.clear();		//Clear post processing list.  List is built below based on selected steps.
	    Tip10iccount = 0.0D;
	    Tip1096count = 0.0D;
	    Tip50iccount = 0.0D;
	    Tip5096count = 0.0D;
	    Tip300iccount = 0.0D;
	    Tip30096count = 0.0D;
	    Tip1000iccount = 0.0D;
		Tip10count_temp = 0;
		Tip10count96_temp = 0;
		Tip50count_temp = 0;
		Tip50count96_temp = 0;
		Tip300count_temp = 0;
		Tip300count96_temp = 0;
		Tip1000count_temp = 0;
		TipTypeCount = "0,0,0,0";
		TipTypeCountExt = "0,0,0,0,0,0,0,0";
	    CP_WorklistTotalRecords = 0;
    	CP_TipID_MixS = "0";
    	CP_TipID_MixD = "0";
    	CP_SourceLabwareType = "-1";			//VASMOD0011  0 = plate, 1 = tube.
      	CP_DestLabwareType = "-1";			//VASMOD0011  0 = plate, 1 = tube.
    	CP_Mix = false;
    	CP_MixS = false;
    	CP_MixD = false;
    	CP_MixVol_Source_Integer = 0;
    	CP_MixVol_Destination_Integer = 0;
	    arrayTipRec_Pos.clear();
	    arrayTipRec_PosEnd.clear();
	    arrayTipRec_Reagent.clear();
	    arrayTipRec_Type.clear();
	    arrayTipRec_GotRecycled.clear();
	    errorMessage = "";
	    arraylistPK_SamplesUnique.clear();
	    arraylistPK_StdCntrls.clear();
	    arraylistPK_DilutionFinal.clear();
	    arraylistPK_SamplesIDAll.clear();
	    PK_ChannelPatternsString = "";
		PK_Volume_DilutionSeries_Sample = "";
		PK_Volume_DilutionSeries_Diluent = "";
		PK_Volume_DilutionSeries_Mix = "";
		PK_Volume_DilutionSeries_Final = "";
		PK_Volume_MRD = "";
		PK_Multi_DilutionPlates = "";
		PK_BatchTracker = "";
		PK_SourceSeq_Final = "";
		PK_DestSeq_Final = "73,74,75,76,77,78,79,80,81,82,83,84,85,86,87,88,89,90,91,92,93,94,95,96";
		PK_SourceSeq = "";
		PK_DestSeq = "";  //"1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,52,53,54,55,56,57,58,59,60,61,62,63,64,65,66,67,68,69,70,71,72,73,74,75,76,77,78,79,80,81,82,83,84,85,86,87,88,89,90,91,92,93,94,95,96";
		PK_LabwarePos_MRD = "0"; 
		PK_LabwarePos_Dilution = "0"; 
		PK_LabwarePos_PreAssay = "0";
		PK_LabwarePos_Assay = "0";
		PK_PreAssay_SourceSeq = "";
		PK_PreAssay_SourceSeq_DilutionRecovery = "";
		PK_PreAssay_DestinationSeq = "";
		PK_PreAssay_StdCntr_SourceSeq = "";
		PK_PreAssay_StdCntr_DestinationSeq = "";
		PK_PreAssay_MRD = "";
		PK_PreAssay_Vol_SampleTransfer = "";
		PK_PreAssay_Vol_DiluentTransfer = "0";
		PK_LabwarePositions = "";
		PK_Dilution_SamplesCntr = 0;
		PK_MRD = 0;
		PK_MRD_VolTotal = 0;
		PK_WorklistTotalRecords = 0;
		PK_WorklistUniqeSamples = 0;
		PK_WorklistAssayWell = 0;
		PK_StdCntrl_Unique_Total = 0;
		PK_StdCntrl_ReadLine_Total = 0;
		PK_Standards_Total = 0;
		PK_Tip50iccount = 0;
		PK_Tip1000iccount = 0;
		PK_Tip30096count = 0;
	    arrayList_CORDNA_IDs.clear();
	    arrayList_CORDNA_Conc.clear();
	    arrayList_CORDNA_VolSource.clear();
	    arrayList_CORDNA_VolDiluent.clear();
		CORDNA_OutS_Dil_Vol = "";
		CORDNA_OutS_Dil_SeqD = "";
		CORDNA_OutS_Dil_MixVol = "";
		CORDNA_OutS_Dil_ChP_T1000 = "";
		CORDNA_OutS_Dil_ChP_T300 = "";
		CORDNA_OutS_Dil_ChP_T50 = "";
		CORDNA_OutS_Stock_Vol = "";
		CORDNA_OutS_Stock_SeqS = "";
		CORDNA_OutS_Stock_SeqD = "";
		CORDNA_OutS_Stock_ChP_T1000 = "";
		CORDNA_OutS_Stock_ChP_T300 = "";
		CORDNA_OutS_Stock_ChP_T50 = "";
		CORDNA_OutS_Final_SeqD = "";
		CORDNA_SamplesTotal = 0;
		CORDNA_OutS_Stock_LiquidClass = "";
		CORDNA_OutS_Diluent_LiquidClass = "";
		CORDNA_OutS_Aliquot_LiquidClass = "";
		CORDNA_LabwareFinal_Type = "";

	    //Get info from AP .cfg file
	    APConfig();
	
	    //Get passed file path for .apr file
	    FilePath_APR = File;
	
	    // Read the Automation Protocol File (.apr)
	    APR.Read(FilePath_APR, ",");
	    
	    // Determine the Layout File
	    LayoutFile = APR.LayoutFile();
	
	    // Array for final output data.  Each primary elements holds data for 1 step.
	 	// Primary element 1 holds data for Step # 1 etc...
	 	// Elements of output array:  0-StepNumber, 1-StepType
	    String[][] arrayGlobalOut = new String[APR.StepNumberTotal() + 2][50];
	    for (int i = 1; i < APR.StepNumberTotal() + 2; i++) for (int h = 0; h < 40; h++) arrayGlobalOut[i][h] = "";
	
	    // Loop for each step in the .apr
	    for (int i = 0; i < APR.StepNumberTotal(); i++) {
	      Step = i + 1;
	      
	      //Reset step specific variables
	      ALQ_Transfers = 0;
	      ALQ_Aliquots = 0;
	      ALQ_Remainders = 0;
	      DestSeqEq = "";
	      SourceSeqEq = "";
	      SourceLabPos = "";
	      DestLabPos = "";
	      SourceLabID = "";
	      AspiratePosition = "";
	      SourceMix = "";
	      SourceMulti = "";
	      SourceShake = "";
	      SourceChill = "";
	      SourceTipRecycle = "";
	      SourceTipPickUp = "";
	      DestLabID = "";
	      DispensePosition = "";
	      DestMix = "";
	      DestMulti = "";
	      DestShake = "";
	      DestChill = "";
	      DestVariable = "";
	      SourceSeqNative = APR.SourceSeq(Step);
	      DestSeqNative = APR.DestSeq(Step);
	      Volume = 0;
	      SWN = 0;
	      DWN = 0;
	      DestMix = "";
	      SPO = "";
	      DPO = "";
	      SPA = "";
	      DPA = "";
	      LiquidType = "";
	      LiquidVolatile = "0";
	      ReagentName = "";
	      StepType = "";
	      VAS_OperatorInstruction = "";
	      ChannelPattern = "";
	      arrayList_SD_VolumesTransfer.clear();
	      arrayList_SD_VolumesDiluent.clear();
	      Cyto01_GripHeight="4";
	      Lico01_GripHeight="4";							//VAS13
		  CP_WorklistTotalRecords = 0;
		  CP_TipID_MixS = "0";
		  CP_TipID_MixD = "0";
		  CP_Mix = false;
		  CP_MixS = false;
		  CP_MixD = false;
		  
		  
	      
	      //manipulate SourceTipRecycle if other parameters to do not match
	      if (DispensePosition.contains("Surface")) SourceTipRecycle = "0";
	
	      if (APR.StepType(Step).equals("add")) {
	        SourceLabPos = APR.LabwareLocationParsed(Step)[SourceLabPosElement];
	        DestLabPos = APR.LabwareLocationParsed(Step)[DestLabPosElement];
	        SourceLabID = APR.AspirateDataParsed(Step)[LabwareElement];
	        AspiratePosition = APR.AspirateDataParsed(Step)[TipPositionElement];
	        SourceMix = APR.AspirateDataParsed(Step)[MixElement];
	        SourceMulti = APR.AspirateDataParsed(Step)[MultiElement];
	        SourceShake = APR.AspirateDataParsed(Step)[ShakeElement];
	        SourceChill = APR.AspirateDataParsed(Step)[ChillElement];
	        SourceTipRecycle = APR.AspirateDataParsed(Step)[TipRecycleElement];
	        SourceTipPickUp = APR.AspirateDataParsed(Step)[TipPickUpElement];
	        DestLabID = APR.DispenseDataParsed(Step)[LabwareElement];
	        DispensePosition = APR.DispenseDataParsed(Step)[TipPositionElement];
	        DestMix = APR.DispenseDataParsed(Step)[MixElement];
	        DestMulti = APR.DispenseDataParsed(Step)[MultiElement];
	        DestShake = APR.DispenseDataParsed(Step)[ShakeElement];
	        DestChill = APR.DispenseDataParsed(Step)[ChillElement];
	        DestVariable = APR.DispenseDataParsed(Step)[TipRecycleElement];
	
	        SourceSeqNative = APR.SourceSeq(Step);
	        DestSeqNative = APR.DestSeq(Step);

	
	        Volume = Integer.parseInt(APR.Volume(Step));
	        SWN = APR.SourceSeqParsed(Step).length;
	        DWN = APR.DestSeqParsed(Step).length;
	        if (SourceLabPos.equals("Tube")) TubePrep("source");
	        if (DestLabPos.equals("Tube")) TubePrep("destination");
	        
	        DestMix = APR.DispenseDataParsed(Step)[MixElement];
	        SPO = SystemLabwareOrientation(SourceLabPos);
	        DPO = SystemLabwareOrientation(DestLabPos);
	        SPA = APR.AspirateDataParsed(Step)[0].substring(APR.AspirateDataParsed(Step)[0].lastIndexOf("_") + 1);
	        DPA = APR.DispenseDataParsed(Step)[0].substring(APR.DispenseDataParsed(Step)[0].lastIndexOf("_") + 1);
	        LiquidType = APR.LiquidParsed(Step)[1];
	        ReagentName = APR.LiquidParsed(Step)[0];
	        StepType = APR.StepType(Step);
	        if (SPA.contains("X")) ResX = APR.AspirateDataParsed(Step)[ResXElement];
	        if (DPA.contains("X")) ResXDest = APR.DispenseDataParsed(Step)[ResXDestElement];
	
	        if (SPO.equals("P")) SeqSortPortrait("source");
	        if (DPO.equals("P")) SeqSortPortrait("destination");
	
	        //manipulate SourceTipRecycle if other parameters to do not match
	        if (DispensePosition.contains("Surface")) SourceTipRecycle = "0";
	
			 ////////////////////////
			// VAS Logical Series //
		   ////////////////////////

	        try
	        {
	          if (!DPA.equals("384") && (DWN > SWN) && (SWN > CNS)) VAStools.errorMessageProcess("Step:" + Step + ".  Cannot determine pipetting pattern");
	          //if ((DWN > SWN) && (DWN%SWN != 0) ) errorMessage = "Step:" + Step + ".  Cannot determine pipetting pattern";
	          if (P96.equals("1")) P96Chk = PIP_96CHK(Step); else P96Chk = "false";
	          Reservoir_Adapt(Step);	          
	          PipettingTechnique = PIP_TECHNIQUE(Step);	          
	          ChannelPattern = CHAN_USED(Step);	          
	          TIP_TYPE(Step);
	          LiquidClass = LIQUID_CLASS(Step);
	          SeqEq(Step);
	          TIP_QUANT(Step);
	          DispenseHeight_Jet_Source = SystemLabwareInfo(LabwareDir + SourceLabID + ".ctr", "Depth.....");
	          DispenseHeight_Jet = SystemLabwareInfo(LabwareDir + DestLabID + ".ctr", "Depth.....");
	          if (PipettingTechnique.equals("pipICaliquot")) Aliquot(Step);
	          if (SPO.equals("P")) SourceLabID += "P";
	          if (DPO.equals("P")) DestLabID += "P"; 
	        }
	        catch (Throwable t)
	        {
	          if (errorMessage.equals("")) errorMessage = "Error::" + t + "Error is Step " + Step;
	          t.printStackTrace();
	          return errorMessage;
	        }
	
	        // populating arrayGlobalOut with final values for current step
	        arrayGlobalOut[Step][0] = String.valueOf(Step);								//The number of the step
	        arrayGlobalOut[Step][1] = StepType;											//Step Type
	        arrayGlobalOut[Step][2] = P96Chk;											//Check for 96Head
	        arrayGlobalOut[Step][3] = PipettingTechnique;								//Pipetting Technique
	        arrayGlobalOut[Step][4] = ChannelPattern;									//Channel Pattern to use
	        arrayGlobalOut[Step][5] = TipID;											//Tip ID. 1-10 See method below for details
	        arrayGlobalOut[Step][6] = "Tip Quantity";									//Tip Quantity
	        arrayGlobalOut[Step][7] = DispenseHeight_Jet;								//Tip Position when dispensing Jet
	        arrayGlobalOut[Step][8] = DispenseHeight_Jet_Source;						//Tip Position when dispensing Jet to Source.  e.g. Pre Aliquot 
	        arrayGlobalOut[Step][9] = APR.Volume(Step);									//Volume
	        arrayGlobalOut[Step][10] = LiquidClass;										//Liquid Class
	        arrayGlobalOut[Step][11] = "Serial Info";									//Serial Dilution Info
	        arrayGlobalOut[Step][12] = String.valueOf(ALQ_Pre);							//Aliquot Pre Volume
	        arrayGlobalOut[Step][13] = String.valueOf(ALQ_Transfers);					//Aliquot Transfers
	        arrayGlobalOut[Step][14] = String.valueOf(ALQ_Aliquots);					//Number of Aliquots per transfer
	        arrayGlobalOut[Step][15] = String.valueOf(ALQ_AliquotsAspVol);				//Volume to aspirate fore each transfer
	        arrayGlobalOut[Step][16] = String.valueOf(ALQ_Remainders);					//Number of remainder dispenses
	        arrayGlobalOut[Step][17] = String.valueOf(ALQ_RemaindersAspVol);			//Volume to aspirate for remainder transfer
	        arrayGlobalOut[Step][18] = SourceLabPos;
	        arrayGlobalOut[Step][19] = DestLabPos;
	        arrayGlobalOut[Step][20] = SourceLabID;
	        arrayGlobalOut[Step][21] = SourceMix;
	        arrayGlobalOut[Step][22] = AspiratePosition;
	        arrayGlobalOut[Step][23] = SourceMulti;
	        arrayGlobalOut[Step][24] = SourceShake;
	        arrayGlobalOut[Step][25] = SourceChill;
	        arrayGlobalOut[Step][26] = SourceTipRecycle;
	        arrayGlobalOut[Step][27] = Tip96StartPos;
	        arrayGlobalOut[Step][28] = DestLabID;
	        arrayGlobalOut[Step][29] = DestMix;
	        arrayGlobalOut[Step][30] = DispensePosition;
	        arrayGlobalOut[Step][31] = DestMulti;
	        arrayGlobalOut[Step][32] = DestShake;
	        arrayGlobalOut[Step][33] = DestChill;
	        arrayGlobalOut[Step][34] = LiquidVolatile;
	        arrayGlobalOut[Step][35] = DestVariable;
	        arrayGlobalOut[Step][36] = "EMPTY";
	        arrayGlobalOut[Step][37] = SourceTipPickUp;
	        arrayGlobalOut[Step][38] = SourceSeqEq;										//Source Sequence Equalized
	        arrayGlobalOut[Step][39] = DestSeqEq;										//Destination Sequence Equalized
	      }//END ADD STEP
	
	      if (APR.StepType(Step).equals("serial")) {
		        
	    	  //Serial Dilution Specific Variable

	    	  //Get info from .apr
	    	  StepType = APR.StepType(Step);
		      ReagentName = APR.LiquidParsed(Step)[0];
		      LiquidType = APR.LiquidParsed(Step)[1];
		      LabPosExtra1 = APR.LiquidParsed(Step)[2];
		      LiquidTypeDiluent = APR.LiquidParsed(Step)[3];
		      LabwareID_Extra1 = APR.LiquidParsed(Step)[4];
		        
		      SourceLabPos = APR.LabwareLocationParsed(Step)[0];
		      DestLabPos = APR.LabwareLocationParsed(Step)[1];
		       
		      SourceLabID = APR.AspirateDataParsed(Step)[0];
		      DestLabID = APR.DispenseDataParsed(Step)[0];

		      SourceSeqNative = APR.SourceSeq(Step);
		      DestSeqNative = APR.DestSeq(Step);
		        		      
		      SWN = APR.SourceSeqParsed(Step).length;
		      DWN = APR.DestSeqParsed(Step).length;
		       
		      SD_VolumesTransfer = APR.Volume(Step);
		      SD_VolumesDiluent = APR.Quadrants(Step);
		      Collections.addAll(arrayList_SD_VolumesTransfer, APR.VolumeParsed(Step));
		      Collections.addAll(arrayList_SD_VolumesDiluent, APR.QuadrantsParsed(Step));
		      
		      //Serial Dilution Set Values
		      SourceMulti = "1";
		      DestMulti = "1";


		          
		      //Process
		      if (SourceLabPos.equals("Tube")) TubePrep("source");
		      SD_ChanPattern();
		      TIP_QUANT(Step);
		      SD_LC_Diluent = "1000tip_" + LiquidTypeDiluent + "_pipICsurface";
		      
		      
		        SPO = SystemLabwareOrientation(SourceLabPos);
		        DPO = SystemLabwareOrientation(DestLabPos);
		        SPA = APR.AspirateDataParsed(Step)[0].substring(APR.AspirateDataParsed(Step)[0].lastIndexOf("_") + 1);
		        DPA = APR.DispenseDataParsed(Step)[0].substring(APR.DispenseDataParsed(Step)[0].lastIndexOf("_") + 1);
		        if (SPO.equals("P")) SeqSortPortrait("source");
		        if (DPO.equals("P")) SeqSortPortrait("destination");

		        if (SPO.equals("P")) SourceLabID += "P";
		        if (DPO.equals("P")) DestLabID += "P"; 
		        
		        SD_SeqPrep();
		        
		        
		      // populating arrayGlobalOut with final values for current step
		      arrayGlobalOut[Step][0] = String.valueOf(Step);							//The number of the step
		      arrayGlobalOut[Step][1] = StepType;										//Step Type	
		      arrayGlobalOut[Step][4] = ChannelPattern;									//Channel Pattern to use
		      arrayGlobalOut[Step][5] = SD_TipsTransfers;								//String containing tip type for each transfer (using tip codes)
		      arrayGlobalOut[Step][8] = SD_VolumesTransfer;								//String containing vol of each sample transfer in series
		      arrayGlobalOut[Step][9] = SD_VolumesDiluent;								//String containing vol of each diluent in series
		      arrayGlobalOut[Step][10] = LiquidType;									//Liquid for Transfers
		      arrayGlobalOut[Step][11] = LIQUID_VOLATILE(LiquidTypeDiluent);			//Diluent Volatility
		      arrayGlobalOut[Step][17] = LabPosExtra1;									//Deck Pos Diluent
		      arrayGlobalOut[Step][18] = SourceLabPos;									//Deck Pos Source
		      arrayGlobalOut[Step][19] = DestLabPos;									//Deck Pos Destination
		      arrayGlobalOut[Step][20] = SourceLabID;									//LabwareID Source
		      arrayGlobalOut[Step][23] = SourceMulti;									//Set to "1" for SD
		      arrayGlobalOut[Step][26] = SD_LC_Diluent;									//Liquid Class Diluent
		      arrayGlobalOut[Step][27] = LabwareID_Extra1;								//LabwareID Diluent
		      arrayGlobalOut[Step][28] = DestLabID;										//LabwareID Destination
		      arrayGlobalOut[Step][29] = String.valueOf(SWN);							//Number of native source positions
		      arrayGlobalOut[Step][31] = DestMulti;										//Set to "1" for SD
		      arrayGlobalOut[Step][34] = LIQUID_VOLATILE(LiquidType);
		      arrayGlobalOut[Step][38] = SD_SourceSeq;									//Source Sequence
		      arrayGlobalOut[Step][39] = SD_DestSeq;									//Destination
				
	      }//END SERIAL DILUTION STEP TYPE
	      
	      if (APR.StepType(Step).equals("MovePlate02")) {
	        StepType = APR.StepType(Step);
	        MV02_moveType = APR.CfgLineRead_Parsed(Step, 2)[0];
	        MV02_sourceRange = APR.CfgLineRead(Step, 3);
	        MV02_destinationRange = APR.CfgLineRead(Step, 4);
	        MV02_labwareConfig = APR.CfgLineRead_Parsed(Step, 5)[0];
	        SPO = SystemLabwareOrientation(MV02_sourceRange.split(",")[0]);			//only checks the first of the range for oriantation
	        DPO = SystemLabwareOrientation(MV02_destinationRange.split(",")[0]);	//only checks the first of the range for oriantation
	        if (SPO.equals("P")) MV02_labwareConfig += "P";
	        if (DPO.equals("P")) MV02_labwareConfig += "P";
	        
	        SPS = "";
	        if (MV02_sourceRange.split(",").length == 1){
	         if (!SystemLabwareStacked(MV02_sourceRange).equals("0")) SPS = SystemLabwareStacked(MV02_sourceRange) + "_" + SystemLabwareStackHeight(MV02_labwareConfig);
	        }
	        DPS = "";
	        if (MV02_destinationRange.split(",").length == 1){
	         if (!SystemLabwareStacked(MV02_destinationRange).equals("0")) DPS = SystemLabwareStacked(MV02_destinationRange) + "_" + SystemLabwareStackHeight(MV02_labwareConfig);
	        }
	       
//	        get cover dimensional info
    	  	int rackHeight;
    	  	int baseToCoverDistance = 0;
    	  	

	        if (MV02_moveType.equals("2")){
	        	rackHeight = Integer.parseInt(SystemLabwareInfo(LabwareDir + MV02_labwareConfig + ".rck", "Dim.Dz......"));							//.rck Height
	        	try {
					baseToCoverDistance = Integer.parseInt(SystemLabwareInfo(LabwareDir + MV02_labwareConfig + ".lid", "RackBaseToCoverBase......"));	//.rck Distance from base of rack to base of cover
				} catch (NumberFormatException e) {
					VAStools.errorMessageProcess("Step::" + Step + "::VAS ERROR: VAS Can't find the dimensions for the selected labware's cover");
					e.printStackTrace();
				}
	        	int gripHeight = (rackHeight - baseToCoverDistance) + 2;		//Mov0040
	        	if (gripHeight < 0){											//Mov0040
	        		VAStools.errorMessageProcess("Step::" + Step + "::VAS ERROR: VAS has found an error in the labware definition for " + MV02_labwareConfig + ".rck\n\nThe Lid is taller than the Plate!");
	        	}else{
	        		MV02_gripHeight = String.valueOf(gripHeight);
	        	}
	        		        	
	        }else{//Handles both Plate Alone and Lid Alone.  Lid Alone is expected to be on the plate.
	        	MV02_gripHeight = "6";		//Mov0040
	        }
	        
	        System.out.println(SPS);
	        System.out.println(DPS);
	
	        arrayGlobalOut[Step][0] = String.valueOf(Step);
	        arrayGlobalOut[Step][1] = StepType;
	        arrayGlobalOut[Step][2] = MV02_gripHeight;
	        arrayGlobalOut[Step][3] = MV02_sourceRange;
	        arrayGlobalOut[Step][4] = MV02_destinationRange;
	        arrayGlobalOut[Step][5] = MV02_labwareConfig;			//Labware config file WiTHOUT .rck
	        arrayGlobalOut[Step][6] = SPS;							//Source Stack info.  
	        arrayGlobalOut[Step][7] = "1";							//Source Multi
	        arrayGlobalOut[Step][8] = DPS;							//Destination Stack info. 
	        arrayGlobalOut[Step][9] = "1";							//DestMulti
	        arrayGlobalOut[Step][10] = "1";							//Source Sequence Holder
	        arrayGlobalOut[Step][11] = "1";							//Destination Sequence Holder
	        arrayGlobalOut[Step][20] = MV02_labwareConfig;			//Repeated from position 5	Needed here for Ham to populate deck
	        arrayGlobalOut[Step][28] = MV02_labwareConfig;			//Repeated from position 5.  Needed here for Ham to populate deck
	      }//END PLATE MOVE STEP TYPE
	
	      if (APR.StepType(Step).equals("Pause")) {
	        StepType = APR.StepType(Step);
	        
	        PauseTimed = "";
	        PauseMinutes = "";
	        PauseSeconds = "";
	        PauseMessage = "";

	        PauseTimed = (APR.CfgLineRead_Parsed(Step, 2)[0]);
	        
	        if (PauseTimed.equals("1")){
		        PauseMinutes = (APR.CfgLineRead_Parsed(Step, 2)[1]);
		        PauseSeconds = (APR.CfgLineRead_Parsed(Step, 2)[2]);
		        PauseMessage = (APR.CfgLineRead_Parsed(Step, 3)[0]);	        	
	        }

	        arrayGlobalOut[Step][0] = String.valueOf(Step);
	        arrayGlobalOut[Step][1] = StepType;
	        arrayGlobalOut[Step][2] = PauseTimed;
	        arrayGlobalOut[Step][3] = PauseMinutes;
	        arrayGlobalOut[Step][4] = PauseSeconds;
	        arrayGlobalOut[Step][5] = PauseMessage;
	        
	      }//END PAUSE PLATE STEP TYPE
	
	      if (APR.StepType(Step).equals("SPE")) {
	    	  
		        StepType = APR.StepType(Step);
		        SPE_Mode = APR.LiquidParsed(Step)[0];				//Borrowed APR.LiquodParsed which is line 3.
		        SPE_FilterLabware = APR.LiquidParsed(Step)[1];		//Not currently used 081413
		        SPE_CollectionLabware = APR.LiquidParsed(Step)[2];	//Not currently used 081413
		        SPE_VacDuration = APR.LiquidParsed(Step)[3];		//Borrowed APR.LiquodParsed which is line 3.
		        SPE_VacPressure = APR.LiquidParsed(Step)[4];		//Borrowed APR.LiquodParsed which is line 3.
		        
		        arrayGlobalOut[Step][0] = String.valueOf(Step);
		        arrayGlobalOut[Step][1] = StepType;
		        arrayGlobalOut[Step][2] = SPE_Mode;
		        arrayGlobalOut[Step][3] = SPE_FilterLabware;
		        arrayGlobalOut[Step][4] = SPE_CollectionLabware;
		        arrayGlobalOut[Step][5] = SPE_VacDuration;
		        arrayGlobalOut[Step][6] = SPE_VacPressure;
		      }//END SPE STEP TYPE
	      
	      if (APR.StepType(Step).equals("CherryPicking02")) {
	    	 
	        StepType = APR.StepType(Step);
	        CP_Worklist = APR.CfgLineRead_Parsed(Step, 2)[0];
	        CP_PlatePosRange = APR.CfgLineRead(Step, 3);
	        if (CP_PlatePosRange.equals("null")){CP_BCselected = "0";}else{CP_BCselected = "1";}									//VASMOD0010
	        SourceLabID = APR.CfgLineRead_Parsed(Step, 4)[0];
	        DestLabID = APR.CfgLineRead_Parsed(Step, 5)[0];
	        LiquidType = APR.CfgLineRead_Parsed(Step, 6)[0];
	        CP_TubePosRange = APR.CfgLineRead(Step, 9);
	        if (SourceLabID.contains("_Tube")){CP_SourceLabwareType = "1";}else{CP_SourceLabwareType = "0";}						//VASMOD0010
	        if (DestLabID.contains("_Tube")){CP_DestLabwareType = "1";}else{CP_DestLabwareType = "0";}								//VASMOD0010
	        
	        
	        CP_MixCycles_Source = APR.CfgLineRead_Parsed(Step, 7)[0];
	        try {CP_MixVol_Source = APR.CfgLineRead_Parsed(Step, 7)[1];} catch (Exception e) {CP_MixVol_Source="0";}
	        CP_MixVol_Source_Integer = Integer.parseInt(CP_MixVol_Source);
	        CP_MixCycles_Destination = APR.CfgLineRead_Parsed(Step, 8)[0];
	        try {CP_MixVol_Destination = APR.CfgLineRead_Parsed(Step, 8)[1];} catch (Exception e) {CP_MixVol_Destination="0";}
	        CP_MixVol_Destination_Integer = Integer.parseInt(CP_MixVol_Destination);
	        
	        if(!CP_MixCycles_Source.equals("0")){CP_MixS = true; CP_Mix = true;}
	        if(!CP_MixCycles_Destination.equals("0")){CP_MixD = true; CP_Mix = true;}
	        
	        VAStools.errorMessageProcess(VAStools.CP_WorklistConfirm(CP_Worklist, CP_PlatePosRange));

	        if (errorMessage.equals("")){
	        	CP_TipType();
		        LiquidClass = TipType + "_" + LiquidType + "_pipICsurface";
		        VAStools.LiquidClassConfirm(LiquidClass, BitPath + "\\HAMILTON\\Config\\ML_STARLiquids.mdb", Step);
		        CP_MixS_LiquidClass = CP_TipType_MixS + "_" + LiquidType + "_pipICsurface";
		        CP_MixD_LiquidClass = CP_TipType_MixD + "_" + LiquidType + "_pipICsurface";
		        
		        arrayGlobalOut[Step][0] = String.valueOf(Step);
		        arrayGlobalOut[Step][1] = StepType;
		        arrayGlobalOut[Step][2] = CP_Worklist;
		        arrayGlobalOut[Step][3] = CP_BCselected;				//VASMOD0010
		        arrayGlobalOut[Step][5] = TipID;
		        arrayGlobalOut[Step][6] = CP_TipID_MixS;				//Source Mixing Tip ID
		        arrayGlobalOut[Step][7] = CP_TipID_MixD;				//Destination Mixing ID
		        arrayGlobalOut[Step][8] = String.valueOf(CP_MixS);		//Mix Source boolean "true" / "false"
		        arrayGlobalOut[Step][9] = String.valueOf(CP_MixD);		//Mix Destination boolean "true" / "false"
		        arrayGlobalOut[Step][10] = LiquidClass;
		        arrayGlobalOut[Step][11] = CP_MixS_LiquidClass;
		        arrayGlobalOut[Step][12] = CP_MixD_LiquidClass;	        
		        arrayGlobalOut[Step][13] = CP_MixCycles_Source;
		        arrayGlobalOut[Step][14] = CP_MixVol_Source;
		        arrayGlobalOut[Step][15] = CP_MixCycles_Destination;
		        arrayGlobalOut[Step][16] = CP_MixVol_Destination;	        
		        arrayGlobalOut[Step][18] = CP_PlatePosRange;
		        arrayGlobalOut[Step][19] = CP_TubePosRange;
		        arrayGlobalOut[Step][20] = SourceLabID;
		        arrayGlobalOut[Step][28] = DestLabID;
		        arrayGlobalOut[Step][29] = CP_SourceLabwareType;
		        arrayGlobalOut[Step][30] = CP_DestLabwareType;
	        }

	      }//END CHERRY PICKING STEP
	      
	      if (APR.StepType(Step).equals("PK")) {
  			  
	    	  PK_LabType_MRD_Diluent = "";
	    	  PK_DeckPosition_MRD_Diluent  = "";
	    	  PK_LabType_Dilution_Diluent = "";
	    	  PK_DeckPosition_Dilution_Diluent = "";
	    	  
		        StepType = APR.StepType(Step);
		        //APR.CfgLineRead_Parsed(Step, 2)[0];
//		    	private static int stepNumber = 0;
//		    	private static int lineLinesPerStep = 10;
//		    	private static int lineStepType = 1;
//		    	private static int lineLiquid = 2;
//		    	private static int lineLabwareLocation = 3;
//		    	private static int lineAspirationData = 4;
//		    	private static int lineDestinationData = 5;
//		    	private static int lineSourceSeq = 6;
//		    	private static int lineDestSeq = 7;
//		    	private static int lineVolume = 8;
//		    	private static int lineQuadrants = 9;
		        
//		        LabwareIn  sample,std,MRD,DilutionPlate,PreAssay,Assay
//		        LabwareOut MRD,Dilution,PreAssay,Assay,sample,std
		        
		        
		        PK_GoMRD = APR.CfgLineRead_Parsed(Step, 5)[0];
		        PK_GoDilution = APR.CfgLineRead_Parsed(Step, 5)[1];
		        PK_GoPreAssay = APR.CfgLineRead_Parsed(Step, 5)[2];
		        PK_GoAssay = APR.CfgLineRead_Parsed(Step, 5)[3];
		        
		        PK_LabType_Sample = APR.CfgLineRead_Parsed(Step, 4)[0];
		        PK_LabType_Std = APR.CfgLineRead_Parsed(Step, 4)[1];
		        PK_LabType_MRD = APR.CfgLineRead_Parsed(Step, 4)[2];
		        PK_LabType_DilutionPlate = APR.CfgLineRead_Parsed(Step, 4)[3];
		        PK_LabType_PreAssay = APR.CfgLineRead_Parsed(Step, 4)[4];
		        PK_LabType_Assay = APR.CfgLineRead_Parsed(Step, 4)[5];
	        	PK_LiquidType_Samples = APR.CfgLineRead_Parsed(Step, 3)[0];
	        	PK_LiquidType_Standards = APR.CfgLineRead_Parsed(Step, 3)[1];
		        PK_LabType_MRD_Diluent = APR.CfgLineRead_Parsed(Step, 6)[0];
		        PK_DeckPosition_MRD_Diluent = APR.CfgLineRead_Parsed(Step, 6)[1];
	        	
		        if (PK_GoMRD.equals("1")){
		        	PK_MRD_Read = (APR.CfgLineRead_Parsed(Step, 2)[1]);
		        	PK_MRD_VolTotal_Read = APR.CfgLineRead_Parsed(Step, 2)[2];
		        	PK_LiquidType_MrdDiluent = APR.CfgLineRead_Parsed(Step, 3)[2];
			        PK_LabType_Sample = APR.CfgLineRead_Parsed(Step, 4)[0];
			        PK_LabType_Std = APR.CfgLineRead_Parsed(Step, 4)[1];
			        PK_LabType_MRD = APR.CfgLineRead_Parsed(Step, 4)[2];
			        
		        }
		        
		        if (PK_GoDilution.equals("1")){
		        	
		        	PK_LiquidType_MrdDiluent = APR.CfgLineRead_Parsed(Step, 3)[2];
		        	PK_LiquidType_DilutionDiluent = APR.CfgLineRead_Parsed(Step, 3)[3];
		        	PK_LabType_MRD = APR.CfgLineRead_Parsed(Step, 4)[2];
		        	PK_LabType_DilutionPlate = APR.CfgLineRead_Parsed(Step, 4)[3];
		        	
		        	PK_LabType_Dilution_Diluent = APR.CfgLineRead_Parsed(Step, 6)[2];
		        	PK_DeckPosition_Dilution_Diluent = APR.CfgLineRead_Parsed(Step, 6)[3];
		        	
			        SourceLabID = APR.CfgLineRead_Parsed(Step, 4)[2];
			        DestLabID = APR.CfgLineRead_Parsed(Step, 4)[3];
			        
		        }else{
		        	PK_LiquidType_DilutionDiluent = "Water";
		        }

		        if (PK_GoPreAssay.equals("1")){
		        	PK_PreAssay_MRD = (APR.CfgLineRead_Parsed(Step, 2)[1]);
		        	PK_PreAssay_Vol_Read = APR.CfgLineRead_Parsed(Step, 2)[3];
		        	PK_LabType_DilutionPlate = APR.CfgLineRead_Parsed(Step, 4)[3];
		        	PK_LabType_PreAssay = APR.CfgLineRead_Parsed(Step, 4)[4];
		        }else{
		        	PK_PreAssay_Vol_Read = "200";
		        }
		        
		        if (PK_GoAssay.equals("1")){
		        	PK_Assay_Vol_Read = APR.CfgLineRead_Parsed(Step, 2)[4];
		        	PK_LabType_PreAssay = APR.CfgLineRead_Parsed(Step, 4)[4];
		        	PK_LabType_Assay = APR.CfgLineRead_Parsed(Step, 4)[5];	
		        }else{
		        	PK_Assay_Vol_Read = "0";
		        }
		        VASPostProcessing.arrayList_ProcessList.add("2");					//Process "2" is creation of PK Summary Report
		        
		        if (TabWindow.security && TabWindow.security_accessLevel.equals("1")){
		        	if (TabWindow.run_Started){										//VASMOD0012
		        		PK_Worklist = TabWindow.security_pKWorklist_Runtime;		//VASMOD0012, PK_Worklist was always set here if security and operator.
		        	}else{															//VASMOD0012
		        		PK_Worklist = APR.CfgLineRead_Parsed(Step, 2)[0];			//VASMOD0012
		        	}																//VASMOD0012
		        }else{
		        	PK_Worklist = APR.CfgLineRead_Parsed(Step, 2)[0];
		        }
		        
		        SourceMulti = "1";
		        
		        //Process info and prepare for export
		        try {
		        	PK_MRDVolumes_Prep();											//VASMOD0012  These functionalities were split out from PK_Prep() in order to be available sooner for PK_WorklistProcessing()
					PK_WorklistProcessing();
					PK_Prep();
					PK_TipType();
				} catch (Exception e) {
					JOptionPane.showMessageDialog(null, "VAS ERROR:0003a::" + e);
					e.printStackTrace();
				}
		        
		        LiquidType = PK_LiquidType_DilutionDiluent;
		        PK_LiquidClass_DilutionDiluent = "1000tip" + "_" + LiquidType + "_" + "pipICsurface";  
		        PK_LiquidType_DilutionSeries = LiquidType;
		        PK_Assay_Vols = PK_PreAssay_Vol_SampleTransfer + "," + PK_Assay_Vol_Read + "," + PK_PreAssay_Vol_DiluentTransfer;
		        PK_TubeNumbers = String.valueOf(PK_WorklistUniqeAll) + "," + String.valueOf(PK_StdCntrl_Unique_Total) + "," + String.valueOf(PK_WorklistUniqeAll-PK_StdCntrl_Unique_Total);
		        if ((PK_WorklistTotalRecords)>96){VAStools.errorMessageProcess("The selected PK file requires more than the maximum of 96 preperations.\n\nPlease select a different PK file before continuing");}
		        if (!VAStools.PK_WorklistConfirm(PK_Worklist)){VAStools.errorMessageProcess("The PK file is invalid!");}		//PKF0033
		        
		        
		        //Load arrayGlobalOut in preparation for output to .apr
		        arrayGlobalOut[Step][0] = String.valueOf(Step);
		        arrayGlobalOut[Step][1] = StepType;
		        arrayGlobalOut[Step][2] = PK_ComponentSelection;
		        arrayGlobalOut[Step][3] = PK_LabType_String;
		        arrayGlobalOut[Step][4] = PK_ChannelPatternsString;
		        arrayGlobalOut[Step][5] = PK_MRD_VolsSummary;					//Summary of required MRD volumes.  Samples<50,DiluentVolume,SamplesNeat
		        arrayGlobalOut[Step][6] = PK_TubeNumbers;
		        arrayGlobalOut[Step][7] = PK_LabwarePositions;									
		        arrayGlobalOut[Step][8] = PK_LiquidType_Samples;				//Stock Sample Liquid Type
		        arrayGlobalOut[Step][9] = PK_LiquidType_Standards;				//Stock Stndrd/Control Liquid Type
		        arrayGlobalOut[Step][10] = PK_LiquidType_MrdDiluent;			//MRD Diluent Liquid Type
		        arrayGlobalOut[Step][11] = PK_LiquidClass_DilutionDiluent;
		        arrayGlobalOut[Step][12] = PK_LiquidType_DilutionSeries;
		        arrayGlobalOut[Step][13] = PK_Volume_DilutionSeries_Sample;
		        arrayGlobalOut[Step][14] = PK_Volume_DilutionSeries_Diluent;
		        arrayGlobalOut[Step][15] = PK_Volume_DilutionSeries_Mix;
		        arrayGlobalOut[Step][16] = PK_Volume_DilutionSeries_Final;
		        arrayGlobalOut[Step][17] = PK_Assay_Vols;								
		        arrayGlobalOut[Step][18] = "1";									//LabwarePos_MRD.  
		        arrayGlobalOut[Step][19] = "2";				
//		        arrayGlobalOut[Step][20] = "16";								
//		        arrayGlobalOut[Step][20] = SourceLabID;
		        arrayGlobalOut[Step][21] = PK_Volume_MRD;						//Volume of each transfer.  Either user input or calculated for NEAT samples.
		        arrayGlobalOut[Step][22] = "";
		        arrayGlobalOut[Step][23] = "1";									//SourceMulti. Needs to be updated!!!!!!	 
		        arrayGlobalOut[Step][24] = PK_Worklist;
		        arrayGlobalOut[Step][25] = PK_LabType_MRD_Diluent;
		        arrayGlobalOut[Step][26] = PK_DeckPosition_MRD_Diluent;
		        arrayGlobalOut[Step][27] = PK_LabType_Dilution_Diluent;
		        arrayGlobalOut[Step][28] = PK_DeckPosition_Dilution_Diluent;
		        arrayGlobalOut[Step][29] = PK_PreAssay_StdCntr_SourceSeq;
		        arrayGlobalOut[Step][30] = PK_PreAssay_StdCntr_DestinationSeq;
		        arrayGlobalOut[Step][31] = PK_Multi_DilutionPlates;				//Number of Dilution Plates Required.  							
		        arrayGlobalOut[Step][32] = PK_PreAssay_SourceSeq;
		        arrayGlobalOut[Step][33] = PK_PreAssay_DestinationSeq;
		        arrayGlobalOut[Step][34] = PK_PreAssay_SourceSeq_DilutionRecovery;
		        arrayGlobalOut[Step][35] = PK_BatchTracker;
//		        arrayGlobalOut[Step][36] = PK_SourceSeq_Final;		//update-pk002 - NO LONGER USED?
//		        arrayGlobalOut[Step][37] = PK_DestSeq_Final;		//update-pk002 - NO LONGER USED?  IF SO DELETE THE VARIABLES/DECLERATIONS!
		        arrayGlobalOut[Step][38] = PK_SourceSeq;										
		        arrayGlobalOut[Step][39] = PK_DestSeq;
		        
		        
		        System.out.println("0  --  " + String.valueOf(Step) + "\n");
		        System.out.println("1  --  " + StepType + "\n");
		        System.out.println("2  --  " + PK_ComponentSelection + "\n");
		        System.out.println("3  --  " + PK_LabType_String + "\n");
		        System.out.println("4  --  " + PK_ChannelPatternsString + "\n");
		        System.out.println("5  --  " + PK_MRD_VolsSummary + "\n");					//Summary of required MRD volumes.  Samples<50,DiluentVolume,SamplesNeat
		        System.out.println("6  --  " + PK_TubeNumbers + "\n");
		        System.out.println("7  --  " + PK_LabwarePositions + "\n");									
		        System.out.println("8  --  " + PK_LiquidType_Samples + "\n");				//Stock Sample Liquid Type
		        System.out.println("9  --  " + PK_LiquidType_Standards + "\n");				//Stock Stndrd/Control Liquid Type
		        System.out.println("10  --  " + PK_LiquidType_MrdDiluent + "\n");			//MRD Diluent Liquid Type
		        System.out.println("11  --  " + PK_LiquidClass_DilutionDiluent + "\n");
		        System.out.println("12  --  " + PK_LiquidType_DilutionSeries + "\n");
		        System.out.println("13  --  " + PK_Volume_DilutionSeries_Sample + "\n");
		        System.out.println("14  --  " + PK_Volume_DilutionSeries_Diluent + "\n");
		        System.out.println("15  --  " + PK_Volume_DilutionSeries_Mix + "\n");
		        System.out.println("16  --  " + PK_Volume_DilutionSeries_Final + "\n");
		        System.out.println("17  --  " + PK_Assay_Vols + "\n");								
		        System.out.println("18  --  " + "1" + "\n");									//LabwarePos_MRD.  
		        System.out.println("19  --  " + "2" + "\n");				
//		        arrayGlobalOut[Step][20] = "16" + "\n");								
//		        arrayGlobalOut[Step][20] = SourceLabID + "\n");
		        System.out.println("21  --  " + PK_Volume_MRD + "\n");						//Volume of each transfer.  Either user input or calculated for NEAT samples.
		        System.out.println("22  --  " + "" + "\n");
		        System.out.println("23  --  " + "1" + "\n");									//SourceMulti. Needs to be updated!!!!!!	 
		        System.out.println("24  --  " + PK_Worklist + "\n");
		        System.out.println("25  --  " + PK_LabType_MRD_Diluent + "\n");
		        System.out.println("26  --  " + PK_DeckPosition_MRD_Diluent + "\n");
		        System.out.println("27  --  " + PK_LabType_Dilution_Diluent + "\n");
		        System.out.println("28  --  " + PK_DeckPosition_Dilution_Diluent + "\n");
		        System.out.println("29  --  " + PK_PreAssay_StdCntr_SourceSeq + "\n");
		        System.out.println("30  --  " + PK_PreAssay_StdCntr_DestinationSeq + "\n");
		        System.out.println("31  --  " + PK_Multi_DilutionPlates + "\n");				//Number of Dilution Plates Required.  							
		        System.out.println("32  --  " + PK_PreAssay_SourceSeq + "\n");
		        System.out.println("33  --  " + PK_PreAssay_DestinationSeq + "\n");
		        System.out.println("34  --  " + PK_PreAssay_SourceSeq_DilutionRecovery + "\n");
		        System.out.println("35  --  " + PK_BatchTracker + "\n");
//		        arrayGlobalOut[Step][36] = PK_SourceSeq_Final + "\n");		//update-pk002 - NO LONGER USED?
//		        arrayGlobalOut[Step][37] = PK_DestSeq_Final + "\n");		//update-pk002 - NO LONGER USED?  IF SO DELETE THE VARIABLES/DECLERATIONS!
		        System.out.println("38  --  " + PK_SourceSeq + "\n");										
		        System.out.println("39  --  " + PK_DestSeq + "\n");
		        ////
		      }
	      
	      if (APR.StepType(Step).equals("Cytomat01")) {
	    	  
	    	  	int rackHeight;
	    	  	int baseToCoverDistance = 0;
	    	  	Cyto01_GripHeight="4";
	    	  	
		        StepType = APR.StepType(Step);
		        Cyto01_ToCyto = APR.LiquidParsed(Step)[0]; 
		        Cyto01_CytoPositions = APR.LabwareLocation(Step);
		        Cyto01_DeckPositions = APR.AspirateData(Step);
		        Cyto01_CoverStatus = APR.DispenseDataParsed(Step)[0];
		        Cyto01_LabwareConfig = APR.SourceSeqParsed(Step)[0] ;
		        Cyto01_TransferPosition = APR.DestSeqParsed(Step)[0];
		
		        if (Cyto01_CoverStatus.equals("1")){
		        	rackHeight = Integer.parseInt(SystemLabwareInfo(LabwareDir + Cyto01_LabwareConfig + ".rck", "Dim.Dz......"));							//.rck Height
		        	try {
						
		        		baseToCoverDistance = Integer.parseInt(SystemLabwareInfo(LabwareDir + Cyto01_LabwareConfig + ".lid", "RackBaseToCoverBase......"));	//.rck Distance from base of rack to base of cover
					} catch (NumberFormatException e) {
						VAStools.errorMessageProcess("Step::" + Step + "::VAS ERROR: VAS Can't find the dimensions for the selected labware's cover");
						e.printStackTrace();
					}
		        	Cyto01_GripHeight = String.valueOf((rackHeight - baseToCoverDistance) + 2);
		        }
		        
		        arrayGlobalOut[Step][0] = String.valueOf(Step);
		        arrayGlobalOut[Step][1] = StepType;
		        arrayGlobalOut[Step][2] = Cyto01_ToCyto;
		        arrayGlobalOut[Step][3] = Cyto01_CytoPositions;
		        arrayGlobalOut[Step][4] = Cyto01_DeckPositions;
		        arrayGlobalOut[Step][5] = Cyto01_GripHeight;
		        arrayGlobalOut[Step][6] = Cyto01_LabwareConfig + ".rck";
		        arrayGlobalOut[Step][7] = Cyto01_TransferPosition;
	      }
	      
	      if (APR.StepType(Step).equals("Liconic01")) {								//VAS 13
	    	  
	    	  	int rackHeight;
	    	  	int baseToCoverDistance = 0;
	    	  	Lico01_GripHeight="4";
	    	  	
		        StepType = APR.StepType(Step);
		        Lico01_ToLico = APR.LiquidParsed(Step)[0]; 
		        Lico01_LicoPositions = APR.LabwareLocation(Step);
		        Lico01_DeckPositions = APR.AspirateData(Step);
		        Lico01_CoverStatus = APR.DispenseDataParsed(Step)[0];
		        Lico01_LabwareConfig = APR.SourceSeqParsed(Step)[0] ;
		        Lico01_TransferPosition = APR.DestSeqParsed(Step)[0];
		
		        if (Lico01_CoverStatus.equals("1")){
		        	rackHeight = Integer.parseInt(SystemLabwareInfo(LabwareDir + Lico01_LabwareConfig + ".rck", "Dim.Dz......"));							//.rck Height
		        	try {
						
		        		baseToCoverDistance = Integer.parseInt(SystemLabwareInfo(LabwareDir + Lico01_LabwareConfig + ".lid", "RackBaseToCoverBase......"));	//.rck Distance from base of rack to base of cover
					} catch (NumberFormatException e) {
						VAStools.errorMessageProcess("Step::" + Step + "::VAS ERROR: VAS Can't find the dimensions for the selected labware's cover");
						e.printStackTrace();
					}
		        	Lico01_GripHeight = String.valueOf((rackHeight - baseToCoverDistance) + 2);
		        }
		        
		        arrayGlobalOut[Step][0] = String.valueOf(Step);
		        arrayGlobalOut[Step][1] = StepType;
		        arrayGlobalOut[Step][2] = Lico01_ToLico;
		        arrayGlobalOut[Step][3] = Lico01_LicoPositions;
		        arrayGlobalOut[Step][4] = Lico01_DeckPositions;
		        arrayGlobalOut[Step][5] = Lico01_GripHeight;
		        arrayGlobalOut[Step][6] = Lico01_LabwareConfig + ".rck";
		        arrayGlobalOut[Step][7] = Lico01_TransferPosition;
	      }
	      
	      if (APR.StepType(Step).equals("CORDNA01")) {
	    	  
	 		  //Local Variables
	 		  APLOG.Set("AP_RunID", "gDN/A_" + runID);												//Log RunID
  
	 		  //Information from TabWindow
	 		  StepType = APR.StepType(Step);
	    	  CORDNA_Worklist = APR.CfgLineRead_Parsed(Step, 2)[0];
	    	  CORDNA_Procedure = APR.CfgLineRead_Parsed(Step, 3)[0];
	    	  CORDNA_TargetConc = APR.CfgLineRead_Parsed(Step, 4)[0];
	    	  CORDNA_TargetVol = APR.CfgLineRead_Parsed(Step, 4)[1];
	    	  CORDNA_LabwareDiluent = APR.CfgLineRead_Parsed(Step, 5)[0];
	    	  CORDNA_LabwareStock = APR.CfgLineRead_Parsed(Step, 5)[1];
	    	  CORDNA_LabwareNorm = APR.CfgLineRead_Parsed(Step, 5)[2];
	    	  CORDNA_LabwareFinal = APR.CfgLineRead_Parsed(Step, 5)[3];
	    	  CORDNA_LabwareDiluent_Pos = APR.CfgLineRead_Parsed(Step, 6)[0];
	    	  CORDNA_LabwareStock_Pos = APR.CfgLineRead_Parsed(Step, 6)[1];
	    	  CORDNA_LabwareNorm_Pos = APR.CfgLineRead_Parsed(Step, 6)[2];
	    	  CORDNA_LabwareFinal_Pos = APR.CfgLineRead_Parsed(Step, 6)[3];
	    	  	  
	    	  if (!CORDNA_LabwareNorm.contains("Tube")){
	    		  CORDNA_LabwareNorm_Pos = VAStools.labwarePosToAP_(CORDNA_LabwareNorm_Pos);
	    		  CORDNA_LabwareNorm_Type = "2";
	    	  }else {
	    		  CORDNA_LabwareNorm_Type = "1";
	    	  }	   
	    	  
	    	  //Normalization Routine only
	    	  if (CORDNA_Procedure.equals("1")){
		 		  VASPostProcessing.arrayList_ProcessList.add("1");										//Process "1" is creation of CORDNA Report
		    	  CORDNA_LabwareDiluent_Pos = VAStools.labwarePosToAP_(CORDNA_LabwareDiluent_Pos); 		  
	    	  }

	    	  //Aliquoting Routine only
	    	  if (CORDNA_Procedure.equals("2")){
		    	  double volTarget = Integer.parseInt(CORDNA_TargetVol);
		    	  
		    	  if (volTarget > 300){
		    		  Tip1000iccount += 1;
		    		  CORDNA_OutS_Aliquot_LiquidClass = "1000tip_gDNA100_pipICsurface";
		    	  }else if (volTarget > 50){
		    		  Tip300iccount += 1;
		    		  CORDNA_OutS_Aliquot_LiquidClass = "300tip_gDNA100_pipICsurface";
		    	  }else {
		    		  Tip50iccount += 1;
		    		  CORDNA_OutS_Aliquot_LiquidClass = "50tip_gDNA100_pipICsurface";
		    	  }  
		    	  
		    	  if (!CORDNA_LabwareFinal.contains("Tube")){
		    		  CORDNA_LabwareFinal_Pos = VAStools.labwarePosToAP_(CORDNA_LabwareFinal_Pos);
		    		  CORDNA_LabwareFinal_Type = "2";
		    	  }else{
		    		  CORDNA_LabwareFinal_Type = "1";
		    	  }
	    	  }

	    	  CORDNA_FileProcessing();
	    	  //Output
	    	  arrayGlobalOut[Step][0] = String.valueOf(Step);
	    	  arrayGlobalOut[Step][1] = StepType;
	    	  arrayGlobalOut[Step][2] = CORDNA_Worklist;
	    	  arrayGlobalOut[Step][3] = CORDNA_Procedure;
	    	  arrayGlobalOut[Step][4] = CORDNA_TargetConc;
	    	  arrayGlobalOut[Step][5] = CORDNA_TargetVol;
	    	  arrayGlobalOut[Step][6] = CORDNA_LabwareDiluent;
	    	  arrayGlobalOut[Step][7] = CORDNA_LabwareStock;
	    	  arrayGlobalOut[Step][8] = CORDNA_LabwareNorm;
	    	  arrayGlobalOut[Step][9] = CORDNA_LabwareNorm_Type;
	    	  arrayGlobalOut[Step][10] = CORDNA_LabwareFinal;	    	  
	    	  arrayGlobalOut[Step][11] = CORDNA_LabwareDiluent_Pos;
	    	  arrayGlobalOut[Step][12] = CORDNA_LabwareStock_Pos;
	    	  arrayGlobalOut[Step][13] = CORDNA_LabwareNorm_Pos;
	    	  arrayGlobalOut[Step][14] = CORDNA_LabwareFinal_Pos;
	    	  arrayGlobalOut[Step][15] = CORDNA_LabwareFinal_Type;	
	    	  arrayGlobalOut[Step][16] = "";
	    	  arrayGlobalOut[Step][17] = "";
	    	  arrayGlobalOut[Step][18] = String.valueOf(CORDNA_SamplesTotal);
	    	  arrayGlobalOut[Step][19] = CORDNA_OutS_Final_SeqD; 	    	  
	    	  arrayGlobalOut[Step][20] = CORDNA_OutS_Dil_Vol;
	    	  arrayGlobalOut[Step][21] = CORDNA_OutS_Dil_MixVol;
	    	  arrayGlobalOut[Step][22] = CORDNA_OutS_Dil_SeqD;
	    	  arrayGlobalOut[Step][23] = CORDNA_OutS_Dil_ChP_T1000;
	    	  arrayGlobalOut[Step][24] = CORDNA_OutS_Dil_ChP_T300;
	    	  arrayGlobalOut[Step][25] = CORDNA_OutS_Dil_ChP_T50;
	    	  arrayGlobalOut[Step][26] = CORDNA_OutS_Diluent_LiquidClass;
	    	  arrayGlobalOut[Step][27] = "";
	    	  arrayGlobalOut[Step][28] = "";
	    	  arrayGlobalOut[Step][29] = runID;
	      	  
	    	  arrayGlobalOut[Step][30] = "";
	    	  arrayGlobalOut[Step][31] = CORDNA_OutS_Stock_Vol;
	    	  arrayGlobalOut[Step][32] = CORDNA_OutS_Stock_SeqS;
	    	  arrayGlobalOut[Step][33] = CORDNA_OutS_Stock_SeqD;
	    	  arrayGlobalOut[Step][34] = CORDNA_OutS_Stock_ChP_T1000;
	    	  arrayGlobalOut[Step][35] = CORDNA_OutS_Stock_ChP_T300;
	    	  arrayGlobalOut[Step][36] = CORDNA_OutS_Stock_ChP_T50;
	    	  arrayGlobalOut[Step][37] = CORDNA_OutS_Stock_LiquidClass;
	    	  arrayGlobalOut[Step][38] = CORDNA_OutS_Aliquot_LiquidClass;
	      }
	      	      
	      if (APR.StepType(Step).equals("LOAD01")) {  	  
	    	  StepType = APR.StepType(Step);

	    	  String Load01_GoAP = APR.CfgLineRead_Parsed(Step, 2)[0];
	    	  String Load01_GoTube = APR.CfgLineRead_Parsed(Step, 3)[0];
	    	  String Load01_GoTips = APR.CfgLineRead_Parsed(Step, 4)[0];
	    	  
	    	  String Load01_LineAP = APR.CfgLineRead(Step, 2);							//All Info for AP.			First element is Go, Rest makes up Range  		
	    	  String Load01_LineTube = APR.CfgLineRead(Step, 3);						//All Info for Tube			First element is Go, Rest makes up Range  	
	    	  
	    	  String Load01_RangeAP = Load01_LineAP.replaceFirst(".{2}", "");			//Only the Range info after removing first character (Go Character)
	    	  String Load01_RangeTube = Load01_LineTube.replaceFirst(".{2}", "");		//Only the Range info after removing first character (Go Character)
	    	  String Load01_RangeTips = "";
	    	  
	    	  String s;
	    	  for (int x=1; x<TipRacksTotal+1; x++){
	    		  if (x==1){s = String.valueOf(x);}else{s = "," + String.valueOf(x);}
	    		  Load01_RangeTips = Load01_RangeTips + s;
	    	  }

	    	  arrayGlobalOut[Step][0] = String.valueOf(Step);
	    	  arrayGlobalOut[Step][1] = StepType;
	    	  arrayGlobalOut[Step][2] = Load01_GoAP;
	    	  arrayGlobalOut[Step][3] = Load01_GoTube;
	    	  arrayGlobalOut[Step][4] = Load01_GoTips;
	    	  arrayGlobalOut[Step][5] = Load01_RangeAP;
	    	  arrayGlobalOut[Step][6] = Load01_RangeTube;
	    	  arrayGlobalOut[Step][7] = Load01_RangeTips;	  
	      }
	      
	      
	    }

	    //Protocol specific data written at end of file - Line 1 after Step Data
	    arrayGlobalOut[(APR.StepNumberTotal() + 1)][0] = "ProtocolData";
	    arrayGlobalOut[(APR.StepNumberTotal() + 1)][1] = TipTypeCount;
	    arrayGlobalOut[(APR.StepNumberTotal() + 1)][2] = TipTypeCountExt;
	    arrayGlobalOut[(APR.StepNumberTotal() + 1)][3] = ("NTR" + TIPNTR);
	    arrayGlobalOut[(APR.StepNumberTotal() + 1)][20] = BitPath;
	    
	    //Write all to file
	    APR.Write(FilePath_APR, arrayGlobalOut, ";");

	    return errorMessage;
	  }//END GO

  public static void VAS_OperatorInstructions()  {
	  if (!VAS_OperatorInstruction.equals("")){
		//JOptionPane.showMessageDialog(null, VAS_OperatorInstruction);  
	  }
  }
  
  public static String PIP_96CHK(int Step)  {
	  
	  	// Determine if a 96 head can be used for each step
		// Return PIP96:  0 = False, 1 = True (96 compatible)

	    String PIP96 = "false";
	    int PIP96CHK_Source = 0;
	    int PIP96CHK_Destination = 0;
	
	    // Check source for 96 head access
	    if ((!SPO.equals("P")) && 
	      (SWN >= 96)) {
	      if ((SPA.equals("384")) && (APR.Quadrants(Step).contains("1"))) {
	        PIP96CHK_Source = 1;
	      }
	      else if ((SPA.equals("96")) || (SPA.equals("1X"))) {
	        PIP96CHK_Source = 1;
	      }
	    }
	    
	// Check destination for 96 head access
    if ((!DPO.equals("P")) && 
      (DWN >= 96)) {
      if ((DPA.equals("384")) && (APR.Quadrants(Step).contains("1"))) {
        PIP96CHK_Destination = 1;
      }
      else if (DPA.equals("96")) {
        PIP96CHK_Destination = 1;
      }
    }

    //  Check for 96 head access to BOTH source and destination
    if ((PIP96CHK_Source == 1) && (PIP96CHK_Destination == 1)) {
      PIP96 = "true";
    }
    return PIP96;
  }

  public static void Reservoir_Adapt(int Step){
	// Rebuild the sequence and modify variable SWN to match
	// Modify the source sequence if the source is a Reservoir 1X and the 96 head is not to be used.
    
	String seqTemp = "";
    if (SPA.contains("X")) {//rebuild the Reservoir1X sequence based on destination 
    	
      if (SPA.equals("1X")) {
        if (P96Chk.equals("false")) {
          SourceSeqNative = "";
          if (DWN >= CNS) {
            for (int i = 1; i < CNS + 1; i++)
            {
              if (i == 1) SourceSeqNative = String.valueOf(i); else {
                SourceSeqNative = SourceSeqNative + "," + String.valueOf(i);
              }
            }

          }else {
        	    
            for (int i = 1; i < DWN + 1; i++)
            {
              if (i == 1) SourceSeqNative = String.valueOf(i); else
                SourceSeqNative = SourceSeqNative + "," + String.valueOf(i);
            }
            
          }
        }
      }
      else{//all other reservoir (e.g. 2X, 4X)
        for (int i = 0; i < SourceSeqNative.split(",").length; i++) {
          int x = Integer.parseInt(SourceSeqNative.split(",")[i]);
          int y = SourceSeqNative.split(",").length;
          int z = Integer.parseInt(ResX) - 1;
          int a = x + y * z;
          seqTemp = seqTemp + String.valueOf(a);
          if (i >= SourceSeqNative.split(",").length - 1) continue; seqTemp = seqTemp + ",";
        }
        SourceSeqNative = seqTemp;
      }
    }
    
    SWN = SourceSeqNative.split(",").length;
    seqTemp = "";
    if (DPA.contains("X")) {
        for (int i = 0; i < DestSeqNative.split(",").length; i++) {
            int x = Integer.parseInt(DestSeqNative.split(",")[i]);
            int y = DestSeqNative.split(",").length;
            int z = Integer.parseInt(ResXDest) - 1;
            int a = x + y * z;
            seqTemp = seqTemp + String.valueOf(a);
            if (i >= DestSeqNative.split(",").length - 1) continue; seqTemp = seqTemp + ",";
          }
        DestSeqNative = seqTemp;
    }
    DWN = DestSeqNative.split(",").length;

  }

  public static String PIP_TECHNIQUE(int Step){
    String Technique = "";
    //volCuttoff determined maximum volume that can be Aliquoted based on presence of 1000ul or 300ul tip.  No aliquoting without these tips.
    
    int volCuttoff = 0;
    if (TIP1000.equals("1")) volCuttoff = 140; else if (TIP300.equals("1")) volCuttoff = 43;

	 //////////////////////////////////
	// Begin Pipetting Technique Logic

    if (P96Chk.equals("true")){
    	// 96 Head
    	// PIP_96_ALIQUOT - TBD
    	//Check for PIP_96_Surface
      if ((DispensePosition.equals("Bottom")) || (DispensePosition.contains("Surface")) || (!DestMix.equals("0")) || (Volume <= 10)) {
        Technique = "pip96surface";
      }
      else{
    	  //Check for PIP_96_Jet
    	  Technique = "pip96jet";
      }
    }
    else //NOT 96
    	
    // Check for PIP_IC_SURFACE
    if ((DispensePosition.contains("Surface")) || (!DestMix.equals("0")) || (Volume <= 10)) {
      Technique = "pipICsurface";
    }
    else 
    if (DispensePosition.contains("Bottom")) {
      Technique = "pipICsurface";
    }
    else 
    
    // Check for PIP_IC_Aliquot
    ///////////////////////////////////if (DestMulti.equals("Variable")){}
    if ((Volume <= volCuttoff) && (Volume >= 10) && (SWN <= CNS) && (DWN * Integer.parseInt(DestMulti) / (SWN * Integer.parseInt(SourceMulti)) > 4)) {
      if ((DWN % SWN == 0) || (SPA.contains("X"))) {
        Technique = "pipICaliquot";
      }
    }
    else
    //Default to PIP_IC_JET if not _SURFACE or _Aliquot or _Serial
    {
      Technique = "pipICjet";
    }
    return Technique;
  }

  public static String SystemLabwareInfo(String LabwareDefFile, String searchString){
	  //Collect information from labware file.
	  //Find searchString and then returns the number (1 or multiple digits) following it.
	  //To be configured for each type of system (e.g. Beckman, Hamilton, Tecan)
    String subString = "";
    String returnString = "";
    File f = new File(LabwareDefFile);
    String newline = "";
    try {
      input = new FileReader(f);
      BufferedReader br = new BufferedReader(input);
      while ((newline = br.readLine()) != null) {
        Pattern pattern = Pattern.compile(searchString);
        Matcher matcher = pattern.matcher(newline);

        if (matcher.find()) {
          subString = matcher.group().substring(1);
        }
      }
      input.close();
    } catch (Exception e) {
    	JOptionPane.showMessageDialog(null, "VAS WARNING::Failed to analyze Labware File: " + LabwareDefFile + "\n\n" + e);
    }
    boolean start = false;
	char[] characters = subString.toCharArray();
	for(int i =0;i<characters.length;i++){
		if(Character.isDigit(characters[i])){
			start = true;
			returnString = returnString + String.valueOf(characters[i]);
		}else{
			if (start == true){break;}
		}
	}
	
    return returnString;
  }

  public static String SystemLabwareOrientation(String LabwareCode){
	//Determine if labware is Portrait(P) or Landscape(L)
	// Collect information from layout file.  
	// To be configured for each type of system (e.g. Beckman, Hamilton, Tecan)

    String Orientation = "L";
    String LabwareAngle = "";
    if (!LabwareCode.equals("Tube")) {//do not apply logic to tubes
      if (Integer.parseInt(LabwareCode) < 10) LabwareCode = "0" + LabwareCode;
      String LabwareID = "AP_00" + LabwareCode;
      
      String LabwareLayoutCode = "";
      String newline = "";
      String newline2 = "";
      String found = "0";
      try {

        File file = new File(LayoutFile);
        input = new FileReader(file);
        BufferedReader br = new BufferedReader(input);
        while ((newline = br.readLine()) != null) {
          Pattern pattern = Pattern.compile("......" + LabwareID);
          Matcher matcher = pattern.matcher(newline);
          if (matcher.find()) {
            found = "1";
            LabwareLayoutCode = matcher.group().substring(0, 2);
            break;
          }
        }
        if (found.equals("0")) VAStools.errorMessageProcess("Step " + Step + ":  Can't find deck position " + LabwareCode);
        input.close();
      } catch (Exception e) {
        JOptionPane.showMessageDialog(null, e);
      }
      try
      {
        File file = new File(LayoutFile);
        input = new FileReader(file);
        BufferedReader brr = new BufferedReader(input);

        Pattern pattern = Pattern.compile(LabwareLayoutCode + ".Angle" + ".....");
        while ((newline = brr.readLine()) != null) {
          newline2 = newline2 + newline;

          Matcher matcher = pattern.matcher(newline2);
          if (matcher.find()) {
            LabwareAngle = matcher.group();
            break;
          }
        }
        input.close();
      } catch (Exception e) {
        JOptionPane.showMessageDialog(null, e);
      }
      if (LabwareAngle.contains("270")) Orientation = "P";
    }
    return Orientation;
  }

  public static String SystemLabwareStacked(String LabwareCode){
	//Determine if labware is stacked and returns the number of stacked plates
	//Returns "0" string if not stacked
	// Collect information from layout file.  
	// To be configured for each type of system (e.g. Beckman, Hamilton, Tecan)
	// For Hamilton:  Layout will contain labware name with "_" if labware is stacked

    String stacked = "0";

    File f = new File(LayoutFile);
    if (Integer.parseInt(LabwareCode) < 10) LabwareCode = "0" + LabwareCode;
    String LabwareID = "AP_00" + LabwareCode + "_....";
    String newline = "";
    try {
      input = new FileReader(f);
      BufferedReader br = new BufferedReader(input);
      while ((newline = br.readLine()) != null) {
        Pattern pattern = Pattern.compile(LabwareID);
        Matcher matcher = pattern.matcher(newline);
        if (!matcher.find())
          continue;
        stacked = "3";
      }

      input.close();
    } catch (Exception e) {
      JOptionPane.showMessageDialog(null, e);
    }
    return stacked;
  }

  public static String SystemLabwareStackHeight(String LabwareCode){
	  
    String labwareConfig = LabwareDir + LabwareCode + ".rck";
    String stackHeight = "";

	// Collect information from labware file.  
	// To be configured for each type of system (e.g. Beckman, Hamilton, Tecan)

    File f = new File(labwareConfig);
    String newline = "";
    try {
      input = new FileReader(f);
      BufferedReader br = new BufferedReader(input);
      while ((newline = br.readLine()) != null) {
        Pattern pattern = Pattern.compile("StackHt...");
        Matcher matcher = pattern.matcher(newline);
        if (matcher.find()) {
          stackHeight = matcher.group().substring(8);
        }
      }
      input.close();
    } catch (Exception e) {
      JOptionPane.showMessageDialog(null, e);
    }
    return stackHeight;
  }

  public static void CP_TipType()
  {
	CP_Vol = "0";
    File f = new File(CP_Worklist);
    String newline = "";
    String commaChar = ",";
    int VOL_element = 0;
    try {
      input = new FileReader(f);//Read header row
      BufferedReader br = new BufferedReader(input);
      newline = br.readLine();
      for(int i =0;i<newline.split(",").length;i++){
    	  if (newline.split(",")[i].equals("VOL")){VOL_element = i;}
      }
      while ((newline = br.readLine()) != null) {//Read data rows
        CP_WorklistTotalRecords += 1;
        String[] fields = newline.split(commaChar);
        if (Integer.parseInt(fields[VOL_element]) > Integer.parseInt(CP_Vol)){
    		CP_Vol = fields[VOL_element];	
    	}
      }
      input.close();
    } catch (Exception e) {
    	VAStools.errorMessageProcess("Step::" + Step + "::VAS ERROR::VAS can't find the Cherry Picking Worklist::" + CP_Worklist);
    	JOptionPane.showMessageDialog(null, e);
    }
    TIP_QUANT(Step);
  }
  public static void PK_TipType(){
	  TIP_QUANT(Step);
  }

  public static void APConfig(){
	// Collect configuration information for target automation system
	// To be designed to match type of system (e.g. Beckman, Hamilton, Tecan)
	//
	// Determined Here:  Tip types available, 96 head available (P96), Independent channel number available (CNS)

    File f = new File(ConfigFileAP);
    String newline = "";
    try {
      input = new FileReader(f);
      BufferedReader br = new BufferedReader(input);
      for (int i = 1; i < 10; i++) br.readLine();

      // Get the available tips
      newline = br.readLine();
      TIP10 = newline.split(",")[1];
      TIP50 = newline.split(",")[2];
      TIP300 = newline.split(",")[3];
      TIP1000 = newline.split(",")[4];
      TIPNTR = newline.split(",")[10];

      // Get the pipetting hardware at line 12
      newline = br.readLine();
      CNS = Integer.parseInt(newline.split(",")[1]);
      P96 = newline.split(",")[2];

      input.close();
    } catch (Exception e) {
      JOptionPane.showMessageDialog(null, e);
    }
  }

  public static void SystemConfig(){
	// Collect configuration information for target automation system
	// To be designed to match type of system (e.g. Beckman, Hamilton, Tecan)

    File f = new File(ConfigFileHamilton);
    String newline = "";
    try {
      input = new FileReader(f);
      BufferedReader br = new BufferedReader(input);

      while ((newline = br.readLine()) != null) {
        Pattern pattern = Pattern.compile("NumberOfChannels.\\d{1,2}.N");
        Matcher matcher = pattern.matcher(newline);
        if (matcher.find()) {
          CNS = Integer.parseInt(matcher.group().split("s.")[1].split(".N")[0]);
        }
      }
      input.close();
    } catch (Exception e) {
      JOptionPane.showMessageDialog(null, e);
    }
  }

  public static String CHAN_USED(int Step) {
    CNU = 0;

    int testVal;
	if (PipettingTechnique.equals("pipICaliquot")){
    	//Currently only for Landscape 96 or 384.  Portrait plates need to follow "same number of wells per col on dest plate" rule.
	      int WellPerCol = 0; int ColEndPos = 0;
	      if (DPO.equals("L")) {
	    	  if (DPA.equals("24")){						//VEF0026
	    		  WellPerCol = 4;							//VEF0026
	    	  }												//VEF0026
	    	  else if (DPA.equals("48")){					//VEF0026
	    		  WellPerCol = 6;							//VEF0026
	    	  }												//VEF0026
	    	  else if (DPA.equals("96")){
	    		  WellPerCol = 8;
	    	  }else{//default to 384
	    		  WellPerCol = 16;
	    	  }
	    	  
	      ColEndPos = WellPerCol;
	      }
	      
	      if (DPO.equals("P")){
	    	  if (DPA.equals("24")){ 							//VEF0026
	    		  WellPerCol = 6; ColEndPos = 21;				//VEF0026
	    	  }else if (DPA.equals("48")){ 						//VEF0026
	    		  WellPerCol = 12; ColEndPos = 41;				//VEF0026
	    	  }else if (DPA.equals("96")){ 
	    		  WellPerCol = 12; ColEndPos = 85;
	    	  }else{
	    		  WellPerCol = 24; ColEndPos = 361;
	    	  }
	      }
	      
	      int WellPerColCurrent = 0;
	      int WellPerColRecord = 0;
	
	      if (SWN == 1){CNU = 1;}else{//Use 1 channel for Aliquoting out of one well (Can be upgraded).  Otherwise continue here...
		      //advance to first used column
	    	  
		      if (DPO.equals("L"))
		      {
		        while (ColEndPos < Integer.parseInt(DestSeqNative.split(",")[0])) {
		          ColEndPos += WellPerCol;
		        }
		      //loop over entire native destination sequence
		        for (int i = 0; i < DestSeqNative.split(",").length; i++) {
		          if (Integer.parseInt(DestSeqNative.split(",")[i]) <= ColEndPos) {
		            WellPerColCurrent++;
		          }
		          else
		          {
		            if (WellPerColRecord == 0) WellPerColRecord = WellPerColCurrent;
		            if (WellPerColRecord != WellPerColCurrent) {
		              CNU = SourceSeqNative.split(",").length;
		            }
		            else {
		              WellPerColRecord = WellPerColCurrent;
		            }
		            //advance to next used column
		            while (ColEndPos < Integer.parseInt(DestSeqNative.split(",")[i])) {
		              ColEndPos += WellPerCol;
		            }
		            WellPerColCurrent = 1;
		          }
		        }
		        //CNU set equal to the number of wells per column 
		        if (WellPerColRecord == 0) CNU = SourceSeqNative.split(",").length;
		        if (CNU == 0) CNU = WellPerColRecord;
		      }else //Portrait Plates
		      {
		    	//advance to first used column
		        do
		          ColEndPos -= WellPerCol;
		        while (ColEndPos > Integer.parseInt(DestSeqNative.split(",")[0]));
		
		        //loop over entire native destination sequence
		        for (int i = 0; i < DestSeqNative.split(",").length; i++) {
		          if (Integer.parseInt(DestSeqNative.split(",")[i]) >= ColEndPos) {
		            WellPerColCurrent++;
		          }
		          else
		          {
		            if (WellPerColRecord == 0) WellPerColRecord = WellPerColCurrent;
		            if (WellPerColRecord != WellPerColCurrent) {
		              CNU = SourceSeqNative.split(",").length;
		            }
		            else {
		              WellPerColRecord = WellPerColCurrent;
		            }
		            //advance to next used column
		            while (ColEndPos > Integer.parseInt(DestSeqNative.split(",")[i])) {
		              ColEndPos -= WellPerCol;
		            }
		            WellPerColCurrent = 1;
		          }
		        }
		        //CNU set equal to the number of wells per column 
		        if (WellPerColRecord == 0) CNU = SourceSeqNative.split(",").length;
		        if (CNU == 0) CNU = WellPerColRecord;
		      }
	      }
	      
	    }
	
	    else{
	    	//not Aliquoting OR the Destination plate is Portrait
		    testVal = 0;
		    
			if (SWN == 1){testVal = DWN;}else{testVal = SWN;}
			
	    	if (testVal < CNS) { CNU = testVal; } else {
	    		CNU = CNS;
	    	}
	    }

	    ChannelPattern = "";
	    for (int i = 0; i < CNU; i++) {
	      ChannelPattern += "1";
	    }
	    for (int i = 0; i < CNS - CNU; i++) {
	      ChannelPattern += "0";
	    }
	
	    return ChannelPattern;
  }

  public static void TIP_TYPE(int Step){
		//-------------------------------------------------------------------------------------------------
		// Determine the tip type to use for current step
		// Aliquot tip type is determined independently
		//
		// Tip IDs
		// 10tip_96 = 1
		// 10tip_IC = 2
		// 50tip_96 = 3
		// 50tip_IC = 4
		// 300tip_96 = 5
		// 300tip_IC = 6
		// 1000tip_IC = 7
		// 50tip_96_NTR = 13
		// 50tip_IC_NTR = 14
		// 300tip_96_NTR = 15
		// 300tip_IC_NTR = 16
		//
		// Uses the tip type required to handle the largest volume of TransferVolume, SourceMix, or DestMix
		//--------------------------------------------------------------------------------------------------

    TipType = "";
    int volMax = Math.max(Volume, Math.max(Integer.parseInt(SourceMix), Integer.parseInt(DestMix)));

    if (PipettingTechnique.equals("pipICaliquot")){
    //Aliquoting only with 1000ul and 300ul tips
      if ((TIP1000.equals("1")) && (!DPA.equals("384"))) { TipType = "1000tip"; TipID = "7"; } else { TipType = "300tip"; TipID = "6"; }
    }
    else {
      if ((TIP10.equals("1")) && 
        (volMax <= 10))
      {
        if (P96Chk.equals("true")) { TipType = "10tip"; TipID = "1"; } else { TipType = "10tip"; TipID = "2";
        }
      }
      if ((TipType.equals("")) && (TIP50.equals("1")) && 
        (volMax <= 50))
      {
        if (P96Chk.equals("true")) { TipType = "50tip"; TipID = "3"; } else { TipType = "50tip"; TipID = "4";
        }
      }
      if ((TipType.equals("")) && (TIP300.equals("1")) && 
        (volMax <= 300))
      {
        if (P96Chk.equals("true")) { TipType = "300tip"; TipID = "5"; } else { TipType = "300tip"; TipID = "6";
        }
      }
      if (TipType.equals("")) {
        TipType = "1000tip"; TipID = "7"; //Not yet compatible with 1000ul 96 head
      }
    }
    //check for NTR tips.  If present, modify tip ID.  Concat "1" with current Tip ID.  Ex.  NTR & 300ul_Core = "1" & "5" = 15.
    if (TIPNTR.equals("1")) TipID = TIPNTR + TipID;
  }

  public static void TIP_QUANT(int Step)
  {
    TipQuant = 0;
    int multiMax;

    
    if (APR.StepType(Step).equals("add")) {
      if (Integer.parseInt(SourceMulti) > Integer.parseInt(DestMulti)){
        	multiMax = Integer.parseInt(SourceMulti);
      }else{multiMax = Integer.parseInt(DestMulti);}
      
      if (P96Chk.equals("true")) {
        if (PipettingTechnique.equals("pip96surface")) TipQuant = SWN * multiMax;   //SourceSeqEq.split(",").length;    
        if (PipettingTechnique.equals("pip96jet")) TipQuant = SWN * Integer.parseInt(SourceMulti); 
      }
      else {
        if (PipettingTechnique.equals("pipICsurface")) TipQuant = Math.max(SWN * Integer.parseInt(SourceMulti), DWN * Integer.parseInt(DestMulti));
        if (PipettingTechnique.equals("pipICjet"))
        {
          if ((SWN == DWN) && (Integer.parseInt(SourceMulti) <= Integer.parseInt(DestMulti)))
          {
            if (SWN <= CNS)
            {
              TipQuant = SWN * Integer.parseInt(SourceMulti);
            }
            else TipQuant = SWN * Integer.parseInt(DestMulti); 
          }
          else {
            TipQuant = SWN * Integer.parseInt(SourceMulti);
          }
        }
        if (PipettingTechnique.equals("pipICaliquot")) TipQuant = CNU;
      }

      if (TipRecycling(Step).equals("no")) {
        if (TipID.equals("1")) Tip1096count += TipQuant;
        if (TipID.equals("2")) Tip10iccount += TipQuant;
        if (TipID.equals("3")  || TipID.equals("13")) Tip5096count += TipQuant;
        if (TipID.equals("4")  || TipID.equals("14")) Tip50iccount += TipQuant;
        if (TipID.equals("5")  || TipID.equals("15")) Tip30096count += TipQuant;
        if (TipID.equals("6")  || TipID.equals("16")) Tip300iccount += TipQuant;
        if (TipID.equals("7")) Tip1000iccount += TipQuant;
      }
    }
    if (APR.StepType(Step).equals("CherryPicking02")) {  
    	
    	CP_TipID_MixS = "0";
    	CP_TipID_MixD = "0";
    	TipID = "";
    	boolean CP_TipAdd_MixS = false;
    	boolean CP_TipAdd_MixD = false;
    	
    	if (!CP_Mix){
	    	if (Integer.parseInt(CP_Vol) <= 10){
		    	if (TIP10.equals("1")) { TipID = "2"; TipType = "10tip";} 
		    	else if (TIP50.equals("1")) { TipID = "4"; TipType = "50tip";} 
	    		else {VAStools.errorMessageProcess("Step::" + Step + "::VAS ERROR: There are no tips available that can ACCURATELY transfer the requested volume of " + CP_Vol + "ul");
	    		} 
		    }
	    	if (((Integer.parseInt(CP_Vol) > 10 ? 1 : 0) & (Integer.parseInt(CP_Vol) <= 50 ? 1 : 0)) != 0){
	    		if (TIP50.equals("1")) { TipID = "4"; TipType = "50tip";}
	    		else if (TIP300.equals("1")) { TipID = "6"; TipType = "300tip";}
	    		else if (TIP1000.equals("1")) { TipID = "7"; TipType = "1000tip"; }
	    		else {VAStools.errorMessageProcess("Step::" + Step + "::VAS ERROR: There are no tips available that can transfer the requested volume of " + CP_Vol + "ul");
	    		}
	    	}
	    	if (((Integer.parseInt(CP_Vol) > 50 ? 1 : 0) & (Integer.parseInt(CP_Vol) <= 300 ? 1 : 0)) != 0){
	    		if (TIP300.equals("1")) { TipID = "6"; TipType = "300tip";} 
		    	else if (TIP1000.equals("1")) { TipID = "7"; TipType = "1000tip"; }
		    	else {VAStools.errorMessageProcess("Step::" + Step + "::VAS ERROR: There are no tips available that can transfer the requested volume of " + CP_Vol + "ul");
		    	}
	    	}
	    	if (Integer.parseInt(CP_Vol) > 300  && Integer.parseInt(CP_Vol) < 1000) {
	    		if (TIP1000.equals("1")){
	    			TipID = "7"; TipType = "1000tip";
	    		}else{
	    			VAStools.errorMessageProcess("Step::" + Step + "::VAS ERROR: There are no tips available that can transfer the requested volume of " + CP_Vol + "ul");
	    		}
	    	}
	    	if (Integer.parseInt(CP_Vol) > 1000) {
	    		VAStools.errorMessageProcess("Step::" + Step + "::VAS ERROR: There are no tips available that can transfer the requested volume of " + CP_Vol + "ul");
	    	}
	    	
    	}else{//With Mixing
    		if (Integer.parseInt(CP_Vol) > 1000){
    			VAStools.errorMessageProcess("Step::" + Step + "::VAS ERROR: There are no tips available that can transfer the requested volume of " + CP_Vol + "ul");
    		}else if (Integer.parseInt(CP_Vol) > 300){//CP_VOL<1000
    			if (TIP1000.equals("1")){
    				TipID = "7"; TipType = "1000tip";
    				CP_TipID_MixS = "7"; CP_TipType_MixS = "1000tip";
    				CP_TipID_MixD = "7"; CP_TipType_MixD = "1000tip";
    			}else{
					VAStools.errorMessageProcess("Step::" + Step + "::VAS ERROR: There are no tips available that can transfer the requested volume of " + CP_Vol + "ul");
    			}
    		}else if (Integer.parseInt(CP_Vol) > 50){//CP_VOL<300
    			if (TIP1000.equals("1")){
    				TipID = "7"; TipType = "1000tip";
    				CP_TipID_MixS = "7"; CP_TipType_MixS = "1000tip";
    				CP_TipID_MixD = "7"; CP_TipType_MixD = "1000tip";
    			}else if (TIP300.equals("1")) {
    				TipID = "6"; TipType = "300tip";
    				CP_TipID_MixS = "6"; CP_TipType_MixS = "300tip";
    				CP_TipID_MixD = "6"; CP_TipType_MixD = "300tip";
    				if (CP_MixVol_Source_Integer>300){errorMessage = "Step::" + Step + "::VAS ERROR: There are no tips available that can mix the requested volume of " + CP_MixVol_Source + "ul";}
    				if (CP_MixVol_Destination_Integer>300){VAStools.errorMessageProcess("Step::" + Step + "::VAS ERROR: There are no tips available that can mix the requested volume of " + CP_MixVol_Destination + "ul");}
    			} 
    			else{
					VAStools.errorMessageProcess("Step::" + Step + "::VAS ERROR: There are no tips available that can transfer the requested volume of " + CP_Vol + "ul");
		    	}
    		}else if (Integer.parseInt(CP_Vol) > 10){//CP_VOL <=50
    			if (TIP300.equals("1")) {
    				TipID = "6"; TipType = "300tip";
    				
    				if (CP_MixVol_Source_Integer > 300){
        				if (TIP1000.equals("1")){
        					CP_TipID_MixS = "7"; CP_TipType_MixS = "1000tip"; CP_TipAdd_MixS = true;
        				}else{
        					VAStools.errorMessageProcess("Step::" + Step + "::VAS ERROR: There are no tips available that can mix the requested volume of " + CP_MixVol_Source + "ul");
        				}
        			}else{
        				CP_TipID_MixS = "6"; CP_TipType_MixS = "300tip";
        			}
        			if (CP_MixVol_Destination_Integer > 300){
        				if (TIP1000.equals("1")){
        					CP_TipID_MixD = "7"; CP_TipType_MixD = "1000tip"; CP_TipAdd_MixD = true;
        				}else{
        					VAStools.errorMessageProcess("Step::" + Step + "::VAS ERROR: There are no tips available that can mix the requested volume of " + CP_MixVol_Destination + "ul");
        				}
        			}else{
        				CP_TipID_MixD = "6"; CP_TipType_MixD = "300tip";
        			}
        			
    			}
		    	else if (TIP50.equals("1")) {
		    		TipID = "4"; TipType = "50tip";
		    		
		    		if (CP_MixVol_Source_Integer > 50){
        				if (TIP1000.equals("1")){
        					CP_TipID_MixS = "7"; CP_TipType_MixS = "1000tip"; CP_TipAdd_MixS = true;
        				}else{
        					VAStools.errorMessageProcess("Step::" + Step + "::VAS ERROR: There are no tips available that can mix the requested volume of " + CP_MixVol_Source + "ul");
        				}
        			}else{
        				CP_TipID_MixS = "4"; CP_TipType_MixS = "50tip";
        			}
        			if (CP_MixVol_Destination_Integer > 50){
        				if (TIP1000.equals("1")){
        					CP_TipID_MixD = "7"; CP_TipType_MixD = "1000tip"; CP_TipAdd_MixD = true;
        				}else{
        					VAStools.errorMessageProcess("Step::" + Step + "::VAS ERROR: There are no tips available that can mix the requested volume of " + CP_MixVol_Destination + "ul");
        				}
        			}else{
        				CP_TipID_MixD = "4"; CP_TipType_MixD = "50tip";
        			}
		    	}
		    	else {VAStools.errorMessageProcess("Step::" + Step + "::VAS ERROR: There are no tips available that can transfer the requested volume of " + CP_Vol + "ul");}
    		}else{//CP<=10
    			if (TIP50.equals("1")){
    				TipID = "4"; TipType = "50tip";	
    			}else if (TIP10.equals("1")){
    				TipID = "2"; TipType = "10tip";	
    			}else{
    				VAStools.errorMessageProcess("Step::" + Step + "::VAS ERROR: There are no tips available that can transfer the requested volume of " + CP_Vol + "ul");
    			}
    			
    			if (CP_MixVol_Source_Integer<=10){//50 or 10 must be available since the transfer volume above is <= 10
        			if (TIP50.equals("1")){
        				CP_TipID_MixS = "4"; CP_TipType_MixS = "50tip";
        			}else if (TIP10.equals("1")){
        				CP_TipID_MixS = "2"; CP_TipType_MixS = "10tip";	
        			}	
    			}

    			if (CP_MixVol_Source_Integer<=50){
    				if (TIP50.equals("1")){
    					CP_TipID_MixS = "4"; CP_TipType_MixS = "50tip"; //If 50s are available they would have been used for the transfer too
    				}else if (TIP1000.equals("1")){
    					CP_TipID_MixS = "7"; CP_TipType_MixS = "1000tip"; CP_TipAdd_MixS = true;
    				}else if (TIP300.equals("1")){
    					CP_TipID_MixS = "6"; CP_TipType_MixS = "300tip"; CP_TipAdd_MixS = true;
    				}						
    			}	
    			
    			if (CP_MixVol_Source_Integer>50){
    				if (TIP1000.equals("1")){
    					CP_TipID_MixS = "7"; CP_TipType_MixS = "1000tip"; CP_TipAdd_MixS = true;
    				}else if (CP_MixVol_Source_Integer>300){
    					VAStools.errorMessageProcess("Step::" + Step + "::VAS ERROR: There are no tips available that can mix the requested volume of " + CP_MixVol_Source + "ul");
    				}else if (TIP300.equals("1")){
    					CP_TipID_MixS = "6"; CP_TipType_MixS = "300tip"; CP_TipAdd_MixS = true;
    				}else{
    					VAStools.errorMessageProcess("Step::" + Step + "::VAS ERROR: There are no tips available that can mix the requested volume of " + CP_MixVol_Source + "ul");
    				}
    			}
    			
    			if (CP_MixVol_Destination_Integer<=10){//50 or 10 must be available since the transfer volume above is <= 10
        			if (TIP50.equals("1")){
        				CP_TipID_MixD = "4"; CP_TipType_MixD = "50tip";
        			}else if (TIP10.equals("1")){
        				CP_TipID_MixD = "2"; CP_TipType_MixD = "10tip";	
        			}	
    			}

    			if (CP_MixVol_Destination_Integer<=50){
    				if (TIP50.equals("1")){
    					CP_TipID_MixD = "4"; CP_TipType_MixD = "50tip"; //If 50s are available they would have been used for the transfer too
    				}else if (TIP1000.equals("1")){
    					CP_TipID_MixD = "7"; CP_TipType_MixD = "1000tip"; CP_TipAdd_MixD = true;
    				}else if (TIP300.equals("1")){
    					CP_TipID_MixD = "6"; CP_TipType_MixD = "300tip"; CP_TipAdd_MixD = true;
    				}						
    			}	
    			
    			if (CP_MixVol_Destination_Integer>50){
    				if (TIP1000.equals("1")){
    					CP_TipID_MixD = "7"; CP_TipType_MixD = "1000tip"; CP_TipAdd_MixD = true;
    				}else if (CP_MixVol_Destination_Integer>300){
    					VAStools.errorMessageProcess("Step::" + Step + "::VAS ERROR: There are no tips available that can mix the requested volume of " + CP_MixVol_Destination + "ul");
    				}else if (TIP300.equals("1")){
    					CP_TipID_MixD = "6"; CP_TipType_MixD = "300tip"; CP_TipAdd_MixD = true;
    				}else{
    					VAStools.errorMessageProcess("Step::" + Step + "::VAS ERROR: There are no tips available that can mix the requested volume of " + CP_MixVol_Destination + "ul");
    				}
    			}
    			
    		}
    	}
	      
	      //check for NTR tips.  If present, modify tip ID.  Concat "1" with current Tip ID.  Ex.  NTR & 300ul_Core = "1" & "5" = 15.
	      if (TIPNTR.equals("1")) {
	    	  TipID = TIPNTR + TipID; 
	    	  CP_TipID_MixS = TIPNTR + CP_TipID_MixS; 
	    	  CP_TipID_MixD = TIPNTR + CP_TipID_MixD;
	      }

	      if (TipID.equals("2")) Tip10iccount += CP_WorklistTotalRecords;
	      if (TipID.equals("4")) Tip50iccount += CP_WorklistTotalRecords;
	      if (TipID.equals("6")) Tip300iccount += CP_WorklistTotalRecords;
	      if (TipID.equals("7")) Tip1000iccount += CP_WorklistTotalRecords;
	      
	      if (CP_TipAdd_MixS){
		      if (CP_TipID_MixS.equals("2")) Tip10iccount += CP_WorklistTotalRecords;
		      if (CP_TipID_MixS.equals("4")) Tip50iccount += CP_WorklistTotalRecords;
		      if (CP_TipID_MixS.equals("6")) Tip300iccount += CP_WorklistTotalRecords;
		      if (CP_TipID_MixS.equals("7")) Tip1000iccount += CP_WorklistTotalRecords; 
	      }
	      if (CP_TipAdd_MixD){
		      if (CP_TipID_MixD.equals("2")) Tip10iccount += CP_WorklistTotalRecords;
		      if (CP_TipID_MixD.equals("4")) Tip50iccount += CP_WorklistTotalRecords;
		      if (CP_TipID_MixD.equals("6")) Tip300iccount += CP_WorklistTotalRecords;
		      if (CP_TipID_MixD.equals("7")) Tip1000iccount += CP_WorklistTotalRecords; 
	      }
	      
	      
    }
    
    if (APR.StepType(Step).equals("PK")) {
        Tip10iccount = 0;
        Tip50iccount = PK_Tip50iccount;
        Tip300iccount = 0;
        Tip1000iccount = PK_Tip1000iccount;
        Tip30096count = PK_Tip30096count;
    }
    
    if (APR.StepType(Step).equals("serial")){
        SD_TipsTransfers = "";
        
        for (int i=0; i<arrayList_SD_VolumesTransfer.size(); i++){
        	int vol = Integer.parseInt(arrayList_SD_VolumesTransfer.get(i));
        	if (vol<=10){
        		if (TIP50.equals("1")){Tip50iccount = Tip50iccount + SWN; SD_TipsTransfers = SD_TipsTransfers + "4,";}
        		else if (TIP10.equals("1")){Tip10iccount = Tip10iccount + SWN; SD_TipsTransfers = SD_TipsTransfers + "2,";}	
        	}else if (vol<=50){
        		if (TIP50.equals("1")){Tip50iccount = Tip50iccount + SWN; SD_TipsTransfers = SD_TipsTransfers + "4,";}
        		else if (TIP300.equals("1")){Tip300iccount = Tip300iccount + SWN; SD_TipsTransfers = SD_TipsTransfers + "6,";}
        		else if (TIP1000.equals("1")){Tip1000iccount = Tip1000iccount + SWN; SD_TipsTransfers = SD_TipsTransfers + "7,";}
        	}else if (vol<=300){
        		if (TIP300.equals("1")){Tip300iccount = Tip300iccount + SWN; SD_TipsTransfers = SD_TipsTransfers + "6,";}
        		else if (TIP1000.equals("1")){Tip1000iccount = Tip1000iccount + SWN; SD_TipsTransfers = SD_TipsTransfers + "7,";}
        	
        	}else {Tip1000iccount = Tip1000iccount + SWN; SD_TipsTransfers = SD_TipsTransfers + "7,";}
        
            //Diluent
            Tip1000iccount = Tip1000iccount + SWN;
        }
        SD_TipsTransfers = SD_TipsTransfers.substring(0, SD_TipsTransfers.length() - 1);  //Remove trailing comma
        
    }
    //Create the TipTypeCount String that will be passed out
    APR.StepNumberTotal();

    Tip10count_temp = (int)Math.ceil(Tip10iccount / 96.0D);
    Tip10count96_temp = (int)Math.ceil(Tip1096count / 96.0D);
    Tip50count_temp = (int)Math.ceil(Tip50iccount / 96.0D);
    Tip50count96_temp = (int)Math.ceil(Tip5096count / 96.0D);
    Tip300count_temp = (int)Math.ceil(Tip300iccount / 96.0D);
    Tip300count96_temp = (int)Math.ceil(Tip30096count / 96.0D);
    Tip1000count_temp = (int)Math.ceil(Tip1000iccount / 96.0D);

    TipTypeCountExt = Tip10count96_temp + "," + Tip10count_temp + "," + Tip50count96_temp + "," + Tip50count_temp + "," + Tip300count96_temp + "," + Tip300count_temp + "," + Tip1000count_temp;
    Tip10count_temp += Tip10count96_temp;
    Tip50count_temp += Tip50count96_temp;
    Tip300count_temp += Tip300count96_temp;
    TipTypeCount = Tip10count_temp + "," + Tip50count_temp + "," + Tip300count_temp + "," + Tip1000count_temp;
    TipRacksTotal = Tip10count_temp + Tip50count_temp + Tip300count_temp + Tip1000count_temp;
    
    System.out.println(Step);
    System.out.println(APR.StepNumberTotal());
    System.out.println(StepType);
    System.out.println(Tip300iccount);
    System.out.println(TipTypeCount);
  }

  public static String TipRecycling(int Step){
		////////////////////////////////////////////////
		//  Handles any required of Recycling of tips.
		//  - Only valid for 96 tips
		//  - Not valid for NTR tips
		//  
		//  - Determines the start position (Tip96StartPos) for this Step based on Recycling parameters
		//  - Returns Recycle Y/N
		////////////////////////////////////////////////

    String Recycle = "no";
    Tip96StartPos = "";
    if ((!TIPNTR.equals("1")) && (P96Chk.equals("true")))
    {
      if ((!PipettingTechnique.equals("pip96surface")) && (SourceTipRecycle.equals("1")) && (arrayTipRec_Reagent.indexOf(ReagentName) != -1) && (TipType.equals(arrayTipRec_Type.get(arrayTipRec_Reagent.indexOf(ReagentName)))) && (arrayTipRec_GotRecycled.get(arrayTipRec_Reagent.indexOf(ReagentName)) == "1"))
      {
        Recycle = "yes";
        Tip96StartPos = (String)arrayTipRec_Pos.get(arrayTipRec_Reagent.indexOf(ReagentName));
      }
      else
      {
        Recycle = "no";

        if (arrayTipRec_Type.lastIndexOf(TipType) == -1) {
          Tip96StartPos = "1";
          arrayTipRec_Pos.add(Tip96StartPos);
          arrayTipRec_PosEnd.add(String.valueOf(TipQuant));
        }
        else {
          Tip96StartPos = String.valueOf(Integer.parseInt((String)arrayTipRec_PosEnd.get(arrayTipRec_Type.lastIndexOf(TipType))) + 1);
          arrayTipRec_Pos.add(Tip96StartPos);
          arrayTipRec_PosEnd.add(String.valueOf(Integer.parseInt(Tip96StartPos) + TipQuant - 1));
        }
        if (SourceTipRecycle.equals("1")) arrayTipRec_GotRecycled.add("1"); else
          arrayTipRec_GotRecycled.add("0");
        arrayTipRec_Reagent.add(ReagentName);
        arrayTipRec_Type.add(TipType);
      }
    }

    return Recycle;
  }

  public static String LIQUID_CLASS(int Step) {
    String LiquidClass = "";
    LiquidClass = TipType + "_" + LiquidType + "_" + PipettingTechnique;
    
    if (LiquidType.equals("EtOH")){
    	LiquidVolatile = "1";
    }
    if (LiquidType.equals("Methanol")){
    	LiquidVolatile = "1";
    }
    if (LiquidType.equals("Acetonitrile")){
    	LiquidVolatile = "1";
    }
    
    VAStools.LiquidClassConfirm(LiquidClass, BitPath + "\\HAMILTON\\Config\\ML_STARLiquids.mdb", Step);
    
    return LiquidClass;
  }

  public static void SeqEq(int Step){

	  //Equalize the Source and Destination sequence to facilitate pipetting under different scenarios
		
	  ///////////////////////////////////////////////////////
	  //SORT any 384 well sequences by quadrant before Equalizing
	  
    int SeqWellId = 0;
    String SeqQuad = "";
    for (int q = 0; q < 8; q++) {
      if (q == 4) SeqQuad = "";
      if (!APR.QuadrantsParsed(Step)[q].equals("1"))
        continue;
      if (q == 0) SeqWellId = -1;	//Quad 1
      if (q == 1) SeqWellId = 15;	//Quad 2
      if (q == 2) SeqWellId = 0;	//Quad 3
      if (q == 3) SeqWellId = 16;	//Quad 4
      if (q == 4) SeqWellId = -1;	//Quad 1
      if (q == 5) SeqWellId = 15;	//Quad 2
      if (q == 6) SeqWellId = 0;	//Quad 3
      if (q == 7) SeqWellId = 16;	//Quad 4

      for (int r = 1; r <= 12; r++) {
        for (int c = 1; c <= 8; c++) {
          SeqWellId += 2;
          if (SeqQuad.equals("")) {
            SeqQuad = String.valueOf(SeqWellId);
          }
          else {
            SeqQuad = SeqQuad + "," + String.valueOf(SeqWellId);
          }
        }
        SeqWellId += 16;
        if ((!SPO.equals("P")) && (q < 4)) SourceSeqNative = SeqQuad;
        if ((DPO.equals("P")) || (q <= 3)) continue; DestSeqNative = SeqQuad;
      }

    }
    
    SourceSeqEq = SourceSeqNative;
    DestSeqEq = DestSeqNative;
	// END 384 quadrant sorting
	/////////////////////////////////////////////
	
	 ////////////////
	//Equalize Here
	
    //JOptionPane.showMessageDialog(null, Math.ceil(DestSeqNative.split(",").length * Integer.parseInt(DestMulti) / (SourceSeqNative.split(",").length * Integer.parseInt(SourceMulti))));
	//bring up source to match if required
    
    //Version 1.1
    //for (int i = 1; i < Math.ceil(DestSeqNative.split(",").length * Integer.parseInt(DestMulti) / (SourceSeqNative.split(",").length * Integer.parseInt(SourceMulti))); i++) {
      //SourceSeqEq = SourceSeqEq + "," + SourceSeqNative;
    //}
    
    int cntr_Seq = 0;
    
    while ((SourceSeqEq.split(",").length * Integer.parseInt(SourceMulti)) < Math.ceil(DestSeqNative.split(",").length * Integer.parseInt(DestMulti))){
    	SourceSeqEq = SourceSeqEq + "," + SourceSeqEq.split(",")[cntr_Seq];
    	cntr_Seq += 1;
    	if (cntr_Seq == SourceSeqNative.split(",").length) {cntr_Seq = 0;}
    }
    
    //bring up destination to match if required
    //Version 1.1
    //for (int i = 1; i < SourceSeqNative.split(",").length * Integer.parseInt(SourceMulti) / (DestSeqNative.split(",").length * Integer.parseInt(DestMulti)); i++)
    //DestSeqEq = DestSeqEq + "," + DestSeqNative;
    
    cntr_Seq = 0;
    while ((DestSeqEq.split(",").length * Integer.parseInt(DestMulti)) < Math.ceil(SourceSeqNative.split(",").length * Integer.parseInt(SourceMulti))){
    	
    	DestSeqEq = DestSeqEq + "," + DestSeqEq.split(",")[cntr_Seq];
    	cntr_Seq += 1;
    	if (cntr_Seq == DestSeqNative.split(",").length) {cntr_Seq = 0;}						//VAS14  -  APF0060  Change > to ==
    } 
    
  }

  
  
  public static void Aliquot(int Step){//Determine necessary information for Aliquoting 
    int AliquotsRequired = DWN * Integer.parseInt(DestMulti) / CNU;
    int AliquotsMax = 0;
    if (TipType.equals("1000tip")) AliquotsMax = (1000 - Volume * 3) / Volume;
    if (TipType.equals("300tip")) AliquotsMax = (300 - Volume * 3) / Volume;

    if (AliquotsMax > AliquotsRequired) {
      ALQ_Transfers = 1;
      ALQ_Aliquots = AliquotsRequired;
      ALQ_Remainders = 0;
    }
    else {
      ALQ_Transfers = AliquotsRequired / AliquotsMax;
      ALQ_Aliquots = AliquotsMax;
      ALQ_Remainders = AliquotsRequired % AliquotsMax;
    }

    ALQ_AliquotsAspVol = Volume * ALQ_Aliquots + Volume * 3;
    ALQ_RemaindersAspVol = Volume * ALQ_Remainders + Volume * 3;
    ALQ_Pre = Volume * 2;
  }

  public static void SeqSortPortrait(String Target) {
    int start = 0; int collim = 0; int rowlim = 0; int cntr = 0; int val = 0; int collen = 0; int welladded = 0;
    String seqTmp = "";
    if (Target.equals("source"))
    {
      if (SPA.equals("24")) { start = 24; collim = 5; rowlim = 7; collen = 4; }				//VEF0026
      if (SPA.equals("48")) { start = 48; collim = 7; rowlim = 9; collen = 6; }				//VEF0026
      if (SPA.equals("96")) { start = 96; collim = 9; rowlim = 13; collen = 8; }
      if (SPA.equals("384")) { start = 384; collim = 17; rowlim = 25; collen = 16; }
    }
    if (Target.equals("destination"))
    {
      if (DPA.equals("24")) { start = 24; collim = 5; rowlim = 7; collen = 4; }				//VEF0026
      if (DPA.equals("48")) { start = 48; collim = 7; rowlim = 9; collen = 6; }				//VEF0026
      if (DPA.equals("96")) { start = 96; collim = 9; rowlim = 13; collen = 8; }
      if (DPA.equals("384")) { start = 384; collim = 17; rowlim = 25; collen = 16;
      }
    }
    for (int i = 1; i < rowlim; i++) {
      val = start;
      for (int h = 1; h < collim; h++) {
        cntr++;
        String element = String.valueOf(val);

        arraySeqPortrait[cntr] = element;
        val--;
      }
      start -= collen;
    }

    if (Target.equals("source")) {
      start = 1; cntr = 0;
      for (int i = 1; i < collim; i++) {
        val = start;
        for (int h = 1; h < rowlim; h++) {
          cntr++;
          for (int j = 0; j < SWN; j++) {
            if (SourceSeqNative.split(",")[j].equals(String.valueOf(val))) {
              seqTmp = seqTmp + arraySeqPortrait[cntr];
              welladded++;
              if (welladded >= SWN) break; seqTmp = seqTmp + ",";
              break;
            }
          }
          val += collen;
        }
        start++;
      }
      SourceSeqNative = seqTmp;
    }

    if (Target.equals("destination")) {
      start = 1; cntr = 0; welladded = 0;
      for (int i = 1; i < collim; i++) {
        val = start;
        for (int h = 1; h < rowlim; h++) {
          cntr++;
          for (int j = 0; j < DWN; j++) {
            if (DestSeqNative.split(",")[j].equals(String.valueOf(val))) {
              seqTmp = seqTmp + arraySeqPortrait[cntr];
              welladded++;
              if (welladded >= DWN) break; seqTmp = seqTmp + ",";
              break;
            }
          }
          val += collen;
        }
        start++;
      }
      DestSeqNative = seqTmp;
    }
  }

  public static void TubePrep(String Target) {
    int tubeSeqLength = 0; int tracker = 0;
    String seqTmp = "";
    if (Target.equals("source")) {
      tubeSeqLength = Integer.parseInt(SourceSeqNative.split(",")[1]) + 1 - Integer.parseInt(SourceSeqNative.split(",")[0]);
      for (int i = 0; i < tubeSeqLength; i++) {
        if (i == 0) { seqTmp = SourceSeqNative.split(",")[0]; tracker = Integer.parseInt(SourceSeqNative.split(",")[0]);
        } else {
          tracker++;
          seqTmp = seqTmp + "," + String.valueOf(tracker);
        }
      }
      SourceSeqNative = seqTmp;
      SWN = SourceSeqNative.split(",").length;

    }

    if (Target.equals("destination")) {
      tubeSeqLength = Integer.parseInt(DestSeqNative.split(",")[1]) + 1 - Integer.parseInt(DestSeqNative.split(",")[0]);
      for (int i = 0; i < tubeSeqLength; i++) {
        if (i == 0) { seqTmp = DestSeqNative.split(",")[0]; tracker = Integer.parseInt(DestSeqNative.split(",")[0]);
        } else {
          tracker++;
          seqTmp = seqTmp + "," + String.valueOf(tracker);
        }
      }
      DestSeqNative = seqTmp;
      DWN = DestSeqNative.split(",").length;
    }
  }
  
  public static String Init (boolean simDev) {

	    //Check for AutoPilot config file
	    File f1 = new File(ConfigFileAP);
	    if (!f1.exists()){return ("AutoPilot Initialization error.\n\nConfiguration file: " + ConfigFileAP + " not found.");}
	    
	    if (simDev){
	    	
	    	
	    }else{
		    //Check for Hamilton config file and determine 32 vs 64 bit OS
		    ConfigFileHamilton = "C:\\Program Files (x86)\\HAMILTON\\Config\\ML_STAR.cfg";
		    File f2 = new File(ConfigFileHamilton);
		    if (f2.exists()){// 64 bit system
			    BitPath = "C:\\Program Files (x86)\\";
		    }else{//32 bit system
		    	BitPath = "C:\\Program Files\\";
		    	ConfigFileHamilton = "C:\\Program Files\\HAMILTON\\Config\\ML_STAR.cfg";
			    File f3 = new File(ConfigFileHamilton);
			    if (!f3.exists()){
			    	return ("AutoPilot Initialization error.\n\nNo Liquid Handling systems detected");
			    	}
			}
		    
		    System.out.println("Operating System directory - " + BitPath);
		    
		    //Check for Labware directory
		    LabwareDir = (BitPath + "\\HAMILTON\\Labware\\APcustom\\");
		    File f4 = new File(LabwareDir);
		    if (!f4.exists()){return ("AutoPilot Initialization error.\n\nLabware directory: " + LabwareDir + " not found.");}	
	    }
	    
	    
	    //If not error
	    return "initialized";
	  }
  
  
  public static void PK_MRDVolumes_Prep(){																	//VASMOD0012  These functionalities were split out from PK_Prep() in order to be available sooner for PK_WorklistProcessing()
	  if (PK_GoMRD.equals("1")){
		  PK_MRD = Integer.parseInt(PK_MRD_Read);
		  PK_MRD_VolTotal = Integer.parseInt(PK_MRD_VolTotal_Read);
		  
		  MRD_SampleVol = PK_MRD_VolTotal/PK_MRD;
		  MRD_DilutionVol = PK_MRD_VolTotal-(PK_MRD_VolTotal/PK_MRD);
		  
		  MRD_SampleVolNeat = (((Integer.parseInt(PK_PreAssay_Vol_Read)*2)+140)/PK_MRD);
		  MRD_DilutionVolNeat = ((Integer.parseInt(PK_PreAssay_Vol_Read)*2)+140) - MRD_SampleVolNeat;  		  
		  
		  PK_MRD_VolsSummary = (String.format("%.1f", MRD_SampleVol) + "," + String.format("%.1f", MRD_DilutionVol) + "," + String.format("%.1f", MRD_SampleVolNeat) + "," + String.format("%.1f", MRD_DilutionVolNeat));
		  
	  }else{
		  PK_MRD_VolsSummary = "0,0,0,0";
	  }
  }
  
  public static void PK_Prep(){
	  
	  int LabwarePos_Count = 0;

	  if (PK_GoMRD.equals("1")){
		  if  ((!PK_LabType_Sample.contains("Tube") && PK_WorklistUniqeSamples > 0) || !PK_LabType_Std.contains("Tube")){// Sample and/or STD type is plate.  Sample Plate will be at position 1 on deck.
			  PK_LabwarePos_MRD = "2";
			  LabwarePos_Count = 2;
		  }else {
			  PK_LabwarePos_MRD = "1";
			  LabwarePos_Count = 1;
		  }  
		  PK_ComponentSelection = "1";
		  }else {
			  PK_ComponentSelection = "0";  
		  }
	  
	  if (PK_GoDilution.equals("1")){
		  PK_ComponentSelection = PK_ComponentSelection + ",1";
		  
		  if (!PK_GoMRD.equals("1")){
			  
			  if(PK_LabType_Sample.contains("Tube")){				//APN0056
				  PK_LabwarePos_MRD = "1" ;
				  PK_LabwarePos_Dilution = "1";
				  LabwarePos_Count = 1;
			  }else{
				  PK_LabwarePos_MRD = "1" ;
				  PK_LabwarePos_Dilution = "2";
				  LabwarePos_Count = 2;				  
			  }

		  }else{
			  LabwarePos_Count ++;
			  PK_LabwarePos_Dilution = String.valueOf(LabwarePos_Count); 
		  }
		  LabwarePos_Count = LabwarePos_Count + (Integer.valueOf(PK_Multi_DilutionPlates) - 1);
	  }else{
		  PK_ComponentSelection = PK_ComponentSelection + ",0";
		  PK_LabwarePos_Dilution = "0"; 									//Done to prevent "" string
		  PK_LabType_DilutionPlate = PK_LabType_MRD;						//Done to prevent "null" string 
	  }
	  
	  if (PK_GoPreAssay.equals("1")){
		  PK_ComponentSelection = PK_ComponentSelection + ",1";

		  if (!PK_GoMRD.equals("1") && !PK_PreAssay_MRD.equals("1")){
			
			int transferVol = Integer.valueOf(PK_PreAssay_Vol_Read)/Integer.valueOf(PK_PreAssay_MRD);
			int dilVol = Integer.valueOf(PK_PreAssay_Vol_Read) - transferVol;
			
			PK_PreAssay_Vol_SampleTransfer  = String.valueOf(transferVol);
			PK_PreAssay_Vol_DiluentTransfer= String.valueOf(dilVol);
			
		  }else{
			PK_PreAssay_Vol_SampleTransfer = PK_PreAssay_Vol_Read;
			PK_PreAssay_Vol_DiluentTransfer= "0";
		  }
		  
		  
		  if (PK_GoDilution.equals("1")){
			  LabwarePos_Count ++;
			  PK_LabwarePos_PreAssay = String.valueOf(LabwarePos_Count); 
		  }else{
			  if (!PK_GoMRD.equals("1")){  //If not Dilution and No MRD, then source to this Pre-Assay has to be 
				  PK_LabwarePos_MRD = "1";
				  LabwarePos_Count = 1;
			  }
			  if(PK_WorklistUniqeSamples != 0){
				  PK_LabwarePos_Dilution = "2";
				  LabwarePos_Count = 2;
			  }
			  LabwarePos_Count ++;
			  PK_LabwarePos_PreAssay = String.valueOf(LabwarePos_Count); 
		  }
		  
	       ///////////////////////////////////////////////////
	      //Record required tips for PreAssay
    	  if (Integer.valueOf(PK_PreAssay_Vol_SampleTransfer)<=50){
    		  PK_Tip50iccount += PK_WorklistTotalRecords;
    	  }else{
    		  PK_Tip1000iccount += PK_WorklistTotalRecords;
    	  }
    	  
    	  if (Integer.valueOf(PK_PreAssay_Vol_DiluentTransfer)!=0){//Diluent always uses 1000tips!
    		  PK_Tip1000iccount += PK_WorklistTotalRecords;
    	  }

	  }else{
		  PK_ComponentSelection = PK_ComponentSelection + ",0";  
	  }
			  
	  if (PK_GoAssay.equals("1")){
		  PK_ComponentSelection = PK_ComponentSelection + ",1";
		  if (!PK_GoPreAssay.equals("1")){
			  PK_LabwarePos_PreAssay = "1";
			  PK_LabwarePos_Assay = "2"; 
		  }else{
			  LabwarePos_Count ++;
			  PK_LabwarePos_Assay = String.valueOf(LabwarePos_Count); 
		  }
	  }else{
		  PK_ComponentSelection = PK_ComponentSelection + ",0";
		  PK_LabwarePos_Assay = "0";
	  }
	  
	  PK_LabwarePositions = PK_LabwarePos_MRD + "," + PK_LabwarePos_Dilution + "," + PK_LabwarePos_PreAssay + "," + PK_LabwarePos_Assay;
      PK_LabType_String = PK_LabType_MRD + "," + PK_LabType_DilutionPlate + "," + PK_LabType_PreAssay + "," + PK_LabType_Assay + "," + PK_LabType_Sample + "," + PK_LabType_Std;
      
	  System.out.println("PK_LabwarePositions  --  " + PK_LabwarePositions);
	  System.out.println("PK_LabType_String  --  " + PK_LabType_String);
	  System.out.println("PK_ComponentSelection  --  " + PK_ComponentSelection);
////
  }
  
  public static void PK_WorklistProcessing()
  {
    boolean finalTransfer;
    
    String newline = "";
    String tabChar = "\t";
    String ChannelUsage = "";
    
    int BatchOfEight = 0;
    int Batches = 0;
    int BatchSizeLast = 0;
    int SeriesNumber = 0;
    int ColumnEnd_Pointer = 0;
    int PosPlate = 0;
    double SampleVol = 0;
    double DilVol = 0;
    double TotalVol = 0;
    double DeadVol = 50;
    double MixVol = 0;
    int Cntr_DestSeq = 0;
    int Cntr_DestSeq_End = 64;
    int sourceSeq_SampleStartPos = 0;
    int PK_PreAssay_SourceSeq_Int;
    
    String worklistType = "0";													//1= Waton, 2 = Gyros .pdf
    String[][] array_PK_Input = new String[1000][6];							//Array containing every line from worklist split into elements
    String[][] arraySamplesUnique = new String[1000][20];						//Array containing each Unique Sample line from worklist split into elements

    ArrayList<String> arraylistPK_AllUnique = new ArrayList<String>();			//Temp holds list of unique sample ID to determine if each is unique

    PK_WorklistTotalRecords = 0;
    PK_WorklistUniqeSamples = 0;
    PK_Dilution_SamplesCntr = 0;
    PK_WorklistUniqeAll = 0;
    
    Boolean PK_UniqueTubeID;
    
	arraylistPK_VolumeFinal.clear();
	arraylistPK_VolumeFinal_Diluent.clear();
	arraylistPK_VolumeFinal_Total.clear();
    for (int i = 0; i < 100; i++){
    	arraylistPK_VolumeFinal.add("");
    	arraylistPK_VolumeFinal_Diluent.add("");
    	arraylistPK_VolumeFinal_Total.add("");
    }
    
    
    if (PK_Worklist.contains(".xls")){
    	worklistType = "3";
    }else if (PK_Worklist.contains(".pdf")){
		worklistType = "2";
	}else if (PK_Worklist.contains(".txt")){
		worklistType = "1";
	}else {
		worklistType = "0";
	}

	if (worklistType.equals("1")){
	     /////////////////////////////////	
	     //Read in all info from Watson Worklist	
	      try {
	    	  File f = new File(PK_Worklist);
	    	  input = new FileReader(f);
			  BufferedReader br = new BufferedReader(input);
			  br.readLine();
			  while ((newline = br.readLine()) != null) {
				  String Sample; if (newline.split(tabChar)[5].equals("")){Sample = "false";}else{Sample = "true";}						//Determine if line is for Sample vs STD/CNTRL/ETC
				  PK_WorklistTotalRecords += 1;																							//Note that array_PK_Input[] becomes 1 based
				  array_PK_Input[PK_WorklistTotalRecords][0] = "1";																		//Plate ID		Only 1 assay plate supported.  AP confirms not more than 96 records in file.
				  array_PK_Input[PK_WorklistTotalRecords][1] = "PK_WorklistTotalRecords";												//Line ID						
				  array_PK_Input[PK_WorklistTotalRecords][2] = newline.split(tabChar)[0];												//Sample ID
				  array_PK_Input[PK_WorklistTotalRecords][3] = newline.split(tabChar)[12];												//Dilution
				  array_PK_Input[PK_WorklistTotalRecords][4] = AlphaToNumeric.ToNumeric_WithZeros(newline.split(tabChar)[18]);			//AssayPlate Well.  Converted to numeric
				  array_PK_Input[PK_WorklistTotalRecords][5] = Sample;																	//boolean indicating if this is a SAMPLE or not.
		
				  arraylistPK_SamplesIDAll.add(newline.split(tabChar)[0]);
				  
			  }
			  br.close();
		} catch (Exception e) {
			JOptionPane.showMessageDialog(null, "VAS Error: error parsing Watson file\n\nerror: " + e);
			e.printStackTrace();
		}
	}
	
	if (worklistType.equals("2")){
	    ////////////////////////////////////////	
	    //Read in all info from Gyros Worklist
    	String gyroPdf_filePath = PK_Worklist;
    	String gyroPdfParsedRaw_filePath = "C:\\Program Files\\Telos\\AutoPilot\\System\\gyrolabParsedInterm.txt";							//Intermediate text file that is created from parsed .pdf.  This file will then be opened and processed below
    	String gyroPdfParsedFinal_filePath = "C:\\Program Files\\Telos\\AutoPilot\\Data\\PK_GyrolabParsed" + gyroPdf_filePath.substring(gyroPdf_filePath.lastIndexOf("\\") , gyroPdf_filePath.lastIndexOf(".")+1) + "txt";							//Text file containing formatted parsed sample info from original .pdf
    	String gyroPdfParsedRaw_textAll = "";																								//All text from .pdf.  Search this for "GyroLab" to confirm the file is a GyroLab "LoadList"

        boolean tableOn = false;
		int plateNumberInt = 0;
		int eSampleNumber;
		int eIDend;
		int eDil;
		int eWell;
		int cntr1 = 0;
    	String id = "";
    	String SampleType = "";
		String currentSampleNumber;
		String nextReadSampleNumber;
		String nextRequiredSampleNumber;
		String writeString;
    	ArrayList<String> arrayList_Interm = new ArrayList<String>();																//arrayList to hold each line from Intermediate text file
    	
    	try {																														//Extract info from PDF to intermediate text file.
			PdfReader reader = new PdfReader(gyroPdf_filePath);
			PdfReaderContentParser parser = new PdfReaderContentParser(reader);
			PrintWriter outInterm = new PrintWriter(new FileOutputStream(gyroPdfParsedRaw_filePath));
			TextExtractionStrategy strategy;
			for (int i = 1; i <= reader.getNumberOfPages(); i++) {
			    strategy = parser.processContent(i, new SimpleTextExtractionStrategy());
			    outInterm.println(strategy.getResultantText());
			    gyroPdfParsedRaw_textAll = gyroPdfParsedRaw_textAll + strategy.getResultantText();
			}
			outInterm.flush();
			outInterm.close();
			reader.close();
		} catch (Exception e) {
			JOptionPane.showMessageDialog(null, "VAS Error: error parsing GyroLab file\n\nerror: " + e);
			e.printStackTrace();
			return;
		}
        
    	if (!gyroPdfParsedRaw_textAll.contains("Gyrolab")){
    		JOptionPane.showMessageDialog(null, "VAS Error: error parsing GyroLab file\n\nerror: File format not recognized"); return;
    	}
    	
		try {
	    	FileReader input;
			File fGyros = new File(gyroPdfParsedRaw_filePath);
	        input = new FileReader(fGyros);
			BufferedReader br = new BufferedReader(input);
			while ((newline = br.readLine()) != null) {											//create arrayList_Interm from pdf
			    
				if (newline.contains("R-MP1")){break;}
				if (newline.equals("Gyrolab Control Loading")){tableOn = false;}
				if (newline.matches("MP._CD.*")){tableOn = false;}

				if (tableOn){
					arrayList_Interm.add(newline);
				}
				
				if (newline.equals("Pos Reagent/Sample Concentration Vol. [l] Well")){tableOn = true;}
			  }
			br.close();
		} catch (Exception e) {
			e.printStackTrace();
		}

        for (int i = 0; i < arrayList_Interm.size(); i++){									//parse fields correctly into array_PK_Input
        	
        	String currentString = arrayList_Interm.get(i);
        	currentSampleNumber = arrayList_Interm.get(i).split(" ")[0];
        	nextRequiredSampleNumber = String.valueOf(Integer.parseInt(currentSampleNumber) + 1);
        	if ((i+1)<arrayList_Interm.size()){nextReadSampleNumber = arrayList_Interm.get(i+1).split(" ")[0];}else{nextReadSampleNumber="0";}
        	
        	writeString = currentString;
        	System.out.println("Current - " + currentSampleNumber);
        	System.out.println("Next Required - " + nextRequiredSampleNumber);
        	System.out.println("Next Read - " + nextReadSampleNumber);
        	//System.out.println("Next array - " + arrayList_Interm.get(i+1));
        	if (!nextReadSampleNumber.equals(nextRequiredSampleNumber) && !nextReadSampleNumber.equals("0")){					//Input lines more than 36 characters split to three different lines
        		String idLine2 = arrayList_Interm.get(i+1);															//Second part of sample id located on second line

        		if (currentString.matches(".*\\s\\d\\s\\D\\d{1,2}")){ 												//Line includes vol and well info indicating that second line of name is on next page. 
        			String idLine1 = writeString.split("\\s\\d\\s\\D\\d{1,2}")[0];			//Strip vol and well from name
        			String volWell = writeString.substring(idLine1.length());				//Record vol and well
        			writeString = idLine1 + " " + idLine2 + volWell;						//Rebuild NameLine1 + NameLine2 + VolWell (parsed from line1)
        			i++;
        		}else{
        			String volWell = arrayList_Interm.get(i+2);								//Record vol and well
	         		writeString = writeString + " " + idLine2 + " " + volWell;				//Rebuild NameLine1 + NameLine2 + VolWell (from line 3)
	        		i++; i++;       			
        		}
        	}
        	
        	eSampleNumber = 0;
    		id="";
        	eDil = writeString.split(" ").length - 3;
        	eWell = writeString.split(" ").length - 1;
        	
        	if (writeString.contains("STD") || writeString.contains("QC") || writeString.contains("blank") || writeString.contains("BLANK") ){eIDend = 2; SampleType = "false";}else{eIDend = 3; SampleType = "true";}
        	if (writeString.split(" ")[eSampleNumber].equals("1")){plateNumberInt++;}
        	for (int a = 1; a < writeString.split(" ").length - eIDend; a++){id = id+writeString.split(" ")[a]+" ";}
        	
        	cntr1++; 																							//Note that array_PK_Input[] becomes 1 based
        	PK_WorklistTotalRecords++;
        	
        	array_PK_Input[cntr1][0] = String.valueOf(plateNumberInt);											//Plate ID
        	array_PK_Input[cntr1][1] = writeString.split(" ")[eSampleNumber];									//Line ID
        	array_PK_Input[cntr1][2] = id;																		//Sample ID
        	array_PK_Input[cntr1][3] = writeString.split(" ")[eDil];											//Dilution ID
        	array_PK_Input[cntr1][4] = AlphaToNumeric.ToNumeric_NoZeros(writeString.split(" ")[eWell]);			//AssayPlate Well.  Converted to numeric
        	array_PK_Input[cntr1][5] = SampleType;																//boolean indicating if this is a SAMPLE or not.
		}
        try {
			PrintWriter out = new PrintWriter(new FileOutputStream(gyroPdfParsedFinal_filePath));
			for (int a = 1; a < PK_WorklistTotalRecords+1; a++){
				out.println(array_PK_Input[a][0]+"\t"+array_PK_Input[a][1]+"\t"+array_PK_Input[a][2]+"\t"+array_PK_Input[a][3]+"\t"+array_PK_Input[a][4]+"\t"+array_PK_Input[a][5]);
				}
			out.flush();
			out.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	if (worklistType.equals("3")){																										//START PKF0033
	     /////////////////////////////////	
	     //Read in all info from Gyros Wizard Generated File
	      try {
	    	  File f = new File(PK_Worklist);
	    	  input = new FileReader(f);
			  BufferedReader br = new BufferedReader(input);
			  br.readLine();
			  
			  while (!(newline = br.readLine()).contains("Index")) {}																			//Advance to table
			  
			  while ((newline = br.readLine()) != null) {																						//Iterate over table
				  String Sample; if (newline.split(tabChar)[0].equals("")){Sample = "false";}else{Sample = "true";}		//Determine if line is for Sample vs STD/CNTRL/ETC
				  PK_WorklistTotalRecords += 1;																									//Note that array_PK_Input[] becomes 1 based
				  array_PK_Input[PK_WorklistTotalRecords][0] = "1";																				//Plate ID		Only 1 assay plate supported.  AP confirms not more than 96 records in file.
				  array_PK_Input[PK_WorklistTotalRecords][1] = "PK_WorklistTotalRecords";														//Line ID						
				  array_PK_Input[PK_WorklistTotalRecords][2] = newline.split(tabChar)[2];														//Sample ID
				  array_PK_Input[PK_WorklistTotalRecords][3] = newline.split(tabChar)[5];														//Dilution
				  array_PK_Input[PK_WorklistTotalRecords][4] = AlphaToNumeric.ToNumeric_NoZeros(newline.split(tabChar)[7]);						//AssayPlate Well.  Converted to numeric
				  array_PK_Input[PK_WorklistTotalRecords][5] = Sample;														
			  }

			  br.close();
		} catch (Exception e) {
			e.printStackTrace();
		}																																//END PKF0033
	}//END FILE PARSING
	

	
	//START PROCESSING RECORDS/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    try {
      for (int x = 1; x < PK_WorklistTotalRecords+1; x++){														//Iterate over each record from input file
    	  
    	PK_UniqueTubeID = false;
    	  	  
    	if (!arraylistPK_AllUnique.contains(array_PK_Input[x][2])){												//Determine if this is a unique ID
    		arraylistPK_AllUnique.add(array_PK_Input[x][2]);													//Add new ID to unique list
    		PK_UniqueTubeID = true;																				//Record that this IS a unique ID
    		PK_WorklistUniqeAll+=1;																				//Record total number of unique IDs
    	}

    	if (array_PK_Input[x][5].equals("false")){																//NON SAMPLES//
    		PK_StdCntrl_ReadLine_Total +=1;																		//Recocrd total number of non samples RECORDS	
      	  if (!arraylistPK_StdCntrls.contains(array_PK_Input[x][2])){											//Record this Standard or Control ID if not yet recorded 
      		arraylistPK_StdCntrls.add(array_PK_Input[x][2]);													//Record Each unique STD/CNTRL									
      		PK_StdCntrl_Unique_Total+=1;																		//Recocrd total number of non samples UNIQUE
      		PK_Volume_MRD = PK_Volume_MRD + (MRD_SampleVol) + ",";  											//Record volume of this Std/QC to transfer to MRD plate.
    	  }  
      	}else{																									//SAMPLES// 	
  	        if (PK_UniqueTubeID){																				//Record unique Samples
  	        	PK_WorklistUniqeSamples+=1;
  	        	arraySamplesUnique[PK_WorklistUniqeSamples] = array_PK_Input[x];
  	        	arraylistPK_SamplesUnique.add(arraySamplesUnique[PK_WorklistUniqeSamples][2]);
  	        	
  	        }	      
  	        //Create SAMPLE Source and Destination sequences for Pre Assay.
  	        if (PK_GoDilution.equals("1")){  //IF Dilution is requested
	   	        PK_PreAssay_SourceSeq_Int = arraylistPK_SamplesUnique.indexOf(array_PK_Input[x][2])+65;     	
  	        }else{							//IF Dilution is not requested than samples will come from MRD plate simply in order starting after STD/QC..
  	        	PK_PreAssay_SourceSeq_Int = (arraylistPK_SamplesUnique.indexOf(array_PK_Input[x][2])+1) + PK_StdCntrl_Unique_Total;
  	        }
  	        PK_PreAssay_SourceSeq = PK_PreAssay_SourceSeq + String.valueOf(PK_PreAssay_SourceSeq_Int) + ",";	
  	        PK_PreAssay_DestinationSeq = PK_PreAssay_DestinationSeq + array_PK_Input[x][4] + ",";
  	        
  	        PK_PreAssay_SourceSeq_DilutionRecovery = PK_PreAssay_SourceSeq_DilutionRecovery + String.valueOf(arraylistPK_SamplesUnique.indexOf(array_PK_Input[x][2])+65) + ",";  
  	        
      	}    	
      	PK_Worklist_SampleID = array_PK_Input[x][2];
      }
      
      if(PK_WorklistUniqeSamples == 0){PK_GoDilution = "0";}	//Accommodates scenario where Dilution is requested but file contains onlys STDs and QC (i.e. NO SAMPLES TO BE DILUTED!).
      
      ////////////////////////////////////////////////////
      // PKF0036
      if (PK_GoMRD.equals("1")){
	      if (PK_LabType_Std.contains("_Tube") && !PK_LabType_Sample.contains("_Tube")){
	    	  sourceSeq_SampleStartPos = 0;
	      }else{
	    	  sourceSeq_SampleStartPos = PK_StdCntrl_Unique_Total;
	      }    	  
      }

       ///////////////////////////////////////////////////
      //Record required tips for MRD, and Assay
      if (PK_GoMRD.equals("1")){
    	  PK_Tip50iccount += PK_WorklistUniqeAll;//Sample
    	  PK_Tip1000iccount += PK_WorklistUniqeAll;//Diluent
      }
      if (PK_GoAssay.equals("1")){
    	  
    	  PK_Tip30096count += 1;
      }else{
    	  PK_Tip30096count += 0;
      }

      ////////////////////////////////////////////////////////
      // Create Standards and Control sequences for Pre Assay
     for (int x = 1; x < PK_StdCntrl_ReadLine_Total+1; x++){
    	  PK_PreAssay_StdCntr_SourceSeq = PK_PreAssay_StdCntr_SourceSeq + (arraylistPK_StdCntrls.indexOf(array_PK_Input[x][2]) + 1) + ",";
    	  PK_PreAssay_StdCntr_DestinationSeq = PK_PreAssay_StdCntr_DestinationSeq + array_PK_Input[x][4] + ",";
      }

      /////////////////////////////////////////////////////////////////////////////
      //Loop over each unique Sample record and prepare for all required dilutions
      for (int x = 1; x < PK_WorklistUniqeSamples+1; x++){
      	
    	  		PK_Dilution_SamplesCntr +=1;
    	  		TotalVol = 0;

	        	PK_Worklist_SampleID = arraySamplesUnique[x][2];
	        	PK_Worklist_Dilution = arraySamplesUnique[x][3].split("\\.")[0];			//update-pk002 - only take integer component of dilution
	        	arraylistPK_DilutionFinal.add(PK_Worklist_Dilution);						//Used for Report
	        	arrayPK_SeqSource.add(String.valueOf(PK_Dilution_SamplesCntr));
	        	arrayPK_Dilutions.add(String.valueOf(PK_Worklist_Dilution));
	        	
	        	arrayPK_Dilutions_BatchOfEight[BatchOfEight] = Integer.parseInt(PK_Worklist_Dilution);
	        	
	        	BatchOfEight +=1;
	        	
	        	if (PK_GoMRD.equals("1")){	
    	        	PK_SourceSeq = PK_SourceSeq + String.valueOf(PK_Dilution_SamplesCntr + sourceSeq_SampleStartPos) + ",";				//Starts at first unique sample and iterates from there

	        		if (Integer.parseInt(PK_Worklist_Dilution)==1){
	        			PK_Volume_MRD = PK_Volume_MRD + MRD_SampleVolNeat + ",";  
	        		}else{

	        			PK_Volume_MRD = PK_Volume_MRD + MRD_SampleVol + ",";													
	        		}
	        	}
	 
    	  		if (PK_GoDilution.equals("1")){
		        	if (BatchOfEight == 8 || x == PK_WorklistUniqeSamples){
		        		BatchSizeLast = BatchOfEight;
		        		Batches = 0;
		        		SeriesNumber +=1;
	        			if (SeriesNumber == 2){ColumnEnd_Pointer = 1; PosPlate = 16;}
		        		if (SeriesNumber == 3){ColumnEnd_Pointer = 2; PosPlate = 32;}
		        		if (SeriesNumber == 4){ColumnEnd_Pointer = 3; PosPlate = 48;}
		        		if (SeriesNumber == 5){ColumnEnd_Pointer = 0; PosPlate = 0;}		//update-pk002
		        		if (SeriesNumber == 6){ColumnEnd_Pointer = 1; PosPlate = 16;}		//update-pk002
		        		if (SeriesNumber == 7){ColumnEnd_Pointer = 2; PosPlate = 32;}		//update-pk002
		        		if (SeriesNumber == 8){ColumnEnd_Pointer = 3; PosPlate = 48;}		//update-pk002
		        		if (SeriesNumber == 9){ColumnEnd_Pointer = 0; PosPlate = 0;}		//update-pk002
		        		if (SeriesNumber == 10){ColumnEnd_Pointer = 1; PosPlate = 16;}		//update-pk002
		        		if (SeriesNumber == 11){ColumnEnd_Pointer = 2; PosPlate = 32;}		//update-pk002
		        		if (SeriesNumber == 12){ColumnEnd_Pointer = 3; PosPlate = 48;}		//update-pk002
		        		
		        		System.out.println("SeriesNumber - " + SeriesNumber);
		        		for (int a = 1; a < 4; a++){		//Three Total Transfers Per "BatchOfEight"
		        			ChannelUsage = "";
		        			Batches +=1;
		        			
			        		for (int i = 0; i < BatchOfEight; i++){
			        			PosPlate +=1;
			        			finalTransfer = false;
			        			////////////////////////////////////////////////////////////////
			        			//Create Sequence (PK_DestSeq) for pipetting to Dilution Plate.
			        			if ((arrayPK_Dilutions_BatchOfEight[i]/100 < 1) && (arrayPK_Dilutions_BatchOfEight[i] != 0)){// Add to final destination sequence
			        				Cntr_DestSeq_End = (64 + ((i+1) + (8*(ColumnEnd_Pointer))));	//update-pk002 ColumnEnd_Pointer replaced SeriesNumber-1
			        				Cntr_DestSeq = Cntr_DestSeq_End;
			        				finalTransfer = true;
			        				System.out.println("END - " + Cntr_DestSeq);
			        			}else {
			        				Cntr_DestSeq = PosPlate;// Add to intermediate destination sequence
			        				System.out.println("MID - " + Cntr_DestSeq);
			        			}
			        			PK_DestSeq = PK_DestSeq + Cntr_DestSeq + ",";//BUILD THE SEQUENCE HERE
			        			
			        			//////////////////////////////////////////////////////////////////////////
			        			//Determine Channels Patterns and Volumes for each transfer in the series
			        			if (arrayPK_Dilutions_BatchOfEight[i] <= 1){// No Dilution required
			        				
			        				if (arrayPK_Dilutions_BatchOfEight[i] == 1){ //Neat requested
			        					ChannelUsage = ChannelUsage + "1";
			        					SampleVol = ((Integer.parseInt(PK_PreAssay_Vol_Read)*2)+70);
			        					if (SampleVol <= 50){
			        						PK_Tip50iccount += 1;//Sample 
			        					}else{
			        						PK_Tip1000iccount += 1;//Sample 
			        					}
			        				}else{  //Subsequent transfers already performed.  No remaining transfers required.
			        					ChannelUsage = ChannelUsage + "0";
			        					SampleVol = 0;
			        				}
			        				arrayPK_Dilutions_BatchOfEight[i] = 0;// Record no more dilutions required
		        					
			        				PK_Volume_DilutionSeries_Sample = PK_Volume_DilutionSeries_Sample + String.valueOf(SampleVol) + ",";
			        				PK_Volume_DilutionSeries_Diluent = PK_Volume_DilutionSeries_Diluent + "0,";
			        				PK_Volume_DilutionSeries_Mix = PK_Volume_DilutionSeries_Mix + "0,";
			        				DilVol = 0;
			        				
			        			}else{// Dilution required
			        				
			        				ChannelUsage = ChannelUsage + "1";
		        					PK_Tip1000iccount += 1;  //Diluent
			        				
			        				if (arrayPK_Dilutions_BatchOfEight[i] >= 1000){//Intermediate Dilution. Perform a 1:100 into 1000ul total
			        					arrayPK_Dilutions_BatchOfEight[i] = arrayPK_Dilutions_BatchOfEight[i] / 100;
				        				
				        				PK_Volume_DilutionSeries_Sample = PK_Volume_DilutionSeries_Sample + "10,";
			        					PK_Volume_DilutionSeries_Diluent = PK_Volume_DilutionSeries_Diluent + "990,";
			        					PK_Volume_DilutionSeries_Mix = PK_Volume_DilutionSeries_Mix + "920,";
			        					PK_Tip50iccount += 1;//Sample
	
			        				}else if (arrayPK_Dilutions_BatchOfEight[i] >= 100.0){//Intermediate Dilution. Perform a 1:10 into 100ul total
			        					arrayPK_Dilutions_BatchOfEight[i] = arrayPK_Dilutions_BatchOfEight[i] / 10;
				        				PK_Volume_DilutionSeries_Sample = PK_Volume_DilutionSeries_Sample + "30,";
			        					PK_Volume_DilutionSeries_Diluent = PK_Volume_DilutionSeries_Diluent + "270,";
			        					PK_Volume_DilutionSeries_Mix = PK_Volume_DilutionSeries_Mix + "225,";
			        					PK_Tip50iccount += 1;  //Sample
	
			        				}else{
			        							
			        					if (arrayPK_Dilutions_BatchOfEight[i]<10){
			        						TotalVol = 250;
			        						if (arrayPK_Dilutions_BatchOfEight[i]<5){
					        					PK_Tip1000iccount += 1;  //Sample
			        						}else{
			        							PK_Tip50iccount += 1;  //Sample
			        						}
			        					}else{
			        						TotalVol = 500;
			        						PK_Tip50iccount += 1;  //Sample
			        					}
			        					
			        					SampleVol = TotalVol/arrayPK_Dilutions_BatchOfEight[i];
			        					DilVol = (TotalVol-SampleVol);
			        					MixVol = TotalVol - DeadVol;
			        					if (MixVol > 1000){
			        						MixVol = 1000;
			        					}
			        					
			        					PK_Volume_DilutionSeries_Sample = PK_Volume_DilutionSeries_Sample + String.format("%.1f", SampleVol) + ",";
			        					PK_Volume_DilutionSeries_Diluent = PK_Volume_DilutionSeries_Diluent + String.format("%.1f", DilVol) + ",";
			        					PK_Volume_DilutionSeries_Mix = PK_Volume_DilutionSeries_Mix + String.format("%.1f", MixVol) + ",";
			        					
			        					arrayPK_Dilutions_BatchOfEight[i]=0;//recording that no additional transfers are required after this one
			        				}	
			        			}
			        			
			        			if (finalTransfer){
			        				arraylistPK_VolumeFinal.set(i + ((SeriesNumber-1)*8), String.format("%.1f", SampleVol));
			        				arraylistPK_VolumeFinal_Diluent.set(i + ((SeriesNumber-1)*8), String.format("%.1f", DilVol));
			        				arraylistPK_VolumeFinal_Total.set(i + ((SeriesNumber-1)*8), String.format("%.1f", SampleVol+DilVol));
			        				
			        				
			        				System.out.println("Pos - " + String.valueOf(i + ((SeriesNumber-1)*8)));
			        				System.out.println("vol - " + String.format("%.1f", SampleVol));
			        				System.out.println("dil - " + String.format("%.1f", DilVol));
			        				System.out.println("total - " + String.format("%.1f", SampleVol+DilVol));
			        				System.out.println("=======================");
			        				
			        			}
			        		}	
			        	
			        		if (x == PK_WorklistUniqeSamples){
			        			for (int i = 0; i < (8-BatchSizeLast); i++){
			        				PK_Volume_DilutionSeries_Sample = PK_Volume_DilutionSeries_Sample + "0,";
			        				PK_Volume_DilutionSeries_Diluent = PK_Volume_DilutionSeries_Diluent + "0,";
			        				PK_Volume_DilutionSeries_Mix = PK_Volume_DilutionSeries_Mix + "0,";
			        				ChannelUsage = ChannelUsage + "0";
				        			PosPlate +=1;
				        			Cntr_DestSeq = PosPlate;
				        			PK_DestSeq = PK_DestSeq + Cntr_DestSeq + ",";
		        				}
			        		}
			        		PK_ChannelPatternsString = PK_ChannelPatternsString + ChannelUsage + ",";
		        		}
		        		
		        		BatchOfEight = 0;
		        		PK_BatchTracker = PK_BatchTracker + String.valueOf(Batches) + ",";	
		        	}    	  			
    	  		}else{
    	  			if (BatchOfEight == 8 || x == PK_WorklistUniqeSamples){
		        		BatchOfEight = 0;
		        		PK_BatchTracker = PK_BatchTracker + String.valueOf(Batches) + ",";	
    	  			}
    	  		}
      }
	
      //Determine number of required Dilution Plates
      if (PK_GoDilution.equals("1")){
    	  PK_Multi_DilutionPlates = String.valueOf((int)(Math.ceil(PK_Dilution_SamplesCntr / 32.0)));
      }else{
    	  PK_Multi_DilutionPlates = "1";
      }
      
      //Remove trailing ","s
      if (PK_GoMRD.equals("1")){
    	  if (PK_SourceSeq.length() > 0){PK_SourceSeq = PK_SourceSeq.substring(0, PK_SourceSeq.length() -1);}				//Source Sequence for SAMPLES to MRD PLATE (i.e. out of STOCK plate/tubes.  Only applicable for Samples, not stdrs QC).  Only Applicable if SAMPLES are present.

    	  PK_Volume_MRD = PK_Volume_MRD.substring(0, PK_Volume_MRD.length() -1);
      }
      if (PK_GoDilution.equals("1")){
	      PK_DestSeq = PK_DestSeq.substring(0, PK_DestSeq.length() -1);						//Destination Sequence for DILUTION PLATE  
	      PK_ChannelPatternsString = PK_ChannelPatternsString.substring(0, PK_ChannelPatternsString.length()-1);
	      PK_Volume_DilutionSeries_Sample = PK_Volume_DilutionSeries_Sample.substring(0, PK_Volume_DilutionSeries_Sample.length() -1);
	      PK_Volume_DilutionSeries_Diluent = PK_Volume_DilutionSeries_Diluent.substring(0, PK_Volume_DilutionSeries_Diluent.length() -1);
	      PK_Volume_DilutionSeries_Mix = PK_Volume_DilutionSeries_Mix.substring(0, PK_Volume_DilutionSeries_Mix.length() -1);  
	      PK_BatchTracker = PK_BatchTracker.substring(0, PK_BatchTracker.length() -1);
      }
      if (PK_GoPreAssay.equals("1")){
          if (PK_PreAssay_SourceSeq.length() >1){//Only applicable if there are SAMPLES present
	    	  PK_PreAssay_SourceSeq = PK_PreAssay_SourceSeq.substring(0, PK_PreAssay_SourceSeq.length() -1);
	          PK_PreAssay_DestinationSeq = PK_PreAssay_DestinationSeq.substring(0, PK_PreAssay_DestinationSeq.length() -1); 
	          PK_PreAssay_SourceSeq_DilutionRecovery = PK_PreAssay_SourceSeq_DilutionRecovery.substring(0, PK_PreAssay_SourceSeq_DilutionRecovery.length() -1);
	          
          }

          if (PK_PreAssay_StdCntr_SourceSeq.length() >0){//Only applicable if there are STANDARDS/CONTROLS present
        	  PK_PreAssay_StdCntr_SourceSeq = PK_PreAssay_StdCntr_SourceSeq.substring(0, PK_PreAssay_StdCntr_SourceSeq.length() -1);
        	  PK_PreAssay_StdCntr_DestinationSeq = PK_PreAssay_StdCntr_DestinationSeq.substring(0, PK_PreAssay_StdCntr_DestinationSeq.length() -1);        	  
          }
 }

      input.close();
    } catch (Exception e) {
    	e.printStackTrace();
    	JOptionPane.showMessageDialog(null, "Error Processing PK STEP\n\nerror: " + e);
    }
  }
  
  //Serial Dilution Logic
  public static void SD_ChanPattern(){
	  for (int a = 0; a < SWN; a++){
		  ChannelPattern = ChannelPattern + "1";
		
	  }
	  for (int a = 0; a < CNS-SWN; a++){
		  ChannelPattern = ChannelPattern + "0";
		
	  }
  }
  
  public static void SD_SeqPrep() {
      //Source labware will consist of the original Source seq plus the Dest Seq (minus the last transfer worth of positions)
	  SD_SourceSeq = "";
	  SD_SourceSeq = SourceSeqNative + ","; 
	  for (int x=0; x<(DestSeqNative.split(",").length-SWN); x++){
		  SD_SourceSeq = SD_SourceSeq + DestSeqNative.split(",")[x] + ",";
	  } 
	  SD_SourceSeq = SD_SourceSeq.substring(0, SD_SourceSeq.length() - 1);  //Remove trailing comma
      
      
      SD_DestSeq = DestSeqNative;

  }
  public static String LIQUID_VOLATILE(String Liquid) {
	    
	    if (Liquid.equals("EtOH")){
	    	return "1";
	    }
	    if (Liquid.equals("Methanol")){
	    	return "1";
	    }
	    if (Liquid.equals("Acetonitrile")){
	    	return "1";
	    }
	    
	    return "0";
	  }
  
  public static String MixAuto_Setup(String labwareFile, String pipettingTip) {    
	    return "0";
  }
  
  public static void CORDNA_FileProcessing()
  {
    File f = new File(CORDNA_Worklist);
    String newline = "";
    String tabChar = "\t";
    String ChP1000;
    String ChP300;
    String ChP50;

    int column_ID = -1;
    int column_ConcStock = -1;
    int column_FinalPosition = -1;
    
	double VolumeRemaining;
	double VolumeThisTransfer;
	double MixVolume;
	int diluentSeqCntr;
	int stockSeqS_Cntr = Integer.parseInt(CORDNA_LabwareStock_Pos);
	int stockSeqD_Cntr;
	int batchOf8_Diluent_Cntr = 0;
	int batchOf8_Stock_Cntr = 0;
	DecimalFormat df1 = new DecimalFormat("#.0");
	
	double concTarget = Integer.parseInt(CORDNA_TargetConc);
    double volTarget = Integer.parseInt(CORDNA_TargetVol);

    if (CORDNA_LabwareNorm_Type.equals("1")){
    	stockSeqD_Cntr = Integer.parseInt(CORDNA_LabwareNorm_Pos);
    	diluentSeqCntr = Integer.parseInt(CORDNA_LabwareNorm_Pos);
    }else{
    	stockSeqD_Cntr = 1;
    	diluentSeqCntr = 1;
    }
    
    try {
      input = new FileReader(f);
      BufferedReader br = new BufferedReader(input);
      newline = br.readLine();//Read header row
      
      //Find each required column in worklist file
      for(int i =0;i<newline.split(tabChar).length;i++){
    	  if (newline.split(tabChar)[i].equals("Ref")){column_ID = i;}
    	  if (newline.split(tabChar)[i].equals("Concentration")){column_ConcStock = i;}
    	  
    	  if (CORDNA_Procedure.equals("2")){
    		  if (newline.split(tabChar)[i].equals("Final_position")){column_FinalPosition = i;}  
    	  }else {column_FinalPosition = 0;}
      }
      
      //Confirm that required columns are present
      if (column_ID == -1 || column_ConcStock == -1 || column_FinalPosition == -1){
    	  VAStools.errorMessageProcess("Step::" + Step + "::VAS ERROR::The Selected Q File is missing some required information");
      }
      
      //Process file and determine required transfer volumes
      while ((newline = br.readLine()) != null) {//Read data rows
        CORDNA_SamplesTotal += 1;
        
        if (CORDNA_Procedure.equals("1")){
        

	        //calculate and recored the required transfer volumes   
	        double concCurrent = (int)(Math.ceil (Double.parseDouble(newline.split(tabChar)[column_ConcStock]) * 1000));
	        double transferVol_Sample = Double.parseDouble(df1.format((concTarget / concCurrent) * volTarget));
	  	  	double transferVol_Diluent = (volTarget - transferVol_Sample);
	  	  	
	  	  	if (concCurrent<concTarget) {VAStools.errorMessageProcess("Step::" + Step + "::VAS ERROR::Some of the Sample Concentrations are lower than the Target Concentration.");}
	  	  	if (transferVol_Sample<1)	{VAStools.errorMessageProcess("Step::" + Step + "::VAS ERROR::The requested routine results in Sample Volumes less than 1ul.");}
			  
	  	  	arrayList_CORDNA_VolSource.add(String.valueOf( transferVol_Sample ));
	  	  	arrayList_CORDNA_VolDiluent.add(String.valueOf( transferVol_Diluent ));
	  	  	VolumeRemaining = transferVol_Diluent;
	  	  	
	        arrayList_CORDNA_IDs.add(newline.split(tabChar)[column_ID]);
	        arrayList_CORDNA_Conc.add(String.valueOf(concCurrent));
	        
	  	  	while (VolumeRemaining > 0){									//Diluent
	  	  		batchOf8_Diluent_Cntr +=1;
	  	  		if (VolumeRemaining > 1000){
	  	  			VolumeThisTransfer = 1000.0;
	  	  		}else{
	  	  			VolumeThisTransfer = VolumeRemaining;
	  	  		}
	  	  		MixVolume = (int)(Math.ceil(VolumeThisTransfer * 0.75));
	  	  		VolumeRemaining = VolumeRemaining - 1000;		
	  	  		//Channel Patterns and Tip Counting
	  	  		if (VolumeThisTransfer > 300){
	  	  			ChP1000 = "1"; ChP300 = "0"; ChP50 = "0";  Tip1000iccount += 1;
	  	  			CORDNA_OutS_Diluent_LiquidClass = CORDNA_OutS_Diluent_LiquidClass + "1000tip_Water_pipICsurface" + ",";
	  	  		}else if (VolumeThisTransfer > 50){
	  	  			ChP1000 = "0"; ChP300 = "1"; ChP50 = "0";  Tip300iccount += 1;
	  	  			CORDNA_OutS_Diluent_LiquidClass = CORDNA_OutS_Diluent_LiquidClass + "300tip_Water_pipICsurface" + ",";
	  	  		}else{
	  	  			ChP1000 = "0"; ChP300 = "0"; ChP50 = "1";  Tip50iccount += 1;
	  	  			CORDNA_OutS_Diluent_LiquidClass = CORDNA_OutS_Diluent_LiquidClass + "50tip_Water_pipICsurface" + ",";
	  	  		}
	  	  		
	  	  	  	CORDNA_OutS_Dil_Vol = CORDNA_OutS_Dil_Vol + String.valueOf(VolumeThisTransfer) + ",";
	  	  	  	CORDNA_OutS_Dil_MixVol = CORDNA_OutS_Dil_MixVol + String.valueOf(MixVolume) + ",";
	  	  	  	CORDNA_OutS_Dil_SeqD = CORDNA_OutS_Dil_SeqD + String.valueOf(diluentSeqCntr) + ",";
	  	  	  	CORDNA_OutS_Dil_ChP_T1000 = CORDNA_OutS_Dil_ChP_T1000 + ChP1000 + ",";
	  	  	  	CORDNA_OutS_Dil_ChP_T300 = CORDNA_OutS_Dil_ChP_T300 + ChP300 + ",";
	  	  	  	CORDNA_OutS_Dil_ChP_T50 = CORDNA_OutS_Dil_ChP_T50 + ChP50 + ",";
	  	  	  	
	  	  	  	if (batchOf8_Diluent_Cntr == 8){
//	  	  	  		CORDNA_OutS_Dil_ChP_T1000 = CORDNA_OutS_Dil_ChP_T1000 + ",";
//	  	  	  		CORDNA_OutS_Dil_ChP_T300 = CORDNA_OutS_Dil_ChP_T300 + ",";
//	  	  	  		CORDNA_OutS_Dil_ChP_T50 = CORDNA_OutS_Dil_ChP_T50 + ",";
	  	  	  		batchOf8_Diluent_Cntr = 0;
	  	  	  	}
	  	  	}
	  	
	  	  	
	  	  	VolumeRemaining = transferVol_Sample;
	  	  	while (VolumeRemaining > 0){									//Stock
	  	  		batchOf8_Stock_Cntr += 1;
	  	  		if (VolumeRemaining > 1000){
		  			VolumeThisTransfer = 1000;
		  		}else{
		  			VolumeThisTransfer = VolumeRemaining;
		  		}
		  		VolumeRemaining = VolumeRemaining - 1000;		
		  		
		  		//Channel Patterns and Tip Counting
		  		if (VolumeThisTransfer > 300){
		  			ChP1000 = "1"; ChP300 = "0"; ChP50 = "0";  Tip1000iccount += 1;
		  			if (concCurrent > 200){CORDNA_OutS_Stock_LiquidClass = CORDNA_OutS_Stock_LiquidClass + "1000tip_gDNA300_pipICsurface" + ",";}
		  			else {CORDNA_OutS_Stock_LiquidClass = CORDNA_OutS_Stock_LiquidClass + "1000tip_gDNA100_pipICsurface" + ",";}
		  		}else if (VolumeThisTransfer > 50){
		  			ChP1000 = "0"; ChP300 = "1"; ChP50 = "0";  Tip300iccount += 1;
		  			if (concCurrent > 200){CORDNA_OutS_Stock_LiquidClass = CORDNA_OutS_Stock_LiquidClass + "300tip_gDNA300_pipICsurface" + ",";}
		  			else {CORDNA_OutS_Stock_LiquidClass = CORDNA_OutS_Stock_LiquidClass + "300tip_gDNA100_pipICsurface" + ",";}
		  		}else{
		  			ChP1000 = "0"; ChP300 = "0"; ChP50 = "1";  Tip50iccount += 1;
		  			if (concCurrent > 200){CORDNA_OutS_Stock_LiquidClass = CORDNA_OutS_Stock_LiquidClass + "50tip_gDNA300_pipICsurface" + ",";}
		  			else {CORDNA_OutS_Stock_LiquidClass = CORDNA_OutS_Stock_LiquidClass + "50tip_gDNA100_pipICsurface" + ",";}
		  		}
		  		
		  		CORDNA_OutS_Stock_Vol = CORDNA_OutS_Stock_Vol + String.valueOf(VolumeThisTransfer) + ",";
		  		CORDNA_OutS_Stock_SeqS = CORDNA_OutS_Stock_SeqS + String.valueOf(stockSeqS_Cntr) + ",";
		  		CORDNA_OutS_Stock_SeqD = CORDNA_OutS_Stock_SeqD + String.valueOf(stockSeqD_Cntr) + ",";
		  	  	CORDNA_OutS_Stock_ChP_T1000 = CORDNA_OutS_Stock_ChP_T1000 + ChP1000 + ",";
		  	  	CORDNA_OutS_Stock_ChP_T300 = CORDNA_OutS_Stock_ChP_T300 + ChP300 + ",";
		  	  	CORDNA_OutS_Stock_ChP_T50 = CORDNA_OutS_Stock_ChP_T50 + ChP50 + ",";
		  	  	
	  	  	  	if (batchOf8_Stock_Cntr == 8){
//	  	  	  		CORDNA_OutS_Stock_ChP_T1000 = CORDNA_OutS_Stock_ChP_T1000 + ",";
//	  	  	  		CORDNA_OutS_Stock_ChP_T300 = CORDNA_OutS_Stock_ChP_T300 + ",";
//	  	  	  		CORDNA_OutS_Stock_ChP_T50 = CORDNA_OutS_Stock_ChP_T50 + ",";
	  	  	  		batchOf8_Stock_Cntr = 0;
	  	  	  	}
	  	  	}
	  	  	stockSeqS_Cntr +=1;
	  	  	stockSeqD_Cntr +=1;
	  	  	diluentSeqCntr += 1;
        }//END Procedure 1 - Normalization
        
  	  	if (CORDNA_Procedure.equals("2")){
  	  		CORDNA_OutS_Final_SeqD = CORDNA_OutS_Final_SeqD + AlphaToNumeric.ToNumeric_WithZeros(newline.split(tabChar)[column_FinalPosition]) + ",";
  	  	}
  	  	
      }//End Reading File
      input.close();
      
      if (CORDNA_Procedure.equals("1")){
	      //Fill arrays for complete set of 8 for processing by Hamilton Venus
		  for(int i =0; i<8-batchOf8_Diluent_Cntr; i++){
			  CORDNA_OutS_Dil_Vol = CORDNA_OutS_Dil_Vol + "0,";
			  CORDNA_OutS_Dil_MixVol = CORDNA_OutS_Dil_MixVol + "0,";
			  CORDNA_OutS_Dil_ChP_T1000 = CORDNA_OutS_Dil_ChP_T1000 + "0,";
			  CORDNA_OutS_Dil_ChP_T300 = CORDNA_OutS_Dil_ChP_T300 + "0,";
			  CORDNA_OutS_Dil_ChP_T50 = CORDNA_OutS_Dil_ChP_T50 + "0,";
			  CORDNA_OutS_Diluent_LiquidClass = CORDNA_OutS_Diluent_LiquidClass + "1000tip_gDNA300_pipICsurface,";
		  }
	  	  for(int i =0; i<8-batchOf8_Stock_Cntr; i++){
	  		  CORDNA_OutS_Stock_Vol = CORDNA_OutS_Stock_Vol + "0,"; 
	  		  CORDNA_OutS_Stock_ChP_T1000 = CORDNA_OutS_Stock_ChP_T1000 + "0,";
	  		  CORDNA_OutS_Stock_ChP_T300 = CORDNA_OutS_Stock_ChP_T300 + "0,";
	  		  CORDNA_OutS_Stock_ChP_T50 = CORDNA_OutS_Stock_ChP_T50 + "0,";
	  		  CORDNA_OutS_Stock_LiquidClass = CORDNA_OutS_Stock_LiquidClass + "1000tip_gDNA300_pipICsurface,";
	  	  }
  	  
	  	  //Remove trailing commas	
	  	  CORDNA_OutS_Dil_Vol = CORDNA_OutS_Dil_Vol.substring(0, CORDNA_OutS_Dil_Vol.length() - 1); 
	  	  CORDNA_OutS_Dil_MixVol = CORDNA_OutS_Dil_MixVol.substring(0, CORDNA_OutS_Dil_MixVol.length() - 1); 
	  	  CORDNA_OutS_Dil_SeqD = CORDNA_OutS_Dil_SeqD.substring(0, CORDNA_OutS_Dil_SeqD.length() - 1); 

	  	  if (CORDNA_OutS_Dil_ChP_T1000.endsWith(",")){CORDNA_OutS_Dil_ChP_T1000 = CORDNA_OutS_Dil_ChP_T1000.substring(0, CORDNA_OutS_Dil_ChP_T1000.length() - 1);} 
	  	  if (CORDNA_OutS_Dil_ChP_T300.endsWith(",")){CORDNA_OutS_Dil_ChP_T300 = CORDNA_OutS_Dil_ChP_T300.substring(0, CORDNA_OutS_Dil_ChP_T300.length() - 1);} 
	  	  if (CORDNA_OutS_Dil_ChP_T50.endsWith(",")){CORDNA_OutS_Dil_ChP_T50 = CORDNA_OutS_Dil_ChP_T50.substring(0, CORDNA_OutS_Dil_ChP_T50.length() - 1);} 

	  	  CORDNA_OutS_Stock_Vol = CORDNA_OutS_Stock_Vol.substring(0, CORDNA_OutS_Stock_Vol.length() - 1); 

		  CORDNA_OutS_Stock_SeqS = CORDNA_OutS_Stock_SeqS.substring(0, CORDNA_OutS_Stock_SeqS.length() - 1);

		  CORDNA_OutS_Stock_SeqD = CORDNA_OutS_Stock_SeqD.substring(0, CORDNA_OutS_Stock_SeqD.length() - 1);


		  if (CORDNA_OutS_Stock_ChP_T1000.endsWith(",")){CORDNA_OutS_Stock_ChP_T1000 = CORDNA_OutS_Stock_ChP_T1000.substring(0, CORDNA_OutS_Stock_ChP_T1000.length() - 1);} 
		  if (CORDNA_OutS_Stock_ChP_T300.endsWith(",")){CORDNA_OutS_Stock_ChP_T300 = CORDNA_OutS_Stock_ChP_T300.substring(0, CORDNA_OutS_Stock_ChP_T300.length() - 1);} 
		  if (CORDNA_OutS_Stock_ChP_T50.endsWith(",")){CORDNA_OutS_Stock_ChP_T50 = CORDNA_OutS_Stock_ChP_T50.substring(0, CORDNA_OutS_Stock_ChP_T50.length() - 1);} 
		  CORDNA_OutS_Stock_LiquidClass = CORDNA_OutS_Stock_LiquidClass.substring(0, CORDNA_OutS_Stock_LiquidClass.length() - 1);  
		  CORDNA_OutS_Diluent_LiquidClass = CORDNA_OutS_Diluent_LiquidClass.substring(0, CORDNA_OutS_Diluent_LiquidClass.length() - 1);  
		  
      }

	  if (CORDNA_Procedure.equals("2")){
		//Remove trailing commas
		  CORDNA_OutS_Final_SeqD = CORDNA_OutS_Final_SeqD.substring(0, CORDNA_OutS_Final_SeqD.length() - 1); 	  
	  }
  	  	
      
    } catch (Exception e) {
    	VAStools.errorMessageProcess("Step::" + Step + "::VAS ERROR::VAS failed to process the Q File: " + CORDNA_Worklist);
    }
    TIP_QUANT(Step);
   
  }
  

  }//END CLASS
