package TelosV2Pkg;

import java.awt.Image;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.EOFException;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.text.DateFormatSymbols;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.Random;
import java.util.concurrent.TimeUnit;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JOptionPane;

import com.healthmarketscience.jackcess.Database;
import com.healthmarketscience.jackcess.DatabaseBuilder;
import com.healthmarketscience.jackcess.Row;
import com.healthmarketscience.jackcess.Table;
import com.itextpdf.text.pdf.PdfReader;
import com.itextpdf.text.pdf.parser.PdfReaderContentParser;
import com.itextpdf.text.pdf.parser.SimpleTextExtractionStrategy;
import com.itextpdf.text.pdf.parser.TextExtractionStrategy;

public class VAStools {
	
	private static Random generator = new Random();

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		// TODO Auto-generated method stub

	}

	public static void SetFocusVenus(){// Set focus to Venus window
		try {
			Process x;
			x = Runtime
					   .getRuntime()
					   .exec("wscript.exe \"C:\\Program Files\\Telos\\AutoPilot\\System\\FocusVenus.VBS\"");
	
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public static boolean isRunning(String process) {
	    boolean found = false;
	    try {
	        File file = File.createTempFile("isRunning",".vbs");
	        file.deleteOnExit();
	        FileWriter fw = new java.io.FileWriter(file);

	        String vbs = "Set WshShell = WScript.CreateObject(\"WScript.Shell\")\n"
	                   + "Set locator = CreateObject(\"WbemScripting.SWbemLocator\")\n"
	                   + "Set service = locator.ConnectServer()\n"
	                   + "Set processes = service.ExecQuery _\n"
	                   + " (\"select * from Win32_Process where name='" + process +"'\")\n"
	                   + "For Each process in processes\n"
	                   + "wscript.echo process.Name \n"
	                   + "Next\n"
	                   + "Set WSHShell = Nothing\n";

	        fw.write(vbs);
	        fw.close();
	        Process p = Runtime.getRuntime().exec("cscript //NoLogo " + file.getPath());
	        BufferedReader input =
	            new BufferedReader
	              (new InputStreamReader(p.getInputStream()));
	        String line;
	        line = input.readLine();
	        if (line != null) {
	            if (line.equals(process)) {
	              found = true;
	            }
	        }
	        input.close();

	    }
	    catch(Exception e){
	        e.printStackTrace();
	    }
	    return found;
	  }

	public static String arrayListToString(ArrayList arrayListInput, String delimiter) {
		String returnString = "";
		
		for (int i = 0; i < arrayListInput.size(); i++) {
			returnString = returnString + arrayListInput.get(i);
			if (i<(arrayListInput.size()-1)){returnString = returnString + delimiter;}
			
		}
		
		return returnString;
	}
	
	/**
	 * This method looks for a Hamilton Liquid Class in ML_STARLiquids.mdb and returns a boolean based on if it is found.
	 */
	public static void LiquidClassConfirm(String liquidClass, String dataBase, Integer Step){
		boolean found = false;
		
		try {
			Database db = DatabaseBuilder.open(new File(dataBase));
			Table table = db.getTable("LiquidClass");
			  for(Row row : table) {
				    if (row.containsValue(liquidClass)){found = true;}
				  }
			  db.close();
		} catch (IOException e2) {
			// TODO Auto-generated catch block
			e2.printStackTrace();
			found = true;
			VAStools.errorMessageProcess( "VAS::Could not access Hamilton's Liquid Class database at " + dataBase);
		}
		
		if (!found){
			VAStools.errorMessageProcess( 
					"Step::" + Step + "::VAS ERROR: VAS has determined that an optimal set of pipetting parameters will be found in the Hamilton Liquid Class " +
					liquidClass + "\n\n" + 
					liquidClass + " does not exist!" + "\n\n" +
					"Please create the Liquid Class " + liquidClass + " before running.");
		}
	}
	
	/**
	 * This method returns an array containing list of AutoPilot Liquid Classes in ML_STARLiquids.mdb.
	 * AutoPilot V1.2.25
	 */
	public static String[] AP_LiquidClassesFromHam(String dataBase){		//AutoPilot V1.2.25
		boolean found = false;
		ArrayList<String> arrayListLiquids = new ArrayList<String>();
		String liquidClass;
		String liquid;
				
		try {
			Database db = DatabaseBuilder.open(new File(dataBase));
			Table table = db.getTable("LiquidClass");
			for(Row row : table) { 
				  liquidClass = String.valueOf(row.get("LiquidClassName"));
				    if (liquidClass.contains("tip_") && liquidClass.contains("_pip") ){
				    	found = true;
				    	liquid = liquidClass.split("_")[1];
				    	if (!arrayListLiquids.contains(liquid)){
				    		arrayListLiquids.add(liquid);
				    	}
				    }
			  }
			  db.close();
		} catch (IOException e2) {
			e2.printStackTrace();
			found = true;
			JOptionPane.showMessageDialog(null, "AutoPilot was not able to Launch \n\n VAS::The current configuration requires access to a Hamilton Liquid Class Database. \n\n VAS could not access Hamilton's Liquid Class database at " + dataBase);
		}
		
		if (!found){
			JOptionPane.showMessageDialog(null,
					"VAS WARNING: VAS does not see any AutoPilot compatible Liquid Class in the Hamilton database.\n\n" +
					"Please create Liquid Classes before using AutoPilot");
		}
		
		String[] arrayLiquids = arrayListLiquids.toArray(new String[arrayListLiquids.size()]);
		return arrayLiquids;
	}
	
	/**
	 * This method checks the selected CP worklist for correct format
	 */
	public static String CP_WorklistConfirm(String file, String deckLocations){
    	String returnValue = "";
    	ArrayList<String> arrayList_FileLines = new ArrayList();
    	
    	//confirm Worklist has correct columns and Column names
        File f = new File(file);
        
        String newline = "";
        int S_ID_element=0; int S_WELL_element=0; int D_ID_element=0; int D_WELL_element=0; int VOL_element=0;
        boolean foundColumn = false;
        try {
          FileReader inputReader = new FileReader(f);
          BufferedReader br = new BufferedReader(inputReader);
          while ((newline = br.readLine()) != null) {
        	  arrayList_FileLines.add(newline);
          }
          inputReader.close();
        } catch (Exception e1) {
        	e1.printStackTrace();
        	if (!returnValue.equals("")){
        		returnValue = "Cherry Picking Error:: Error accessing Worklist File.\n\nPlease select a different file.";	
        	}
        	
        }
             
        newline = arrayList_FileLines.get(0);

          for(int i =0;i<newline.split(",").length;i++){  
        	  if (newline.split(",")[i].equals("S_ID")){S_ID_element = i; foundColumn = true;}
          }if (!foundColumn){returnValue = "Worklist File is Missing the Source ID column.\nThe Column's header should be S_ID\n\nPlease fix or select a differnt Worklist File.";} foundColumn = false;
          
          for(int i =0;i<newline.split(",").length;i++){
        	  if (newline.split(",")[i].equals("S_WELL")){S_WELL_element = i; foundColumn = true;}
          }if (!foundColumn){returnValue = "Worklist File is Missing the Source Well column.\nThe Column's header should be S_WELL\n\nPlease fix or select a differnt Worklist File.";} foundColumn = false;        
          
          for(int i =0;i<newline.split(",").length;i++){
        	  if (newline.split(",")[i].equals("D_ID")){D_ID_element = i; foundColumn = true;}
          }if (!foundColumn){returnValue = "Worklist File is Missing the Destination ID column.\nThe Column's header should be D_ID\n\nPlease fix or select a differnt Worklist File.";} foundColumn = false;   
          
          for(int i =0;i<newline.split(",").length;i++){
        	  if (newline.split(",")[i].equals("D_WELL")){D_WELL_element = i; foundColumn = true;}
          }if (!foundColumn){returnValue = "Worklist File is Missing the Destination Well column.\nThe Column's header should be D_WELL\n\nPlease fix or select a differnt Worklist File.";} foundColumn = false;  
          
          for(int i =0;i<newline.split(",").length;i++){
        	  if (newline.split(",")[i].equals("VOL")){VOL_element = i; foundColumn = true;}
          }if (!foundColumn){returnValue = "Worklist File is Missing the Volume column.\nThe Column's header should be VOL\n\nPlease fix or select a differnt Worklist File.";} foundColumn = false;  
        
        if (returnValue.equals("")){       	
            for (int i = 1; i < arrayList_FileLines.size(); i++) {
            	newline = arrayList_FileLines.get(i);
//              Confirm Well info is numeric =
	            if (VASengine.CP_SourceLabwareType.equals("0")){
	         		try {
	         			Integer.parseInt(newline.split(",")[S_WELL_element]);
	           	  	} catch(NumberFormatException e) {
	           	  		returnValue = "You have indicated that your Source labware is/are Plates.  Therefore, the S_WELL column of the Worklist File should contain the plate wells!\nThe values should be numeric.  ex '1,48,96'\n\nPlease fix or select a differnt Worklist File.";
	           	  	}
	         	}else if (VASengine.CP_SourceLabwareType.equals("1")){
	          		if (!newline.split(",")[S_WELL_element].equals("")){
	          			returnValue = "You have indicated that your Source labware is/are Tubes.  Therefore, the S_WELL column of the Worklist File should be blank!\n\n\nPlease fix or select a differnt Worklist File.";
	          		}
	         	} 
	        	 
	         	if (VASengine.CP_DestLabwareType.equals("0")){
	         		try {
	         			Integer.parseInt(newline.split(",")[D_WELL_element]);
	           	  	} catch(NumberFormatException e) {
	           	  		returnValue = "You have indicated that your Destination labware is/are Plates.  Therefore, the D_WELL column of the Worklist File should contain the plate wells!\nThe values should be numeric.  ex '1,48,96'\n\nPlease fix or select a differnt Worklist File.";
	           	  	}
	         	}else if (VASengine.CP_DestLabwareType.equals("1")){
	          		if (!newline.split(",")[D_WELL_element].equals("")){
	          			returnValue = "You have indicated that your Destination labware is/are Tubes.  Therefore, the D_WELL column of the Worklist File should be blank!\n\n\nPlease fix or select a differnt Worklist File.";
	          		}
	         	} 
              
              try { 
          		Integer.parseInt(newline.split(",")[VOL_element]);
          	  } catch(NumberFormatException e) {
          		returnValue = "The S_VOL column of the Worklist File should contain the volume to transfer in (ul)!\n\nThe values should be numeric.  ex '10 or 200'\n\nPlease fix or select a differnt Worklist File.";
          	  }	
            }
        }


        
        if (!returnValue.equals("")){
        	returnValue = "Cherry Picking Error:: " + returnValue;
        }
        return returnValue;
	
	}
	
	
	/**
	 * This method checks the selected PK worklist for correct format
	 */
	public static boolean PK_WorklistConfirm(String file){						//PKF0033 This function was added
		boolean returnValue = true;
		boolean formatCheck;
		String errorMessage = "";
		String worklistType;
		String newline;
		String tabChar = "\t";
		FileReader input;  FileReader input2;
		int samplesTotal=0;
		int count = 0;
		ArrayList<String> arraylistPK_SampleIds_All = new ArrayList<String>();		//All Sample IDs including repeats
		
	    if (file.contains(".xls")){
	    	worklistType = "3";
	    }else if (file.contains(".pdf")){
			worklistType = "2";
		}else if (file.contains(".txt")){
			worklistType = "1";
		}else {
			worklistType = "0";
		}
	    

	    if (worklistType.equals("1")){																										//START PKF0033
		     /////////////////////////////////	
		     //Read in all info from Watson Generated File
		      try {
		    	  File f = new File(file);
		    	  input = new FileReader(f);
				  BufferedReader br0 = new BufferedReader(input);
				  BufferedReader br = new BufferedReader(input);
				  
				  	newline = br0.readLine(); //Advance past column headers
				  
					while ((newline = br0.readLine()) != null) {
			    	count = 0;
			    	
			    	if (returnValue && newline.split(tabChar)[0].equals("")){returnValue = false; errorMessage = "The selected Watson file is missing one or more Sample ID values in column one";}		//Sample ID

			    	if (returnValue && !newline.split(tabChar)[5].equals("")){ //identified as Sample (i.e. not std or QC)
			    		
			    		if (returnValue && newline.split(tabChar)[12].equals("")){returnValue = false; errorMessage = "The selected Watson file is missing one or more Dilution values in column thirteen";}		//Sample ID
				    	if ((returnValue && Integer.parseInt(newline.split(tabChar)[12]) >= 1000000)){returnValue = false; errorMessage = "The selected Watons file contains a dilution value greater than or equal to 1,000,000\nThe Maximum dilution is 999,999";}
			    		
			    		for (int x = 0; x<arraylistPK_SampleIds_All.size(); x++) {
				    		 if (newline.split(tabChar)[0].equals(arraylistPK_SampleIds_All.get(x))){	 
				    		 count++;
				    		 }
			    		 }
			    	 } 
		 		 	if (count==2){
		 		 		errorMessage = "The selected Watson files contains samples in greater than duplicate\n\nExample::" + newline.split(tabChar)[0];		
		    			returnValue = false;
		    			break;
				 	} 
			    	 arraylistPK_SampleIds_All.add(newline.split(tabChar)[0]); 
					if (returnValue && !newline.split(tabChar)[18].matches("\\D\\d\\d")){returnValue = false; errorMessage = "The selected Watson file contains incorrect formatting of Well designations.\nExample: Well \"" + newline.split(tabChar)[18] + "\" for sample " + newline.split(tabChar)[0] + "\n\nThe format of the Well designations should be three character alpha numeric.\n Example: A01, G10, B09";}		//AssayPlate Well.  Should contain zeros													
			    }
		      } catch (Exception e) {
		    	  	returnValue = false;
					errorMessage = "VAS Error: error parsing Watson File.\n\nPlease confirm that the file was created by Watson and was not modified.\n\nError::" + e.toString();
					e.printStackTrace();
			  }    
		} 
	    
	    
		if (worklistType.equals("2")){																										//START PKF0033
		     /////////////////////////////////	
		     //Read in all info from Gyros Watson .PDF file
			String gyroPdfParsedRaw_textAll = "";
		    try {																														//Extract info from PDF to intermediate text file.
				PdfReader reader = new PdfReader(file);
				PdfReaderContentParser parser = new PdfReaderContentParser(reader);
				TextExtractionStrategy strategy;
				for (int i = 1; i <= reader.getNumberOfPages(); i++) {
				    strategy = parser.processContent(i, new SimpleTextExtractionStrategy());
				    gyroPdfParsedRaw_textAll = gyroPdfParsedRaw_textAll + strategy.getResultantText();
				}
				reader.close();
				
				if (!gyroPdfParsedRaw_textAll.contains("Gyrolab Control Loading")){returnValue = false; errorMessage = "The selected Gyros .pdf file is not formatted correctly.";}
				if (gyroPdfParsedRaw_textAll.contains("Dilution")){returnValue = false; errorMessage = "The selected Gyros .pdf file is not formatted correctly.\n\nIf you are attempting to select a file generated by the Gyrolab Wizard\nplease select the corrosponding excel file instead.";}
				if (gyroPdfParsedRaw_textAll.contains("MP2")){returnValue = false; errorMessage = "The selected Gyros .pdf file requests the creation of more than one Microtitre Plate.";}
				
		    } catch (Exception e) {
	    	  	returnValue = false;
				errorMessage = "VAS Error: error parsing Gyrolas .pdf file.\n\nPlease confirm that the Gyrolab file was created by using the correct Watson information.\n\nError::" + e.toString();
				e.printStackTrace();
			} 
		}
		
		if (worklistType.equals("3")){																										//START PKF0033
		     /////////////////////////////////	
		     //Read in all info from Gyros Wizard Generated File
		      try {
		    	  File f = new File(file);
		    	  input = new FileReader(f);
				  BufferedReader br = new BufferedReader(input);
				  
				  //check for "Index" in header of column 1 as this is used to identify beginning of table.
				  formatCheck = false;
				  while ((newline = br.readLine()) != null) {
					  if(newline.contains("Index")){formatCheck = true;}
				  }
				  br.close();
				  if (!formatCheck) {returnValue = false; errorMessage = "The selected Gyros Wizard file does not have the correct column headers";}
				  
				  if (returnValue){
			    	  File f2 = new File(file);
			    	  input2 = new FileReader(f2);
					  BufferedReader br2 = new BufferedReader(input2);
					  while (!(newline = br2.readLine()).contains("Index")) {}	//Advance to table
					  while ((newline = br2.readLine()) != null) {				//Iterate over table																		
						  samplesTotal++;
						  if (samplesTotal>96){returnValue = false; errorMessage = "The selected Gyros Wizard file requires more than the maximum of 96 preperations.";}		//Sample ID
						  if (newline.split(tabChar)[2].equals("")){returnValue = false; errorMessage = "The selected Gyros Wizard file is missing one or more Sample ID values in column three";}		//Sample ID
					    	count = 0;
				    		for (int x = 0; x<arraylistPK_SampleIds_All.size(); x++) {
					    		 if (newline.split(tabChar)[2].equals(arraylistPK_SampleIds_All.get(x))){
					    		 count++;
					    		 }
				    		 }
				 		 	if (count==2){
				 		 		returnValue = false; errorMessage = "The selected Gyros Wizard file contains samples listed in greater than duplicates."; break;		
						 	} 
					    	 arraylistPK_SampleIds_All.add(newline.split(tabChar)[2]);
 
						  if (!newline.split(tabChar)[0].equals("")){if (newline.split(tabChar)[5].equals("")){returnValue = false; errorMessage = "The selected Gyros Wizard file is missing one or more Dilution value/s in column six";}}			//Dilution
						  if (returnValue && !newline.split(tabChar)[0].equals("")){if (Integer.parseInt(newline.split(tabChar)[5]) >= 1000000){returnValue = false; errorMessage = "The selected Gyros Wizard file contains a dilution value greater than or equal to 1,000,000\nThe Maximum dilution is 999,999";}}
						  try {
							if (newline.split(tabChar)[7].equals("")){returnValue = false; errorMessage = "The selected Gyros Wizard file is missing one or more well designations in colum eight";}			//AssayPlate Well.  Missing													
							if (!newline.split(tabChar)[7].matches("\\D(?![0])\\d{1,2}")){returnValue = false; errorMessage = "The selected Gyros Wizard file contains incorrect formatting of Well designations.\nExample: Well \"" + newline.split(tabChar)[7] + "\" for sample " + newline.split(tabChar)[2] + "\n\nThe format of the Well designations should be alpha numeric without extra zeros.\n Example: A1, G10, B9";}		//AssayPlate Well.  Should not contain zeros													
						  } catch (Exception e) {
							returnValue = false; errorMessage = "The selected Gyros Wizard file is missing one or more Well designations in colum eight";
						}
					  }
	
					  br2.close();					  
				  }
					
			} catch (Exception e) {
				returnValue = false;
				errorMessage = "VAS Error: error parsing Gyros Wizard File.\n\nPlease confirm that the file was created by the Gyros Wizard and was not modified.\n\nError::" + e.toString();
				e.printStackTrace();
			}
		}
		
		if (!returnValue){
			JOptionPane.showMessageDialog(null, errorMessage);
			JOptionPane.showMessageDialog(null, "Please select a different file");
		}
		return returnValue;
	}
	
	
	
	
	/**
	 * This method returns labware properties.
	 *  For Hamilton
     *	labwareParamaters 1=.rck Height,  2=.rck number of wells, 5=.ctr depth,  6=.ctr Total Volume,  10=grip position WITH LID
	 */
    public static String SystemLabwareData(int labwareParameter, String LabwareDefFile_WithoutExtension){
    	// Collect information from labware file.  
    	// To be configured for each type of system (e.g. Beckman, Hamilton, Tecan)

    	FileReader input;
    	String returnValue = "";
    	String s = "";
    	String labwareHeight = "";
    	String labwareColumns = "";
    	String labwareRows = "";
    	String lidToBase = "";
    	String segments = "";
    	String gripHeightWithLid = "";
    	String fileRack = LabwareDefFile_WithoutExtension + ".rck";
    	String fileContainer = LabwareDefFile_WithoutExtension + ".ctr";
    	String fileLid = LabwareDefFile_WithoutExtension + ".lid";
    	String newline = "";
    	String lineIN_Rack = "";
    	String lineIN_Container = "";
    	String lineIN_Lid = "";
    	
    	boolean matched = false;
    	
        // .rck file
    	if (labwareParameter == 1 || labwareParameter == 2|| labwareParameter == 10){
    		matched = false;
        	File f = new File(fileRack);
            try {
              input = new FileReader(f);
              BufferedReader br = new BufferedReader(input);
              while ((newline = br.readLine()) != null) {
            	  lineIN_Rack = lineIN_Rack + newline;
              }
              input.close();
            }catch (Exception e) {
            	VAStools.errorMessageProcess("VAS ERROR::VAS cannot process the labware file " + LabwareDefFile_WithoutExtension +"\n\nError ID::"+e);
            }
            
            // GET RACK HEIGHT
    		Pattern patternDimDz = Pattern.compile("Dim.Dz......");
            Matcher matcherDimDz = patternDimDz.matcher(lineIN_Rack);
            if (matcherDimDz.find()) {
            	s = matcherDimDz.group().substring(1);
            	matched = true;
            } 
              if (matched){
    	          boolean start = false;
    	          char[] characters = s.toCharArray();
    	          for(int i =0;i<characters.length;i++){//Start recording at first character that is a digit.  Stop recording at next character that is not a digit.  This will capture digits up to the "."
    	  			if(Character.isDigit(characters[i])){
    	  				start = true;
    	  				labwareHeight = labwareHeight + String.valueOf(characters[i]);
    	  			}else{
    	  				if (start == true){break;}
    	  			}
    	          }       	  
              }else{
              	VAStools.errorMessageProcess("VAS ERROR::VAS cannot process the labware file " + LabwareDefFile_WithoutExtension +"\n\nError ID::Labware Height not found");
              }
              
              // GET RACK NUMBER OF WELLS
              //Columns
              Pattern patternColumns = Pattern.compile("Columns......");
              Matcher matcherColumns = patternColumns.matcher(lineIN_Rack);
              matched = false;
              if (matcherColumns.find()) {
              	s = matcherColumns.group().substring(1);
              	matched = true;
              } 
              if (matched){
      	          boolean start = false;
      	          char[] characters = s.toCharArray();
      	          for(int i =0;i<characters.length;i++){//Start recording at first character that is a digit.  Stop recording at next character that is not a digit.  This will capture digits up to the "."
      	  			if(Character.isDigit(characters[i])){
      	  				start = true;
      	  				labwareColumns = labwareColumns + String.valueOf(characters[i]);
      	  			}else{
      	  				if (start == true){break;}
      	  			}
      	          }       	  
              }else{
                	VAStools.errorMessageProcess("VAS ERROR::VAS cannot process the labware file " + LabwareDefFile_WithoutExtension +"\n\nError ID::Labware Columns not found");
              }
              //Rows
              Pattern patternRows = Pattern.compile("Rows......");
              Matcher matcherRows = patternRows.matcher(lineIN_Rack);
              matched = false;
              if (matcherRows.find()) {
              	s = matcherRows.group().substring(1);
              	matched = true;
              } 
              if (matched){
      	          boolean start = false;
      	          char[] characters = s.toCharArray();
      	          for(int i =0;i<characters.length;i++){//Start recording at first character that is a digit.  Stop recording at next character that is not a digit.  This will capture digits up to the "."
      	  			if(Character.isDigit(characters[i])){
      	  				start = true;
      	  				labwareRows = labwareRows + String.valueOf(characters[i]);
      	  			}else{
      	  				if (start == true){break;}
      	  			}
      	          }       	  
              }else{
                	VAStools.errorMessageProcess("VAS ERROR::VAS cannot process the labware file " + LabwareDefFile_WithoutExtension +"\n\nError ID::Labware Rows not found");
              }  

              //Return Value based on parameter selected.  Total Well number is calculated based on Columns*Rows  
              if (labwareParameter == 1){returnValue = labwareHeight;}
              if (labwareParameter == 2){returnValue = String.valueOf(Integer.parseInt(labwareColumns)*Integer.parseInt(labwareRows));}
            matched = false;
    	}
    	
    	if (labwareParameter == 10){//grip position WITH LID

    		matched = false;
    		File f = new File(fileLid);
            try {
              input = new FileReader(f);
              BufferedReader br = new BufferedReader(input);
              while ((newline = br.readLine()) != null) {
            	  lineIN_Lid = lineIN_Lid + newline;
              }
              input.close();
            }catch (Exception e) {
            	VAStools.errorMessageProcess("VAS ERROR::VAS cannot process the labware file " + LabwareDefFile_WithoutExtension +"\n\nError ID::"+e);
            }
            
    		Pattern patternRackBaseToCoverBase = Pattern.compile("RackBaseToCoverBase......");
        	Matcher matcherRackBaseToCoverBase = patternRackBaseToCoverBase.matcher(lineIN_Lid);
            if (matcherRackBaseToCoverBase.find()) {
            	s = matcherRackBaseToCoverBase.group().substring(1);
            	matched = true;
            } 
            if (matched){
    	          boolean start = false;
    	          char[] characters = s.toCharArray();
    	          for(int i =0;i<characters.length;i++){//Start recording at first character that is a digit.  Stop recording at next character that is not a digit.  This will capture digits up to the "."
    	  			if(Character.isDigit(characters[i])){
    	  				start = true;
    	  				lidToBase = lidToBase + String.valueOf(characters[i]);
    	  			}else{
    	  				if (start == true){break;}
    	  			}
    	          }
    	          
    	          gripHeightWithLid = String.valueOf(  (Integer.parseInt(labwareHeight) - Integer.parseInt(lidToBase))  + 2);
            }else{
              VAStools.errorMessageProcess("VAS ERROR::VAS cannot process the labware file " + LabwareDefFile_WithoutExtension +"\n\nError ID::Lid Dimensions Not Found");
            }
            returnValue = gripHeightWithLid;
    	}
    	
    	if (labwareParameter == 6){/////////////// NOT COMPLETE!
    		matched = false;
    		File f = new File(fileContainer);
            try {
              input = new FileReader(f);
              BufferedReader br = new BufferedReader(input);
              while ((newline = br.readLine()) != null) {
            	  lineIN_Container = lineIN_Container + newline;
              }
              input.close();
            }catch (Exception e) {
            	VAStools.errorMessageProcess("VAS ERROR::VAS cannot process the labware file " + LabwareDefFile_WithoutExtension +"\n\nError ID::"+e);
            }
              Pattern patternSegments = Pattern.compile("Segments......");
              Matcher matcherSegments = patternSegments.matcher(lineIN_Container);
              if (matcherSegments.find()) {
              	s = matcherSegments.group().substring(1);
              	matched = true;
              }               
              if (matched){
    	          boolean start = false;
    	          char[] characters = s.toCharArray();
    	          for(int i =0;i<characters.length;i++){//Start recording at first character that is a digit.  Stop recording at next character that is not a digit.  This will capture digits up to the "."
    	  			if(Character.isDigit(characters[i])){
    	  				start = true;
    	  				segments = segments + String.valueOf(characters[i]);
    	  			}else{
    	  				if (start == true){break;}
    	  			}
    	          }

    	          for(int i =1;i<Integer.parseInt(segments)+1;i++){
    	        	 /////////////// SCROLLING THROUGH SHAPES started here but not completed
    	          }
              }else{
              	VAStools.errorMessageProcess("VAS ERROR::VAS cannot process the labware file " + LabwareDefFile_WithoutExtension +"\n\nError ID::Lid Dimensions Not Found");
              }
            
            returnValue = segments;	
    	}
        
        return returnValue;
      }
    
    /**
	 * This method adds the position to AP_00 or AP_000
	 */
	public static String labwarePosToAP_(String position){
		if (position.length()==1){
			return ("AP_000" + position);
		}else{
			return ("AP_00" + position);
		}
	}
	
	/**
	 * This method sets the sent message into errorMessage if there is not already an error message logged
	 */
	public static void errorMessageProcess(String message){
		if (VASengine.errorMessage.equals("")){
			VASengine.errorMessage = message;
		}
	}
	
	public static String dateTime() {
		java.util.Date date = new java.util.Date();
		Calendar cal = Calendar.getInstance(); 
		cal.setTime(date); 
		int day = cal.get(Calendar.DAY_OF_MONTH);
		int month = cal.get(Calendar.MONTH); 
		int year = cal.get(Calendar.YEAR);
		SimpleDateFormat sdf = new SimpleDateFormat("HH:mm:ss");
		String DateStamp = Integer.toString(day) + "-" + ((new DateFormatSymbols().getMonths()[month]).substring(0,3))  + "-" + Integer.toString(year);
		String TimeStamp = sdf.format(date); 
		String DateTimeStamp = DateStamp + "_" + TimeStamp;
		return DateTimeStamp;
	}
	
	public static void RunID(){  //This is only executed from TW at runtime
		String runid = "";
		int r = 0;
		   while(r < 1){ 
		   r = generator.nextInt();
		   }
		   runid = dateTime() + "_" + Integer.toString(r);
		   VASengine.runID = runid;
	}
	
    /**
	 * This method set the image of a JLabel
	 * target label, imageFile, index, indexZeroMessage, width, height
	 */
	public static void setJLabelImage(JLabel target, String imageFile, int index, String indexZeroMessage, int width, int height){
		String graphicFile = "C:/Program Files/TELOS/AutoPilot/Graphics/" + imageFile;
		
		if (new File(graphicFile).exists()) {	        			
			ImageIcon imageIcon = new ImageIcon(graphicFile); // load the image to a imageIcon
			Image image = imageIcon.getImage(); // transform it 
			Image newimg = image.getScaledInstance(width, height,  java.awt.Image.SCALE_SMOOTH); // scale it the smooth way  
			imageIcon = new ImageIcon(newimg);  // transform it back	        			
			target.setText("");
			target.setIcon(imageIcon);
		}else{
			if (index == 0){
				target.setIcon(new ImageIcon(""));
				target.setText(indexZeroMessage);	
			}else{
				target.setIcon(new ImageIcon(""));
				PanelPKB2.lblPkBlank.setText("Image of Labware Not Available");	
			}
		}
	}
	
	public static boolean passwordAccept(String passwordAttempt){
//		8 characters
//		at lest one uppercase
//		no whitespace
		
		if (passwordAttempt.length()<7){
			JOptionPane.showMessageDialog(null, "Your password must contain at least seven characters."," ", JOptionPane.INFORMATION_MESSAGE);					
			return false;
		}
		
		if(passwordAttempt.contains(" ")){
			JOptionPane.showMessageDialog(null, "Your password cannot contain any spaces."," ", JOptionPane.INFORMATION_MESSAGE);					
			return false;
		}
		
		boolean hasUppercase = !passwordAttempt.equals(passwordAttempt.toLowerCase());
		boolean hasLowercase = !passwordAttempt.equals(passwordAttempt.toUpperCase());
		if(!hasUppercase){
			JOptionPane.showMessageDialog(null, "Your password must contain at least one uppercase character."," ", JOptionPane.INFORMATION_MESSAGE);					
			return false;
		}
		if(!hasLowercase){
			JOptionPane.showMessageDialog(null, "Your password must contain at least one lowercase character."," ", JOptionPane.INFORMATION_MESSAGE);					
			return false;
		}
		
		boolean hasSpecial   = !passwordAttempt.matches("[A-Za-z0-9 ]*");
		if(hasSpecial){
			JOptionPane.showMessageDialog(null, "Your password can only contain the characters A-Z, a-z, 0-9."," ", JOptionPane.INFORMATION_MESSAGE);					
			return false;
		}
		
		boolean hasDigit = passwordAttempt.matches(".*\\d+.*");
		if(!hasDigit){
			JOptionPane.showMessageDialog(null, "Your password must contain at least one numeric digit."," ", JOptionPane.INFORMATION_MESSAGE);					
			return false;
		}
  		  
		return true;
	}
	
	  public static void copyFile(String source, String destination) throws IOException{
		  String newline = System.getProperty("line.separator");
		  String data = null;
		  File in = new File(source);
		  File out = new File(destination);
	  	
			try { 

				
				final BufferedReader br = new BufferedReader(new FileReader(in));
				final BufferedWriter bw = new BufferedWriter(new FileWriter(out));

			    while ((data = br.readLine()) != null) {	
			        //Process each line and add output to Dest.txt file
			        bw.write(data);
			        bw.write(newline);
			        System.out.println("WRITE   - -   "+data);
			   }
			   br.close();
			   bw.close();
			   
			}catch ( EOFException eof ) {}
	  }
		
}