/**
	 * AutoPilot
  	 * Copyright 	- Telos Scientific - 
	 * Author 		- Jesse A. Campbell
	 * Description	- Main window (Auto Pilot 1.0) 
	 * 
	 * Version update information
	 * 09.21.12 JC - AutoPilot 1.2.3 - Fixed 1X reservoir to tubes.  Required setting SWN and DWN in VAS.TubePrep
	 * 09.25.12 JC - AutoPilot 1.2.4 - Added Volatile fluid determination to VAS.
	 * 09.29.12 JC - AutoPilot 1.2.5 - component disabling (e.g. buttons/tabs) to guide Operator.  
	 * 								 - VAS execution moved out of standard save routine.
	 * 								 - VAS execution carried out by VASi() with new intermediate file in System Folder
	 * 								 - Additional VAS logic for predetermining if Pipetting Technique is feasible
	 * 09.30.12 RSC- AutoPilot 1.2.6 - Added PK tab.
	 * 10.21.12 RSC- AutoPilot       - Added new field to PK tab.
	 * 10.21.12 JC - AutoPilot 1.2.7 - 	Modified public JComboBox getdPlateType().  This is the Dest deck pos pulldown for LT step.  Action listener now only collects selected index.
	 *					               	Confirm close of Advance windows:  AddButton1 and AdvButton1 check for Advanced Windows before executing actions.  Displayes Adv window if it is open.
	 *								   	Tool tip for splash screen set to NULL after selected deck layout or opening existing method
	 *									Fixed Open Chooser.  Cancel does not erase .apr info and does not enable TabWindow.
	 *									Fixed SaveAs button.  Cancel does not cause hang.
	 * 11.16.12 RSC- AutoPilot 1.2.8 -  Reformat PK tab.
	 * 11.21.12 RSC- AutoPilot 1.2.8 -  Reformat PK tab - #2.
	 * 
	 * 12.06.12 JC- AutoPilot 1.2.9 -   Updated Vas to 1.2.
	 * 12.31.12 JC - AutoPilot 1.2.9.12 - Made Assay options invisible for Beta Launch.
	 * 01.18.13 JC - AutoPilot 1.2.9.13 - Fixed Advanced WindowS frameActive boolean to be false with OK button action listener.
	 * 				- Fixed Delete Button - Edit step then Delete button then "NO".  Returns to the step being edited.
	 * 12.17.12 RSC- AutoPilot 1.2.10 - Updates to PK tab - #3
	 * 12.30.12 RSC- AutoPilot 1.2.11 - Updates to PK tab - Pre Assay and Assay Volume
	 * 03.17.13 RSC- AutoPilot 1.2.12 - Update PK tab to visual editor. 
	 * 03.31.13 RSC- AutoPilot 1.2.13 - Edit PK tab 
	 * 04.19.13 JC - AutoPilot 1.2.14 	- PK Module: Modified output format for PK module.  
	 * 									- PK Module: Added Pre-Assay and Assay Volume fields to interface.
	 * 									- PK Module: Re-arranged interface.
	 * 05.11.13 RSC- AutoPilot 1.2.15   - Make PK tab stand alone. Can't be used with other tabs.
	 * 05.20.13 JC - AutoPilot 1.2.16   - Minor Mods to PK modulate Interface to match PK Mod Version V1.0.18
	 * 05.23.13 JC - AutoPilot 1.2.17 	- Fixed page 2+ edits (element number = stepNumber - 16 (not 15))
	 * 05.23.14 JC - AutoPilot 1.2.18 	- Added step numbers, Right Arrow dissable/enable control.
	 * 									- PK interface: added "(diluent)" labels.
	 * 07.07.14 JC - AutoPilot 1.2.18   - Started rebuilding Serial Dilution
	 * 07.08.13 RSC- AutoPilot 1.2.18   - Added Diluent Deck Position to Serial Dilution
	 * 07.10.13 JC - AutoPilot 1.2.18   - Finish Serial Dilution V1.0
	 * 08.14.13 RSC- AutoPilot 1.2.18   - Move plate and SPE updates. 
	 * 08.14.13 JC - AutoPilot 1.2.18   - Icon for SPE and Step numbers for updates.
	 * 09.01.13 JC - AutoPilot 1.2.18   - Serial Dilution Updated to Version 1.4 - //SD1.4
	 * 									- SD1.4 - Serial Scheme Window is populated with row number equal to VAS interpretation of pattern
	 * 									- SD1.4 - DilWindow and SerialSchemePanel updated
	 * 10.25.13 RSC- AutoPilot 1.2.19   - Add deck position to PK module. 
	 * 11.02.13 RSC- AutoPilot 1.2.19   - Updates to deck position to PK module and dilution reservoirs. 
	 * 01.26.14 RSC- AutoPilot 1.2.20   - Cytomat module with cover pulldown
	 * 02.06.14	JC-	 AutoPilot 1.2.20	- AP-0001- Cherry Picking Pulldowns display 48 well plates  
	 * 02.08.14	JC-	 AutoPilot 1.2.21	- New Move Plate version  
	 * 02.12.14	RSC-			  		- Update to platesAll logic for cytomat and move plate #2	
	 * 03.06.14	RSC- AutoPilot 1.2.22	- New Cherry Pick
	 * 03.27.14 JC - AutoPilot 1.2.23	- Created and implemented VAStools.LiquidClassConfirm.  Requires new project resource libraries
	 * 									- Created and implemented VAStools.CP_WorklistConfirm to confirm format of CP worklist.
	 * 04.24.14 JC - AutoPilot 1.2.23   - Module Versions
	 * 										PK - 2.B.02 - Beta Version 
	 * 										MovePlate V2.1.0 - Move all plate types, move plates with and without lids, and just lids.
	 * 										Cytomat V2.0.0 -  Move all plate types (e.g. 48 well), move plates with and without lids, new graphics
	 * 										Cherry Picking V2.2.1 - Mixing, Barcodes
	 * 04.29.14 JC - AutoPilot 1.2.23   - Temporarily - SourceTube button visible.false (update_SD0020).  Need to fix DILUTION SCHEME button to work with source tubes (DONE)!
	 * 05.18.14 RSC- AutoPilot 1.2.24   - Corielle DNA Normalization
	 * 05.26.14 JC - AutoPilot 1.2.24   - Corielle DNA Normalization - Procedure selection determines required inputs.  Not yet functional!
	 * 06.30.14 JC - AutoPilot 1.2.24	- Latest
	 * 08.03.14 RSC- AutoPilot 1.2.25	- Addition of Panel48, Condition plate display on plate type pull down. Blank panel as default
	 * 09.01.14 RSC- AutoPilot 1.2.25	- Addition of Panel24, Blank page at start.
	 * 09.15.14 RSC- AutoPilot 1.2.26	- Fix edit check for destination deck position.
	 * 
	 * 09.16.14 JC - AutoPilot 1.3.1    - Modifications as listed below  (Note: sent to Andover as SP30 before 1.2.25 above existed.
	 * 									
	 * 									- Serial Minor Patch SD2.1.1  Fixed SD doesn't have to be first step.
	 * 									- schemePanel.table VIA DilWindow - end editing of any active cell on OK button so that info is saved to datamodel and thus can be recoreded/saved to .apr
	 * 									- ReporterPK removed splitting off of volume digits so that all are displayed.
	 * 
	 * 									- APF0019  -  Liquid Types are obtained directly from Hamilton .mdb
	 * 									- APF0020  -  Replaced (this.stepnbr < 15)  with  (this.stepnbr < 16)
	 * 									- APF0021  -  Fix PK dilution values being set to null.  
	 * 									- APF0022  -  Fix updSer() so that subsequent Icons are correct.  
	 * 									- APF0023  -  Fix SD Res type refresh on Edit step.
	 * 									- APF0024  -  Fix PK dilution values being set to null.  
	 *   								- APF0025  -  Confirm PK MRD text inputs are greater than 0.  
	 *   
	 *   09/17/24 - AutoPilot 1.3.2		- VEF0026  -  Modification to accommodate 24 and 48 well plate pipetting
	 *   								- APF0027  -  Fix Step Processing > 15, >30 etc.. 
	 *   								- APF0028  -  Added Protocol Page button enabling only when appropriate
	 *   								- APF0029  -  Cut and Copy of steps always remains enabled.
	 *   								- APF0030  -  Temporarily removed (commented out the add) menuItemInsert because it needs some fixing.
	 *   09/17/24 - AutoPilot 1.3.3		- APF0031  -  Changing width of Plate Type Comboboxes in LT steps.
	 *   09/18/24 - AutoPilot 1.3.05	- CPF0032  -  Worklist File text window ToolTip get set to text (i.e. selected file)
	 *   										   -  CP_MODULE Fixed Worklist File Name Parsing for Writing to Step Icon	
	 *   09/22/14 - AutoPilot 1.3.10	- Release  -  Released as 
	 *   
	 *   09/24/24 - AutoPilot 1.3.11	- PKF0033  -  PK Module V2.2.00
	 *   											  Accept Gyros Worklist (.xls) files.  NOTE: the .xls files are actually just text files with the wrong extension.  Therefore they are processed as text files.
	 *   09/24/24 - AutoPilot 1.3.12				  In VASengine and TabWindow and VAStools - check file formats for Add, update, and RUN.
	 *   								- PKF0034  -  THESE WERE NOT COMMENTED.  The pre-assay will be available to be run without dilution - All pkcheck3.setEnabled(false); replaced with pkcheck3.setEnabled(true);  
	 *   								  PKF0035     More accommodations to allow Pre-Assay without dilution
	 *   								  PKF0036  -  Change Start Pos of Samples to be A1 if stds are in tubes
	 *   10/08/14 - AutoPilot 1.3.13    - PKF0037  -  Accommodate worklists that don't contains samples.
	 *   10/27/14 - AutoPilot 1.3.13    - APN0038  -  Addition of Load Module
	 *   											  Error in UpdLoad()
	 *   10/31/14						- APN0039	  Modifications to Load Module components.  Added input checking.  Icons etc
	 *   10/31/14						- Mov0040	  Move Plate Module V2.1.1 - changed default grip height to 6 to accommodate deep well plates
	 *   11/20/14						- APN0041     New functionality for LT Plate Selection.  Combobox up, hover displays graphic if present....
	 *   11/27/14						- APN0042     Tubes added to Cherry02.  
	 *   12/03/14						- APF0043	  Modified Reservoir size criteria to be based on labware Dx..... size.   Dx<126 indicates not SBS.
	 *   12/03/14						- APF0044	  Serial Dilution to accomodate 384 destination
	 *   	 
	 *   01/14/14						- APN0045	  Tubes Pulldown to include new index 0	and graphics  
	 *   01/29/14 JC					- APN0046	  PK MODULE 3 with Graphic updates from JC										
	 *   01/29/15 RSC 
	 *   02/02/14 JC					- APN0047	  Reset graphics on Splash()	
	 *   02/18/14 JC					- APN0048	  Updates to Security.  Not all commented
	 *   											  Add confirmation on application close		
	 *   04/02/15 JC					- APN0049	  Updates for security
	 *   04/09/15 JC					- APF0050	  Fixed Tube Type pulldown -1 on Edit		
	 *   04/16/15 JC					- APN0051	  Selecting Worklist at Runtime	
	 *   04/21/15 JC					- APN0051	  Continuation..
	 *   04/22/15 JC					- APN0052	  Encryption of Username and Password 		
	 *   											  Encryption of Secure Directories	
	 *   04/24/15 JC					- APN0053     Reset PK properties button screen on logof	
	 *   04/28/15 JC					- APN0054     LiquidTransfer - check for well selection on plates and tube numbers on tubes	
	 *   05/21/15 JC					- APN0055     LiquidTransfer - check for well selection on plates and tube numbers on tubes				
	 *   07/28/15 RSC					- APN0055     Minor updates for PK changes			
	 *   07/29/15 JC					- APN0056     Security Enhancments			
	 *   08/21/15 JC+RSC				- APN0057     Minor updates and fixes for PK, Pause, Close, Security, Operator Save PK (11/30/15)			
	 *   11/29/15 RSC					- APN0058	  Add new LICONIC module.
	 *   12/11/15 JC					- APN0059 	  remove punctuation from Audit Trail input text
	 *   12/16/15 JC					- APF0060 	  Fixed dest multi Dest sequence equalization.  (VAS14)
	 *   12/30/15 JC					- APF0061	  Fixed MRD reporting for PK in VASReporter
	 * 				*/

package TelosV2Pkg;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GradientPaint;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.HeadlessException;
import java.awt.Image;
import java.awt.Insets;
import java.awt.Paint;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.FilenameFilter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Scanner;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.AbstractAction;
import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.DefaultListCellRenderer;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JRadioButton;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.JTextPane;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.border.Border;
import javax.swing.border.TitledBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileSystemView;

import TelosV2Pkg.PanelPK1.DirectoryRestrictedFileSystemView;

//import Aliquoting.TabWindow;

public class TabWindow extends JFrame
{
	public static String APversion = "AutoPilot V2.0.00";				//Requires Ham 2.0.04 or later
	public static String PKversion = "PK Module V3.0.02"; 				//.02 addition of Pre-Assay MrD 
	public static String CPversion = "Cherry Picking Module V2.2.1";
	public static String MPversion = "Move Plate Module V2.1.1";
	public static String CYTOversion = "Cytomat Module V2.0.0";
	public static String CORDNAversion = "gDN/A V1.2.0";
	public static String SDversion = "Serial Dilution V2.1.0";
	public static String LoadVersion = "Load Module V1.0.02";
	public static String Licversion = "Liconic Module V1.0.0";
	public static String SecurityVersion = "Security Module V1.1.00";
	public static Process p;
	
  JPopupMenu menuStep = new JPopupMenu("Popup");
  private JMenuBar menuBar = new JMenuBar(); // Window menu bar

  JPopupMenu menuAdmin = new JPopupMenu("Admin");
  
  private static final long serialVersionUID = 1L;
  private JPanel jContentPane = null;

  private JTabbedPane jTabbedPane1 = null;

  private JPanel dataWindow1 = null;

  private JPanel liqPanel = null;

  private JPanel liqdataPanel = null;

  private JPanel liqtrayPanel = null;

  private JPanel liqPanel1 = null;

  private JPanel liqPanel2 = null;

  private JPanel liqPanel1top = null;

  private JPanel liqPanel2top = null;

  private JPanel liqPanel1bot = null;

  private JPanel liqPanel2bot = null;

  private JPanel serPanel = null;

  private JPanel serdataPanel = null;

  private JPanel sertrayPanel = null;

  private JPanel serPanel1 = null;

  private JPanel serPanel1top = null;

  private JPanel serPanel2 = null;

  private JPanel serPanel2top = null;

  private JPanel brewPanel = null;

  private JPanel brewdataPanel = null;

  private JPanel brewtrayPanel = null;

  private JPanel brewPanel1 = null;

  private JPanel brewPanel2 = null;

  private JPanel brewPanel1top = null;

  private JPanel brewPanel2top = null;

  private JPanel brewPanel1bot = null;

  private JPanel brewPanel2bot = null;

  private JPanel incPanel = null;

  private JPanel incdataPanel = null;
  
  private JPanel readPanel = null;

  private JPanel readdataPanel = null;

  private JPanel cherryPanel = null;

  private JPanel cherrydataPanel = null;

  private JPanel cherry02Panel = null;

  private JPanel cherry02dataPanel = null;

  private CherryPanel02 cherry02dataPanel2 = null;

  private JPanel pkPanel = null;

  private JPanel pkdataPanel = null;

  private JPanel pkdataPanel1 = null;

  private PanelPK1 pkdataPanel2x = null;

  private JPanel pkdataPanel2 = null;

  private JPanel dnaPanel = null;

  private JPanel dnadataPanel1 = null;

  private PanelDNAnorm dnadataPanel2 = null;

  private JPanel loadPanel = null;

  private JPanel loaddnaPanel = null;

  private JPanel loaddataPanel1 = null;

  private LoadPanel1 loaddataPanel2 = null;

  private JPanel washPanel = null;

  private JPanel washdataPanel = null;

  private JPanel spePanel = null;

  private JPanel spedataPanel = null;

  private JPanel movePanel = null;

  private JPanel movePlatePanel = null;

  private JPanel pausePanel = null;
  
  private JPanel pausedataPanel1 = null;
  
  private PausePanel pausedataPanel2 = null;
  
  private JPanel SplashPanel = null;

  private JPanel movedataPanel = null;

  private JPanel movePlatedataPanel = null;

  private JPanel res12Xpanel = null;

  private JPanel res12Xpanel2 = null;

  private JPanel restextWindow = null;

  private JPanel restextWindow2 = null;

  private JPanel cytopanel = null;

  private JPanel cytodatapanel = null;

  private CytoPanel cytodatapanel2 = null;

  private JPanel movplv2panel = null;

  private JPanel movplv2datapanel = null;

  private MovePlatePanel02 movplv2datapanel2 = null;

  private JPanel licpanel = null;

  private JPanel licdatapanel = null;

  private LicoPanel licodatapanel2 = null;
 
  private PanelBlank2 panelbr = new PanelBlank2();
  
  private PanelBlank1 panelbl = new PanelBlank1();
  
  private Panel24 panel24r = new Panel24();

  private Panel48 panel48r = new Panel48();

  private Panel96 panel96r = new Panel96();

  private Panel384 panel384r = new Panel384();

  private Panel24 panel24l = new Panel24();

  private Panel48 panel48l = new Panel48();

  private Panel96 panel96l = new Panel96();

  private Panel384 panel384l = new Panel384();

  private Panel96 spanel96l = new Panel96();

  private Panel96 spanel96r = new Panel96();
  
  private Panel384 spanel384r = new Panel384();

  private TubePanel tubepanel = new TubePanel();

  private TubePanel tubepanel2 = new TubePanel();
  
  private ReservoirPanel ReservoirPanel1 = new ReservoirPanel();			//APN0047
  private ReservoirPanel ReservoirPanel2 = new ReservoirPanel();			//APN0047

  private TubePanel stubepanel = new TubePanel();

  private Res4XPanel res4Xpanel = new Res4XPanel();

  private Res4XPanel res4Xpanel2 = new Res4XPanel();

  private Res24XPanel res24Xpanel = new Res24XPanel();

  private Res24XPanel res24Xpanel2 = new Res24XPanel();

  private TubePanel brtubepanel = new TubePanel();

  private TubePanel brtubepanel2 = new TubePanel();

  private Res4XPanel brrespanel1 = new Res4XPanel();

  private Res4XPanel brrespanel2 = new Res4XPanel();

  JMenuItem menuItemEdit = new JMenuItem("Edit this step");
  JMenuItem menuItemInsert = new JMenuItem("Insert after this step");
  JMenuItem menuItemCut = new JMenuItem("Cut this step");
  JMenuItem menuItemCopy = new JMenuItem("Copy this step");
  JMenuItem menuItemPaste = new JMenuItem("Paste after this step");

  JMenuItem menuadmin1 = new JMenuItem("Admin");
  JMenuItem menuadmin2 = new JMenuItem("Admin2");
  JMenuItem menuadmin3 = new JMenuItem("Admin3");
  
  public static LogInWindow loginWindow;
  public static AuditTrail auditTrail = new AuditTrail();
  public static DirectorySelectionWindow directorySelect = new DirectorySelectionWindow();
  
  public static UserAccountWindow userAccountWindow = new UserAccountWindow();
  DilWindow dilwin = new DilWindow();
  AdvWindow advwin = new AdvWindow();
  AdvWindowD advwind = new AdvWindowD();

  private JFileChooser jFileChooser = null;
  private JFileChooser jFileChooser2 = null;
  private JFileChooser jFileChooser3 = null;
  private JFileChooser jFileChooser4 = null;
  private JFileChooser jFileChooser5 = null;

  private JCheckBox lmix = new JCheckBox(" MIX ");
  private JCheckBox lchill = new JCheckBox(" CHILL ");
  private JCheckBox lchill1 = new JCheckBox(" CHILL ");

  private JCheckBox lmix2 = new JCheckBox(" MIX ");
  private JCheckBox lshake = new JCheckBox(" SHAKE ");
  private JCheckBox brmix = new JCheckBox(" MIX ");
  private JCheckBox brmix2 = new JCheckBox(" MIX ");
  private JCheckBox brchill = new JCheckBox(" CHILL ");
  private JCheckBox brchill2 = new JCheckBox(" CHILL ");
  private JCheckBox pkcheck1 = new JCheckBox();
  public JCheckBox pkcheck2 = new JCheckBox();
  private JCheckBox pkcheck3 = new JCheckBox();
  private JCheckBox pkcheck4 = new JCheckBox();
  public static JButton saveButton = null; public static JButton saveAsButton = null; public static JButton rmvButton = null; private JButton editButton = null; public static JButton newButton = null; private JButton addButton = null; private JButton checkButton = null; private JButton infoButton = null; public static JButton layoutButton = null; public static JButton runButton = null; public static JButton layoutSelectButton = null;
  private JButton addButton1 = null; private JButton addButton2 = null; private JButton addButton3 = null; private JButton addButton4 = null; private JButton addButton5 = null; private JButton addButton6 = null; private JButton addButton7 = null; private JButton addButton8 = null; private JButton addButton9 = null; private JButton addButton10 = null; private JButton addButton11 = null;private JButton addButton12 = null;private JButton addButton13 = null ;
  private JButton updButton1 = null; private JButton updButton2 = null; private JButton updButton3 = null; private JButton updButton4 = null; private JButton updButton5 = null; private JButton updButton6 = null; private JButton updButton7 = null; private JButton updButton8 = null; private JButton updButton9 = null; private JButton updButton10 = null; private JButton updButton11 = null; private JButton updButton13 = null; 
  private JButton readButton = null; private JButton serialButton = null; public static JButton loadButton = null; private JButton workButton = null;private JButton watsonButton = null; public static JButton LogOffButton = null;  public static JButton securityButton = null;  public static JButton securityAccountButton = null;
  private JButton advButton1 = null; private JButton advButton2 = null; private JButton advButton3 = null;
  private JButton arrowButton1 = null; private JButton arrowButton2 = null;
  private JButton btnAdd = new JButton("  Add Step  ");
  private JButton btnUpdateStep = new JButton("Update Step");
  JButton cyt_btnAddStep = null;
  JButton cyt_btnUpdateStep = null;

  JButton movplv2_btnAddStep = null;
  JButton movplv2_btnUpdateStep = null;

  JButton dna_btnAddStep = null;
  JButton dna_btnUpdateStep = null;

  JButton load_btnAddStep = null;
  JButton load_btnUpdateStep = null;

  JButton cherry02_btnAddStep = null;
  JButton cherry02_btnUpdateStep = null;

  JButton lic_btnAddStep = null;
  JButton lic_btnUpdateStep = null;

  private JRadioButton platebutton = null; private JRadioButton resbutton = null; private JRadioButton tubebutton = null;
  private JRadioButton platebutton2 = null; private JRadioButton resbutton2 = null; private JRadioButton tubebutton2 = null;
  private JRadioButton tubebutton3 = null; private JRadioButton resbutton3 = null;
  private JRadioButton brtubebutton1 = null; private JRadioButton brresbutton1 = null;
  private JRadioButton brtubebutton2 = null; private JRadioButton brresbutton2 = null;
  private JRadioButton tubebutton5 = null; private JRadioButton platebutton3 = null;
  private JRadioButton abutton = null; private JRadioButton bbutton = null; private JRadioButton sbutton = null;
  private JRadioButton button48l = null; private JRadioButton button96l = null; private JRadioButton button384l = null;
  private JRadioButton button48l2 = null; private JRadioButton button96l2 = null; private JRadioButton button384l2 = null;
  private JRadioButton button96s = null; private JRadioButton button384s = null;
  private JRadioButton assembutton = null; private JRadioButton disassembutton = null; private JRadioButton vacbutton = null;
  private JRadioButton linc1button = null; private JRadioButton linc2button = null; private JRadioButton linc3button = null;
  private JRadioButton linc4button = null; private JRadioButton linc5button = null; private JRadioButton linc6button = null;
  private JRadioButton sinc1button = null; private JRadioButton sinc2button = null; private JRadioButton sinc3button = null;
  private JRadioButton sinc4button = null; private JRadioButton sinc5button = null; private JRadioButton sinc6button = null;
  private JCheckBox s1inc = new JCheckBox("Increment");
  private JCheckBox s2inc = new JCheckBox("Increment");
  private JCheckBox s3inc = new JCheckBox("Increment");
  private JCheckBox s4inc = new JCheckBox("Increment");

  ButtonGroup lbg = new ButtonGroup();
  ButtonGroup lbg2 = new ButtonGroup();
  ButtonGroup lbg3 = new ButtonGroup();
  ButtonGroup lbg4 = new ButtonGroup();
  ButtonGroup sbg = new ButtonGroup();
  ButtonGroup sbg2 = new ButtonGroup();
  ButtonGroup brbg1 = new ButtonGroup();
  ButtonGroup brbg2 = new ButtonGroup();
  ButtonGroup hbg = new ButtonGroup();
  ButtonGroup assbg = new ButtonGroup();

  private JComboBox sliqtype = null; private JComboBox dliqtype = null;
  private JComboBox plate = null; private JComboBox wash = null; private JComboBox note = null; public static JComboBox source = null; private JComboBox source2 = null; private JComboBox source3 = null; private JComboBox reader = null; private JComboBox liqtype = null;
  public static JComboBox dest = null; private JComboBox dest1 = null; private JComboBox dest2 = null; private JComboBox bliqtype = null;
  private JComboBox source4 = null; private JComboBox dest3 = null;
  private JComboBox source5 = null; private JComboBox dest4 = null;
  private JComboBox chsource = null; private JComboBox chdest = null; private JComboBox chliq = null;
  private JComboBox pksource = null; 
  private JComboBox pkdil = null; 
  private JComboBox pksamdil = null; 
  private JComboBox pkstddil = null; 
  private JComboBox pkliq = null; 
  private JComboBox pkvol = null; 
  private JComboBox pkassay = null;
  private JComboBox pkPassay = null;
  private JComboBox pkMRDdil = null;
  private JComboBox pkMRDsource = null;  
  private JComboBox pkSsource = null;  
  private JComboBox pkDsource = null;  
  private JComboBox pkSTDsource = null;
  private JComboBox pkDilsource = null;  
  private JComboBox pkDilreserve = null;  
  private JComboBox pkdeck1 = null; 
  private JComboBox pkdeck2 = null; 
  private JComboBox splatetype = null;
  private JComboBox splatetype2 = null;
  private JComboBox splatetype3 = null;
  private JComboBox splatetype4 = null;
  private JComboBox dplatetype = null;
  private JComboBox dplatetype2 = null;
  private JComboBox dplatetype3 = null;
  private JComboBox filter = null;
  private JComboBox collection = null;
  private JComboBox move = null;
  private JComboBox track1 = null;
  private JComboBox track2 = null;
  
  private FileInputStream input;
  private static FileReader input_FileReader;
  private JLabel label1 = new JLabel("          Comments             ");
  private JLabel label18 = new JLabel("    |    ");
  private JLabel label18b = new JLabel("    |    ");
  private JLabel label18c = new JLabel("    |    ");
  private JLabel label18d = new JLabel("    |    ");
  private JLabel label18e = new JLabel("    |    ");
  private JLabel label19 = new JLabel("           ");
  private JLabel label20 = new JLabel("           ");
  private JLabel label21 = new JLabel("        121");
  private JLabel label22 = new JLabel("        122");
  private JLabel label23 = new JLabel("        123");
  private JLabel label24 = new JLabel("        124");
  private JLabel label25 = new JLabel("        125");
  private JLabel label26 = new JLabel("        126");
  private JLabel label266 = new JLabel("        1266");
  private JLabel label2667 = new JLabel("        12667");
  private JLabel label2668 = new JLabel("        12668");
  private JLabel label2669 = new JLabel("        12669");
  private JLabel label27 = new JLabel("           ");
  private JLabel label28 = new JLabel("           ");
  private JLabel label29 = new JLabel("           ");
  private JLabel label30 = new JLabel("           ");
  private JLabel llabel1 = new JLabel("           ");
  private JLabel llabel2 = new JLabel("           ");
  private JLabel llabel3 = new JLabel("           ");
  private JLabel llabel4 = new JLabel("           ");
  private JLabel llabel5 = new JLabel("           ");
  private JLabel llabel6 = new JLabel("           ");
  private JLabel dlabel1 = new JLabel("_____________");
  private JLabel dlabel2 = new JLabel("_____________ ");
  private JLabel dlabel3 = new JLabel("_____________ ");
  private JLabel dlabel4 = new JLabel("_____________ ");
  private JLabel dlabel5 = new JLabel("_____________ ");
  private JLabel dlabel6 = new JLabel("_____________ ");
  private JLabel rlabel1 = new JLabel(" Reagent Name");
  private JLabel rlabel2 = new JLabel("Volume (ul)");
  private JLabel rlabel3 = new JLabel("Liquid Type");
  private JLabel rlabel4 = new JLabel("Volume (ul)");
  private JLabel stnlabel = new JLabel("Stock Name ");
  private JLabel stlabel = new JLabel("Stock Liquid Type ");
  private JLabel dlnlabel = new JLabel("Diluent Name");
  private JLabel dllabel = new JLabel("Diluent Liquid Type");
  private JLabel dlllabel = new JLabel("Diluent Labware");
  private JLabel brwlabel = new JLabel("Brew Reagent Name");
  private JLabel brnlabel = new JLabel("New Reagent Name");
  private JLabel brvlabel = new JLabel("Volume (ul)");
  private JLabel brltlabel = new JLabel("Liquid Type");
  private JLabel ilabel = new JLabel("  Minutes");
  private JLabel wmlabel = new JLabel(" Method ");
  private JLabel ilabelpm = new JLabel("  +/- Minutes");
  private JLabel templabel = new JLabel("  Temperature(c)");
  private JLabel itemplabel = new JLabel("  Temperature(c)");
  private JLabel blanklabel1 = new JLabel("            ");
  private JLabel blanklabel2 = new JLabel("            ");
  private JLabel blanklabel3 = new JLabel("            ");
  private JLabel blanklabel4 = new JLabel("            ");
  private JLabel blanklabel5 = new JLabel("            ");
  private JLabel blanklabel6 = new JLabel("            ");
  private JLabel blanklabel7 = new JLabel("            ");
  private JLabel blanklabel8 = new JLabel("            ");
  private JLabel blanklabel9 = new JLabel("            ");
  private JLabel blanklabel14 = new JLabel("            ");
  private JLabel blanklabel10 = new JLabel("            ");
  private JLabel blanklabel11 = new JLabel("            ");
  private JLabel blanklabel12 = new JLabel("            ");
  private JLabel blanklabel13 = new JLabel("            ");
  private JLabel blanklabel15 = new JLabel("            ");
  private JLabel blanklabel16 = new JLabel("            ");
  private JLabel ilabel1 = new JLabel("            ");
  private JLabel ilabel2 = new JLabel("            ");
  private JLabel ilabel3 = new JLabel("            ");
  private JLabel ilabel4 = new JLabel("            ");
  private JLabel relabel1 = new JLabel(" Reader     ");
  private JLabel relabel2 = new JLabel(" Read File  ");
  private JLabel relabel3 = new JLabel("            ");
  private JLabel relabel4 = new JLabel("            ");
  private JLabel chlabel1 = new JLabel(" Worklist File ");
  private JLabel chlabel2 = new JLabel(" First Plate Position ");
  private JLabel chlabel3 = new JLabel(" Source Labware ");
  private JLabel chlabel4 = new JLabel(" Destination Labware ");
  private JLabel chlabel5 = new JLabel("            ");
  private JLabel chlabel6 = new JLabel("            ");
  private JLabel chlabel7 = new JLabel("            ");
  private JLabel chlabel8 = new JLabel("            ");
  private JLabel chlabel9 = new JLabel("            ");
  private JLabel chlabel10 = new JLabel("            ");
  private JLabel chlabel11 = new JLabel("            ");
  private JLabel chlabel12 = new JLabel("            ");
  private JLabel chlabel13 = new JLabel("            ");
  private JLabel chlabel14 = new JLabel("            ");
  private JLabel chlabel15 = new JLabel("            ");
  private JLabel chlabel16 = new JLabel("            ");
  private JLabel chlabel17 = new JLabel("            ");
  private JLabel chlabel18 = new JLabel("            ");
  private JLabel chlabel19 = new JLabel("            ");
  private JLabel chlabelto = new JLabel(" Last Plate Position");
  private JLabel chlabel20 = new JLabel(" Liquid Type ");
  private JLabel chlabel21 = new JLabel("            ");
  private JLabel chlabel22 = new JLabel("            ");
  private JLabel chlabel23 = new JLabel("            ");
  private JLabel chlabel24 = new JLabel("            ");
  private JLabel chlabel25 = new JLabel("            ");
  private JLabel chlabel26 = new JLabel("            ");
  private JLabel chlabel27 = new JLabel("            ");
  private JLabel chlabel28 = new JLabel("            ");
  private JLabel chlabel29 = new JLabel("            ");
  private JLabel chlabel30 = new JLabel("            ");
  private JLabel chlabel31 = new JLabel("            ");
  private JLabel chlabel32 = new JLabel("            ");
  private JLabel chlabel33 = new JLabel("            ");
  private JLabel pklabel1 = new JLabel("Watson Worklist");
  private JLabel pklabel2 = new JLabel("Create Assay Plate");
  private JLabel pklabel2b = new JLabel("Source Labware");
  private JLabel pklabel3 = new JLabel("Dilution Labware");
  private JLabel pklabel4 = new JLabel("Assay Labware");
  private JLabel pklabel4b = new JLabel("Pre Assay Labware");
  private JLabel pklabel4c = new JLabel("STD Labware");
  private JLabel pklabel5 = new JLabel("MRD");
  private JLabel pklabel6 = new JLabel("MRD Diluent");
  private JLabel pklabel6b = new JLabel("MRD Volume");
  private JLabel pklabel6c = new JLabel("Pre Assay Volume");
  private JLabel pklabel6d = new JLabel("Assay Volume");
  private JLabel pklabel7 = new JLabel("Dilution Diluent");
  private JLabel pklabel8 = new JLabel("Create MRD");
  private JLabel pklabel9 = new JLabel("MRD Labware");
  private JLabel pklabel10 = new JLabel("Sample Labware");
  private JLabel pklabel11 = new JLabel("Create Dilution Plates");
  private JLabel pklabel12 = new JLabel("Create Pre Assay Plates");
  private JLabel pklabel13 = new JLabel("Create Assay Plate ");
  private JLabel pklabel14 = new JLabel("            ");
  private JLabel pklabel15 = new JLabel("            ");
  private JLabel pklabel16 = new JLabel("            ");
  private JLabel pklabel17 = new JLabel("            ");
  private JLabel pklabel18 = new JLabel("            ");
  private JLabel pklabel19 = new JLabel("            ");
  private JLabel pklabelto = new JLabel(" Last Plate Position");
  private JLabel pklabel20 = new JLabel(" Matrix Liquid ");
  private JLabel pklabel21 = new JLabel("            ");
  private JLabel pklabel22 = new JLabel("            ");
  private JLabel pklabel23 = new JLabel("            ");
  private JLabel pklabel24 = new JLabel("            ");
  private JLabel pklabel25 = new JLabel("            ");
  private JLabel pklabel26 = new JLabel("            ");
  private JLabel pklabel27 = new JLabel("            ");
  private JLabel pklabel28 = new JLabel("            ");
  private JLabel pklabel29 = new JLabel("            ");
  private JLabel pklabel30 = new JLabel("            ");
  private JLabel pklabel31 = new JLabel("            ");
  private JLabel pklabel32 = new JLabel("            ");
  private JLabel pklabel33 = new JLabel("            ");
  private JLabel pklabel34 = new JLabel("            ");
  private JLabel pklabel35 = new JLabel("            ");
  private JLabel pklabel36 = new JLabel("            ");
  private JLabel pklabel37 = new JLabel("            ");
  private JLabel pklabel38 = new JLabel("            ");
  private JLabel pklabel39 = new JLabel("            ");
  private JLabel pklabel40 = new JLabel("            ");
  private JLabel pklabel41 = new JLabel("            ");
  private JLabel pklabel42 = new JLabel("            ");
  private JLabel pklabel43 = new JLabel("            ");
  private JLabel pklabel44 = new JLabel("            ");
  private JLabel pklabel45 = new JLabel("            ");
  private JLabel pklabel46 = new JLabel("            ");
  private JLabel pklabel47 = new JLabel("            ");
  private JLabel pklabel48 = new JLabel("            ");
  private JLabel pklabel49 = new JLabel("            ");
  private JLabel pklabel50 = new JLabel("            ");
  private JLabel pklabel51 = new JLabel("            ");
  private JLabel pklabel52 = new JLabel("            ");
  private JLabel pklabel53 = new JLabel("            ");
  private JLabel pklabel54 = new JLabel("            ");
  private JLabel pklabel55 = new JLabel("            ");
  private JLabel pklabel56 = new JLabel("            ");
  private JLabel pklabel57 = new JLabel("            ");
  private JLabel pklabel58 = new JLabel("            ");
  private JLabel pklabel59 = new JLabel("            ");
  private JLabel pklabel60 = new JLabel("            ");
  private JLabel pklabel61 = new JLabel("            ");
  private JLabel pklabel62 = new JLabel("            ");
  private JLabel pklabel63 = new JLabel("            ");
  private JLabel pklabel64 = new JLabel("            ");
  private JLabel pklabel65 = new JLabel("            ");
  private JLabel pklabel66 = new JLabel("            ");
  private JLabel pklabel67 = new JLabel("            ");
  private JLabel pklabel68 = new JLabel("            ");
  private JLabel pklabel69 = new JLabel("            ");
  public JLabel pklabel70 = new JLabel("(diluent)");
  public JLabel pklabel71 = new JLabel("(diluent)");
  public JLabel lblPosition = new JLabel("Position");

  private JLabel spelabel1 = new JLabel("Filter      ");
  private JLabel spelabel2 = new JLabel("Collection  ");
  private JLabel spelabel3 = new JLabel("Time (Sec)     ");
  private JLabel spelabel4 = new JLabel("Pressure (mbar)");
  private JLabel spelabel5 = new JLabel("            ");
  private JLabel spelabel6 = new JLabel("            ");
  private JLabel spelabel7 = new JLabel("            ");
  private JLabel spelabel8 = new JLabel("            ");
  private JLabel spelabel9 = new JLabel("            ");
  private JLabel spelabel10 = new JLabel("            ");
  private JLabel spelabel11 = new JLabel("            ");
  private JLabel movlabel1 = new JLabel("Move Assay Plate to: ");
  private JLabel movplabel1 = new JLabel("Plate      ");
  private JLabel movplabel2 = new JLabel("           ");
  private JLabel movplabel3 = new JLabel("Source     ");
  private JLabel movplabel4 = new JLabel("Destination");
  private JLabel movplabel5 = new JLabel("           ");
  private JLabel movplabel6 = new JLabel("           ");
  private JLabel movplabel7 = new JLabel("           ");
  private JLabel movplabel8 = new JLabel("           ");
  private JLabel movplabel9 = new JLabel("           ");
  private JLabel movplabel10 = new JLabel("           ");
  private JLabel movplabel11 = new JLabel("           ");
  private JLabel wlabel1 = new JLabel("            ");
  private JLabel wlabel2 = new JLabel("            ");
  private JLabel wlabel3 = new JLabel("            ");
  private JLabel wlabel4 = new JLabel("            ");
  private JLabel wlabel5 = new JLabel("            ");
  private JLabel wlabel6 = new JLabel("            ");
  private JLabel ltubel1 = new JLabel("Tube# ");
  private JLabel stubel1 = new JLabel("First Tube ");
  private JLabel stubel2 = new JLabel("Last Tube ");
  private JLabel btubel1 = new JLabel("Tube# ");
  private JLabel botlab1 = new JLabel(" Mix (ul)  ");
  private JLabel botlab2 = new JLabel("           ");
  private JLabel botlab3 = new JLabel("           ");
  private JLabel botlab4 = new JLabel("           ");
  private JLabel botlab5 = new JLabel(" Mix (ul)  ");
  private JLabel botlab6 = new JLabel("           ");
  private JLabel botlab7 = new JLabel("           ");
  private JLabel botlab8 = new JLabel("           ");
  private JLabel serlab1 = new JLabel("           ");
  private JLabel serlab2 = new JLabel("           ");
  private JLabel serlab3 = new JLabel("           ");
  private JLabel serlab4 = new JLabel("           ");
  private JLabel serlab5 = new JLabel("           ");
  private JLabel serlab6 = new JLabel("           ");
  private JLabel serlab7 = new JLabel("           ");
  private JLabel serlab8 = new JLabel("           ");
  private JLabel serlab9 = new JLabel("           ");
  private JLabel serlab10 = new JLabel("           ");
  private JLabel serlab11 = new JLabel("           ");
  private JLabel serlab12 = new JLabel("           ");
  private JLabel brwlab1 = new JLabel("           ");
  private JLabel brwlab2 = new JLabel("           ");
  private JLabel brwlab3 = new JLabel("           ");
  private JLabel brwlab4 = new JLabel("           ");
  private JLabel brwlab5 = new JLabel("           ");
  private JLabel brwlab6 = new JLabel("           ");
  private JLabel brwlab7 = new JLabel("           ");
  private JLabel brwlab8 = new JLabel("           ");
  private JLabel brwlab9 = new JLabel("           ");
  private JLabel brwlab10 = new JLabel("           ");

  private JPanel textWindow = null;

  private JTextPane[] dTextArea = new JTextPane[15];

  private JTextField lmixt1 = new JTextField(4);
  private JTextField lmixt2 = new JTextField(4);
  private JTextField text1 = new JTextField(13);
  private JTextField text2 = new JTextField(13);
  private JTextField text3 = new JTextField(13);
  private JTextField text4 = new JTextField(13);
  private JTextField text4pm = new JTextField(13);
  private JTextField itemp = new JTextField(2);
  private JTextField text6 = new JTextField(13);
  private JTextField temp = new JTextField(2);
  private JTextField text7 = new JTextField(13);
//  private JTextField text8 = new JTextField(13);
  private JTextField pkm_text_wat = new JTextField();
  private JTextField pkm_text_1 = new JTextField();
  private JTextField pkm_text_ul = new JTextField();
  private JTextField pkm_text_ul_preAssay = new JTextField();
  private JTextField pkm_text_ul_Assay = new JTextField();
  private JTextField text9 = new JTextField(13);
  private JTextField text10 = new JTextField(13);
  private JTextField text11 = new JTextField(13);
  private JTextField text12 = new JTextField(13);
  private JTextField wmethod = new JTextField(13);
  private JTextField time = new JTextField(13);
  private JTextField pressure = new JTextField(13);
//  private JTextField cyt_text_1 = new JTextField();
//  private JTextField cyt_text_2 = new JTextField();
  
  private JLabel lblSamples = new JLabel("Samples:");
  private JLabel lblStandards = new JLabel("Standards:");
  private JLabel lblDiluent1 = new JLabel("Diluent:");
  private JLabel lblDiluent2 = new JLabel("Diluent:");


  private JTextField dltext = new JTextField(13);
  private JTextField sttext = new JTextField(13);
  private JTextField brtext = new JTextField(13);
  private JTextField brvol = new JTextField(13);
  private JTextField brntext = new JTextField(13);
  private JTextArea[] resTextArea = new JTextArea[15];
  private JTextArea[] resTextArea2 = new JTextArea[15];

  private String res1 = "";
  private String cytloc = "";
  private String cytdeckloc = "";
  private String cytfromto = "";
  private String cytcover = "";
  private int cyt_fromto = 0;
  private int cyt_cover = 0;
  private String cyt_plate = "";
  private String cyt_tsp = "";
  private int cyt_plateindex = 0;
  private String licbutton = "";

  private String licloc = "";
  private String licdeckloc = "";
  private String licfromto = "";
  private String liccover = "";
  private int lic_fromto = 0;
  private int lic_cover = 0;
  private String lic_plate = "";
  private String lic_tsp = "";
  private int lic_plateindex = 0;

  private String startloc = "";
  private String endloc = "";
  private String movetype = "";
  private int mvp_type = 0;
  private int mvp_plateindex = 0;
  private String mvp_plate = "";
  
  private int ch2_slabindex = 0;
  private int ch2_dlabindex = 0;
  private int ch2_splateindex = 0;
  private int ch2_dplateindex = 0;
  private String ch2_splate = "";
  private String ch2_dplate = "";
  private String ch2_liqtype = "";
  private String ch2_plateloc = "";
  private String ch2_tubeloc = "";
  private String ch2_smix = "";
  private String ch2_dmix = "";
  private String ch2_scycle = "";
  private String ch2_dcycle = "";
  private String pminutes = "";
  private String pseconds = "";
  
  private int smixval = 0;
  private int dmixval = 0;
  
  private int dna_procindex = 0;
  private int dna_plateindex = 0;
  private int dna_splateindex = 0;
  private int dna_nplateindex = 0;
  private int dna_dplateindex = 0;
  private int dna_deckindex1 = 0;
  private int dna_deckindex2 = 0;
  private int dna_deckindex3 = 0;
  private int dna_deckindex4 = 0;
  private String dna_qfile = "";
  private String dna_con = "";
  private String dna_vol = "";
  private String dna_dplate = "";
  private String dna_splate = "";
  private String dna_nplate = "";
  private String dna_plate = "";
  private String dna_deck1 = "";
  private String dna_deck2 = "";
  private String dna_deck3 = "";
  private String dna_deck4 = "";
  private String proc = "";
  
  private int load_plateindex = 0;
  private int load_tubeindex = 0;
  private int load_tipindex = 0;
  private String load_plate = "";
  private String load_tube = "";
  private String load_tip = "";
  private String load_plindex = "";
  private String load_tuindex = "";
  private String load_tiindex = "";

  
  public String layoutFileSelected = "";
  public String aprActive = "";
  public JLabel lres1 = new JLabel("Reservoir # ");
  public JTextField reservoir = new JTextField(2);
  public String text = "";
  public Component componentSelectedS = TabWindow.this.panelbr;
  public Component componentSelectedD = TabWindow.this.panelbr;
  
  private JTextField stepnum = new JTextField(2);
  private static final String title = "ELISA SETUP";
  private String[] tracks = { "1", "2", "3", "4", "5", "6", 
    "7", "8", "9", "10", "11", "12", 
    "13", "14", "15", "16", "17", "18", 
    "19", "20", "21", "22", "23", "24", 
    "25", "26", "27", "28", "29", "30", 
    "31", "32", "33", "34", "35", "36", 
    "37", "38", "39", "40", "41", "42", 
    "43", "44", "45", "46", "47", "48", 
    "49", "50", "51", "52", "53", "54" };

  String[] arrayLabware = new String[100];

  ArrayList<String> arrayPlates24 = new ArrayList();		//AP-0001
  ArrayList<String> arrayPlates48 = new ArrayList();		//AP-0001
  ArrayList<String> arrayPlates96 = new ArrayList();
  ArrayList<String> arrayPlates384 = new ArrayList();
  ArrayList<String> arrayReservoirs = new ArrayList();
  ArrayList<String> arrayTubes = new ArrayList();
  public static ArrayList<String> arrayList_sdVolumesTransfer = new ArrayList<String>();
  public static ArrayList<String> arrayList_sdVolumesDiluent = new ArrayList<String>();
  
  private static String[] plates24;		//AP-0001
  private static String[] plates48;		//AP-0001
  private static String[] plates96;
  private static String[] plates96tubes;
  private static String[] platesAll;
  private static String[] plates384;
  private static String[] platetypes;
  private static String[] filters;
  private static String[] tubes;
  private static String[] reservoirs;
  private static String[] liqtypes;
  private static String[] platesTubes;
  private static String[] tubesAll;
  private static String[] deckpositions;
  private static String[] reservoirsAll;
  
  
  private String[] admins = { " Admin1  ", "  Admin2  ", "  Admin3  " };
  
  private String[] tabs;
  private String[] washes = { "   1X   ", "   2X   ", "   3X   ", "   4X   ", "   5X   ", "   6X   " };
  private String[] readers = { "LMax384II", "    M5   ", " Plus384 ", "    MSD  " };
  private String[] sources2 = { " Reservoir_1X ", " Reservoir_4X ", " Reservoir_5X ", " Reservoir_24X " };
  private String[] names = { "C:/Program Files/TELOS/AutoPilot/Config/back.png", "C:/Program Files/TELOS/AutoPilot/Config/forward.png", "C:/Program Files/TELOS/AutoPilot/Config/open.png", 
    "C:/Program Files/TELOS/AutoPilot/Config/save.png", "C:/Program Files/TELOS/AutoPilot/Config/new.png", "C:/Program Files/TELOS/AutoPilot/Config/delete.png", 
    "C:/Program Files/TELOS/AutoPilot/Config/vertLine1.png", "C:/Program Files/TELOS/AutoPilot/Config/WaterTransfer1.png", "C:/Program Files/TELOS/AutoPilot/Config/move.png", 
    "C:/Program Files/TELOS/AutoPilot/Config/add.png", "C:/Program Files/TELOS/AutoPilot/Config/update.png", "C:/Program Files/TELOS/AutoPilot/Config/info.png", 
    "C:/Program Files/TELOS/AutoPilot/Config/edit.png", "C:/Program Files/TELOS/AutoPilot/Config/layout.png", "C:/Program Files/TELOS/AutoPilot/Config/run.png", 
    "C:/Program Files/TELOS/AutoPilot/Config/add16.png", "C:/Program Files/TELOS/AutoPilot/Config/update16.png", "C:/Program Files/TELOS/AutoPilot/Config/exclamation.png", 
    "C:/Program Files/TELOS/AutoPilot/Config/ok.png", "C:/Program Files/TELOS/AutoPilot/Config/cherry.png", "C:/Program Files/TELOS/AutoPilot/Config/pause.png", 
    "C:/Program Files/TELOS/AutoPilot/Config/saveas.png", "C:/Program Files/TELOS/AutoPilot/Config/molecule.png", "C:/Program Files/TELOS/AutoPilot/Config/PK.png", 
    "C:/Program Files/TELOS/AutoPilot/Config/wash.png", "C:/Program Files/TELOS/AutoPilot/Config/incubate.png", "C:/Program Files/TELOS/AutoPilot/Config/Read.png", 
    "C:/Program Files/TELOS/AutoPilot/Config/SerialDilution.png", "C:/Program Files/TELOS/AutoPilot/Config/SPE.png", "C:/Program Files/TELOS/AutoPilot/Config/Cytomat.png", 
    "C:/Program Files/TELOS/AutoPilot/Config/user_admin.png", "C:/Program Files/TELOS/AutoPilot/Config/security.png", "C:/Program Files/TELOS/AutoPilot/Config/Load.png", "C:/Program Files/TELOS/AutoPilot/Config/Accounts.png"};

  private Icon[] icons = { new ImageIcon(this.names[0]), 
    new ImageIcon(this.names[1]), 
    new ImageIcon(this.names[2]), 
    new ImageIcon(this.names[3]), 
    new ImageIcon(this.names[4]), 
    new ImageIcon(this.names[5]), 
    new ImageIcon(this.names[6]), 
    new ImageIcon(this.names[7]),//Liquid Transfer Droplet 
    new ImageIcon(this.names[8]),//Move 
    new ImageIcon(this.names[9]), 
    new ImageIcon(this.names[10]), 
    new ImageIcon(this.names[11]), 
    new ImageIcon(this.names[12]), 
    new ImageIcon(this.names[13]), 
    new ImageIcon(this.names[14]), 
    new ImageIcon(this.names[15]), 
    new ImageIcon(this.names[16]), 
    new ImageIcon(this.names[17]), 
    new ImageIcon(this.names[18]), 
    new ImageIcon(this.names[19]), //Cherry
    new ImageIcon(this.names[20]),
    new ImageIcon(this.names[21]),
    new ImageIcon(this.names[22]),
    new ImageIcon(this.names[23]),
    new ImageIcon(this.names[24]),//wash
    new ImageIcon(this.names[25]),//incubate
    new ImageIcon(this.names[26]),//read
    new ImageIcon(this.names[27]),//Serial
    new ImageIcon(this.names[28]),//SPE
    new ImageIcon(this.names[29]),//Cytomat
    new ImageIcon(this.names[30]),//LogOff
    new ImageIcon(this.names[31]),//Security
    new ImageIcon(this.names[32]),//Load
    new ImageIcon(this.names[33]),//Accounts
    };
    
  private  ImageIcon images1 = new ImageIcon("C:/Program Files/TELOS/AutoPilot/Config/FROM_Cytomat.png");
  private  ImageIcon images2 = new ImageIcon("C:/Program Files/TELOS/AutoPilot/Config/TO_Cytomat.png");
  private  String petString1 = "FROM_Cytomat";
  private  String petString2 = "TO_Cytomat";
  
  private JLabel labelVerticalLine = new JLabel(this.icons[6]);
  private String de = " ";
  private String de1 = " ";
  private String de2 = " ";
  private String de3 = " ";
  private String de4 = " ";
  private String p11 = "";
  private String w1 = "";
  private String n1 = "";
  private String mix1 = "";
  private String mix2 = "";
  private String s1 = "";
  private String so1 = "";
  private String so2 = "";
  private String so3 = "";
  private String so4 = "";
  private String so5 = "";
  private String chso = "";
  private String chde = "";
  private String chltyp = "";
  private String pkso = "";
  private String pkdi = "";
  private String pkltyp = "";
  private String pkas = "";
  private String pkck1 = "";
  private String pkck2 = "";
  private String pkck3 = "";
  private String pkck4 = "";
  private String pkmrddil = "";
  private String pkdildil = "";
  private String pkmrdlab = "";
  private String pksamlab = "";
  private String pkdillab = "";
  private String pkdillab2 = "";
  private String pkdillab3 = "";
  private String pkmrddeck = "";
  private String pkdildeck = "";
  private String pkpasslab = "";
  private String pkstdlab = "";
  private String pkasslab = "";
  private String pksam = "";
  private String pkstd = "";

  private String tr1 = "";
  private String tr2 = "";
  private String fil = "";
  private String col = "";
  private String mov = "";
  private String movpl = "";
  private String ti = "";
  private String pre = "";
  private String brso = "";
  private String r1 = "";
  private String rs1 = "";
  private String rs2 = "";
  private String lt1 = "";
  private String slt1 = "";
  private String dlt1 = "";
  private String blt1 = "";
  private String spl1 = "";
  private String spl1u = "";
  private String spl2 = "";
  private String spl2u = "";
  private String spl3 = "";
  private String spl3u = "";
  private String spl4 = "";
  private String spl4u = "";
  private String dpl1 = "";
  private String dpl1u = "";
  private String dpl2 = "";
  private String dpl2u = "";
  private String dpl3 = "";
  private String dpl3u = "";
  private String bpl1 = "";
  private String bpl2 = "";
  private String pkd1u = "";
  private String pkd2u = "";
  private String pkl1u = "";
  private String pkl2u = "";
  private String h1 = "1";
  private String save1 = " SAVE Protocol";
  private String hline3 = "-";
  private String hline4 = "-";
  private String hline5 = "-";
  private String hline6 = "-";
  private String hline7 = "-";
  private String hline8 = "-";
  private String hline9 = "-";
  private String hline10 = "-";
  private String path = " ";
  
  public static String security_userLoggedIn = "";
  public static String security_validationTitle = "";
  public static String security_updateString = "";
  public static String security_auditTrailString = "-";		//Final format is DateTime;Author;Action;Reason;version
  public static String security_accessLevel = "0";
  public static String security_pKWorklist_Runtime;
  public static boolean security_Validated = false;
  
  public static boolean run_Started = false;				//APN0057
  public static boolean run_Completed = false;

  private boolean mrdon = false;
  private boolean dilon = false;
  private boolean assayon = false;
  private boolean preassayon = false;

  int counter = 0;
  int value = 0;
  int valuem1 = 0;
  int valuem2 = 0;
  int readCount = 0;
  int p24count = 0;	//AP-0001
  int p48count = 0;	//AP-0001
  int p96count = 0;
  int p384count = 0;
  int pALLcount = 0; //AP-0001
  int patcount = 0; //AP-0001
  int pat96count = 0;   
  int tallcount = 0; //AP-0001
  int rallcount = 0; //AP-0001
  int fcount = 0;
  int ptcount = 0;
  int tcount = 0;
  int rcount = 0;
  int lcount = 0;
  int tabcount = 0;
  int pagecount = 0;
  int stepnbr = 0;
  int stepboxnbr = 0;
  int rnum = 0;
  int rnum2 = 0;
  int cntrGlobal1 = 0;
  int cntrGlobal2 = 0;
  int cntrGlobal3 = 0;
  int cntrGlobal4 = 0;
  int nonPK = 0;
  int cntPK = 0;

  public static boolean security = true;
  boolean PKmod = false;
  boolean rHam;
  boolean rHam2;
  boolean saveAs = false;
  boolean update = false;
  boolean load = false;
  boolean showtipwalkaway = false;
  boolean showchill = false;
  boolean showshake = false;
  boolean showtube = false;
  boolean showres = false;

  
  List dList = new List();
  List tList = new List();
  List fList = new List();
  List eList = new List();
  List bList = new List();
  List loadList = new List();

  List dbList = new List();
  List deList = new List();

  List tbList = new List();
  List teList = new List();

  List cdList = new List();
  List ctList = new List();
  List cfList = new List();

  Font font = new Font("Arial", 0, 9);
  Font font10 = new Font("Arial", 0, 11);
  Font font15 = new Font("Arial", 1, 15);
  Font font25 = new Font("Arial", 1, 25);

  private Color regColor = new Color(10, 155, 255);
  private Color incColor = new Color(10, 255, 155);
  private Color wshColor = new Color(255, 155, 10);
  private Color brewColor = new Color(255, 255, 100);
  private Color serColor = new Color(100, 155, 100);
  private Color readColor = new Color(100, 255, 100);
  private Color speColor = new Color(100, 180, 60);
  private Color movColor = new Color(100, 120, 140);
  private Color movplColor = new Color(100, 120, 160);
  private Color pauColor = new Color(100, 120, 180);
  private Color panelColor = new Color(248, 248, 255);
  private Color saveColor = new Color(100, 120, 180);
  private Color steelColor = new Color(176, 196, 222);
  private Color steelDrkColor = new Color(70, 130, 180);

  Border blackline = BorderFactory.createLineBorder(Color.black);
  Border redline = BorderFactory.createLineBorder(Color.red, 3);
  Border clearline = BorderFactory.createLineBorder(this.panelColor, 5);
  Border greenline = BorderFactory.createLineBorder(Color.GREEN, 3);

  private JTabbedPane getJTabbedPane1()
  {
	  
    if (this.jTabbedPane1 == null) {
      this.jTabbedPane1 = new JTabbedPane();
      
      int tindex = tabSearch(this.tabs, "Liquid Transfer");
      if (tindex > -1)
      {
        this.jTabbedPane1.addTab("Liquid Transfer  ", this.icons[7], getLiqPanel(), "Create a step to transfer fluid from a source to a destination");
      }

      tindex = tabSearch(this.tabs, "Serial Dilution");
      if (tindex > -1)
      {
        this.jTabbedPane1.addTab("Serial Dilution  ", this.icons[27], getSerPanel(), SDversion);
      }
      tindex = tabSearch(this.tabs, "Brew");
      if (tindex > -1)
      {
        this.jTabbedPane1.addTab("Brew", null, getBrewPanel(), null);
      }
      tindex = tabSearch(this.tabs, "Wash");
      if (tindex > -1)
      {
        this.jTabbedPane1.addTab("Wash  ", this.icons[24], getWashPanel(), null);
      }
      tindex = tabSearch(this.tabs, "Incubate");
      if (tindex > -1)
      {
        this.jTabbedPane1.addTab("Incubate  ", this.icons[25], getIncPanel(), null);
      }
      tindex = tabSearch(this.tabs, "Read");
      if (tindex > -1)
      {
        this.jTabbedPane1.addTab("Read  ", this.icons[26], getReadPanel(), null);
      }
      tindex = tabSearch(this.tabs, "Pause");
      if (tindex > -1)
      {
        this.jTabbedPane1.addTab("Pause ", this.icons[20], getPausePanel(), null);
      }
      tindex = tabSearch(this.tabs, "Move");
      if (tindex > -1)
      {
        this.jTabbedPane1.addTab("Move  ", null, getMovePanel(), "Move Labware from source location to destination location");
      }
      tindex = tabSearch(this.tabs, "Move Plate");
      if (tindex > -1)
      {
        this.jTabbedPane1.addTab("Move Plate ", this.icons[8], getMovePlatePanel(), "Move Labware from source location to destination location");
      }
      tindex = tabSearch(this.tabs, "SPE");
      if (tindex > -1)
      {
        this.jTabbedPane1.addTab("SPE ", this.icons[28], getSPEPanel(), null);
      }
      tindex = tabSearch(this.tabs, "Cherry Picking");
      if (tindex > -1)
      {
        this.jTabbedPane1.addTab("Cherry Picking ", this.icons[19], getCherryPanel(), CPversion);
      }
      tindex = tabSearch(this.tabs, "PK");
      if (tindex > -1)
      {
    	PKmod = true;
        this.jTabbedPane1.addTab("PK  ", this.icons[23], getPKPanel(), PKversion);
      }

      tindex = tabSearch(this.tabs, "Cytomat01");
      if (tindex > -1)
      {
        this.jTabbedPane1.addTab("Cytomat01  ", this.icons[29], getCytoPanel(), CYTOversion);
      }

      tindex = tabSearch(this.tabs, "Liconic01");
      if (tindex > -1)
      {
        this.jTabbedPane1.addTab("Liconic01  ", this.icons[29], getLicPanel(), Licversion);
      }

      tindex = tabSearch(this.tabs, "MovePlate02");
      if (tindex > -1)
      {
        this.jTabbedPane1.addTab("Move Plate  ", this.icons[8], getMovePlate2Panel(), MPversion);
      }

      tindex = tabSearch(this.tabs, "CherryPick02");
      if (tindex > -1)
      {
        this.jTabbedPane1.addTab("Cherry Picking ", this.icons[19], getCherry02Panel(), CPversion);
      }
      tindex = tabSearch(this.tabs, "CORDNA01");
      if (tindex > -1)
      {
        this.jTabbedPane1.addTab("gDN/A ", this.icons[8], getDNAPanel(), CORDNAversion);
      }
      tindex = tabSearch(this.tabs, "LOAD01");
      if (tindex > -1)
      {
        this.jTabbedPane1.addTab("Load   ", this.icons[32], getLoadPanel(), LoadVersion);
      }

      this.jTabbedPane1.addTab("Splash", this.icons[22], getSplashPanel(), "Intro");
      
      this.jTabbedPane1.addChangeListener(new ChangeListener(){
		@Override
		public void stateChanged(ChangeEvent arg0) {
			// TODO Auto-generated method stub	
			
			
			if (jTabbedPane1.getSelectedIndex() == jTabbedPane1.indexOfTab("Splash")){
				buttonSetEnable1();
				//JOptionPane.showMessageDialog(null, "Please select the next step from the TABS above to add to your protocol", "NOTEBOOK", 1);
			}else{
				buttonSetDisable1();
				//addButton.setEnabled(true);
			}
				
		}  
      });
         
      this.jTabbedPane1.setBackground(this.panelColor);
      this.jTabbedPane1.setOpaque(true);
      //this.jTabbedPane1.setTabPlacement(JTabbedPane.BOTTOM);
    }

    return this.jTabbedPane1;
  }

  public JComboBox getSrc2()
  {
    if (this.source2 == null) {
      this.source2 = new JComboBox(); 
      for (int i = 0; i < this.rcount; i++) {
          this.source2.addItem(this.reservoirs[i]);
      }
      this.source2.setSelectedIndex(0);
      this.so2 = ((String)this.source2.getSelectedItem());
      this.source2.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          JComboBox cb = (JComboBox)e.getSource();
          TabWindow.this.so2 = ((String)cb.getSelectedItem());
        } } );
    }
    return this.source2;
  }

  
  public JComboBox getSrc3()
  {
    if (this.source3 != null) {
      this.source3.removeAllItems();
    }
    if (this.source3 == null) {
      this.source3 = new JComboBox();
    }
    if (this.platebutton3.isSelected())
    {
      for (int i = 0; i < this.p96count; i++) {
        this.source3.addItem(this.plates96[i]);
      }
    }
    else if (this.tubebutton3.isSelected())
    {
      for (int i = 0; i < this.tcount; i++) {
        this.source3.addItem(this.tubes[i]);
      }
    }
    this.source3.setSelectedIndex(0);
    this.so3 = ((String)this.source3.getSelectedItem());
    this.source3.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        JComboBox cb = (JComboBox)e.getSource();
        TabWindow.this.so3 = ((String)cb.getSelectedItem());
      }
    });
    return this.source3;
  }

  public JComboBox getSrc4()
  {
    if (this.source4 != null) {
      this.source4.removeAllItems();
      this.btubel1.setVisible(false);
    }

    if (this.source4 == null) {
      this.source4 = new JComboBox();
      this.btubel1.setVisible(false);
    }

    if (this.brresbutton1.isSelected())
    {
      for (int i = 0; i < this.rcount; i++) {
        this.source4.addItem(this.reservoirs[i]);
      }
    }
    else if (this.brtubebutton1.isSelected())
    {
      for (int i = 0; i < this.tcount; i++)
        this.source4.addItem(this.tubes[i]);
      this.btubel1.setVisible(true);
    }
    if ((this.rcount > 0) || (this.tcount > 0))
    {
      this.source4.setSelectedIndex(0);
      this.so4 = ((String)this.source4.getSelectedItem());
    }
    this.source4.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        JComboBox cb = (JComboBox)e.getSource();
        TabWindow.this.so4 = ((String)cb.getSelectedItem());
      }
    });
    return this.source4;
  }

  public JComboBox getSrc5()
  {
    if (this.source5 == null) {
      this.source5 = new JComboBox();
      for (int i = 0; i < this.p96count; i++)
      {
        this.source5.addItem(this.plates96[i]);
      }
      for (int i = 0; i < this.p384count; i++)
      {
        this.source5.addItem(this.plates384[i]);
      }
      this.source5.setSelectedIndex(0);
      this.so5 = ((String)this.source5.getSelectedItem());
      this.source5.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          JComboBox cb = (JComboBox)e.getSource();
          TabWindow.this.so5 = ((String)cb.getSelectedItem());
        } } );
    }
    return this.source5;
  }

  public JComboBox getCherrySrc()
  {
    if (this.chsource == null) {
      this.chsource = new JComboBox();
      for (int i = 0; i < this.pALLcount; i++)	//AP-0001
      {
        this.chsource.addItem(this.platesAll[i]);
      }
//      for (int i = 0; i < this.p96count; i++)		//AP-0001 - Commented out here.  Ready for deletion
//      {
//        this.chsource.addItem(this.plates96[i]);
//      }
//      for (int i = 0; i < this.p384count; i++)
//      {
//        this.chsource.addItem(this.plates384[i]);
//      }
      this.chsource.setSelectedIndex(0);
      this.chso = ((String)this.chsource.getSelectedItem());
      this.chsource.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          JComboBox cb = (JComboBox)e.getSource();
          TabWindow.this.chso = ((String)cb.getSelectedItem());
        } } );
    }
    return this.chsource;
  }

  public JComboBox getCherryLtyp()
  {
    if (this.chliq == null) {
      this.chliq = new JComboBox(this.liqtypes);
      this.chliq.setSelectedIndex(0);
      this.chltyp = ((String)this.chliq.getSelectedItem());
      this.chliq.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          JComboBox cb = (JComboBox)e.getSource();
          TabWindow.this.chltyp = ((String)cb.getSelectedItem());
        } } );
    }
    return this.chliq;
  }

  public JComboBox getCherryDest()
  {
    if (this.chdest == null) {
      this.chdest = new JComboBox();
      for (int i = 0; i < this.pALLcount; i++)	//AP-0001
      {
        this.chdest.addItem(this.platesAll[i]);
      }
//      for (int i = 0; i < this.p96count; i++)		//AP-0001 - Commented out here.  Ready for deletion
//      {
//        this.chdest.addItem(this.plates96[i]);
//      }
//      for (int i = 0; i < this.p384count; i++)
//      {
//        this.chdest.addItem(this.plates384[i]);
//      }
      this.chdest.setSelectedIndex(0);
      this.chde = ((String)this.chdest.getSelectedItem());
      this.chdest.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          JComboBox cb = (JComboBox)e.getSource();
          TabWindow.this.chde = ((String)cb.getSelectedItem());
        } } );
    }
    return this.chdest;
  }
  public JComboBox getPKSrc()
  {
    if (this.pksource == null) {
      this.pksource = new JComboBox();
      this.pksource.setEnabled(false);
      for (int i = 0; i < this.p96count; i++)
      {
        this.pksource.addItem(this.plates96[i]);
      }
      for (int i = 0; i < this.tcount; i++)
      {
        this.pksource.addItem(this.tubes[i]);
      }
      this.pksource.setSelectedIndex(0);
      this.pkso = ((String)this.pksource.getSelectedItem());
      this.pksource.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          JComboBox cb = (JComboBox)e.getSource();
          TabWindow.this.pkso = ((String)cb.getSelectedItem());
        } } );
    }
    return this.pksource;
  }

  public JComboBox getPKMRDSrc()
  {
    if (this.pkMRDsource == null) {
      this.pkMRDsource = new JComboBox();
      this.pkMRDsource.setEnabled(false);
      for (int i = 0; i < this.p96count; i++)
      {
        this.pkMRDsource.addItem(this.plates96[i]);
      }
      this.pkMRDsource.setSelectedIndex(0);
      this.pkmrdlab = ((String)this.pkMRDsource.getSelectedItem());
      this.pkMRDsource.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          JComboBox cb = (JComboBox)e.getSource();
          TabWindow.this.pkmrdlab = ((String)cb.getSelectedItem());
        } } );
    }
    return this.pkMRDsource;
  }

  public JComboBox getPKSTDSrc()
  {
    if (this.pkSTDsource == null) {
      this.pkSTDsource = new JComboBox();
      this.pkSTDsource.setEnabled(false);
      for (int i = 0; i < this.p96count; i++)
      {
        this.pkSTDsource.addItem(this.plates96[i]);
      }
      for (int i = 0; i < this.tcount; i++)
      {
        this.pkSTDsource.addItem(this.tubes[i]);
      }
      this.pkSTDsource.setSelectedIndex(0);
      this.pkstdlab = ((String)this.pkSTDsource.getSelectedItem());
      this.pkSTDsource.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          JComboBox cb = (JComboBox)e.getSource();
          TabWindow.this.pkstdlab = ((String)cb.getSelectedItem());
          if (pkstdlab.contains("_96")){
        	  pkSsource.setSelectedIndex(pkSTDsource.getSelectedIndex());
          }

        } } );
    }
    return this.pkSTDsource;
  }

  public JComboBox getPKDILSrc()
  {
    if (this.pkDilsource == null) {
      this.pkDilsource = new JComboBox();
      this.pkDilsource.setEnabled(false);
      for (int i = 0; i < this.rcount; i++)
      {
        this.pkDilsource.addItem(this.reservoirs[i]);
      }
      this.pkDilsource.setSelectedIndex(0);
      this.pkdillab2 = ((String)this.pkDilsource.getSelectedItem());
      this.pkDilsource.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          JComboBox cb = (JComboBox)e.getSource();
          TabWindow.this.pkdillab2 = ((String)cb.getSelectedItem());
        } } );
    }
    return this.pkDilsource;
  }

  public JComboBox getPKDILRes()
  {
    if (this.pkDilreserve == null) {
      this.pkDilreserve = new JComboBox();
      this.pkDilreserve.setEnabled(false);
      for (int i = 0; i < this.rcount; i++)
      {
        this.pkDilreserve.addItem(this.reservoirs[i]);
      }
      this.pkDilreserve.setSelectedIndex(0);
      this.pkdillab3 = ((String)this.pkDilreserve.getSelectedItem());
      this.pkDilreserve.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          JComboBox cb = (JComboBox)e.getSource();
          TabWindow.this.pkdillab3 = ((String)cb.getSelectedItem());
        } } );
    }
    return this.pkDilreserve;
  }


  public JComboBox getSSrc()
  {
    if (this.pkSsource == null) {
      this.pkSsource = new JComboBox();
      this.pkSsource.setEnabled(false);
      for (int i = 0; i < this.p96count; i++)
      {
        this.pkSsource.addItem(this.plates96[i]);
      }
      for (int i = 0; i < this.tcount; i++)
      {
        this.pkSsource.addItem(this.tubes[i]);
      }
      this.pkSsource.setSelectedIndex(0);
      this.pksamlab = ((String)this.pkSsource.getSelectedItem());
      this.pkSsource.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          JComboBox cb = (JComboBox)e.getSource();
          TabWindow.this.pksamlab = ((String)cb.getSelectedItem());
          if (pksamlab.contains("_96") && pkstdlab.contains("_96")){
        	  pkSTDsource.setSelectedIndex(pkSsource.getSelectedIndex());
          }
          if (pksamlab.contains("_Tube") && pkstdlab.contains("_96")){
        	  pkSTDsource.setSelectedIndex(pkSsource.getSelectedIndex());
          }
        } } );
    }
    return this.pkSsource;
  }

  public JComboBox getDSrc()
  {
    if (this.pkDsource == null) {
      this.pkDsource = new JComboBox();
      this.pkDsource.setEnabled(false);
      for (int i = 0; i < this.p96count; i++)
      {
        this.pkDsource.addItem(this.plates96[i]);
      }
      this.pkDsource.setSelectedIndex(0);
      this.pkdillab = ((String)this.pkDsource.getSelectedItem());
      this.pkDsource.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          JComboBox cb = (JComboBox)e.getSource();
          TabWindow.this.pkdillab = ((String)cb.getSelectedItem());
        } } );
    }
    return this.pkDsource;
  }

  public JComboBox getPKAssay()
  {
    if (this.pkassay == null) {
      this.pkassay = new JComboBox();
      this.pkassay.setEnabled(false);
      for (int i = 0; i < this.p96count; i++)
      {
        this.pkassay.addItem(this.plates96[i]);
      }
      this.pkassay.setSelectedIndex(0);
      this.pkasslab = ((String)this.pkassay.getSelectedItem());
      this.pkassay.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          JComboBox cb = (JComboBox)e.getSource();
          TabWindow.this.pkasslab = ((String)cb.getSelectedItem());
        } } );
    }
    return this.pkassay;
  }

  public JComboBox getPKPAssay()
  {
    if (this.pkPassay == null) {
      this.pkPassay = new JComboBox();
      this.pkPassay.setEnabled(false);
      for (int i = 0; i < this.p96count; i++)
      {
        this.pkPassay.addItem(this.plates96[i]);
      }
      this.pkPassay.setSelectedIndex(0);
      this.pkpasslab = ((String)this.pkPassay.getSelectedItem());
      this.pkPassay.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          JComboBox cb = (JComboBox)e.getSource();
          TabWindow.this.pkpasslab = ((String)cb.getSelectedItem());
        } } );
    }
    return this.pkPassay;
  }

  

  public JComboBox getPKDil()
  {
    if (this.pkdil == null) {
      this.pkdil = new JComboBox(this.liqtypes);
      this.pkdil.setSelectedIndex(0);
      this.pkdil.setEnabled(false);
      this.pkdildil = ((String)this.pkdil.getSelectedItem());
      this.pkdil.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          JComboBox cb = (JComboBox)e.getSource();
          TabWindow.this.pkdildil = ((String)cb.getSelectedItem());
        } } );
    }
    return this.pkdil;
  }

  public JComboBox getPKSAMDil()
  {
    if (this.pksamdil == null) {
      this.pksamdil = new JComboBox(this.liqtypes);
      this.pksamdil.setSelectedIndex(0);
      this.pksamdil.setEnabled(false);
      this.pksam = ((String)this.pksamdil.getSelectedItem());
      this.pksamdil.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          JComboBox cb = (JComboBox)e.getSource();
          TabWindow.this.pksam = ((String)cb.getSelectedItem());
        } } );
    }
    return this.pksamdil;
  }

  public JComboBox getPKSTDDil()
  {
    if (this.pkstddil == null) {
      this.pkstddil = new JComboBox(this.liqtypes);
      this.pkstddil.setSelectedIndex(0);
      this.pkstddil.setEnabled(false);
      this.pkstd = ((String)this.pkstddil.getSelectedItem());
      this.pkstddil.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          JComboBox cb = (JComboBox)e.getSource();
          TabWindow.this.pkstd = ((String)cb.getSelectedItem());
        } } );
    }
    return this.pkstddil;
  }


  
  public JComboBox getPKVol()
  {
    if (this.pkdil == null) {
      this.pkdil = new JComboBox(this.liqtypes);
      this.pkdil.setSelectedIndex(0);
      this.pkdil.setEnabled(false);
      this.pkdildil = ((String)this.pkdil.getSelectedItem());
      this.pkdil.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          JComboBox cb = (JComboBox)e.getSource();
          TabWindow.this.pkdildil = ((String)cb.getSelectedItem());
        } } );
    }
    return this.pkdil;
  }

  public JComboBox getPKDeck1()
  {
    if (this.pkdeck1 == null) {
      this.pkdeck1 = new JComboBox(this.platetypes);
      this.pkdeck1.setSelectedIndex(0);
      this.pkdeck1.setEnabled(false);
//      this.pkdeck1.setVisible(false);
      this.pkmrddeck = ((String)this.pkdeck1.getSelectedItem());
      this.pkdeck1.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          JComboBox cb = (JComboBox)e.getSource();
          TabWindow.this.pkmrddeck = ((String)cb.getSelectedItem());
        } } );
    }
    return this.pkdeck1;
  }

  public JComboBox getPKDeck2()
  {
    if (this.pkdeck2 == null) {
      this.pkdeck2 = new JComboBox(this.platetypes);
      this.pkdeck2.setSelectedIndex(0);
      this.pkdeck2.setEnabled(false);
//      this.pkdeck2.setVisible(false);
      this.pkdildeck = ((String)this.pkdeck2.getSelectedItem());
      this.pkdeck2.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          JComboBox cb = (JComboBox)e.getSource();
          TabWindow.this.pkdildeck = ((String)cb.getSelectedItem());
        } } );
    }
    return this.pkdeck2;
  }
  
  public JComboBox getMRDDil()
  {
    if (this.pkMRDdil == null) {
      this.pkMRDdil = new JComboBox(this.liqtypes);
      this.pkMRDdil.setSelectedIndex(0);
      this.pkMRDdil.setEnabled(false);
      this.pkmrddil = ((String)this.pkMRDdil.getSelectedItem());
      this.pkMRDdil.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          JComboBox cb = (JComboBox)e.getSource();
          TabWindow.this.pkmrddil = ((String)cb.getSelectedItem());
        } } );
    }
    return this.pkMRDdil;
  }

  public JComboBox getPKLtyp()
  {
    if (this.pkliq == null) {
      this.pkliq = new JComboBox(this.liqtypes);
      this.pkliq.setSelectedIndex(0);
      this.pkltyp = ((String)this.pkliq.getSelectedItem());
      this.pkliq.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          JComboBox cb = (JComboBox)e.getSource();
          TabWindow.this.pkltyp = ((String)cb.getSelectedItem());
        } } );
    }
    return this.pkliq;
  }


  public JComboBox getSrc()
  {
    if (this.source != null) {
      this.source.removeAllItems();
    }

    if (this.source == null) {
      this.source = new JComboBox();

    }
    
    if (this.platebutton.isSelected())
    	
    {
        for (int i = 0; i < this.pALLcount; i++)	
        {
          this.source.addItem(this.platesAll[i]);
        }
//      for (int i = 0; i < this.p96count; i++)
//      {
//        this.source.addItem(this.plates96[i]);
//     }
//      this.button48l2.setVisible(true);
//      this.button96l2.setVisible(true);
//      this.button384l2.setVisible(true);
      this.lchill.setVisible(true);
    }
    else if (this.tubebutton.isSelected())
    {
        for (int i = 0; i < this.tallcount; i++)			//APN0045 replaced tcount
            this.source.addItem(this.tubesAll[i]);			//APN0045 replaced tubes[i]
      this.lchill.setVisible(false);

    }
    else if (this.resbutton.isSelected())
    {
    	for (int i = 0; i < this.rallcount; i++) {					//APN0047 replaced rcount
	       this.source.addItem(this.reservoirsAll[i]);				//APN0047 replaced reservoirs[i]
	       this.lchill.setVisible(true);   	  
    	}
    }

    this.source.setSelectedIndex(0);
    this.s1 = ((String)this.source.getSelectedItem());
    
//  APN0041
    class MyComboBoxRenderer extends DefaultListCellRenderer  {
        /**
         * 
         */
        private static final long serialVersionUID = 2746090194775905713L;

        public Component getListCellRendererComponent(JList list, Object value,  
                int index, boolean isSelected, boolean cellHasFocus) {

        	if (isSelected) {
        		
        		String graphicFile = "C:/Program Files/TELOS/AutoPilot/Graphics/" + String.valueOf(value) + ".png";

        		if (platebutton.isSelected()){
	        		if (new File(graphicFile).exists()) {
	        			PanelBlank1.label1.setText("");
	        			PanelBlank1.label1.setIcon(new ImageIcon(graphicFile));
	        		}else{
	        			if (index == 0){
	        				PanelBlank1.label1.setText("");
	        				PanelBlank1.label1.setIcon(new ImageIcon("C:/Program Files/TELOS/AutoPilot/Graphics/introS.png"));
	        			}else{
	        				PanelBlank1.label1.setIcon(new ImageIcon(""));
	        				PanelBlank1.label1.setText("NO IMAGE OF " + String.valueOf(value) + " AVAILABLE");	
	        			}
	        		}  
	        		
	                TabWindow.this.liqPanel1.remove(componentSelectedS);
	                TabWindow.this.splatetype.setVisible(true);
	                TabWindow.this.liqPanel1.add(TabWindow.this.panelbl, "Center");
	                TabWindow.this.liqPanel1.revalidate();
	                TabWindow.this.liqPanel1.repaint();    
        		}

        		if (resbutton.isSelected()){
	        		if (new File(graphicFile).exists()) {															//APN0047
	        			ReservoirPanel1.label1.setText("");															//APN0047
	                    ImageIcon imageIcon = new ImageIcon(graphicFile); // load the image to a imageIcon
	        			Image image = imageIcon.getImage(); // transform it 
	        			Image newimg = image.getScaledInstance(360, 220,  java.awt.Image.SCALE_SMOOTH); // scale it the smooth way  
	        			imageIcon = new ImageIcon(newimg);  // transform it back
	        			
	        			ReservoirPanel1.label1.setIcon(imageIcon);													//APN0047
	        		}else{																							//APN0047
	        			ReservoirPanel1.label1.setIcon(new ImageIcon(""));											//APN0047
	        			if (index == 0){																			//APN0047
	        				ReservoirPanel1.label1.setText("SELECT RESERVOIR TYPE ABOVE");							//APN0047
	        			}else{																						//APN0047
	        				ReservoirPanel1.label1.setText("NO IMAGE OF " + String.valueOf(value) + " AVAILABLE");	//APN0047
	        			}																							//APN0047
	        		}
        		}
        		
        		if (tubebutton.isSelected()){
	        		if (new File(graphicFile).exists()) {
	        			tubepanel.label1.setText("");
	        			tubepanel.label1.setIcon(new ImageIcon(graphicFile));
	        		}else{
	        			tubepanel.label1.setIcon(new ImageIcon(""));
	        			if (index == 0){
	        				tubepanel.label1.setText("SELECT TUBE TYPE ABOVE");
	        			}else{
	        				tubepanel.label1.setText("NO IMAGE OF " + String.valueOf(value) + " AVAILABLE");
	        			}
	        		}  
        		}

              setOpaque(true);
              setBackground(list.getSelectionBackground());
              setForeground(list.getSelectionForeground());
              
        	}else{
                setOpaque(true);
              setBackground(list.getBackground());
              setForeground(list.getForeground());
        	}
        		
        	
          setFont(list.getFont());
          setText((value == null) ? "" : value.toString());
          return this;
        }
      }

    source.setRenderer(new MyComboBoxRenderer());
    
	BoundsPopupMenuListener listener = new BoundsPopupMenuListener(true, true);
	source.addPopupMenuListener( listener );
	source.setPrototypeDisplayValue("ItemWWW");
    
//    END APN0041
	
    this.source.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        JComboBox cb = (JComboBox)e.getSource();
        TabWindow.this.s1 = ((String)cb.getSelectedItem());
        if (TabWindow.this.s1 != null) {
          int index1 = TabWindow.this.s1.lastIndexOf('_');
          int index2 = TabWindow.this.s1.lastIndexOf('X');
          if (TabWindow.this.resbutton.isSelected())
          {
          	if (cb.getSelectedIndex() != 0){							//APN0047  Added If condition.	
	            String nres = TabWindow.this.s1.substring(index1 + 1, index2);
	
	            TabWindow.this.rnum = Integer.parseInt(nres);
	
				try {//APF0043
					//TabWindow.this.rHam = TabWindow.this.s1.contains("Ham");  //removed as part of APF0043
					int rackHeight = Integer.parseInt(VASengine.SystemLabwareInfo(VASengine.BitPath+"HAMILTON\\LabWare\\APcustom\\" + s1 + ".rck", "Dim.Dx......"));
					TabWindow.this.rHam = (rackHeight<126);
				} catch (NumberFormatException e1) {
					// TODO Auto-generated catch block
			    	JOptionPane.showMessageDialog(null, "VAS WARNING::Failed to analyze Labware File: " + VASengine.BitPath+"HAMILTON\\LabWare\\APcustom\\" + s1 + ".rck" + "\n\n" + e1);
				}
	            
				TabWindow.this.rs1 = "1"; 						//APN0047
				//APN0047 commented out below
//	            TabWindow.this.liqPanel1.removeAll();
//	            TabWindow.this.source.revalidate();
//	            TabWindow.this.splatetype.setVisible(true);
//	            TabWindow.this.liqPanel1.add(TabWindow.this.getLiqPanel1Top(), "North");
//	            TabWindow.this.liqPanel1.add(TabWindow.this.getres12Xpanel(TabWindow.this.rnum), "Center");
//	            TabWindow.this.liqPanel1.add(TabWindow.this.getLiqPanel1Bot(), "South");
//	            TabWindow.this.liqPanel1.revalidate();
//	            TabWindow.this.liqPanel1.repaint();
	          //APN0047 END commented out below
            }
          }
          if (TabWindow.this.s1.endsWith("_24"))
          {
        	  componentSelectedS = panel24l;															//END APN0041
              TabWindow.this.liqPanel1.removeAll();
              TabWindow.this.splatetype.setVisible(true);
              TabWindow.this.liqPanel1.add(TabWindow.this.getLiqPanel1Top(), "North");
              TabWindow.this.liqPanel1.add(TabWindow.this.panel24l, "Center");
              TabWindow.this.liqPanel1.add(TabWindow.this.getLiqPanel1Bot(), "South");
              TabWindow.this.liqPanel1top.revalidate();
              TabWindow.this.liqPanel1top.repaint();
              TabWindow.this.liqPanel1.revalidate();
              TabWindow.this.liqPanel1.repaint();
          }
          if (TabWindow.this.s1.endsWith("_48"))
          {
        	  componentSelectedS = panel48l;															//END APN0041
              TabWindow.this.liqPanel1.removeAll();
              TabWindow.this.splatetype.setVisible(true);
              TabWindow.this.liqPanel1.add(TabWindow.this.getLiqPanel1Top(), "North");
              TabWindow.this.liqPanel1.add(TabWindow.this.panel48l, "Center");
              TabWindow.this.liqPanel1.add(TabWindow.this.getLiqPanel1Bot(), "South");
              TabWindow.this.liqPanel1top.revalidate();
              TabWindow.this.liqPanel1top.repaint();
              TabWindow.this.liqPanel1.revalidate();
              TabWindow.this.liqPanel1.repaint();
          }
          if (TabWindow.this.s1.endsWith("_96"))
          {
        	  componentSelectedS = panel96l;															//END APN0041
              TabWindow.this.liqPanel1.removeAll();
              TabWindow.this.splatetype.setVisible(true);
              TabWindow.this.liqPanel1.add(TabWindow.this.getLiqPanel1Top(), "North");
              TabWindow.this.liqPanel1.add(TabWindow.this.panel96l, "Center");
              TabWindow.this.liqPanel1.add(TabWindow.this.getLiqPanel1Bot(), "South");
              TabWindow.this.liqPanel1top.revalidate();
              TabWindow.this.liqPanel1top.repaint();
              TabWindow.this.liqPanel1.revalidate();
              TabWindow.this.liqPanel1.repaint();
          }
          if (TabWindow.this.s1.endsWith("_384"))
          {
        	  componentSelectedS = panel384l;															//END APN0041
              TabWindow.this.liqPanel1.removeAll();
              TabWindow.this.splatetype.setVisible(true);
              TabWindow.this.liqPanel1.add(TabWindow.this.getLiqPanel1Top(), "North");
              TabWindow.this.liqPanel1.add(TabWindow.this.panel384l, "Center");
              TabWindow.this.liqPanel1.add(TabWindow.this.getLiqPanel1Bot(), "South");
              TabWindow.this.liqPanel1top.revalidate();
              TabWindow.this.liqPanel1top.repaint();
              TabWindow.this.liqPanel1.revalidate();
              TabWindow.this.liqPanel1.repaint();
          }
        }
      }
    });
    return this.source;
  }

  public JComboBox getDest()
  {
    if (this.dest != null) {
      this.dest.removeAllItems();
    }
    if (this.dest == null) {
      this.dest = new JComboBox();
    }
    if (this.platebutton2.isSelected())
    {
      for (int i = 0; i < this.pALLcount; i++)	
      {
        this.dest.addItem(this.platesAll[i]);
      }
//      for (int i = 0; i < this.p96count; i++)
//        this.dest.addItem(this.plates96[i]);
      this.lchill1.setVisible(true);
      this.lshake.setVisible(true);
//      this.button48l.setVisible(true);
//      this.button96l.setVisible(true);
//      this.button384l.setVisible(true);
    }
    else if (this.resbutton2.isSelected())
    {
      for (int i = 0; i < this.rallcount; i++) {					//APN0047 replaced rcount
        this.dest.addItem(this.reservoirsAll[i]);					//APN0047 replaced reservoirs[i]
      }
    }
    else if (this.tubebutton2.isSelected())
    {
        for (int i = 0; i < this.tallcount; i++)					//APN0045 replaced tcount
            this.dest.addItem(this.tubesAll[i]);					//APN0045 replaced tubes[i]
      this.lchill1.setVisible(false);
      this.lshake.setVisible(false);
    }
    this.dest.setSelectedIndex(0);
    this.de = ((String)this.dest.getSelectedItem());

//  APN0041
    class MyComboBoxRenderer extends DefaultListCellRenderer  {
        /**
         * 
         */
        private static final long serialVersionUID = 2746090194775905713L;

        public Component getListCellRendererComponent(JList list, Object value,  
                int index, boolean isSelected, boolean cellHasFocus) {

        	if (isSelected) {
        		
        		String graphicFile = "C:/Program Files/TELOS/AutoPilot/Graphics/" + String.valueOf(value) + ".png";

        		if (platebutton2.isSelected()){
        			if (new File(graphicFile).exists()) {
            			PanelBlank2.label1.setIcon(new ImageIcon(graphicFile));
            		}else{
 	        			if (index == 0){																						//APN0045
	        				PanelBlank2.label1.setText("");																		//APN0045
	        				PanelBlank2.label1.setIcon(new ImageIcon("C:/Program Files/TELOS/AutoPilot/Graphics/introD.png"));	//APN0045
	        			}else{																									//APN0045
	        				PanelBlank2.label1.setIcon(new ImageIcon(""));														//APN0045
	        				PanelBlank2.label1.setText("NO IMAGE OF " + String.valueOf(value) + " AVAILABLE");					//APN0045	
	        			}																										//APN0045       		
            		}
                    TabWindow.this.liqPanel2.remove(componentSelectedD);
                    TabWindow.this.dplatetype.setVisible(true);
                    TabWindow.this.liqPanel2.add(TabWindow.this.panelbr, "Center");
                    TabWindow.this.liqPanel2.revalidate();
                    TabWindow.this.liqPanel2.repaint();
        		}
        		
        		if (resbutton2.isSelected()){
	        		if (new File(graphicFile).exists()) {															//APN0047
	        			ReservoirPanel2.label1.setText("");															//APN0047
	        			
	                    ImageIcon imageIcon = new ImageIcon(graphicFile); // load the image to a imageIcon
	        			Image image = imageIcon.getImage(); // transform it 
	        			Image newimg = image.getScaledInstance(360, 220,  java.awt.Image.SCALE_SMOOTH); // scale it the smooth way  
	        			imageIcon = new ImageIcon(newimg);  // transform it back
	        			
	        			ReservoirPanel2.label1.setIcon(imageIcon);													//APN0047
	        		}else{																							//APN0047
	        			ReservoirPanel2.label1.setIcon(new ImageIcon(""));											//APN0047
	        			if (index == 0){																			//APN0047
	        				ReservoirPanel2.label1.setText("SELECT RESERVOIR TYPE ABOVE");							//APN0047
	        			}else{																						//APN0047
	        				ReservoirPanel2.label1.setText("NO IMAGE OF " + String.valueOf(value) + " AVAILABLE");	//APN0047
	        			}																							//APN0047
	        		}
        		}
        			
        		if (tubebutton2.isSelected()){																	//APN0045
	        		if (new File(graphicFile).exists()) {														//APN0045
	        			tubepanel2.label1.setText("");															//APN0045
	        			tubepanel2.label1.setIcon(new ImageIcon(graphicFile));									//APN0045
	        		}else{																						//APN0045
	        			tubepanel2.label1.setIcon(new ImageIcon(""));											//APN0045
	        			if (index == 0){																		//APN0045
	        				tubepanel2.label1.setText("SELECT TUBE TYPE ABOVE");								//APN0045
	        			}else{																					//APN0045
	        				tubepanel2.label1.setText("NO IMAGE OF " + String.valueOf(value) + " AVAILABLE");	//APN0045
	        			}																						//APN0045
	        		}																							//APN0045
        		}																								//APN0045
        		
              setOpaque(true);
              setBackground(list.getSelectionBackground());
              setForeground(list.getSelectionForeground());
              
        	}else{
                setOpaque(true);
              setBackground(list.getBackground());
              setForeground(list.getForeground());
        	}
        		
        	
          setFont(list.getFont());
          setText((value == null) ? "" : value.toString());
          return this;
        }
      }

    dest.setRenderer(new MyComboBoxRenderer());
    
	BoundsPopupMenuListener listener = new BoundsPopupMenuListener(true, true);
	dest.addPopupMenuListener( listener );
	dest.setPrototypeDisplayValue("ItemWWW");
    
//    END APN0041
    
    
    
    this.dest.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        JComboBox cb = (JComboBox)e.getSource();
        TabWindow.this.de = ((String)cb.getSelectedItem());
        
        if (TabWindow.this.de != null) {
          int index1 = TabWindow.this.de.lastIndexOf('_');
          int index2 = TabWindow.this.de.lastIndexOf('X');
          if (TabWindow.this.resbutton2.isSelected())
          {
        	if (cb.getSelectedIndex() != 0){											//APN0047  Added If condition.
	          String nres2 = TabWindow.this.de.substring(index1 + 1, index2);
	          TabWindow.this.rnum2 = Integer.parseInt(nres2);        		  
        	  
				try {//APF0043
					//TabWindow.this.rHam2 = TabWindow.this.de.contains("Ham");  //removed as part of APF0043
					int rackHeight = Integer.parseInt(VASengine.SystemLabwareInfo(VASengine.BitPath+"HAMILTON\\LabWare\\APcustom\\" + de + ".rck", "Dim.Dx......"));
					TabWindow.this.rHam2 = (rackHeight<126);  
				} catch (NumberFormatException e1) {
					// TODO Auto-generated catch block
			    	JOptionPane.showMessageDialog(null, "VAS WARNING::Failed to analyze Labware File: " + VASengine.BitPath+"HAMILTON\\LabWare\\APcustom\\" + de + ".rck" + "\n\n" + e1);
				}
				
				TabWindow.this.rs2 = "1"; 						//APN0047
				//APN0047 commented out below
//				TabWindow.this.liqPanel2.removeAll();
//	            TabWindow.this.dest.revalidate();
//	            TabWindow.this.dplatetype.setVisible(true);
//	            TabWindow.this.liqPanel2.add(TabWindow.this.getLiqPanel2Top(), "North");
//	            TabWindow.this.liqPanel2.add(TabWindow.this.getres12Xpanel2(TabWindow.this.rnum2), "Center");
//	            TabWindow.this.liqPanel2.add(TabWindow.this.getLiqPanel2Bot(), "South");
//	            TabWindow.this.liqPanel2.revalidate();
//	            TabWindow.this.liqPanel2.repaint();
	            //APN0047 END commented out
				
	        	  }
          }
          if (TabWindow.this.de.endsWith("_24"))
          {
        	  componentSelectedD = panel24r;																	//  APN0041
              TabWindow.this.liqPanel2.removeAll();
              TabWindow.this.dplatetype.setVisible(true);
              TabWindow.this.liqPanel2.add(TabWindow.this.getLiqPanel2Top(), "North");
              TabWindow.this.liqPanel2.add(TabWindow.this.panel24r, "Center");
              TabWindow.this.liqPanel2.add(TabWindow.this.getLiqPanel2Bot(), "South");
              TabWindow.this.liqPanel2top.revalidate();
              TabWindow.this.liqPanel2top.repaint();
              TabWindow.this.liqPanel2.revalidate();
              TabWindow.this.liqPanel2.repaint();
          }
          if (TabWindow.this.de.endsWith("_48"))
          {
        	  componentSelectedD = panel48r;																	//  APN0041
              TabWindow.this.liqPanel2.removeAll();
              TabWindow.this.dplatetype.setVisible(true);
              TabWindow.this.liqPanel2.add(TabWindow.this.getLiqPanel2Top(), "North");
              TabWindow.this.liqPanel2.add(TabWindow.this.panel48r, "Center");
              TabWindow.this.liqPanel2.add(TabWindow.this.getLiqPanel2Bot(), "South");
              TabWindow.this.liqPanel2top.revalidate();
              TabWindow.this.liqPanel2top.repaint();
              TabWindow.this.liqPanel2.revalidate();
              TabWindow.this.liqPanel2.repaint();
          }
          if (TabWindow.this.de.endsWith("_96"))
          {
        	  componentSelectedD = panel96r;																	//  APN0041
              TabWindow.this.liqPanel2.removeAll();
              TabWindow.this.dplatetype.setVisible(true);
              TabWindow.this.liqPanel2.add(TabWindow.this.getLiqPanel2Top(), "North");
              TabWindow.this.liqPanel2.add(TabWindow.this.panel96r, "Center");
              TabWindow.this.liqPanel2.add(TabWindow.this.getLiqPanel2Bot(), "South");
              TabWindow.this.liqPanel2top.revalidate();
              TabWindow.this.liqPanel2top.repaint();
              TabWindow.this.liqPanel2.revalidate();
              TabWindow.this.liqPanel2.repaint();
          }
          if (TabWindow.this.de.endsWith("_384"))
          {
        	  componentSelectedD = panel384r;																	//  APN0041
              TabWindow.this.liqPanel2.removeAll();
              TabWindow.this.dplatetype.setVisible(true);
              TabWindow.this.liqPanel2.add(TabWindow.this.getLiqPanel2Top(), "North");
              TabWindow.this.liqPanel2.add(TabWindow.this.panel384r, "Center");
              TabWindow.this.liqPanel2.add(TabWindow.this.getLiqPanel2Bot(), "South");
              TabWindow.this.liqPanel2top.revalidate();
              TabWindow.this.liqPanel2top.repaint();
              TabWindow.this.liqPanel2.revalidate();
              TabWindow.this.liqPanel2.repaint();
          }
        }
      }
    });
    return this.dest;
  }

  public JComboBox getDest2()
  {
    if (this.dest2 != null) {
      this.dest2.removeAllItems();
    }
    if (this.dest2 == null) {
      this.dest2 = new JComboBox();
    }

    if (this.button96s.isSelected())
    {
      for (int i = 0; i < this.p96count; i++) {
        this.dest2.addItem(this.plates96[i]);
      }

    }
   else if (this.button384s.isSelected())
    {
      for (int i = 0; i < this.p384count; i++) {
        this.dest2.addItem(this.plates384[i]);
      }
    }
    this.dest2.setSelectedIndex(0);
    this.de2 = ((String)this.dest.getSelectedItem());

    this.dest2.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        JComboBox cb = (JComboBox)e.getSource();
        TabWindow.this.de2 = ((String)cb.getSelectedItem());
      }
    });
    return this.dest2;
  }

  public JComboBox getDest3()
  {
    if (this.dest3 != null) {
      this.dest3.removeAllItems();
    }
    if (this.dest3 == null) {
      this.dest3 = new JComboBox();
    }
    if (this.brresbutton2.isSelected())
    {
      for (int i = 0; i < this.rcount; i++) {
        this.dest3.addItem(this.reservoirs[i]);
      }
    }
    else if (this.brtubebutton2.isSelected())
    {
      for (int i = 0; i < this.tcount; i++)
        this.dest3.addItem(this.tubes[i]);
    }
    if ((this.rcount > 0) || (this.tcount > 0))
    {
      this.dest3.setSelectedIndex(0);
      this.de3 = ((String)this.dest3.getSelectedItem());
    }
    this.dest3.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        JComboBox cb = (JComboBox)e.getSource();
        TabWindow.this.de3 = ((String)cb.getSelectedItem());
      }
    });
    return this.dest3;
  }

  public JComboBox getFilter()
  {
    if (this.filter == null) {
      this.filter = new JComboBox();
      this.filter.addItem(" Thom_Filter_96     ");
      this.filter.addItem(" Mill_Filter_96     ");
      this.filter.addItem(" GE_Filter_96     ");
    }
    this.filter.setSelectedIndex(0);
    this.fil = ((String)this.filter.getSelectedItem());
    this.filter.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        JComboBox cb = (JComboBox)e.getSource();
        TabWindow.this.fil = ((String)cb.getSelectedItem());
      }
    });
    return this.filter;
  }

  public JComboBox getTrack1()
  {
    if (this.track1 == null) {
      this.track1 = new JComboBox(this.tracks);
    }
    this.track1.setSelectedIndex(0);
    this.tr1 = ((String)this.track1.getSelectedItem());
    this.track1.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        JComboBox cb = (JComboBox)e.getSource();
        TabWindow.this.tr1 = ((String)cb.getSelectedItem());
      }
    });
    return this.track1;
  }
  
  public JCheckBox getPKCheck1()
  {
    this.pkcheck1.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	    if (pkcheck1.isSelected()) {
	    	
	    	  pksam = ((String) pksamdil.getSelectedItem());								//APF0024  this line was added			
	    	  pkstd = ((String)pkstddil.getSelectedItem());									//APF0024  this line was added			
	    	  pkmrddil = ((String)pkMRDdil.getSelectedItem());								//APF0024  this line was added			
	    	  pkstdlab = ((String)pkSTDsource.getSelectedItem());							//APF0024  this line was added			
	    	  pksamlab = ((String)pkSsource.getSelectedItem());								//APF0024  this line was added			
	    	  pkmrdlab = ((String)pkMRDsource.getSelectedItem());							//APF0024  this line was added				
	    	  pkmrddeck = ((String)pkdeck1.getSelectedItem());								//APF0024  this line was added			
	    	  pkdillab2 = ((String)pkDilsource.getSelectedItem());							//APF0024  this line was added
	    	  
	    	  TabWindow.this.pkm_text_1.setEnabled(true);
	    	  TabWindow.this.pkm_text_ul.setEnabled(true);
//	    	  TabWindow.this.pkdil.setEnabled(true);
	    	  TabWindow.this.pksamdil.setEnabled(true);
	    	  TabWindow.this.pkstddil.setEnabled(true);
	    	  TabWindow.this.pkMRDdil.setEnabled(true);
	    	  TabWindow.this.pkSTDsource.setEnabled(true);
	    	  TabWindow.this.pkSsource.setEnabled(true);
	    	  TabWindow.this.pkMRDsource.setEnabled(true);
	    	  TabWindow.this.lblSamples.setVisible(true);
//	    	  TabWindow.this.lblStandards.setVisible(true);
//	    	  TabWindow.this.lblDiluent1.setVisible(true);
//	    	  TabWindow.this.pklabel70.setVisible(true);
//	    	  TabWindow.this.pkdeck1.setVisible(true);
	    	  TabWindow.this.pkdeck1.setEnabled(true);
	    	  TabWindow.this.pkDilsource.setEnabled(true);
//	    	  TabWindow.this.lblPosition.setVisible(true);
    	      }
	    else
	         {
	    	  TabWindow.this.pkm_text_1.setEnabled(false);
	    	  TabWindow.this.pkm_text_ul.setEnabled(false);
	  	      if (!pkcheck2.isSelected()) {
		    	  TabWindow.this.pkMRDsource.setEnabled(false);
		    	  TabWindow.this.pkMRDdil.setEnabled(false);
		    	  TabWindow.this.pkdeck1.setEnabled(false);
		    	  TabWindow.this.pkDilsource.setEnabled(false);
	  	      }
//	    	  TabWindow.this.pkdil.setEnabled(false);
	    	  TabWindow.this.pksamdil.setEnabled(false);
	    	  TabWindow.this.pkstddil.setEnabled(false);
	    	  TabWindow.this.pkSTDsource.setEnabled(false);
	    	  TabWindow.this.pkSsource.setEnabled(false);
	    	  TabWindow.this.pkdeck1.setEnabled(false);
	    	  TabWindow.this.pkDilsource.setEnabled(false);
//	    	  TabWindow.this.lblSamples.setVisible(false);
//	    	  TabWindow.this.lblStandards.setVisible(false);
//	    	  TabWindow.this.lblDiluent1.setVisible(false);
	         }
      }
    });
    return pkcheck1;
  }

  public JCheckBox getPKCheck2()
  {
    this.pkcheck2.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
  	    if (pkcheck2.isSelected()) {
  	    	
  	    	pkdildil = (String) pkdil.getSelectedItem();								//APF0021  this line was added  -  Dilution Diluent Liquid
  	    	pkdillab = ((String) pkDsource.getSelectedItem());							//APF0021  this line was added  -  Dilution Labware
  	    	pkmrdlab = ((String)pkMRDsource.getSelectedItem());							//APF0021  this line was added  -  MRD Labware
          	pkmrddil = (String) pkMRDdil.getSelectedItem();								//APF0021  this line was added  -  MRD Diluent Liquid
          	pkdildeck = (String) pkdeck2.getSelectedItem();								//APF0021  this line was added  -  Dilution Diluent Deck Pos
  	    	pkdillab3 = (String) pkDilreserve.getSelectedItem();						//APF0021  this line was added  -  Dilution Diluent Labware
  	    	
	    	  TabWindow.this.pkdil.setEnabled(true);
	    	  TabWindow.this.pkDsource.setEnabled(true);
	    	  TabWindow.this.pkMRDsource.setEnabled(true);
	    	  TabWindow.this.pkMRDdil.setEnabled(true);
	    	  TabWindow.this.pkcheck3.setEnabled(true);
	    	  
//	    	  TabWindow.this.pklabel70.setVisible(true);
//	    	  TabWindow.this.pklabel71.setVisible(true);
//	    	  TabWindow.this.pkdeck1.setVisible(true);
	    	  TabWindow.this.pkdeck1.setEnabled(true);
//	    	  TabWindow.this.pkdeck2.setVisible(true);
//	    	  TabWindow.this.pkdeck1.setEnabled(false);
	    	  TabWindow.this.pkdeck2.setEnabled(true);
//	    	  TabWindow.this.pkDilsource.setEnabled(false);
	    	  TabWindow.this.pkDilsource.setEnabled(true);
	    	  TabWindow.this.pkDilreserve.setEnabled(true);
	  	      if (!pkcheck1.isSelected()) {
		    	  TabWindow.this.pkdeck1.setEnabled(false);
		    	  TabWindow.this.pkDilsource.setEnabled(false);
	  	      }
	  	      
//	    	  TabWindow.this.lblDiluent1.setVisible(true);
//	    	  TabWindow.this.lblPosition.setVisible(true);
//	          TabWindow.this.lblDiluent2.setVisible(true);

  	      }
	    else
	         {
//	    		  //TabWindow.this.pkDsource.setSelectedIndex(0);		//APF0021  Disabled this line.  TO BE DELETED
		    	  TabWindow.this.pkDsource.setEnabled(false);
//		    	  //TabWindow.this.pkdil.setSelectedIndex(0);			//APF0021  Disabled this line.  TO BE DELETED
		    	  TabWindow.this.pkdil.setEnabled(false);
		    	  TabWindow.this.pklabel71.setVisible(false);
//		    	  TabWindow.this.pkdeck2.setVisible(false);
		    	  TabWindow.this.pkdeck2.setEnabled(false);
		    	  TabWindow.this.pkDilreserve.setEnabled(false);
//		    	  TabWindow.this.lblDiluent1.setVisible(false);
//		    	  TabWindow.this.lblDiluent2.setVisible(false);
//		    	  TabWindow.this.lblPosition.setVisible(false);
		    	  
	  	      if (!pkcheck1.isSelected()) {
		    	  TabWindow.this.pkMRDsource.setEnabled(false);
		    	  TabWindow.this.pkMRDdil.setEnabled(false);
//		    	  TabWindow.this.pklabel70.setVisible(false);
//		    	  TabWindow.this.pkdeck1.setVisible(false);
		    	  TabWindow.this.pkdeck1.setEnabled(false);
		    	  TabWindow.this.pkDilsource.setEnabled(false);
	  	      }
	  	      
//		  	      //TabWindow.this.pkcheck3.setSelected(false);								//PKF0035  Commented out
//		    	  //TabWindow.this.pkcheck3.setEnabled(true);								//PKF0035  Commented out
//		    	  //TabWindow.this.pkm_text_ul_preAssay.setText("");						//PKF0035  Commented out
//		    	  //TabWindow.this.pkm_text_ul_preAssay.setEnabled(false);					//PKF0035  Commented out
//		    	  //TabWindow.this.pkPassay.setSelectedIndex(0);							//PKF0035  Commented out
//		    	  //TabWindow.this.pkPassay.setEnabled(false);								//PKF0035  Commented out
//		    	  
//		    	  //TabWindow.this.pkcheck4.setSelected(false);								//PKF0035  Commented out
//		    	  //TabWindow.this.pkcheck4.setEnabled(false);								//PKF0035  Commented out
//		    	  //TabWindow.this.pkm_text_ul_Assay.setText("");							//PKF0035  Commented out
//		    	  //TabWindow.this.pkm_text_ul_Assay.setEnabled(false);						//PKF0035  Commented out
//		    	  //TabWindow.this.pkassay.setSelectedIndex(0);								//PKF0035  Commented out
//		    	  //TabWindow.this.pkassay.setEnabled(false);								//PKF0035  Commented out
	         }
      }
    });
    return pkcheck2;
  }

  public JCheckBox getPKCheck3()
  {
    this.pkcheck3.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	    if (pkcheck3.isSelected()) {
	    	  
	    	  TabWindow.this.pkm_text_ul_preAssay.setEnabled(true);
	    	  TabWindow.this.pkPassay.setEnabled(true);
	    	  
	    	  TabWindow.this.pkcheck4.setEnabled(true);
    	      }
	    else
	         {            
	    	  TabWindow.this.pkm_text_ul_preAssay.setText("");
	    	  TabWindow.this.pkm_text_ul_preAssay.setEnabled(false);
	    	  TabWindow.this.pkPassay.setSelectedIndex(0);
	    	  TabWindow.this.pkPassay.setEnabled(false);
	    	
	    	  TabWindow.this.pkcheck4.setSelected(false);
	    	  TabWindow.this.pkcheck4.setEnabled(false);
	    	  TabWindow.this.pkm_text_ul_Assay.setText("");
	    	  TabWindow.this.pkm_text_ul_Assay.setEnabled(false);
	    	  TabWindow.this.pkassay.setSelectedIndex(0);
	    	  TabWindow.this.pkassay.setEnabled(false);
	         }
      }
    });
    return pkcheck3;
  }

  public JCheckBox getPKCheck4()
  {
    this.pkcheck4.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
	    if (pkcheck4.isSelected()) {
	    	  TabWindow.this.pkassay.setEnabled(true);
	    	  TabWindow.this.pkm_text_ul_Assay.setEnabled(true);
    	      }
	    else
	         {
	    	  TabWindow.this.pkm_text_ul_Assay.setText("");
	    	  TabWindow.this.pkm_text_ul_Assay.setEnabled(false);
	    	  TabWindow.this.pkassay.setSelectedIndex(0);
	    	  TabWindow.this.pkassay.setEnabled(false);
	         }
      }
    });
    return pkcheck4;
  }

  public JComboBox getTrack2()
  {
    if (this.track2 == null) {
      this.track2 = new JComboBox(this.tracks);
    }
    this.track2.setSelectedIndex(0);
    this.tr2 = ((String)this.track2.getSelectedItem());
    this.track2.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        JComboBox cb = (JComboBox)e.getSource();
        TabWindow.this.tr2 = ((String)cb.getSelectedItem());
      }
    });
    return this.track2;
  }

  public JComboBox getCollection()
  {
    if (this.collection == null) {
      this.collection = new JComboBox();
      this.collection.addItem(" None               ");
      for (int i = 0; i < this.p96count; i++)
      {
        this.collection.addItem(this.plates96[i]);
      }

    }

    this.collection.setSelectedIndex(0);
    this.col = ((String)this.collection.getSelectedItem());
    this.collection.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        JComboBox cb = (JComboBox)e.getSource();
        TabWindow.this.col = ((String)cb.getSelectedItem());
        int cpx = TabWindow.this.collection.getSelectedIndex();
        if (cpx != 0)
        {
          TabWindow.this.hline6 = TabWindow.this.col.trim();
        }
      }
    });
    return this.collection;
  }

  public JComboBox getMove()
  {
    if (this.move == null) {
      this.move = new JComboBox();
      this.move.addItem(" Trash              ");
    }

    this.move.setSelectedIndex(0);
    this.mov = ((String)this.move.getSelectedItem());
    this.move.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        JComboBox cb = (JComboBox)e.getSource();
        TabWindow.this.mov = ((String)cb.getSelectedItem());
      }
    });
    return this.move;
  }

  public JComboBox getRdr()
  {
    if (this.reader == null) {
      this.reader = new JComboBox(this.readers);
      this.reader.setSelectedIndex(0);
      this.r1 = ((String)this.reader.getSelectedItem());
      this.reader.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          JComboBox cb = (JComboBox)e.getSource();
          TabWindow.this.r1 = ((String)cb.getSelectedItem());
        } } );
    }
    return this.reader;
  }

  public JComboBox getLiqType()
  {
    if (this.liqtype == null) {
      this.liqtype = new JComboBox(this.liqtypes);
      this.liqtype.setSelectedIndex(0);
      this.lt1 = ((String)this.liqtype.getSelectedItem());
      this.liqtype.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          JComboBox cb = (JComboBox)e.getSource();
          TabWindow.this.lt1 = ((String)cb.getSelectedItem());
        } } );
    }
    return this.liqtype;
  }

  public JComboBox getSLiqType()
  {
    if (this.sliqtype == null) {
      this.sliqtype = new JComboBox(this.liqtypes);
      this.sliqtype.setSelectedIndex(0);
      this.slt1 = ((String)this.sliqtype.getSelectedItem());
      this.sliqtype.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          JComboBox cb = (JComboBox)e.getSource();
          TabWindow.this.slt1 = ((String)cb.getSelectedItem());
        } } );
    }
    return this.sliqtype;
  }

  public JComboBox getDLiqType()
  {
    if (this.dliqtype == null) {
      this.dliqtype = new JComboBox(this.liqtypes);
      this.dliqtype.setSelectedIndex(0);
      this.dlt1 = ((String)this.dliqtype.getSelectedItem());
      this.dliqtype.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          JComboBox cb = (JComboBox)e.getSource();
          TabWindow.this.dlt1 = ((String)cb.getSelectedItem());
        } } );
    }
    return this.dliqtype;
  }

  public JComboBox getBLiqType()
  {
    if (this.bliqtype == null) {
      this.bliqtype = new JComboBox(this.liqtypes);
      this.bliqtype.setSelectedIndex(0);
      this.blt1 = ((String)this.bliqtype.getSelectedItem());
      this.bliqtype.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          JComboBox cb = (JComboBox)e.getSource();
          TabWindow.this.blt1 = ((String)cb.getSelectedItem());
        } } );
    }
    return this.bliqtype;
  }

  private JRadioButton getPlateButton()
  {
    if (this.platebutton == null) {
      this.platebutton = new JRadioButton();
      this.platebutton.setSelected(true);
      this.platebutton.setText("Plate");
      this.platebutton.setBackground(this.panelColor);
      this.platebutton.setHorizontalAlignment(0);
      this.lbg.add(this.platebutton);
      this.platebutton.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          TabWindow.this.liqPanel1.removeAll();
          TabWindow.this.getSrc();
          TabWindow.this.source.revalidate();
          TabWindow.this.splatetype.setVisible(true);
          TabWindow.this.liqPanel1.add(TabWindow.this.getLiqPanel1Top(), "North");
          TabWindow.this.liqPanel1.add(TabWindow.this.panelbl, "Center");
          TabWindow.this.liqPanel1.add(TabWindow.this.getLiqPanel1Bot(), "South");
          TabWindow.this.liqPanel1.revalidate();
          TabWindow.this.liqPanel1.repaint();
        } } );
    }
    return this.platebutton;
  }

  private JRadioButton getPlateButton2()
  {
    if (this.platebutton2 == null) {
      this.platebutton2 = new JRadioButton();
      this.platebutton2.setSelected(true);
      this.platebutton2.setText("Plate");
      this.platebutton2.setBackground(this.panelColor);
      this.platebutton2.setHorizontalAlignment(0);
      this.lbg2.add(this.platebutton2);
      this.platebutton2.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          TabWindow.this.liqPanel2.removeAll();
          TabWindow.this.getDest();
          TabWindow.this.dest.revalidate();
          TabWindow.this.dplatetype.setVisible(true);
          TabWindow.this.liqPanel2.add(TabWindow.this.getLiqPanel2Top(), "North");
          TabWindow.this.liqPanel2.add(TabWindow.this.panelbr, "Center");
          TabWindow.this.liqPanel2.add(TabWindow.this.getLiqPanel2Bot(), "South");
          TabWindow.this.liqPanel2.revalidate();
          TabWindow.this.liqPanel2.repaint();
          TabWindow.this.liqPanel2top.revalidate();
          TabWindow.this.liqPanel2top.repaint();
        } } );
    }
    return this.platebutton2;
  }

  private JRadioButton getTubeButton2()
  {
    if (this.tubebutton2 == null) {
      this.tubebutton2 = new JRadioButton();
      this.tubebutton2.setText("Tube");
      this.tubebutton2.setHorizontalAlignment(0);
      this.tubebutton2.setBackground(this.panelColor);
      this.lbg2.add(this.tubebutton2);
      if (!this.showtube) {
        this.tubebutton2.setEnabled(false);
      }
      this.tubebutton2.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          TabWindow.this.liqPanel2.removeAll();
          TabWindow.this.getDest();
          TabWindow.this.dest.revalidate();
          TabWindow.this.dest.repaint();
          TabWindow.this.liqPanel2.add(TabWindow.this.getLiqPanel2Top(), "North");
          TabWindow.this.liqPanel2.add(TabWindow.this.tubepanel2, "Center");
          TabWindow.this.liqPanel2.add(TabWindow.this.getLiqPanel2Bot(), "South");
          TabWindow.this.dplatetype.setVisible(false);
          TabWindow.this.liqPanel2.revalidate();
          TabWindow.this.liqPanel2.repaint();
          TabWindow.this.s2inc.setVisible(false);
          TabWindow.this.liqPanel2top.revalidate();
          TabWindow.this.liqPanel2top.repaint();
        } } );
    }
    return this.tubebutton2;
  }

  private JRadioButton getResButton()
  {
    if (this.resbutton == null) {
      this.resbutton = new JRadioButton();
      this.resbutton.setText("Reservoir");
      this.resbutton.setHorizontalAlignment(0);
      this.resbutton.setBackground(this.panelColor);
      this.lbg.add(this.resbutton);
      if (!this.showres) {
        this.resbutton.setEnabled(false);
      }
      this.resbutton.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          TabWindow.this.liqPanel1.removeAll();
          TabWindow.this.getSrc();
          TabWindow.this.source.revalidate();
          TabWindow.this.dplatetype.setVisible(true);
          TabWindow.this.liqPanel1.add(TabWindow.this.getLiqPanel1Top(), "North");
//          TabWindow.this.liqPanel1.add(TabWindow.this.getres12Xpanel(TabWindow.this.rnum), "Center");		//APN0047 Commented out
          TabWindow.this.liqPanel1.add(TabWindow.this.ReservoirPanel1, "Center");							//APN0047 Replaced above		
          TabWindow.this.liqPanel1.add(TabWindow.this.getLiqPanel1Bot(), "South");
          TabWindow.this.liqPanel1.revalidate();
          TabWindow.this.liqPanel1.repaint();
          TabWindow.this.splatetype.setVisible(true);
          TabWindow.this.liqPanel1.revalidate();
          TabWindow.this.liqPanel1.repaint();
        } } );
    }
    return this.resbutton;
  }

  private JRadioButton getResButton2()
  {
    if (this.resbutton2 == null) {
      this.resbutton2 = new JRadioButton();
      this.resbutton2.setText("Reservoir");
      this.resbutton2.setHorizontalAlignment(0);
      this.resbutton2.setBackground(this.panelColor);
      this.lbg2.add(this.resbutton2);
      if (!this.showres) {
        this.resbutton2.setEnabled(false);
      }
      this.resbutton2.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          
          TabWindow.this.getDest();
          TabWindow.this.dest.revalidate();
          TabWindow.this.dplatetype.setVisible(true);
          TabWindow.this.liqPanel2.removeAll();
          TabWindow.this.liqPanel2.add(TabWindow.this.getLiqPanel2Top(), "North");
//          TabWindow.this.liqPanel2.add(TabWindow.this.getres12Xpanel2(TabWindow.this.rnum2), "Center");	//APN0047 COMMENTED OUT
          TabWindow.this.liqPanel2.add(TabWindow.this.ReservoirPanel2, "Center");							//APN0047 Replaced above		
          TabWindow.this.liqPanel2.add(TabWindow.this.getLiqPanel2Bot(), "South");
          TabWindow.this.liqPanel2.revalidate();
          TabWindow.this.liqPanel2.repaint();
          TabWindow.this.liqPanel2.revalidate();
          TabWindow.this.liqPanel2.repaint();

        } } );
    }
    return this.resbutton2;
  }

  private JRadioButton getbutton48l()
  {
    if (this.button48l == null) {
      this.button48l = new JRadioButton();
      this.button48l.setText("48");
      this.button48l.setHorizontalAlignment(0);
      this.button48l.setSelected(false);
      this.button48l.setBackground(this.panelColor);
      this.lbg3.add(this.button48l);
      this.button48l.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          TabWindow.this.liqPanel2.removeAll();
          TabWindow.this.getDest();
          TabWindow.this.dest.removeAllItems();
          for (int i = 0; i < TabWindow.this.p48count; i++)
            TabWindow.this.dest.addItem(TabWindow.this.plates48[i]);
          TabWindow.this.dest.revalidate();
          TabWindow.this.dplatetype.setVisible(true);
          TabWindow.this.liqPanel2.add(TabWindow.this.getLiqPanel2Top(), "North");
          TabWindow.this.liqPanel2.add(TabWindow.this.panel48r, "Center");
          TabWindow.this.liqPanel2.add(TabWindow.this.getLiqPanel2Bot(), "South");
          TabWindow.this.liqPanel2.revalidate();
          TabWindow.this.liqPanel2.repaint();
          TabWindow.this.liqPanel2top.revalidate();
          TabWindow.this.liqPanel2top.repaint();
          if (TabWindow.this.tubebutton2.isSelected())
          {
            TabWindow.this.button48l.setVisible(false);
          }
        }

      });
    }
    else if (this.tubebutton2.isSelected())
    {
      this.button48l.setVisible(false);
    }

    return this.button48l;
  }

  private JRadioButton getbutton48l2()
  {
    if (this.button48l2 == null) {
      this.button48l2 = new JRadioButton();
      this.button48l2.setText("48");
      this.button48l2.setHorizontalAlignment(0);
      this.button48l2.setSelected(false);
      this.button48l2.setBackground(this.panelColor);
      this.lbg4.add(this.button48l2);
      this.button48l2.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          TabWindow.this.liqPanel1.removeAll();
          TabWindow.this.getSrc();
          TabWindow.this.source.removeAllItems();
          for (int i = 0; i < TabWindow.this.p48count; i++)
            TabWindow.this.source.addItem(TabWindow.this.plates48[i]);
          TabWindow.this.source.revalidate();
          TabWindow.this.splatetype.setVisible(true);
          TabWindow.this.liqPanel1.add(TabWindow.this.getLiqPanel1Top(), "North");
          TabWindow.this.liqPanel1.add(TabWindow.this.panel48l, "Center");
          TabWindow.this.liqPanel1.add(TabWindow.this.getLiqPanel1Bot(), "South");
          TabWindow.this.liqPanel1top.revalidate();
          TabWindow.this.liqPanel1top.repaint();
          TabWindow.this.liqPanel1.revalidate();
          TabWindow.this.liqPanel1.repaint();
          if (TabWindow.this.tubebutton.isSelected())
          {
            TabWindow.this.button48l2.setVisible(false);
            TabWindow.this.button96l2.setVisible(false);
            TabWindow.this.button384l2.setVisible(false);
          }
        }

      });
    }
    else if (this.tubebutton.isSelected())
    {
      this.button48l2.setVisible(false);
      this.button96l2.setVisible(false);
      this.button384l2.setVisible(false);
    }

    return this.button48l2;
  }
  
  
  private JRadioButton getbutton96l()
  {
    if (this.button96l == null) {
      this.button96l = new JRadioButton();
      this.button96l.setText("96");
      this.button96l.setHorizontalAlignment(0);
      this.button96l.setSelected(true);
      this.button96l.setBackground(this.panelColor);
      this.lbg3.add(this.button96l);
      this.button96l.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          TabWindow.this.liqPanel2.removeAll();
          TabWindow.this.getDest();
          TabWindow.this.dest.removeAllItems();
          for (int i = 0; i < TabWindow.this.p96count; i++)
            TabWindow.this.dest.addItem(TabWindow.this.plates96[i]);
          TabWindow.this.dest.revalidate();
          TabWindow.this.dplatetype.setVisible(true);
          TabWindow.this.liqPanel2.add(TabWindow.this.getLiqPanel2Top(), "North");
          TabWindow.this.liqPanel2.add(TabWindow.this.panel96r, "Center");
          TabWindow.this.liqPanel2.add(TabWindow.this.getLiqPanel2Bot(), "South");
          TabWindow.this.liqPanel2.revalidate();
          TabWindow.this.liqPanel2.repaint();
          TabWindow.this.liqPanel2top.revalidate();
          TabWindow.this.liqPanel2top.repaint();
          if (TabWindow.this.tubebutton2.isSelected())
          {
            TabWindow.this.button96l.setVisible(false);
          }
        }

      });
    }
    else if (this.tubebutton2.isSelected())
    {
      this.button96l.setVisible(false);
    }

    return this.button96l;
  }

  private JRadioButton getbutton96l2()
  {
    if (this.button96l2 == null) {
      this.button96l2 = new JRadioButton();
      this.button96l2.setText("96");
      this.button96l2.setHorizontalAlignment(0);
      this.button96l2.setSelected(true);
      this.button96l2.setBackground(this.panelColor);
      this.lbg4.add(this.button96l2);
      this.button96l2.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          TabWindow.this.liqPanel1.removeAll();
          TabWindow.this.getSrc();
          TabWindow.this.source.removeAllItems();
          for (int i = 0; i < TabWindow.this.p96count; i++)
            TabWindow.this.source.addItem(TabWindow.this.plates96[i]);
          TabWindow.this.source.revalidate();
          TabWindow.this.splatetype.setVisible(true);
          TabWindow.this.liqPanel1.add(TabWindow.this.getLiqPanel1Top(), "North");
          TabWindow.this.liqPanel1.add(TabWindow.this.panel96l, "Center");
          TabWindow.this.liqPanel1.add(TabWindow.this.getLiqPanel1Bot(), "South");
          TabWindow.this.liqPanel1top.revalidate();
          TabWindow.this.liqPanel1top.repaint();
          TabWindow.this.liqPanel1.revalidate();
          TabWindow.this.liqPanel1.repaint();
          if (TabWindow.this.tubebutton.isSelected())
          {
            TabWindow.this.button96l2.setVisible(false);
            TabWindow.this.button384l2.setVisible(false);
          }
        }

      });
    }
    else if (this.tubebutton.isSelected())
    {
      this.button96l2.setVisible(false);
      this.button384l2.setVisible(false);
    }

    return this.button96l2;
  }

  private JRadioButton getbutton96s()
  {
    if (this.button96s == null) {
      this.button96s = new JRadioButton();
      this.button96s.setText("96");
      this.button96s.setHorizontalAlignment(0);
      this.button96s.setSelected(true);
      this.button96s.setBackground(this.panelColor);
      this.sbg2.add(this.button96s);
      this.button96s.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          TabWindow.this.serPanel2.removeAll();
          TabWindow.this.getDest2();
          TabWindow.this.dest2.removeAllItems();
          for (int i = 0; i < TabWindow.this.p96count; i++)
            TabWindow.this.dest2.addItem(TabWindow.this.plates96[i]);
          TabWindow.this.dest2.revalidate();

          TabWindow.this.serPanel2.add(TabWindow.this.getSerPanel2Top(), "North");
          TabWindow.this.serPanel2.add(TabWindow.this.spanel96r, "Center");
          TabWindow.this.serPanel2.revalidate();
          TabWindow.this.serPanel2.repaint();
        } } );
    }
    return this.button96s;
  }

  private JRadioButton getbutton384l()
  {
    if (this.button384l == null) {
      this.button384l = new JRadioButton();
      this.button384l.setText("384");
      this.button384l.setHorizontalAlignment(0);
      this.button384l.setBackground(this.panelColor);
      this.lbg3.add(this.button384l);
      this.button384l.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          TabWindow.this.liqPanel2.removeAll();
          TabWindow.this.getDest();
          TabWindow.this.dest.removeAllItems();
          for (int i = 0; i < TabWindow.this.p384count; i++)
            TabWindow.this.dest.addItem(TabWindow.this.plates384[i]);
          TabWindow.this.dest.revalidate();
          TabWindow.this.dplatetype.setVisible(true);
          TabWindow.this.liqPanel2.add(TabWindow.this.getLiqPanel2Top(), "North");
          TabWindow.this.liqPanel2.add(TabWindow.this.panel384r, "Center");
          TabWindow.this.liqPanel2.add(TabWindow.this.getLiqPanel2Bot(), "South");
          TabWindow.this.liqPanel2.revalidate();
          TabWindow.this.liqPanel2.repaint();
        } } );
    }
    return this.button384l;
  }

  private JRadioButton getbutton384l2()
  {
    if (this.button384l2 == null) {
      this.button384l2 = new JRadioButton();
      this.button384l2.setText("384");
      this.button384l2.setHorizontalAlignment(0);
      this.button384l2.setBackground(this.panelColor);
      this.lbg4.add(this.button384l2);
      this.button384l2.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          TabWindow.this.liqPanel1.removeAll();
          TabWindow.this.getSrc();
          TabWindow.this.source.removeAllItems();
          for (int i = 0; i < TabWindow.this.p384count; i++)
            TabWindow.this.source.addItem(TabWindow.this.plates384[i]);
          TabWindow.this.source.revalidate();
          TabWindow.this.splatetype.setVisible(true);
          TabWindow.this.liqPanel1.add(TabWindow.this.getLiqPanel1Top(), "North");
          TabWindow.this.liqPanel1.add(TabWindow.this.panel384l, "Center");
          TabWindow.this.liqPanel1.add(TabWindow.this.getLiqPanel1Bot(), "South");
          TabWindow.this.liqPanel1.revalidate();
          TabWindow.this.liqPanel1.repaint();
        } } );
    }
    return this.button384l2;
  }

  private JRadioButton getbutton384s()
  {
    if (this.button384s == null) {
      this.button384s = new JRadioButton();
      this.button384s.setText("384");
      this.button384s.setHorizontalAlignment(0);
      this.button384s.setBackground(this.panelColor);
      this.sbg2.add(this.button384s);
      this.button384s.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          TabWindow.this.serPanel2.removeAll();
          TabWindow.this.getDest2();
          TabWindow.this.dest2.removeAllItems();
          for (int i = 0; i < TabWindow.this.p384count; i++)
            TabWindow.this.dest2.addItem(TabWindow.this.plates384[i]);
          TabWindow.this.dest2.revalidate();
          TabWindow.this.serPanel2top.revalidate();
          TabWindow.this.serPanel2top.repaint();
          TabWindow.this.serPanel2.add(TabWindow.this.getSerPanel2Top(), "North");
          TabWindow.this.serPanel2.add(TabWindow.this.spanel384r, "Center");
          TabWindow.this.serPanel2.revalidate();
          TabWindow.this.serPanel2.repaint();
        } } );
    }
    return this.button384s;
  }

  private JRadioButton getBrResButton1()
  {
    if (this.brresbutton1 == null) {
      this.brresbutton1 = new JRadioButton();
      this.brresbutton1.setText("Res");
      this.brresbutton1.setBackground(this.panelColor);
      this.brresbutton1.setHorizontalAlignment(0);
      this.brresbutton1.setSelected(true);
      this.brbg1.add(this.brresbutton1);
      if (!this.showres) {
        this.brresbutton1.setEnabled(false);
      }
      this.brresbutton1.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          TabWindow.this.brewPanel1.removeAll();
          TabWindow.this.getSrc4();
          TabWindow.this.source4.revalidate();
          TabWindow.this.source4.repaint();
          TabWindow.this.brewPanel1.add(TabWindow.this.getBrewPanel1Top(), "North");
          TabWindow.this.brewPanel1.add(TabWindow.this.brrespanel1, "Center");
          TabWindow.this.brewPanel1.add(TabWindow.this.getBrewPanel1Bot(), "South");
          TabWindow.this.brewPanel1.revalidate();
          TabWindow.this.brewPanel1.repaint();
        } } );
    }
    return this.brresbutton1;
  }

  private JRadioButton getBrResbutton2()
  {
    if (this.brresbutton2 == null) {
      this.brresbutton2 = new JRadioButton();
      this.brresbutton2.setText("Res");
      this.brresbutton2.setHorizontalAlignment(0);
      this.brresbutton2.setBackground(this.panelColor);
      this.brresbutton2.setSelected(true);
      this.brbg2.add(this.brresbutton2);
      if (!this.showres) {
        this.brresbutton2.setEnabled(false);
      }
      this.brresbutton2.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          TabWindow.this.brewPanel2.removeAll();
          TabWindow.this.getDest3();
          TabWindow.this.dest3.revalidate();
          TabWindow.this.dest3.repaint();
          TabWindow.this.brewPanel2.add(TabWindow.this.getBrewPanel2Top(), "North");
          TabWindow.this.brewPanel2.add(TabWindow.this.brrespanel2, "Center");
          TabWindow.this.brewPanel2.add(TabWindow.this.getBrewPanel2Bot(), "South");
          TabWindow.this.brewPanel2.revalidate();
          TabWindow.this.brewPanel2.repaint();
        } } );
    }
    return this.brresbutton2;
  }

  private JRadioButton getTubeButton()
  {
    if (this.tubebutton == null) {
      this.tubebutton = new JRadioButton();
      this.tubebutton.setText("Tube");
      this.tubebutton.setHorizontalAlignment(0);
      this.tubebutton.setBackground(this.panelColor);
      this.lbg.add(this.tubebutton);
      if (!this.showtube) {
        this.tubebutton.setEnabled(false);
      }
      this.tubebutton.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          TabWindow.this.liqPanel1.removeAll();
          TabWindow.this.getSrc();
          TabWindow.this.source.revalidate();
          TabWindow.this.liqPanel1.add(TabWindow.this.getLiqPanel1Top(), "North");
          TabWindow.this.liqPanel1.add(TabWindow.this.tubepanel, "Center");
          TabWindow.this.liqPanel1.add(TabWindow.this.getLiqPanel1Bot(), "South");
          TabWindow.this.splatetype.setVisible(false);
          TabWindow.this.liqPanel1.revalidate();
          TabWindow.this.liqPanel1.repaint();
        } } );
    }
    return this.tubebutton;
  }

  private JRadioButton getPlateButton3() {
    if (this.platebutton3 == null) {
      this.platebutton3 = new JRadioButton();
      this.platebutton3.setSelected(true);
      this.platebutton3.setText("Plate");
      this.platebutton3.setHorizontalAlignment(0);
      this.platebutton3.setBackground(this.panelColor);
      this.sbg.add(this.platebutton3);
      this.platebutton3.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          TabWindow.this.serPanel1.removeAll();
          TabWindow.this.getSrc3();
          TabWindow.this.source3.revalidate();
          TabWindow.this.splatetype2.setVisible(true);
          TabWindow.this.serPanel1.add(TabWindow.this.getSerPanel1Top(), "North");
          TabWindow.this.serPanel1.add(TabWindow.this.spanel96l, "Center");
          TabWindow.this.serPanel1.revalidate();
          TabWindow.this.serPanel1.repaint();
        } } );
    }
    return this.platebutton3;
  }

  private JRadioButton getlinc1Button() {
    if (this.linc1button == null) {
      this.linc1button = new JRadioButton();
      this.linc1button.setVisible(false);
      this.linc1button.setSelected(false);
      this.linc1button.setText("Increment");
      this.linc1button.setHorizontalAlignment(0);
      this.linc1button.setSelected(false);

      this.linc1button.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          if (TabWindow.this.linc1button.isSelected())
          {
            TabWindow.this.linc2button.setSelected(false);
            TabWindow.this.linc3button.setSelected(false);
          }
        } } );
    }
    return this.linc1button;
  }
  private JRadioButton getlinc2Button() {
    if (this.linc2button == null) {
      this.linc2button = new JRadioButton();
      this.linc2button.setVisible(false);
      this.linc2button.setSelected(false);
      this.linc2button.setText("Increment Row");
      this.linc2button.setSelected(false);
      this.linc2button.setHorizontalAlignment(0);

      this.linc2button.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          if (TabWindow.this.linc2button.isSelected())
          {
            TabWindow.this.linc1button.setSelected(false);
            TabWindow.this.linc3button.setSelected(false);
          }
        }
      });
    }
    return this.linc2button;
  }
  private JRadioButton getlinc3Button() {
    if (this.linc3button == null) {
      this.linc3button = new JRadioButton();
      this.linc3button.setSelected(true);
      this.linc3button.setVisible(false);
      this.linc3button.setText("Increment Col");
      this.linc3button.setSelected(false);
      this.linc3button.setHorizontalAlignment(0);

      this.linc3button.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          if (TabWindow.this.linc3button.isSelected())
          {
            TabWindow.this.linc1button.setSelected(false);
            TabWindow.this.linc2button.setSelected(false);
          }
        } } );
    }
    return this.linc3button;
  }
  private JRadioButton getlinc4Button() {
    if (this.linc4button == null) {
      this.linc4button = new JRadioButton();
      this.linc4button.setVisible(false);
      this.linc4button.setSelected(false);
      this.linc4button.setText("Increment");
      this.linc4button.setHorizontalAlignment(0);

      this.linc4button.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          if (TabWindow.this.linc4button.isSelected())
          {
            TabWindow.this.linc5button.setSelected(false);
            TabWindow.this.linc6button.setSelected(false);
          }
        }
      });
    }
    return this.linc4button;
  }
  private JRadioButton getlinc5Button() {
    if (this.linc5button == null) {
      this.linc5button = new JRadioButton();
      this.linc5button.setVisible(false);
      this.linc5button.setSelected(false);
      this.linc5button.setText("Increment Row");
      this.linc5button.setHorizontalAlignment(0);

      this.linc5button.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          if (TabWindow.this.linc5button.isSelected())
          {
            TabWindow.this.linc4button.setSelected(false);
            TabWindow.this.linc6button.setSelected(false);
          }
        }
      });
    }
    return this.linc5button;
  }
  private JRadioButton getlinc6Button() {
    if (this.linc6button == null) {
      this.linc6button = new JRadioButton();
      this.linc6button.setVisible(false);
      this.linc6button.setSelected(false);
      this.linc6button.setText("Increment Col");
      this.linc6button.setHorizontalAlignment(0);

      this.linc6button.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          if (TabWindow.this.linc6button.isSelected())
          {
            TabWindow.this.linc4button.setSelected(false);
            TabWindow.this.linc5button.setSelected(false);
          }
        }
      });
    }
    return this.linc6button;
  }

  private JRadioButton getsinc1Button() {
    if (this.sinc1button == null) {
      this.sinc1button = new JRadioButton();
      this.sinc1button.setVisible(false);
      this.sinc1button.setSelected(false);
      this.sinc1button.setText("Increment");
      this.sinc1button.setHorizontalAlignment(0);
      this.sinc1button.setSelected(false);

      this.sinc1button.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          if (TabWindow.this.sinc1button.isSelected())
          {
            TabWindow.this.sinc2button.setSelected(false);
            TabWindow.this.sinc3button.setSelected(false);
          }
        } } );
    }
    return this.sinc1button;
  }
  private JRadioButton getsinc2Button() {
    if (this.sinc2button == null) {
      this.sinc2button = new JRadioButton();
      this.sinc2button.setVisible(false);
      this.sinc2button.setSelected(false);
      this.sinc2button.setText("Increment Row");
      this.sinc2button.setSelected(false);
      this.sinc2button.setHorizontalAlignment(0);

      this.sinc2button.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          if (TabWindow.this.sinc2button.isSelected())
          {
            TabWindow.this.sinc1button.setSelected(false);
            TabWindow.this.sinc3button.setSelected(false);
          }
        }
      });
    }
    return this.sinc2button;
  }
  private JRadioButton getsinc3Button() {
    if (this.sinc3button == null) {
      this.sinc3button = new JRadioButton();
      this.sinc3button.setSelected(true);
      this.sinc3button.setVisible(false);
      this.sinc3button.setText("Increment Col");
      this.sinc3button.setSelected(false);
      this.sinc3button.setHorizontalAlignment(0);

      this.sinc3button.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          if (TabWindow.this.sinc3button.isSelected())
          {
            TabWindow.this.sinc1button.setSelected(false);
            TabWindow.this.sinc2button.setSelected(false);
          }
        } } );
    }
    return this.sinc3button;
  }
  private JRadioButton getsinc4Button() {
    if (this.sinc4button == null) {
      this.sinc4button = new JRadioButton();
      this.sinc4button.setVisible(false);
      this.sinc4button.setSelected(false);
      this.sinc4button.setText("Increment");
      this.sinc4button.setHorizontalAlignment(0);

      this.sinc4button.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          if (TabWindow.this.sinc4button.isSelected())
          {
            TabWindow.this.sinc5button.setSelected(false);
            TabWindow.this.sinc6button.setSelected(false);
          }
        }
      });
    }
    return this.sinc4button;
  }
  private JRadioButton getsinc5Button() {
    if (this.sinc5button == null) {
      this.sinc5button = new JRadioButton();
      this.sinc5button.setVisible(false);
      this.sinc5button.setSelected(false);
      this.sinc5button.setText("Increment Row");
      this.sinc5button.setHorizontalAlignment(0);

      this.sinc5button.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          if (TabWindow.this.sinc5button.isSelected())
          {
            TabWindow.this.sinc4button.setSelected(false);
            TabWindow.this.sinc6button.setSelected(false);
          }
        }
      });
    }
    return this.sinc5button;
  }
  private JRadioButton getsinc6Button() {
    if (this.sinc6button == null) {
      this.sinc6button = new JRadioButton();
      this.sinc6button.setVisible(false);
      this.sinc6button.setSelected(false);
      this.sinc6button.setText("Increment Col");
      this.sinc6button.setHorizontalAlignment(0);

      this.sinc6button.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          if (TabWindow.this.sinc6button.isSelected())
          {
            TabWindow.this.sinc4button.setSelected(false);
            TabWindow.this.sinc5button.setSelected(false);
          }
        }
      });
    }
    return this.sinc6button;
  }

  private JRadioButton getAssemButton() {
    if (this.assembutton == null) {
      this.assembutton = new JRadioButton();
      this.assembutton.setSelected(true);
      this.assembutton.setText("  Assemble      ");
      this.assembutton.setBackground(this.panelColor);
      this.assembutton.setHorizontalAlignment(0);
      this.time.setVisible(false);
      this.pressure.setVisible(false);
      this.spelabel3.setVisible(false);
      this.spelabel4.setVisible(false);
      this.assbg.add(this.assembutton);
      this.assembutton.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          if (TabWindow.this.assembutton.isSelected())
          {
//            TabWindow.this.getFilter();
//            TabWindow.this.getCollection();
//            TabWindow.this.filter.setVisible(true);
//            TabWindow.this.collection.setVisible(true);
//            TabWindow.this.spelabel1.setVisible(true);
//            TabWindow.this.spelabel2.setVisible(true);
            TabWindow.this.time.setVisible(false);
            TabWindow.this.pressure.setVisible(false);
            TabWindow.this.spelabel3.setVisible(false);
            TabWindow.this.spelabel4.setVisible(false);
            TabWindow.this.spePanel.revalidate();
            TabWindow.this.spePanel.repaint();
          }
        }
      });
    }
    return this.assembutton;
  }

  private JRadioButton getVacButton() {
    if (this.vacbutton == null) {
      this.vacbutton = new JRadioButton();
      this.vacbutton.setSelected(true);
      this.vacbutton.setText(" Apply Vacuum");
      this.vacbutton.setHorizontalAlignment(0);
      this.vacbutton.setBackground(this.panelColor);
      this.assbg.add(this.vacbutton);
      this.vacbutton.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          if (TabWindow.this.vacbutton.isSelected())
          {
            TabWindow.this.filter.setVisible(false);
            TabWindow.this.collection.setVisible(false);
            TabWindow.this.spelabel1.setVisible(false);
            TabWindow.this.spelabel2.setVisible(false);
            TabWindow.this.time.setVisible(true);
            TabWindow.this.pressure.setVisible(true);
            TabWindow.this.spelabel3.setVisible(true);
            TabWindow.this.spelabel4.setVisible(true);
            TabWindow.this.spePanel.revalidate();
            TabWindow.this.spePanel.repaint();
          }
        }
      });
    }
    return this.vacbutton;
  }

  private JRadioButton getDisAssemButton() {
    if (this.disassembutton == null) {
      this.disassembutton = new JRadioButton();
      this.disassembutton.setSelected(true);
      this.disassembutton.setText(" Disassemble ");
      this.disassembutton.setHorizontalAlignment(0);
      this.disassembutton.setBackground(this.panelColor);
      this.assbg.add(this.disassembutton);
      this.disassembutton.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          if (TabWindow.this.disassembutton.isSelected())
          {
//            TabWindow.this.getFilter();
//            TabWindow.this.getCollection();
//            TabWindow.this.filter.setVisible(true);
//            TabWindow.this.collection.setVisible(true);
//            TabWindow.this.spelabel1.setVisible(true);
//            TabWindow.this.spelabel2.setVisible(true);
            TabWindow.this.time.setVisible(false);
            TabWindow.this.pressure.setVisible(false);
            TabWindow.this.spelabel3.setVisible(false);
            TabWindow.this.spelabel4.setVisible(false);
            TabWindow.this.spePanel.revalidate();
            TabWindow.this.spePanel.repaint();
          }
        }
      });
    }
    return this.disassembutton;
  }

  private JRadioButton getTubeButton3()
  {
    if (this.tubebutton3 == null) {
      this.tubebutton3 = new JRadioButton();
      this.tubebutton3.setText("Tube");
      this.tubebutton3.setHorizontalAlignment(0);
      this.tubebutton3.setBackground(this.panelColor);
      this.sbg.add(this.tubebutton3);
      if (!this.showtube) {
        this.tubebutton3.setEnabled(false);
      }
      this.tubebutton3.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          TabWindow.this.serPanel1.removeAll();
          TabWindow.this.getSrc3();
          TabWindow.this.source3.revalidate();
          TabWindow.this.splatetype2.setVisible(false);
          TabWindow.this.source3.repaint();
          TabWindow.this.serPanel1.add(TabWindow.this.getSerPanel1Top(), "North");
          TabWindow.this.serPanel1.add(TabWindow.this.stubepanel, "Center");
          TabWindow.this.serPanel1.revalidate();
          TabWindow.this.serPanel1.repaint();
        }
      });
    }
    return this.tubebutton3;
  }

  private JRadioButton getBrTubeButton1()
  {
    if (this.brtubebutton1 == null) {
      this.brtubebutton1 = new JRadioButton();
      this.brtubebutton1.setBackground(this.panelColor);
      this.brtubebutton1.setText("Tube");
      this.brtubebutton1.setHorizontalAlignment(0);
      this.brbg1.add(this.brtubebutton1);
      if (!this.showtube) {
        this.brtubebutton1.setEnabled(false);
      }
      this.brtubebutton1.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          TabWindow.this.brewPanel1.removeAll();
          TabWindow.this.getSrc4();
          TabWindow.this.source4.revalidate();
          TabWindow.this.source4.repaint();
          TabWindow.this.brewPanel1.add(TabWindow.this.getBrewPanel1Top(), "North");
          TabWindow.this.brewPanel1.add(TabWindow.this.brtubepanel, "Center");
          TabWindow.this.brewPanel1.add(TabWindow.this.getBrewPanel1Bot(), "South");
          TabWindow.this.brewPanel1.revalidate();
          TabWindow.this.brewPanel1.repaint();
        } } );
    }
    return this.brtubebutton1;
  }

  private JRadioButton getBrTubebutton2()
  {
    if (this.brtubebutton2 == null) {
      this.brtubebutton2 = new JRadioButton();
      this.brtubebutton2.setText("Tube");
      this.brtubebutton2.setBackground(this.panelColor);
      this.brtubebutton2.setHorizontalAlignment(0);
      this.brbg2.add(this.brtubebutton2);
      if (!this.showtube) {
        this.brtubebutton2.setEnabled(false);
      }
      this.brtubebutton2.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          TabWindow.this.brewPanel2.removeAll();
          TabWindow.this.getDest3();
          TabWindow.this.dest3.revalidate();
          TabWindow.this.dest3.repaint();
          TabWindow.this.brewPanel2.add(TabWindow.this.getBrewPanel2Top(), "North");
          TabWindow.this.brewPanel2.add(TabWindow.this.brtubepanel2, "Center");
          TabWindow.this.brewPanel2.add(TabWindow.this.getBrewPanel2Bot(), "South");
          TabWindow.this.brewPanel2.revalidate();
          TabWindow.this.brewPanel2.repaint();
        } } );
    }
    return this.brtubebutton2;
  }

  private JRadioButton getAButton()
  {
    if (this.abutton == null) {
      this.abutton = new JRadioButton();
      this.abutton.setSelected(true);
      this.abutton.setText("Abv");
      this.abutton.setBackground(this.panelColor);
      this.abutton.setHorizontalAlignment(0);
      this.hbg.add(this.abutton);
      this.abutton.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          TabWindow.this.h1 = "1";
        } } );
    }
    return this.abutton;
  }

  private JRadioButton getBButton()
  {
    if (this.bbutton == null) {
      this.bbutton = new JRadioButton();

      this.bbutton.setText("Bot");
      this.bbutton.setBackground(this.panelColor);
      this.bbutton.setHorizontalAlignment(0);
      this.hbg.add(this.bbutton);
      this.bbutton.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          TabWindow.this.h1 = "3";
        } } );
    }
    return this.bbutton;
  }

  private JRadioButton getSButton()
  {
    if (this.sbutton == null) {
      this.sbutton = new JRadioButton();
      this.sbutton.setSelected(true);
      this.sbutton.setText("Surf");
      this.sbutton.setBackground(this.panelColor);
      this.sbutton.setHorizontalAlignment(0);
      this.hbg.add(this.sbutton);
      this.sbutton.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          TabWindow.this.h1 = "2";
        } } );
    }
    return this.sbutton;
  }

  private JPanel getDataWindow1()
  {
    if (this.dataWindow1 == null) {
      this.dataWindow1 = new JPanel()
      {
        Color color1 = TabWindow.this.steelColor;
        Color color2 = this.color1.brighter();

        protected void paintComponent(Graphics g) {
          if (!isOpaque())
          {
            super.paintComponent(g);
          }

          Graphics2D g2d = (Graphics2D)g;

          int w = getWidth();
          int h = getHeight();

          GradientPaint gp = new GradientPaint(
            0.0F, h, this.color1, 
            0.0F, 0.0F, this.color2);

          g2d.setPaint(gp);
          g2d.fillRect(0, 0, w, h);

          TabWindow.this.dataWindow1.setOpaque(false);
          super.paintComponent(g);
          TabWindow.this.dataWindow1.setOpaque(true);
        }
      };
      FlowLayout flowLayout = new FlowLayout(0, 0, 0);

      this.dataWindow1.setLayout(flowLayout);
      this.dataWindow1.add(getLoadButton());
      this.dataWindow1.add(getSaveButton());
      this.dataWindow1.add(getSaveAsButton());
      this.dataWindow1.add(getNewButton());
      this.dataWindow1.add(this.label18);
      this.dataWindow1.add(getAddButton());
      this.dataWindow1.add(getCheckButton());
      this.dataWindow1.add(getRmvButton());
      this.dataWindow1.add(this.label18b);
      this.dataWindow1.add(getLayoutButton());
      this.dataWindow1.add(getLayoutSelectButton());
      this.dataWindow1.add(this.label18d);
      this.dataWindow1.add(getRunButton());
      this.dataWindow1.add(this.label18c);
      if (security)
      {
    	this.dataWindow1.add(getLogOffButton());
    	this.dataWindow1.add(getsecurityButton());
    	this.dataWindow1.add(getsecurityAccountButton());
    	this.dataWindow1.add(this.label18e);
    	this.dataWindow1.add(this.label18e);
      }
      this.dataWindow1.add(getInfoButton());
    }

    return this.dataWindow1;
  }

  private JPanel getLiqPanel()
  {
    if (this.liqPanel == null) {
      this.liqPanel = new JPanel();
      this.liqPanel.setBackground(this.panelColor);
      this.liqPanel.setLayout(new BorderLayout());
      this.liqPanel.add(getLiqDataPanel(), "North");
      this.liqPanel.add(getLiqTrayPanel(), "Center");
    }
    return this.liqPanel;
  }

  private JPanel getLiqDataPanel()
  {
    if (this.liqdataPanel == null) {
      this.liqdataPanel = new JPanel();
      GridLayout gridLayout = new GridLayout();
      gridLayout.setRows(2);
      gridLayout.setColumns(6);
      gridLayout.setHgap(10);
      gridLayout.setVgap(3);
      this.liqdataPanel.setBackground(this.panelColor);
      this.liqdataPanel.setLayout(gridLayout);

      this.liqdataPanel.add(this.rlabel1);
      this.liqdataPanel.add(this.rlabel2);
      this.liqdataPanel.add(this.rlabel3);
      this.liqdataPanel.add(this.llabel1);
      this.liqdataPanel.add(this.llabel2);
      this.liqdataPanel.add(getAddButton1());

      this.liqdataPanel.add(this.text2);
      this.liqdataPanel.add(this.text3);
      this.liqdataPanel.add(getLiqType());
      this.liqdataPanel.add(this.llabel3);
      this.liqdataPanel.add(this.llabel4);
      this.liqdataPanel.add(getUpdButton1());
    }
    return this.liqdataPanel;
  }

  private JPanel getSerDataPanel()
  {
    if (this.serdataPanel == null) {
      this.serdataPanel = new JPanel();
      GridLayout gridLayout4 = new GridLayout();
      gridLayout4.setRows(2);
      gridLayout4.setColumns(8);
      gridLayout4.setHgap(5);
      gridLayout4.setVgap(2);
      this.serdataPanel.setBackground(this.panelColor);
      this.serdataPanel.setLayout(gridLayout4);
      
      this.serdataPanel.add(this.stnlabel);
      this.serdataPanel.add(this.stlabel);
      this.serdataPanel.add(new JLabel(""));
      this.serdataPanel.add(new JLabel(""));
      this.serdataPanel.add(this.dllabel);
      this.serdataPanel.add(this.dlllabel);
      this.serdataPanel.add(new JLabel("Diluent Deck Position"));
      this.serdataPanel.add(getAddButton6());
      this.serdataPanel.add(this.sttext);
      this.serdataPanel.add(getSLiqType());
      this.serdataPanel.add(new JLabel(""));
      this.serdataPanel.add(new JLabel(""));
      this.serdataPanel.add(getDLiqType());
      this.serdataPanel.add(getSrc2());
//      this.serdataPanel.add(new JLabel(""));
      this.serdataPanel.add(getsPlateType4());
      this.serdataPanel.add(getUpdButton6());
    }
    return this.serdataPanel;
  }

  private JPanel getBrewPanel()
  {
    if (this.brewPanel == null) {
      this.brewPanel = new JPanel();
      this.brewPanel.setBackground(this.panelColor);
      this.brewPanel.setLayout(new BorderLayout());
      this.brewPanel.add(getBrewDataPanel(), "North");
      this.brewPanel.add(getBrewTrayPanel(), "Center");
    }
    return this.brewPanel;
  }

  private JPanel getBrewDataPanel()
  {
    if (this.brewdataPanel == null) {
      this.brewdataPanel = new JPanel();
      GridLayout gridLayout = new GridLayout();
      gridLayout.setRows(2);
      gridLayout.setColumns(5);
      gridLayout.setHgap(10);
      gridLayout.setVgap(3);
      this.brewdataPanel.setBackground(this.panelColor);
      this.brewdataPanel.setLayout(gridLayout);
      this.brewdataPanel.add(this.brwlabel);
      this.brewdataPanel.add(this.brvlabel);
      this.brewdataPanel.add(this.brltlabel);
      this.brewdataPanel.add(this.brnlabel);
      this.brewdataPanel.add(getAddButton7());
      this.brewdataPanel.add(this.brtext);
      this.brewdataPanel.add(this.brvol);
      this.brewdataPanel.add(getBLiqType());
      this.brewdataPanel.add(this.brntext);
      this.brewdataPanel.add(getUpdButton7());
    }

    return this.brewdataPanel;
  }

  private JPanel getWashDataPanel()
  {
    if (this.washdataPanel == null) {
      this.washdataPanel = new JPanel();
      GridLayout gridLayout = new GridLayout();
      gridLayout.setRows(1);
      gridLayout.setColumns(8);
      gridLayout.setHgap(10);
      gridLayout.setVgap(3);
      this.washdataPanel.setBackground(this.panelColor);
      this.washdataPanel.add(this.wmlabel);
      this.washdataPanel.add(this.wmethod);
      this.washdataPanel.add(getAddButton3());
      this.washdataPanel.add(getUpdButton3());

      this.washdataPanel.add(this.wlabel5);
    }
    return this.washdataPanel;
  }

  private JPanel getReadDataPanel()
  {
    if (this.readdataPanel == null) {
      this.readdataPanel = new JPanel();
      GridLayout gridLayout = new GridLayout();
      gridLayout.setRows(2);
      gridLayout.setColumns(5);
      gridLayout.setHgap(30);
      gridLayout.setVgap(3);
      this.readdataPanel.setBackground(this.panelColor);
      this.readdataPanel.setLayout(gridLayout);
      this.readdataPanel.add(this.relabel1);
      this.readdataPanel.add(this.relabel2);
      this.readdataPanel.add(this.relabel3);
      this.readdataPanel.add(this.templabel);
      this.readdataPanel.add(getAddButton4());
      this.readdataPanel.add(getRdr());
      this.readdataPanel.add(getReadButton());
      this.readdataPanel.add(this.text6);
      this.readdataPanel.add(this.temp);
      this.readdataPanel.add(getUpdButton4());
    }

    return this.readdataPanel;
  }

  private JPanel getCherryDataPanel()
  {
    if (this.cherrydataPanel == null) {
      this.cherrydataPanel = new JPanel();
      GridLayout gridLayout = new GridLayout();
      gridLayout.setRows(6);
      gridLayout.setColumns(7);
      gridLayout.setHgap(30);
      gridLayout.setVgap(3);
      this.cherrydataPanel.setBackground(this.panelColor);
      this.cherrydataPanel.setLayout(gridLayout);
      this.cherrydataPanel.add(this.chlabel5);
      this.cherrydataPanel.add(this.chlabel1);
      this.cherrydataPanel.add(this.text7);
      this.cherrydataPanel.add(getWorklistButton());
      this.cherrydataPanel.add(this.chlabel6);
      this.cherrydataPanel.add(this.chlabel7);
      this.cherrydataPanel.add(getAddButton11());

      this.cherrydataPanel.add(this.chlabel26);
      this.cherrydataPanel.add(this.chlabel2);
      this.cherrydataPanel.add(getTrack1());
      this.cherrydataPanel.add(this.chlabel27);
      this.cherrydataPanel.add(this.chlabel28);
      this.cherrydataPanel.add(this.chlabel29);
      this.cherrydataPanel.add(getUpdButton11());

      this.cherrydataPanel.add(this.chlabel8);
      this.cherrydataPanel.add(this.chlabelto);
      this.cherrydataPanel.add(getTrack2());
      this.cherrydataPanel.add(this.chlabel30);
      this.cherrydataPanel.add(this.chlabel31);
      this.cherrydataPanel.add(this.chlabel9);
      this.cherrydataPanel.add(this.chlabel32);

      this.cherrydataPanel.add(this.chlabel10);
      this.cherrydataPanel.add(this.chlabel3);
      this.cherrydataPanel.add(getCherrySrc());
      this.cherrydataPanel.add(this.chlabel11);
      this.cherrydataPanel.add(this.chlabel12);
      this.cherrydataPanel.add(this.chlabel13);
      this.cherrydataPanel.add(this.chlabel14);
      this.cherrydataPanel.add(this.chlabel15);
      this.cherrydataPanel.add(this.chlabel4);
      this.cherrydataPanel.add(getCherryDest());
      this.cherrydataPanel.add(this.chlabel16);
      this.cherrydataPanel.add(this.chlabel17);
      this.cherrydataPanel.add(this.chlabel18);
      this.cherrydataPanel.add(this.chlabel19);
      this.cherrydataPanel.add(this.chlabel21);
      this.cherrydataPanel.add(this.chlabel20);
      this.cherrydataPanel.add(getCherryLtyp());
      this.cherrydataPanel.add(this.chlabel22);
      this.cherrydataPanel.add(this.chlabel23);
      this.cherrydataPanel.add(this.chlabel24);
      this.cherrydataPanel.add(this.chlabel25);
    }

    return this.cherrydataPanel;
  }
  private JPanel getPKDataPanel()
  {
    if (this.pkdataPanel == null) {
		GridBagLayout gridBagLayout = new GridBagLayout();
		gridBagLayout.columnWidths = new int[]{0, 98, 0, 0, 24};
		gridBagLayout.rowHeights = new int[]{0, 0, 0};
		gridBagLayout.columnWeights = new double[]{0.0, 0.0, 0.0, 0.0, Double.MIN_VALUE};
		gridBagLayout.rowWeights = new double[]{0.0, 0.0, Double.MIN_VALUE};
		setLayout(gridBagLayout);
//		this.setBackground( panelColor );
		
		JLabel lblNewLabel = new JLabel("Worklist:");
		GridBagConstraints gbc_lblNewLabel = new GridBagConstraints();
		gbc_lblNewLabel.insets = new Insets(0, 0, 5, 5);
		gbc_lblNewLabel.anchor = GridBagConstraints.EAST;
		gbc_lblNewLabel.gridx = 0;
		gbc_lblNewLabel.gridy = 0;
		add(lblNewLabel, gbc_lblNewLabel);
		
		pkm_text_wat = new JTextField();
		GridBagConstraints gbc_textField = new GridBagConstraints();
		gbc_textField.anchor = GridBagConstraints.WEST;
		gbc_textField.insets = new Insets(0, 0, 5, 5);
		gbc_textField.gridx = 1;
		gbc_textField.gridy = 0;
		add(pkm_text_wat, gbc_textField);
		pkm_text_wat.setColumns(13);
		
		JButton btnNewButton = new JButton("Browse");
		btnNewButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
		          watsonFile();
			}
		});
		btnNewButton.setHorizontalAlignment(SwingConstants.LEFT);
		GridBagConstraints gbc_btnNewButton = new GridBagConstraints();
		gbc_btnNewButton.anchor = GridBagConstraints.WEST;
		gbc_btnNewButton.insets = new Insets(0, 0, 5, 5);
		gbc_btnNewButton.gridx = 2;
		gbc_btnNewButton.gridy = 0;
		add(btnNewButton, gbc_btnNewButton);
		
		btnAdd.setHorizontalAlignment(SwingConstants.RIGHT);
		GridBagConstraints gbc_btnAdd = new GridBagConstraints();
		gbc_btnAdd.insets = new Insets(0, 0, 5, 0);
		gbc_btnAdd.gridx = 7;
		gbc_btnAdd.gridy = 0;
		add(btnAdd, gbc_btnAdd);
		
		btnUpdateStep.setHorizontalAlignment(SwingConstants.RIGHT);
		GridBagConstraints gbc_btnUpdateStep = new GridBagConstraints();
		gbc_btnUpdateStep.gridx = 7;
		gbc_btnUpdateStep.gridy = 1;
		add(btnUpdateStep, gbc_btnUpdateStep);

    }

    return this.pkdataPanel;
  }

  private JPanel getPKDataPanel1()
  {
    if (this.pkdataPanel1 == null) {
        this.pkdataPanel1 = new JPanel();
		GridBagLayout gridBagLayout = new GridBagLayout();
		gridBagLayout.columnWidths = new int[]{0, 98, 0, 0, 24};
		gridBagLayout.rowHeights = new int[]{0, 0, 0};
		gridBagLayout.columnWeights = new double[]{0.0, 0.0, 0.0, 0.0, Double.MIN_VALUE};
		gridBagLayout.rowWeights = new double[]{0.0, 0.0, Double.MIN_VALUE};
		this.pkdataPanel1.setLayout(gridBagLayout);
		this.pkdataPanel1.setBackground(this.panelColor);
//		this.setBackground( panelColor );	
//		JLabel lblNewLabel = new JLabel("     LIMS / LIS FILE:");
		JLabel lblNewLabel = new JLabel("                     ");
		GridBagConstraints gbc_lblNewLabel = new GridBagConstraints();
		gbc_lblNewLabel.insets = new Insets(0, 0, 5, 5);
		gbc_lblNewLabel.anchor = GridBagConstraints.EAST;
		gbc_lblNewLabel.gridx = 0;
		gbc_lblNewLabel.gridy = 0;
		this.pkdataPanel1.add(lblNewLabel, gbc_lblNewLabel);
		
		JLabel lblNewLabel_1 = new JLabel("                     ");
		GridBagConstraints gbc_lblNewLabel_1 = new GridBagConstraints();
		gbc_lblNewLabel_1.insets = new Insets(0, 0, 5, 5);
		gbc_lblNewLabel_1.anchor = GridBagConstraints.EAST;
		gbc_lblNewLabel_1.gridx = 1;
		gbc_lblNewLabel_1.gridy = 0;
		this.pkdataPanel1.add(lblNewLabel_1, gbc_lblNewLabel_1);

		JLabel lblNewLabel_2 = new JLabel("                     ");
		GridBagConstraints gbc_lblNewLabel_2 = new GridBagConstraints();
		gbc_lblNewLabel_2.insets = new Insets(0, 0, 5, 5);
		gbc_lblNewLabel_2.anchor = GridBagConstraints.EAST;
		gbc_lblNewLabel_2.gridx = 2;
		gbc_lblNewLabel_2.gridy = 0;
		this.pkdataPanel1.add(lblNewLabel_2, gbc_lblNewLabel_2);

//		pkm_text_wat = new JTextField();
//		GridBagConstraints gbc_textField = new GridBagConstraints();
//		gbc_textField.anchor = GridBagConstraints.WEST;
//		gbc_textField.insets = new Insets(0, 0, 5, 5);
//		gbc_textField.gridx = 1;
//		gbc_textField.gridy = 0;
//		this.pkdataPanel1.add(pkm_text_wat, gbc_textField);
//		pkm_text_wat.setColumns(40);
		
//		JButton btnNewButton = new JButton("Browse");
//		btnNewButton.setToolTipText("Browse to select your LIMS / LIS File");
//		btnNewButton.addActionListener(new ActionListener() {
//			public void actionPerformed(ActionEvent arg0) {
//		          watsonFile();
//			}
//		});
//		btnNewButton.setHorizontalAlignment(SwingConstants.LEFT);
//		GridBagConstraints gbc_btnNewButton = new GridBagConstraints();
//		gbc_btnNewButton.anchor = GridBagConstraints.WEST;
//		gbc_btnNewButton.insets = new Insets(0, 0, 5, 5);
//		gbc_btnNewButton.gridx = 2;
//		gbc_btnNewButton.gridy = 0;
//		this.pkdataPanel1.add(btnNewButton, gbc_btnNewButton);
		
//		JButton btnAdd = new JButton("  Add Step  ");
		btnAdd.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
		    	  mrdon = pkdataPanel2x.getMRDon();
		    	  dilon = pkdataPanel2x.getDILon();
		    	  preassayon = pkdataPanel2x.getPRAon();
		    	  assayon = pkdataPanel2x.getASSon();
				if (pkCheckInputs())
		          {
		            TabWindow.this.addPK();
		            pkdataPanel2x.resetScreen();
		          }
		          else
		          {
//		        	JOptionPane.showMessageDialog(null, "Incomplete PK data, Cannot add PK step", " ", 2); 
		          }
			}
		});
		
		btnAdd.setHorizontalAlignment(SwingConstants.RIGHT);
		GridBagConstraints gbc_btnAdd = new GridBagConstraints();
		gbc_btnAdd.insets = new Insets(0, 0, 5, 0);
		gbc_btnAdd.gridx = 7;
		gbc_btnAdd.gridy = 0;
		this.pkdataPanel1.add(btnAdd, gbc_btnAdd);
		
//		JButton btnUpdateStep = new JButton("Update Step");
		btnUpdateStep.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
		    	  mrdon = pkdataPanel2x.getMRDon();
		    	  dilon = pkdataPanel2x.getDILon();
		    	  preassayon = pkdataPanel2x.getPRAon();
		    	  assayon = pkdataPanel2x.getASSon();

		          if (pkCheckInputs())	  
		          {
//		            for (int d = 0; d < 15; d++) {TabWindow.this.dTextArea[d].setToolTipText("Click to edit the protocol\n\n" + TabWindow.this.dTextArea[d].getText());}
		        	TabWindow.this.updPK();
		            pkdataPanel2x.resetScreen();
		          }
		          else
		          {
//		        	JOptionPane.showMessageDialog(null, "Incomplete PK data, Cannot add PK step", " ", 2); 
		          }
		          
		          
			}
		});
		btnUpdateStep.setHorizontalAlignment(SwingConstants.RIGHT);
		GridBagConstraints gbc_btnUpdateStep = new GridBagConstraints();
		gbc_btnUpdateStep.gridx = 7;
		gbc_btnUpdateStep.gridy = 1;
		btnUpdateStep.setEnabled(false);
		this.pkdataPanel1.add(btnUpdateStep, gbc_btnUpdateStep);
    }
    return this.pkdataPanel1;
  } 

  private boolean pkCheckInputs(){
	  boolean returnVal = true;
	  int num;
	  
	  ///Check LIMS file.  Make sure it is there, can be accessed, and contains only singlet or duplicate samples
		int count = 0;
		String newline; String tabChar = "\t";
	    ArrayList<String> arraylistPK_SampleIds_All = new ArrayList<String>();		//All Sample IDs including repeats
		File f = new File(pkdataPanel2x.getpkmtextwat());

	    try {
	    	FileReader inputWatson = new FileReader(f);
	        BufferedReader br0 = new BufferedReader(inputWatson);
			br0.readLine();
		    br0.close();
		} catch (IOException e) {
			JOptionPane.showMessageDialog(null, "Cannot process the selected file");		
			returnVal = false;
			e.printStackTrace();
		}
	    
	  /////////////////////////////////  
	  //Confirm check boxes and inputs  
      if (returnVal){
    	  if (
	    	TabWindow.this.pkdataPanel2x.getpkmtextwat().trim().equals("") 
	    	||(!this.mrdon && !this.dilon && !this.preassayon && !this.assayon)
	    	||(this.mrdon && (TabWindow.this.pkdataPanel2x.getpkmtext1().trim().equals("") 
	    	|| TabWindow.this.pkdataPanel2x.getpkmrdsrc().trim().equals("0") || TabWindow.this.pkdataPanel2x.getpkssrc().trim().equals("0")
	    	|| TabWindow.this.pkdataPanel2x.getpkstdsrc().trim().equals("0") || TabWindow.this.pkdataPanel2x.getpkmmrddil().trim().equals("0")
	    	|| TabWindow.this.pkdataPanel2x.getpkmsamdil().trim().equals("0") || TabWindow.this.pkdataPanel2x.getpkmstddil().trim().equals("0")
	    	|| TabWindow.this.pkdataPanel2x.getpkmtextul().trim().equals("") || pkdataPanel2x.getpkdeck1() == "0"))
	    	
	    	||((!this.mrdon && this.dilon && this.preassayon && (pkdataPanel2x.getpkssrc() == "0" 
	    	|| TabWindow.this.pkdataPanel2x.getpkstdsrc().trim().equals("0"))) ) 
	    	
	    	||((this.dilon && (pkdataPanel2x.getpkdeck2() == "0" 
	    	|| TabWindow.this.pkdataPanel2x.getpkdil().trim().equals("0") 
	    	|| TabWindow.this.pkdataPanel2x.getpkdilres().trim().equals("0")))  )
	    	
	    	||(this.preassayon && (TabWindow.this.pkdataPanel2x.getpkmpassayul().trim().equals("")  
	    	|| TabWindow.this.pkdataPanel2x.getpassaylab().trim().equals("0")) )
	    	
	    	||(this.assayon && (TabWindow.this.pkdataPanel2x.getpkmassayul().trim().equals("") 
	    	|| TabWindow.this.pkdataPanel2x.getassaylab().trim().equals("0")) )
	    	)
    	 	{
    	  JOptionPane.showMessageDialog(null, "Input for the PK step is incomplete!");
    	  returnVal = false;
    	 	} 
      }
      	

      /////////////////////////////
      // are they integers?!
	  try{
		  if (this.mrdon){
			  num = Integer.parseInt(TabWindow.this.pkdataPanel2x.getpkmtextul().trim());
//			  num = Integer.parseInt(TabWindow.this.pkm_text_ul.getText().trim());
			  num = Integer.parseInt(TabWindow.this.pkdataPanel2x.getpkmtextul().trim());
			  }
		  
		  if (this.preassayon){num = Integer.parseInt(TabWindow.this.pkdataPanel2x.getpkmpassayul().trim());}
		  if (this.assayon){num = Integer.parseInt(TabWindow.this.pkdataPanel2x.getpkmassayul().trim());}
		  
		} catch (NumberFormatException e) {
			JOptionPane.showMessageDialog(null, "Integer values expected as input");		
			returnVal = false;
		}
	
	  try {
		  if (this.mrdon){
			if (!(Integer.parseInt(TabWindow.this.pkdataPanel2x.getpkmtext1().trim()) > 0)){  JOptionPane.showMessageDialog(null, "Dilution and Volume must be greater than zero");  returnVal = false;}		//APF0025  Added Line
//			if (!(Integer.parseInt(TabWindow.this.pkm_text_ul.getText().trim()) > 0)){  JOptionPane.showMessageDialog(null, "Dilution and Volume must be greater than zero");  returnVal = false;}	//APF0025  Added Line  
			if (!(Integer.parseInt(TabWindow.this.pkdataPanel2x.getpkmtextul().trim()) > 0)){  JOptionPane.showMessageDialog(null, "Dilution and Volume must be greater than zero");  returnVal = false;}	//APF0025  Added Line  
		  }
	} catch (NumberFormatException e) {
		// TODO Auto-generated catch block
		e.printStackTrace();
	} catch (HeadlessException e) {
		// TODO Auto-generated catch block
		e.printStackTrace();
	}
	  
	  try {
		  if (!this.mrdon && preassayon){
			if ((Integer.parseInt(TabWindow.this.pkdataPanel2x.getpkmtext1().trim()) > 1 && TabWindow.this.pkdataPanel2x.getpkdilsrc1().trim().equals("0") ))
				{  JOptionPane.showMessageDialog(null, "Input for the PK step is incomplete!");
				   returnVal = false;
				}		//APF0025  Added Line
		  }
	} catch (NumberFormatException e) {
		// TODO Auto-generated catch block
		e.printStackTrace();
	} catch (HeadlessException e) {
		// TODO Auto-generated catch block
		e.printStackTrace();
	}
	  
	  if (returnVal){															//PKF0033
		  if (!VAStools.PK_WorklistConfirm(pkm_text_wat.getText())){			//PKF0033
			returnVal = false;													//PKF0033
		  }																		//PKF0033
	  }																			//PKF0033
	  
	  return returnVal;
  }

  private PanelPK1 getPKDataPanel2x()
  {
	  if (this.pkdataPanel2x == null) {
        this.pkdataPanel2x = new PanelPK1(plates96, tubesAll, deckpositions, reservoirsAll, liqtypes);
        this.pkdataPanel2x.setBackground(this.panelColor);
        this.pkdataPanel2x.setSize(650, 424);
        this.pkdataPanel2x.setAlignmentX(Component.LEFT_ALIGNMENT);
	}
	 return pkdataPanel2x;
  }
 
  // Create the PK input fields
  private JPanel getPKDataPanel2()
  {
    if (this.pkdataPanel2 == null) {
      this.pkdataPanel2 = new JPanel();
		GridBagLayout gridBagLayout = new GridBagLayout();
		gridBagLayout.columnWidths = new int[]{0, 0, 0, 0, 0, 30, 30, 30, 30, 30, 50, 0, 100, 100, 100, 20};
		gridBagLayout.rowHeights = new int[]{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
		gridBagLayout.columnWeights = new double[]{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,Double.MIN_VALUE};
		gridBagLayout.rowWeights = new double[]{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, Double.MIN_VALUE};
		this.pkdataPanel2.setLayout(gridBagLayout);
		this.pkdataPanel2.setBackground(this.panelColor);
		
		JLabel lblProcess = new JLabel("   Process");
		lblProcess.setFont(font15);
		GridBagConstraints gbc_lblProcess = new GridBagConstraints();
		gbc_lblProcess.insets = new Insets(0, 0, 5, 5);
		gbc_lblProcess.gridx = 0;
		gbc_lblProcess.gridy = 0;
		this.pkdataPanel2.add(lblProcess, gbc_lblProcess);
		
		JLabel lblLabware = new JLabel("Labware");
		lblLabware.setFont(font15);
		GridBagConstraints gbc_lblLabware = new GridBagConstraints();
		gbc_lblLabware.insets = new Insets(0, 0, 5, 5);
		gbc_lblLabware.gridx = 12;
		gbc_lblLabware.gridy = 0;
		this.pkdataPanel2.add(lblLabware, gbc_lblLabware);
		
		JLabel lblLiquid = new JLabel("Liquid");
		lblLiquid.setFont(font15);
		GridBagConstraints gbc_lblLiquid = new GridBagConstraints();
		gbc_lblLiquid.insets = new Insets(0, 0, 5, 0);
		gbc_lblLiquid.gridx = 13;
		gbc_lblLiquid.gridy = 0;
		this.pkdataPanel2.add(lblLiquid, gbc_lblLiquid);
		
//		lblPosition.setVisible(false) ;
		lblPosition.setFont(font15);
		GridBagConstraints gbc_lblPosition = new GridBagConstraints();
		gbc_lblLabware.insets = new Insets(0, 0, 5, 5);
		gbc_lblLabware.gridx = 14;
		gbc_lblLabware.gridy = 0;
		this.pkdataPanel2.add(lblPosition, gbc_lblPosition);
		
		JLabel lblNewLabel_0 = new JLabel(" ");
		GridBagConstraints gbc_lblNewLabel_0 = new GridBagConstraints();
		gbc_lblNewLabel_0.insets = new Insets(0, 0, 5, 5);
		gbc_lblNewLabel_0.gridx = 1;
		gbc_lblNewLabel_0.gridy = 1;
		this.pkdataPanel2.add(lblNewLabel_0, gbc_lblNewLabel_0);
		
		JLabel lblMrd = new JLabel("     MRD");
		GridBagConstraints gbc_lblMrd = new GridBagConstraints();
		gbc_lblMrd.anchor = GridBagConstraints.WEST;
		gbc_lblMrd.insets = new Insets(0, 0, 5, 5);
		gbc_lblMrd.gridx = 0;
		gbc_lblMrd.gridy = 2;
		this.pkdataPanel2.add(lblMrd, gbc_lblMrd);
		
		GridBagConstraints gbc_checkBox_3 = new GridBagConstraints();
		gbc_checkBox_3.insets = new Insets(0, 0, 5, 5);
		gbc_checkBox_3.gridx = 1;
		gbc_checkBox_3.gridy = 2;
		this.pkdataPanel2.add(getPKCheck1(), gbc_checkBox_3);
		
		JLabel label_5 = new JLabel(" ");
		GridBagConstraints gbc_label_5 = new GridBagConstraints();
		gbc_label_5.anchor = GridBagConstraints.EAST;
		gbc_label_5.insets = new Insets(0, 0, 5, 5);
		gbc_label_5.gridx = 2;
		gbc_label_5.gridy = 2;
		this.pkdataPanel2.add(label_5, gbc_label_5);
		
		JLabel label_1 = new JLabel("1:");
		label_1.setFont(font15);
		GridBagConstraints gbc_label_1 = new GridBagConstraints();
		gbc_label_1.anchor = GridBagConstraints.EAST;
		gbc_label_1.insets = new Insets(0, 0, 5, 5);
		gbc_label_1.gridx = 6;
		gbc_label_1.gridy = 2;
		this.pkdataPanel2.add(label_1, gbc_label_1);
		
		pkm_text_1 = new JTextField();
		GridBagConstraints gbc_textField = new GridBagConstraints();
		gbc_textField.anchor = GridBagConstraints.EAST;
		gbc_textField.fill = GridBagConstraints.HORIZONTAL;
		gbc_textField.insets = new Insets(0, 0, 5, 5);
		gbc_textField.gridx = 7;
		gbc_textField.gridy = 2;
        this.pkm_text_1.setEnabled(false);
		this.pkdataPanel2.add(pkm_text_1, gbc_textField);
		pkm_text_1.setColumns(2);
		

		JLabel label_2 = new JLabel("in");
		GridBagConstraints gbc_label_2 = new GridBagConstraints();
		gbc_label_2.anchor = GridBagConstraints.CENTER;
		gbc_label_2.insets = new Insets(0, 0, 5, 5);
		gbc_label_2.gridx = 8;
		gbc_label_2.gridy = 2;
		this.pkdataPanel2.add(label_2, gbc_label_2);
		
		pkm_text_ul = new JTextField();
		GridBagConstraints gbc_textField_1 = new GridBagConstraints();
		gbc_textField_1.anchor = GridBagConstraints.EAST;
		gbc_textField_1.fill = GridBagConstraints.HORIZONTAL;
		gbc_textField_1.insets = new Insets(0, 0, 5, 5);
		gbc_textField_1.gridx = 9;
		gbc_textField_1.gridy = 2;
        this.pkm_text_ul.setEnabled(false);
		this.pkdataPanel2.add(pkm_text_ul, gbc_textField_1);
		pkm_text_ul.setColumns(5);
		
		JLabel lblUl = new JLabel("ul");
		GridBagConstraints gbc_lblUl = new GridBagConstraints();
		gbc_lblUl.anchor = GridBagConstraints.WEST;
		gbc_lblUl.insets = new Insets(0, 0, 5, 5);
		gbc_lblUl.gridx = 10;
		gbc_lblUl.gridy = 2;
		this.pkdataPanel2.add(lblUl, gbc_lblUl);
		
		GridBagConstraints gbc_comboBox_4 = new GridBagConstraints();
		gbc_comboBox_4.insets = new Insets(0, 0, 5, 5);
		gbc_comboBox_4.gridx = 12;
		gbc_comboBox_4.gridy = 2;
		this.pkdataPanel2.add(getPKMRDSrc(), gbc_comboBox_4);
		
		pklabel70.setVisible(false);     //"(diluent")
		GridBagConstraints gbc_pk_liqNote_1 = new GridBagConstraints();
		gbc_pk_liqNote_1.anchor = GridBagConstraints.WEST;
		gbc_pk_liqNote_1.insets = new Insets(0, 0, 5, 5);
		gbc_pk_liqNote_1.gridx = 15;
		gbc_pk_liqNote_1.gridy = 2;
		this.pkdataPanel2.add(pklabel70, gbc_pk_liqNote_1);  
		
		//JLabel lblSamples = new JLabel(" Samples");
//		lblSamples.setVisible(false);
		GridBagConstraints gbc_lblSamples = new GridBagConstraints();
		gbc_lblSamples.anchor = GridBagConstraints.EAST;
		gbc_lblSamples.insets = new Insets(0, 0, 5, 5);
		gbc_lblSamples.gridx = 9;
		gbc_lblSamples.gridy = 3;
		this.pkdataPanel2.add(lblSamples, gbc_lblSamples);
		
		GridBagConstraints gbc_comboBox = new GridBagConstraints();
		gbc_comboBox.insets = new Insets(0, 0, 5, 5);
		gbc_comboBox.gridx = 12;
		gbc_comboBox.gridy = 3;
		this.pkdataPanel2.add(getSSrc(), gbc_comboBox);

		String[] ltypes = TabWindow.returnltypes();

		GridBagConstraints gbc_comboBox_2 = new GridBagConstraints();
		gbc_comboBox_2.insets = new Insets(0, 0, 5, 0);
		gbc_comboBox_2.gridx = 13;
		gbc_comboBox_2.gridy = 3;
		this.pkdataPanel2.add(getPKSAMDil(), gbc_comboBox_2);
		
		//JLabel lblStandards = new JLabel(" Standards");
//		lblStandards.setVisible(false);
		GridBagConstraints gbc_lblStandards = new GridBagConstraints();
		gbc_lblStandards.anchor = GridBagConstraints.EAST;
		gbc_lblStandards.insets = new Insets(0, 0, 5, 5);
		gbc_lblStandards.gridx = 9;
		gbc_lblStandards.gridy = 4;
		this.pkdataPanel2.add(lblStandards, gbc_lblStandards);
		
		GridBagConstraints gbc_comboBox_1 = new GridBagConstraints();
		gbc_comboBox_1.insets = new Insets(0, 0, 5, 5);
		gbc_comboBox_1.gridx = 12;
		gbc_comboBox_1.gridy = 4;
		this.pkdataPanel2.add(getPKSTDSrc(), gbc_comboBox_1);
		
		JComboBox comboBox_3 = new JComboBox(TabWindow.returnltypes());
		GridBagConstraints gbc_comboBox_3 = new GridBagConstraints();
		gbc_comboBox_3.insets = new Insets(0, 0, 5, 0);
		gbc_comboBox_3.gridx = 13;
		gbc_comboBox_3.gridy = 4;
		this.pkdataPanel2.add(getPKSTDDil(), gbc_comboBox_3);
				
		JLabel lblNewLabel_1 = new JLabel(" ");
		GridBagConstraints gbc_lblNewLabel_1 = new GridBagConstraints();
		gbc_lblNewLabel_1.insets = new Insets(0, 0, 5, 5);
		gbc_lblNewLabel_1.gridx = 1;
		gbc_lblNewLabel_1.gridy = 5;
		this.pkdataPanel2.add(lblNewLabel_1, gbc_lblNewLabel_1);
		
		//JLabel lblDiluent1 = new JLabel(" Diluents:");
//		lblDiluent1.setVisible(false);
		GridBagConstraints gbc_lblDiluent1 = new GridBagConstraints();
		gbc_lblDiluent1.anchor = GridBagConstraints.EAST;
		gbc_lblDiluent1.insets = new Insets(0, 0, 5, 5);
		gbc_lblDiluent1.gridx = 9;
		gbc_lblDiluent1.gridy = 5;
		this.pkdataPanel2.add(lblDiluent1, gbc_lblDiluent1);
		
		GridBagConstraints gbc_comboBox_12 = new GridBagConstraints();
		gbc_comboBox_12.fill = GridBagConstraints.HORIZONTAL;
		gbc_comboBox_12.insets = new Insets(0, 0, 5, 5);
		gbc_comboBox_12.gridx = 12;
		gbc_comboBox_12.gridy = 5;
		this.pkdataPanel2.add(getPKDILSrc(), gbc_comboBox_12);
		
		GridBagConstraints gbc_comboBox_5 = new GridBagConstraints();
		gbc_comboBox_5.insets = new Insets(0, 0, 5, 0);
		gbc_comboBox_5.gridx = 13;
		gbc_comboBox_5.gridy = 5;
		this.pkdataPanel2.add(getMRDDil(), gbc_comboBox_5);
		
		GridBagConstraints gbc_comboBox_10 = new GridBagConstraints();
		gbc_comboBox_10.insets = new Insets(0, 0, 5, 5);
		gbc_comboBox_10.gridx = 14;
		gbc_comboBox_10.gridy = 5;
		this.pkdataPanel2.add(getPKDeck1(), gbc_comboBox_10);
		
		JLabel lblNewLabel_12 = new JLabel(" ");
		GridBagConstraints gbc_lblNewLabel_12 = new GridBagConstraints();
		gbc_lblNewLabel_12.insets = new Insets(0, 0, 5, 5);
		gbc_lblNewLabel_12.gridx = 0;
		gbc_lblNewLabel_12.gridy = 6;
		this.pkdataPanel2.add(lblNewLabel_12, gbc_lblNewLabel_12);
		
		JLabel lblDilution = new JLabel("     Dilution");
		GridBagConstraints gbc_lblDilution = new GridBagConstraints();
		gbc_lblDilution.anchor = GridBagConstraints.WEST;
		gbc_lblDilution.insets = new Insets(0, 0, 5, 5);
		gbc_lblDilution.gridx = 0;
		gbc_lblDilution.gridy = 7;
		this.pkdataPanel2.add(lblDilution, gbc_lblDilution);
		
		GridBagConstraints gbc_checkBox = new GridBagConstraints();
		gbc_checkBox.insets = new Insets(0, 0, 5, 5);
		gbc_checkBox.gridx = 1;
		gbc_checkBox.gridy = 7;
		this.pkdataPanel2.add(getPKCheck2(), gbc_checkBox);
		
		GridBagConstraints gbc_comboBox_6 = new GridBagConstraints();
		gbc_comboBox_6.insets = new Insets(0, 0, 5, 5);
		gbc_comboBox_6.gridx = 12;
		gbc_comboBox_6.gridy = 7;
		this.pkdataPanel2.add(getDSrc(), gbc_comboBox_6);
		
		JLabel lblNewLabel_2 = new JLabel(" ");
		GridBagConstraints gbc_lblNewLabel_2 = new GridBagConstraints();
		gbc_lblNewLabel_2.insets = new Insets(0, 0, 5, 5);
		gbc_lblNewLabel_2.gridx = 1;
		gbc_lblNewLabel_2.gridy = 8;
		this.pkdataPanel2.add(lblNewLabel_2, gbc_lblNewLabel_2);
		
		//JLabel lblDiluent2 = new JLabel(" Diluents:");
//		lblDiluent2.setVisible(false);
		GridBagConstraints gbc_lblDiluent2 = new GridBagConstraints();
		gbc_lblDiluent2.anchor = GridBagConstraints.EAST;
		gbc_lblDiluent2.insets = new Insets(0, 0, 5, 5);
		gbc_lblDiluent2.gridx = 9;
		gbc_lblDiluent2.gridy = 8;
		this.pkdataPanel2.add(lblDiluent2, gbc_lblDiluent2);
		
		GridBagConstraints gbc_comboBox_13 = new GridBagConstraints();
		gbc_comboBox_13.fill = GridBagConstraints.HORIZONTAL;
		gbc_comboBox_13.insets = new Insets(0, 0, 5, 5);
		gbc_comboBox_13.gridx = 12;
		gbc_comboBox_13.gridy = 8;
		this.pkdataPanel2.add(getPKDILRes(), gbc_comboBox_13);
		
		GridBagConstraints gbc_comboBox_7 = new GridBagConstraints();
		gbc_comboBox_7.insets = new Insets(0, 0, 5, 0);
		gbc_comboBox_7.gridx = 13;
		gbc_comboBox_7.gridy = 8;
		this.pkdataPanel2.add(getPKDil(), gbc_comboBox_7);
		
		GridBagConstraints gbc_comboBox_11 = new GridBagConstraints();
		gbc_comboBox_11.insets = new Insets(0, 0, 5, 5);
		gbc_comboBox_11.gridx = 14;
		gbc_comboBox_11.gridy = 8;
		this.pkdataPanel2.add(getPKDeck2(), gbc_comboBox_11);
		
		pklabel71.setVisible(false);     //"(diluent")
		GridBagConstraints gbc_pk_liqNote_2 = new GridBagConstraints();
		gbc_pk_liqNote_2.anchor = GridBagConstraints.WEST;
		gbc_pk_liqNote_2.insets = new Insets(0, 0, 5, 5);
		gbc_pk_liqNote_2.gridx = 15;
		gbc_pk_liqNote_2.gridy = 8;
		this.pkdataPanel2.add(pklabel71, gbc_pk_liqNote_2);  
		
		JLabel lblNewLabel_5 = new JLabel(" ");
		GridBagConstraints gbc_lblNewLabel_5 = new GridBagConstraints();
		gbc_lblNewLabel_5.insets = new Insets(0, 0, 5, 5);
		gbc_lblNewLabel_5.gridx = 1;
		gbc_lblNewLabel_5.gridy = 9;
		this.pkdataPanel2.add(lblNewLabel_5, gbc_lblNewLabel_5);
		
		JLabel lblNewLabel = new JLabel("     Pre-Assay ");
		GridBagConstraints gbc_lblNewLabel = new GridBagConstraints();
		gbc_lblNewLabel.insets = new Insets(0, 0, 5, 5);
		gbc_lblNewLabel.gridx = 0;
		gbc_lblNewLabel.gridy = 10;
		this.pkdataPanel2.add(lblNewLabel, gbc_lblNewLabel);
		
		GridBagConstraints gbc_checkBox_1 = new GridBagConstraints();
		gbc_checkBox_1.insets = new Insets(0, 0, 5, 5);
		gbc_checkBox_1.gridx = 1;
		gbc_checkBox_1.gridy = 10;
		this.pkdataPanel2.add(getPKCheck3(), gbc_checkBox_1);
		this.pkcheck3.setEnabled(true);

		pkm_text_ul_preAssay = new JTextField();
		GridBagConstraints gbc_textField_preAssay = new GridBagConstraints();
		gbc_textField_preAssay.anchor = GridBagConstraints.EAST;
		gbc_textField_preAssay.fill = GridBagConstraints.HORIZONTAL;
		gbc_textField_preAssay.insets = new Insets(0, 0, 5, 5);
		gbc_textField_preAssay.gridx = 9;
		gbc_textField_preAssay.gridy = 10;
        this.pkm_text_ul_preAssay.setEnabled(false);
		this.pkdataPanel2.add(pkm_text_ul_preAssay, gbc_textField_preAssay);
		pkm_text_ul_preAssay.setColumns(5);
		
		JLabel lblUlPreAssay = new JLabel("ul");
		GridBagConstraints gbc_lblUlPreAssay = new GridBagConstraints();
		gbc_lblUlPreAssay.anchor = GridBagConstraints.WEST;
		gbc_lblUlPreAssay.insets = new Insets(0, 0, 5, 5);
		gbc_lblUlPreAssay.gridx = 10;
		gbc_lblUlPreAssay.gridy = 10;
		this.pkdataPanel2.add(lblUlPreAssay, gbc_lblUlPreAssay);
		
		GridBagConstraints gbc_comboBox_8 = new GridBagConstraints();
		gbc_comboBox_8.insets = new Insets(0, 0, 5, 5);
		gbc_comboBox_8.gridx = 12;
		gbc_comboBox_8.gridy = 10;
		this.pkdataPanel2.add(getPKPAssay(), gbc_comboBox_8);

		JLabel lblNewLabel_3 = new JLabel("");
		GridBagConstraints gbc_lblNewLabel_3 = new GridBagConstraints();
		gbc_lblNewLabel_3.insets = new Insets(0, 0, 5, 5);
		gbc_lblNewLabel_3.gridx = 1;
		gbc_lblNewLabel_3.gridy = 11;
		this.pkdataPanel2.add(lblNewLabel_3, gbc_lblNewLabel_3);
		
		JLabel lblAssay = new JLabel("     Assay");
		GridBagConstraints gbc_lblAssay = new GridBagConstraints();
		gbc_lblAssay.anchor = GridBagConstraints.WEST;
		gbc_lblAssay.insets = new Insets(0, 0, 0, 5);
		gbc_lblAssay.gridx = 0;
		gbc_lblAssay.gridy = 11;
		this.pkdataPanel2.add(lblAssay, gbc_lblAssay);
		
		GridBagConstraints gbc_checkBox_2 = new GridBagConstraints();
		gbc_checkBox_2.insets = new Insets(0, 0, 0, 5);
		gbc_checkBox_2.gridx = 1;
		gbc_checkBox_2.gridy = 11;
		this.pkdataPanel2.add(getPKCheck4(), gbc_checkBox_2);
		this.pkcheck4.setEnabled(false);

		pkm_text_ul_Assay = new JTextField();
		GridBagConstraints gbc_textField_Assay = new GridBagConstraints();
		gbc_textField_Assay.anchor = GridBagConstraints.EAST;
		gbc_textField_Assay.fill = GridBagConstraints.HORIZONTAL;
		gbc_textField_Assay.insets = new Insets(0, 0, 5, 5);
		gbc_textField_Assay.gridx = 9;
		gbc_textField_Assay.gridy = 11;
        this.pkm_text_ul_Assay.setEnabled(false);
		this.pkdataPanel2.add(pkm_text_ul_Assay, gbc_textField_Assay);
		pkm_text_ul_Assay.setColumns(5);
		
		JLabel lblUlAssay = new JLabel("ul");
		GridBagConstraints gbc_lblUlAssay = new GridBagConstraints();
		gbc_lblUlAssay.anchor = GridBagConstraints.WEST;
		gbc_lblUlAssay.insets = new Insets(0, 0, 5, 5);
		gbc_lblUlAssay.gridx = 10;
		gbc_lblUlAssay.gridy = 11;
		this.pkdataPanel2.add(lblUlAssay, gbc_lblUlAssay);
		
		GridBagConstraints gbc_comboBox_9 = new GridBagConstraints();
		gbc_comboBox_9.insets = new Insets(0, 0, 0, 5);
		gbc_comboBox_9.gridx = 12;
		gbc_comboBox_9.gridy = 11;
		this.pkdataPanel2.add(getPKAssay(), gbc_comboBox_9);

    }

    return this.pkdataPanel2;
  } 

  
  private JPanel getSplashPanel()
  {
    if (this.SplashPanel == null) {
      this.SplashPanel = new JPanel()
		{
          Color color1 = steelDrkColor;
          Color color2 = color1.darker();
          @Override
          protected void paintComponent( Graphics g ) {
              if ( !isOpaque( ) )
	                {
	                    super.paintComponent( g );
	                    //return;
	                }
             Graphics2D g2d = (Graphics2D)g;
            
              int w = getWidth( );
              int h = getHeight( );
               
              // Paint a gradient from top to bottom
              GradientPaint gp = new GradientPaint(
                  0, h, color1,
                  0, 0, color2 );

              g2d.setPaint( gp );
              g2d.fillRect( 0, 0, w, h );
              ////
           
              SplashPanel.setOpaque( false );
              Image castle = new ImageIcon("C:\\Program Files\\Telos\\AutoPilot\\Config\\Splash.jpg").getImage();
	            g2d.drawImage(castle, 0, 0, getWidth(), getHeight(), this);
              super.paintComponent( g );
              //aliqPanel.setOpaque( true );
          };
	};
      this.SplashPanel.setBackground(this.panelColor);
      //this.SplashPanel.setBorder(clearline);

      this.SplashPanel.setLayout(new FlowLayout());
      //this.coverPanel.add(new JLabel("YO"));
      //this.pausePanel.add(getUpdButton9());
    }
    return this.SplashPanel;
  }
  
  private JPanel getSPEDataPanel() {
    if (this.spedataPanel == null) {
      this.spedataPanel = new JPanel();
      GridLayout gridLayout = new GridLayout();
      gridLayout.setRows(4);
      gridLayout.setColumns(5);
      gridLayout.setHgap(30);
      gridLayout.setVgap(3);
      this.spedataPanel.setLayout(gridLayout);
      this.spedataPanel.setBackground(this.panelColor);
      this.spedataPanel.add(getAssemButton());
      this.spedataPanel.add(this.spelabel1);
      this.spedataPanel.add(this.spelabel2);
      this.spedataPanel.add(this.spelabel6);
      this.spedataPanel.add(getAddButton5());
      this.spedataPanel.add(getDisAssemButton());
      this.spedataPanel.add(getFilter());
      this.spedataPanel.add(getCollection());
      this.spedataPanel.add(this.spelabel7);
      this.spedataPanel.add(getUpdButton5());

      this.spedataPanel.add(getVacButton());
      this.spedataPanel.add(this.spelabel3);
      this.spedataPanel.add(this.spelabel4);
      this.spedataPanel.add(this.spelabel5);
      this.spedataPanel.add(this.spelabel9);
      this.spedataPanel.add(this.spelabel10);
      this.spedataPanel.add(this.time);
      this.spedataPanel.add(this.pressure);
      this.filter.setVisible(false);
      this.collection.setVisible(false);
      this.spelabel1.setVisible(false);
      this.spelabel2.setVisible(false);
    }

    return this.spedataPanel;
  }

  private JPanel getMoveDataPanel() {
    if (this.movedataPanel == null) {
      this.movedataPanel = new JPanel();
      FlowLayout flowLayout = new FlowLayout();
      this.movedataPanel.setLayout(flowLayout);
      this.movedataPanel.setBackground(this.panelColor);
      this.movedataPanel.add(this.movlabel1);
      this.movedataPanel.add(getMove());
      this.movedataPanel.add(getAddButton8());
      this.movedataPanel.add(getUpdButton8());
    }
    return this.movedataPanel;
  }

  private JPanel getMovePlateDataPanel() {
    if (this.movePlatedataPanel == null) {
      this.movePlatedataPanel = new JPanel();
      GridLayout gridLayout = new GridLayout();
      gridLayout.setRows(2);
      gridLayout.setColumns(7);
      gridLayout.setHgap(10);
      gridLayout.setVgap(3);
      this.movePlatedataPanel.setLayout(gridLayout);
      this.movePlatedataPanel.setBackground(this.panelColor);
      this.movePlatedataPanel.add(this.movplabel6);
      this.movePlatedataPanel.add(this.movplabel1);
      this.movePlatedataPanel.add(this.movplabel2);
      this.movePlatedataPanel.add(this.movplabel3);
      this.movePlatedataPanel.add(this.movplabel4);
      this.movePlatedataPanel.add(this.movplabel8);
      this.movePlatedataPanel.add(getAddButton10());
      this.movePlatedataPanel.add(this.movplabel7);
      this.movePlatedataPanel.add(getSrc5());
      this.movePlatedataPanel.add(this.movplabel5);
      this.movePlatedataPanel.add(getsPlateType3());
      this.movePlatedataPanel.add(getdPlateType3());
      this.movePlatedataPanel.add(this.movplabel9);
      this.movePlatedataPanel.add(getUpdButton10());
    }
    return this.movePlatedataPanel;
  }

  private JPanel getIncDataPanel()
  {
    if (this.incdataPanel == null) {
      this.incdataPanel = new JPanel();
      GridLayout gridLayout = new GridLayout();
      gridLayout.setRows(3);
      gridLayout.setColumns(8);
      //gridLayout.setHgap(10);
      //gridLayout.setVgap(3);
      this.incdataPanel.setBackground(this.panelColor);
      this.incdataPanel.setLayout(gridLayout);
      
      this.incdataPanel.add(this.ilabel);
      this.incdataPanel.add(this.text4);
      this.incdataPanel.add(this.blanklabel1);
      this.incdataPanel.add(this.blanklabel2);
      this.incdataPanel.add(this.blanklabel3);
      this.incdataPanel.add(this.blanklabel4);
      this.incdataPanel.add(getAddButton2());
      this.incdataPanel.add(this.blanklabel5);
      
      this.incdataPanel.add(this.ilabelpm);
      this.incdataPanel.add(this.text4pm);
      this.incdataPanel.add(this.blanklabel6);
      this.incdataPanel.add(this.blanklabel7);
      this.incdataPanel.add(this.blanklabel8);
      this.incdataPanel.add(this.blanklabel9);
      this.incdataPanel.add(getUpdButton2());
      this.incdataPanel.add(this.blanklabel10);
      
      this.incdataPanel.add(this.itemplabel);
      this.incdataPanel.add(this.itemp);
      this.incdataPanel.add(this.blanklabel11);
      this.incdataPanel.add(this.blanklabel12);
      this.incdataPanel.add(this.blanklabel13);
      this.incdataPanel.add(this.blanklabel14);
      this.incdataPanel.add(this.blanklabel15);
      this.incdataPanel.add(this.blanklabel16);
    }

    return this.incdataPanel;
  }

  private JPanel getLiqTrayPanel() {
    if (this.liqtrayPanel == null) {
      this.liqtrayPanel = new JPanel();
      GridLayout gridLayout2 = new GridLayout();
      gridLayout2.setRows(1);
      gridLayout2.setColumns(2);
      this.liqtrayPanel.setLayout(gridLayout2);
      this.liqtrayPanel.add(getLiqPanel1());
      this.liqtrayPanel.add(getLiqPanel2());
    }
    return this.liqtrayPanel;
  }

  private JPanel getLiqPanel1() {
    if (this.liqPanel1 == null) {
      this.liqPanel1 = new JPanel();
      this.liqPanel1.setLayout(new BorderLayout());
      this.liqPanel1.setBackground(this.panelColor);
      TitledBorder liqBorder1 = BorderFactory.createTitledBorder("Source");
      liqBorder1.setTitleJustification(2);
      liqBorder1.setTitleFont(this.font15);
      liqBorder1.setTitlePosition(2);
      this.liqPanel1.setBorder(liqBorder1);
      this.liqPanel1.add(getLiqPanel1Top(), "North");
      this.liqPanel1.add(this.panelbl, "Center");
      this.liqPanel1.add(getLiqPanel1Bot(), "South");
    }
    return this.liqPanel1;
  }

  private JPanel getLiqPanel1Top() {
    if (this.liqPanel1top == null) {
      this.liqPanel1top = new JPanel();
      GridLayout gridLayout3 = new GridLayout();
      gridLayout3.setRows(2);
      gridLayout3.setColumns(3);				//APF0031 Changed from (5).  commented out JLabel("") below to accomodate
      gridLayout3.setVgap(2);
      this.liqPanel1top.setLayout(gridLayout3);
      this.liqPanel1top.setBackground(this.panelColor);

      this.liqPanel1top.add(getPlateButton());
      this.liqPanel1top.add(getResButton());
      this.liqPanel1top.add(getTubeButton());
//      this.liqPanel1top.add(new JLabel(" "));
//      this.liqPanel1top.add(new JLabel(" "));
//      this.liqPanel1top.add(getbutton48l2());
//      this.liqPanel1top.add(getbutton96l2());
//      this.liqPanel1top.add(getbutton384l2());

      this.liqPanel1top.add(getsPlateType());
      this.liqPanel1top.add(getSrc());					
      this.liqPanel1top.add(new JLabel(" "));
//      this.liqPanel1top.add(new JLabel(" "));
//      this.liqPanel1top.add(new JLabel(" "));
//      this.liqPanel1top.add(getlinc1Button());
//      this.liqPanel1top.add(getlinc2Button());
//      this.liqPanel1top.add(getlinc3Button());
    }

    return this.liqPanel1top;
  }

  private JPanel getLiqPanel1Bot() {
    if (this.liqPanel1bot == null) {
      this.liqPanel1bot = new JPanel();
      GridLayout gridLayout4 = new GridLayout();
      gridLayout4.setRows(1);
      gridLayout4.setColumns(10);
      gridLayout4.setHgap(2);
      this.liqPanel1bot.setLayout(gridLayout4);
      this.liqPanel1bot.setBackground(this.panelColor);
      this.lmixt1.setBorder(this.blackline);
      this.lmixt1.setText("0");
      this.liqPanel1bot.add(this.lmixt1);
      this.liqPanel1bot.add(this.botlab1);
      this.liqPanel1bot.add(this.botlab2);
      this.liqPanel1bot.add(this.botlab3);
      this.liqPanel1bot.add(this.botlab4);
      this.liqPanel1bot.add(getAdvButton1());
    }

    return this.liqPanel1bot;
  }
  private JPanel getLiqPanel2Bot() {
    if (this.liqPanel2bot == null) {
      this.liqPanel2bot = new JPanel();
      GridLayout gridLayout5 = new GridLayout();
      gridLayout5.setRows(1);
      gridLayout5.setColumns(8);
      gridLayout5.setHgap(2);
      this.liqPanel2bot.setLayout(gridLayout5);
      this.liqPanel2bot.setBackground(this.panelColor);
      this.lchill1.setBackground(this.panelColor);
      this.lshake.setBackground(this.panelColor);
      this.lmixt2.setBorder(this.blackline);
      this.lmixt2.setText("0");
      this.liqPanel2bot.add(this.lmixt2);
      this.liqPanel2bot.add(this.botlab5);
      this.liqPanel2bot.add(this.botlab6);
      this.liqPanel2bot.add(this.botlab7);
      this.liqPanel2bot.add(this.botlab8);
      this.liqPanel2bot.add(getAdvButton2());
    }
    return this.liqPanel2bot;
  }

  private JPanel getLiqPanel2Top() {
    if (this.liqPanel2top == null) {
      this.liqPanel2top = new JPanel();
      GridLayout gridLayout3 = new GridLayout();
      gridLayout3.setRows(2);
      gridLayout3.setColumns(3);
      this.liqPanel2top.setLayout(gridLayout3);
      this.liqPanel2top.setBackground(this.panelColor);
      this.liqPanel2top.add(getPlateButton2());
      this.liqPanel2top.add(getResButton2());
      this.liqPanel2top.add(getTubeButton2());
//      this.liqPanel2top.add(new JLabel(" "));
//      this.liqPanel2top.add(new JLabel(" "));
//      this.liqPanel2top.add(getbutton48l());
//      this.liqPanel2top.add(getbutton96l());
//      this.liqPanel2top.add(getbutton384l());

      this.liqPanel2top.add(getdPlateType());
      this.liqPanel2top.add(getDest());
      this.liqPanel2top.add(new JLabel(" "));
//      this.liqPanel2top.add(new JLabel(" "));

//      this.liqPanel2top.add(getlinc4Button());
//      this.liqPanel2top.add(getlinc5Button());
//      this.liqPanel2top.add(getlinc6Button());
    }
    return this.liqPanel2top;
  }

  private JPanel getLiqPanel2() {
    if (this.liqPanel2 == null) {
      this.liqPanel2 = new JPanel();
      this.liqPanel2.setLayout(new BorderLayout());
      TitledBorder liqBorder2 = BorderFactory.createTitledBorder("Destination");
      liqBorder2.setTitleJustification(2);
      liqBorder2.setTitleFont(this.font15);
      liqBorder2.setTitlePosition(2);
      
      this.liqPanel2.setBorder(liqBorder2);
      this.liqPanel2.setBackground(this.panelColor);
      this.liqPanel2.add(getLiqPanel2Top(), "North");
      this.liqPanel2.add(this.panelbr, "Center");
      this.liqPanel2.add(getLiqPanel2Bot(), "South");
    }
    return this.liqPanel2;
  }

  private JPanel getBrewTrayPanel() {
    if (this.brewtrayPanel == null) {
      this.brewtrayPanel = new JPanel();
      GridLayout gridLayout2 = new GridLayout();
      gridLayout2.setRows(1);
      gridLayout2.setColumns(2);
      this.brewtrayPanel.setLayout(gridLayout2);
      this.brewtrayPanel.add(getBrewPanel1());
      this.brewtrayPanel.add(getBrewPanel2());
    }
    return this.brewtrayPanel;
  }

  private JPanel getBrewPanel1() {
    if (this.brewPanel1 == null) {
      this.brewPanel1 = new JPanel();
      this.brewPanel1.setBackground(this.panelColor);
      this.brewPanel1.setLayout(new BorderLayout());
      TitledBorder brewBorder1 = BorderFactory.createTitledBorder("Source");
      brewBorder1.setTitleJustification(2);
      this.brewPanel1.setBorder(brewBorder1);
      this.brewPanel1.add(getBrewPanel1Top(), "North");
      this.brewPanel1.add(this.brrespanel1, "Center");
      this.brewPanel1.add(getBrewPanel1Bot(), "South");
    }
    return this.brewPanel1;
  }

  private JPanel getBrewPanel1Top() {
    if (this.brewPanel1top == null) {
      this.brewPanel1top = new JPanel();
      this.brewPanel1top.setBackground(this.panelColor);
      this.brewPanel1top.add(getBrResButton1());
      this.brewPanel1top.add(getBrTubeButton1());
      this.brewPanel1top.add(getSrc4());
    }
    return this.brewPanel1top;
  }

  private JPanel getBrewPanel2Top() {
    if (this.brewPanel2top == null) {
      this.brewPanel2top = new JPanel();
      this.brewPanel2top.setBackground(this.panelColor);

      this.brewPanel2top.add(getBrResbutton2());
      this.brewPanel2top.add(getBrTubebutton2());
      this.brewPanel2top.add(getDest3());
    }
    return this.brewPanel2top;
  }

  private JPanel getBrewPanel1Bot()
  {
    if (this.brewPanel1bot == null) {
      this.brewPanel1bot = new JPanel();
      GridLayout gridLayout4 = new GridLayout();
      gridLayout4.setRows(1);
      gridLayout4.setColumns(6);
      this.brmix.setBackground(this.panelColor);
      this.brchill.setBackground(this.panelColor);
      this.brewPanel1bot.setBackground(this.panelColor);
      this.brewPanel1bot.setLayout(gridLayout4);
      this.brewPanel1bot.add(this.brmix);
      this.brewPanel1bot.add(this.brchill);
      this.brewPanel1bot.add(this.brwlab1);
      this.brewPanel1bot.add(this.brwlab2);
      this.brewPanel1bot.add(this.brwlab3);
      this.brewPanel1bot.add(this.brwlab4);
    }
    return this.brewPanel1bot;
  }

  private JPanel getBrewPanel2Bot() {
    if (this.brewPanel2bot == null) {
      this.brewPanel2bot = new JPanel();
      this.brewPanel2bot.setBackground(this.panelColor);
      GridLayout gridLayout5 = new GridLayout();
      gridLayout5.setRows(1);
      gridLayout5.setColumns(6);
      this.brewPanel2bot.setLayout(gridLayout5);
      this.brchill2.setVisible(false);
      this.brewPanel2bot.add(this.brchill2);
      this.brewPanel2bot.add(this.brwlab5);
      this.brewPanel2bot.add(this.brwlab6);
      this.brewPanel2bot.add(this.brwlab7);
      this.brewPanel2bot.add(this.brwlab8);
      this.brewPanel2bot.add(this.brwlab9);
    }

    return this.brewPanel2bot;
  }

  private JPanel getBrewPanel2() {
    if (this.brewPanel2 == null) {
      this.brewPanel2 = new JPanel();
      this.brewPanel2.setLayout(new BorderLayout());
      this.brewPanel2.setBackground(this.panelColor);
      TitledBorder brewBorder2 = BorderFactory.createTitledBorder("Destination");
      brewBorder2.setTitleJustification(2);
      this.brewPanel2.setBorder(brewBorder2);
      this.brewPanel2.add(getBrewPanel2Top(), "North");
      this.brewPanel2.add(this.brrespanel2, "Center");
      this.brewPanel2.add(getBrewPanel2Bot(), "South");
    }
    return this.brewPanel2;
  }

  private JPanel getSerPanel1Top() {
    if (this.serPanel1top == null) {
      this.serPanel1top = new JPanel();
      GridLayout gridLayout3 = new GridLayout();
      gridLayout3.setRows(2);
      gridLayout3.setColumns(5);

      this.serPanel1top.setBackground(this.panelColor);
      this.serPanel1top.setLayout(gridLayout3);
      this.serPanel1top.add(getPlateButton3());
      this.serPanel1top.add(getTubeButton3());  
//      tubebutton3.setVisible(false);//update_SD0020
      this.serPanel1top.add(this.serlab1);
      this.serPanel1top.add(this.serlab2);
      this.serPanel1top.add(this.serlab3);

      this.serPanel1top.add(getsPlateType2());
      this.serPanel1top.add(getSrc3());
      this.serPanel1top.add(getsinc1Button());
      this.serPanel1top.add(getsinc2Button());
      this.serPanel1top.add(getsinc3Button());
    }

    return this.serPanel1top;
  }

  private JPanel getSerPanel2Top() {
    if (this.serPanel2top == null) {
      this.serPanel2top = new JPanel();
      GridLayout gridLayout3 = new GridLayout();
      gridLayout3.setRows(2);
      gridLayout3.setColumns(5);
      this.serPanel2top.setBackground(this.panelColor);
      this.serPanel2top.setLayout(gridLayout3);
      this.serPanel2top.add(getbutton96s());
      this.serPanel2top.add(getbutton384s());

      this.serPanel2top.add(this.serlab7);
      this.serPanel2top.add(this.serlab8);
      this.serPanel2top.add(getSerialButton());
      this.serPanel2top.add(getdPlateType2());
      this.serPanel2top.add(getDest2());
      this.serPanel2top.add(getsinc4Button());
      this.serPanel2top.add(getsinc5Button());
      this.serPanel2top.add(getsinc6Button());
    }

    return this.serPanel2top;
  }

  private JPanel getSerPanel1() {
    if (this.serPanel1 == null) {
      this.serPanel1 = new JPanel();
      this.serPanel1.setLayout(new BorderLayout());
      this.serPanel1.setBackground(this.panelColor);
      TitledBorder serBorder1 = BorderFactory.createTitledBorder("Source");
      serBorder1.setTitleJustification(2);
      this.serPanel1.setBorder(serBorder1);
      this.serPanel1.add(getSerPanel1Top(), "North");
      this.serPanel1.add(this.spanel96l, "Center");
    }
    return this.serPanel1;
  }

  private JPanel getSerPanel2() {
    if (this.serPanel2 == null) {
      this.serPanel2 = new JPanel();
      this.serPanel2.setBackground(this.panelColor);
      this.serPanel2.setLayout(new BorderLayout());
      TitledBorder serBorder2 = BorderFactory.createTitledBorder("Destination");
      serBorder2.setTitleJustification(2);
      this.serPanel2.setBorder(serBorder2);
      this.serPanel2.add(getSerPanel2Top(), "North");
      this.serPanel2.add(this.spanel96r, "Center");
    }
    return this.serPanel2;
  }

  private JPanel getSerTrayPanel() {
    if (this.sertrayPanel == null) {
      this.sertrayPanel = new JPanel();
      GridLayout gridLayout = new GridLayout();
      gridLayout.setRows(1);
      gridLayout.setColumns(2);
      this.sertrayPanel.setLayout(gridLayout);
      this.sertrayPanel.add(getSerPanel1());
      this.sertrayPanel.add(getSerPanel2());
    }
    return this.sertrayPanel;
  }

  private JPanel getSerPanel()
  {
    if (this.serPanel == null) {
      this.serPanel = new JPanel();
      this.serPanel.setLayout(new BorderLayout());
      this.serPanel.setBackground(this.panelColor);
      this.serPanel.add(getSerDataPanel(), "North");
      this.serPanel.add(getSerTrayPanel(), "Center");
    }
    return this.serPanel;
  }

  private JPanel getIncPanel()
  {
    if (this.incPanel == null) {
      this.incPanel = new JPanel();
      this.incPanel.setBackground(this.panelColor);
      this.incPanel.setLayout(new BorderLayout());
      this.incPanel.add(getIncDataPanel(), "North");
    }
    return this.incPanel;
  }

  private JPanel getReadPanel()
  {
    if (this.readPanel == null) {
      this.readPanel = new JPanel();
      this.readPanel.setLayout(new BorderLayout());
      this.readPanel.setBackground(this.panelColor);
      this.readPanel.add(getReadDataPanel(), "North");
    }
    return this.readPanel;
  }

  private JPanel getCherryPanel()
  {
    if (this.cherryPanel == null) {
      this.cherryPanel = new JPanel();
      this.cherryPanel.setLayout(new BorderLayout());
      this.cherryPanel.setBackground(this.panelColor);
      this.cherryPanel.add(getCherryDataPanel(), "North");
    }
    return this.cherryPanel;
  }
  
  private JPanel getPKPanel()
  {
    if (this.pkPanel == null) {
      this.pkPanel = new JPanel();
      this.pkPanel.setLayout(new BorderLayout());
      this.pkPanel.setBackground(this.panelColor);
      this.pkPanel.add(getPKDataPanel1(), "North");
//      this.pkPanel.add(getPKDataPanel2(), "Center");
      this.pkPanel.add(getPKDataPanel2x(), "Center");
    }
    return this.pkPanel;
  }

  private JPanel getSPEPanel()
  {
    if (this.spePanel == null) {
      this.spePanel = new JPanel();
      this.spePanel.setBackground(this.panelColor);
      this.spePanel.setLayout(new BorderLayout());
      this.spePanel.add(getSPEDataPanel(), "North");
    }
    return this.spePanel;
  }

  private JPanel getMovePanel()
  {
    if (this.movePanel == null) {
      this.movePanel = new JPanel();
      this.movePanel.setBackground(this.panelColor);
      this.movePanel.setLayout(new BorderLayout());
      this.movePanel.add(getMoveDataPanel(), "North");
    }
    return this.movePanel;
  }

  private JPanel getMovePlatePanel()
  {
    if (this.movePlatePanel == null) {
      this.movePlatePanel = new JPanel();
      this.movePlatePanel.setBackground(this.panelColor);
      this.movePlatePanel.setLayout(new BorderLayout());
      this.movePlatePanel.add(getMovePlateDataPanel(), "North");
    }
    return this.movePlatePanel;
  }

  private JPanel getPausePanel()
  {
    if (this.pausePanel == null) {
      this.pausePanel = new JPanel();
      this.pausePanel.setBackground(this.panelColor);
      this.pausePanel.setLayout(new BorderLayout());
      this.pausePanel.add(getPausePanel1(), BorderLayout.NORTH);
      this.pausePanel.add(getPausePanel2(), BorderLayout.CENTER);
    }
    return this.pausePanel;
  }
  private JPanel getPausePanel1()
  {
    if (this.pausedataPanel1 == null) {
      this.pausedataPanel1 = new JPanel();
      this.pausedataPanel1.setBackground(this.panelColor);
      this.pausedataPanel1.setLayout(new FlowLayout());
      this.pausedataPanel1.add(getAddButton9());
      this.pausedataPanel1.add(getUpdButton9());
    }
    return this.pausedataPanel1;
  }

  private PausePanel getPausePanel2()
  {
	  if (this.pausedataPanel2 == null) {
        this.pausedataPanel2 = new PausePanel();
        this.pausedataPanel2.setBackground(this.panelColor);
        this.pausedataPanel2.setSize(650, 424);
        this.pausedataPanel2.setAlignmentX(Component.LEFT_ALIGNMENT);
	}
	 return pausedataPanel2;
  }

 
  
  
  
  private JPanel getCytoPanel()
  {
	    if (this.cytopanel == null) {
	        this.cytopanel = new JPanel();
	        this.cytopanel.setBackground(this.panelColor);
	        this.cytopanel.setLayout(new BorderLayout());
	        this.cytopanel.add(getCytoDataPanel(), BorderLayout.NORTH);
	        this.cytopanel.add(getCytoDataPanel2(), BorderLayout.CENTER);
	      }
	      return this.cytopanel;
  }

  private JPanel getCytoDataPanel()
  {
	    if (this.cytodatapanel == null) {
	    	this.cytodatapanel = new JPanel();
	        this.cytodatapanel.setBackground(this.panelColor);
		      GridLayout gridLayout = new GridLayout();
		      gridLayout.setRows(2);
		      gridLayout.setColumns(6);
		      gridLayout.setHgap(10);
		      gridLayout.setVgap(3);
//		      this.cytodatapanel.setBackground(this.panelColor);
		      this.cytodatapanel.setLayout(gridLayout);
		      
		      JLabel label_1 = new JLabel("  ");
		      this.cytodatapanel.add(label_1);
		      
		      JLabel label_2 = new JLabel("  ");
		      this.cytodatapanel.add(label_2);
		      
		      JLabel label_3 = new JLabel("  ");
		      this.cytodatapanel.add(label_3);
		      
		      JLabel lblNewLabel = new JLabel(" ");
		      this.cytodatapanel.add(lblNewLabel);
		      
		      JLabel label = new JLabel("  ");
		      this.cytodatapanel.add(label);
		      
		      cyt_btnAddStep = new JButton("Add step");
              cyt_btnAddStep.addActionListener(new ActionListener()
		      {
		        public void actionPerformed(ActionEvent e) {

		        	boolean cytoInputCheck = true;
		        	String errorMessage = "";
		        	String work1=cvtToCsv(cytodatapanel2.getCytoLoc());  
		        	String work2=cvtToCsv(cytodatapanel2.getDeckLoc()); 
		        	cyt_plateindex = cytodatapanel2.getPlateIndex(); 
		        	  
		        	if(work1.equals("fail")){cytoInputCheck = false; errorMessage = "Incorrect Range of Positions, Cannot add Cytomat step";}
		        	if(work2.equals("fail")){cytoInputCheck = false; errorMessage = "Incorrect Range of Positions, Cannot add Cytomat step";}
		        	if(work1.split(",").length != work2.split(",").length){cytoInputCheck = false; errorMessage = " Incorrect Range of Positions, Cannot add Cytomat step";}  
		        	if (cyt_plateindex == 0) {cytoInputCheck = false; errorMessage = "Please selecte the type of plate that will be moved";}

		        	
					if (cytoInputCheck)
			          {
			        	  cytloc = work1;
			        	  cytdeckloc = work2;
			        	  cyt_fromto = cytodatapanel2.getFTIndex();
			        	  cyt_cover = cytodatapanel2.getCoverIndex();
			        	  cyt_plate = cytodatapanel2.getPlateType();
			        	  cyt_tsp = cytodatapanel2.getTSP();
			        	  
			        	  TabWindow.this.addCytomat();
			          }
			          else
			          {
			        	JOptionPane.showMessageDialog(null, errorMessage, " ", 2); 
			          }
		        } } );
              
              this.cytodatapanel.add(cyt_btnAddStep);

		      JLabel label_4 = new JLabel("  ");
		      this.cytodatapanel.add(label_4);
		      
		      JLabel label_5 = new JLabel("  ");
		      this.cytodatapanel.add(label_5);
		      
		      JLabel label_6 = new JLabel("  ");
		      this.cytodatapanel.add(label_6);
		      
		      JLabel label_7 = new JLabel(" ");
		      this.cytodatapanel.add(label_7);
	      
		      JLabel label_8 = new JLabel("  ");
		      this.cytodatapanel.add(label_8);
		      
		        cyt_btnUpdateStep = new JButton("Update step");
		        cyt_btnUpdateStep.setEnabled(false);
				// JButton btnUpdate = new JButton("UPDATE");
				cyt_btnUpdateStep.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent arg0) {
		        	
				        	boolean cytoInputCheck = true;
				        	String errorMessage = "";
				        	String work1=cvtToCsv(cytodatapanel2.getCytoLoc()); 
				        	String work2=cvtToCsv(cytodatapanel2.getDeckLoc());
				        	cyt_plateindex = cytodatapanel2.getPlateIndex();
				        	
				        	if(work1.equals("fail")){cytoInputCheck = false; errorMessage = "Incorrect Range of Positions, Cannot add Cytomat step";}
				        	if(work2.equals("fail")){cytoInputCheck = false; errorMessage = "Incorrect Range of Positions, Cannot add Cytomat step";}
				        	if(work1.split(",").length != work2.split(",").length){cytoInputCheck = false; errorMessage = " Incorrect Range of Positions, Cannot add Cytomat step";}  
				        	if (cyt_plateindex == 0) {cytoInputCheck = false; errorMessage = "Please selecte the type of plate that will be moved";}
 	
							if (cytoInputCheck)
					          {
					        	  cytloc = work1;
					        	  cytdeckloc = work2;
					        	  cyt_fromto = cytodatapanel2.getFTIndex();
					        	  cyt_cover = cytodatapanel2.getCoverIndex();
					        	  cyt_plate = cytodatapanel2.getPlateType();
					        	  cyt_tsp = cytodatapanel2.getTSP();
					        	  
						        	TabWindow.this.updCytomat();
					          }
					          else
					          {
					        	JOptionPane.showMessageDialog(null, errorMessage, " ", 2); 
					          }
		
					}
				});
		      this.cytodatapanel.add(cyt_btnUpdateStep);

			

	      }
	      return this.cytodatapanel;
  }

  private CytoPanel getCytoDataPanel2()
  {
	  if (this.cytodatapanel2 == null) {
        this.cytodatapanel2 = new CytoPanel(platesAll, platetypes);
        this.cytodatapanel2.setBackground(this.panelColor);
        this.cytodatapanel2.setSize(650, 424);
        this.cytodatapanel2.setAlignmentX(Component.LEFT_ALIGNMENT);
	}
	 return cytodatapanel2;
  }

  private JPanel getLicPanel()
  {
	    if (this.licpanel == null) {
	        this.licpanel = new JPanel();
	        this.licpanel.setBackground(this.panelColor);
	        this.licpanel.setLayout(new BorderLayout());
	        this.licpanel.add(getLicDataPanel(), BorderLayout.NORTH);
	        this.licpanel.add(getLicoDataPanel2(), BorderLayout.CENTER);
	      }
	      return this.licpanel;
  }

  private JPanel getLicDataPanel()
  {
	    if (this.licdatapanel == null) {
	    	this.licdatapanel = new JPanel();
	        this.licdatapanel.setBackground(this.panelColor);
		      GridLayout gridLayout = new GridLayout();
		      gridLayout.setRows(2);
		      gridLayout.setColumns(6);
		      gridLayout.setHgap(10);
		      gridLayout.setVgap(3);
//		      this.licdatapanel.setBackground(this.panelColor);
		      this.licdatapanel.setLayout(gridLayout);
		      
		      JLabel label_1 = new JLabel("  ");
		      this.licdatapanel.add(label_1);
		      
		      JLabel label_2 = new JLabel("  ");
		      this.licdatapanel.add(label_2);
		      
		      JLabel label_3 = new JLabel("  ");
		      this.licdatapanel.add(label_3);
		      
		      JLabel lblNewLabel = new JLabel(" ");
		      this.licdatapanel.add(lblNewLabel);
		      
		      JLabel label = new JLabel("  ");
		      this.licdatapanel.add(label);
		      
		      lic_btnAddStep = new JButton("Add step");
              lic_btnAddStep.addActionListener(new ActionListener()
		      {
		        public void actionPerformed(ActionEvent e) {
		        	boolean licoInputCheck = true;
		        	String errorMessage = "";
		        	String work1=cvtToCsv(licodatapanel2.getLicoLoc());  
		        	String work2=cvtToCsv(licodatapanel2.getDeckLoc()); 
		        	lic_plateindex = licodatapanel2.getPlateIndex(); 
		        	  
		        	if(work1.equals("fail")){licoInputCheck = false; errorMessage = "Incorrect Range of Positions, Cannot add Liconic step";}
		        	if(work2.equals("fail")){licoInputCheck = false; errorMessage = "Incorrect Range of Positions, Cannot add Liconic step";}
		        	if(work1.split(",").length != work2.split(",").length){licoInputCheck = false; errorMessage = " Incorrect Range of Positions, Cannot add Liconic step";}  
		        	if (lic_plateindex == 0) {licoInputCheck = false; errorMessage = "Please selecte the type of plate that will be moved";}

		        	
					if (licoInputCheck)
			          {
			        	  licloc = work1;
			        	  licdeckloc = work2;
			        	  lic_fromto = licodatapanel2.getFTIndex();
			        	  lic_cover = licodatapanel2.getCoverIndex();
			        	  lic_plate = licodatapanel2.getPlateType();
			        	  lic_tsp = licodatapanel2.getTSP();
			        	  
			        	  TabWindow.this.addLiconic();
			          }
			          else
			          {
			        	JOptionPane.showMessageDialog(null, errorMessage, " ", 2); 
			          }

		        } } );
              
              this.licdatapanel.add(lic_btnAddStep);

		      JLabel label_4 = new JLabel("  ");
		      this.licdatapanel.add(label_4);
		      
		      JLabel label_5 = new JLabel("  ");
		      this.licdatapanel.add(label_5);
		      
		      JLabel label_6 = new JLabel("  ");
		      this.licdatapanel.add(label_6);
		      
		      JLabel label_7 = new JLabel(" ");
		      this.licdatapanel.add(label_7);
	      
		      JLabel label_8 = new JLabel("  ");
		      this.licdatapanel.add(label_8);
		      
		        lic_btnUpdateStep = new JButton("Update step");
		        lic_btnUpdateStep.setEnabled(false);
				// JButton btnUpdate = new JButton("UPDATE");
				lic_btnUpdateStep.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent arg0) {
			        	boolean licoInputCheck = true;
			        	String errorMessage = "";
			        	String work1=cvtToCsv(licodatapanel2.getLicoLoc()); 
			        	String work2=cvtToCsv(licodatapanel2.getDeckLoc());
			        	lic_plateindex = licodatapanel2.getPlateIndex();
			        	
			        	if(work1.equals("fail")){licoInputCheck = false; errorMessage = "Incorrect Range of Positions, Cannot add Liconic step";}
			        	if(work2.equals("fail")){licoInputCheck = false; errorMessage = "Incorrect Range of Positions, Cannot add Liconic step";}
			        	if(work1.split(",").length != work2.split(",").length){licoInputCheck = false; errorMessage = " Incorrect Range of Positions, Cannot add Liconic step";}  
			        	if (lic_plateindex == 0) {licoInputCheck = false; errorMessage = "Please selecte the type of plate that will be moved";}
	
						if (licoInputCheck)
				          {
				        	  licloc = work1;
				        	  licdeckloc = work2;
				        	  lic_fromto = licodatapanel2.getFTIndex();
				        	  lic_cover = licodatapanel2.getCoverIndex();
				        	  lic_plate = licodatapanel2.getPlateType();
				        	  lic_tsp = licodatapanel2.getTSP();
				        	  
				        	  TabWindow.this.updLiconic();
				          }
				          else
				          {
				        	JOptionPane.showMessageDialog(null, errorMessage, " ", 2); 
				          }
	
					}
				});
		      this.licdatapanel.add(lic_btnUpdateStep);

	      }
	      return this.licdatapanel;
  }

  private LicoPanel getLicoDataPanel2()
  {
	  if (this.licodatapanel2 == null) {
        this.licodatapanel2 = new LicoPanel(platesAll, platetypes);
        this.licodatapanel2.setBackground(this.panelColor);
        this.licodatapanel2.setSize(650, 424);
        this.licodatapanel2.setAlignmentX(Component.LEFT_ALIGNMENT);
	}
	 return licodatapanel2;
  }

  private JPanel getMovePlate2Panel()
  {
	    if (this.movplv2panel == null) {
	        this.movplv2panel = new JPanel();
	        this.movplv2panel.setBackground(this.panelColor);
	        this.movplv2panel.setLayout(new BorderLayout());
	        this.movplv2panel.add(getMovPl2DataPanel(), BorderLayout.NORTH);
	        this.movplv2panel.add(getMovPl2DataPanel2(), BorderLayout.CENTER);
	      }
	      return this.movplv2panel;
  }

  private JPanel getMovPl2DataPanel()
  {
	    if (this.movplv2datapanel == null) {
	    	this.movplv2datapanel = new JPanel();
	        this.movplv2datapanel.setBackground(this.panelColor);
		      GridLayout gridLayout = new GridLayout();
		      gridLayout.setRows(2);
		      gridLayout.setColumns(6);
		      gridLayout.setHgap(10);
		      gridLayout.setVgap(3);
//		      this.movplv2datapanel.setBackground(this.panelColor);
		      this.movplv2datapanel.setLayout(gridLayout);
		      
		      JLabel label_1 = new JLabel("  ");
		      this.movplv2datapanel.add(label_1);
		      
		      JLabel label_2 = new JLabel("  ");
		      this.movplv2datapanel.add(label_2);
		      
		      JLabel label_3 = new JLabel("  ");
		      this.movplv2datapanel.add(label_3);
		      
		      JLabel lblNewLabel = new JLabel(" ");
		      this.movplv2datapanel.add(lblNewLabel);
		      
		      JLabel label = new JLabel("  ");
		      this.movplv2datapanel.add(label);
		      
		      movplv2_btnAddStep = new JButton("Add step");
              movplv2_btnAddStep.addActionListener(new ActionListener()
		      {
		        public void actionPerformed(ActionEvent e) {

		        	boolean mvpl2InputCheck = true;
		        	String errorMessage = "";
		        	String work1=cvtToCsv(movplv2datapanel2.getStartLoc()); 
		        	String work2=cvtToCsv(movplv2datapanel2.getEndLoc());
		        	
		        	if(work1.equals("fail")){mvpl2InputCheck = false;  errorMessage = "Cannot interpret START locations.";}
		        	else if(work2.equals("fail")){mvpl2InputCheck = false;  errorMessage = "Cannot interpret END locations.";}
		        	else if(work1.split(",").length != work2.split(",").length){mvpl2InputCheck = false;  errorMessage = "START and END location ranges do not match.";}  

		        	mvp_type = movplv2datapanel2.getTypeIndex();
		        	mvp_plateindex = movplv2datapanel2.getPlateIndex();
		        	if (mvp_type == 0) {mvpl2InputCheck = false; errorMessage = "Please select the Move type";}
		        	if (mvp_plateindex == 0) {mvpl2InputCheck = false; errorMessage = "Please select the Plate type";}
		        		
					if (mvpl2InputCheck)
			          {
			        	  startloc = work1;
			        	  endloc = work2;
			        	  mvp_plate = movplv2datapanel2.getPlateType();
			        	  
			        	  TabWindow.this.addMovePlate2();
			          }
			          else
			          {
			        	JOptionPane.showMessageDialog(null, errorMessage, " ", 2); 
			          }
		        } } );
              
              this.movplv2datapanel.add(movplv2_btnAddStep);

		      JLabel label_4 = new JLabel("  ");
		      this.movplv2datapanel.add(label_4);
		      
		      JLabel label_5 = new JLabel("  ");
		      this.movplv2datapanel.add(label_5);
		      
		      JLabel label_6 = new JLabel("  ");
		      this.movplv2datapanel.add(label_6);
		      
		      JLabel label_7 = new JLabel(" ");
		      this.movplv2datapanel.add(label_7);
	      
		      JLabel label_8 = new JLabel("  ");
		      this.movplv2datapanel.add(label_8);
		      
		        movplv2_btnUpdateStep = new JButton("Update step");
		        movplv2_btnUpdateStep.setEnabled(false);
				movplv2_btnUpdateStep.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent arg0) {
		        	
				        	boolean mvpl2InputCheck = true;
				        	String errorMessage = "";
				        	String work1=cvtToCsv(movplv2datapanel2.getStartLoc()); 
				        	String work2=cvtToCsv(movplv2datapanel2.getEndLoc());
				        	  
				        	if(work1.equals("fail")){mvpl2InputCheck = false; errorMessage = "Cannot interpret START locations";}
				        	if(work2.equals("fail")){mvpl2InputCheck = false; errorMessage = "Cannot interpret END locations";}
				        	if(work1.split(",").length != work2.split(",").length){mvpl2InputCheck = false;  errorMessage = "START and END location ranges do not match.";}  
				        	mvp_type = movplv2datapanel2.getTypeIndex();
				        	mvp_plateindex = movplv2datapanel2.getPlateIndex();
				        	if (mvp_type == 0) {mvpl2InputCheck = false; errorMessage = "Please select the Move type";}
				        	if (mvp_plateindex == 0) {mvpl2InputCheck = false; errorMessage = "Please select the Plate type";}
 	
							if (mvpl2InputCheck)
					          {
					        	  startloc = work1;
					        	  endloc = work2;
					        	  mvp_type = movplv2datapanel2.getTypeIndex();
					        	  mvp_plate = movplv2datapanel2.getPlateType();
					        	  
						        	TabWindow.this.updMovePlate2();
					          }
					          else
					          {
					        	JOptionPane.showMessageDialog(null, errorMessage, " ", 2); 
					          }
		
					}
				});
		      this.movplv2datapanel.add(movplv2_btnUpdateStep);

	      }
	      return this.movplv2datapanel;
  }

  private MovePlatePanel02 getMovPl2DataPanel2()
  {
	  if (this.movplv2datapanel2 == null) {
        this.movplv2datapanel2 = new MovePlatePanel02(platesAll, platetypes);
        this.movplv2datapanel2.setBackground(this.panelColor);
        this.movplv2datapanel2.setSize(650, 424);
        this.movplv2datapanel2.setAlignmentX(Component.LEFT_ALIGNMENT);
	}
	 return movplv2datapanel2;
  }

  private JPanel getDNADataPanel1()
  {
	    if (this.dnadataPanel1 == null) {
	    	this.dnadataPanel1 = new JPanel();
	        this.dnadataPanel1.setBackground(this.panelColor);
		      GridLayout gridLayout = new GridLayout();
		      gridLayout.setRows(2);
		      gridLayout.setColumns(6);
		      gridLayout.setHgap(10);
		      gridLayout.setVgap(3);
//		      this.dnadataPanel1.setBackground(this.panelColor);
		      this.dnadataPanel1.setLayout(gridLayout);
		      
		      JLabel label_1 = new JLabel("  ");
		      this.dnadataPanel1.add(label_1);
		      
		      JLabel label_2 = new JLabel("  ");
		      this.dnadataPanel1.add(label_2);
		      
		      JLabel label_3 = new JLabel("  ");
		      this.dnadataPanel1.add(label_3);
		      
		      JLabel lblNewLabel = new JLabel(" ");
		      this.dnadataPanel1.add(lblNewLabel);
		      
		      JLabel label = new JLabel("  ");
		      this.dnadataPanel1.add(label);
		      
		      dna_btnAddStep = new JButton("Add step");
              dna_btnAddStep.addActionListener(new ActionListener()
		      {
		        public void actionPerformed(ActionEvent e) {

		        	boolean dnaInputCheck = true;
		        	String errorMessage = "";
		        	dna_plateindex = dnadataPanel2.getPlateIndex(); 
		        	dna_splateindex = dnadataPanel2.getSPlateIndex(); 
		        	dna_nplateindex = dnadataPanel2.getNPlateIndex(); 
		        	dna_dplateindex = dnadataPanel2.getDPlateIndex(); 
		        	dna_procindex = dnadataPanel2.getProcIndex();
 	        	    dna_con = dnadataPanel2.getCon();
		        	dna_vol = dnadataPanel2.getVol();
		        	dna_qfile = dnadataPanel2.getQFile();
		        	dna_deckindex1 = dnadataPanel2.getDeckIndex1(); 
		        	dna_deckindex2 = dnadataPanel2.getDeckIndex2(); 
		        	dna_deckindex3 = dnadataPanel2.getDeckIndex3(); 
		        	dna_deckindex4 = dnadataPanel2.getDeckIndex4(); 
		        	
		        	if (dna_procindex == 0){
		        		dnaInputCheck = false;
		        	}else if (dna_procindex == 1){
			        	if (dna_qfile.equals("Click here to select the Q file")    
			        			|| dna_con.equals("")   || dna_vol.equals("")
			        			|| dna_splateindex == 0 || dna_nplateindex == 0
			        			|| dna_dplateindex == 0 || dna_deckindex1 == 0  
			        			|| dna_deckindex2 == 0  || dna_deckindex3 == 0) 
			        			{dnaInputCheck = false;}
		        	}else{
		        		if(dna_plateindex == 0  ||  dna_deckindex4 == 0 ||
		        		   dna_nplateindex == 0 ||  dna_deckindex3 == 0)
		        		{dnaInputCheck = false;}	
		        	}
 	
					if (dnaInputCheck)
			          {
			        	  dna_dplate = dnadataPanel2.getDPlateType();
			        	  dna_splate = dnadataPanel2.getSPlateType();
			        	  dna_nplate = dnadataPanel2.getNPlateType();
			        	  dna_plate = dnadataPanel2.getPlateType();
			        	  dna_procindex = dnadataPanel2.getProcIndex();
			        	  dna_con = dnadataPanel2.getCon();
			        	  dna_vol = dnadataPanel2.getVol();
			        	  dna_qfile = dnadataPanel2.getQFile();
				          dna_deck1 = dnadataPanel2.getDeck1(); 
				          dna_deck2 = dnadataPanel2.getDeck2(); 
				          dna_deck3 = dnadataPanel2.getDeck3(); 
				          dna_deck4 = dnadataPanel2.getDeck4(); 
			        	  
			        	  TabWindow.this.addDNANorm();
			          }
			          else
			          {
			        	  errorMessage = "Incomplete DNA Normalization input";
			        	  JOptionPane.showMessageDialog(null, errorMessage, " ", 2); 
			          }
		        } } );
              
              this.dnadataPanel1.add(dna_btnAddStep);

		      JLabel label_4 = new JLabel("  ");
		      this.dnadataPanel1.add(label_4);
		      
		      JLabel label_5 = new JLabel("  ");
		      this.dnadataPanel1.add(label_5);
		      
		      JLabel label_6 = new JLabel("  ");
		      this.dnadataPanel1.add(label_6);
		      
		      JLabel label_7 = new JLabel(" ");
		      this.dnadataPanel1.add(label_7);
	      
		      JLabel label_8 = new JLabel("  ");
		      this.dnadataPanel1.add(label_8);
		      
		        dna_btnUpdateStep = new JButton("Update step");
		        dna_btnUpdateStep.setEnabled(false);
				// JButton btnUpdate = new JButton("UPDATE");
				dna_btnUpdateStep.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent arg0) {
		        	
				        	boolean dnaInputCheck = true;
				        	String errorMessage = "";
				        	dna_plateindex = dnadataPanel2.getPlateIndex(); 
				        	dna_splateindex = dnadataPanel2.getSPlateIndex(); 
				        	dna_nplateindex = dnadataPanel2.getNPlateIndex(); 
				        	dna_dplateindex = dnadataPanel2.getDPlateIndex(); 
				        	dna_procindex = dnadataPanel2.getProcIndex();
		 	        	    dna_con = dnadataPanel2.getCon();
				        	dna_vol = dnadataPanel2.getVol();
				        	dna_qfile = dnadataPanel2.getQFile();
				        	dna_deckindex1 = dnadataPanel2.getDeckIndex1(); 
				        	dna_deckindex2 = dnadataPanel2.getDeckIndex2(); 
				        	dna_deckindex3 = dnadataPanel2.getDeckIndex3(); 
				        	dna_deckindex4 = dnadataPanel2.getDeckIndex4(); 
				        	
				        	if (dna_procindex == 0){
				        		dnaInputCheck = false;
				        	}else if (dna_procindex == 1){
					        	if (dna_qfile.equals("Click here to select the Q file")    
					        			|| dna_con.equals("")   || dna_vol.equals("")
					        			|| dna_splateindex == 0 || dna_nplateindex == 0
					        			|| dna_dplateindex == 0 || dna_deckindex1 == 0  
					        			|| dna_deckindex2 == 0  || dna_deckindex3 == 0) 
					        			{dnaInputCheck = false;}
				        	}else{
				        		if(dna_plateindex == 0  ||  dna_deckindex4 == 0 ||
				        		   dna_nplateindex == 0 ||  dna_deckindex3 == 0)
				        		{dnaInputCheck = false;}	
				        	}
				        	
							if (dnaInputCheck)
					          {
					        	  dna_dplate = dnadataPanel2.getDPlateType();
					        	  dna_splate = dnadataPanel2.getSPlateType();
					        	  dna_nplate = dnadataPanel2.getNPlateType();
					        	  dna_plate = dnadataPanel2.getPlateType();
					        	  dna_procindex = dnadataPanel2.getProcIndex();
					        	  dna_con = dnadataPanel2.getCon();
					        	  dna_vol = dnadataPanel2.getVol();
					        	  dna_qfile = dnadataPanel2.getQFile();
						          dna_deck1 = dnadataPanel2.getDeck1(); 
						          dna_deck2 = dnadataPanel2.getDeck2(); 
						          dna_deck3 = dnadataPanel2.getDeck3(); 
						          dna_deck4 = dnadataPanel2.getDeck4(); 
					        	  
   					        	  TabWindow.this.updDNANorm();
					          }
					          else
					          {
					        	errorMessage = "Incomplete DNA Normalization input";
					        	JOptionPane.showMessageDialog(null, errorMessage, " ", 2); 
					          }
		
					}
				});
		      this.dnadataPanel1.add(dna_btnUpdateStep);

	      }
	      return this.dnadataPanel1;
  }


  private PanelDNAnorm getDNADataPanel2()
  {
	  if (this.dnadataPanel2 == null) {
        this.dnadataPanel2 = new PanelDNAnorm(platesTubes, tubesAll, deckpositions, reservoirsAll);
        this.dnadataPanel2.setBackground(this.panelColor);
        this.dnadataPanel2.setSize(650, 424);
        this.dnadataPanel2.setAlignmentX(Component.LEFT_ALIGNMENT);
	}
	 return dnadataPanel2;
  }

  private JPanel getLoadDataPanel1()
  {
	    if (this.loaddataPanel1 == null) {
	    	this.loaddataPanel1 = new JPanel();
	        this.loaddataPanel1.setBackground(this.panelColor);
		      GridLayout gridLayout = new GridLayout();
		      gridLayout.setRows(2);
		      gridLayout.setColumns(6);
		      gridLayout.setHgap(10);
		      gridLayout.setVgap(3);
		      this.loaddataPanel1.setLayout(gridLayout);
		      
		      JLabel label_1 = new JLabel("  ");
		      this.loaddataPanel1.add(label_1);
		      
		      JLabel label_2 = new JLabel("  ");
		      this.loaddataPanel1.add(label_2);
		      
		      JLabel label_3 = new JLabel("  ");
		      this.loaddataPanel1.add(label_3);
		      
		      JLabel lblNewLabel = new JLabel(" ");
		      this.loaddataPanel1.add(lblNewLabel);
		      
		      JLabel label = new JLabel("  ");
		      this.loaddataPanel1.add(label);
		      
		      load_btnAddStep = new JButton("Add step");
              load_btnAddStep.addActionListener(new ActionListener()
		      {
		        public void actionPerformed(ActionEvent e) {

		        	boolean loadInputCheck = true;
		        	String errorMessage = "";
		        	load_plateindex = loaddataPanel2.getPlateIndex(); 
		        	load_tubeindex = loaddataPanel2.getTubeIndex(); 
		        	load_tipindex = loaddataPanel2.getTipIndex(); 
	        		String work1=cvtToCsv(loaddataPanel2.getPlateLoc());
	        		String work2=cvtToCsv(loaddataPanel2.getTubeLoc());
	        		String work3=cvtToCsv(loaddataPanel2.getTipLoc());
 	        	    load_plate = work1;
		        	load_tube = work2;
		        	load_tip = work3;
		        	
		        	if (load_plateindex == 0 && load_tubeindex == 0 && load_tipindex == 0){
		        		loadInputCheck = false;
		        		errorMessage = "Please select a Load Action to continue";
		        	}
		        	if (load_plateindex != 0){
		        		if (loaddataPanel2.getPlateLoc().equals("")){
		        			loadInputCheck = false;
		        			errorMessage = "Please input the deck locations of the Plates and/or Reservoirs";
		        		}else if (work1.equals("fail")){
		        			loadInputCheck = false;
		        			errorMessage = "The Deck Locations for the Plates and Reservoirs can not be interpreted";
		        		}
		        	}	
		        	if (load_tubeindex != 0){
		        		if (loaddataPanel2.getTubeLoc().equals("")){
		        			loadInputCheck = false;
		        			errorMessage = "Please input the deck locations of the Tubes";
		        		}else if (work2.equals("fail")){
		        			loadInputCheck = false;
		        			errorMessage = "The Deck Locations for the Tubes can not be interpreted";
		        		}
		        	}
					if (loadInputCheck)
			          {
			        	  TabWindow.this.addLoad();
			          }
			          else
			          {
			        	  JOptionPane.showMessageDialog(null, errorMessage, " ", 2); 
			          }
		        } } );
              
              this.loaddataPanel1.add(load_btnAddStep);

		      JLabel label_4 = new JLabel("  ");
		      this.loaddataPanel1.add(label_4);
		      
		      JLabel label_5 = new JLabel("  ");
		      this.loaddataPanel1.add(label_5);
		      
		      JLabel label_6 = new JLabel("  ");
		      this.loaddataPanel1.add(label_6);
		      
		      JLabel label_7 = new JLabel(" ");
		      this.loaddataPanel1.add(label_7);
	      
		      JLabel label_8 = new JLabel("  ");
		      this.loaddataPanel1.add(label_8);
		      
		        load_btnUpdateStep = new JButton("Update step");
		        load_btnUpdateStep.setEnabled(false);
				load_btnUpdateStep.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent arg0) {
		        	
				        	boolean loadInputCheck = true;
				        	String errorMessage = "";
				        	load_plateindex = loaddataPanel2.getPlateIndex(); 
				        	load_tubeindex = loaddataPanel2.getTubeIndex(); 
				        	load_tipindex = loaddataPanel2.getTipIndex(); 
			        		String work1=cvtToCsv(loaddataPanel2.getPlateLoc());
			        		String work2=cvtToCsv(loaddataPanel2.getTubeLoc());
			        		String work3=cvtToCsv(loaddataPanel2.getTipLoc());
		 	        	    load_plate = work1;
				        	load_tube = work2;
				        	load_tip = work3;
				        	
				        	if (load_plateindex == 0 && load_tubeindex == 0 && load_tipindex == 0){
				        		loadInputCheck = false;
				        		errorMessage = "Please select a Load Action to continue";
				        	}
				        	if (load_plateindex != 0){
				        		if (loaddataPanel2.getPlateLoc().equals("")){
				        			loadInputCheck = false;
				        			errorMessage = "Please input the deck locations of the Plates";
				        		}else if (work1.equals("fail")){
				        			loadInputCheck = false;
				        			errorMessage = "The Deck Locations for the Plates and Reservoirs can not be interpreted";
				        		}
				        	}	
				        	if (load_tubeindex != 0){
				        		if (loaddataPanel2.getTubeLoc().equals("")){
				        			loadInputCheck = false;
				        			errorMessage = "Please input the deck locations of the Tubes";
				        		}else if (work2.equals("fail")){
				        			loadInputCheck = false;
				        			errorMessage = "The Deck Locations for the Tubes can not be interpreted";
				        		}
				        	}
							if (loadInputCheck)
					          {
   					        	  TabWindow.this.updLoad();
					          }
					          else
					          {
					        	JOptionPane.showMessageDialog(null, errorMessage, " ", 2); 
					          }
		
					}
				});
		      this.loaddataPanel1.add(load_btnUpdateStep);

	      }
	      return this.loaddataPanel1;
  }

  private LoadPanel1 getLoadDataPanel2()
  {
	  if (this.loaddataPanel2 == null) {
        this.loaddataPanel2 = new LoadPanel1();
        this.loaddataPanel2.setBackground(this.panelColor);
        this.loaddataPanel2.setSize(650, 424);
        this.loaddataPanel2.setAlignmentX(Component.LEFT_ALIGNMENT);
	}
	 return loaddataPanel2;
  }

  private JPanel getCherry02Panel()
  {
	    if (this.cherry02Panel == null) {
	        this.cherry02Panel = new JPanel();
	        this.cherry02Panel.setBackground(this.panelColor);
	        this.cherry02Panel.setLayout(new BorderLayout());
	        this.cherry02Panel.add(getCherry02DataPanel(), BorderLayout.NORTH);
	        this.cherry02Panel.add(getCherry02DataPanel2(), BorderLayout.CENTER);
	      }
	      return this.cherry02Panel;
  }

  private JPanel getCherry02DataPanel()
  {
	    if (this.cherry02dataPanel == null) {
	    	this.cherry02dataPanel = new JPanel();
	        this.cherry02dataPanel.setBackground(this.panelColor);
		      GridLayout gridLayout = new GridLayout();
		      gridLayout.setRows(2);
		      gridLayout.setColumns(6);
		      gridLayout.setHgap(10);
		      gridLayout.setVgap(3);
		      this.cherry02dataPanel.setLayout(gridLayout);
		      
		      JLabel label_1 = new JLabel("  ");
		      this.cherry02dataPanel.add(label_1);
		      
		      JLabel label_2 = new JLabel("  ");
		      this.cherry02dataPanel.add(label_2);
		      
		      JLabel label_3 = new JLabel("  ");
		      this.cherry02dataPanel.add(label_3);
		      
		      JLabel lblNewLabel = new JLabel(" ");
		      this.cherry02dataPanel.add(lblNewLabel);
		      
		      JLabel label = new JLabel("  ");
		      this.cherry02dataPanel.add(label);
		      
		      cherry02_btnAddStep = new JButton("Add step");
              cherry02_btnAddStep.addActionListener(new ActionListener()
		      {
		        public void actionPerformed(ActionEvent e) {

		        	boolean cherryInputCheck = true;
		        	String errorMessage = "";
		        	
		        	//confirm Worklist has correct format
//		        	errorMessage = VAStools.CP_WorklistConfirm(cherry02dataPanel2.chp_text_2.getText(), cherry02dataPanel2.getDeckLoc());
		        	if (!errorMessage.equals("")){cherryInputCheck = false;}
		        			            
		            String work1 = "null"; 
		            String work2 = "null"; 
		        	if (cherry02dataPanel2.getBarIndex() > 0 )
		        	{
			        	if (!cherry02dataPanel2.getPlateLoc().equals(""))
			        	{
			        		work1=cvtToCsv(cherry02dataPanel2.getPlateLoc());
					        if(work1.equals("fail"))
					        {
					        	cherryInputCheck = false;
					        	errorMessage = "Cannot interpret PLATE DECK positions.";
					        }
			        	}
			        	if (!cherry02dataPanel2.getTubeLoc().equals(""))
			        	{
			        		work2=cvtToCsv(cherry02dataPanel2.getTubeLoc());
					        if(work2.equals("fail"))
					        {
					        	cherryInputCheck = false;
					        	errorMessage = "Cannot interpret TUBE DECK positions.";
					        }
			        		
			        	}
		        	}

		        	ch2_liqtype = cherry02dataPanel2.getLiqType();
		        	ch2_smix = cherry02dataPanel2.getSMix();
		        	ch2_dmix = cherry02dataPanel2.getDMix();
		            if (ch2_smix.length() < 1)
		                TabWindow.this.smixval = 0;
		              else
		                try {
		                  TabWindow.this.smixval = Integer.parseInt(ch2_smix);
		                } catch (NumberFormatException exp) {
		                  TabWindow.this.smixval = 0;
		                  TabWindow.this.cherry02dataPanel2.setText3("");
		                }
		            if (ch2_dmix.length() < 1)
		                TabWindow.this.dmixval = 0;
		              else
		                try {
		                  TabWindow.this.dmixval = Integer.parseInt(ch2_dmix);
		                } catch (NumberFormatException exp) {
		                  TabWindow.this.dmixval = 0;
		                  TabWindow.this.cherry02dataPanel2.setText4("");
		                }
		        	ch2_scycle = cherry02dataPanel2.getSCycle();
		        	ch2_dcycle = cherry02dataPanel2.getDCycle();
		        	ch2_slabindex = cherry02dataPanel2.getSLabIndex();
		        	ch2_dlabindex = cherry02dataPanel2.getDLabIndex();
		        	ch2_splateindex = cherry02dataPanel2.getSPlateIndex();
		        	ch2_dplateindex = cherry02dataPanel2.getDPlateIndex();
		        	if (ch2_splateindex == 0) {cherryInputCheck = false; errorMessage = "Please select the Source plate type";}
		        	if (ch2_dplateindex == 0) {cherryInputCheck = false; errorMessage = "Please select the Destination plate type";}
		        	if (ch2_slabindex == 0) {cherryInputCheck = false; errorMessage = "Please select the Labware type";}
		        	if (ch2_dlabindex == 0) {cherryInputCheck = false; errorMessage = "Please select the Labware type";}
		        	if (!ch2_scycle.equals("0"))
		        	{
		        	if (smixval < 1 || smixval > 1000) {cherryInputCheck = false; errorMessage = "Source mix volume must be between 1 - 1000ul";}
		        	}
		        	if (!ch2_dcycle.equals("0"))
		        	{
		        	if (dmixval < 1 || dmixval > 1000) {cherryInputCheck = false; errorMessage = "Destination mix volume must be between 1 - 1000ul";}
		        	}
		        	
		        	
					if (cherryInputCheck)
			          {
			        	  ch2_plateloc = work1;
			        	  ch2_tubeloc = work2;
			        	  TabWindow.this.addCherry02();
			          }
			          else
			          {
			        	JOptionPane.showMessageDialog(null, errorMessage, " ", 2); 
			          }
		        } } );
              
              this.cherry02dataPanel.add(cherry02_btnAddStep);

		      JLabel label_4 = new JLabel("  ");
		      this.cherry02dataPanel.add(label_4);
		      
		      JLabel label_5 = new JLabel("  ");
		      this.cherry02dataPanel.add(label_5);
		      
		      JLabel label_6 = new JLabel("  ");
		      this.cherry02dataPanel.add(label_6);
		      
		      JLabel label_7 = new JLabel(" ");
		      this.cherry02dataPanel.add(label_7);
	      
		      JLabel label_8 = new JLabel("  ");
		      this.cherry02dataPanel.add(label_8);
		      
		        cherry02_btnUpdateStep = new JButton("Update step");
		        cherry02_btnUpdateStep.setEnabled(false);
				cherry02_btnUpdateStep.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent arg0) {
		        	
		        	boolean cherryInputCheck = true;
		        	String errorMessage = "";
		            String work1 = "null"; 
		            String work2 = "null"; 
		        	if (cherry02dataPanel2.getBarIndex() > 0 )
		        	{
			        	if (!cherry02dataPanel2.getPlateLoc().equals(""))
			        	{
			        		work1=cvtToCsv(cherry02dataPanel2.getPlateLoc());
					        if(work1.equals("fail"))
					        {
					        	cherryInputCheck = false;
					        	errorMessage = "Cannot interpret PLATE DECK positions.";
					        }
			        	}
			        	if (!cherry02dataPanel2.getTubeLoc().equals(""))
			        	{
			        		work2=cvtToCsv(cherry02dataPanel2.getTubeLoc());
					        if(work2.equals("fail"))
					        {
					        	cherryInputCheck = false;
					        	errorMessage = "Cannot interpret TUBE DECK positions.";
					        }
			        		
			        	}
		        	}

		        	if (cherry02dataPanel2.getBarIndex() < 1)
		        	{
		        		work1 = "null";
		        		work2 = "null";
		        		errorMessage = "";
		        		cherryInputCheck = true;
		        	}

		        	ch2_liqtype = cherry02dataPanel2.getLiqType();
		        	ch2_smix = cherry02dataPanel2.getSMix();
		        	ch2_dmix = cherry02dataPanel2.getDMix();
		            if (ch2_smix.length() < 1)
		                TabWindow.this.smixval = 0;
		              else
		                try {
		                  TabWindow.this.smixval = Integer.parseInt(ch2_smix);
		                } catch (NumberFormatException exp) {
		                  TabWindow.this.smixval = 0;
		                  TabWindow.this.cherry02dataPanel2.setText3("");
		                }
		            if (ch2_dmix.length() < 1)
		                TabWindow.this.dmixval = 0;
		              else
		                try {
		                  TabWindow.this.dmixval = Integer.parseInt(ch2_dmix);
		                } catch (NumberFormatException exp) {
		                  TabWindow.this.dmixval = 0;
		                  TabWindow.this.cherry02dataPanel2.setText4("");
		                }
		        	ch2_scycle = cherry02dataPanel2.getSCycle();
		        	ch2_dcycle = cherry02dataPanel2.getDCycle();
		        	ch2_splateindex = cherry02dataPanel2.getSPlateIndex();
		        	ch2_dplateindex = cherry02dataPanel2.getDPlateIndex();
		        	ch2_slabindex = cherry02dataPanel2.getSLabIndex();
		        	ch2_dlabindex = cherry02dataPanel2.getDLabIndex();
		        	if (ch2_splateindex == 0) {cherryInputCheck = false; errorMessage = "Please select the Plate type";}
		        	if (ch2_dplateindex == 0) {cherryInputCheck = false; errorMessage = "Please select the Plate type";}
		        	if (ch2_slabindex == 0) {cherryInputCheck = false; errorMessage = "Please select the Labware type";}
		        	if (ch2_dlabindex == 0) {cherryInputCheck = false; errorMessage = "Please select the Labware type";}
		        	if (!ch2_scycle.equals("0"))
		        	{
		        	if (smixval < 1 || smixval > 1000) {cherryInputCheck = false; errorMessage = "Invalid mix volume";}
		        	}
		        	if (!ch2_dcycle.equals("0"))
		        	{
		        	if (dmixval < 1 || dmixval > 1000) {cherryInputCheck = false; errorMessage = "Invalid mix volume";}
		        	}
		        	
		        		
					if (cherryInputCheck)
			          {
			        	  ch2_plateloc = work1;
			        	  ch2_tubeloc = work2;
			        	  TabWindow.this.updCherry02();
			          }
			          else
			          {
			        	JOptionPane.showMessageDialog(null, errorMessage, " ", 2); 
			          }
		        } } );
		      this.cherry02dataPanel.add(cherry02_btnUpdateStep);

	      }
	      return this.cherry02dataPanel;
  }

  private CherryPanel02 getCherry02DataPanel2()
  {
	  if (this.cherry02dataPanel2 == null) {
        this.cherry02dataPanel2 = new CherryPanel02(platesAll, liqtypes, tubesAll);
        this.cherry02dataPanel2.setBackground(this.panelColor);
        this.cherry02dataPanel2.setSize(650, 424);
        this.cherry02dataPanel2.setAlignmentX(Component.LEFT_ALIGNMENT);
	}
	 return cherry02dataPanel2;
  }

  private JPanel getres12Xpanel(int r)
  {
    if (this.res12Xpanel == null)
    {
      this.res12Xpanel = new JPanel();
      this.res12Xpanel.setLayout(new FlowLayout());
    }
    this.res12Xpanel.removeAll();
    this.res12Xpanel.add(getTextWindow(r));
    this.res12Xpanel.setBackground(panelColor);
    return this.res12Xpanel;
  }

  private JPanel getres12Xpanel2(int r)  
  {
    if (this.res12Xpanel2 == null)
    {
      this.res12Xpanel2 = new JPanel();
      this.res12Xpanel2.setLayout(new FlowLayout());
    }
    this.res12Xpanel2.removeAll();
    this.res12Xpanel2.add(getTextWindow2(r));
    this.res12Xpanel2.setBackground(panelColor);
    return this.res12Xpanel2;
  }

  private JPanel getDNAPanel()
  {
    if (this.dnaPanel == null) {
      this.dnaPanel = new JPanel();
      this.dnaPanel.setLayout(new BorderLayout());
      this.dnaPanel.setBackground(this.panelColor);
      this.dnaPanel.add(getDNADataPanel1(), "North");
      this.dnaPanel.add(getDNADataPanel2(), "Center");
    }
    return this.dnaPanel;
  }

  private JPanel getLoadPanel()
  {
    if (this.loadPanel == null) {
      this.loadPanel = new JPanel();
      this.loadPanel.setLayout(new BorderLayout());
      this.loadPanel.setBackground(this.panelColor);
      this.loadPanel.add(getLoadDataPanel1(), "North");
      this.loadPanel.add(getLoadDataPanel2(), "Center");
    }
    return this.loadPanel;
  }

  public JPanel getTextWindow(int r) { 
	  
	  cntrGlobal2 = r;
	  this.restextWindow = new JPanel();
      this.restextWindow.setLayout(new GridLayout(1, 12));

      int height = 14;
      int width;
      if (rHam == true){
    	  width = 4;   	  
      }else{
    	  width = Math.round(32/r);
      }

      for (int d = 0; d < r; d++) {
        this.resTextArea[d] = new JTextArea(height, width);
        this.resTextArea[d].setVisible(false);
        this.resTextArea[d].setBorder(this.blackline);
      }
      
      //Auto Select if _1X
      if (r == 1){
    	  TabWindow.this.resTextArea[r-1].setBackground(Color.blue);
    	  TabWindow.this.rs1 = "1";
      }
      
      for (cntrGlobal1 = 0; cntrGlobal1 < r; cntrGlobal1++) {

          this.resTextArea[cntrGlobal1].setEditable(false);  
          
          this.resTextArea[cntrGlobal1].addMouseListener(new MouseListener()
          {
          	Integer x = new Integer(cntrGlobal1);
            public void mouseClicked(MouseEvent e)
            {
              TabWindow.this.rs1 = String.valueOf(x +1);

              for (int d = 0; d < cntrGlobal2; d++) {
              TabWindow.this.resTextArea[d].setBackground(Color.white);
              }
              TabWindow.this.resTextArea[x].setBackground(Color.blue); 
            }
            public void mouseEntered(MouseEvent e){}  
            public void mouseExited(MouseEvent e){}
            public void mousePressed(MouseEvent e){}
            public void mouseReleased(MouseEvent e){}
          });
         this.restextWindow.add(this.resTextArea[cntrGlobal1]); 
      }
      for (int res = 0; res < r; res++)
          this.resTextArea[res].setVisible(true);
      
	  return this.restextWindow;
  }
	    
	        
  private JPanel getTextWindow2(int r) {   
	  cntrGlobal4 = r;
	  this.restextWindow2 = new JPanel();
      this.restextWindow2.setLayout(new GridLayout(1, 12));

      if (r == 0){																//APN0047
    	  this.resTextArea2[0] = new JTextArea(1, 1);
    	  this.resTextArea2[0].setText("SELECT RESERVOIR TYPE ABOVE");			//APN0047
      }else{																	//APN0047
    	  this.resTextArea2[0].setText("");										//APN0047
	      int height = 14;
	      int width;
	      if (rHam2 == true){
	    	  width = 4;   	  
	      }else{
	    	  width = Math.round(32/r);
	      }
	    	  
	      for (int d = 0; d < r; d++) {
	        this.resTextArea2[d] = new JTextArea(height, width);
	        this.resTextArea2[d].setVisible(false);
	        this.resTextArea2[d].setBorder(this.blackline);
	      }
	      
	      //Auto Select if _1X
	      if (r == 1){
	    	  TabWindow.this.resTextArea2[r-1].setBackground(Color.blue);
	    	  TabWindow.this.rs2 = "1";
	      }
	      
	      for (cntrGlobal3 = 0; cntrGlobal3 < r; cntrGlobal3++) {
	
	          this.resTextArea2[cntrGlobal3].setEditable(false);  
	          
	          this.resTextArea2[cntrGlobal3].addMouseListener(new MouseListener()
	          {
	          	Integer x = new Integer(cntrGlobal3);
	            public void mouseClicked(MouseEvent e)
	            {
	              TabWindow.this.rs2 = String.valueOf(x +1);
	
	              for (int d = 0; d < cntrGlobal4; d++) {
	              TabWindow.this.resTextArea2[d].setBackground(Color.white);
	              }
	              TabWindow.this.resTextArea2[x].setBackground(Color.blue); 
	            }
	            public void mouseEntered(MouseEvent e){}  
	            public void mouseExited(MouseEvent e){}
	            public void mousePressed(MouseEvent e){}
	            public void mouseReleased(MouseEvent e){}
	          });
	         this.restextWindow2.add(this.resTextArea2[cntrGlobal3]); 
	      }
	
	      for (int res = 0; res < r; res++)
	          this.resTextArea2[res].setVisible(true);  
      }
      
      
      
      
	  return this.restextWindow2;
  }

	   
  private JButton getReadButton()
  {
    if (this.readButton == null) {
      this.readButton = new JButton();
      this.readButton.setSize(20, 20);
      this.readButton.setText("    BROWSE     ");
      this.readButton.setHorizontalAlignment(0);
      this.readButton.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          TabWindow.this.readFile();
        } } );
    }
    return this.readButton;
  }

  private JButton getWorklistButton()
  {
    if (this.workButton == null) {
      this.workButton = new JButton();
      this.workButton.setSize(20, 20);
      this.workButton.setText("    BROWSE     ");
      this.workButton.setHorizontalAlignment(0);
      this.workButton.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          TabWindow.this.workFile();
        } } );
    }
    return this.workButton;
  }
  private JButton getWatsonButton()
  {
    if (this.watsonButton == null) {
      this.watsonButton = new JButton();
      this.watsonButton.setSize(20, 20);
      this.watsonButton.setText("    BROWSE     ");
      this.watsonButton.setHorizontalAlignment(0);
      this.watsonButton.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          TabWindow.this.watsonFile();
        } } );
    }
    return this.watsonButton;
  }

  private JButton getAdvButton1()
  {
    if (this.advButton1 == null) {
      this.advButton1 = new JButton();
      this.advButton1.setSize(20, 20);
      this.advButton1.setText("Advanced");
      this.advButton1.setHorizontalAlignment(0);
      this.advButton1.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
        	
        	AdvWindow.frameActive = true;
        	TabWindow.this.advwin.showTipWalkAway(TabWindow.this.showtipwalkaway);
        	TabWindow.this.advwin.showChillBox(TabWindow.this.showchill);
        	TabWindow.this.advwin.showShakeBox(TabWindow.this.showshake);
        	TabWindow.this.advwin.setVisible(true);
        } } );
    }
    return this.advButton1;
  }

  private JButton getArrowButton1()
  {
    if (this.arrowButton1 == null) {
      this.arrowButton1 = new JButton();
      this.arrowButton1.setSize(20, 20);
      this.arrowButton1.setIcon(this.icons[0]);
      this.arrowButton1.setContentAreaFilled(false);

      this.arrowButton1.setHorizontalAlignment(0);
      this.arrowButton1.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          if (TabWindow.this.pagecount >= 1) {
            TabWindow.this.pagecount -= 1;
            TabWindow.this.dTextArea[0].setText("");
            TabWindow.this.dTextArea[1].setText("");
            TabWindow.this.dTextArea[2].setText("");
            TabWindow.this.dTextArea[3].setText("");
            TabWindow.this.dTextArea[4].setText("");
            TabWindow.this.dTextArea[5].setText("");
            TabWindow.this.dTextArea[6].setText("");
            TabWindow.this.dTextArea[7].setText("");
            TabWindow.this.dTextArea[8].setText("");
            TabWindow.this.dTextArea[9].setText("");
            TabWindow.this.dTextArea[10].setText("");
            TabWindow.this.dTextArea[11].setText("");
            TabWindow.this.dTextArea[12].setText("");
            TabWindow.this.dTextArea[13].setText("");
            TabWindow.this.dTextArea[14].setText("");
            TabWindow.this.dTextArea[0].setVisible(false);
            TabWindow.this.dTextArea[1].setVisible(false);
            TabWindow.this.dTextArea[2].setVisible(false);
            TabWindow.this.dTextArea[3].setVisible(false);
            TabWindow.this.dTextArea[4].setVisible(false);
            TabWindow.this.dTextArea[5].setVisible(false);
            TabWindow.this.dTextArea[6].setVisible(false);
            TabWindow.this.dTextArea[7].setVisible(false);
            TabWindow.this.dTextArea[8].setVisible(false);
            TabWindow.this.dTextArea[9].setVisible(false);
            TabWindow.this.dTextArea[10].setVisible(false);
            TabWindow.this.dTextArea[11].setVisible(false);
            TabWindow.this.dTextArea[12].setVisible(false);
            TabWindow.this.dTextArea[13].setVisible(false);
            TabWindow.this.dTextArea[14].setVisible(false);
               
            TabWindow.this.loadtext();
          }
        } } );
    }
    return this.arrowButton1;
  }

  private JButton getArrowButton2()
  {
    if (this.arrowButton2 == null) {
      this.arrowButton2 = new JButton();
      this.arrowButton2.setSize(20, 20);

      this.arrowButton2.setContentAreaFilled(false);
      this.arrowButton2.setIcon(this.icons[1]);
      this.arrowButton2.setHorizontalAlignment(0);
      this.arrowButton2.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          TabWindow.this.pagecount += 1;
          TabWindow.this.dTextArea[0].setText("");
          TabWindow.this.dTextArea[1].setText("");
          TabWindow.this.dTextArea[2].setText("");
          TabWindow.this.dTextArea[3].setText("");
          TabWindow.this.dTextArea[4].setText("");
          TabWindow.this.dTextArea[5].setText("");
          TabWindow.this.dTextArea[6].setText("");
          TabWindow.this.dTextArea[7].setText("");
          TabWindow.this.dTextArea[8].setText("");
          TabWindow.this.dTextArea[9].setText("");
          TabWindow.this.dTextArea[10].setText("");
          TabWindow.this.dTextArea[11].setText("");
          TabWindow.this.dTextArea[12].setText("");
          TabWindow.this.dTextArea[13].setText("");
          TabWindow.this.dTextArea[14].setText("");
          TabWindow.this.dTextArea[0].setVisible(false);
          TabWindow.this.dTextArea[1].setVisible(false);
          TabWindow.this.dTextArea[2].setVisible(false);
          TabWindow.this.dTextArea[3].setVisible(false);
          TabWindow.this.dTextArea[4].setVisible(false);
          TabWindow.this.dTextArea[5].setVisible(false);
          TabWindow.this.dTextArea[6].setVisible(false);
          TabWindow.this.dTextArea[7].setVisible(false);
          TabWindow.this.dTextArea[8].setVisible(false);
          TabWindow.this.dTextArea[9].setVisible(false);
          TabWindow.this.dTextArea[10].setVisible(false);
          TabWindow.this.dTextArea[11].setVisible(false);
          TabWindow.this.dTextArea[12].setVisible(false);
          TabWindow.this.dTextArea[13].setVisible(false);
          TabWindow.this.dTextArea[14].setVisible(false);
          TabWindow.this.loadtext();
        } } );
    }
    return this.arrowButton2;
  }

  private JButton getAdvButton2()
  {
    if (this.advButton2 == null) {
      this.advButton2 = new JButton();
      this.advButton2.setSize(20, 20);
      this.advButton2.setText("Advanced");
      this.advButton2.setHorizontalAlignment(0);
      this.advButton2.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
        	
        	AdvWindowD.frameActive = true;
        	TabWindow.this.advwin.showTipWalkAway(TabWindow.this.showtipwalkaway);
          TabWindow.this.advwind.showChillBox(TabWindow.this.showchill);
          TabWindow.this.advwind.showShakeBox(TabWindow.this.showshake);
          TabWindow.this.advwind.setVisible(true);
        } } );
    }
    return this.advButton2;
  }

  private JButton getAdvButton3()
  {
    if (this.advButton3 == null) {
      this.advButton3 = new JButton();
      this.advButton3.setSize(20, 20);
      this.advButton3.setText("    Advanced     ");
      this.advButton3.setHorizontalAlignment(0);
      this.advButton3.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          TabWindow.this.advwin.setVisible(true);
        } } );
    }
    return this.advButton3;
  }

  private void readFile()
  {
    int state = getJFileChooser2().showSaveDialog(this);
    if (state == 0) {
      File f = getJFileChooser2().getSelectedFile();
      this.path = f.getPath();
      this.text6.setText(this.path);
      this.text6.setEditable(false);
    }
  }

  private void workFile()
  {
    int state = getJFileChooser3().showSaveDialog(this);
    if (state == 0) {
      File f = getJFileChooser3().getSelectedFile();
      this.path = f.getPath();
      this.text7.setText(this.path);
      this.text7.setEditable(false);
    }
  }

  private void watsonFile()
  {
    int state = getJFileChooser3().showOpenDialog(this);
    if (state == 0) {
      File f = getJFileChooser3().getSelectedFile();
      this.path = f.getPath();
      if (VAStools.PK_WorklistConfirm(this.path)){			//PKF0033
          this.pkm_text_wat.setText(this.path);
          this.pkm_text_wat.setEditable(false);
      }else{
    	  this.pkm_text_wat.setText("");
      }
    }
  }

  private JFileChooser getJFileChooser()
  {
    if (this.jFileChooser == null) {
      this.jFileChooser = new JFileChooser();
      this.jFileChooser.setMultiSelectionEnabled(false);
      FileFilter filter1 = new textFileFilter(".apr", new String[] { "APR" });
      this.jFileChooser.setFileFilter(filter1);
    }

    return this.jFileChooser;
  }

  private JFileChooser getJFileChooser2()
  {
    if (this.jFileChooser2 == null) {
      this.jFileChooser2 = new JFileChooser();
      this.jFileChooser2.setMultiSelectionEnabled(false);
      FileFilter filter2 = new textFileFilter(".ppr", new String[] { "PPR" });
      this.jFileChooser2.setFileFilter(filter2);
    }

    return this.jFileChooser2;
  }

  private JFileChooser getJFileChooser3()
  {
    if (this.jFileChooser3 == null) {
      this.jFileChooser3 = new JFileChooser();
      jFileChooser3.setDialogTitle("Select Worklist");
      this.jFileChooser3.setApproveButtonText("Select");
      this.jFileChooser3.setMultiSelectionEnabled(false);
      this.jFileChooser3.setAcceptAllFileFilterUsed(false);
      FileFilter filter3 = new textFileFilter(".txt or .pdf or .xls", new String[] { "TXT","PDF","XLS" });
      this.jFileChooser3.setFileFilter(filter3);
    }
    return this.jFileChooser3;
  }

  private JFileChooser getJFileChooser4()
  {
    if (this.jFileChooser4 == null) {
      this.jFileChooser4 = new JFileChooser();
      jFileChooser4.setDialogTitle("Select Layout File");
      this.jFileChooser4.setApproveButtonText("Select");
      this.jFileChooser4.setMultiSelectionEnabled(false);
      this.jFileChooser4.setAcceptAllFileFilterUsed(false);
      FileFilter filterLayout = new textFileFilter(".lay", new String[] { "LAY" });
      this.jFileChooser4.setFileFilter(filterLayout);
    }
    return this.jFileChooser4;
  }

  private int layoutSelect()
  {
    int state = getJFileChooser4().showOpenDialog(this);
    if (state == 0) {
      File f = getJFileChooser4().getSelectedFile();
      this.layoutFileSelected = f.getPath();
      this.layoutSelectButton.setIcon(this.icons[18]);
      this.layoutSelectButton.setToolTipText(this.layoutFileSelected);
    }
    return state;
  }

  private JButton getSaveButton()
  {
    if (this.saveButton == null) {
      this.saveButton = new JButton();
      this.saveButton.setPreferredSize(new Dimension(34, 34));
      this.saveButton.setContentAreaFilled(false);
      this.saveButton.setFocusPainted(false);
      this.saveButton.setIcon(this.icons[3]);
      this.saveButton.setToolTipText("Save the current Automation Protocol");

      this.saveButton.addFocusListener(new FocusAdapter() {
        public void focusGained(FocusEvent fe) {
          TabWindow.this.saveButton.setBorder(null);
        }
        public void focusLost(FocusEvent e) {
          TabWindow.this.saveButton.setBorder(null);
        }
      });
      this.saveButton.addMouseListener(new MouseAdapter() {
        public void mouseEntered(MouseEvent me) {
          TabWindow.this.saveButton.setContentAreaFilled(true);
        }
        public void mouseExited(MouseEvent me) {
          TabWindow.this.saveButton.setContentAreaFilled(false);
        }
      });
      this.saveButton.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
        	
        	if (security){
        		if (security_auditTrailString.contains("AT0") || security_auditTrailString.contains("AT1")){
        			security_auditTrailString = security_auditTrailString.replace("AT1", "AT0");	//1 indicates that protocol is valid	
        		}	
	      		security_Validated = false;
	      		security_validationTitle = "Not Validated";        		
        	}

      		
        	if (aprActive.equals("")){
        		int state = getJFileChooser().showSaveDialog(saveAsButton);
            	
            	if (state == 0){
	            	File f = getJFileChooser().getSelectedFile();
	            	TabWindow.this.saveFile(f);	
            	}
            	
            	
        	}else{
        		File f = new File(aprActive);
        		TabWindow.this.saveFile(f);
        	}
        	
    		TabWindow.this.loadButton.setEnabled(true);
    		//TabWindow.this.jTabbedPane1.setSelectedIndex(0);	

        } } );
    }
    return this.saveButton;
  }
  
  private JButton getSaveAsButton()
  {
    if (this.saveAsButton == null) {
      this.saveAsButton = new JButton();
      this.saveAsButton.setPreferredSize(new Dimension(34, 34));
      this.saveAsButton.setContentAreaFilled(false);
      this.saveAsButton.setFocusPainted(false);
      this.saveAsButton.setIcon(this.icons[21]);
      this.saveAsButton.setToolTipText("Save the current Automation Protocol with a new name");

      this.saveAsButton.addFocusListener(new FocusAdapter() {
        public void focusGained(FocusEvent fe) {
          TabWindow.this.saveAsButton.setBorder(null);
        }
        public void focusLost(FocusEvent e) {
          TabWindow.this.saveAsButton.setBorder(null);
        }
      });
      this.saveAsButton.addMouseListener(new MouseAdapter() {
        public void mouseEntered(MouseEvent me) {
          TabWindow.this.saveAsButton.setContentAreaFilled(true);
        }
        public void mouseExited(MouseEvent me) {
          TabWindow.this.saveAsButton.setContentAreaFilled(false);
        }
      });
      this.saveAsButton.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
        	
        	int state = getJFileChooser().showSaveDialog(saveAsButton);
        	if (state == 0){
	        	File f = getJFileChooser().getSelectedFile();
	        	saveAs = true;
	        	TabWindow.this.saveFile(f);
	        	saveAs = false;
	        	TabWindow.this.loadButton.setEnabled(true);
	        	Splash();	
        	}
        	
        } } );
    }
    return this.saveAsButton;
  }

  
 
  private JButton getLogOffButton(){
    if (this.LogOffButton == null) {
      this.LogOffButton = new JButton();
      this.LogOffButton.setPreferredSize(new Dimension(34, 34));
      this.LogOffButton.setContentAreaFilled(false);
      this.LogOffButton.setFocusPainted(false);
      this.LogOffButton.setIcon(this.icons[30]);
      this.LogOffButton.setToolTipText("Log Off the current user");

      this.LogOffButton.addFocusListener(new FocusAdapter() {
        public void focusGained(FocusEvent fe) {
          TabWindow.this.LogOffButton.setBorder(null);
        }
        public void focusLost(FocusEvent e) {
          TabWindow.this.LogOffButton.setBorder(null);
        }
      });
      this.LogOffButton.addMouseListener(new MouseAdapter() {
        public void mouseEntered(MouseEvent me) {
          TabWindow.this.LogOffButton.setContentAreaFilled(true);
        }
        public void mouseExited(MouseEvent me) {
          TabWindow.this.LogOffButton.setContentAreaFilled(false);
        }
      });
      this.LogOffButton.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
        	
      	  int state = JOptionPane.showConfirmDialog(null, "Are you sure that you want to Log Off of the account: " + security_userLoggedIn + " ?");     //APN0057 added user account info
          if (state == 0)
          {
            TabWindow.this.clearData();
            
            
      	  jTabbedPane1.setEnabled(false);
      	  buttonSetDisable4();
      	  loginWindow.setVisible(true);

          }
          
        } } );
    }
    return this.LogOffButton;
  }
  
  
  private JButton getsecurityButton()
  {
    if (this.securityButton == null) {
      this.securityButton = new JButton();
      this.securityButton.setPreferredSize(new Dimension(34, 34));
      this.securityButton.setContentAreaFilled(false);
      this.securityButton.setFocusPainted(false);
      this.securityButton.setIcon(this.icons[31]);
      this.securityButton.setToolTipText("Security Tools for " + SecurityVersion);
      
      //Create the popup menu.
      final JPopupMenu popup = new JPopupMenu();
      popup.add(new JMenuItem(new AbstractAction("Validate Protocol") {
          public void actionPerformed(ActionEvent e) {

      		if (aprActive.equals("")){
      			JOptionPane.showMessageDialog(null, "A protocol must be saved before Validating!");
      		}else{
      			
        		if (security_auditTrailString.contains("AT0") || security_auditTrailString.contains("AT1")){
        			security_auditTrailString = security_auditTrailString.replace("AT0", "AT1");	//1 indicates that protocol is valid	
        		}
          		security_Validated = true;
          		security_validationTitle = "Validated";
          		security_updateString = "Validate Protocol";
      			File f = new File(aprActive);
      			
      			saveFile(f);
      		}
          }
      }));
      
      popup.add(new JMenuItem(new AbstractAction("View Audit Trail") {
          public void actionPerformed(ActionEvent e) {
        		if (aprActive.equals("")){
          			JOptionPane.showMessageDialog(null, "There is currently no open protocol to display Audit Trail");
          		}else{
          			auditTrail.viewWindow();          			
          		}
         }
      }));
      
      popup.add(new JMenuItem(new AbstractAction("Edit Secure Directories") {
          public void actionPerformed(ActionEvent e) {
        	  directorySelect.setVisible(true);
         }
      }));
      
      this.securityButton.addFocusListener(new FocusAdapter() {
        public void focusGained(FocusEvent fe) {
          TabWindow.this.securityButton.setBorder(null);
        }
        public void focusLost(FocusEvent e) {
          TabWindow.this.securityButton.setBorder(null);
        }
      });
      this.securityButton.addMouseListener(new MouseAdapter() {
          public void mousePressed(MouseEvent e) {
        	  if (securityButton.isEnabled()){
        		popup.show(e.getComponent(), e.getX(), e.getY());  
        	  }
          }
        public void mouseEntered(MouseEvent me) {
          TabWindow.this.securityButton.setContentAreaFilled(true);
        }
        public void mouseExited(MouseEvent me) {
          TabWindow.this.securityButton.setContentAreaFilled(false);
        } } );
    }
    return this.securityButton;
  }
  
  private JButton getsecurityAccountButton(){
	    if (this.securityAccountButton == null) {
	      this.securityAccountButton = new JButton();
	      this.securityAccountButton.setPreferredSize(new Dimension(34, 34));
	      this.securityAccountButton.setContentAreaFilled(false);
	      this.securityAccountButton.setFocusPainted(false);
	      this.securityAccountButton.setIcon(this.icons[33]);
	      this.securityAccountButton.setToolTipText("Create User Account");

	      this.securityAccountButton.addFocusListener(new FocusAdapter() {
	        public void focusGained(FocusEvent fe) {
	          TabWindow.this.securityAccountButton.setBorder(null);
	        }
	        public void focusLost(FocusEvent e) {
	          TabWindow.this.securityAccountButton.setBorder(null);
	        }
	      });
	      this.securityAccountButton.addMouseListener(new MouseAdapter() {
	        public void mouseEntered(MouseEvent me) {
	          TabWindow.this.securityAccountButton.setContentAreaFilled(true);
	        }
	        public void mouseExited(MouseEvent me) {
	          TabWindow.this.securityAccountButton.setContentAreaFilled(false);
	        }
	      });
	      this.securityAccountButton.addActionListener(new ActionListener() {
	          public void actionPerformed(ActionEvent e) {
	        	  userAccountWindow.loadAccount();
	        	  userAccountWindow.setAlwaysOnTop(true);
	        	  userAccountWindow.setLocationRelativeTo(null);  
	        	  userAccountWindow.setVisible(true);
	          }      
	      } );
	    }
	    return this.securityAccountButton;
	  }
  
  
  private JButton getLoadButton()
  {
    if (this.loadButton == null) {
      this.loadButton = new JButton();
      this.loadButton.setPreferredSize(new Dimension(34, 34));
      this.loadButton.setContentAreaFilled(false);
      this.loadButton.setFocusPainted(false);
      this.loadButton.setIcon(this.icons[2]);

      this.loadButton.setToolTipText("Open an existing Automation Protocol");
      this.loadButton.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          if (TabWindow.this.loadFile() == true){//loadFile() returns True if the operator does not Cancel during the chooser dialogue.  Then the next code is executed.
        	  jTabbedPane1.setToolTipText(null);

        	  Splash();  
          }
        }
      });
      this.loadButton.addFocusListener(new FocusAdapter() {
        public void focusGained(FocusEvent fe) {
          TabWindow.this.loadButton.setBorder(null);
        }
        public void focusLost(FocusEvent e) {
          TabWindow.this.loadButton.setBorder(null);
        }
      });
      this.loadButton.addMouseListener(new MouseAdapter() {
        public void mouseEntered(MouseEvent me) {
          TabWindow.this.loadButton.setContentAreaFilled(true);
        }
        public void mouseExited(MouseEvent me) {
          TabWindow.this.loadButton.setContentAreaFilled(false);
        } } );
    }
    return this.loadButton;
  }

  private JButton getLayoutSelectButton()
  {
    if (this.layoutSelectButton == null) {
      this.layoutSelectButton = new JButton();
      this.layoutSelectButton.setPreferredSize(new Dimension(34, 34));
      this.layoutSelectButton.setContentAreaFilled(false);
      this.layoutSelectButton.setFocusPainted(false);
      this.layoutSelectButton.setIcon(this.icons[17]);

      this.layoutSelectButton.setToolTipText("Select the Deck Layout for this protocol");
      this.layoutSelectButton.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          
        	if (TabWindow.this.layoutSelect() == 0){
        		jTabbedPane1.setToolTipText(null);
        		Splash();
        	}

        }
      });
      this.layoutSelectButton.addFocusListener(new FocusAdapter() {
        public void focusGained(FocusEvent fe) {
          TabWindow.this.layoutSelectButton.setBorder(null);
        }
        public void focusLost(FocusEvent e) {
          TabWindow.this.layoutSelectButton.setBorder(null);
        }
      });
      this.layoutSelectButton.addMouseListener(new MouseAdapter() {
        public void mouseEntered(MouseEvent me) {
          TabWindow.this.layoutSelectButton.setContentAreaFilled(true);
        }
        public void mouseExited(MouseEvent me) {
          TabWindow.this.layoutSelectButton.setContentAreaFilled(false);
        } } );
    }
    return this.layoutSelectButton;
  }

  private JButton getRmvButton()
  {
    if (this.rmvButton == null) {
      this.rmvButton = new JButton();

      this.rmvButton.setHorizontalAlignment(0);
      this.rmvButton.setPreferredSize(new Dimension(34, 34));
      this.rmvButton.setContentAreaFilled(false);
      this.rmvButton.setFocusPainted(false);
      this.rmvButton.setIcon(this.icons[5]);
      this.rmvButton.setToolTipText("Delete the selected step!");
      this.rmvButton.setEnabled(false);

      this.rmvButton.addFocusListener(new FocusAdapter() {
        public void focusGained(FocusEvent fe) {
          TabWindow.this.rmvButton.setBorder(null);
        }
        public void focusLost(FocusEvent e) {
          TabWindow.this.rmvButton.setBorder(null);
        }
      });
      this.rmvButton.addMouseListener(new MouseAdapter() {
        public void mouseEntered(MouseEvent me) {
          TabWindow.this.rmvButton.setContentAreaFilled(true);
        }
        public void mouseExited(MouseEvent me) {
          TabWindow.this.rmvButton.setContentAreaFilled(false);
        }
      });
      this.rmvButton.addActionListener(new ActionListener()
      {
        public void actionPerformed(ActionEvent e)
        {
        	
            int rmvCheck = JOptionPane.showOptionDialog(null, 
                    "Delete this step?", 
                    null, 
                    0, 
                    3, 
                    null, null, null);

                  if (rmvCheck == 0) {
                	  TabWindow.this.rmvAnyLink(TabWindow.this.stepnbr);
                	  Splash();
                  }
          
        } } );
    }
    return this.rmvButton;
  }

  private JButton getCheckButton()
  {
    if (this.checkButton == null) {
      this.checkButton = new JButton();
      this.checkButton.setEnabled(false);
      this.checkButton.setHorizontalAlignment(0);
      this.checkButton.setPreferredSize(new Dimension(34, 34));
      this.checkButton.setContentAreaFilled(false);
      this.checkButton.setFocusPainted(false);
      this.checkButton.setIcon(this.icons[10]);
      this.checkButton.setEnabled(false);

      this.checkButton.addFocusListener(new FocusAdapter() {
        public void focusGained(FocusEvent fe) {
          TabWindow.this.checkButton.setBorder(null);
        }
        public void focusLost(FocusEvent e) {
          TabWindow.this.checkButton.setBorder(null);
        }
      });
      this.checkButton.addMouseListener(new MouseAdapter() {
        public void mouseEntered(MouseEvent me) {
          TabWindow.this.checkButton.setContentAreaFilled(true);
        }
        public void mouseExited(MouseEvent me) {
          TabWindow.this.checkButton.setContentAreaFilled(false);
        }
      });
      this.checkButton.addActionListener(new ActionListener()
      {
        public void actionPerformed(ActionEvent e)
        {
          if (TabWindow.this.updButton1.isEnabled()) TabWindow.this.updButton1.doClick();

          if (TabWindow.this.updButton10.isEnabled()) TabWindow.this.updButton10.doClick();

          TabWindow.this.checkButton.setEnabled(false);
        } } );
    }
    return this.checkButton;
  }

  private JButton getAddButton()
  {
    if (this.addButton == null) {
      this.addButton = new JButton();

      this.addButton.setHorizontalAlignment(0);
      this.addButton.setPreferredSize(new Dimension(34, 34));
      this.addButton.setContentAreaFilled(false);
      this.addButton.setFocusPainted(false);
      this.addButton.setIcon(this.icons[9]);

      this.addButton.addFocusListener(new FocusAdapter() {
        public void focusGained(FocusEvent fe) {
          TabWindow.this.addButton.setBorder(null);
        }
        public void focusLost(FocusEvent e) {
          TabWindow.this.addButton.setBorder(null);
        }
      });
      this.addButton.addMouseListener(new MouseAdapter() {
        public void mouseEntered(MouseEvent me) {
          TabWindow.this.addButton.setContentAreaFilled(true);
        }
        public void mouseExited(MouseEvent me) {
          TabWindow.this.addButton.setContentAreaFilled(false);
        }
      });
      this.addButton.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          if (TabWindow.this.addButton1.isVisible()) TabWindow.this.addButton1.doClick();

          if (TabWindow.this.addButton10.isVisible()) TabWindow.this.addButton10.doClick();
        }
      });
    }
    return this.addButton;
  }

  private JButton getInfoButton()
  {
    if (this.infoButton == null) {
      this.infoButton = new JButton();

      this.infoButton.setHorizontalAlignment(0);
      this.infoButton.setPreferredSize(new Dimension(34, 34));
      this.infoButton.setContentAreaFilled(false);
      this.infoButton.setFocusPainted(false);
      this.infoButton.setIcon(this.icons[11]);
      this.infoButton.setToolTipText("HELP");

      this.infoButton.addFocusListener(new FocusAdapter() {
        public void focusGained(FocusEvent fe) {
          TabWindow.this.infoButton.setBorder(null);
        }
        public void focusLost(FocusEvent e) {
          TabWindow.this.infoButton.setBorder(null);
        }
      });
      this.infoButton.addMouseListener(new MouseAdapter() {
        public void mouseEntered(MouseEvent me) {
          TabWindow.this.infoButton.setContentAreaFilled(true);
        }
        public void mouseExited(MouseEvent me) {
          TabWindow.this.infoButton.setContentAreaFilled(false);
        }
      });
      this.infoButton.addActionListener(new ActionListener()
      {
        public void actionPerformed(ActionEvent e)
        {
          try
          {

            Process p;
            if (new File("C:\\Program Files\\TELOS\\AutoPilot\\Config\\APHelp.pdf").exists()) {
              p = 
                Runtime.getRuntime()
                .exec("rundll32 url.dll,FileProtocolHandler C:\\Program Files\\TELOS\\AutoPilot\\Config\\APHelp.pdf");
            }
            else
              JOptionPane.showMessageDialog(null, "Help file not found.  Please contact Telos Scientific for assistance.");
          }
          catch (IOException e1) {
            e1.printStackTrace();
          }
        } } );
    }
    return this.infoButton;
  }

  private JButton getLayoutButton()
  {
    if (this.layoutButton == null) {
      this.layoutButton = new JButton();

      this.layoutButton.setHorizontalAlignment(0);
      this.layoutButton.setPreferredSize(new Dimension(34, 34));
      this.layoutButton.setContentAreaFilled(false);
      this.layoutButton.setFocusPainted(false);
      this.layoutButton.setIcon(this.icons[13]);
      this.layoutButton.setToolTipText("View the Deck Layout");

      this.layoutButton.addFocusListener(new FocusAdapter() {
        public void focusGained(FocusEvent fe) {
          TabWindow.this.layoutButton.setBorder(null);
        }
        public void focusLost(FocusEvent e) {
          TabWindow.this.layoutButton.setBorder(null);
        }
      });
      this.layoutButton.addMouseListener(new MouseAdapter() {
        public void mouseEntered(MouseEvent me) {
          TabWindow.this.layoutButton.setContentAreaFilled(true);
        }
        public void mouseExited(MouseEvent me) {
          TabWindow.this.layoutButton.setContentAreaFilled(false);
        }
      });
      this.layoutButton.addActionListener(new ActionListener()
      {
        public void actionPerformed(ActionEvent e)
        {
          try
          {
            Process p;
            if (new File(TabWindow.this.layoutFileSelected).exists()) {
              String layoutExecuteString = "rundll32 url.dll,FileProtocolHandler " + TabWindow.this.layoutFileSelected;
              p = 
                Runtime.getRuntime()
                .exec(layoutExecuteString);
            }
            else {
              JOptionPane.showMessageDialog(null, "You must first associate a Deck Layout by clicking on red exclamation mark");
            }
          } catch (IOException e1) {
            e1.printStackTrace();
          }
        } } );
    }
    return this.layoutButton;
  }

  private JButton getRunButton()
  {
    if (this.runButton == null) {
      this.runButton = new JButton();
      this.runButton.setHorizontalAlignment(0);
      this.runButton.setPreferredSize(new Dimension(34, 34));
      this.runButton.setContentAreaFilled(false);
      this.runButton.setFocusPainted(false);
      this.runButton.setIcon(this.icons[14]);
      this.runButton.setToolTipText("Run this protocol!");

      this.runButton.addFocusListener(new FocusAdapter() {
        public void focusGained(FocusEvent fe) {
          TabWindow.this.runButton.setBorder(null);
        }
        public void focusLost(FocusEvent e) {
          TabWindow.this.runButton.setBorder(null);
        }
      });
      this.runButton.addMouseListener(new MouseAdapter() {
        public void mouseEntered(MouseEvent me) {
          TabWindow.this.runButton.setContentAreaFilled(true);
        }
        public void mouseExited(MouseEvent me) {
          TabWindow.this.runButton.setContentAreaFilled(false);
        }
      });
      this.runButton.addActionListener(new ActionListener()
      {
        public void actionPerformed(ActionEvent e) {
        	run_Started = true;									//APN0057  Run starts here so set to true.
	        run_Completed = false;
        	while ((run_Completed == false)){
        		run_Completed = true;
	         	boolean run;
		        boolean securityConfirmed = true;
		        
		        if (security){									//security turned on
		        	if (security_accessLevel.equals("1")){		//access level = Operator
		        		if (!security_Validated){				//Method Not Validated
		        			JOptionPane.showMessageDialog(null, "The protocol is Not Validated and cannot be used for routine operation.");
		        			securityConfirmed = false;
		        			run_Started = false;				//APN0057  Run ends here so set to false.
		        			return;
		        		}
		        	}
		        	
	        		if (cntPK>0 && security_accessLevel.equals("1")){
	        			int state = getJFileChooser5().showOpenDialog(null);  
	        			if (state != 0){
	        				JOptionPane.showMessageDialog(null, "You must select a worklist file to continue with the PK run!");
	        				run_Started = false;				//APN0057  Run ends here so set to false.
	        				return;
	        			}else{																				//APN0051
	        				String filePath = jFileChooser5.getSelectedFile().getPath();					//APN0051
	        				if (VAStools.PK_WorklistConfirm(filePath)){										//APN0051
	        					security_pKWorklist_Runtime = filePath; 									//APN0051
	        				}else{																			//APN0051
	        					run_Started = false;										//APN0057  Run ends here so set to false.
	        					return;																		//APN0051
	        				}																				//APN0051
	        			}																					//APN0051
	        		}	
	        	}
		        
		        if (securityConfirmed){
			        VAStools.RunID(); 													//Create new runID at runtime
			        APLOG.Clear(); //Clear Internal Log
			        
			        String VASerror = "";
			        //Check for VAS errors
			        VASerror = saveVASi();
		          
		            //Continue RUN routine if not VAS errors
		            if (VASerror.equals("")){
		            	run = true;
		            	//Save Routine
		                if (TabWindow.this.aprActive.equals("")){
		          			int state = getJFileChooser().showSaveDialog(saveAsButton);
		              		if (state==0){
			              		File f = getJFileChooser().getSelectedFile();
			              		if (TabWindow.this.saveFile(f) == false){run = false;}
		              		}else{run = false;}
		          	  	}else{
		          	  		run = false;
		          	  		int saveCheck = 1;
		          	  		if (security_accessLevel.equals("1") && security_Validated && cntPK>0){				//APN0051
		          	  			saveCheck = 0;																	//APN0051  Will Automatically save in order to update with runtime selected worklist.  No other modifications are possible because Operator is logged in
		          	  		}else if(security_accessLevel.equals("1") && security_Validated){
		          	  			
		          	  		}else{
			          	  		saveCheck = JOptionPane.showOptionDialog(null, 
			                  	"Save protocol before Running?", 
			                  	null, 
			                  	1, 
			                  	3, 
			                  	null, null, null);	
		          	  		}
		          		  	
		
		                	if (saveCheck == 0) {//Yes selected
		                		
		                    	if (security && !security_accessLevel.equals("1")){	
		                    		if (security_auditTrailString.contains("AT0") || security_auditTrailString.contains("AT1")){
		                    			security_auditTrailString = security_auditTrailString.replace("AT1", "AT0");	//1 indicates that protocol is valid	
		                    		}
		            	      		security_Validated = false;
		            	      		security_validationTitle = "Not Validated";
		            	      		security_updateString = "Save Protocol";
		                    	}	
		                    	
		                		File f = new File(aprActive);
		                		if (TabWindow.this.saveFile(f) == false){run = false;}else{run = true;}				//SAVE HERE
		                		
		                	}else{
		                		
		                	}
		                	if (saveCheck == 1) {//No Selected
		                		run = true;
		                	}
		          	  	}
		                //Run Execution
			            if (run){
			                	
				            try
				            {
				              InputStream in = new FileInputStream(TabWindow.this.layoutFileSelected);
				              OutputStream out = new FileOutputStream(VASengine.BitPath + "HAMILTON\\Methods\\AutoPilot\\AP.lay");
				              byte[] buf = new byte[1024];
				              int len;
				              while ((len = in.read(buf)) > 0)
				              {
				            	 
				                out.write(buf, 0, len);
				              }
				              in.close();
				              out.close();
				            }
				            catch (IOException e1) {
				              JOptionPane.showMessageDialog(null, "Error: Failed to create layout file");
				              e1.printStackTrace();
				            }          	
				              
				              try
				              {
				                PrintStream out = new PrintStream(new FileOutputStream("C:\\Program Files\\TELOS\\AutoPilot\\Config\\APrun.cfg"));
				                out.println(TabWindow.this.aprActive);
				                out.close();
				              } catch (FileNotFoundException f) {
				                f.printStackTrace();
				              }
				              
				              try
				              {
				              	if (!VAStools.isRunning("HxRun.exe")){
			
				              		String p_str;
				              		
				              		//Process p;
				  	            	if (new File(VASengine.BitPath+"Hamilton\\Methods\\AutoPilot\\AP.hsl").exists()){  
			
				  	            		String runString = "\"" + VASengine.BitPath + "HAMILTON\\BIN\\HXRUN.EXE\" \"" + VASengine.BitPath + "HAMILTON\\METHODS\\AutoPilot\\AP.hsl\" /t";
				  	            		
				  	            		p = Runtime.getRuntime().exec(runString);
		
				  	            		
				  	            		//p = Runtime.getRuntime().exec("\"C:\\PROGRAM FILES\\HAMILTON\\BIN\\HXRUN.EXE\" \"C:\\PROGRAM FILES\\HAMILTON\\METHODS\\AutoPilot\\AP.hsl\" /t");
				  	            		//p = Runtime.getRuntime().exec("rundll32 url.dll,FileProtocolHandler "+VASengine.BitPath+"Hamilton\\Methods\\AutoPilot\\AP.hsl"); 
				  	            		 Runtime.getRuntime (  ) .addShutdownHook ( new Thread (  )   {  
				  			 				   public void run (  )   {  
				  			 				       p.destroy(); 
				  			 				    }  
				  			 			 });
				  			 				
				  			 				BufferedReader p_in = new BufferedReader(new InputStreamReader(p.getInputStream()));
				  			 				BufferedReader p_err = new BufferedReader(new InputStreamReader(p.getErrorStream()));
				  			 				System.out.println("AP::Venus has been Started");
		
				  			 				try {
				  			 					while ((p_str = p_in.readLine()) != null) {
				  			 						System.out.println("VenusCom::" + p_str);//Print any/all output from Venus
				  			 						APLOG.SetVenus(p_str);
				  			 					
				  			 					}
				  			 					 VASPostProcessing.Execute();  //Execute any required post processing routines
				  			 					
				  			 					
				  			 				}catch (IOException e1) {
				  		 			 			// TODO Auto-generated catch block
				  		 			 			e1.printStackTrace();
				  		 			 		}
				  	            	
				  	            	}else{
				  	            		JOptionPane.showMessageDialog(null, "Venus component failure 001.  Please contact Telos Scientific for assistance.");
				  	            	}	
				              	}else{
				              		VAStools.SetFocusVenus();	
				              	}
				              		
				              }
				                
				              catch (IOException e1) {
				              	JOptionPane.showMessageDialog(null, e1);
				              	e1.printStackTrace();
				              }
			           }else{JOptionPane.showMessageDialog(null, "The Run was cancelled");}
		            }else{
		            	JOptionPane.showMessageDialog(null, "Sorry for the delay.\n\nVAS can't run your protocol.");
		            }	        	
		        }       		
        	
        	//Run Completed Determinations
		        if (cntPK>0 && security_accessLevel.equals("1")){
          	  		if (JOptionPane.showOptionDialog(null, 
		                  	"Would you like to load another Worklist?", 
		                  	null, 
		                  	0, 
		                  	3, 
		                  	null, null, null)   == 0){
          	  			run_Completed = false;
          	  		};
		        }
        	}
        	run_Started = false;				//APN0057  Run ends here so set to false.
        }
      });
    }
    return this.runButton;
  }

  private JButton getNewButton()
  {
    if (this.newButton == null) {
      this.newButton = new JButton();
      this.newButton.setHorizontalAlignment(0);

      this.newButton.setPreferredSize(new Dimension(34, 34));
      this.newButton.setContentAreaFilled(false);
      this.newButton.setFocusPainted(false);
      this.newButton.setIcon(this.icons[4]);
      this.newButton.setToolTipText("Create new protocol (this will clear current protocol!)");

      this.newButton.addFocusListener(new FocusAdapter() {
        public void focusGained(FocusEvent fe) {
          TabWindow.this.newButton.setBorder(null);
        }
        public void focusLost(FocusEvent e) {
          TabWindow.this.newButton.setBorder(null);
        }
      });
      this.newButton.addMouseListener(new MouseAdapter() {
        public void mouseEntered(MouseEvent me) {
          TabWindow.this.newButton.setContentAreaFilled(true);
        }
        public void mouseExited(MouseEvent me) {
          TabWindow.this.newButton.setContentAreaFilled(false);
        }
      });
      this.newButton.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          int state = JOptionPane.showConfirmDialog(null, "Are you sure you want to clear the existing protocol?");
          if (state == 0)
          {
            TabWindow.this.clearData();
            TabWindow.this.loadButton.setEnabled(true);
            
      	  jTabbedPane1.setEnabled(false);
    	  buttonSetDisable1();
    	  
    	  addButton.setEnabled(false);
    	  checkButton.setEnabled(false);
    	  rmvButton.setEnabled(false);
    	  
    	  layoutSelectButton.setEnabled(true);
          }
          else
          {
            return;
          }
        } } );
    }
    return this.newButton;
  }

  private JButton getAddButton1()
  {
    if (this.addButton1 == null) {
      this.addButton1 = new JButton();
      this.addButton1.setSize(10, 10);
      this.addButton1.setText("              Add Step");
      this.addButton1.setIcon(this.icons[15]);
      this.addButton1.setToolTipText("Add this step to the protocol");
      this.addButton1.setHorizontalAlignment(2);
      this.addButton1.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {

          if (AdvWindow.frameActive) {advwin.setVisible(true);}
          else if(AdvWindowD.frameActive){advwind.setVisible(true);}else{
        	  
          int chkspl = 1; int chkdpl = 1;
          String vol = TabWindow.this.text3.getText().trim();
          TabWindow.this.mix1 = TabWindow.this.lmixt1.getText().trim();
          if (TabWindow.this.mix1.length() < 1) {
            TabWindow.this.valuem1 = 0;
            TabWindow.this.mix1 = "0";
          } else {
            try {
              TabWindow.this.valuem1 = Integer.parseInt(TabWindow.this.mix1);
            } catch (NumberFormatException exp) {
              TabWindow.this.valuem1 = 0;
              TabWindow.this.lmixt1.setText("0");
            }
          }
          TabWindow.this.mix2 = TabWindow.this.lmixt2.getText().trim();
          if (TabWindow.this.mix2.length() < 1) {
            TabWindow.this.valuem2 = 0;
            TabWindow.this.mix2 = "0";
          } else {
            try {
              TabWindow.this.valuem2 = Integer.parseInt(TabWindow.this.mix2);
            } catch (NumberFormatException exp) {
              TabWindow.this.valuem2 = 0;
              TabWindow.this.lmixt2.setText("0");
            }
          }
          if (vol.length() < 1)
            TabWindow.this.value = 0;
          else
            try {
              TabWindow.this.value = Integer.parseInt(vol);
            } catch (NumberFormatException exp) {
              TabWindow.this.value = 0;
              TabWindow.this.text3.setText("");
            }
          try
          {
            chkspl = Integer.parseInt(TabWindow.this.spl1);
          } catch (NumberFormatException exp) {
            chkspl = 0;
          }
          try {
            chkdpl = Integer.parseInt(TabWindow.this.dpl1);
          } catch (NumberFormatException exp) {
            chkdpl = 0;
          }
          
          boolean pass = true;																									//APN0054
          //SOURCE																												//APN0054
          if (s1.trim().endsWith("Tube")){																						//APN0054
              if (tubepanel.getFirstTube().equals("")){pass = false;}															//APN0054
              if (tubepanel.getLastTube().equals("")){pass = false;};															//APN0054
              if (!pass){JOptionPane.showMessageDialog(null, "Please input the first and last source tube");}					//APN0054
      	  }else if(s1.trim().contains("Select tube type")){																		//APN0054 
      		  pass = false;																										//APN0054 
      		  JOptionPane.showMessageDialog(null, "Please select the source tube type");										//APN0054 
      	  }else{																												//APN0054
	          if (s1.trim().endsWith("_24")){																					//APN0054
	        	  pass = panel24l.CheckTray();																					//APN0054
	          }																													//APN0054
	          if (s1.trim().endsWith("_48")){																					//APN0054
	        	  pass = panel48l.CheckTray();																					//APN0054
	          }																													//APN0054
	          if (s1.trim().endsWith("_96")){																					//APN0054
	              pass = panel96l.CheckTray();																					//APN0054
	          }																													//APN0054
	          if ((s1.trim().endsWith("_384"))){																				//APN0054
	        	  pass = panel384l.CheckTray();																					//APN0054
	          }																													//APN0054
	          																													//APN0054
	          if (!pass){JOptionPane.showMessageDialog(null, "Please select the source well/s");}								//APN0054
      	  }																														//APN0054         
          //DESTINATION																											//APN0054
          if (pass){																											//APN0054
	          if (de.trim().endsWith("Tube")){																					//APN0054
	              if (tubepanel2.getFirstTube().equals("")){pass = false;}														//APN0054
	              if (tubepanel2.getLastTube().equals("")){pass = false;};														//APN0054
	              if (!pass){JOptionPane.showMessageDialog(null, "Please input the first and last destination tube");}			//APN0054
	          }else if(de.trim().contains("Select tube type")){																	//APN0054 
	      		  pass = false;																									//APN0054 
	      		  JOptionPane.showMessageDialog(null, "Please select the destination tube type");								//APN0054 
	      	  }else{																											//APN0054
		          if (de.trim().endsWith("_24")){																				//APN0054
		        	  pass = panel24r.CheckTray();																				//APN0054
		          }																												//APN0054
		          if (de.trim().endsWith("_48")){																				//APN0054
		        	  pass = panel48r.CheckTray();																				//APN0054
		          }																												//APN0054
		          if (de.trim().endsWith("_96")){																				//APN0054
		        	  pass = panel96r.CheckTray();																				//APN0054
		          }																												//APN0054
		          if (de.trim().endsWith("_384"))																				//APN0054
		          {																												//APN0054
		        	  pass = panel384r.CheckTray();																				//APN0054
		          }	          																									//APN0054
		          if (!pass){JOptionPane.showMessageDialog(null, "Please select the destination well/s");}						//APN0054
	          }        	  																										//APN0054
          }																														//APN0054

          //APN0055
          if ((TabWindow.this.text2.getText().contains(",")) || (TabWindow.this.text2.getText().contains("/")) || (TabWindow.this.text2.getText().contains("\\")) || (TabWindow.this.text2.getText().contains("*")) || (TabWindow.this.text2.getText().contains("<")) || (TabWindow.this.text2.getText().contains(">")) || (TabWindow.this.text2.getText().contains("?")) || (TabWindow.this.text2.getText().contains("|")))
          {
            JOptionPane.showMessageDialog(null, "The 'Reagent Name' can't contain any of the following carachters: \n \\ / , * < > ? | \"", " ", 2);
            pass = false;
          }
          
          
          if ((TabWindow.this.text2.getText().trim().equals("")) || (TabWindow.this.value == 0) || ((chkspl == 0) && (!TabWindow.this.tubebutton.isSelected()))  || (s1.equals("Plate type") && (!TabWindow.this.tubebutton.isSelected())) || ((chkdpl == 0) && (!TabWindow.this.tubebutton2.isSelected())) || (de.equals("Plate type") && (!TabWindow.this.tubebutton2.isSelected())) || (TabWindow.this.valuem1 > 1000) || (TabWindow.this.valuem2 > 1000) || ((TabWindow.this.resbutton.isSelected()) && (TabWindow.this.rs1.equals(""))) || ((TabWindow.this.resbutton2.isSelected()) && (TabWindow.this.rs2.equals(""))))
          {
            if ((TabWindow.this.text2.getText().trim().equals("")) || (TabWindow.this.value == 0))
            {
              JOptionPane.showMessageDialog(null, "Invalid Reagent Data Entered", " ", 2);
            }

            if ((s1.equals("Plate type")) && (!TabWindow.this.tubebutton.isSelected()))
            {
              JOptionPane.showMessageDialog(null, "Please select the Source Plate Type", " ", 2);
            }
            if ((chkspl == 0) && (!TabWindow.this.tubebutton.isSelected()))
            {
              JOptionPane.showMessageDialog(null, "Please select the Source Deck Position", " ", 2);
            }
            if ((chkdpl == 0) && (!TabWindow.this.tubebutton2.isSelected()))
            {
              JOptionPane.showMessageDialog(null, "Please select the Destination Deck Position", " ", 2);
            }
            if ((de.equals("Plate type")) && (!TabWindow.this.tubebutton2.isSelected()))
            {
              JOptionPane.showMessageDialog(null, "Please select the Destination Plate Type", " ", 2);
            }
            if ((TabWindow.this.valuem1 > 1000) || (TabWindow.this.valuem2 > 1000))
            {
              JOptionPane.showMessageDialog(null, "Invalid Mix volume", " ", 2);
            }
            if ((TabWindow.this.resbutton.isSelected()) && (TabWindow.this.rs1.equals("")))
            {
              JOptionPane.showMessageDialog(null, "Please click on the source reservoir", " ", 2);
            }
            if ((TabWindow.this.resbutton2.isSelected()) && (TabWindow.this.rs2.equals("")))
            {
              JOptionPane.showMessageDialog(null, "Please click on the destination reservoir", " ", 2);
            }
            pass = false;
          }


          if (pass)
          {
            TabWindow.this.addReg();
          }
          
        } }} );
    }
    return this.addButton1;
  }

  private JButton getAddButton5() {
    if (this.addButton5 == null) {
      this.addButton5 = new JButton();
      this.addButton5.setSize(20, 20);
      this.addButton5.setText("     Add Step       ");
      this.addButton5.setHorizontalAlignment(0);
      this.addButton5.setIcon(this.icons[9]);

      this.addButton5.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          TabWindow.this.addSPE();
        } } );
    }
    return this.addButton5;
  }

  private JButton getAddButton2()
  {
    if (this.addButton2 == null) {
      this.addButton2 = new JButton();
      this.addButton2.setSize(20, 20);
      this.addButton2.setText("     Add Step       ");
      this.addButton2.setHorizontalAlignment(0);
      this.addButton2.setIcon(this.icons[9]);
      this.addButton2.addActionListener(new ActionListener()
      {
        public void actionPerformed(ActionEvent e) {
          String min = TabWindow.this.text4.getText().trim();
          String min2 = TabWindow.this.text4pm.getText().trim();
          if (min.length() < 1)
            TabWindow.this.value = 0;
          else {
            try {
              TabWindow.this.value = Integer.parseInt(min);
            } catch (NumberFormatException exp) {
              TabWindow.this.value = 0;
              TabWindow.this.text4.setText("");
            }
          }
          if ((min2.length() > 1) && (!min2.equals("0"))) {
            try {
              TabWindow.this.value = Integer.parseInt(min2);
            } catch (NumberFormatException exp) {
              TabWindow.this.value = 0;
              TabWindow.this.text4pm.setText("0");
            }
          }

          if (TabWindow.this.value == 0)
          {
            JOptionPane.showMessageDialog(null, "Invalid Incubation Data Entered", " ", 2);
          }
          else
          {
            TabWindow.this.addInc();
          }
        }
      });
    }
    return this.addButton2;
  }

  private JButton getAddButton3()
  {
    if (this.addButton3 == null) {
      this.addButton3 = new JButton();
      this.addButton3.setSize(20, 20);
      this.addButton3.setText("     ADD STEP       ");
      this.addButton3.setHorizontalAlignment(0);
      this.addButton3.setIcon(this.icons[9]);
      this.addButton3.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          TabWindow.this.addWash();
          Splash();
        } } );
    }
    return this.addButton3;
  }

  private JButton getUpdButton1()
  {
    if (this.updButton1 == null) {
      this.updButton1 = new JButton();
      this.updButton1.setEnabled(false);
      this.updButton1.setSize(20, 20);
      this.updButton1.setText("           Update Step");
      this.updButton1.setIcon(this.icons[16]);
      this.updButton1.setToolTipText("Save changes to current step");
      this.updButton1.setHorizontalAlignment(2);
      this.updButton1.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
       
    	  if (AdvWindow.frameActive) {advwin.setVisible(true);}
          else if(AdvWindowD.frameActive){advwind.setVisible(true);}else{
        		 
        	 
	          for (int d = 0; d < 15; d++) TabWindow.this.dTextArea[d].setToolTipText("Click to edit the protocol");
	          int chkspl = 0; int chkdpl = 0;
	          String vol = TabWindow.this.text3.getText().trim();
	          TabWindow.this.mix1 = TabWindow.this.lmixt1.getText().trim();
	          if (TabWindow.this.mix1.length() < 1) {
	            TabWindow.this.valuem1 = 0;
	            TabWindow.this.mix1 = "0";
	          } else {
	            try {
	              TabWindow.this.valuem1 = Integer.parseInt(TabWindow.this.mix1);
	            } catch (NumberFormatException exp) {
	              TabWindow.this.valuem1 = 0;
	              TabWindow.this.lmixt1.setText("0");
	            }
	          }
	          TabWindow.this.mix2 = TabWindow.this.lmixt2.getText().trim();
	          if (TabWindow.this.mix2.length() < 1) {
	            TabWindow.this.valuem2 = 0;
	            TabWindow.this.mix2 = "0";
	          } else {
	            try {
	              TabWindow.this.valuem2 = Integer.parseInt(TabWindow.this.mix2);
	            } catch (NumberFormatException exp) {
	              TabWindow.this.valuem2 = 0;
	              TabWindow.this.lmixt2.setText("0");
	            }
	          }
	          if (vol.length() < 1)
	            TabWindow.this.value = 0;
	          else
	            try {
	              TabWindow.this.value = Integer.parseInt(vol);
	            } catch (NumberFormatException exp) {
	              TabWindow.this.value = 0;
	              TabWindow.this.text3.setText("");
	            }
	          try
	          {
	            chkspl = Integer.parseInt(TabWindow.this.spl1);
	          } catch (NumberFormatException exp) {
	            chkspl = 0;
	          }
	          try {
	            chkdpl = Integer.parseInt(TabWindow.this.dpl1);
	          } catch (NumberFormatException exp) {
	            chkdpl = 0;
	          }
	
	          boolean pass = true;																									//APN0054
	          //SOURCE																												//APN0054
	          if (s1.trim().endsWith("Tube")){																						//APN0054
	              if (tubepanel.getFirstTube().equals("")){pass = false;}															//APN0054
	              if (tubepanel.getLastTube().equals("")){pass = false;};															//APN0054
	              if (!pass){JOptionPane.showMessageDialog(null, "Please input the first and last source tube");}					//APN0054
	      	  }else if(s1.trim().contains("Select tube type")){																		//APN0054 
	      		  pass = false;																										//APN0054 
	      		  JOptionPane.showMessageDialog(null, "Please select the source tube type");										//APN0054 
	      	  }else{																												//APN0054
		          if (s1.trim().endsWith("_24")){																					//APN0054
		        	  pass = panel24l.CheckTray();																					//APN0054
		          }																													//APN0054
		          if (s1.trim().endsWith("_48")){																					//APN0054
		        	  pass = panel48l.CheckTray();																					//APN0054
		          }																													//APN0054
		          if (s1.trim().endsWith("_96")){																					//APN0054
		              pass = panel96l.CheckTray();																					//APN0054
		          }																													//APN0054
		          if ((s1.trim().endsWith("_384"))){																				//APN0054
		        	  pass = panel384l.CheckTray();																					//APN0054
		          }																													//APN0054
		          if (!pass){JOptionPane.showMessageDialog(null, "Please select the source well/s");}								//APN0054
	      	  }																														//APN0054         
	          //DESTINATION																											//APN0054
	          if (pass){																											//APN0054
		          if (de.trim().endsWith("Tube")){																					//APN0054
		              if (tubepanel2.getFirstTube().equals("")){pass = false;}														//APN0054
		              if (tubepanel2.getLastTube().equals("")){pass = false;};														//APN0054
		              if (!pass){JOptionPane.showMessageDialog(null, "Please input the first and last destination tube");}			//APN0054
		          }else if(de.trim().contains("Select tube type")){																	//APN0054 
		      		  pass = false;																									//APN0054 
		      		  JOptionPane.showMessageDialog(null, "Please select the destination tube type");								//APN0054 
		      	  }else{																											//APN0054
			          if (de.trim().endsWith("_24")){																				//APN0054
			        	  pass = panel24r.CheckTray();																				//APN0054
			          }																												//APN0054
			          if (de.trim().endsWith("_48")){																				//APN0054
			        	  pass = panel48r.CheckTray();																				//APN0054
			          }																												//APN0054
			          if (de.trim().endsWith("_96")){																				//APN0054
			        	  pass = panel96r.CheckTray();																				//APN0054
			          }																												//APN0054
			          if (de.trim().endsWith("_384"))																				//APN0054
			          {																												//APN0054
			        	  pass = panel384r.CheckTray();																				//APN0054
			          }	          																									//APN0054
			          if (!pass){JOptionPane.showMessageDialog(null, "Please select the destination well/s");}						//APN0054
		          }        	  																										//APN0054
	          }																														//APN0054
	          
	          //APN0055
	          if ((TabWindow.this.text2.getText().contains(",")) || (TabWindow.this.text2.getText().contains("/")) || (TabWindow.this.text2.getText().contains("\\")) || (TabWindow.this.text2.getText().contains("*")) || (TabWindow.this.text2.getText().contains("<")) || (TabWindow.this.text2.getText().contains(">")) || (TabWindow.this.text2.getText().contains("?")) || (TabWindow.this.text2.getText().contains("|")))
	          {
	            JOptionPane.showMessageDialog(null, "The 'Reagent Name' can't contain any of the following carachters: \n \\ / , * < > ? | \"", " ", 2);
	            pass = false;
	          }
	          
	          if ((TabWindow.this.text2.getText().trim().equals("")) || (TabWindow.this.value == 0) || ((chkspl == 0) && (!TabWindow.this.tubebutton.isSelected()))  || (s1.equals("Plate type") && (!TabWindow.this.tubebutton.isSelected())) || ((chkdpl == 0) && (!TabWindow.this.tubebutton2.isSelected())) || (de.equals("Plate type") && (!TabWindow.this.tubebutton2.isSelected())) || (TabWindow.this.valuem1 > 1000) || (TabWindow.this.valuem2 > 1000) || ((TabWindow.this.resbutton.isSelected()) && (TabWindow.this.rs1.equals(""))) || ((TabWindow.this.resbutton2.isSelected()) && (TabWindow.this.rs2.equals(""))))
	          {
	            if ((TabWindow.this.text2.getText().trim().equals("")) || (TabWindow.this.value == 0))
	            {
	              JOptionPane.showMessageDialog(null, "Invalid Reagent Data Entered", " ", 2);
	            }
	            if ((s1.equals("Plate type")) && (!TabWindow.this.tubebutton.isSelected()))
	            {
	              JOptionPane.showMessageDialog(null, "Please select the Source Plate Type", " ", 2);
	            }
	            if ((chkspl == 0) && (!TabWindow.this.tubebutton.isSelected()))
	            {
	              JOptionPane.showMessageDialog(null, "Please select the Source Deck Position", " ", 2);
	            }
	            if ((chkdpl == 0) && (!TabWindow.this.tubebutton2.isSelected()))
	            {
	              JOptionPane.showMessageDialog(null, "Please select the Destination Deck Position", " ", 2);
	            }
	            if ((de.equals("Plate type")) && (!TabWindow.this.tubebutton2.isSelected()))
	            {
	              JOptionPane.showMessageDialog(null, "Please select the Destination Plate Type", " ", 2);
	            }
	            if ((TabWindow.this.valuem1 > 1000) || (TabWindow.this.valuem2 > 1000))
	            {
	              JOptionPane.showMessageDialog(null, "Invalid Mix volume", " ", 2);
	            }
	            if ((TabWindow.this.resbutton.isSelected()) && (TabWindow.this.rs1.equals("")))
	            {
	              JOptionPane.showMessageDialog(null, "Please click on the source reservoir", " ", 2);
	            }
	            if ((TabWindow.this.resbutton2.isSelected()) && (TabWindow.this.rs2.equals("")))
	            {
	              JOptionPane.showMessageDialog(null, "Please click on the destination reservoir", " ", 2);
	            }
	            pass = false;
	          }

	          if (pass)
	          {
	            TabWindow.this.updReg();
	          }
	          buttonSetEnable1();
	        }
        }
      });
    }
    
    return this.updButton1;
  }

  private JButton getUpdButton2()
  {
    if (this.updButton2 == null) {
      this.updButton2 = new JButton();
      this.updButton2.setEnabled(false);
      this.updButton2.setSize(20, 20);
      this.updButton2.setText("     Update Step       ");
      this.updButton2.setHorizontalAlignment(0);
      this.updButton2.setIcon(this.icons[16]);
      this.updButton2.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          for (int d = 0; d < 15; d++) TabWindow.this.dTextArea[d].setToolTipText("Click to edit the protocol");
          String min = TabWindow.this.text4.getText().trim();
          String min2 = TabWindow.this.text4pm.getText().trim();
          if (min.length() < 1)
            TabWindow.this.value = 0;
          else {
            try {
              TabWindow.this.value = Integer.parseInt(min);
            } catch (NumberFormatException exp) {
              TabWindow.this.value = 0;
              TabWindow.this.text4.setText("");
            }
          }
          if ((min2.length() > 1) && (!min2.equals("0"))) {
            try {
              TabWindow.this.value = Integer.parseInt(min2);
            } catch (NumberFormatException exp) {
              TabWindow.this.value = 0;
              TabWindow.this.text4pm.setText("0");
            }
          }

          if (TabWindow.this.value == 0)
          {
            JOptionPane.showMessageDialog(null, "Invalid Incubation Data Entered", " ", 2);
          }
          else
          {
            TabWindow.this.updInc();
          }
        }
        
      });
    }
    
    return this.updButton2;
  }

  private JButton getUpdButton3()
  {
    if (this.updButton3 == null) {
      this.updButton3 = new JButton();
      this.updButton3.setEnabled(false);
      this.updButton3.setSize(20, 20);
      this.updButton3.setText("     Update Step       ");
      this.updButton3.setHorizontalAlignment(0);
      this.updButton3.setIcon(this.icons[16]);
      this.updButton3.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          TabWindow.this.updWash();
        } } );
    }
    return this.updButton3;
  }

  private JButton getUpdButton4()
  {
    if (this.updButton4 == null) {
      this.updButton4 = new JButton();
      this.updButton4.setEnabled(false);
      this.updButton4.setSize(20, 20);
      this.updButton4.setText("     Update Step       ");
      this.updButton4.setHorizontalAlignment(0);
      this.updButton4.setIcon(this.icons[16]);
      this.updButton4.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          TabWindow.this.updRead();
        } } );
    }
    return this.updButton4;
  }

  private JButton getUpdButton8()
  {
    if (this.updButton8 == null) {
      this.updButton8 = new JButton();
      this.updButton8.setEnabled(false);
      this.updButton8.setSize(20, 20);
      this.updButton8.setText("     Update Step       ");
      this.updButton8.setHorizontalAlignment(0);
      this.updButton8.setIcon(this.icons[16]);
      this.updButton8.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          TabWindow.this.updMove();
        } } );
    }
    return this.updButton8;
  }

  private JButton getUpdButton9()
  {
    if (this.updButton9 == null) {
      this.updButton9 = new JButton();
      this.updButton9.setEnabled(false);
      this.updButton9.setSize(20, 20);
      this.updButton9.setText("     Update Step       ");
      this.updButton9.setHorizontalAlignment(0);
      this.updButton9.setIcon(this.icons[16]);
      this.updButton9.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
        	boolean error = false;
            int psec = 0;
            int pmin = 0;
            String psec2 = "";
            String pmin2 = "";
            pmin2 = pausedataPanel2.getMinutes();
            psec2 = pausedataPanel2.getSeconds();
            
            try {
              pmin = Integer.parseInt(pmin2);
              pminutes = String.valueOf(pmin);
            } catch (NumberFormatException exp) {
              error = true;
              pminutes = "";
            }
            try {
              psec = Integer.parseInt(psec2);
              pseconds = String.valueOf(psec);
            } catch (NumberFormatException exp) {
              error = true;
              pseconds = "";
            }
            String message = pausedataPanel2.getMessage().trim();
            if (message.contains("*"))
            {
            	error = true;
            }
            if (message.contains("/"))
            {
            	error = true;
            }
            if (message.contains("\\"))
            {
            	error = true;
            }
            if (message.contains("="))
            {
            	error = true;
            }
            if (message.contains("."))
            {
            	error = true;
            }
            if (message.contains(","))
            {
            	error = true;
            }
           
            if (pausedataPanel2.isTimed() && error)
            {
              JOptionPane.showMessageDialog(null, "Error in pause data", " ", 2);
            }
            else
            {
                TabWindow.this.updPause();
            }
        } } );
    }
    return this.updButton9;
  }

  private JButton getUpdButton10()
  {
    if (this.updButton10 == null) {
      this.updButton10 = new JButton();
      this.updButton10.setEnabled(false);
      this.updButton10.setSize(20, 20);
      this.updButton10.setText("     Update Step       ");
      this.updButton10.setIcon(this.icons[10]);
      this.updButton10.setHorizontalAlignment(0);
      this.updButton10.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          for (int d = 0; d < 15; d++) TabWindow.this.dTextArea[d].setToolTipText("Click to edit the protocol");
          TabWindow.this.updMovePlate();
        } } );
    }
    return this.updButton10;
  }

  private JButton getUpdButton11()
  {
    if (this.updButton11 == null) {
      this.updButton11 = new JButton();
      this.updButton11.setEnabled(false);
      this.updButton11.setSize(20, 20);
      this.updButton11.setText("     Update Step       ");
      this.updButton11.setIcon(this.icons[10]);
      this.updButton11.setHorizontalAlignment(0);
      this.updButton11.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          for (int d = 0; d < 15; d++) TabWindow.this.dTextArea[d].setToolTipText("Click to edit the protocol");
          TabWindow.this.updCherry();
        } } );
    }
    return this.updButton11;
  }
  
  private JButton getUpdButton6()								//Update Button for Serial Dilution
  {
    if (this.updButton6 == null) {
      this.updButton6 = new JButton();
      this.updButton6.setEnabled(false);
      this.updButton6.setSize(20, 20);
      this.updButton6.setText("     Update Step       ");
      this.updButton6.setHorizontalAlignment(0);
      this.updButton6.setIcon(this.icons[16]);
      this.updButton6.addActionListener(new ActionListener()
      {
        public void actionPerformed(ActionEvent e) {
        	int chksp2 = 1; int chkdp2 = 1; int chksp4 = 1;
        	
        	double schemeTransfers = 0.0;  	//SD1.4
        	
        	if (button96s.isSelected()){
	        	if (tubebutton3.isSelected()){
	        		schemeTransfers = spanel96r.dwnCount()/ (  (Integer.parseInt(TabWindow.this.stubepanel.getLastTube()) - Integer.parseInt(TabWindow.this.stubepanel.getFirstTube())  ) +1);  
	        	}else{
	        		schemeTransfers = spanel96r.dwnCount()/spanel96l.dwnCount();
	        	}        		
        	}

        	
        	if (button384s.isSelected()){		//APF0044
            	if (tubebutton3.isSelected()){
             		schemeTransfers = spanel384r.dwnCount()/ (  (Integer.parseInt(TabWindow.this.stubepanel.getLastTube()) - Integer.parseInt(TabWindow.this.stubepanel.getFirstTube())  ) +1);  
            	}else{
             		schemeTransfers = spanel384r.dwnCount()/spanel96l.dwnCount();
            	}
        	}
        	
            try
            {
            	if (!tubebutton3.isSelected()){
            		chksp2 = Integer.parseInt(TabWindow.this.spl2);
            	}
            } catch (NumberFormatException exp) {
              chksp2 = 0;
            }
            try {
              chkdp2 = Integer.parseInt(TabWindow.this.dpl1);
            } catch (NumberFormatException exp) {
              chkdp2 = 0;
            }
            try {
                chksp4 = Integer.parseInt(TabWindow.this.spl4);
              } catch (NumberFormatException exp) {
                chksp4 = 0;
              }
            if (chksp2 == 0) 
            {
              JOptionPane.showMessageDialog(null, "Please select the Source Deck Position", " ", 2);
            }
            else if (chksp2 == 0) 
            {
              JOptionPane.showMessageDialog(null, "Please select the Destination Deck Position", " ", 2);
            }
            else if (chksp4 == 0) 
            {
              JOptionPane.showMessageDialog(null, "Please select the Diluent Deck Position", " ", 2);
            }else if(arrayList_sdVolumesTransfer.size() != schemeTransfers){ 
            	JOptionPane.showMessageDialog(null, "Please update Dilution Scheme", " ", 2);  	
            }else{
            	TabWindow.this.updSerial();
            }
          
        } } );
    }
    return this.updButton6;
  }

  private JButton getUpdButton5()
  {
    if (this.updButton5 == null) {
      this.updButton5 = new JButton();
      this.updButton5.setEnabled(false);
      this.updButton5.setSize(20, 20);
      this.updButton5.setText("     Update Step       ");
      this.updButton5.setHorizontalAlignment(0);
      this.updButton5.setIcon(this.icons[16]);
      this.updButton5.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          TabWindow.this.col = ((String)TabWindow.this.collection.getSelectedItem());
          TabWindow.this.fil = ((String)TabWindow.this.filter.getSelectedItem());
          TabWindow.this.updSPE();
        } } );
    }
    return this.updButton5;
  }

  private JButton getUpdButton7()
  {
    if (this.updButton7 == null) {
      this.updButton7 = new JButton();
      this.updButton7.setEnabled(false);
      this.updButton7.setSize(20, 20);
      this.updButton7.setText("     Update Step       ");
      this.updButton7.setHorizontalAlignment(0);
      this.updButton7.setIcon(this.icons[16]);
      this.updButton7.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          TabWindow.this.updBrew();
        } } );
    }
    return this.updButton7;
  }

  private JButton getAddButton4()
  {
    if (this.addButton4 == null) {
      this.addButton4 = new JButton();
      this.addButton4.setSize(20, 20);
      this.addButton4.setText("     Add Step       ");
      this.addButton4.setHorizontalAlignment(0);
      this.addButton4.setIcon(this.icons[9]);
      this.addButton4.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          if ((TabWindow.this.text6.getText().trim().equals("")) && (!TabWindow.this.r1.trim().equals("MSD")))
          {
            JOptionPane.showMessageDialog(null, "Reader File is blank, Cannot add READ step", " ", 2);
          }
          else
          {
            TabWindow.this.addRead();
          }
        }
      });
    }
    return this.addButton4;
  }

  private JButton getAddButton6()											//Add button for Serial Dilution		
  {
    if (this.addButton6 == null) {
      this.addButton6 = new JButton();
      this.addButton6.setSize(20, 20);
      this.addButton6.setText("      Add Step         ");
      this.addButton6.setHorizontalAlignment(0);
      this.addButton6.setIcon(this.icons[9]);
      this.addButton6.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
            int chksp2 = 1; int chkdp2 = 1; int chksp4 = 1;
        	double schemeTransfers = 0.0;  	//SD1.4
        	
        	if (button96s.isSelected()){	
	         	if (tubebutton3.isSelected()){
	        		schemeTransfers = spanel96r.dwnCount()/ (  (Integer.parseInt(TabWindow.this.stubepanel.getLastTube()) - Integer.parseInt(TabWindow.this.stubepanel.getFirstTube())  ) +1);  
	        	}else{
	        		schemeTransfers = spanel96r.dwnCount()/spanel96l.dwnCount();
	        	}       		
        	}

        	if (button384s.isSelected()){		//APF0044
            	if (tubebutton3.isSelected()){
            		schemeTransfers = spanel384r.dwnCount()/ (  (Integer.parseInt(TabWindow.this.stubepanel.getLastTube()) - Integer.parseInt(TabWindow.this.stubepanel.getFirstTube())  ) +1);  
            	}else{
            		schemeTransfers = spanel384r.dwnCount()/spanel96l.dwnCount();
            	}
        	}
            try
            {
            	if (!tubebutton3.isSelected()){
            		chksp2 = Integer.parseInt(TabWindow.this.spl2);
            	}
            } catch (NumberFormatException exp) {
              chksp2 = 0;
            }
            try {
              chkdp2 = Integer.parseInt(TabWindow.this.dpl1);
            } catch (NumberFormatException exp) {
              chkdp2 = 0;
            }
            try {
                chksp4 = Integer.parseInt(TabWindow.this.spl4);
              } catch (NumberFormatException exp) {
                chksp4 = 0;
              }
            
            if (chksp2 == 0) 
            {
              JOptionPane.showMessageDialog(null, "Please select the Source Deck Position", " ", 2);
            }
            else if (chksp2 == 0) 
            {
              JOptionPane.showMessageDialog(null, "Please select the Destination Deck Position", " ", 2);
            }
            else if (chksp4 == 0) 
            {
              JOptionPane.showMessageDialog(null, "Please select the Diluent Deck Position", " ", 2);
            }
            else if(arrayList_sdVolumesTransfer.size() != schemeTransfers){ 
            	JOptionPane.showMessageDialog(null, "Please update Dilution Scheme", " ", 2);  	
            }else{
              TabWindow.this.addSerial();
            }
        }
      });
    }
    return this.addButton6;
  }

  private JButton getAddButton7()
  {
    if (this.addButton7 == null) {
      this.addButton7 = new JButton();
      this.addButton7.setSize(20, 20);
      this.addButton7.setText("      Add Step         ");
      this.addButton7.setHorizontalAlignment(0);
      this.addButton7.setIcon(this.icons[9]);
      this.addButton7.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          if (TabWindow.this.brtext.getText().trim().equals(""))
          {
            JOptionPane.showMessageDialog(null, "Brew Reagent Name is blank, Cannot add Brew step", " ", 2);
          }
          else
          {
            TabWindow.this.addBrew();
          }
        }
      });
    }
    return this.addButton7;
  }

  private JButton getAddButton8()
  {
    if (this.addButton8 == null) {
      this.addButton8 = new JButton();
      this.addButton8.setSize(20, 20);
      this.addButton8.setText("      Add Step         ");
      this.addButton8.setHorizontalAlignment(0);
      this.addButton8.setIcon(this.icons[9]);
      this.addButton8.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          TabWindow.this.addMove();
        }
      });
    }
    return this.addButton8;
  }

  private JButton getAddButton9()
  {
    if (this.addButton9 == null) {
      this.addButton9 = new JButton();
      this.addButton9.setSize(20, 20);
      this.addButton9.setText("      Add Step         ");
      this.addButton9.setHorizontalAlignment(0);
      this.addButton9.setIcon(this.icons[9]);
      this.addButton9.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {

        	boolean error = false;
            int psec = 0;
            int pmin = 0;
            String psec2 = "";
            String pmin2 = "";
            pmin2 = pausedataPanel2.getMinutes();
            psec2 = pausedataPanel2.getSeconds();
            
            try {
              pmin = Integer.parseInt(pmin2);
              pminutes = String.valueOf(pmin);
            } catch (NumberFormatException exp) {
              error = true;
              pminutes = "";
            }
            try {
              psec = Integer.parseInt(psec2);
              pseconds = String.valueOf(psec);
            } catch (NumberFormatException exp) {
              error = true;
              pseconds = "";
            }
            String message = pausedataPanel2.getMessage().trim();
            if (message.contains("*"))
            {
            	error = true;
            }
            if (message.contains("/"))
            {
            	error = true;
            }
            if (message.contains("\\"))
            {
            	error = true;
            }
            if (message.contains("="))
            {
            	error = true;
            }
            if (message.contains("."))
            {
            	error = true;
            }
            if (message.contains(","))
            {
            	error = true;
            }
           
            if (pausedataPanel2.isTimed() && error)
            {
              JOptionPane.showMessageDialog(null, "Error in pause data", " ", 2);
            }
            else
            {
            	TabWindow.this.addPause();
            }
        }
      });
    }
    return this.addButton9;
  }

  private JButton getAddButton10()
  {
    if (this.addButton10 == null) {
      this.addButton10 = new JButton();
      this.addButton10.setSize(20, 20);
      this.addButton10.setText("      Add Step         ");
      this.addButton10.setIcon(this.icons[9]);
      this.addButton10.setHorizontalAlignment(0);
      this.addButton10.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          int rvalue1 = 0;
          int rvalue2 = 0;
          try {
            rvalue1 = Integer.parseInt(TabWindow.this.spl3);
          } catch (NumberFormatException exp) {
            rvalue1 = 0;
          }
          try {
            rvalue2 = Integer.parseInt(TabWindow.this.dpl3);
          } catch (NumberFormatException exp) {
            rvalue2 = 0;
          }
          if ((rvalue1 == 0) || (rvalue2 == 0))
          {
            JOptionPane.showMessageDialog(null, "Invalid Deck Position Entered", " ", 2);
          }
          else
          {
            TabWindow.this.addMovePlate();
          }
        }
      });
    }
    return this.addButton10;
  }

  private JButton getAddButton11()
  {
    if (this.addButton11 == null) {
      this.addButton11 = new JButton();
      this.addButton11.setSize(20, 20);
      this.addButton11.setText("     Add Step       ");
      this.addButton11.setHorizontalAlignment(0);
      this.addButton11.setIcon(this.icons[9]);
      this.addButton11.addActionListener(new ActionListener()
      {
        public void actionPerformed(ActionEvent e) {
          if (TabWindow.this.text7.getText().trim().equals(""))
          {
            JOptionPane.showMessageDialog(null, "Worklist File is blank, Cannot add Cherry Pick step", " ", 2);
          }
          else
          {
            TabWindow.this.addCherry();
          }
        } } );
    }
    return this.addButton11;
  }
  private JButton getAddButton12()
  {
    if (this.addButton12 == null) {
      this.addButton12 = new JButton();
      this.addButton12.setSize(20, 20);
      this.addButton12.setText("     Add Step       ");
      this.addButton12.setHorizontalAlignment(0);
      this.addButton12.setIcon(this.icons[9]);
      this.addButton12.addActionListener(new ActionListener()
      {
        public void actionPerformed(ActionEvent e) {
          if (TabWindow.this.pkm_text_wat.getText().trim().equals("") || pkcheck1.isSelected() && (TabWindow.this.pkm_text_1.getText().trim().equals("") || TabWindow.this.pkm_text_ul.getText().trim().equals("")))
          {
            JOptionPane.showMessageDialog(null, "Incomplete PK data, Cannot add PK step", " ", 2);
          }
          else
          {
            TabWindow.this.addPK();
          }
        } } );
    }
    return this.addButton12;
  }

  private JButton getAddButton13()
  {
    if (this.addButton13 == null) {
      this.addButton13 = new JButton();
      this.addButton13.setSize(20, 20);
      this.addButton13.setText("     Add Step       ");
      this.addButton13.setHorizontalAlignment(0);
      this.addButton13.setIcon(this.icons[9]);
      this.addButton13.addActionListener(new ActionListener()
      {
        public void actionPerformed(ActionEvent e) {
          if (TabWindow.this.pkm_text_wat.getText().trim().equals("") || pkcheck1.isSelected() && (TabWindow.this.pkm_text_1.getText().trim().equals("") || TabWindow.this.pkm_text_ul.getText().trim().equals("")))
          {
            JOptionPane.showMessageDialog(null, "Incomplete PK data, Cannot add PK step", " ", 2);
          }
          else
          {
            TabWindow.this.addPK();
          }
        } } );
    }
    return this.addButton13;
  }

  private JButton getSerialButton()
  {
    if (this.serialButton == null) {
      this.serialButton = new JButton();
      this.serialButton.setSize(30, 30);
      this.serialButton.setText(" DILUTION SCHEME ");
      this.serialButton.setFont(font10);
      this.serialButton.setHorizontalAlignment(0);
      this.serialButton.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
  	
        	double schemeTransfers = 0.0;  	//SD1.4
        	
        	if (button96s.isSelected()){
            	if (tubebutton3.isSelected()){
            		schemeTransfers = spanel96r.dwnCount()/ (  (Integer.parseInt(TabWindow.this.stubepanel.getLastTube()) - Integer.parseInt(TabWindow.this.stubepanel.getFirstTube())  ) +1);  
            	}else{
            		schemeTransfers = spanel96r.dwnCount()/spanel96l.dwnCount();
            	}
        	}

        	if (button384s.isSelected()){		//APF0044
            	if (tubebutton3.isSelected()){
            		schemeTransfers = spanel384r.dwnCount()/ (  (Integer.parseInt(TabWindow.this.stubepanel.getLastTube()) - Integer.parseInt(TabWindow.this.stubepanel.getFirstTube())  ) +1);  
            	}else{
            		schemeTransfers = spanel384r.dwnCount()/spanel96l.dwnCount();
            	}
        	}
        	
        	
        	if (!(schemeTransfers == Math.round(schemeTransfers))){					//SD1.4
        		JOptionPane.showMessageDialog(null, "VAS cannot interpret the requested Serial Dilution Scheme Pattern");
        	}else{
            	//Erase Any Existing Data in Table
//            	for (int i=0; i < SerialSchemePanel.dataModel.getRowCount(); i++){ //SerialSchemePanel.data.length
//            		SerialSchemePanel.dataModel.setValueAt("", i, 0);
//            		SerialSchemePanel.dataModel.setValueAt("", i, 1);
//            		SerialSchemePanel.dataModel.setValueAt("", i, 2);
//        		}
            	
            	while (SerialSchemePanel.dataModel.getRowCount()>0){
            		SerialSchemePanel.dataModel.removeRow(0);
            	}
            	
            	//Populate Transfer and Diluent Columns of Table based on arrayLists
            	//Lists will be empty if this is the first time table was accessed
            	//Lists are cleared and then populated with the "OK" button of DilWindow

            	String rowValue1 = "";
            	String rowValue2 = "";
            	
        		for (int row = 0; row < schemeTransfers; row++) {
        			if (row < arrayList_sdVolumesTransfer.size()){
        				rowValue1 = arrayList_sdVolumesTransfer.get(row);
        				rowValue2 = arrayList_sdVolumesDiluent.get(row);
        			}else{
        				rowValue1 = "";
        				rowValue2 = "";
        			}
        			
                	String[] rowValues = {
                			String.valueOf(row+1),
                  			rowValue1,
                  			rowValue2,
                  	};

        			SerialSchemePanel.dataModel.addRow(rowValues);					//SD1.4
        		}
        		
        		//Make Table Visible
            	TabWindow.this.dilwin.setVisible(true);
        	}
        }
      });
    }
    return this.serialButton;
  }

  private JPanel getWashPanel()
  {
    if (this.washPanel == null) {
      this.washPanel = new JPanel();
      this.washPanel.setBackground(this.panelColor);
      this.washPanel.setLayout(new BorderLayout());
      this.washPanel.add(getWashDataPanel(), "North");
    }
    return this.washPanel;
  }

  public JComboBox getsPlateType()
  {
    if (this.splatetype == null) {
      this.splatetype = new JComboBox(this.platetypes);
      this.splatetype.setSelectedIndex(0);
      this.spl1 = ((String)this.splatetype.getSelectedItem());

      this.splatetype.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          TabWindow.this.spl1 = ((String)TabWindow.this.splatetype.getSelectedItem());
          int spx = TabWindow.this.splatetype.getSelectedIndex();

          if (TabWindow.this.platebutton.isSelected())
          {
            //TabWindow.this.getSrc();
            TabWindow.this.source.revalidate();
            TabWindow.this.liqPanel1top.revalidate();
            TabWindow.this.liqPanel1top.repaint();
            TabWindow.this.liqPanel1.revalidate();
            TabWindow.this.liqPanel1.repaint();
          }
        }
      });
    }
    return this.splatetype;
  }

  public JComboBox getsPlateType2()
  {
    if (this.splatetype2 == null) {
      this.splatetype2 = new JComboBox(this.platetypes);
      this.splatetype2.setSelectedIndex(0);
      this.spl2 = ((String)this.splatetype2.getSelectedItem());
      this.s3inc.setVisible(false);
      this.splatetype2.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          TabWindow.this.spl2 = ((String)TabWindow.this.splatetype2.getSelectedItem());
          int spx = TabWindow.this.splatetype2.getSelectedIndex();

          TabWindow.this.sinc1button.setVisible(false);
          TabWindow.this.sinc2button.setVisible(false);
          TabWindow.this.sinc3button.setVisible(false);

          TabWindow.this.serPanel1top.revalidate();
          TabWindow.this.serPanel1top.repaint();
        }
      });
    }
    return this.splatetype2;
  }

  public JComboBox getsPlateType3()
  {
    if (this.splatetype3 == null) {
      this.splatetype3 = new JComboBox(this.platetypes);
      this.splatetype3.setSelectedIndex(0);
      this.spl3 = ((String)this.splatetype3.getSelectedItem());
      this.splatetype3.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          TabWindow.this.spl3 = ((String)TabWindow.this.splatetype3.getSelectedItem());
        } } );
    }
    return this.splatetype3;
  }

  public JComboBox getsPlateType4()
  {
    if (this.splatetype4 == null) {
      this.splatetype4 = new JComboBox(this.platetypes);
      this.splatetype4.setSelectedIndex(0);
      this.spl4 = ((String)this.splatetype4.getSelectedItem());
      this.splatetype4.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          TabWindow.this.spl4 = ((String)TabWindow.this.splatetype4.getSelectedItem());
          int spx = TabWindow.this.splatetype4.getSelectedIndex();
        }
      });
    }
    return this.splatetype4;
  }

  public JComboBox getdPlateType()
  {
    if (this.dplatetype == null) {
      this.dplatetype = new JComboBox(this.platetypes);
      this.dplatetype.setSelectedIndex(0);
      this.dpl1 = ((String)this.dplatetype.getSelectedItem());

      this.dplatetype.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          TabWindow.this.dpl1 = ((String)TabWindow.this.dplatetype.getSelectedItem());
        }
      });
    }
    return this.dplatetype;
  }

  public JComboBox getdPlateType2()
  {
    if (this.dplatetype2 == null) {
      this.dplatetype2 = new JComboBox(this.platetypes);
      this.dplatetype2.setSelectedIndex(0);
      this.dpl2 = ((String)this.dplatetype2.getSelectedItem());
      this.s4inc.setVisible(false);
      this.dplatetype2.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          TabWindow.this.dpl2 = ((String)TabWindow.this.dplatetype2.getSelectedItem());
          int spx = TabWindow.this.dplatetype2.getSelectedIndex();
          if (spx == 29)
          {
            TabWindow.this.sinc4button.setVisible(true);
            TabWindow.this.sinc5button.setVisible(true);
            TabWindow.this.sinc6button.setVisible(true);

            TabWindow.this.serPanel2top.revalidate();
            TabWindow.this.serPanel2top.repaint();
          }
          else
          {
            TabWindow.this.sinc4button.setVisible(false);
            TabWindow.this.sinc5button.setVisible(false);
            TabWindow.this.sinc6button.setVisible(false);

            TabWindow.this.serPanel2top.revalidate();
            TabWindow.this.serPanel2top.repaint();
          }
        }
      });
    }
    return this.dplatetype2;
  }

  public JComboBox getdPlateType3()
  {
    if (this.dplatetype3 == null) {
      this.dplatetype3 = new JComboBox(this.platetypes);
      this.dplatetype3.setSelectedIndex(0);
      this.dpl3 = ((String)this.dplatetype3.getSelectedItem());
      this.dplatetype3.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          TabWindow.this.dpl3 = ((String)TabWindow.this.dplatetype3.getSelectedItem());
        } } );
    }
    return this.dplatetype3;
  }

  private JComboBox getWash()
  {
    if (this.wash == null) {
      this.wash = new JComboBox(this.washes);
      this.wash.setSelectedIndex(0);
      this.w1 = ((String)this.wash.getSelectedItem());
      this.wash.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          JComboBox cb = (JComboBox)e.getSource();
          TabWindow.this.w1 = ((String)cb.getSelectedItem());
        } } );
    }
    return this.wash;
  }

  public static void main(String[] args)
  {
    try
    {
      UIManager.setLookAndFeel("com.sun.java.swing.plaf.nimbus.NimbusLookAndFeel");
    }
    catch (Exception localException)
    {
    }

    SwingUtilities.invokeLater(new Runnable() {
      public void run() {
        TabWindow thisClass = new TabWindow();
        thisClass.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
        thisClass.setVisible(true);
      }
    });
  
  }

  public TabWindow()
  {
	  initialize();
  }

  public void initialize()
  {

	  //Intialize LogWriter.  Creates LogFile
//	  LogWriter.Init();
//	  LogWriter.Write("WOWLOWOWOWO");
	  
	//Initialize VAS.  Checks for required files and 32vs64 bit system  

	String VASinit = VASengine.Init(false);		  
	if (VASinit != "initialized"){					  
		JOptionPane.showMessageDialog(null, VASinit); 
	  		ExitApp();									  
	}
		 APLOG.Init();//Initialize Internal Log														
			
    loadCFG();

    setSize(1250, 720);
    setResizable(true);
    setContentPane(getJContentPane());
    setTitle(APversion);
    this.setIconImage(Toolkit.getDefaultToolkit()
    		   .getImage(this.names[22]));
    this.setJMenuBar(menuBar);
    this.setLocationRelativeTo(null);		//APN0049
    Splash();
    
    jTabbedPane1.setEnabled(false);
    jTabbedPane1.setToolTipText("To begin a new protocol, please start by clicking the Red Exclamation Mark to select the Deck Layout");

    //create a listener for all exit events
    addWindowListener(new WindowAdapter()
    {
       public void windowClosing(WindowEvent e)
       {
    	   if (layoutFileSelected.equals("")){	//APN0057 changed from aprActive																														//APN0051
    		   System.exit(0);
    	   }else{
	    	   if (security){
	    		   loginWindow.setAlwaysOnTop(false);//APN0049   
	    	   }
	 	   
	    	   if (security_accessLevel.equals("1") && security_Validated){																					//APN0051
	    		   System.exit(0);
	    	   }else{
		     	   Object[] options = { "Save", "Don't Save", "Cancel" };																					//APN0048																																		
		    	   int reply = JOptionPane.showOptionDialog(null, "Do you want to save the changes you made to the current protocol?", "AutoPilot",			//APN0048	
		    	   JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,																					//APN0048
		    	   null, options, options[0]);																												//APN0048   	   
		    	   if (reply == 0){																															//APN0048
		    	     	if (security){																														//APN0048
		            		if (security_auditTrailString.contains("AT0") || security_auditTrailString.contains("AT1")){									//APN0048
		            			security_auditTrailString = security_auditTrailString.replace("AT1", "AT0");	//1 indicates that protocol is valid		//APN0048
		            		}																																//APN0048
		    	      		security_Validated = false;																										//APN0048
		    	      		security_validationTitle = "Not Validated";        																				//APN0048
		            	}																																	//APN0048
		            	if (aprActive.equals("")){																											//APN0048
		            		int state = getJFileChooser().showSaveDialog(saveAsButton);																		//APN0048
		                	if (state == 0){																												//APN0048
		    	            	File f = getJFileChooser().getSelectedFile();																				//APN0048
		    	            	TabWindow.this.saveFile(f);																									//APN0048
		                	}																																//APN0048
		            	}else{																																//APN0048
		            		File f = new File(aprActive);																									//APN0048
		            		TabWindow.this.saveFile(f);																										//APN0048
		            	}																																	//APN0048
		            	System.exit(0);																														//APN0048
		    	   } 																																		//APN0048
		    	   if (reply == 1){System.exit(0);}																											//APN0048
		    	   if (reply ==2){loginWindow.setAlwaysOnTop(true);}//APN0049   		   
	    	   }    		   
    	   }

																															//APN0048
       }																																			//APN0048
    });																																				//APN0048
    
    if(security){
    	buttonSetDisable4();
    	loginWindow = new LogInWindow();
    	loginWindow.setVisible(true);					//APN0049
    	loginWindow.setAlwaysOnTop(true);				//APN0049
    	loginWindow.setLocationRelativeTo(null);		//APN0049
    }else{
    	buttonSetDisable3(); 
    }

  }
  

  private JPanel getJContentPane()
  {
    if (this.jContentPane == null) {
      this.jContentPane = new JPanel()

		{
          Color color1 = steelDrkColor;
          Color color2 = color1.darker();
          @Override
          protected void paintComponent( Graphics g ) {
              if ( !isOpaque( ) )
	                {
	                    super.paintComponent( g );
	                    //return;
	                }
             Graphics2D g2d = (Graphics2D)g;
            
              int w = getWidth( );
              int h = getHeight( );
               
              // Paint a gradient from top to bottom
              GradientPaint gp = new GradientPaint(
                  0, h, color1,
                  0, 0, color2 );

              g2d.setPaint( gp );
              g2d.fillRect( 0, 0, w, h );
              ////
           
              jContentPane.setOpaque( false );
              Image castle = new ImageIcon("C:\\Program Files\\Telos\\AutoPilot\\Config\\Splash.jpg").getImage();
	            g2d.drawImage(castle, 0, 0, null);
              super.paintComponent( g );
              //aliqPanel.setOpaque( true );
          };
	};
      this.jContentPane.setLayout(new BorderLayout());

      this.jContentPane.add(getDataWindow1(), "North");
      this.jContentPane.add(getJTabbedPane1(), "Center");
      this.jContentPane.add(getTextWindow(), "South");
    }
    return this.jContentPane;
  }

  private JPanel getTextWindow()
  {
    this.menuStep.add(this.menuItemEdit);
//    //this.menuStep.add(this.menuItemInsert);				APF0030
    this.menuStep.add(this.menuItemCut);
    this.menuItemCut.setEnabled(false);
    this.menuStep.add(this.menuItemCopy);
    this.menuStep.add(this.menuItemPaste);
    this.menuItemPaste.setEnabled(false);

    this.menuItemEdit.addActionListener(
      new ActionListener()
    {
      public void actionPerformed(ActionEvent e) {
    	if (security_accessLevel.equals("1") && security_Validated == true){													//APN0051
    		JOptionPane.showMessageDialog(null, "The current user does not have edit privilages for Validated methods");		//APN0051
    	}else{																														
          TabWindow.this.update = true;						
//        TabWindow.this.dTextArea[TabWindow.this.stepboxnbr].setCaretPosition(25);
          TabWindow.this.dTextArea[TabWindow.this.stepboxnbr].select(0, 1);
          TabWindow.this.dTextArea[TabWindow.this.stepboxnbr].insertIcon(TabWindow.this.icons[12]);  //xxxx
          //for (int d = 0; d < 15; d++) TabWindow.this.dTextArea[d].setToolTipText("Click the update button to save your changes");
          TabWindow.this.editFile(TabWindow.this.stepnbr);
          buttonSetDisable2();  		
    	}
      }
    });
    this.menuItemInsert.addActionListener(
      new ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        int tabindex = TabWindow.this.jTabbedPane1.getSelectedIndex();
        String tabtext = TabWindow.this.jTabbedPane1.getTitleAt(tabindex);
        if (tabtext.trim().equals("Liquid Transfer"))
        {
          int chkspl = 1; int chkdpl = 1;
          String vol = TabWindow.this.text3.getText().trim();
          TabWindow.this.mix1 = TabWindow.this.lmixt1.getText().trim();
          if (TabWindow.this.mix1.length() < 1) {
            TabWindow.this.valuem1 = 0;
            TabWindow.this.mix1 = "0";
          } else {
            try {
              TabWindow.this.valuem1 = Integer.parseInt(TabWindow.this.mix1);
            } catch (NumberFormatException exp) {
              TabWindow.this.valuem1 = 0;
              TabWindow.this.lmixt1.setText("0");
            }
          }
          TabWindow.this.mix2 = TabWindow.this.lmixt2.getText().trim();
          if (TabWindow.this.mix2.length() < 1) {
            TabWindow.this.valuem2 = 0;
            TabWindow.this.mix2 = "0";
          }
          else {
            try {
              TabWindow.this.valuem2 = Integer.parseInt(TabWindow.this.mix2);
            } catch (NumberFormatException exp) {
              TabWindow.this.valuem2 = 0;
              TabWindow.this.lmixt2.setText("0");
            }
          }
          if (vol.length() < 1)
            TabWindow.this.value = 0;
          else
            try {
              TabWindow.this.value = Integer.parseInt(vol);
            } catch (NumberFormatException exp) {
              TabWindow.this.value = 0;
              TabWindow.this.text3.setText("");
            }
          try
          {
            chkspl = Integer.parseInt(TabWindow.this.spl1);
          } catch (NumberFormatException exp) {
            chkspl = 0;
          }
          try {
            chkdpl = Integer.parseInt(TabWindow.this.dpl1);
          } catch (NumberFormatException exp) {
            chkdpl = 0;
          }
          if ((TabWindow.this.text2.getText().trim().equals("")) || (TabWindow.this.value == 0) || ((chkspl == 0) && (!TabWindow.this.tubebutton.isSelected())) || ((chkdpl == 0) && (!TabWindow.this.tubebutton2.isSelected())) || (TabWindow.this.valuem1 > 1000) || (TabWindow.this.valuem2 > 1000) || ((TabWindow.this.resbutton.isSelected()) && (TabWindow.this.rs1.equals(""))) || ((TabWindow.this.resbutton2.isSelected()) && (TabWindow.this.rs2.equals(""))))
          {
            if ((TabWindow.this.text2.getText().trim().equals("")) || (TabWindow.this.value == 0))
            {
              JOptionPane.showMessageDialog(null, "Invalid Reagent Data Entered", " ", 2);
            }
            if ((chkspl == 0) && (!TabWindow.this.tubebutton.isSelected()))
            {
              JOptionPane.showMessageDialog(null, "Please select the Source Deck Position", " ", 2);
            }
            if ((chkdpl == 0) && (!TabWindow.this.tubebutton2.isSelected()))
            {
              JOptionPane.showMessageDialog(null, "Please select the Destination Deck Position", " ", 2);
            }
            if ((TabWindow.this.valuem1 > 1000) || (TabWindow.this.valuem2 > 1000))
            {
              JOptionPane.showMessageDialog(null, "Invalid Mix volume", " ", 2);
            }
            if ((TabWindow.this.resbutton.isSelected()) && (TabWindow.this.rs1.equals("")))
            {
              JOptionPane.showMessageDialog(null, "Please click on the source reservoir", " ", 2);
            }
            if ((TabWindow.this.resbutton2.isSelected()) && (TabWindow.this.rs2.equals("")))
            {
              JOptionPane.showMessageDialog(null, "Please click on the destination reservoir", " ", 2);
            }

          }
          else
          {
            TabWindow.this.insertStep(TabWindow.this.stepnbr);
          }
        }
        if (tabtext.trim().equals("Move Plate"))
        {
          int rvalue1 = 0;
          int rvalue2 = 0;
          try
          {
            rvalue1 = Integer.parseInt(TabWindow.this.spl3);
          } catch (NumberFormatException exp) {
            rvalue1 = 0;
          }
          try {
            rvalue2 = Integer.parseInt(TabWindow.this.dpl3);
          } catch (NumberFormatException exp) {
            rvalue2 = 0;
          }
          if ((rvalue1 == 0) || (rvalue2 == 0))
          {
            JOptionPane.showMessageDialog(null, "Invalid Deck Position Entered", " ", 2);
          }
          else
          {
            TabWindow.this.insertStep(TabWindow.this.stepnbr);
          }
        }
      }
    });
    this.menuItemCut.addActionListener(
      new ActionListener()
    {
      public void actionPerformed(ActionEvent e) {
        TabWindow.this.cutStep(TabWindow.this.stepnbr);
        TabWindow.this.menuItemPaste.setEnabled(true);
//        //TabWindow.this.menuItemCut.setEnabled(false);			//APF0029
//        //TabWindow.this.menuItemCopy.setEnabled(false);			//APF0029
      }
    });
    this.menuItemCopy.addActionListener(
      new ActionListener()
    {
      public void actionPerformed(ActionEvent e) {
        TabWindow.this.copyStep(TabWindow.this.stepnbr);
        TabWindow.this.menuItemPaste.setEnabled(true);
//        //TabWindow.this.menuItemCut.setEnabled(false);			//APF0029
//        //TabWindow.this.menuItemCopy.setEnabled(false);			//APF0029
      }
    });
    this.menuItemPaste.addActionListener(
      new ActionListener()
    {
      public void actionPerformed(ActionEvent e) {
        TabWindow.this.pasteStep(TabWindow.this.stepnbr);
        TabWindow.this.menuItemPaste.setEnabled(false);
      }
    });
    if (this.textWindow == null) {
      this.textWindow = new JPanel();

      this.textWindow.setLayout(new GridLayout(1, 17));
      TitledBorder regBorder = BorderFactory.createTitledBorder("Your Protocol");
      regBorder.setTitleJustification(2);
      regBorder.setTitlePosition(1);
      this.textWindow.setBorder(regBorder);
      this.textWindow.setBackground(this.panelColor);
      this.textWindow.setToolTipText("The protocol window displays each step that you add to your protocol");
      this.textWindow.add(getArrowButton1());
      arrowButton1.setEnabled(false);		//APF0028
      
      
      for (int d = 0; d < 15; d++) {
        this.dTextArea[d] = new JTextPane()
        {
          protected void paintComponent(Graphics g)
          {
            int width = getWidth();
            int height = getHeight();

            Paint paint = new GradientPaint(0.0F, 0.0F, TabWindow.this.steelColor, 
              0.0F, getHeight(), TabWindow.this.steelColor.brighter(), true);

            ((Graphics2D)g).setPaint(paint);

            g.fillRoundRect(0, 0, width - 1, height - 1, 35, 30);

            ((Graphics2D)g).setPaint(TabWindow.this.steelDrkColor);
            g.drawRoundRect(0, 0, width - 1, height - 1, 35, 30);
            super.paintComponent(g);
          }
        };
        this.dTextArea[d].setToolTipText("Click to edit the protocol"); 
        this.dTextArea[d].setOpaque(false);

        this.dTextArea[d].setEditable(false);
        this.dTextArea[d].setVisible(true);
      }

      this.dTextArea[0].setEditable(false);
      this.dTextArea[0].setVisible(true);
      this.dTextArea[0].addMouseListener(new MouseListener()
      {
        public void mouseClicked(MouseEvent e)
        {
          if (!TabWindow.this.update)
          {
            if (TabWindow.this.pagecount == 0)
            {
              TabWindow.this.stepnbr = 1;
            }
            else
            {
              TabWindow.this.stepnbr = ((15*TabWindow.this.pagecount)+1);				//APF0027
            }
            TabWindow.this.stepboxnbr = 0;
            TabWindow.this.menuStep.show(e.getComponent(), e.getX(), e.getY());
          }
        }

        public void mouseEntered(MouseEvent e)
        {
        }

        public void mouseExited(MouseEvent e)
        {
        }

        public void mousePressed(MouseEvent e)
        {
        }

        public void mouseReleased(MouseEvent e)
        {
        }
      });
      this.textWindow.add(this.dTextArea[0]);
      this.dTextArea[1].setEditable(false);
      this.dTextArea[1].setVisible(true);
      this.dTextArea[1].addMouseListener(new MouseListener()
      {
        public void mouseClicked(MouseEvent e)
        {
          if (!TabWindow.this.update)
          {
            if (TabWindow.this.pagecount == 0)
            {
              TabWindow.this.stepnbr = 2;
            }
            else
            {
              TabWindow.this.stepnbr = ((15*TabWindow.this.pagecount)+2);				//APF0027
            }
            TabWindow.this.stepboxnbr = 1;
            TabWindow.this.menuStep.show(e.getComponent(), e.getX(), e.getY());
          }
        }

        public void mouseEntered(MouseEvent e)
        {
        }

        public void mouseExited(MouseEvent e)
        {
        }

        public void mousePressed(MouseEvent e)
        {
        }

        public void mouseReleased(MouseEvent e)
        {
        }
      });
      this.textWindow.add(this.dTextArea[1]);
      this.dTextArea[2].setEditable(false);
      this.dTextArea[2].setVisible(true);
      this.dTextArea[2].addMouseListener(new MouseListener()
      {
        public void mouseClicked(MouseEvent e)
        {
          if (!TabWindow.this.update)
          {
            if (TabWindow.this.pagecount == 0)
            {
              TabWindow.this.stepnbr = 3;
            }
            else
            {
            	TabWindow.this.stepnbr = ((15*TabWindow.this.pagecount)+3);				//APF0027
            }
            TabWindow.this.stepboxnbr = 2;
            TabWindow.this.menuStep.show(e.getComponent(), e.getX(), e.getY());
          }
        }

        public void mouseEntered(MouseEvent e)
        {
        }

        public void mouseExited(MouseEvent e)
        {
        }

        public void mousePressed(MouseEvent e)
        {
        }

        public void mouseReleased(MouseEvent e)
        {
        }
      });
      this.textWindow.add(this.dTextArea[2]);
      this.dTextArea[3].setEditable(false);
      this.dTextArea[3].setVisible(true);
      this.dTextArea[3].addMouseListener(new MouseListener()
      {
        public void mouseClicked(MouseEvent e)
        {
          if (!TabWindow.this.update)
          {
            if (TabWindow.this.pagecount == 0)
            {
              TabWindow.this.stepnbr = 4;
            }
            else
            {
            	TabWindow.this.stepnbr = ((15*TabWindow.this.pagecount)+4);				//APF0027
            }
            TabWindow.this.stepboxnbr = 3;
            TabWindow.this.menuStep.show(e.getComponent(), e.getX(), e.getY());
          }
        }

        public void mouseEntered(MouseEvent e)
        {
        }

        public void mouseExited(MouseEvent e)
        {
        }

        public void mousePressed(MouseEvent e)
        {
        }

        public void mouseReleased(MouseEvent e)
        {
        }
      });
      this.textWindow.add(this.dTextArea[3]);
      this.dTextArea[4].setEditable(false);
      this.dTextArea[4].setVisible(true);
      this.dTextArea[4].addMouseListener(new MouseListener()
      {
        public void mouseClicked(MouseEvent e)
        {
          if (!TabWindow.this.update)
          {
            if (TabWindow.this.pagecount == 0)
            {
              TabWindow.this.stepnbr = 5;
            }
            else
            {
            	TabWindow.this.stepnbr = ((15*TabWindow.this.pagecount)+5);				//APF0027
            }
            TabWindow.this.stepboxnbr = 4;
            TabWindow.this.menuStep.show(e.getComponent(), e.getX(), e.getY());
          }
        }

        public void mouseEntered(MouseEvent e)
        {
        }

        public void mouseExited(MouseEvent e)
        {
        }

        public void mousePressed(MouseEvent e)
        {
        }

        public void mouseReleased(MouseEvent e)
        {
        }
      });
      this.textWindow.add(this.dTextArea[4]);
      this.dTextArea[5].setEditable(false);
      this.dTextArea[5].setVisible(true);
      this.dTextArea[5].addMouseListener(new MouseListener()
      {
        public void mouseClicked(MouseEvent e)
        {
          if (!TabWindow.this.update)
          {
            if (TabWindow.this.pagecount == 0)
            {
              TabWindow.this.stepnbr = 6;
            }
            else
            {
            	TabWindow.this.stepnbr = ((15*TabWindow.this.pagecount)+6);				//APF0027
            }
            TabWindow.this.stepboxnbr = 5;
            TabWindow.this.menuStep.show(e.getComponent(), e.getX(), e.getY());
          }
        }

        public void mouseEntered(MouseEvent e)
        {
        }

        public void mouseExited(MouseEvent e)
        {
        }

        public void mousePressed(MouseEvent e)
        {
        }

        public void mouseReleased(MouseEvent e)
        {
        }
      });
      this.textWindow.add(this.dTextArea[5]);
      this.dTextArea[6].setEditable(false);
      this.dTextArea[6].setVisible(true);
      this.dTextArea[6].addMouseListener(new MouseListener()
      {
        public void mouseClicked(MouseEvent e)
        {
          if (!TabWindow.this.update)
          {
            if (TabWindow.this.pagecount == 0)
            {
              TabWindow.this.stepnbr = 7;
            }
            else
            {
            	TabWindow.this.stepnbr = ((15*TabWindow.this.pagecount)+7);				//APF0027
            }
            TabWindow.this.stepboxnbr = 6;
            TabWindow.this.menuStep.show(e.getComponent(), e.getX(), e.getY());
          }
        }

        public void mouseEntered(MouseEvent e)
        {
        }

        public void mouseExited(MouseEvent e)
        {
        }

        public void mousePressed(MouseEvent e)
        {
        }

        public void mouseReleased(MouseEvent e)
        {
        }
      });
      this.textWindow.add(this.dTextArea[6]);
      this.dTextArea[7].setEditable(false);
      this.dTextArea[7].setVisible(true);
      this.dTextArea[7].addMouseListener(new MouseListener()
      {
        public void mouseClicked(MouseEvent e)
        {
          if (!TabWindow.this.update)
          {
            if (TabWindow.this.pagecount == 0)
            {
              TabWindow.this.stepnbr = 8;
            }
            else
            {
            	TabWindow.this.stepnbr = ((15*TabWindow.this.pagecount)+8);				//APF0027
            }
            TabWindow.this.stepboxnbr = 7;
            TabWindow.this.menuStep.show(e.getComponent(), e.getX(), e.getY());
          }
        }

        public void mouseEntered(MouseEvent e)
        {
        }

        public void mouseExited(MouseEvent e)
        {
        }

        public void mousePressed(MouseEvent e)
        {
        }

        public void mouseReleased(MouseEvent e)
        {
        }
      });
      this.textWindow.add(this.dTextArea[7]);
      this.dTextArea[8].setEditable(false);
      this.dTextArea[8].setVisible(true);
      this.dTextArea[8].addMouseListener(new MouseListener()
      {
        public void mouseClicked(MouseEvent e)
        {
          if (!TabWindow.this.update)
          {
            if (TabWindow.this.pagecount == 0)
            {
              TabWindow.this.stepnbr = 9;
            }
            else
            {
            	TabWindow.this.stepnbr = ((15*TabWindow.this.pagecount)+9);				//APF0027
            }
            TabWindow.this.stepboxnbr = 8;
            TabWindow.this.menuStep.show(e.getComponent(), e.getX(), e.getY());
          }
        }

        public void mouseEntered(MouseEvent e)
        {
        }

        public void mouseExited(MouseEvent e)
        {
        }

        public void mousePressed(MouseEvent e)
        {
        }

        public void mouseReleased(MouseEvent e)
        {
        }
      });
      this.textWindow.add(this.dTextArea[8]);
      this.dTextArea[9].setEditable(false);
      this.dTextArea[9].setVisible(true);
      this.dTextArea[9].addMouseListener(new MouseListener()
      {
        public void mouseClicked(MouseEvent e)
        {
          if (!TabWindow.this.update)
          {
            if (TabWindow.this.pagecount == 0)
            {
              TabWindow.this.stepnbr = 10;
            }
            else
            {
            	TabWindow.this.stepnbr = ((15*TabWindow.this.pagecount)+10);				//APF0027
            }
            TabWindow.this.stepboxnbr = 9;
            TabWindow.this.menuStep.show(e.getComponent(), e.getX(), e.getY());
          }
        }

        public void mouseEntered(MouseEvent e)
        {
        }

        public void mouseExited(MouseEvent e)
        {
        }

        public void mousePressed(MouseEvent e)
        {
        }

        public void mouseReleased(MouseEvent e)
        {
        }
      });
      this.textWindow.add(this.dTextArea[9]);
      this.dTextArea[10].setEditable(false);
      this.dTextArea[10].setVisible(true);
      this.dTextArea[10].addMouseListener(new MouseListener()
      {
        public void mouseClicked(MouseEvent e)
        {
          if (!TabWindow.this.update)
          {
            if (TabWindow.this.pagecount == 0)
            {
              TabWindow.this.stepnbr = 11;
            }
            else
            {
            	TabWindow.this.stepnbr = ((15*TabWindow.this.pagecount)+11);				//APF0027
            }
            TabWindow.this.stepboxnbr = 10;
            TabWindow.this.menuStep.show(e.getComponent(), e.getX(), e.getY());
          }
        }

        public void mouseEntered(MouseEvent e)
        {
        }

        public void mouseExited(MouseEvent e)
        {
        }

        public void mousePressed(MouseEvent e)
        {
        }

        public void mouseReleased(MouseEvent e)
        {
        }
      });
      this.textWindow.add(this.dTextArea[10]);
      this.dTextArea[11].setEditable(false);
      this.dTextArea[11].setVisible(true);
      this.dTextArea[11].addMouseListener(new MouseListener()
      {
        public void mouseClicked(MouseEvent e)
        {
          if (!TabWindow.this.update)
          {
            if (TabWindow.this.pagecount == 0)
            {
              TabWindow.this.stepnbr = 12;
            }
            else
            {
            	TabWindow.this.stepnbr = ((15*TabWindow.this.pagecount)+12);				//APF0027
            }
            TabWindow.this.stepboxnbr = 11;
            TabWindow.this.menuStep.show(e.getComponent(), e.getX(), e.getY());
          }
        }

        public void mouseEntered(MouseEvent e)
        {
        }

        public void mouseExited(MouseEvent e)
        {
        }

        public void mousePressed(MouseEvent e)
        {
        }

        public void mouseReleased(MouseEvent e)
        {
        }
      });
      this.textWindow.add(this.dTextArea[11]);
      this.dTextArea[12].setEditable(false);
      this.dTextArea[12].setVisible(true);
      this.dTextArea[12].addMouseListener(new MouseListener()
      {
        public void mouseClicked(MouseEvent e)
        {
          if (!TabWindow.this.update)
          {
            if (TabWindow.this.pagecount == 0)
            {
              TabWindow.this.stepnbr = 13;
            }
            else
            {
            	TabWindow.this.stepnbr = ((15*TabWindow.this.pagecount)+13);				//APF0027
            }
            TabWindow.this.stepboxnbr = 12;
            TabWindow.this.menuStep.show(e.getComponent(), e.getX(), e.getY());
          }
        }

        public void mouseEntered(MouseEvent e)
        {
        }

        public void mouseExited(MouseEvent e)
        {
        }

        public void mousePressed(MouseEvent e)
        {
        }

        public void mouseReleased(MouseEvent e)
        {
        }
      });
      this.textWindow.add(this.dTextArea[12]);
      this.dTextArea[13].setEditable(false);
      this.dTextArea[13].setVisible(true);
      this.dTextArea[13].addMouseListener(new MouseListener()
      {
        public void mouseClicked(MouseEvent e)
        {
          if (!TabWindow.this.update)
          {
            if (TabWindow.this.pagecount == 0)
            {
              TabWindow.this.stepnbr = 14;
            }
            else
            {
            	TabWindow.this.stepnbr = ((15*TabWindow.this.pagecount)+14);				//APF0027
            }
            TabWindow.this.stepboxnbr = 13;
            TabWindow.this.menuStep.show(e.getComponent(), e.getX(), e.getY());
          }
        }

        public void mouseEntered(MouseEvent e)
        {
        }

        public void mouseExited(MouseEvent e)
        {
        }

        public void mousePressed(MouseEvent e)
        {
        }

        public void mouseReleased(MouseEvent e)
        {
        }
      });
      this.textWindow.add(this.dTextArea[13]);
      this.dTextArea[14].setEditable(false);
      this.dTextArea[14].setVisible(true);
      this.dTextArea[14].addMouseListener(new MouseListener()
      {
        public void mouseClicked(MouseEvent e)
        {
          if (!TabWindow.this.update)
          {
            if (TabWindow.this.pagecount == 0)
            {
              TabWindow.this.stepnbr = 15;
            }
            else
            {
            	TabWindow.this.stepnbr = ((15*TabWindow.this.pagecount)+15);				//APF0027
            }
            TabWindow.this.stepboxnbr = 14;
            TabWindow.this.menuStep.show(e.getComponent(), e.getX(), e.getY());
          }
        }

        public void mouseEntered(MouseEvent e)
        {
        }

        public void mouseExited(MouseEvent e)
        {
        }

        public void mousePressed(MouseEvent e)
        {
        }

        public void mouseReleased(MouseEvent e)
        {
        }
      });
      this.textWindow.add(this.dTextArea[14]);
      this.textWindow.add(getArrowButton2());
      arrowButton2.setEnabled(false);
    }

    for (int d = 0; d < 15; d++) {
      this.dTextArea[d].setVisible(false);
    }
    return this.textWindow;
  }

  private void addReg()
  {
    this.counter = 0;
    nonPK = nonPK + 1;
    pagecount = (int) ((Math.ceil(nonPK/15.0)) - 1);		//APF0028
    String data = "";
    
    String c1 = "0";
    String c2 = "0";
    String sh1 = "0";
    String sh2 = "0";
    String tpwa = "0";
    String v1 = "0";
    String v2 = "0";
    String line3 = "";
    String line4 = "";
    String q1 = "0";
    String q2 = "0";
    String q3 = "0";
    String q4 = "0";
    String q5 = "0";
    String q6 = "0";
    String q7 = "0";
    String q8 = "0";
    String i1 = "0";
    String i2 = "0";
    String i3 = "0";
    String i4 = "0";
    String tube1 = "";
    String tube2 = "";
    String tube3 = "";
    String tube4 = "";
    String resnum = "";
    String asp1 = "";
    String mp1 = "";
    String asp2 = "";
    String mp2 = "";

    data = data + "   \n";
    data = data + "\n " + this.text2.getText().trim();
    data = data + "\n " + this.text3.getText().trim() + " ul" + "\n";
    if (this.spl1.equals("Deck Position")) this.spl1 = "T";
    data = data + " Deck " + this.spl1;
    data = data + " to " + this.dpl1 + "\n";

    line3 = line3 + this.text2.getText().trim();
    line3 = line3 + "," + this.lt1.trim() + ",-,-,-";

    asp1 = this.advwin.getMode();
    mp1 = this.advwin.getMPA();
    data = data + " Multi: " + mp1;

    asp2 = this.advwind.getMode();
    mp2 = this.advwind.getMPA();
    data = data + "-" + mp2;

    if (this.tubepanel.incIsSelected())
    {
      i1 = "1";
    }
    if (this.tubepanel2.incIsSelected())
    {
      i2 = "1";
    }

//    if (this.linc1button.isSelected())
//    {
//      i3 = "1";
//    }
//    if (this.linc2button.isSelected())
//   {
//      i3 = "2";
//    }
//    if (this.linc3button.isSelected())
//   {
//      i3 = "3";
//    }
//    if (this.linc4button.isSelected())
//    {
//      i4 = "1";
//    }
//    if (this.linc5button.isSelected())
//    {
//      i4 = "2";
//    }
//    if (this.linc6button.isSelected())
//    {
//      i4 = "3";
//    }
    if (this.advwin.getChill() > 0)
    {
      c1 = "1";
    }
    if (this.advwin.getTPWA() > 0)
    {
      tpwa = "1";
    }
    if (this.advwin.getShake() > 0)
    {
      sh1 = "1";
    }
    if (this.advwin.getVStart() > 0)
    {
      data = data + " Recycle Tips\n";
      v1 = "1";
    }

    if (this.advwind.getChill() > 0)
    {
      c2 = "1";
    }
    if (this.advwind.getShake() > 0)
    {
      sh2 = "1";
    }
    if (this.advwind.getVStart() > 0)
    {
      data = data + " Recycle Tips\n";
      v2 = "1";
    }

    String sourceseq = " ";
    String destseq = "0";
    tube1 = this.tubepanel.getFirstTube();
    tube2 = this.tubepanel.getLastTube();
    tube3 = this.tubepanel2.getFirstTube();
    tube4 = this.tubepanel2.getLastTube();
    resnum = this.res24Xpanel.getReservoir();


    //SOURCE
    if (this.s1.trim().endsWith("_1X")){
    	if (rHam){
    		sourceseq = "1,2,3,4,5,6,7,8";
    	}else {
    		sourceseq = "1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,52,53,54,55,56,57,58,59,60,61,62,63,64,65,66,67,68,69,70,71,72,73,74,75,76,77,78,79,80,81,82,83,84,85,86,87,88,89,90,91,92,93,94,95,96";
    	}   	
    }
    else if (this.s1.matches(".*_.{1,2}X")){
    	sourceseq = "1,2,3,4,5,6,7,8";
    }
    if (this.s1.trim().endsWith("Tube")){
        sourceseq = tube1 + "," + tube2 + "," + i1;
        this.spl1 = "Tube";
    }
    if (this.s1.trim().endsWith("_24")){
        this.panel24l.loadTray();
        sourceseq = this.panel24l.getTray().trim();
    }
    if (this.s1.trim().endsWith("_48")){
        this.panel48l.loadTray();
        sourceseq = this.panel48l.getTray().trim();
    }
    if (this.s1.trim().endsWith("_96")){
        this.panel96l.loadTray();
        sourceseq = this.panel96l.getTray().trim();
    }
    if ((this.s1.trim().endsWith("_384"))){
        q1 = this.panel384l.getQ1();
        q2 = this.panel384l.getQ2();
        q3 = this.panel384l.getQ3();
        q4 = this.panel384l.getQ4();
        this.panel384l.loadTray();
        sourceseq = this.panel384l.getTray().trim();
    }
    
    //DESTINATION
    if (this.de.trim().endsWith("_1X")){

    	if (rHam2){
    		destseq = "1,2,3,4,5,6,7,8";
    	}else {
    		destseq = "1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,52,53,54,55,56,57,58,59,60,61,62,63,64,65,66,67,68,69,70,71,72,73,74,75,76,77,78,79,80,81,82,83,84,85,86,87,88,89,90,91,92,93,94,95,96";
    	}
    }
    else if (this.de.matches(".*_.{1,2}X")){
    	destseq = "1,2,3,4,5,6,7,8";
    }
    if (this.de.trim().endsWith("Tube")){
        destseq = tube3 + "," + tube4 + "," + i2;
        this.dpl1 = "Tube";
    }
    if (this.de.trim().endsWith("_24")){
    	this.panel24r.loadTray();
    	destseq = this.panel24r.getTray().trim();
    }
    if (this.de.trim().endsWith("_48")){
    	this.panel48r.loadTray();
    	destseq = this.panel48r.getTray().trim();
    }
    if (this.de.trim().endsWith("_96")){
    	this.panel96r.loadTray();
    	destseq = this.panel96r.getTray().trim();
    }
    if (this.de.trim().endsWith("_384"))
    {
      q5 = this.panel384r.getQ1();
      q6 = this.panel384r.getQ2();
      q7 = this.panel384r.getQ3();
      q8 = this.panel384r.getQ4();
      this.panel384r.loadTray();
      destseq = this.panel384r.getTray().trim();
    }
    
    ///////////////////////////////////////////////////////////////////////////////////////////////////
    
 
    if ((this.s1.trim().endsWith("_1X")) && (this.de.trim().equals("Set_24X")))
    {
      sourceseq = "1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,52,53,54,55,56,57,58,59,60,61,62,63,64,65,66,67,68,69,70,71,72,73,74,75,76,77,78,79,80,81,82,83,84,85,86,87,88,89,90,91,92,93,94,95,96";

      destseq = resnum;

      this.spl2 = "Set";
    }

    if ((this.s1.trim().endsWith("_4X")) && (this.de.trim().equals("Set_24X")))
    {
      sourceseq = "1,2,3,4,5,6,7,8";
      destseq = resnum;

      this.spl2 = "Set";
    }
    if ((this.s1.trim().endsWith("_5X")) && (this.de.trim().equals("Set_24X")))
    {
      sourceseq = "1,2,3,4,5,6,7,8";
      destseq = resnum;

      this.spl2 = "Set";
    }
    if ((this.s1.trim().equals("Reservoir_24X")) && (this.de.trim().equals("Set_24X")))
    {
      sourceseq = "1,2,3,4,5,6,7,8";
      destseq = resnum;

      this.dpl1 = "Set";
    }

    if ((this.spl1.trim().equals("Stock")) && (!this.s1.trim().endsWith("_24X")))
    {
      this.hline5 = this.s1.trim();
    }
    if ((this.spl1.trim().equals("Intermediate")) && (!this.s1.trim().endsWith("_24X")))
    {
      this.hline6 = this.s1.trim();
    }
    if ((this.spl1.trim().equals("Assay")) && (!this.s1.trim().endsWith("_24X")))
    {
      this.hline7 = this.s1.trim();
    }
    if ((this.spl1.trim().equals("Set")) && (!this.s1.trim().endsWith("_24X")))
    {
      this.hline8 = this.s1.trim();
    }
    if ((this.dpl1.trim().equals("Stock")) && (!this.de.trim().endsWith("_24X")))
    {
      this.hline5 = this.de.trim();
    }
    if ((this.dpl1.trim().equals("Intermediate")) && (!this.de.trim().endsWith("_24X")))
    {
      this.hline6 = this.de.trim();
    }
    if ((this.dpl1.trim().equals("Assay")) && (!this.de.trim().endsWith("_24X")))
    {
      this.hline7 = this.de.trim();
    }
    if ((this.dpl1.trim().equals("Set")) && (!this.de.trim().endsWith("_24X")))
    {
      this.hline8 = this.de.trim();
    }
    if (this.s1.trim().endsWith("Tube"))
    {
      this.hline9 = this.s1.trim();
    }
    if (this.de.trim().endsWith("Tube"))
    {
      this.hline9 = this.de.trim();
    }

    line4 = line4 + this.spl1.trim() + "," + this.dpl1.trim();

    if (this.s1.trim().equals("Set_24X"))
    {
      this.s1 = "Reservoir_24X";
    }
    if (this.de.trim().equals("Set_24X"))
    {
      this.de = "Reservoir_24X";
    }

    if (sourceseq.endsWith(","))
      sourceseq = sourceseq.substring(0, sourceseq.length() - 1);
    if (destseq.endsWith(",")) {
      destseq = destseq.substring(0, destseq.length() - 1);
    }
    String type = "REG";
    this.dList.insertAtBack(data);
    this.tList.insertAtBack(type);
    data = "";
    type = "";
    this.fList.insertAtBack("step");
    this.fList.insertAtBack("add");
    this.fList.insertAtBack(line3);
    this.fList.insertAtBack(line4);
    this.fList.insertAtBack(this.s1.trim() + "," + this.mix1 + "," + asp1 + "," + mp1 + "," + sh1 + "," + c1 + "," + tpwa + "," + v1 + "," + this.rs1);
    this.fList.insertAtBack(this.de.trim() + "," + this.mix2 + "," + asp2 + "," + mp2 + "," + sh2 + "," + c2 + "," + i4 + "," + v2 + "," + this.rs2);
    this.fList.insertAtBack(sourceseq);
    this.fList.insertAtBack(destseq);
    this.fList.insertAtBack(this.text3.getText().trim());
    this.fList.insertAtBack(q1 + "," + q2 + "," + q3 + "," + q4 + "," + q5 + "," + q6 + "," + q7 + "," + q8);

    this.dTextArea[0].setText("");
    this.dTextArea[1].setText("");
    this.dTextArea[2].setText("");
    this.dTextArea[3].setText("");
    this.dTextArea[4].setText("");
    this.dTextArea[5].setText("");
    this.dTextArea[6].setText("");
    this.dTextArea[7].setText("");
    this.dTextArea[8].setText("");
    this.dTextArea[9].setText("");
    this.dTextArea[10].setText("");
    this.dTextArea[11].setText("");
    this.dTextArea[12].setText("");
    this.dTextArea[13].setText("");
    this.dTextArea[14].setText("");

    this.dTextArea[0].setVisible(false);
    this.dTextArea[1].setVisible(false);
    this.dTextArea[2].setVisible(false);
    this.dTextArea[3].setVisible(false);
    this.dTextArea[4].setVisible(false);
    this.dTextArea[5].setVisible(false);
    this.dTextArea[6].setVisible(false);
    this.dTextArea[7].setVisible(false);
    this.dTextArea[8].setVisible(false);
    this.dTextArea[9].setVisible(false);
    this.dTextArea[10].setVisible(false);
    this.dTextArea[11].setVisible(false);
    this.dTextArea[12].setVisible(false);
    this.dTextArea[13].setVisible(false);
    this.dTextArea[14].setVisible(false);
    loadtext();

    this.text2.setText("");
    this.text3.setText("");
    this.lmixt1.setText("0");
    this.lmixt2.setText("0");
    this.lchill.setSelected(false);
    this.lchill1.setSelected(false);
    this.lmix.setSelected(false);
    this.lmix2.setSelected(false);
    this.lshake.setSelected(false);
//    this.linc1button.setVisible(false);
//    this.linc2button.setVisible(false);
//    this.linc3button.setVisible(false);
//    this.linc4button.setVisible(false);
//    this.linc5button.setVisible(false);
//    this.linc6button.setVisible(false);
    this.saveButton.setEnabled(true);
    this.tubepanel.setFirstTube(" ");
    this.tubepanel.setLastTube(" ");
    this.tubepanel2.setFirstTube(" ");
    this.tubepanel2.setLastTube(" ");

    this.source.setSelectedIndex(0);
    this.dest.setSelectedIndex(0);
    this.splatetype.setSelectedIndex(0);
    this.dplatetype.setSelectedIndex(0);
    this.splatetype.revalidate();
    this.splatetype.repaint();
    this.dplatetype.revalidate();
    this.dplatetype.repaint();
    this.platebutton.setSelected(true);
    this.platebutton2.setSelected(true);
//    this.button96l.setSelected(true);
//    this.button96l2.setSelected(true);
    i1 = "0";
    i2 = "0";
    i3 = "0";
    i4 = "0";
    c1 = "0";
    sh1 = "0";
    v1 = "0";
    asp1 = "";
    mp1 = "";
    c2 = "0";
    sh2 = "0";
    v2 = "0";
    asp2 = "";
    mp2 = "";
    this.advwin.setChill(false);
    this.advwin.setShake(false);
    this.advwin.setVStart(false);
    this.advwin.setASPMode("SMART");
    this.advwin.setMPA(1);
    this.advwind.setChill(false);
    this.advwind.setShake(false);
    this.advwind.setVStart(false);
    this.advwind.setASPMode("SMART");
    this.advwind.setMPA(1);

    this.liqPanel1.removeAll();
    this.liqPanel2.removeAll();
    getSrc();
    for (int d = 0; d < 12; d++)
    {
      if (this.resTextArea[d] == null)
        continue;
      this.resTextArea[d].setBorder(this.blackline);
    }

    for (int d2 = 0; d2 < 12; d2++)
    {
      if (this.resTextArea2[d2] == null)
        continue;
      this.resTextArea2[d2].setBorder(this.blackline);
    }

    for (int t = 0; t < this.tabcount; t++)
    {
        this.jTabbedPane1.setEnabledAt(t, true);
    }
    pkEnableDisable();

    this.source.revalidate();
    this.source.repaint();
    getDest();
    this.dest.revalidate();
    this.dest.repaint();
    this.dplatetype.setVisible(true);
    this.splatetype.setVisible(true);
    this.s1inc.setVisible(false);
    this.s2inc.setVisible(false);
    this.liqPanel1.add(getLiqPanel1Top(), "North");
    this.liqPanel1.add(this.panelbl, "Center");
    this.liqPanel1.add(getLiqPanel1Bot(), "South");
    this.liqPanel1.revalidate();
    this.liqPanel1.repaint();
    this.liqPanel2.add(getLiqPanel2Top(), "North");
    this.liqPanel2.add(this.panelbr, "Center");
    this.liqPanel2.add(getLiqPanel2Bot(), "South");
    this.liqPanel2.revalidate();
    this.liqPanel2.repaint();

    Splash();
  }

  private int Splash()
  {
	  jTabbedPane1.setSelectedIndex(jTabbedPane1.indexOfTab("Splash"));
	  
	  ImageIcon imageIcon = new ImageIcon("");
	  
	  VAStools.setJLabelImage(PanelPKB2.lblPkBlank, "TelosLogo.png", -1, "", 199, 74);							//APN0047
	  
	  PanelBlank1.label1.setIcon(new ImageIcon("C:/Program Files/TELOS/AutoPilot/Graphics/introS.png"));		//APN0047
	  PanelBlank2.label1.setIcon(new ImageIcon("C:/Program Files/TELOS/AutoPilot/Graphics/introD.png"));		//APN0047
	  ReservoirPanel1.label1.setText("SELECT RESERVOIR ABOVE");													//APN0047
	  ReservoirPanel1.label1.setIcon(imageIcon);																//APN0047
	  ReservoirPanel2.label1.setText("SELECT RESERVOIR ABOVE");													//APN0047
	  ReservoirPanel2.label1.setIcon(imageIcon);																//APN0047
	  tubepanel.label1.setText("SELECT TUBE TYPE ABOVE");														//APN0047
	  tubepanel2.label1.setText("SELECT TUBE TYPE ABOVE");														//APN0047
	  
	  //Splash screen specific UI modifications
	  
//	  if (!security_accessLevel.equals("1")){  //If not operator
//		  
//	  }
	  
	  jTabbedPane1.setEnabled(true);
	  buttonSetEnable1();
	  
	  addButton.setEnabled(false);
	  checkButton.setEnabled(false);
	  rmvButton.setEnabled(false);

	return 1;
  }
  
  private void addSerial()
  {
    this.counter = 0;
    nonPK = nonPK + 1;
    pagecount = (int) ((Math.ceil(nonPK/15.0)) - 1);		//APF0028
    String data = "";

    String line3 = "";
    String line4 = "";
    String line6 = "";
    String dilution = "";
    String transfer = "";
    String tube1 = "";
    String tube2 = "";
    
    String i1 = "0";
    String i2 = "0";
    String i3 = "0";
    String i4 = "0";

    data = data + "   \n";
    data = data + "\n " + this.sttext.getText().trim() + "\n ";
    
	//sttext = Stock Name    
	//slt1 = Stock Liquid Type
	//spl4 = Diluent Deck Position
	//dlt1 = Diluent Liquid Type
    //so2 = Diluent Labware

    line3 = line3 + this.sttext.getText().trim();
    line3 = line3 + "," + this.slt1.trim() + "," + this.spl4.trim() + "," + this.dlt1.trim() + "," + this.so2.trim();

    String sourceseq = "0";
    String destseq = "0";
	
    transfer = VAStools.arrayListToString(TabWindow.arrayList_sdVolumesTransfer, ",");   //this.dilwin.getDilution();
    dilution = VAStools.arrayListToString(TabWindow.arrayList_sdVolumesDiluent, ",");	//this.dilwin.getTransfer();
	
    arrayList_sdVolumesTransfer.clear();
    arrayList_sdVolumesDiluent.clear();
    
    
    if (this.stubepanel.incIsSelected())
    {
      i1 = "1";
    }

    if (this.sinc1button.isSelected())
    {
      i3 = "1";
    }
    if (this.sinc2button.isSelected())
    {
      i3 = "2";
    }
    if (this.sinc3button.isSelected())
    {
      i3 = "3";
    }
    if (this.sinc4button.isSelected())
    {
      i4 = "1";
    }
    if (this.sinc5button.isSelected())
    {
      i4 = "2";
    }
    if (this.sinc6button.isSelected())
    {
      i4 = "3";
    }

    tube1 = this.stubepanel.getFirstTube();
    tube2 = this.stubepanel.getLastTube();

    line6 = line6 + this.de2.trim() + ",-,-,-," + i4 + ",-";

    if ((this.so3.trim().endsWith("Tube")) && (this.de2.trim().endsWith("_96")))
    {
      this.spanel96r.loadTray();
      sourceseq = tube1 + "," + tube2 + "," + i1;
      destseq = this.spanel96r.getTray().trim();
      this.spl2 = "Tube";
    }

    if ((this.so3.trim().endsWith("Tube")) && (this.de2.trim().endsWith("_384")))
    {
      this.spanel384r.loadTray();
      sourceseq = tube1 + "," + tube2 + "," + i1;
      destseq = this.spanel384r.getTray().trim();
      this.spl2 = "Tube";
    }

    if ((this.so3.trim().endsWith("X")) && (this.de2.trim().endsWith("_96")))
    {
      this.spanel96r.loadTray();
      sourceseq = "0";
      destseq = this.spanel96r.getTray().trim();
      this.spl2 = "Reservoir";
    }
    if ((this.so3.trim().endsWith("_96")) && (this.de2.trim().endsWith("_96")))
    {
      this.spanel96l.loadTray();
      this.spanel96r.loadTray();
      sourceseq = this.spanel96l.getTray().trim();
      destseq = this.spanel96r.getTray().trim();
    }

    if ((this.so3.trim().endsWith("_96")) && (this.de2.trim().endsWith("_384")))
    {
      this.spanel96l.loadTray();
      this.spanel384r.loadTray();
      sourceseq = this.spanel96l.getTray().trim();
      destseq = this.spanel384r.getTray().trim();
    }

    if ((this.spl2.trim().equals("Stock")) && (!this.so3.trim().endsWith("_24")))
    {
      this.hline5 = this.so3.trim();
    }
    if ((this.spl2.trim().equals("Intermediate")) && (!this.so3.trim().endsWith("_24")))
    {
      this.hline6 = this.so3.trim();
    }
    if ((this.spl2.trim().equals("Assay")) && (!this.so3.trim().endsWith("_24")))
    {
      this.hline7 = this.so3.trim();
    }
    if ((this.spl2.trim().equals("Set")) && (!this.so3.trim().endsWith("_24")))
    {
      this.hline8 = this.so3.trim();
    }
    if ((this.dpl2.trim().equals("Stock")) && (!this.de2.trim().endsWith("_24")))
    {
      this.hline5 = this.de2.trim();
    }
    if ((this.dpl2.trim().equals("Intermediate")) && (!this.de2.trim().endsWith("_24")))
    {
      this.hline6 = this.de2.trim();
    }
    if ((this.dpl2.trim().equals("Assay")) && (!this.de2.trim().endsWith("_24")))
    {
      this.hline7 = this.de2.trim();
    }
    if ((this.dpl2.trim().equals("Set")) && (!this.de2.trim().endsWith("_24")))
    {
      this.hline8 = this.de2.trim();
    }
    if (this.so3.trim().endsWith("Tube"))
    {
      this.hline9 = this.so3.trim();
    }
    if (this.de2.trim().endsWith("Tube"))
    {
      this.hline9 = this.de2.trim();
    }

    line4 = line4 + this.spl2.trim() + "," + this.dpl2.trim();

    if (sourceseq.endsWith(","))
      sourceseq = sourceseq.substring(0, sourceseq.length() - 1);
    if (destseq.endsWith(",")) {
      destseq = destseq.substring(0, destseq.length() - 1);
    }
    String type = "SER";
    this.dList.insertAtBack(data);
    this.tList.insertAtBack(type);
    data = "";
    type = "";
    this.fList.insertAtBack("step");
    this.fList.insertAtBack("serial");
    this.fList.insertAtBack(line3);
    this.fList.insertAtBack(line4);
    this.fList.insertAtBack(this.so3.trim() + ",-,-," + i3);
    this.fList.insertAtBack(line6);
    this.fList.insertAtBack(sourceseq);
    this.fList.insertAtBack(destseq);
    this.fList.insertAtBack(transfer);
    this.fList.insertAtBack(dilution);

    this.text2.setText("");
    this.text3.setText("");

    this.dTextArea[0].setText("");
    this.dTextArea[1].setText("");
    this.dTextArea[2].setText("");
    this.dTextArea[3].setText("");
    this.dTextArea[4].setText("");
    this.dTextArea[5].setText("");
    this.dTextArea[6].setText("");
    this.dTextArea[7].setText("");
    this.dTextArea[8].setText("");
    this.dTextArea[9].setText("");
    this.dTextArea[10].setText("");
    this.dTextArea[11].setText("");
    this.dTextArea[12].setText("");
    this.dTextArea[13].setText("");
    this.dTextArea[14].setText("");

    this.dTextArea[0].setVisible(false);
    this.dTextArea[1].setVisible(false);
    this.dTextArea[2].setVisible(false);
    this.dTextArea[3].setVisible(false);
    this.dTextArea[4].setVisible(false);
    this.dTextArea[5].setVisible(false);
    this.dTextArea[6].setVisible(false);
    this.dTextArea[7].setVisible(false);
    this.dTextArea[8].setVisible(false);
    this.dTextArea[9].setVisible(false);
    this.dTextArea[10].setVisible(false);
    this.dTextArea[11].setVisible(false);
    this.dTextArea[12].setVisible(false);
    this.dTextArea[13].setVisible(false);
    this.dTextArea[14].setVisible(false);
    loadtext();

    this.sttext.setText("");
    this.dltext.setText("");
    this.source2.setSelectedIndex(0);
    this.dest2.setSelectedIndex(0);
    this.splatetype2.setSelectedIndex(0);
    this.splatetype2.revalidate();
    this.splatetype2.repaint();
    this.splatetype4.setSelectedIndex(0);
    this.splatetype4.revalidate();
    this.splatetype4.repaint();
    this.dplatetype2.setSelectedIndex(0);
    this.dplatetype2.revalidate();
    this.dplatetype2.repaint();

    this.platebutton3.setSelected(true);
    this.s3inc.setSelected(false);
    this.s4inc.setSelected(false);
    i1 = "0";
    i2 = "0";
    this.serPanel1.removeAll();
    this.button96s.setSelected(true);
    getSrc2();
    this.source2.revalidate();
    this.source2.repaint();
    getSrc3();
    this.source3.revalidate();
    this.source3.repaint();
    getDest2();
    this.dest2.revalidate();
    this.dest2.repaint();
    this.splatetype2.setVisible(true);
    this.dplatetype2.setVisible(true);
    this.s3inc.setVisible(false);
    this.s4inc.setVisible(false);

    this.serPanel1.add(getSerPanel1Top(), "North");
    this.serPanel1.add(this.spanel96l, "Center");
    this.serPanel1.revalidate();
    this.serPanel1.repaint();
    
    Splash();
  }

  private void addBrew()
  {
    this.counter = 0;
    nonPK = nonPK + 1;
    pagecount = (int) ((Math.ceil(nonPK/15.0)) - 1);		//APF0028
    String m1 = "0";
    String m2 = "0";
    String c1 = "0";
    String i1 = "0";
    String i2 = "0";
    String tube1 = "";
    String tube2 = "";
    String tube3 = "";
    String tube4 = "";

    String data = "";
    String line3 = "";
    String line4 = "";
    String line5 = "";
    String line6 = "";

    data = data + "\n BREW\n ";
    data = data + "\n " + this.brntext.getText().trim() + "\n";

    line3 = line3 + this.brtext.getText().trim();
    line3 = line3 + "," + this.blt1.trim() + "," + this.brntext.getText().trim() + "," + this.so4.trim() + ",-";

    String sourceseq = " ";
    String destseq = "0";

    if (this.brtubepanel.incIsSelected())
    {
      i1 = "1";
    }
    if (this.brtubepanel2.incIsSelected())
    {
      i2 = "1";
    }

    if (this.brchill.isSelected())
    {
      data = data + "Chilled\n";
      c1 = "1";
    }

    if (this.brmix.isSelected())
    {
      data = data + "Mix\n";
      m1 = "1";
    }

    if (this.brmix2.isSelected())
    {
      m2 = "1";
    }

    tube1 = this.brtubepanel.getFirstTube();
    tube2 = this.brtubepanel.getLastTube();
    tube3 = this.brtubepanel2.getFirstTube();
    tube4 = this.brtubepanel2.getLastTube();

    line5 = line5 + this.so4.trim() + "," + m1 + "," + c1 + ",-";
    line6 = line6 + this.de3.trim() + "," + m2 + ",-,-,-,-";

    if (this.so4.trim().endsWith("Tube"))
    {
      sourceseq = tube1 + "," + tube2 + "," + i1;
      this.hline9 = this.so4.trim();
      this.bpl1 = "Tube";
    }
    if (this.so4.trim().endsWith("_1X"))
    {
      sourceseq = "1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,52,53,54,55,56,57,58,59,60,61,62,63,64,65,66,67,68,69,70,71,72,73,74,75,76,77,78,79,80,81,82,83,84,85,86,87,88,89,90,91,92,93,94,95,96";

      this.bpl1 = "Reservoir";
    }
    if (this.so4.trim().endsWith("_2X"))
    {
      sourceseq = "1,2,3,4,5,6,7,8";
      this.bpl1 = "Reservoir";
    }
    if (this.so4.trim().endsWith("_4X"))
    {
      sourceseq = "1,2,3,4,5,6,7,8";
      this.bpl1 = "Reservoir";
    }
    if (this.so4.trim().endsWith("_5X"))
    {
      sourceseq = "1,2,3,4,5,6,7,8";
      this.bpl1 = "Reservoir";
    }
    if (this.so4.trim().endsWith("_24X"))
    {
      sourceseq = "1,2,3,4,5,6,7,8";
      this.bpl1 = "Reservoir";
    }

    if (this.de3.trim().endsWith("_1X"))
    {
      destseq = "1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,52,53,54,55,56,57,58,59,60,61,62,63,64,65,66,67,68,69,70,71,72,73,74,75,76,77,78,79,80,81,82,83,84,85,86,87,88,89,90,91,92,93,94,95,96";

      this.bpl2 = "Reservoir";
    }
    if (this.de3.trim().endsWith("_2X"))
    {
      destseq = "1,2,3,4,5,6,7,8";
      this.bpl2 = "Reservoir";
    }
    if (this.de3.trim().endsWith("_4X"))
    {
      destseq = "1,2,3,4,5,6,7,8";
      this.bpl2 = "Reservoir";
    }
    if (this.de3.trim().endsWith("_5X"))
    {
      destseq = "1,2,3,4,5,6,7,8";
      this.bpl2 = "Reservoir";
    }
    if (this.de3.trim().endsWith("_24X"))
    {
      destseq = "1,2,3,4,5,6,7,8";
      this.bpl2 = "Reservoir";
    }
    if (this.de3.trim().endsWith("Tube"))
    {
      destseq = tube3 + "," + tube4 + "," + i2;
      this.hline9 = this.de3.trim();
      this.bpl2 = "Tube";
    }

    line4 = line4 + this.bpl1.trim() + "," + this.bpl2.trim();

    String type = "BRW";
    this.dList.insertAtBack(data);
    this.tList.insertAtBack(type);
    data = "";
    type = "";
    this.fList.insertAtBack("step");
    this.fList.insertAtBack("brew");
    this.fList.insertAtBack(line3);
    this.fList.insertAtBack(line4);
    this.fList.insertAtBack(line5);
    this.fList.insertAtBack(line6);
    this.fList.insertAtBack(sourceseq);
    this.fList.insertAtBack(destseq);
    this.fList.insertAtBack(this.brvol.getText().trim());
    this.fList.insertAtBack("-");

    this.brtext.setText("");
    this.brntext.setText("");

    this.dTextArea[0].setText("");
    this.dTextArea[1].setText("");
    this.dTextArea[2].setText("");
    this.dTextArea[3].setText("");
    this.dTextArea[4].setText("");
    this.dTextArea[5].setText("");
    this.dTextArea[6].setText("");
    this.dTextArea[7].setText("");
    this.dTextArea[8].setText("");
    this.dTextArea[9].setText("");
    this.dTextArea[10].setText("");
    this.dTextArea[11].setText("");
    this.dTextArea[12].setText("");
    this.dTextArea[13].setText("");
    this.dTextArea[14].setText("");

    this.dTextArea[0].setVisible(false);
    this.dTextArea[1].setVisible(false);
    this.dTextArea[2].setVisible(false);
    this.dTextArea[3].setVisible(false);
    this.dTextArea[4].setVisible(false);
    this.dTextArea[5].setVisible(false);
    this.dTextArea[6].setVisible(false);
    this.dTextArea[7].setVisible(false);
    this.dTextArea[8].setVisible(false);
    this.dTextArea[9].setVisible(false);
    this.dTextArea[10].setVisible(false);
    this.dTextArea[11].setVisible(false);
    this.dTextArea[12].setVisible(false);
    this.dTextArea[13].setVisible(false);
    this.dTextArea[14].setVisible(false);
    loadtext();
    this.brvol.setText("");
    this.bpl1 = "";
    this.bpl2 = "";
    this.brmix.setSelected(false);
    this.brchill.setSelected(false);
    this.brtubepanel.setFirstTube(" ");
    this.brtubepanel.setLastTube(" ");
    this.brtubepanel2.setFirstTube(" ");
    this.brtubepanel2.setLastTube(" ");

    this.source4.setSelectedIndex(0);
    this.dest3.setSelectedIndex(0);
    this.brresbutton1.setSelected(true);
    this.brresbutton2.setSelected(true);
    this.brewPanel1.removeAll();
    getSrc4();
    this.source4.revalidate();
    this.source4.repaint();
    this.brewPanel1.add(getBrewPanel1Top(), "North");
    this.brewPanel1.add(this.brrespanel1, "Center");
    this.brewPanel1.add(getBrewPanel1Bot(), "South");
    this.brewPanel1.revalidate();
    this.brewPanel1.repaint();
    this.brewPanel2.removeAll();
    getDest3();
    this.dest3.revalidate();
    this.dest3.repaint();
    this.brewPanel2.add(getBrewPanel2Top(), "North");
    this.brewPanel2.add(this.brrespanel2, "Center");
    this.brewPanel2.add(getBrewPanel2Bot(), "South");
    this.brewPanel2.revalidate();
    this.brewPanel2.repaint();

    while (!this.bList.isEmpty())
    {
      this.bList.removeFromBack();
    }
    while (!this.eList.isEmpty())
    {
      this.eList.removeFromBack();
    }
    while (!this.dbList.isEmpty())
    {
      this.dbList.removeFromBack();
    }
    while (!this.deList.isEmpty())
    {
      this.deList.removeFromBack();
    }
    while (!this.tbList.isEmpty())
    {
      this.tbList.removeFromBack();
    }
    while (!this.teList.isEmpty())
    {
      this.teList.removeFromBack();
    }
    for (int t = 0; t < this.tabcount; t++)
    {
        this.jTabbedPane1.setEnabledAt(t, true);
    }

    pkEnableDisable();

    //Return to Splash screen
    Splash();
  }

  private void addWash()
  {
    this.counter = 0;
    nonPK = nonPK + 1;
    pagecount = (int) ((Math.ceil(nonPK/15.0)) - 1);		//APF0028
    String output = " ";
    output = output + "RUN: " + this.text1.getText();
    String line4 = "";

    String data = "";

    line4 = this.wmethod.getText().trim();
    if (line4.trim().equals(""))
    {
      line4 = "-";
    }

    data = data + "   \n";
    data = data + "\n " + this.wmethod.getText().trim() + "\n";

    String type = "WSH";

    this.dList.insertAtBack(data);
    this.tList.insertAtBack(type);

    this.fList.insertAtBack("step");
    this.fList.insertAtBack("wash");

    this.fList.insertAtBack("-");
    this.fList.insertAtBack(line4);
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");

    this.dTextArea[0].setText("");
    this.dTextArea[1].setText("");
    this.dTextArea[2].setText("");
    this.dTextArea[3].setText("");
    this.dTextArea[4].setText("");
    this.dTextArea[5].setText("");
    this.dTextArea[6].setText("");
    this.dTextArea[7].setText("");
    this.dTextArea[8].setText("");
    this.dTextArea[9].setText("");
    this.dTextArea[10].setText("");
    this.dTextArea[11].setText("");
    this.dTextArea[12].setText("");
    this.dTextArea[13].setText("");
    this.dTextArea[14].setText("");

    this.dTextArea[0].setVisible(false);
    this.dTextArea[1].setVisible(false);
    this.dTextArea[2].setVisible(false);
    this.dTextArea[3].setVisible(false);
    this.dTextArea[4].setVisible(false);
    this.dTextArea[5].setVisible(false);
    this.dTextArea[6].setVisible(false);
    this.dTextArea[7].setVisible(false);
    this.dTextArea[8].setVisible(false);
    this.dTextArea[9].setVisible(false);
    this.dTextArea[10].setVisible(false);
    this.dTextArea[11].setVisible(false);
    this.dTextArea[12].setVisible(false);
    this.dTextArea[13].setVisible(false);
    this.dTextArea[14].setVisible(false);
    loadtext();
    this.wmethod.setText("");
    for (int t = 0; t < this.tabcount; t++)
    {
        this.jTabbedPane1.setEnabledAt(t, true);
    }
    pkEnableDisable();

    //Return to Splash screen
    Splash();
  }

  private void updSerial()
  {
    String data = "";
    String line3 = "";
    String line4 = "";
    String line6 = "";
    String dilution = "";
    String tube1 = "";
    String tube2 = "";
    String transfer = "";
    String i1 = "0";
    String i2 = "0";
    String i3 = "0";
    String i4 = "0";
    String m1 = "0";
    String m2 = "0";
    String c1 = "0";
    String c2 = "0";
    String sh = "0";
    String v1 = "0";

    ListNode currentBList = this.bList.getFirstNode();
    ListNode currentFList = this.fList.getFirstNode();
    ListNode currentEList = this.eList.getFirstNode();

    ListNode currentDBList = this.dbList.getFirstNode();
    ListNode currentDList = this.dList.getFirstNode();
    ListNode currentDEList = this.deList.getFirstNode();

    ListNode currentTBList = this.tbList.getFirstNode();
    ListNode currentTList = this.tList.getFirstNode();
    ListNode currentTEList = this.teList.getFirstNode();

    data = data + "   \n";
    data = data + "\n " + this.sttext.getText().trim() + "\n ";

    String sourceseq = "0";
    String destseq = "0";
	
    transfer = VAStools.arrayListToString(TabWindow.arrayList_sdVolumesTransfer, ",");  //this.dilwin.getDilution();
    dilution = VAStools.arrayListToString(TabWindow.arrayList_sdVolumesDiluent, ",");  //this.dilwin.getTransfer();
	
    arrayList_sdVolumesTransfer.clear();
    arrayList_sdVolumesDiluent.clear();
    
    if (this.stubepanel.incIsSelected())
    {
      i1 = "1";
    }

    if (this.sinc1button.isSelected())
    {
      i3 = "1";
    }
    if (this.sinc2button.isSelected())
    {
      i3 = "2";
    }
    if (this.sinc3button.isSelected())
    {
      i3 = "3";
    }
    if (this.sinc4button.isSelected())
    {
      i4 = "1";
    }
    if (this.sinc5button.isSelected())
    {
      i4 = "2";
    }
    if (this.sinc6button.isSelected())
    {
      i4 = "3";
    }

    updText(data);
    int element;
//    if (this.stepnbr < 16) element = this.stepnbr - 1; else element = this.stepnbr - 16;		//APF0020 
    if ((this.stepnbr % 15) == 0) {element = 14;}  else{element = ((this.stepnbr % 15)-1);}		//APF0027
    
    this.dTextArea[element].setCaretPosition(0);
    this.dTextArea[element].insertIcon(this.icons[27]);
    
    data = data + "\n" + this.dlt1.trim() + "\n";
//    data = data + this.dltext.getText().trim() + "\n";
    data = data + this.so3.trim() + "\n";

    line3 = line3 + this.sttext.getText().trim();
//    line3 = line3 + "," + this.slt1.trim() + "," + this.dltext.getText().trim() + "," + this.dlt1.trim() + "," + this.so2.trim();
      line3 = line3 + "," + this.slt1.trim() + "," + this.spl4.trim() + "," + this.dlt1.trim() + "," + this.so2.trim();

//      transfer = VAStools.arrayListToString(TabWindow.arrayList_sdVolumesTransfer, ",");  //this.dilwin.getDilution();
//      dilution = VAStools.arrayListToString(TabWindow.arrayList_sdVolumesDiluent, ",");  //this.dilwin.getTransfer();

    if (this.stubepanel.incIsSelected())
    {
      i1 = "1";
    }

    if (this.sinc1button.isSelected())
    {
      i3 = "1";
    }
    if (this.sinc2button.isSelected())
    {
      i3 = "2";
    }
    if (this.sinc3button.isSelected())
    {
      i3 = "3";
    }
    if (this.sinc4button.isSelected())
    {
      i4 = "1";
    }
    if (this.sinc5button.isSelected())
    {
      i4 = "2";
    }
    if (this.sinc6button.isSelected())
    {
      i4 = "3";
    }

    tube1 = this.stubepanel.getFirstTube();
    tube2 = this.stubepanel.getLastTube();

    line6 = line6 + this.de2.trim() + ",-,-,-," + i4 + ",-";

    if ((this.so3.trim().endsWith("Tube")) && (this.de2.trim().endsWith("_96")))
    {
      this.spanel96r.loadTray();
      sourceseq = tube1 + "," + tube2 + "," + i1;
      destseq = this.spanel96r.getTray().trim();
      this.spl2 = "Tube";
    }
    if ((this.so3.trim().endsWith("X")) && (this.de2.trim().endsWith("_96")))
    {
      this.spanel96r.loadTray();
      sourceseq = "0";
      destseq = this.spanel96r.getTray().trim();
      this.spl2 = "Reservoir";
    }
    if ((this.so3.trim().endsWith("_96")) && (this.de2.trim().endsWith("_96")))
    {
      this.spanel96l.loadTray();
      this.spanel96r.loadTray();
      sourceseq = this.spanel96l.getTray().trim();
      destseq = this.spanel96r.getTray().trim();
    }

    if ((this.spl2.trim().equals("Stock")) && (!this.so3.trim().endsWith("_24")))
    {
      this.hline5 = this.so3.trim();
    }
    if ((this.spl2.trim().equals("Intermediate")) && (!this.so3.trim().endsWith("_24")))
    {
      this.hline6 = this.so3.trim();
    }
    if ((this.spl2.trim().equals("Assay")) && (!this.so3.trim().endsWith("_24")))
    {
      this.hline7 = this.so3.trim();
    }
    if ((this.spl2.trim().equals("Set")) && (!this.so3.trim().endsWith("_24")))
    {
      this.hline8 = this.so3.trim();
    }
    if ((this.dpl2.trim().equals("Stock")) && (!this.de2.trim().endsWith("_24")))
    {
      this.hline5 = this.de2.trim();
    }
    if ((this.dpl2.trim().equals("Intermediate")) && (!this.de2.trim().endsWith("_24")))
    {
      this.hline6 = this.de2.trim();
    }
    if ((this.dpl2.trim().equals("Assay")) && (!this.de2.trim().endsWith("_24")))
    {
      this.hline7 = this.de2.trim();
    }
    if ((this.dpl2.trim().equals("Set")) && (!this.de2.trim().endsWith("_24")))
    {
      this.hline8 = this.de2.trim();
    }
    if (this.so3.trim().endsWith("Tube"))
    {
      this.hline9 = this.so3.trim();
    }
    if (this.de2.trim().endsWith("Tube"))
    {
      this.hline9 = this.de2.trim();
    }

    line4 = line4 + this.spl2.trim() + "," + this.dpl2.trim();

    if (sourceseq.endsWith(","))
      sourceseq = sourceseq.substring(0, sourceseq.length() - 1);
    if (destseq.endsWith(",")) {
      destseq = destseq.substring(0, destseq.length() - 1);
    }

    while (!this.fList.isEmpty())
    {
      this.fList.removeFromBack();
    }

    while (!this.dList.isEmpty())
    {
      this.dList.removeFromBack();
    }

    while (!this.tList.isEmpty())			//APF0022  added this line
    {										//APF0022  added this line
      this.tList.removeFromBack();			//APF0022  added this line
    }										//APF0022  added this line
    
    while (currentBList != null)
    {
      String list = currentBList.data.toString();
      this.fList.insertAtBack(currentBList.data.toString());
      currentBList = currentBList.next;
    }

    this.fList.insertAtBack("step");
    this.fList.insertAtBack("serial");
    this.fList.insertAtBack(line3);
    this.fList.insertAtBack(line4);
    this.fList.insertAtBack(this.so3.trim() + ",-,-," + i3);
    this.fList.insertAtBack(line6);
    this.fList.insertAtBack(sourceseq);
    this.fList.insertAtBack(destseq);
    this.fList.insertAtBack(transfer);
    this.fList.insertAtBack(dilution);

    while (currentEList != null)
    {
      this.fList.insertAtBack(currentEList.data.toString());
      currentEList = currentEList.next;
    }

    while (currentDBList != null)
    {
      this.dList.insertAtBack(currentDBList.data.toString());
      currentDBList = currentDBList.next;
    }

    this.dList.insertAtBack(data);

    while (currentDEList != null)
    {
      this.dList.insertAtBack(currentDEList.data.toString());
      currentDEList = currentDEList.next;
    }

    while (currentTBList != null)
    {
      this.tList.insertAtBack(currentTBList.data.toString());
      currentTBList = currentTBList.next;
    }

    this.tList.insertAtBack("SER");

    while (currentTEList != null)
    {
      this.tList.insertAtBack(currentTEList.data.toString());
      currentTEList = currentTEList.next;
    }

    while (!this.bList.isEmpty())
    {
      this.bList.removeFromBack();
    }
    while (!this.eList.isEmpty())
    {
      this.eList.removeFromBack();
    }
    while (!this.dbList.isEmpty())
    {
      this.dbList.removeFromBack();
    }
    while (!this.deList.isEmpty())
    {
      this.deList.removeFromBack();
    }
    while (!this.tbList.isEmpty())
    {
      this.tbList.removeFromBack();
    }
    while (!this.teList.isEmpty())
    {
      this.teList.removeFromBack();
    }

    this.updButton6.setEnabled(false);

    this.update = false;
    this.addButton6.setEnabled(true);
    this.rmvButton.setEnabled(false);

    for (int t = 0; t < this.tabcount; t++)
    {
        this.jTabbedPane1.setEnabledAt(t, true);
    }
    pkEnableDisable();

    this.sttext.setText("");
    this.dltext.setText("");
    this.source2.setSelectedIndex(0);
    this.dest2.setSelectedIndex(0);
    this.splatetype4.setSelectedIndex(0);
    this.splatetype4.revalidate();
    this.splatetype4.repaint();
    this.splatetype2.setSelectedIndex(0);
    this.splatetype2.revalidate();
    this.splatetype2.repaint();
    this.dplatetype2.setSelectedIndex(0);
    this.dplatetype2.revalidate();
    this.dplatetype2.repaint();

    this.s3inc.setSelected(false);
    this.s4inc.setSelected(false);
    i1 = "0";
    i2 = "0";
    this.button96s.setSelected(true);
    this.serPanel1.removeAll();
    getSrc2();
    this.source2.revalidate();
    this.source2.repaint();
    getSrc3();
    this.source3.revalidate();
    this.source3.repaint();
    getDest2();
    this.dest2.revalidate();
    this.dest2.repaint();
    this.splatetype2.setVisible(true);
    this.dplatetype2.setVisible(true);
    this.s3inc.setVisible(false);
    this.s4inc.setVisible(false);

    this.serPanel1.add(getSerPanel1Top(), "North");
    this.serPanel1.add(this.spanel96l, "Center");
    this.serPanel1.revalidate();
    this.serPanel1.repaint();
    
    Splash();
  }

  private void updReg()
  {
    String m1 = "0";
    String m2 = "0";
    String c1 = "0";
    String c2 = "0";
    String sh1 = "0";
    String sh2 = "0";
    String tpwa = "0";
    String v1 = "0";
    String v2 = "0";
    String line3 = "";
    String line4 = "";
    String i1 = "0";
    String i2 = "0";
    String i3 = "0";
    String i4 = "0";
    String q1 = "0";
    String q2 = "0";
    String q3 = "0";
    String q4 = "0";
    String q5 = "0";
    String q6 = "0";
    String q7 = "0";
    String q8 = "0";
    String tube1 = "";
    String tube2 = "";
    String tube3 = "";
    String tube4 = "";
    String resnum = "";
    String asp1 = "";
    String mp1 = "";
    String asp2 = "";
    String mp2 = "";
    String data = "";

    ListNode currentBList = this.bList.getFirstNode();
    ListNode currentFList = this.fList.getFirstNode();
    ListNode currentEList = this.eList.getFirstNode();

    ListNode currentDBList = this.dbList.getFirstNode();
    ListNode currentDList = this.dList.getFirstNode();
    ListNode currentDEList = this.deList.getFirstNode();

    ListNode currentTBList = this.tbList.getFirstNode();
    ListNode currentTList = this.tList.getFirstNode();
    ListNode currentTEList = this.teList.getFirstNode();

    line3 = line3 + this.text2.getText().trim();
    line3 = line3 + "," + this.lt1.trim() + ",-,-,-";

    data = data + "   \n";
    data = data + "\n " + this.text2.getText().trim();
    data = data + "\n " + this.text3.getText().trim() + " ul" + "\n";
    if (this.spl1.equals("Deck Position")) this.spl1 = "T";
    data = data + " Deck " + this.spl1;
    data = data + " to " + this.dpl1 + "\n";

    line3 = line3 + this.text2.getText().trim();
    line3 = line3 + "," + this.lt1.trim() + ",-,-,-";

    asp1 = this.advwin.getMode();
    mp1 = this.advwin.getMPA();
    data = data + " Multi: " + mp1;

    asp2 = this.advwind.getMode();
    mp2 = this.advwind.getMPA();
    data = data + "-" + mp2;

    if (this.tubepanel.incIsSelected())
    {
      i1 = "1";
    }
    if (this.tubepanel2.incIsSelected())
    {
      i2 = "1";
    }

//    if (this.linc1button.isSelected())
//    {
//      i3 = "1";
//    }
//    if (this.linc2button.isSelected())
//    {
//      i3 = "2";
//    }
//    if (this.linc3button.isSelected())
//    {
//      i3 = "3";
//    }
//    if (this.linc4button.isSelected())
//    {
//      i4 = "1";
//    }
//    if (this.linc5button.isSelected())
//    {
//      i4 = "2";
//    }
//    if (this.linc6button.isSelected())
//    {
//      i4 = "3";
//    }
    if (this.advwin.getChill() > 0)
    {
      c1 = "1";
    }
    if (this.advwin.getShake() > 0)
    {
      sh1 = "1";
    }
    if (this.advwin.getVStart() > 0)
    {
      data = data + " Recycle Tips\n";
      v1 = "1";
    }
    if (this.advwin.getTPWA() > 0)
    {
      tpwa = "1";
    }

    if (this.advwind.getChill() > 0)
    {
      c2 = "1";
    }
    if (this.advwind.getShake() > 0)
    {
      sh2 = "1";
    }
    if (this.advwind.getVStart() > 0)
    {
      data = data + " Recycle Tips\n";
      v2 = "1";
    }

    updText(data);
    int element;
    //if (this.stepnbr < 16) element = this.stepnbr - 1; else element = this.stepnbr - 16;		//APF0020 
    if ((this.stepnbr % 15) == 0) {element = 14;}  else{element = ((this.stepnbr % 15)-1);}		//APF0027
    this.dTextArea[element].setCaretPosition(0);
    this.dTextArea[element].insertIcon(this.icons[7]);

    String sourceseq = " ";
    String destseq = "0";
    tube1 = this.tubepanel.getFirstTube();
    tube2 = this.tubepanel.getLastTube();
    tube3 = this.tubepanel2.getFirstTube();
    tube4 = this.tubepanel2.getLastTube();
    resnum = this.res24Xpanel.getReservoir();

    //SOURCE
    if (this.s1.trim().endsWith("_1X")){
    	if (rHam){
    		sourceseq = "1,2,3,4,5,6,7,8";
    	}else {
    		sourceseq = "1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,52,53,54,55,56,57,58,59,60,61,62,63,64,65,66,67,68,69,70,71,72,73,74,75,76,77,78,79,80,81,82,83,84,85,86,87,88,89,90,91,92,93,94,95,96";
    	}
    }
    else if (this.s1.matches(".*_.{1,2}X")){
    	sourceseq = "1,2,3,4,5,6,7,8";
    }
    if (this.s1.trim().endsWith("Tube")){
        sourceseq = tube1 + "," + tube2 + "," + i1;
        this.spl1 = "Tube";
    }
    if (this.s1.trim().endsWith("_24")){
        this.panel24l.loadTray();
        sourceseq = this.panel24l.getTray().trim();
    }
    if (this.s1.trim().endsWith("_48")){
        this.panel48l.loadTray();
        sourceseq = this.panel48l.getTray().trim();
    }
    if (this.s1.trim().endsWith("_96")){
        this.panel96l.loadTray();
        sourceseq = this.panel96l.getTray().trim();
    }
    if ((this.s1.trim().endsWith("_384"))){
        q1 = this.panel384l.getQ1();
        q2 = this.panel384l.getQ2();
        q3 = this.panel384l.getQ3();
        q4 = this.panel384l.getQ4();
        this.panel384l.loadTray();
        sourceseq = this.panel384l.getTray().trim();
    }
    
    //DESTINATION
    if (this.de.trim().endsWith("_1X")){
    	if (rHam2){
    		destseq = "1,2,3,4,5,6,7,8";
    	}else {
    	destseq = "1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,52,53,54,55,56,57,58,59,60,61,62,63,64,65,66,67,68,69,70,71,72,73,74,75,76,77,78,79,80,81,82,83,84,85,86,87,88,89,90,91,92,93,94,95,96";
    	}
    }
    else if (this.de.matches(".*_.{1,2}X")){
    	destseq = "1,2,3,4,5,6,7,8";
    }
    if (this.de.trim().endsWith("Tube")){
        destseq = tube3 + "," + tube4 + "," + i2;
        this.dpl1 = "Tube";
    }
    if (this.de.trim().endsWith("_24")){
    	this.panel24r.loadTray();
    	destseq = this.panel24r.getTray().trim();
    }
    if (this.de.trim().endsWith("_48")){
    	this.panel48r.loadTray();
    	destseq = this.panel48r.getTray().trim();
    }
    if (this.de.trim().endsWith("_96")){
    	this.panel96r.loadTray();
    	destseq = this.panel96r.getTray().trim();
    }
    if (this.de.trim().endsWith("_384"))
    {
      q5 = this.panel384r.getQ1();
      q6 = this.panel384r.getQ2();
      q7 = this.panel384r.getQ3();
      q8 = this.panel384r.getQ4();
      this.panel384r.loadTray();
      destseq = this.panel384r.getTray().trim();
    }
    
    ///////////////////////////////////////////////////////////////////////////////////////////////////
    

    if ((this.s1.trim().endsWith("_1X")) && (this.de.trim().equals("Set_24X")))
    {
      sourceseq = "1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,52,53,54,55,56,57,58,59,60,61,62,63,64,65,66,67,68,69,70,71,72,73,74,75,76,77,78,79,80,81,82,83,84,85,86,87,88,89,90,91,92,93,94,95,96";

      destseq = resnum;

      this.spl2 = "Set";
    }

    if ((this.s1.trim().endsWith("_4X")) && (this.de.trim().equals("Set_24X")))
    {
      sourceseq = "1,2,3,4,5,6,7,8";
      destseq = resnum;

      this.spl2 = "Set";
    }
    if ((this.s1.trim().endsWith("_5X")) && (this.de.trim().equals("Set_24X")))
    {
      sourceseq = "1,2,3,4,5,6,7,8";
      destseq = resnum;

      this.spl2 = "Set";
    }
    if ((this.s1.trim().equals("Reservoir_24X")) && (this.de.trim().equals("Set_24X")))
    {
      sourceseq = "1,2,3,4,5,6,7,8";
      destseq = resnum;

      this.dpl1 = "Set";
    }

    if ((this.spl1.trim().equals("Stock")) && (!this.s1.trim().endsWith("_24X")))
    {
      this.hline5 = this.s1.trim();
    }
    if ((this.spl1.trim().equals("Intermediate")) && (!this.s1.trim().endsWith("_24X")))
    {
      this.hline6 = this.s1.trim();
    }
    if ((this.spl1.trim().equals("Assay")) && (!this.s1.trim().endsWith("_24X")))
    {
      this.hline7 = this.s1.trim();
    }
    if ((this.spl1.trim().equals("Set")) && (!this.s1.trim().endsWith("_24X")))
    {
      this.hline8 = this.s1.trim();
    }
    if ((this.dpl1.trim().equals("Stock")) && (!this.de.trim().endsWith("_24X")))
    {
      this.hline5 = this.de.trim();
    }
    if ((this.dpl1.trim().equals("Intermediate")) && (!this.de.trim().endsWith("_24X")))
    {
      this.hline6 = this.de.trim();
    }
    if ((this.dpl1.trim().equals("Assay")) && (!this.de.trim().endsWith("_24X")))
    {
      this.hline7 = this.de.trim();
    }
    if ((this.dpl1.trim().equals("Set")) && (!this.de.trim().endsWith("_24X")))
    {
      this.hline8 = this.de.trim();
    }
    if (this.s1.trim().endsWith("Tube"))
    {
      this.hline9 = this.s1.trim();
    }
    if (this.de.trim().endsWith("Tube"))
    {
      this.hline9 = this.de.trim();
    }

    line4 = line4 + this.spl1.trim() + "," + this.dpl1.trim();

    if (this.s1.trim().equals("Set_24X"))
    {
      this.s1 = "Reservoir_24X";
    }
    if (this.de.trim().equals("Set_24X"))
    {
      this.de = "Reservoir_24X";
    }

    if (sourceseq.endsWith(","))
      sourceseq = sourceseq.substring(0, sourceseq.length() - 1);
    if (destseq.endsWith(",")) {
      destseq = destseq.substring(0, destseq.length() - 1);
    }

    while (!this.fList.isEmpty())
    {
      this.fList.removeFromBack();
    }

    while (!this.dList.isEmpty())
    {
      this.dList.removeFromBack();
    }

    while (!this.tList.isEmpty())
    {
      this.tList.removeFromBack();
    }

    while (currentBList != null)
    {
      String list = currentBList.data.toString();
      this.fList.insertAtBack(currentBList.data.toString());
      currentBList = currentBList.next;
    }

    this.fList.insertAtBack("step");
    this.fList.insertAtBack("add");
    this.fList.insertAtBack(line3);
    this.fList.insertAtBack(line4);
    this.fList.insertAtBack(this.s1.trim() + "," + this.mix1 + "," + asp1 + "," + mp1 + "," + sh1 + "," + c1 + "," + tpwa + "," + v1 + "," + this.rs1);
    this.fList.insertAtBack(this.de.trim() + "," + this.mix2 + "," + asp2 + "," + mp2 + "," + sh2 + "," + c2 + "," + i4 + "," + v2 + "," + this.rs2);

    this.fList.insertAtBack(sourceseq);
    this.fList.insertAtBack(destseq);
    this.fList.insertAtBack(this.text3.getText().trim());
    this.fList.insertAtBack(q1 + "," + q2 + "," + q3 + "," + q4 + "," + q5 + "," + q6 + "," + q7 + "," + q8);

    while (currentEList != null)
    {
      this.fList.insertAtBack(currentEList.data.toString());
      currentEList = currentEList.next;
    }

    while (currentDBList != null)
    {
      this.dList.insertAtBack(currentDBList.data.toString());
      currentDBList = currentDBList.next;
    }

    this.dList.insertAtBack(data);

    while (currentDEList != null)
    {
      this.dList.insertAtBack(currentDEList.data.toString());
      currentDEList = currentDEList.next;
    }

    while (currentTBList != null)
    {
      this.tList.insertAtBack(currentTBList.data.toString());
      currentTBList = currentTBList.next;
    }

    this.tList.insertAtBack("REG");

    while (currentTEList != null)
    {
      this.tList.insertAtBack(currentTEList.data.toString());
      currentTEList = currentTEList.next;
    }

    while (!this.bList.isEmpty())
    {
      this.bList.removeFromBack();
    }
    while (!this.eList.isEmpty())
    {
      this.eList.removeFromBack();
    }
    while (!this.dbList.isEmpty())
    {
      this.dbList.removeFromBack();
    }
    while (!this.deList.isEmpty())
    {
      this.deList.removeFromBack();
    }
    while (!this.tbList.isEmpty())
    {
      this.tbList.removeFromBack();
    }
    while (!this.teList.isEmpty())
    {
      this.teList.removeFromBack();
    }

    this.update = false;
    this.updButton1.setEnabled(false);
    this.checkButton.setEnabled(false);
    this.rmvButton.setEnabled(false);
    this.addButton1.setEnabled(true);
    this.addButton.setEnabled(true);

    for (int t = 0; t < this.tabcount; t++)
    {
        this.jTabbedPane1.setEnabledAt(t, true);
    }
    pkEnableDisable();

    this.text2.setText("");
    this.text3.setText("");
    this.lmixt1.setText("0");
    this.lmixt2.setText("0");
    this.lchill.setSelected(false);
    this.lmix.setSelected(false);
    this.lmix2.setSelected(false);
    this.lshake.setSelected(false);
//    this.linc1button.setVisible(false);
//    this.linc2button.setVisible(false);
//    this.linc3button.setVisible(false);
//    this.linc4button.setVisible(false);
//    this.linc5button.setVisible(false);
//    this.linc6button.setVisible(false);
    this.saveButton.setEnabled(true);
    this.tubepanel.setFirstTube(" ");
    this.tubepanel.setLastTube(" ");
    this.tubepanel2.setFirstTube(" ");
    this.tubepanel2.setLastTube(" ");

    this.source.setSelectedIndex(0);
    this.dest.setSelectedIndex(0);
    this.splatetype.setSelectedIndex(0);
    this.dplatetype.setSelectedIndex(0);
    this.splatetype.revalidate();
    this.splatetype.repaint();
    this.dplatetype.revalidate();
    this.dplatetype.repaint();
    this.platebutton.setSelected(true);
    this.platebutton2.setSelected(true);
//    this.button96l.setSelected(true);
//    this.button96l2.setSelected(true);
    i1 = "0";
    i2 = "0";
    i3 = "0";
    i4 = "0";
    c1 = "0";
    sh1 = "0";
    v1 = "0";
    asp1 = "";
    mp1 = "";
    c2 = "0";
    sh2 = "0";
    v2 = "0";
    asp2 = "";
    mp2 = "";
    this.advwin.setChill(false);
    this.advwin.setShake(false);
    this.advwin.setVStart(false);
    this.advwin.setASPMode("SMART");
    this.advwin.setMPA(1);
    this.advwind.setChill(false);
    this.advwind.setShake(false);
    this.advwind.setVStart(false);
    this.advwind.setASPMode("SMART");
    this.advwind.setMPA(1);
    this.panel96l.clearTray();
    this.panel96r.clearTray();
    this.panel48l.clearTray();
    this.panel48r.clearTray();
    this.panel24l.clearTray();
    this.panel24r.clearTray();
    this.liqPanel1.removeAll();
    this.liqPanel2.removeAll();
    getSrc();
    this.source.revalidate();
    this.source.repaint();
    getDest();
    this.dest.revalidate();
    this.dest.repaint();
    this.dplatetype.setVisible(true);
    this.splatetype.setVisible(true);
    this.s1inc.setVisible(false);
    this.s2inc.setVisible(false);
    for (int d = 0; d < 12; d++)
    {
      if (this.resTextArea[d] == null)
        continue;
      this.resTextArea[d].setBorder(this.blackline);
    }

    for (int d2 = 0; d2 < 12; d2++)
    {
      if (this.resTextArea2[d2] == null)
        continue;
      this.resTextArea2[d2].setBorder(this.blackline);
    }

    this.liqPanel1.add(getLiqPanel1Top(), "North");
    this.liqPanel1.add(this.panelbl, "Center");
    this.liqPanel1.add(getLiqPanel1Bot(), "South");
    this.liqPanel1.revalidate();
    this.liqPanel1.repaint();
    this.liqPanel2.add(getLiqPanel2Top(), "North");
    this.liqPanel2.add(this.panelbr, "Center");
    this.liqPanel2.add(getLiqPanel2Bot(), "South");
    this.liqPanel2.revalidate();
    this.liqPanel2.repaint();
    
    Splash();
  }

  private void updWash()
  {
    ListNode currentBList = this.bList.getFirstNode();
    ListNode currentFList = this.fList.getFirstNode();
    ListNode currentEList = this.eList.getFirstNode();

    ListNode currentDBList = this.dbList.getFirstNode();
    ListNode currentDList = this.dList.getFirstNode();
    ListNode currentDEList = this.deList.getFirstNode();

    ListNode currentTBList = this.tbList.getFirstNode();
    ListNode currentTList = this.tList.getFirstNode();
    ListNode currentTEList = this.teList.getFirstNode();

    String line4 = "";
    String data = "";

    while (!this.fList.isEmpty())
    {
      this.fList.removeFromBack();
    }

    while (!this.dList.isEmpty())
    {
      this.dList.removeFromBack();
    }

    while (!this.tList.isEmpty())
    {
      this.tList.removeFromBack();
    }

    while (currentBList != null)
    {
      this.fList.insertAtBack(currentBList.data.toString());
      currentBList = currentBList.next;
    }

    line4 = this.wmethod.getText().trim();

    data = data + "   \n";
    data = data + "\n " + this.wmethod.getText().trim() + "\n";

    if (line4.trim().equals(""))
    {
      line4 = "-";
    }

    updText(data);

    this.fList.insertAtBack("step");
    this.fList.insertAtBack("wash");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack(line4);
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");

    while (currentEList != null)
    {
      this.fList.insertAtBack(currentEList.data.toString());
      currentEList = currentEList.next;
    }

    while (currentDBList != null)
    {
      this.dList.insertAtBack(currentDBList.data.toString());
      currentDBList = currentDBList.next;
    }

    this.dList.insertAtBack(data);

    while (currentDEList != null)
    {
      this.dList.insertAtBack(currentDEList.data.toString());
      currentDEList = currentDEList.next;
    }

    while (currentTBList != null)
    {
      this.tList.insertAtBack(currentTBList.data.toString());
      currentTBList = currentTBList.next;
    }

    this.tList.insertAtBack("WSH");

    while (currentTEList != null)
    {
      this.tList.insertAtBack(currentTEList.data.toString());
      currentTEList = currentTEList.next;
    }

    while (!this.bList.isEmpty())
    {
      this.bList.removeFromBack();
    }
    while (!this.eList.isEmpty())
    {
      this.eList.removeFromBack();
    }
    while (!this.dbList.isEmpty())
    {
      this.dbList.removeFromBack();
    }
    while (!this.deList.isEmpty())
    {
      this.deList.removeFromBack();
    }
    while (!this.tbList.isEmpty())
    {
      this.tbList.removeFromBack();
    }
    while (!this.teList.isEmpty())
    {
      this.teList.removeFromBack();
    }

    this.wmethod.setText("");

    this.updButton3.setEnabled(false);

    this.addButton3.setEnabled(true);
    this.rmvButton.setEnabled(false);

    this.update = false;
    for (int t = 0; t < this.tabcount; t++)
    {
        this.jTabbedPane1.setEnabledAt(t, true);
    }
    pkEnableDisable();

  }

  private void updInc()
  {
    ListNode currentBList = this.bList.getFirstNode();
    ListNode currentFList = this.fList.getFirstNode();
    ListNode currentEList = this.eList.getFirstNode();

    ListNode currentDBList = this.dbList.getFirstNode();
    ListNode currentDList = this.dList.getFirstNode();
    ListNode currentDEList = this.deList.getFirstNode();

    ListNode currentTBList = this.tbList.getFirstNode();
    ListNode currentTList = this.tList.getFirstNode();
    ListNode currentTEList = this.teList.getFirstNode();

    String line3 = "";
    String line4 = "";
    String line5 = "";
    String data = "";

    while (!this.fList.isEmpty())
    {
      this.fList.removeFromBack();
    }

    while (!this.dList.isEmpty())
    {
      this.dList.removeFromBack();
    }

    while (!this.tList.isEmpty())
    {
      this.tList.removeFromBack();
    }

    while (currentBList != null)
    {
      this.fList.insertAtBack(currentBList.data.toString());
      currentBList = currentBList.next;
    }

    line3 = this.text4.getText().trim();

    line4 = this.itemp.getText().trim();
    if (line4.trim().equals(""))
    {
      line4 = "0";
    }
    line5 = this.text4pm.getText().trim();
    if (line5.trim().equals(""))
    {
      line5 = "0";
    }

    data = data + "   \n";
    data = data + "\n " + this.text4.getText().trim() + " min";
    data = data + "\n " + this.itemp.getText().trim() + " (c)" + "\n";

    updText(data);

    this.fList.insertAtBack("step");
    this.fList.insertAtBack("incubate");
    this.fList.insertAtBack(line3);
    this.fList.insertAtBack(line4);
    this.fList.insertAtBack(line5);

    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");

    while (currentEList != null)
    {
      this.fList.insertAtBack(currentEList.data.toString());
      currentEList = currentEList.next;
    }

    while (currentDBList != null)
    {
      this.dList.insertAtBack(currentDBList.data.toString());
      currentDBList = currentDBList.next;
    }

    this.dList.insertAtBack(data);

    while (currentDEList != null)
    {
      this.dList.insertAtBack(currentDEList.data.toString());
      currentDEList = currentDEList.next;
    }

    while (currentTBList != null)
    {
      this.tList.insertAtBack(currentTBList.data.toString());
      currentTBList = currentTBList.next;
    }

    this.tList.insertAtBack("INC");

    while (currentTEList != null)
    {
      this.tList.insertAtBack(currentTEList.data.toString());
      currentTEList = currentTEList.next;
    }

    while (!this.bList.isEmpty())
    {
      this.bList.removeFromBack();
    }
    while (!this.eList.isEmpty())
    {
      this.eList.removeFromBack();
    }
    while (!this.dbList.isEmpty())
    {
      this.dbList.removeFromBack();
    }
    while (!this.deList.isEmpty())
    {
      this.deList.removeFromBack();
    }
    while (!this.tbList.isEmpty())
    {
      this.tbList.removeFromBack();
    }
    while (!this.teList.isEmpty())
    {
      this.teList.removeFromBack();
    }

    this.text4.setText(" ");
    this.text4pm.setText("0");

    this.updButton2.setEnabled(false);

    this.update = false;
    this.addButton2.setEnabled(true);
    this.rmvButton.setEnabled(false);

    for (int t = 0; t < this.tabcount; t++)
    {
        this.jTabbedPane1.setEnabledAt(t, true);
    }
    pkEnableDisable();

  }

  private void updMove()
  {
    ListNode currentBList = this.bList.getFirstNode();
    ListNode currentFList = this.fList.getFirstNode();
    ListNode currentEList = this.eList.getFirstNode();

    ListNode currentDBList = this.dbList.getFirstNode();
    ListNode currentDList = this.dList.getFirstNode();
    ListNode currentDEList = this.deList.getFirstNode();

    ListNode currentTBList = this.tbList.getFirstNode();
    ListNode currentTList = this.tList.getFirstNode();
    ListNode currentTEList = this.teList.getFirstNode();

    String line4 = "";
    String data = "";

    while (!this.fList.isEmpty())
    {
      this.fList.removeFromBack();
    }

    while (!this.dList.isEmpty())
    {
      this.dList.removeFromBack();
    }

    while (!this.tList.isEmpty())
    {
      this.tList.removeFromBack();
    }

    while (currentBList != null)
    {
      this.fList.insertAtBack(currentBList.data.toString());
      currentBList = currentBList.next;
    }

    data = data + "\n MOVE\n ";
    data = data + "\n " + this.mov;

    this.fList.insertAtBack("step");
    this.fList.insertAtBack("move");
    this.fList.insertAtBack(this.mov);
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");

    updText(data);

    while (currentEList != null)
    {
      this.fList.insertAtBack(currentEList.data.toString());
      currentEList = currentEList.next;
    }

    while (currentDBList != null)
    {
      this.dList.insertAtBack(currentDBList.data.toString());
      currentDBList = currentDBList.next;
    }

    this.dList.insertAtBack(data);

    while (currentDEList != null)
    {
      this.dList.insertAtBack(currentDEList.data.toString());
      currentDEList = currentDEList.next;
    }

    while (currentTBList != null)
    {
      this.tList.insertAtBack(currentTBList.data.toString());
      currentTBList = currentTBList.next;
    }

    this.tList.insertAtBack("MOV");

    while (currentTEList != null)
    {
      this.tList.insertAtBack(currentTEList.data.toString());
      currentTEList = currentTEList.next;
    }

    while (!this.bList.isEmpty())
    {
      this.bList.removeFromBack();
    }
    while (!this.eList.isEmpty())
    {
      this.eList.removeFromBack();
    }
    while (!this.dbList.isEmpty())
    {
      this.dbList.removeFromBack();
    }
    while (!this.deList.isEmpty())
    {
      this.deList.removeFromBack();
    }
    while (!this.tbList.isEmpty())
    {
      this.tbList.removeFromBack();
    }
    while (!this.teList.isEmpty())
    {
      this.teList.removeFromBack();
    }

    this.wmethod.setText("");

    this.updButton8.setEnabled(false);

    this.update = false;
    this.addButton8.setEnabled(true);
    this.rmvButton.setEnabled(false);

    for (int t = 0; t < this.tabcount; t++)
    {
        this.jTabbedPane1.setEnabledAt(t, true);
    }
    pkEnableDisable();

  }

  private void updMovePlate()
  {
    ListNode currentBList = this.bList.getFirstNode();
    ListNode currentFList = this.fList.getFirstNode();
    ListNode currentEList = this.eList.getFirstNode();

    ListNode currentDBList = this.dbList.getFirstNode();
    ListNode currentDList = this.dList.getFirstNode();
    ListNode currentDEList = this.deList.getFirstNode();

    ListNode currentTBList = this.tbList.getFirstNode();
    ListNode currentTList = this.tList.getFirstNode();
    ListNode currentTEList = this.teList.getFirstNode();

    String data = "";

    while (!this.fList.isEmpty())
    {
      this.fList.removeFromBack();
    }

    while (!this.dList.isEmpty())
    {
      this.dList.removeFromBack();
    }

    while (!this.tList.isEmpty())
    {
      this.tList.removeFromBack();
    }

    while (currentBList != null)
    {
      this.fList.insertAtBack(currentBList.data.toString());
      currentBList = currentBList.next;
    }

    data = data + "   \n ";
    data = data + this.so5 + "\n" + "Deck " + this.spl3 + " to " + this.dpl3 + "\n\n";

    this.fList.insertAtBack("step");
    this.fList.insertAtBack("move plate");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack(this.spl3 + "," + this.dpl3);
    this.fList.insertAtBack(this.so5);
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");

    updText(data);
    int element;
//    if (this.stepnbr < 16) element = this.stepnbr - 1; else element = this.stepnbr - 16;		//APF0020 
    if ((this.stepnbr % 15) == 0) {element = 14;}  else{element = ((this.stepnbr % 15)-1);}		//APF0027
    
    this.dTextArea[element].setCaretPosition(2);
    this.dTextArea[element].insertIcon(this.icons[8]);	//updMovePlate()

    while (currentEList != null)
    {
      this.fList.insertAtBack(currentEList.data.toString());
      currentEList = currentEList.next;
    }

    while (currentDBList != null)
    {
      this.dList.insertAtBack(currentDBList.data.toString());
      currentDBList = currentDBList.next;
    }

    this.dList.insertAtBack(data);

    while (currentDEList != null)
    {
      this.dList.insertAtBack(currentDEList.data.toString());
      currentDEList = currentDEList.next;
    }

    while (currentTBList != null)
    {
      this.tList.insertAtBack(currentTBList.data.toString());
      currentTBList = currentTBList.next;
    }

    this.tList.insertAtBack("MOVPL");

    while (currentTEList != null)
    {
      this.tList.insertAtBack(currentTEList.data.toString());
      currentTEList = currentTEList.next;
    }

    while (!this.bList.isEmpty())
    {
      this.bList.removeFromBack();
    }
    while (!this.eList.isEmpty())
    {
      this.eList.removeFromBack();
    }
    while (!this.dbList.isEmpty())
    {
      this.dbList.removeFromBack();
    }
    while (!this.deList.isEmpty())
    {
      this.deList.removeFromBack();
    }
    while (!this.tbList.isEmpty())
    {
      this.tbList.removeFromBack();
    }
    while (!this.teList.isEmpty())
    {
      this.teList.removeFromBack();
    }

    this.updButton10.setEnabled(false);
    this.checkButton.setEnabled(false);

    this.update = false;
    this.addButton.setEnabled(true);
    this.addButton10.setEnabled(true);
    this.rmvButton.setEnabled(false);
    this.saveButton.setEnabled(true);
    this.saveButton.setEnabled(true);
    this.source5.setSelectedIndex(0);
    this.splatetype3.setSelectedIndex(0);
    this.dplatetype3.setSelectedIndex(0);
    this.splatetype3.revalidate();
    this.splatetype3.repaint();
    this.dplatetype3.revalidate();
    this.dplatetype3.repaint();

    for (int t = 0; t < this.tabcount; t++)
    {
        this.jTabbedPane1.setEnabledAt(t, true);
    }
    pkEnableDisable();
   
    //Return to Splash screen
    Splash();
  }

  private void updCherry()
  {
    ListNode currentBList = this.bList.getFirstNode();
    ListNode currentFList = this.fList.getFirstNode();
    ListNode currentEList = this.eList.getFirstNode();

    ListNode currentDBList = this.dbList.getFirstNode();
    ListNode currentDList = this.dList.getFirstNode();
    ListNode currentDEList = this.deList.getFirstNode();

    ListNode currentTBList = this.tbList.getFirstNode();
    ListNode currentTList = this.tList.getFirstNode();
    ListNode currentTEList = this.teList.getFirstNode();

    String data = "";

    while (!this.fList.isEmpty())
    {
      this.fList.removeFromBack();
    }

    while (!this.dList.isEmpty())
    {
      this.dList.removeFromBack();
    }

    while (!this.tList.isEmpty())
    {
      this.tList.removeFromBack();
    }

    while (currentBList != null)
    {
      this.fList.insertAtBack(currentBList.data.toString());
      currentBList = currentBList.next;
    }

    data = data + "\n\n " + this.text7.getText().trim();
    data = data + "\n " + this.tr1.trim() + " to " + this.tr2.trim();

    this.fList.insertAtBack("step");
    this.fList.insertAtBack("Cherry Picking");
    this.fList.insertAtBack(this.text7.getText().trim() + "," + this.chltyp.trim());
    this.fList.insertAtBack(this.tr1.trim() + "," + this.tr2.trim());
    this.fList.insertAtBack(this.chso.trim());
    this.fList.insertAtBack(this.chde.trim());
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");

    updText(data);
    int element;
//    if (this.stepnbr < 16) element = this.stepnbr - 1; else element = this.stepnbr - 16;		//APF0020 
    if ((this.stepnbr % 15) == 0) {element = 14;}  else{element = ((this.stepnbr % 15)-1);}		//APF0027
    
    this.dTextArea[element].setCaretPosition(0);
    this.dTextArea[element].insertIcon(this.icons[19]);

    while (currentEList != null)
    {
      this.fList.insertAtBack(currentEList.data.toString());
      currentEList = currentEList.next;
    }

    while (currentDBList != null)
    {
      this.dList.insertAtBack(currentDBList.data.toString());
      currentDBList = currentDBList.next;
    }

    this.dList.insertAtBack(data);

    while (currentDEList != null)
    {
      this.dList.insertAtBack(currentDEList.data.toString());
      currentDEList = currentDEList.next;
    }

    while (currentTBList != null)
    {
      this.tList.insertAtBack(currentTBList.data.toString());
      currentTBList = currentTBList.next;
    }

    this.tList.insertAtBack("CHR");

    while (currentTEList != null)
    {
      this.tList.insertAtBack(currentTEList.data.toString());
      currentTEList = currentTEList.next;
    }

    while (!this.bList.isEmpty())
    {
      this.bList.removeFromBack();
    }
    while (!this.eList.isEmpty())
    {
      this.eList.removeFromBack();
    }
    while (!this.dbList.isEmpty())
    {
      this.dbList.removeFromBack();
    }
    while (!this.deList.isEmpty())
    {
      this.deList.removeFromBack();
    }
    while (!this.tbList.isEmpty())
    {
      this.tbList.removeFromBack();
    }
    while (!this.teList.isEmpty())
    {
      this.teList.removeFromBack();
    }

    this.updButton11.setEnabled(false);
    this.checkButton.setEnabled(false);

    this.update = false;
    this.addButton.setEnabled(true);
    this.addButton11.setEnabled(true);
    this.rmvButton.setEnabled(false);
    this.saveButton.setEnabled(true);
    this.track1.setSelectedIndex(0);
    this.track2.setSelectedIndex(0);
    this.chsource.setSelectedIndex(0);
    this.chdest.setSelectedIndex(0);
    this.chliq.setSelectedIndex(0);
    this.text7.setText("");

    for (int t = 0; t < this.tabcount; t++)
    {
        this.jTabbedPane1.setEnabledAt(t, true);
    }
    pkEnableDisable();
    
    //Return to Splash screen
    Splash();
  }

  private void updCherry02()
  {
    ListNode currentBList = this.bList.getFirstNode();
    ListNode currentFList = this.fList.getFirstNode();
    ListNode currentEList = this.eList.getFirstNode();

    ListNode currentDBList = this.dbList.getFirstNode();
    ListNode currentDList = this.dList.getFirstNode();
    ListNode currentDEList = this.deList.getFirstNode();

    ListNode currentTBList = this.tbList.getFirstNode();
    ListNode currentTList = this.tList.getFirstNode();
    ListNode currentTEList = this.teList.getFirstNode();

    String data = "";
    String line3 = "";
    String line4 = "";
    String line5 = "";
    String line6 = "";
    String line7 = "";
    String line8 = "";
    String line9 = "";
    String line10 = "";

    while (!this.fList.isEmpty())
    {
      this.fList.removeFromBack();
    }

    while (!this.dList.isEmpty())
    {
      this.dList.removeFromBack();
    }

    while (!this.tList.isEmpty())
    {
      this.tList.removeFromBack();
    }

    while (currentBList != null)
    {
      this.fList.insertAtBack(currentBList.data.toString());
      currentBList = currentBList.next;
    }

//    	Text writting to Step Icon.
//    	Get the file name from path and split into multiple lines to allow defined wrapping in icon 
    	String fileName = cherry02dataPanel2.getWorkFile().trim().substring(cherry02dataPanel2.getWorkFile().trim().lastIndexOf("\\") + 1);
    	
		if (fileName.length() > 12){fileName = new StringBuilder(fileName).insert(12, "...\n").toString();}		//CPF0032
		if (fileName.length() > 27)fileName = new StringBuilder(fileName).insert(27, "...\n").toString();		//CPF0032
    	
    	
    	data = data + "\n\n" + fileName;
//    	data = data + "\n " + ch2_plateloc.trim();
//    	data = data + "\n " + ch2_liqtype.trim();

	ch2_smix = cherry02dataPanel2.getSMix();
	ch2_dmix = cherry02dataPanel2.getDMix();
	ch2_scycle = cherry02dataPanel2.getSCycle();
	ch2_dcycle = cherry02dataPanel2.getDCycle();

    line3 = this.cherry02dataPanel2.getWorkFile().trim();
    line4 = ch2_plateloc;
    line5 = this.cherry02dataPanel2.getSPlateType().trim();
    line6 = this.cherry02dataPanel2.getDPlateType().trim();
    line7 = ch2_liqtype.trim();
    line8 = ch2_scycle + "," + ch2_smix  ;
    line9 = ch2_dcycle + "," + ch2_dmix  ;
    line10 = ch2_tubeloc;
    		
    this.fList.insertAtBack("step");
    this.fList.insertAtBack("CherryPicking02");
    this.fList.insertAtBack(line3);
    this.fList.insertAtBack(line4);
    this.fList.insertAtBack(line5);
    this.fList.insertAtBack(line6);
    this.fList.insertAtBack(line7);
    this.fList.insertAtBack(line8);
    this.fList.insertAtBack(line9);
    this.fList.insertAtBack(line10);

    updText(data);
    
    int element;
//    if (this.stepnbr < 16) element = this.stepnbr - 1; else element = this.stepnbr - 16;		//APF0020 
    if ((this.stepnbr % 15) == 0) {element = 14;}  else{element = ((this.stepnbr % 15)-1);}		//APF0027
    
    this.dTextArea[element].setCaretPosition(0);
    this.dTextArea[element].insertIcon(this.icons[19]);

    while (currentEList != null)
    {
      this.fList.insertAtBack(currentEList.data.toString());
      currentEList = currentEList.next;
    }

    while (currentDBList != null)
    {
      this.dList.insertAtBack(currentDBList.data.toString());
      currentDBList = currentDBList.next;
    }

    this.dList.insertAtBack(data);

    while (currentDEList != null)
    {
      this.dList.insertAtBack(currentDEList.data.toString());
      currentDEList = currentDEList.next;
    }

    while (currentTBList != null)
    {
      this.tList.insertAtBack(currentTBList.data.toString());
      currentTBList = currentTBList.next;
    }

    this.tList.insertAtBack("CHR2");

    while (currentTEList != null)
    {
      this.tList.insertAtBack(currentTEList.data.toString());
      currentTEList = currentTEList.next;
    }

    while (!this.bList.isEmpty())
    {
      this.bList.removeFromBack();
    }
    while (!this.eList.isEmpty())
    {
      this.eList.removeFromBack();
    }
    while (!this.dbList.isEmpty())
    {
      this.dbList.removeFromBack();
    }
    while (!this.deList.isEmpty())
    {
      this.deList.removeFromBack();
    }
    while (!this.tbList.isEmpty())
    {
      this.tbList.removeFromBack();
    }
    while (!this.teList.isEmpty())
    {
      this.teList.removeFromBack();
    }

    this.cherry02_btnUpdateStep.setEnabled(false);
    this.checkButton.setEnabled(false);

    this.update = false;
    this.addButton.setEnabled(true);
    this.cherry02_btnAddStep.setEnabled(true);
    this.rmvButton.setEnabled(false);
    this.saveButton.setEnabled(true);
    this.cherry02dataPanel2.setSLabIndex(0);
    this.cherry02dataPanel2.setDLabIndex(0);
    this.cherry02dataPanel2.setSPlateIndex(0);
    this.cherry02dataPanel2.setDPlateIndex(0);
    this.cherry02dataPanel2.setLiqIndex(0);
    this.cherry02dataPanel2.setBarIndex(0);
    this.cherry02dataPanel2.setText1("");
    this.cherry02dataPanel2.setText2("");
    this.cherry02dataPanel2.setText3("");
    this.cherry02dataPanel2.setText4("");
    this.cherry02dataPanel2.setSCycle1(0);
    this.cherry02dataPanel2.setDCycle1(0);
    this.cherry02dataPanel2.setT1Enabled(false);
    this.cherry02dataPanel2.setT3Enabled(false);
    this.cherry02dataPanel2.setT4Enabled(false);

    for (int t = 0; t < this.tabcount; t++)
    {
        this.jTabbedPane1.setEnabledAt(t, true);
    }
    pkEnableDisable();
    
    //Return to Splash screen
    Splash();
  }
  
  private void updPK()
  {
  
    ListNode currentBList = this.bList.getFirstNode();
    ListNode currentFList = this.fList.getFirstNode();
    ListNode currentEList = this.eList.getFirstNode();

    ListNode currentDBList = this.dbList.getFirstNode();
    ListNode currentDList = this.dList.getFirstNode();
    ListNode currentDEList = this.deList.getFirstNode();

    ListNode currentTBList = this.tbList.getFirstNode();
    ListNode currentTList = this.tList.getFirstNode();
    ListNode currentTEList = this.teList.getFirstNode();

    String data = "";

    while (!this.fList.isEmpty())
    {
      this.fList.removeFromBack();
    }

    while (!this.dList.isEmpty())
    {
      this.dList.removeFromBack();
    }

    while (!this.tList.isEmpty())
    {
      this.tList.removeFromBack();
    }

    while (currentBList != null)
    {
      this.fList.insertAtBack(currentBList.data.toString());
      currentBList = currentBList.next;
    }
    String line3 = "";
    String line4 = "";
    String line5 = "";
    String line6 = "";
    String line7 = "";

    pksam = this.pkdataPanel2x.getpkmsamdil().trim();
    pkstd = this.pkdataPanel2x.getpkmstddil().trim();
    pkmrddeck = this.pkdataPanel2x.getpkdeck1().trim();
	pkdillab = this.pkdataPanel2x.getpkdilsrc2().trim();
    pkdillab2 = this.pkdataPanel2x.getpkdilsrc1().trim();
    pkmrddil = this.pkdataPanel2x.getpkmmrddil().trim();
    pkdildil = this.pkdataPanel2x.getpkdil().trim();
    pkdildeck = this.pkdataPanel2x.getpkdeck2().trim();
    pkdillab3 = this.pkdataPanel2x.getpkdilres().trim();



	pksamlab = this.pkdataPanel2x.getpkssrc().trim();
	pkstdlab = this.pkdataPanel2x.getpkstdsrc().trim();
	pkmrdlab = this.pkdataPanel2x.getpkmrdsrc().trim();
	pkdillab = this.pkdataPanel2x.getpkdilsrc2().trim();
	pkpasslab = this.pkdataPanel2x.getpassaylab().trim();
	pkasslab = this.pkdataPanel2x.getassaylab().trim();
    if (!mrdon)
    {
//  	  pksam = "null";
//  	  pkstd = "null";
//	  pkmrddeck = "null";
      pkmrdlab = "null";
  	  pkm_text_1.setText("");
  	  pkm_text_ul.setText("");
//  	  pkdillab2 = "null";
      if (!dilon)
      {
      	  pkmrddil = "null";
    	  pkdildil = "null";
    	  pkdildeck = "null";
      	  pkdillab3 = "null";
      }
      if (!preassayon)
      {
    	  pkstdlab = "null";
//    	  TabWindow.this.pkMRDsource.setEnabled(false);
//    	  TabWindow.this.pkMRDdil.setEnabled(false);
      }
    }

    if (!dilon)
    {
  	  pkdildil = "null";
	  pkdildeck = "null";
  	  pkdillab3 = "null";
    }


	line3 = this.pkdataPanel2x.getpkmtextwat().trim() + "," + this.pkdataPanel2x.getpkmtext1().trim()
	+ ","  + this.pkdataPanel2x.getpkmtextul().trim() + ","  + this.pkdataPanel2x.getpkmpassayul().trim() + ","  
    + this.pkdataPanel2x.getpkmassayul().trim();

    line4 = this.pksam.trim() + "," + this.pkstd.trim()
    + "," + this.pkmrddil.trim() + "," + this.pkdildil.trim();

    if (mrdon)
    {
      pkck1 = "1";
    }
    else
    {
  	  pkmrdlab = "null";
      pkck1 = "0";
      if (!dilon)
      {
    	  pkmrdlab = "null";
      }
      if (!preassayon)
      {
    	  pkstdlab = "null";
      }
    }

    if (dilon)
    {
      pkck2 = "1";
    }
    else
    {
      pkck2 = "0";
      pkdillab = "null";
//	  TabWindow.this.pkDsource.setEnabled(false);
      if (!mrdon)
      {
    	  pkmrdlab = "null";
//    	  TabWindow.this.pkMRDsource.setEnabled(false);
//    	  TabWindow.this.pkMRDdil.setEnabled(false);
      }
    }
    if (preassayon)
    {
      pkck3 = "1";
    }
    else
    {
      pkck3 = "0";
      pkpasslab = "null";
//	  TabWindow.this.pkPassay.setEnabled(false);
    }
    
    if (assayon)
    {
      pkck4 = "1";
    }
    else
    {
      pkck4 = "0";
      pkasslab = "null";
//	  TabWindow.this.pkPassay.setEnabled(false);
//	  TabWindow.this.text12.setEnabled(false);
    }
    
    line5 = this.pksamlab.trim() + "," + this.pkstdlab.trim() + "," + this.pkmrdlab.trim() 
    + "," + this.pkdillab.trim() + ","  + this.pkpasslab.trim()+ "," + this.pkasslab.trim();

    line6 = pkck1 +  ","  + pkck2 +  "," + pkck3 +  ","  + pkck4 ;

    line7 = pkdillab2 + "," + pkmrddeck +  "," + pkdillab3 + "," + pkdildeck;
    

    data = "\n\nPK SAMPLE PREP";

    this.fList.insertAtBack("step");
    this.fList.insertAtBack("PK");
    this.fList.insertAtBack(line3);
    this.fList.insertAtBack(line4);
    this.fList.insertAtBack(line5);
    this.fList.insertAtBack(line6);
    this.fList.insertAtBack(line7);
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");

    updText(data);
    int element;
//    if (this.stepnbr < 16) element = this.stepnbr - 1; else element = this.stepnbr - 16;		//APF0020 
    if ((this.stepnbr % 15) == 0) {element = 14;}  else{element = ((this.stepnbr % 15)-1);}		//APF0027
    
    this.dTextArea[element].setCaretPosition(0);
    this.dTextArea[element].insertIcon(this.icons[23]);

    while (currentEList != null)
    {
      this.fList.insertAtBack(currentEList.data.toString());
      currentEList = currentEList.next;
    }

    while (currentDBList != null)
    {
      this.dList.insertAtBack(currentDBList.data.toString());
      currentDBList = currentDBList.next;
    }

    this.dList.insertAtBack(data);

    while (currentDEList != null)
    {
      this.dList.insertAtBack(currentDEList.data.toString());
      currentDEList = currentDEList.next;
    }

    while (currentTBList != null)
    {
      this.tList.insertAtBack(currentTBList.data.toString());
      currentTBList = currentTBList.next;
    }

    this.tList.insertAtBack("CHR");

    while (currentTEList != null)
    {
      this.tList.insertAtBack(currentTEList.data.toString());
      currentTEList = currentTEList.next;
    }

    while (!this.bList.isEmpty())
    {
      this.bList.removeFromBack();
    }
    while (!this.eList.isEmpty())
    {
      this.eList.removeFromBack();
    }
    while (!this.dbList.isEmpty())
    {
      this.dbList.removeFromBack();
    }
    while (!this.deList.isEmpty())
    {
      this.deList.removeFromBack();
    }
    while (!this.tbList.isEmpty())
    {
      this.tbList.removeFromBack();
    }
    while (!this.teList.isEmpty())
    {
      this.teList.removeFromBack();
    }

    this.btnUpdateStep.setEnabled(false);
    this.checkButton.setEnabled(false);

    this.update = false;
    this.addButton.setEnabled(true);
    this.btnAdd.setEnabled(true);
//    this.addButton12.setEnabled(true);
    this.rmvButton.setEnabled(false);
    this.saveButton.setEnabled(true);

//    this.pkdataPanel2x.setpkmtextwat("");
//    this.pkdataPanel2x.setpkmtext1("");
//    this.pkdataPanel2x.setpkmtextul("");
    this.pkdataPanel2x.setmrdindex(0);
    this.pkdataPanel2x.setsamindex(0);
    this.pkdataPanel2x.setstdindex(0);
    this.pkdataPanel2x.setmrddilindex(0);
    this.pkdataPanel2x.setsamdilindex(0);
    this.pkdataPanel2x.setstddilindex(0);
    this.pkdataPanel2x.setdeck1index(0);
    this.pkdataPanel2x.setdilsrcindex(0);
    this.pkdataPanel2x.setMRDon(false);
//    this.pkdataPanel2x.mrdon = false;

    this.pkdataPanel2x.setDILon(false);
//    this.pkdataPanel2x.dilon = false;
    this.pkdataPanel2x.setdilsrcindex2(0);
    this.pkdataPanel2x.setdilresindex(0);
    this.pkdataPanel2x.setdeck2index(0);
    this.pkdataPanel2x.setpkmpassayul("");
    
    this.pkdataPanel2x.setPRAon(false);
//    this.pkdataPanel2x.preassayon = false;
    this.pkdataPanel2x.setpassaylabindex(0);

    this.pkdataPanel2x.setASSon(false);
//    this.pkdataPanel2x.assayon = false;
    this.pkdataPanel2x.setassaylabindex(0);
    
//    this.pkstddil.setSelectedIndex(0);
//    this.pksamdil.setSelectedIndex(0);
//    this.pkMRDdil.setSelectedIndex(0);
//    this.pkdil.setSelectedIndex(0);
//    this.pkstddil.setEnabled(false);
//    this.pksamdil.setEnabled(false);
//    this.pkMRDdil.setEnabled(false);
//    this.pkdil.setEnabled(false);
//	this.pkDsource.setEnabled(false);
//    this.pkm_text_wat.setText("");
//    this.pkm_text_wat.setEditable(true);
//    this.pkm_text_1.setText("");
//    this.pkm_text_1.setEditable(true);
//    this.pkm_text_ul.setText("");
//    this.pkm_text_ul.setEditable(true);
//    this.pkassay.setSelectedIndex(0);
//    this.pkassay.setEnabled(false);
//    this.pkPassay.setSelectedIndex(0);
//    this.pkPassay.setEnabled(false);
//    this.pkMRDsource.setSelectedIndex(0);
//    this.pkMRDsource.setEnabled(false);
//    this.pkSsource.setSelectedIndex(0);
//    this.pkSsource.setEnabled(false);
//    this.pkDsource.setSelectedIndex(0);
//    this.pkDsource.setEnabled(false);
//    this.pkSTDsource.setEnabled(false);
//    this.pkSTDsource.setSelectedIndex(0);
//    this.pkdeck1.setEnabled(false);
//    this.pkdeck1.setVisible(false);
//    this.pkdeck1.setSelectedIndex(0);
//    this.pkdeck2.setEnabled(false);
//    this.pkdeck2.setVisible(false);
//    this.pkdeck2.setSelectedIndex(0);
//	this.pkDilsource.setEnabled(false);
//	this.pkDilreserve.setEnabled(false);
//	this.lblDiluent2.setVisible(false);
//    this.pkcheck1.setSelected(false);
//    this.pkcheck2.setSelected(false);
//    this.pkcheck3.setSelected(false);
//    this.pkcheck4.setSelected(false);

    for (int t = 0; t < this.tabcount; t++)
    {
        this.jTabbedPane1.setEnabledAt(t, true);
    }
    pkEnableDisable();
    
    //Return to Splash screen
    Splash();
  }

  private void updPKsave()
  {
  
    ListNode currentBList = this.bList.getFirstNode();
    ListNode currentFList = this.fList.getFirstNode();
    ListNode currentEList = this.eList.getFirstNode();

    ListNode currentDBList = this.dbList.getFirstNode();
    ListNode currentDList = this.dList.getFirstNode();
    ListNode currentDEList = this.deList.getFirstNode();

    ListNode currentTBList = this.tbList.getFirstNode();
    ListNode currentTList = this.tList.getFirstNode();
    ListNode currentTEList = this.teList.getFirstNode();

    String data = "";

    while (!this.fList.isEmpty())
    {
      this.fList.removeFromBack();
    }

    while (!this.dList.isEmpty())
    {
      this.dList.removeFromBack();
    }

    while (!this.tList.isEmpty())
    {
      this.tList.removeFromBack();
    }

    while (currentBList != null)
    {
      this.fList.insertAtBack(currentBList.data.toString());
      currentBList = currentBList.next;
    }
    String line3 = "";
    String line4 = "";
    String line5 = "";
    String line6 = "";
    String line7 = "";

    if (!pkcheck1.isSelected())
    {
  	  pksam = "null";
  	  pkstd = "null";
  	  pkm_text_1.setText("0");
  	  pkm_text_ul.setText("0");
  	  pkmrddeck = "null";
  	  pkdillab2 = "null";
  	  
  	  if (!pkcheck2.isSelected())
      {
  	  	  pkmrddil = "null";
  	  	  
  	  	  pkdildil = "null";
    	  pkdildeck = "null";
      	  pkdillab3 = "null";
      	  
      }
    }

    if (!pkcheck2.isSelected())
    {
  	  pkdildil = "null";
	  pkdildeck = "null";
  	  pkdillab3 = "null";
    }

    line3 = this.pkm_text_wat.getText().trim() + "," + this.pkm_text_1.getText().trim()
	+ ","  + this.pkm_text_ul.getText().trim() + ","  + this.pkm_text_ul_preAssay.getText().trim() + ","  
    + this.pkm_text_ul_Assay.getText().trim();

    line4 = this.pksam.trim() + "," + this.pkstd.trim()
    + "," + this.pkmrddil.trim() + "," + this.pkdildil.trim();
    		
    if (pkcheck1.isSelected())
    {
      pkck1 = "1";
    }
    else
    {
      pkck1 = "0";
      if (!pkcheck2.isSelected())
      {
    	  pkmrdlab = "null";
    	  TabWindow.this.pkMRDsource.setEnabled(false);
    	  TabWindow.this.pkMRDdil.setEnabled(false);
      }
      pksamlab = "null";
	  TabWindow.this.pksamdil.setEnabled(false);
	  TabWindow.this.pkstddil.setEnabled(false);
	  TabWindow.this.pkMRDdil.setEnabled(false);
	  TabWindow.this.pkSsource.setEnabled(false);
    }

    if (pkcheck2.isSelected())
    {
      pkck2 = "1";
    }
    else
    {
      pkck2 = "0";
      pkdillab = "null";
	  TabWindow.this.pkDsource.setEnabled(false);
      if (!pkcheck1.isSelected())
      {
    	  pkmrdlab = "null";
    	  TabWindow.this.pkMRDsource.setEnabled(false);
    	  TabWindow.this.pkMRDdil.setEnabled(false);
      }
    }
    if (pkcheck3.isSelected())
    {
      pkck3 = "1";
    }
    else
    {
      pkck3 = "0";
      pkpasslab = "null";
	  TabWindow.this.pkPassay.setEnabled(false);
    }
    
    if (pkcheck4.isSelected())
    {
      pkck4 = "1";
    }
    else
    {
      pkck4 = "0";
      pkasslab = "null";
	  TabWindow.this.pkPassay.setEnabled(false);
//	  TabWindow.this.text12.setEnabled(false);
    }
    
    line5 = this.pksamlab.trim() + "," + this.pkstdlab.trim() + "," + this.pkmrdlab.trim() 
    + "," + this.pkdillab.trim() + ","  + this.pkpasslab.trim()+ "," + this.pkasslab.trim();

    line6 = pkck1 +  ","  + pkck2 +  "," + pkck3 +  ","  + pkck4 ;

    line7 = pkdillab2 + "," + pkmrddeck +  "," + pkdillab3 + "," + pkdildeck;
    

    data = "\n\nPK SAMPLE PREP";

    this.fList.insertAtBack("step");
    this.fList.insertAtBack("PK");
    this.fList.insertAtBack(line3);
    this.fList.insertAtBack(line4);
    this.fList.insertAtBack(line5);
    this.fList.insertAtBack(line6);
    this.fList.insertAtBack(line7);
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");

    updText(data);
    int element;
//    if (this.stepnbr < 16) element = this.stepnbr - 1; else element = this.stepnbr - 16;		//APF0020 
    if ((this.stepnbr % 15) == 0) {element = 14;}  else{element = ((this.stepnbr % 15)-1);}		//APF0027
    
    this.dTextArea[element].setCaretPosition(0);
    this.dTextArea[element].insertIcon(this.icons[23]);

    while (currentEList != null)
    {
      this.fList.insertAtBack(currentEList.data.toString());
      currentEList = currentEList.next;
    }

    while (currentDBList != null)
    {
      this.dList.insertAtBack(currentDBList.data.toString());
      currentDBList = currentDBList.next;
    }

    this.dList.insertAtBack(data);

    while (currentDEList != null)
    {
      this.dList.insertAtBack(currentDEList.data.toString());
      currentDEList = currentDEList.next;
    }

    while (currentTBList != null)
    {
      this.tList.insertAtBack(currentTBList.data.toString());
      currentTBList = currentTBList.next;
    }

    this.tList.insertAtBack("CHR");

    while (currentTEList != null)
    {
      this.tList.insertAtBack(currentTEList.data.toString());
      currentTEList = currentTEList.next;
    }

    while (!this.bList.isEmpty())
    {
      this.bList.removeFromBack();
    }
    while (!this.eList.isEmpty())
    {
      this.eList.removeFromBack();
    }
    while (!this.dbList.isEmpty())
    {
      this.dbList.removeFromBack();
    }
    while (!this.deList.isEmpty())
    {
      this.deList.removeFromBack();
    }
    while (!this.tbList.isEmpty())
    {
      this.tbList.removeFromBack();
    }
    while (!this.teList.isEmpty())
    {
      this.teList.removeFromBack();
    }

    this.btnUpdateStep.setEnabled(false);
    this.checkButton.setEnabled(false);

    this.update = false;
    this.addButton.setEnabled(true);
    this.btnAdd.setEnabled(true);
//    this.addButton12.setEnabled(true);
    this.rmvButton.setEnabled(false);
    this.saveButton.setEnabled(true);
    this.pkstddil.setSelectedIndex(0);
    this.pksamdil.setSelectedIndex(0);
    this.pkMRDdil.setSelectedIndex(0);
    this.pkdil.setSelectedIndex(0);
    this.pkstddil.setEnabled(false);
    this.pksamdil.setEnabled(false);
    this.pkMRDdil.setEnabled(false);
    this.pkdil.setEnabled(false);
	this.pkDsource.setEnabled(false);
    this.pkm_text_wat.setText("");
    this.pkm_text_wat.setEditable(true);
    this.pkm_text_1.setText("");
    this.pkm_text_1.setEditable(true);
    this.pkm_text_ul.setText("");
    this.pkm_text_ul.setEditable(true);
    this.pkassay.setSelectedIndex(0);
    this.pkassay.setEnabled(false);
    this.pkPassay.setSelectedIndex(0);
    this.pkPassay.setEnabled(false);
    this.pkMRDsource.setSelectedIndex(0);
    this.pkMRDsource.setEnabled(false);
    this.pkSsource.setSelectedIndex(0);
    this.pkSsource.setEnabled(false);
    this.pkDsource.setSelectedIndex(0);
    this.pkDsource.setEnabled(false);
    this.pkSTDsource.setEnabled(false);
    this.pkSTDsource.setSelectedIndex(0);
    this.pkdeck1.setEnabled(false);
//    this.pkdeck1.setVisible(false);
    this.pkdeck1.setSelectedIndex(0);
    this.pkdeck2.setEnabled(false);
//    this.pkdeck2.setVisible(false);
    this.pkdeck2.setSelectedIndex(0);
	this.pkDilsource.setEnabled(false);
	this.pkDilreserve.setEnabled(false);
	this.lblDiluent2.setVisible(false);
    this.pkcheck1.setSelected(false);
    this.pkcheck2.setSelected(false);
    this.pkcheck3.setSelected(false);
    this.pkcheck4.setSelected(false);

    for (int t = 0; t < this.tabcount; t++)
    {
        this.jTabbedPane1.setEnabledAt(t, true);
    }
    pkEnableDisable();
    
    //Return to Splash screen
    Splash();
  }

  private void updPause()
  {
    ListNode currentBList = this.bList.getFirstNode();
    ListNode currentFList = this.fList.getFirstNode();
    ListNode currentEList = this.eList.getFirstNode();

    ListNode currentDBList = this.dbList.getFirstNode();
    ListNode currentDList = this.dList.getFirstNode();
    ListNode currentDEList = this.deList.getFirstNode();

    ListNode currentTBList = this.tbList.getFirstNode();
    ListNode currentTList = this.tList.getFirstNode();
    ListNode currentTEList = this.teList.getFirstNode();

    String line3 = "";
    String line4 = "";
    String data = "";
    String tchk = "";

    while (!this.fList.isEmpty())
    {
      this.fList.removeFromBack();
    }

    while (!this.dList.isEmpty())
    {
      this.dList.removeFromBack();
    }

    while (!this.tList.isEmpty())
    {
      this.tList.removeFromBack();
    }

    while (currentBList != null)
    {
      this.fList.insertAtBack(currentBList.data.toString());
      currentBList = currentBList.next;
    }

    data = data + "\n\n PAUSE";

    if (pausedataPanel2.isTimed())
    {
    	tchk = "1";
    }
    else
    {
    	tchk = "0";
    }
    
    line3 = tchk + "," + pminutes + "," + 
    		pseconds;
    
    line4 = pausedataPanel2.getMessage().trim();

    this.fList.insertAtBack("step");
    this.fList.insertAtBack("Pause");
    this.fList.insertAtBack(line3);
    this.fList.insertAtBack(line4);
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");

    updText(data);
    int element;
//    if (this.stepnbr < 16) element = this.stepnbr - 1; else element = this.stepnbr - 16;		//APF0020 
    if ((this.stepnbr % 15) == 0) {element = 14;}  else{element = ((this.stepnbr % 15)-1);}		//APF0027
    
    this.dTextArea[element].setCaretPosition(0);
    this.dTextArea[element].insertIcon(this.icons[20]);
    
    while (currentEList != null)
    {
      this.fList.insertAtBack(currentEList.data.toString());
      currentEList = currentEList.next;
    }

    while (currentDBList != null)
    {
      this.dList.insertAtBack(currentDBList.data.toString());
      currentDBList = currentDBList.next;
    }

    this.dList.insertAtBack(data);

    while (currentDEList != null)
    {
      this.dList.insertAtBack(currentDEList.data.toString());
      currentDEList = currentDEList.next;
    }

    while (currentTBList != null)
    {
      this.tList.insertAtBack(currentTBList.data.toString());
      currentTBList = currentTBList.next;
    }

    this.tList.insertAtBack("PAU");

    while (currentTEList != null)
    {
      this.tList.insertAtBack(currentTEList.data.toString());
      currentTEList = currentTEList.next;
    }

    while (!this.bList.isEmpty())
    {
      this.bList.removeFromBack();
    }
    while (!this.eList.isEmpty())
    {
      this.eList.removeFromBack();
    }
    while (!this.dbList.isEmpty())
    {
      this.dbList.removeFromBack();
    }
    while (!this.deList.isEmpty())
    {
      this.deList.removeFromBack();
    }
    while (!this.tbList.isEmpty())
    {
      this.tbList.removeFromBack();
    }
    while (!this.teList.isEmpty())
    {
      this.teList.removeFromBack();
    }

    this.updButton9.setEnabled(false);

    this.update = false;
    this.addButton9.setEnabled(true);
    this.rmvButton.setEnabled(false);
    this.pausedataPanel2.setMinutes(" ");
    this.pausedataPanel2.setSeconds(" ");
    this.pausedataPanel2.setMessage(" ");
    this.pausedataPanel2.setTimed(false);

    for (int t = 0; t < this.tabcount; t++)
    {
        this.jTabbedPane1.setEnabledAt(t, true);
    }
    pkEnableDisable();
    
    //Return to Splash screen
    Splash();
  }

  private void updBrew()
  {
    ListNode currentBList = this.bList.getFirstNode();
    ListNode currentFList = this.fList.getFirstNode();
    ListNode currentEList = this.eList.getFirstNode();

    ListNode currentDBList = this.dbList.getFirstNode();
    ListNode currentDList = this.dList.getFirstNode();
    ListNode currentDEList = this.deList.getFirstNode();

    ListNode currentTBList = this.tbList.getFirstNode();
    ListNode currentTList = this.tList.getFirstNode();
    ListNode currentTEList = this.teList.getFirstNode();

    String m1 = "0";
    String m2 = "0";
    String c1 = "0";
    String i1 = "0";
    String i2 = "0";
    String tube1 = "";
    String tube2 = "";
    String tube3 = "";
    String tube4 = "";
    String data = "";

    String line3 = "";
    String line4 = "";
    String line5 = "";
    String line6 = "";
    String sourceseq = " ";
    String destseq = "0";

    data = data + "\n BREW\n ";
    data = data + "\n " + this.brntext.getText().trim() + "\n";

    if (this.brtubepanel.incIsSelected())
    {
      i1 = "1";
    }
    if (this.brtubepanel2.incIsSelected())
    {
      i2 = "1";
    }

    if (this.brchill.isSelected())
    {
      c1 = "1";
    }

    if (this.brmix.isSelected())
    {
      m1 = "1";
    }

    if (this.brmix2.isSelected())
    {
      m2 = "1";
    }

    tube1 = this.brtubepanel.getFirstTube();
    tube2 = this.brtubepanel.getLastTube();
    tube3 = this.brtubepanel2.getFirstTube();
    tube4 = this.brtubepanel2.getLastTube();

    if (this.so4.trim().endsWith("Tube"))
    {
      sourceseq = tube1 + "," + tube2 + "," + i1;
      this.hline9 = this.so4.trim();
      this.bpl1 = "Tube";
    }
    if (this.so4.trim().endsWith("_1X"))
    {
      sourceseq = "1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,52,53,54,55,56,57,58,59,60,61,62,63,64,65,66,67,68,69,70,71,72,73,74,75,76,77,78,79,80,81,82,83,84,85,86,87,88,89,90,91,92,93,94,95,96";

      this.bpl1 = "Reservoir";
    }
    if (this.so4.trim().endsWith("_2X"))
    {
      sourceseq = "1,2,3,4,5,6,7,8";
      this.bpl1 = "Reservoir";
    }
    if (this.so4.trim().endsWith("_4X"))
    {
      sourceseq = "1,2,3,4,5,6,7,8";
      this.bpl1 = "Reservoir";
    }
    if (this.so4.trim().endsWith("_5X"))
    {
      sourceseq = "1,2,3,4,5,6,7,8";
      this.bpl1 = "Reservoir";
    }
    if (this.so4.trim().endsWith("_24X"))
    {
      sourceseq = "1,2,3,4,5,6,7,8";
      this.bpl1 = "Reservoir";
    }

    if (this.de3.trim().endsWith("_1X"))
    {
      destseq = "1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,52,53,54,55,56,57,58,59,60,61,62,63,64,65,66,67,68,69,70,71,72,73,74,75,76,77,78,79,80,81,82,83,84,85,86,87,88,89,90,91,92,93,94,95,96";

      this.bpl2 = "Reservoir";
    }
    if (this.de3.trim().endsWith("_2X"))
    {
      destseq = "1,2,3,4,5,6,7,8";
      this.bpl2 = "Reservoir";
    }
    if (this.de3.trim().endsWith("_4X"))
    {
      destseq = "1,2,3,4,5,6,7,8";
      this.bpl2 = "Reservoir";
    }
    if (this.de3.trim().endsWith("_5X"))
    {
      destseq = "1,2,3,4,5,6,7,8";
      this.bpl2 = "Reservoir";
    }
    if (this.de3.trim().endsWith("_24X"))
    {
      destseq = "1,2,3,4,5,6,7,8";
      this.bpl2 = "Reservoir";
    }
    if (this.de3.trim().endsWith("Tube"))
    {
      destseq = tube3 + "," + tube4 + "," + i2;
      this.hline9 = this.de3.trim();
      this.bpl2 = "Tube";
    }

    line3 = line3 + this.brtext.getText().trim();
    line3 = line3 + "," + this.blt1.trim() + "," + this.brntext.getText().trim() + "," + this.so4.trim() + ",-";
    line4 = line4 + this.bpl1.trim() + "," + this.bpl2.trim();
    line5 = line5 + this.so4.trim() + "," + m1 + "," + c1 + ",-";
    line6 = line6 + this.de3.trim() + "," + m2 + ",-,-,-,-";

    while (!this.fList.isEmpty())
    {
      this.fList.removeFromBack();
    }

    while (!this.dList.isEmpty())
    {
      this.dList.removeFromBack();
    }

    while (!this.tList.isEmpty())
    {
      this.tList.removeFromBack();
    }

    while (currentBList != null)
    {
      this.fList.insertAtBack(currentBList.data.toString());
      currentBList = currentBList.next;
    }

    updText(data);

    this.fList.insertAtBack("step");
    this.fList.insertAtBack("brew");
    this.fList.insertAtBack(line3);
    this.fList.insertAtBack(line4);
    this.fList.insertAtBack(line5);
    this.fList.insertAtBack(line6);
    this.fList.insertAtBack(sourceseq);
    this.fList.insertAtBack(destseq);
    this.fList.insertAtBack(this.brvol.getText().trim());
    this.fList.insertAtBack("-");

    while (currentEList != null)
    {
      this.fList.insertAtBack(currentEList.data.toString());
      currentEList = currentEList.next;
    }

    while (currentDBList != null)
    {
      this.dList.insertAtBack(currentDBList.data.toString());
      currentDBList = currentDBList.next;
    }

    this.dList.insertAtBack(data);

    while (currentDEList != null)
    {
      this.dList.insertAtBack(currentDEList.data.toString());
      currentDEList = currentDEList.next;
    }

    while (currentTBList != null)
    {
      this.tList.insertAtBack(currentTBList.data.toString());
      currentTBList = currentTBList.next;
    }

    this.tList.insertAtBack("BRW");

    while (currentTEList != null)
    {
      this.tList.insertAtBack(currentTEList.data.toString());
      currentTEList = currentTEList.next;
    }

    while (!this.bList.isEmpty())
    {
      this.bList.removeFromBack();
    }
    while (!this.eList.isEmpty())
    {
      this.eList.removeFromBack();
    }
    while (!this.dbList.isEmpty())
    {
      this.dbList.removeFromBack();
    }
    while (!this.deList.isEmpty())
    {
      this.deList.removeFromBack();
    }
    while (!this.tbList.isEmpty())
    {
      this.tbList.removeFromBack();
    }
    while (!this.teList.isEmpty())
    {
      this.teList.removeFromBack();
    }

    this.brvol.setText("");
    this.brtext.setText("");
    this.brntext.setText("");
    this.bpl1 = "";
    this.bpl2 = "";
    this.brmix.setSelected(false);
    this.brchill.setSelected(false);
    this.source4.setSelectedIndex(0);
    this.dest3.setSelectedIndex(0);
    this.brresbutton1.setSelected(true);
    this.brresbutton2.setSelected(true);
    this.brewPanel1.removeAll();
    getSrc4();
    this.source4.revalidate();
    this.source4.repaint();
    this.brewPanel1.add(getBrewPanel1Top(), "North");
    this.brewPanel1.add(this.brrespanel1, "Center");
    this.brewPanel1.add(getBrewPanel1Bot(), "South");
    this.brewPanel1.revalidate();
    this.brewPanel1.repaint();
    this.brewPanel2.removeAll();
    getDest3();
    this.dest3.revalidate();
    this.dest3.repaint();
    this.brewPanel2.add(getBrewPanel2Top(), "North");
    this.brewPanel2.add(this.brrespanel2, "Center");
    this.brewPanel2.add(getBrewPanel2Bot(), "South");
    this.brewPanel2.revalidate();
    this.brewPanel2.repaint();

    this.addButton7.setEnabled(true);
    this.updButton7.setEnabled(false);
    this.rmvButton.setEnabled(false);
    this.update = false;
    for (int t = 0; t < this.tabcount; t++)
    {
        this.jTabbedPane1.setEnabledAt(t, true);
    }
    pkEnableDisable();
    
//    this.dTextArea[0].setBorder(this.blackline);
//    this.dTextArea[1].setBorder(this.blackline);
//    this.dTextArea[2].setBorder(this.blackline);
//    this.dTextArea[3].setBorder(this.blackline);
//    this.dTextArea[4].setBorder(this.blackline);
//    this.dTextArea[5].setBorder(this.blackline);
//    this.dTextArea[6].setBorder(this.blackline);
//    this.dTextArea[7].setBorder(this.blackline);
//    this.dTextArea[8].setBorder(this.blackline);
//    this.dTextArea[9].setBorder(this.blackline);
//    this.dTextArea[10].setBorder(this.blackline);
//    this.dTextArea[11].setBorder(this.blackline);
//    this.dTextArea[12].setBorder(this.blackline);
//    this.dTextArea[13].setBorder(this.blackline);
//    this.dTextArea[14].setBorder(this.blackline);
    
    //Return to Splash screen
    Splash();
  }

  private void updRead()
  {
    ListNode currentBList = this.bList.getFirstNode();
    ListNode currentFList = this.fList.getFirstNode();
    ListNode currentEList = this.eList.getFirstNode();

    ListNode currentDBList = this.dbList.getFirstNode();
    ListNode currentDList = this.dList.getFirstNode();
    ListNode currentDEList = this.deList.getFirstNode();

    ListNode currentTBList = this.tbList.getFirstNode();
    ListNode currentTList = this.tList.getFirstNode();
    ListNode currentTEList = this.teList.getFirstNode();

    String line4 = "";
    String data = "";
    String t1 = "";

    while (!this.fList.isEmpty())
    {
      this.fList.removeFromBack();
    }

    while (!this.dList.isEmpty())
    {
      this.dList.removeFromBack();
    }

    while (!this.tList.isEmpty())
    {
      this.tList.removeFromBack();
    }

    data = data + "   \n";
    data = data + "\n " + this.r1.trim();
    data = data + "\n " + this.text6.getText().trim();
    data = data + "\n " + this.temp.getText().trim() + " (c)";

    String type = "RDR";
    t1 = this.temp.getText().trim();
    if (t1.equals(""))
    {
      t1 = "0";
    }

    line4 = this.text6.getText().trim();
    if (line4.trim().equals(""))
    {
      line4 = "-";
    }

    while (currentBList != null)
    {
      this.fList.insertAtBack(currentBList.data.toString());
      currentBList = currentBList.next;
    }

    updText(data);

    this.fList.insertAtBack("step");
    this.fList.insertAtBack("read");
    this.fList.insertAtBack(this.r1.trim() + "," + t1);
    this.fList.insertAtBack(line4);

    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");

    while (currentEList != null)
    {
      this.fList.insertAtBack(currentEList.data.toString());
      currentEList = currentEList.next;
    }

    while (currentDBList != null)
    {
      this.dList.insertAtBack(currentDBList.data.toString());
      currentDBList = currentDBList.next;
    }

    this.dList.insertAtBack(data);

    while (currentDEList != null)
    {
      this.dList.insertAtBack(currentDEList.data.toString());
      currentDEList = currentDEList.next;
    }

    while (currentTBList != null)
    {
      this.tList.insertAtBack(currentTBList.data.toString());
      currentTBList = currentTBList.next;
    }

    this.tList.insertAtBack("RDR");

    while (currentTEList != null)
    {
      this.tList.insertAtBack(currentTEList.data.toString());
      currentTEList = currentTEList.next;
    }

    while (!this.bList.isEmpty())
    {
      this.bList.removeFromBack();
    }
    while (!this.eList.isEmpty())
    {
      this.eList.removeFromBack();
    }
    while (!this.dbList.isEmpty())
    {
      this.dbList.removeFromBack();
    }
    while (!this.deList.isEmpty())
    {
      this.deList.removeFromBack();
    }
    while (!this.tbList.isEmpty())
    {
      this.tbList.removeFromBack();
    }
    while (!this.teList.isEmpty())
    {
      this.teList.removeFromBack();
    }

    this.updButton4.setEnabled(false);
    this.rmvButton.setEnabled(false);
    this.update = false;
    for (int t = 0; t < this.tabcount; t++)
    {
        this.jTabbedPane1.setEnabledAt(t, true);
    }
    pkEnableDisable();
    
//    this.dTextArea[0].setBorder(this.blackline);
//    this.dTextArea[1].setBorder(this.blackline);
//    this.dTextArea[2].setBorder(this.blackline);
//    this.dTextArea[3].setBorder(this.blackline);
//    this.dTextArea[4].setBorder(this.blackline);
//    this.dTextArea[5].setBorder(this.blackline);
//    this.dTextArea[6].setBorder(this.blackline);
//    this.dTextArea[7].setBorder(this.blackline);
//    this.dTextArea[8].setBorder(this.blackline);
//    this.dTextArea[9].setBorder(this.blackline);
//    this.dTextArea[10].setBorder(this.blackline);
//    this.dTextArea[11].setBorder(this.blackline);
//    this.dTextArea[12].setBorder(this.blackline);
//    this.dTextArea[13].setBorder(this.blackline);
//    this.dTextArea[14].setBorder(this.blackline);

    //Return to Splash screen
    Splash();

  }

  private void updSPE()
  {
    ListNode currentBList = this.bList.getFirstNode();
    ListNode currentFList = this.fList.getFirstNode();
    ListNode currentEList = this.eList.getFirstNode();

    ListNode currentDBList = this.dbList.getFirstNode();
    ListNode currentDList = this.dList.getFirstNode();
    ListNode currentDEList = this.deList.getFirstNode();

    ListNode currentTBList = this.tbList.getFirstNode();
    ListNode currentTList = this.tList.getFirstNode();
    ListNode currentTEList = this.teList.getFirstNode();

    String b1 = "0";
    String line3 = "";
    String line4 = "";
    String line5 = "";
    String data = "";

    while (!this.fList.isEmpty())
    {
      this.fList.removeFromBack();
    }

    while (!this.dList.isEmpty())
    {
      this.dList.removeFromBack();
    }

    while (!this.tList.isEmpty())
    {
      this.tList.removeFromBack();
    }

    while (currentBList != null)
    {
      this.fList.insertAtBack(currentBList.data.toString());
      currentBList = currentBList.next;
    }

    this.ti = this.time.getText().trim();
    this.pre = this.pressure.getText().trim();
    this.fil = this.fil.trim();
    this.col = this.col.trim();

    data = data + "\n\n SPE ";

    if (this.assembutton.isSelected())
    {
      b1 = "1";
      this.col = "-";
      this.fil = "-";
      this.ti = "-";
      this.pre = "-";
      data = data + "\n Assemble";
    }
    if (this.disassembutton.isSelected())
    {
      b1 = "2";
      this.col = "-";
      this.fil = "-";
      this.ti = "-";
      this.pre = "-";
      data = data + "\n Disassemble";
    }
    if (this.vacbutton.isSelected())
    {
      b1 = "3";
      this.col = "-";
      this.fil = "-";
      data = data + "\n Apply Vac";
    }

    updText(data);
    int element;
//    if (this.stepnbr < 16) element = this.stepnbr - 1; else element = this.stepnbr - 16;		//APF0020 
    if ((this.stepnbr % 15) == 0) {element = 14;}  else{element = ((this.stepnbr % 15)-1);}		//APF0027
    
    this.dTextArea[element].setCaretPosition(0);
    this.dTextArea[element].insertIcon(this.icons[28]);

    this.fList.insertAtBack("step");
    this.fList.insertAtBack("SPE");
    this.fList.insertAtBack(b1 + "," + this.fil + "," + this.col + "," + this.ti + "," + this.pre);
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");

    while (currentEList != null)
    {
      this.fList.insertAtBack(currentEList.data.toString());
      currentEList = currentEList.next;
    }

    while (currentDBList != null)
    {
      this.dList.insertAtBack(currentDBList.data.toString());
      currentDBList = currentDBList.next;
    }

    this.dList.insertAtBack(data);

    while (currentDEList != null)
    {
      this.dList.insertAtBack(currentDEList.data.toString());
      currentDEList = currentDEList.next;
    }

    while (currentTBList != null)
    {
      this.tList.insertAtBack(currentTBList.data.toString());
      currentTBList = currentTBList.next;
    }

    this.tList.insertAtBack("SPE");

    while (currentTEList != null)
    {
      this.tList.insertAtBack(currentTEList.data.toString());
      currentTEList = currentTEList.next;
    }

    while (!this.bList.isEmpty())
    {
      this.bList.removeFromBack();
    }
    while (!this.eList.isEmpty())
    {
      this.eList.removeFromBack();
    }
    while (!this.dbList.isEmpty())
    {
      this.dbList.removeFromBack();
    }
    while (!this.deList.isEmpty())
    {
      this.deList.removeFromBack();
    }
    while (!this.tbList.isEmpty())
    {
      this.tbList.removeFromBack();
    }
    while (!this.teList.isEmpty())
    {
      this.teList.removeFromBack();
    }

    this.time.setText("");
    this.pressure.setText("");
    this.collection.setSelectedIndex(0);
    this.filter.setSelectedIndex(0);
    
    this.time.setVisible(false);
    this.pressure.setVisible(false);
    this.spelabel3.setVisible(false);
    this.spelabel4.setVisible(false);

    this.updButton5.setEnabled(false);
    this.rmvButton.setEnabled(false);
    this.addButton5.setEnabled(true);
    this.update = false;
    for (int t = 0; t < this.tabcount; t++)
    {
        this.jTabbedPane1.setEnabledAt(t, true);
    }
    pkEnableDisable();
    
//    this.dTextArea[0].setBorder(this.blackline);
//    this.dTextArea[1].setBorder(this.blackline);
//    this.dTextArea[2].setBorder(this.blackline);
//    this.dTextArea[3].setBorder(this.blackline);
//    this.dTextArea[4].setBorder(this.blackline);
//    this.dTextArea[5].setBorder(this.blackline);
//    this.dTextArea[6].setBorder(this.blackline);
//    this.dTextArea[7].setBorder(this.blackline);
//    this.dTextArea[8].setBorder(this.blackline);
//    this.dTextArea[9].setBorder(this.blackline);
//    this.dTextArea[10].setBorder(this.blackline);
//    this.dTextArea[11].setBorder(this.blackline);
//    this.dTextArea[12].setBorder(this.blackline);
//    this.dTextArea[13].setBorder(this.blackline);
//    this.dTextArea[14].setBorder(this.blackline);

    //Return to Splash screen
    Splash();
  }

  private void updCytomat()
  {
    ListNode currentBList = this.bList.getFirstNode();
    ListNode currentFList = this.fList.getFirstNode();
    ListNode currentEList = this.eList.getFirstNode();

    ListNode currentDBList = this.dbList.getFirstNode();
    ListNode currentDList = this.dList.getFirstNode();
    ListNode currentDEList = this.deList.getFirstNode();

    ListNode currentTBList = this.tbList.getFirstNode();
    ListNode currentTList = this.tList.getFirstNode();
    ListNode currentTEList = this.teList.getFirstNode();

    String data = "";

    while (!this.fList.isEmpty())
    {
      this.fList.removeFromBack();
    }

    while (!this.dList.isEmpty())
    {
      this.dList.removeFromBack();
    }

    while (!this.tList.isEmpty())
    {
      this.tList.removeFromBack();
    }

    while (currentBList != null)
    {
      this.fList.insertAtBack(currentBList.data.toString());
      currentBList = currentBList.next;
    }

    data = data + "   \n\n";
    data = data + "Cytomat " + this.cytloc + " to " + this.cytdeckloc + "\n\n";
    int line3 = cyt_fromto;
    int line6 = cyt_cover;
    this.fList.insertAtBack("step");
    this.fList.insertAtBack("Cytomat01");
    this.fList.insertAtBack(line3);
    this.fList.insertAtBack(this.cytloc);
    this.fList.insertAtBack(this.cytdeckloc);
    this.fList.insertAtBack(line6);
    this.fList.insertAtBack(this.cyt_plate);
    this.fList.insertAtBack(this.cyt_tsp);
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");


    updText(data);
    int element;
//    if (this.stepnbr < 16) element = this.stepnbr - 1; else element = this.stepnbr - 16;		//APF0020 
    if ((this.stepnbr % 15) == 0) {element = 14;}  else{element = ((this.stepnbr % 15)-1);}		//APF0027
    
    this.dTextArea[element].setCaretPosition(0);
    this.dTextArea[element].insertIcon(this.icons[29]);	//updCytomat()

    while (currentEList != null)
    {
      this.fList.insertAtBack(currentEList.data.toString());
      currentEList = currentEList.next;
    }

    while (currentDBList != null)
    {
      this.dList.insertAtBack(currentDBList.data.toString());
      currentDBList = currentDBList.next;
    }

    this.dList.insertAtBack(data);

    while (currentDEList != null)
    {
      this.dList.insertAtBack(currentDEList.data.toString());
      currentDEList = currentDEList.next;
    }

    while (currentTBList != null)
    {
      this.tList.insertAtBack(currentTBList.data.toString());
      currentTBList = currentTBList.next;
    }

    this.tList.insertAtBack("CYT");

    while (currentTEList != null)
    {
      this.tList.insertAtBack(currentTEList.data.toString());
      currentTEList = currentTEList.next;
    }

    while (!this.bList.isEmpty())
    {
      this.bList.removeFromBack();
    }
    while (!this.eList.isEmpty())
    {
      this.eList.removeFromBack();
    }
    while (!this.dbList.isEmpty())
    {
      this.dbList.removeFromBack();
    }
    while (!this.deList.isEmpty())
    {
      this.deList.removeFromBack();
    }
    while (!this.tbList.isEmpty())
    {
      this.tbList.removeFromBack();
    }
    while (!this.teList.isEmpty())
    {
      this.teList.removeFromBack();
    }

    this.cyt_btnUpdateStep.setEnabled(false);
    this.checkButton.setEnabled(false);

    this.update = false;
    this.addButton.setEnabled(true);
    this.cyt_btnAddStep.setEnabled(true);
    this.rmvButton.setEnabled(false);
    this.saveButton.setEnabled(true);
    this.saveButton.setEnabled(true);
    this.cytodatapanel2.setText1("");
    this.cytodatapanel2.setText2("");
    this.cytodatapanel2.setFTIndex(0);
    this.cytodatapanel2.setCoverIndex(0);
    this.cytodatapanel2.setPlateIndex(0);
    this.cytodatapanel2.setTSPIndex(1);

    for (int t = 0; t < this.tabcount; t++)
    {
        this.jTabbedPane1.setEnabledAt(t, true);
    }
    pkEnableDisable();
   
    //Return to Splash screen
    Splash();
  }

  private void updMovePlate2()
  {
    ListNode currentBList = this.bList.getFirstNode();
    ListNode currentFList = this.fList.getFirstNode();
    ListNode currentEList = this.eList.getFirstNode();

    ListNode currentDBList = this.dbList.getFirstNode();
    ListNode currentDList = this.dList.getFirstNode();
    ListNode currentDEList = this.deList.getFirstNode();

    ListNode currentTBList = this.tbList.getFirstNode();
    ListNode currentTList = this.tList.getFirstNode();
    ListNode currentTEList = this.teList.getFirstNode();

    String data = "";

    while (!this.fList.isEmpty())
    {
      this.fList.removeFromBack();
    }

    while (!this.dList.isEmpty())
    {
      this.dList.removeFromBack();
    }

    while (!this.tList.isEmpty())
    {
      this.tList.removeFromBack();
    }

    while (currentBList != null)
    {
      this.fList.insertAtBack(currentBList.data.toString());
      currentBList = currentBList.next;
    }

    data = data + "   \n\n";
    data = data + "Move Plate from " + this.startloc + " to " + this.endloc + "\n\n";

    int line3 = mvp_type;
    String line6 = mvp_plate;

    this.fList.insertAtBack("step");
    this.fList.insertAtBack("MovePlate02");
    this.fList.insertAtBack(line3);
    this.fList.insertAtBack(this.startloc);
    this.fList.insertAtBack(this.endloc);
    this.fList.insertAtBack(line6);
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");


    updText(data);
    int element;
//    if (this.stepnbr < 16) element = this.stepnbr - 1; else element = this.stepnbr - 16;		//APF0020 
    if ((this.stepnbr % 15) == 0) {element = 14;}  else{element = ((this.stepnbr % 15)-1);}		//APF0027
    
    this.dTextArea[element].setCaretPosition(0);
    this.dTextArea[element].insertIcon(this.icons[8]);	//updMovePlate2() XXX

    while (currentEList != null)
    {
      this.fList.insertAtBack(currentEList.data.toString());
      currentEList = currentEList.next;
    }

    while (currentDBList != null)
    {
      this.dList.insertAtBack(currentDBList.data.toString());
      currentDBList = currentDBList.next;
    }

    this.dList.insertAtBack(data);

    while (currentDEList != null)
    {
      this.dList.insertAtBack(currentDEList.data.toString());
      currentDEList = currentDEList.next;
    }

    while (currentTBList != null)
    {
      this.tList.insertAtBack(currentTBList.data.toString());
      currentTBList = currentTBList.next;
    }

    this.tList.insertAtBack("MVP2");

    while (currentTEList != null)
    {
      this.tList.insertAtBack(currentTEList.data.toString());
      currentTEList = currentTEList.next;
    }

    while (!this.bList.isEmpty())
    {
      this.bList.removeFromBack();
    }
    while (!this.eList.isEmpty())
    {
      this.eList.removeFromBack();
    }
    while (!this.dbList.isEmpty())
    {
      this.dbList.removeFromBack();
    }
    while (!this.deList.isEmpty())
    {
      this.deList.removeFromBack();
    }
    while (!this.tbList.isEmpty())
    {
      this.tbList.removeFromBack();
    }
    while (!this.teList.isEmpty())
    {
      this.teList.removeFromBack();
    }

    this.movplv2_btnUpdateStep.setEnabled(false);
    this.checkButton.setEnabled(false);

    this.update = false;
    this.addButton.setEnabled(true);
    this.movplv2_btnAddStep.setEnabled(true);
    this.rmvButton.setEnabled(false);
    this.saveButton.setEnabled(true);
    this.saveButton.setEnabled(true);
    this.movplv2datapanel2.setText1("");
    this.movplv2datapanel2.setText2("");
    this.movplv2datapanel2.setTypeIndex(0);
    this.movplv2datapanel2.setPlateIndex(0);

    for (int t = 0; t < this.tabcount; t++)
    {
        this.jTabbedPane1.setEnabledAt(t, true);
    }
    pkEnableDisable();
   
    //Return to Splash screen
    Splash();
  }

  private void updDNANorm()
  {
    ListNode currentBList = this.bList.getFirstNode();
    ListNode currentFList = this.fList.getFirstNode();
    ListNode currentEList = this.eList.getFirstNode();

    ListNode currentDBList = this.dbList.getFirstNode();
    ListNode currentDList = this.dList.getFirstNode();
    ListNode currentDEList = this.deList.getFirstNode();

    ListNode currentTBList = this.tbList.getFirstNode();
    ListNode currentTList = this.tList.getFirstNode();
    ListNode currentTEList = this.teList.getFirstNode();

    String data = "";

    while (!this.fList.isEmpty())
    {
      this.fList.removeFromBack();
    }

    while (!this.dList.isEmpty())
    {
      this.dList.removeFromBack();
    }

    while (!this.tList.isEmpty())
    {
      this.tList.removeFromBack();
    }

    while (currentBList != null)
    {
      this.fList.insertAtBack(currentBList.data.toString());
      currentBList = currentBList.next;
    }

    data = data + "   \n\n";
    data = data + "DNA Normalization " + this.dna_con + " and " + this.dna_vol + "\n\n";

    String line3 = dna_qfile;
    int line4 = dna_procindex;
    String line5 = this.dna_con + "," + this.dna_vol;
    String line6 = this.dna_dplate + "," + this.dna_splate + "," + this.dna_nplate + ","  + this.dna_plate;
    String line7 = dna_deck1 + "," + dna_deck2 + "," + dna_deck3 + "," + dna_deck4 ;
    
    this.fList.insertAtBack("step");
    this.fList.insertAtBack("CORDNA01");
    this.fList.insertAtBack(line3);
    this.fList.insertAtBack(line4);
    this.fList.insertAtBack(line5);
    this.fList.insertAtBack(line6);
    this.fList.insertAtBack(line7);
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");

    updText(data);
    int element;
//    if (this.stepnbr < 16) element = this.stepnbr - 1; else element = this.stepnbr - 16;		//APF0020 
    if ((this.stepnbr % 15) == 0) {element = 14;}  else{element = ((this.stepnbr % 15)-1);}		//APF0027
    
    this.dTextArea[element].setCaretPosition(0);
    this.dTextArea[element].insertIcon(this.icons[29]);	//updCytomat()

    while (currentEList != null)
    {
      this.fList.insertAtBack(currentEList.data.toString());
      currentEList = currentEList.next;
    }

    while (currentDBList != null)
    {
      this.dList.insertAtBack(currentDBList.data.toString());
      currentDBList = currentDBList.next;
    }

    this.dList.insertAtBack(data);

    while (currentDEList != null)
    {
      this.dList.insertAtBack(currentDEList.data.toString());
      currentDEList = currentDEList.next;
    }

    while (currentTBList != null)
    {
      this.tList.insertAtBack(currentTBList.data.toString());
      currentTBList = currentTBList.next;
    }

    this.tList.insertAtBack("DNA");

    while (currentTEList != null)
    {
      this.tList.insertAtBack(currentTEList.data.toString());
      currentTEList = currentTEList.next;
    }

    while (!this.bList.isEmpty())
    {
      this.bList.removeFromBack();
    }
    while (!this.eList.isEmpty())
    {
      this.eList.removeFromBack();
    }
    while (!this.dbList.isEmpty())
    {
      this.dbList.removeFromBack();
    }
    while (!this.deList.isEmpty())
    {
      this.deList.removeFromBack();
    }
    while (!this.tbList.isEmpty())
    {
      this.tbList.removeFromBack();
    }
    while (!this.teList.isEmpty())
    {
      this.teList.removeFromBack();
    }

    this.dna_btnUpdateStep.setEnabled(false);
    this.checkButton.setEnabled(false);

    this.update = false;
    this.addButton.setEnabled(true);
    this.dna_btnAddStep.setEnabled(true);
    this.rmvButton.setEnabled(false);
    this.saveButton.setEnabled(true);
    this.saveButton.setEnabled(true);
    this.dnadataPanel2.setText("Click here to select the Q file");
    this.dnadataPanel2.setText1("");
    this.dnadataPanel2.setText2("");
    this.dnadataPanel2.setProcIndex(0);
    this.dnadataPanel2.setSPlateIndex(0);
    this.dnadataPanel2.setNPlateIndex(0);
    this.dnadataPanel2.setDPlateIndex(0);
    this.dnadataPanel2.setPlateIndex(0);
    this.dnadataPanel2.setDeckIndex1(0);
    this.dnadataPanel2.setDeckIndex2(0);
    this.dnadataPanel2.setDeckIndex3(0);
    this.dnadataPanel2.setDeckIndex4(0);

    for (int t = 0; t < this.tabcount; t++)
    {
        this.jTabbedPane1.setEnabledAt(t, true);
    }
    pkEnableDisable();
   
    //Return to Splash screen
    Splash();
  }

  private void updLoad()
  {
    ListNode currentBList = this.bList.getFirstNode();
    ListNode currentFList = this.fList.getFirstNode();
    ListNode currentEList = this.eList.getFirstNode();

    ListNode currentDBList = this.dbList.getFirstNode();
    ListNode currentDList = this.dList.getFirstNode();
    ListNode currentDEList = this.deList.getFirstNode();

    ListNode currentTBList = this.tbList.getFirstNode();
    ListNode currentTList = this.tList.getFirstNode();
    ListNode currentTEList = this.teList.getFirstNode();

    String data = "";

    while (!this.fList.isEmpty())
    {
      this.fList.removeFromBack();
    }

    while (!this.dList.isEmpty())
    {
      this.dList.removeFromBack();
    }

    while (!this.tList.isEmpty())
    {
      this.tList.removeFromBack();
    }

    while (currentBList != null)
    {
      this.fList.insertAtBack(currentBList.data.toString());
      currentBList = currentBList.next;
    }

    data = data + "   \n\n";
    data = data + "Load" + "\n\n";

//    String line3 = load_plindex + "," + this.load_plate ;
//    String line4 = load_tuindex + "," + this.load_tube ;
//    String line5 = load_tiindex + "," + this.load_tip ;
    String line3 = load_plateindex + "," + this.load_plate ;
    String line4 = load_tubeindex + "," + this.load_tube ;
    String line5 = load_tipindex + "," + this.load_tip ;
    
    
    
    this.fList.insertAtBack("step");
    this.fList.insertAtBack("LOAD01");
    this.fList.insertAtBack(line3);
    this.fList.insertAtBack(line4);
    this.fList.insertAtBack(line5);
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");

    updText(data);
    int element;
//    if (this.stepnbr < 16) element = this.stepnbr - 1; else element = this.stepnbr - 16;		//APF0020 
    if ((this.stepnbr % 15) == 0) {element = 14;}  else{element = ((this.stepnbr % 15)-1);}		//APF0027
    
    this.dTextArea[element].setCaretPosition(0);
    this.dTextArea[element].insertIcon(this.icons[32]);	//updLoad

    while (currentEList != null)
    {
      this.fList.insertAtBack(currentEList.data.toString());
      currentEList = currentEList.next;
    }

    while (currentDBList != null)
    {
      this.dList.insertAtBack(currentDBList.data.toString());
      currentDBList = currentDBList.next;
    }

    this.dList.insertAtBack(data);

    while (currentDEList != null)
    {
      this.dList.insertAtBack(currentDEList.data.toString());
      currentDEList = currentDEList.next;
    }

    while (currentTBList != null)
    {
      this.tList.insertAtBack(currentTBList.data.toString());
      currentTBList = currentTBList.next;
    }

    this.tList.insertAtBack("LOAD01");

    while (currentTEList != null)
    {
      this.tList.insertAtBack(currentTEList.data.toString());
      currentTEList = currentTEList.next;
    }

    while (!this.bList.isEmpty())
    {
      this.bList.removeFromBack();
    }
    while (!this.eList.isEmpty())
    {
      this.eList.removeFromBack();
    }
    while (!this.dbList.isEmpty())
    {
      this.dbList.removeFromBack();
    }
    while (!this.deList.isEmpty())
    {
      this.deList.removeFromBack();
    }
    while (!this.tbList.isEmpty())
    {
      this.tbList.removeFromBack();
    }
    while (!this.teList.isEmpty())
    {
      this.teList.removeFromBack();
    }

    this.load_btnUpdateStep.setEnabled(false);
    this.checkButton.setEnabled(false);

    this.update = false;
    this.addButton.setEnabled(true);
    this.load_btnAddStep.setEnabled(true);
    this.rmvButton.setEnabled(false);
    this.saveButton.setEnabled(true);
    this.saveButton.setEnabled(true);
    this.loaddataPanel2.setPlateText("");
    this.loaddataPanel2.setTubeText("");
    this.loaddataPanel2.setTipText("");
    this.loaddataPanel2.setPlateIndex(0);
    this.loaddataPanel2.setTubeIndex(0);
    this.loaddataPanel2.setTipIndex(0);

    for (int t = 0; t < this.tabcount; t++)
    {
        this.jTabbedPane1.setEnabledAt(t, true);
    }
    pkEnableDisable();
   
    //Return to Splash screen
    Splash();
  }

  private void updLiconic()
  {
    ListNode currentBList = this.bList.getFirstNode();
    ListNode currentFList = this.fList.getFirstNode();
    ListNode currentEList = this.eList.getFirstNode();

    ListNode currentDBList = this.dbList.getFirstNode();
    ListNode currentDList = this.dList.getFirstNode();
    ListNode currentDEList = this.deList.getFirstNode();

    ListNode currentTBList = this.tbList.getFirstNode();
    ListNode currentTList = this.tList.getFirstNode();
    ListNode currentTEList = this.teList.getFirstNode();

    String data = "";

    while (!this.fList.isEmpty())
    {
      this.fList.removeFromBack();
    }

    while (!this.dList.isEmpty())
    {
      this.dList.removeFromBack();
    }

    while (!this.tList.isEmpty())
    {
      this.tList.removeFromBack();
    }

    while (currentBList != null)
    {
      this.fList.insertAtBack(currentBList.data.toString());
      currentBList = currentBList.next;
    }

    data = data + "   \n\n";
    data = data + "Liconic " + this.licloc + " to " + this.licdeckloc + "\n\n";
    int line3 = lic_fromto;
    int line6 = lic_cover;
    this.fList.insertAtBack("step");
    this.fList.insertAtBack("Liconic01");
    this.fList.insertAtBack(line3);
    this.fList.insertAtBack(this.licloc);
    this.fList.insertAtBack(this.licdeckloc);
    this.fList.insertAtBack(line6);
    this.fList.insertAtBack(this.lic_plate);
    this.fList.insertAtBack(this.lic_tsp);
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");

    updText(data);
    int element;
//    if (this.stepnbr < 16) element = this.stepnbr - 1; else element = this.stepnbr - 16;		//APF0020 
    if ((this.stepnbr % 15) == 0) {element = 14;}  else{element = ((this.stepnbr % 15)-1);}		//APF0027
    
    this.dTextArea[element].setCaretPosition(0);
    this.dTextArea[element].insertIcon(this.icons[29]);	//updLiconic()

    while (currentEList != null)
    {
      this.fList.insertAtBack(currentEList.data.toString());
      currentEList = currentEList.next;
    }

    while (currentDBList != null)
    {
      this.dList.insertAtBack(currentDBList.data.toString());
      currentDBList = currentDBList.next;
    }

    this.dList.insertAtBack(data);

    while (currentDEList != null)
    {
      this.dList.insertAtBack(currentDEList.data.toString());
      currentDEList = currentDEList.next;
    }

    while (currentTBList != null)
    {
      this.tList.insertAtBack(currentTBList.data.toString());
      currentTBList = currentTBList.next;
    }

    this.tList.insertAtBack("CYT");

    while (currentTEList != null)
    {
      this.tList.insertAtBack(currentTEList.data.toString());
      currentTEList = currentTEList.next;
    }

    while (!this.bList.isEmpty())
    {
      this.bList.removeFromBack();
    }
    while (!this.eList.isEmpty())
    {
      this.eList.removeFromBack();
    }
    while (!this.dbList.isEmpty())
    {
      this.dbList.removeFromBack();
    }
    while (!this.deList.isEmpty())
    {
      this.deList.removeFromBack();
    }
    while (!this.tbList.isEmpty())
    {
      this.tbList.removeFromBack();
    }
    while (!this.teList.isEmpty())
    {
      this.teList.removeFromBack();
    }

    this.lic_btnUpdateStep.setEnabled(false);
    this.checkButton.setEnabled(false);

    this.update = false;
    this.addButton.setEnabled(true);
    this.lic_btnAddStep.setEnabled(true);
    this.rmvButton.setEnabled(false);
    this.saveButton.setEnabled(true);
    this.licodatapanel2.setText1("");
    this.licodatapanel2.setText2("");
    this.licodatapanel2.setTSPIndex(1);
    this.licodatapanel2.setPlateIndex(0);

    for (int t = 0; t < this.tabcount; t++)
    {
        this.jTabbedPane1.setEnabledAt(t, true);
    }
    pkEnableDisable();
   
    //Return to Splash screen
    Splash();
  }

  private void addSPE()
  {
    this.counter = 0;
    nonPK = nonPK + 1;
    pagecount = (int) ((Math.ceil(nonPK/15.0)) - 1);		//APF0028
    String b1 = "0";
    String output = " ";
    String data = "";

    this.ti = this.time.getText().trim();
    this.pre = this.pressure.getText().trim();
    this.fil = this.fil.trim();
    this.col = this.col.trim();
    this.hline10 = this.fil;
    if (!this.col.equals("None"))
    {
      this.hline10 = this.fil;
    }

    data = data + "\n\n SPE";

    if (this.assembutton.isSelected())
    {
      b1 = "1";
      this.col = "-";
      this.fil = "-";
      this.ti = "-";
      this.pre = "-";
      data = data + "\n Assemble";
    }
    if (this.disassembutton.isSelected())
    {
      b1 = "2";
      this.col = "-";
      this.fil = "-";
      this.ti = "-";
      this.pre = "-";
      data = data + "\n Disassemble";
    }
    if (this.vacbutton.isSelected())
    {
      b1 = "3";
      this.col = "-";
      this.fil = "-";
      data = data + "\n Apply Vac";
    }

    String type = "SPE";

    this.dList.insertAtBack(data);
    this.tList.insertAtBack(type);

    this.fList.insertAtBack("step");
    this.fList.insertAtBack("SPE");
    this.fList.insertAtBack(b1 + "," + this.fil + "," + this.col + "," + this.ti + "," + this.pre);
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");

    this.dTextArea[0].setText("");
    this.dTextArea[1].setText("");
    this.dTextArea[2].setText("");
    this.dTextArea[3].setText("");
    this.dTextArea[4].setText("");
    this.dTextArea[5].setText("");
    this.dTextArea[6].setText("");
    this.dTextArea[7].setText("");
    this.dTextArea[8].setText("");
    this.dTextArea[9].setText("");
    this.dTextArea[10].setText("");
    this.dTextArea[11].setText("");
    this.dTextArea[12].setText("");
    this.dTextArea[13].setText("");
    this.dTextArea[14].setText("");

    this.dTextArea[0].setVisible(false);
    this.dTextArea[1].setVisible(false);
    this.dTextArea[2].setVisible(false);
    this.dTextArea[3].setVisible(false);
    this.dTextArea[4].setVisible(false);
    this.dTextArea[5].setVisible(false);
    this.dTextArea[6].setVisible(false);
    this.dTextArea[7].setVisible(false);
    this.dTextArea[8].setVisible(false);
    this.dTextArea[9].setVisible(false);
    this.dTextArea[10].setVisible(false);
    this.dTextArea[11].setVisible(false);
    this.dTextArea[12].setVisible(false);
    this.dTextArea[13].setVisible(false);
    this.dTextArea[14].setVisible(false);
    loadtext();
    this.time.setText("");
    this.pressure.setText("");
    this.assembutton.setSelected(true);
    this.disassembutton.setSelected(false);
    this.vacbutton.setSelected(false);
//    getFilter();
//    getCollection();
//    this.filter.setVisible(true);
//    this.collection.setVisible(true);
//    this.spelabel1.setVisible(true);
//    this.spelabel2.setVisible(true);
    this.time.setVisible(false);
    this.pressure.setVisible(false);
    this.spelabel3.setVisible(false);
    this.spelabel4.setVisible(false);
    this.spePanel.revalidate();
    this.spePanel.repaint();
    for (int t = 0; t < this.tabcount; t++)
    {
        this.jTabbedPane1.setEnabledAt(t, true);
    }
    pkEnableDisable();

    //Return to Splash screen
    Splash();

  }

  private void addMove()
  {
    this.counter = 0;
    nonPK = nonPK + 1;
    pagecount = (int) ((Math.ceil(nonPK/15.0)) - 1);		//APF0028
    String b1 = "0";
    String output = " ";
    String data = "";

    this.mov = this.mov.trim();

    data = data + "\n MOVE\n ";
    data = data + "\n " + this.mov;

    String type = "MOV";

    this.dList.insertAtBack(data);
    this.tList.insertAtBack(type);

    this.fList.insertAtBack("step");
    this.fList.insertAtBack("move");
    this.fList.insertAtBack(this.mov);
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");

    this.dTextArea[0].setText("");
    this.dTextArea[1].setText("");
    this.dTextArea[2].setText("");
    this.dTextArea[3].setText("");
    this.dTextArea[4].setText("");
    this.dTextArea[5].setText("");
    this.dTextArea[6].setText("");
    this.dTextArea[7].setText("");
    this.dTextArea[8].setText("");
    this.dTextArea[9].setText("");
    this.dTextArea[10].setText("");
    this.dTextArea[11].setText("");
    this.dTextArea[12].setText("");
    this.dTextArea[13].setText("");
    this.dTextArea[14].setText("");

    this.dTextArea[0].setVisible(false);
    this.dTextArea[1].setVisible(false);
    this.dTextArea[2].setVisible(false);
    this.dTextArea[3].setVisible(false);
    this.dTextArea[4].setVisible(false);
    this.dTextArea[5].setVisible(false);
    this.dTextArea[6].setVisible(false);
    this.dTextArea[7].setVisible(false);
    this.dTextArea[8].setVisible(false);
    this.dTextArea[9].setVisible(false);
    this.dTextArea[10].setVisible(false);
    this.dTextArea[11].setVisible(false);
    this.dTextArea[12].setVisible(false);
    this.dTextArea[13].setVisible(false);
    this.dTextArea[14].setVisible(false);
    loadtext();
    for (int t = 0; t < this.tabcount; t++)
    {
        this.jTabbedPane1.setEnabledAt(t, true);
    }
    pkEnableDisable();

    //Return to Splash screen
    Splash();

  }

  private void addMovePlate()
  {
    this.counter = 0;
    nonPK = nonPK + 1;
    pagecount = (int) ((Math.ceil(nonPK/15.0)) - 1);		//APF0028
    String b1 = "0";
    String output = " ";
    String data = "";

    data = data + "   \n ";
    data = data + this.so5 + "\n" + "Deck " + this.spl3 + " to " + this.dpl3 + "\n\n";

    String type = "MOVPL";

    this.dList.insertAtBack(data);
    this.tList.insertAtBack(type);

    this.fList.insertAtBack("step");
    this.fList.insertAtBack("move plate");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack(this.spl3 + "," + this.dpl3);
    this.fList.insertAtBack(this.so5);
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");

    this.dTextArea[0].setText("");
    this.dTextArea[1].setText("");
    this.dTextArea[2].setText("");
    this.dTextArea[3].setText("");
    this.dTextArea[4].setText("");
    this.dTextArea[5].setText("");
    this.dTextArea[6].setText("");
    this.dTextArea[7].setText("");
    this.dTextArea[8].setText("");
    this.dTextArea[9].setText("");
    this.dTextArea[10].setText("");
    this.dTextArea[11].setText("");
    this.dTextArea[12].setText("");
    this.dTextArea[13].setText("");
    this.dTextArea[14].setText("");

    this.dTextArea[0].setVisible(false);
    this.dTextArea[1].setVisible(false);
    this.dTextArea[2].setVisible(false);
    this.dTextArea[3].setVisible(false);
    this.dTextArea[4].setVisible(false);
    this.dTextArea[5].setVisible(false);
    this.dTextArea[6].setVisible(false);
    this.dTextArea[7].setVisible(false);
    this.dTextArea[8].setVisible(false);
    this.dTextArea[9].setVisible(false);
    this.dTextArea[10].setVisible(false);
    this.dTextArea[11].setVisible(false);
    this.dTextArea[12].setVisible(false);
    this.dTextArea[13].setVisible(false);
    this.dTextArea[14].setVisible(false);
    loadtext();
    this.saveButton.setEnabled(true);
    this.source5.setSelectedIndex(0);

    this.splatetype3.setSelectedIndex(0);
    this.dplatetype3.setSelectedIndex(0);
    this.splatetype3.revalidate();
    this.splatetype3.repaint();
    this.dplatetype3.revalidate();
    this.dplatetype3.repaint();
    
    for (int t = 0; t < this.tabcount; t++)
    {
        this.jTabbedPane1.setEnabledAt(t, true);
    }
    pkEnableDisable();

    //Return to Splash screen
    Splash();
  }

  private void addPause()
  {
    this.counter = 0;
    nonPK = nonPK + 1;
    pagecount = (int) ((Math.ceil(nonPK/15.0)) - 1);		//APF0028
    
    String data = "";
    String line3 = "";
    String line4 = "";
    String tchk = "";
    
    data = data + "\n\n PAUSE";

    String type = "PAU";

    this.dList.insertAtBack(data);
    this.tList.insertAtBack(type);

    if (pausedataPanel2.isTimed())
    {
    	tchk = "1";
    }
    else
    {
    	tchk = "0";
    	pminutes = "";
    	pseconds = "";
    }

    line3 = tchk + "," + pminutes + "," + 
    		pseconds;
    
    line4 = pausedataPanel2.getMessage().trim();

    this.fList.insertAtBack("step");
    this.fList.insertAtBack("Pause");
    this.fList.insertAtBack(line3);
    this.fList.insertAtBack(line4);
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");

    this.dTextArea[0].setText("");
    this.dTextArea[1].setText("");
    this.dTextArea[2].setText("");
    this.dTextArea[3].setText("");
    this.dTextArea[4].setText("");
    this.dTextArea[5].setText("");
    this.dTextArea[6].setText("");
    this.dTextArea[7].setText("");
    this.dTextArea[8].setText("");
    this.dTextArea[9].setText("");
    this.dTextArea[10].setText("");
    this.dTextArea[11].setText("");
    this.dTextArea[12].setText("");
    this.dTextArea[13].setText("");
    this.dTextArea[14].setText("");

    this.dTextArea[0].setVisible(false);
    this.dTextArea[1].setVisible(false);
    this.dTextArea[2].setVisible(false);
    this.dTextArea[3].setVisible(false);
    this.dTextArea[4].setVisible(false);
    this.dTextArea[5].setVisible(false);
    this.dTextArea[6].setVisible(false);
    this.dTextArea[7].setVisible(false);
    this.dTextArea[8].setVisible(false);
    this.dTextArea[9].setVisible(false);
    this.dTextArea[10].setVisible(false);
    this.dTextArea[11].setVisible(false);
    this.dTextArea[12].setVisible(false);
    this.dTextArea[13].setVisible(false);
    this.dTextArea[14].setVisible(false);
    
    loadtext();
    
    this.pausedataPanel2.setMinutes(" ");
    this.pausedataPanel2.setSeconds(" ");
    this.pausedataPanel2.setMessage(" ");
    this.pausedataPanel2.setTimed(false);

    for (int t = 0; t < this.tabcount; t++)
    {
        this.jTabbedPane1.setEnabledAt(t, true);
    }
    pkEnableDisable();

    //Return to Splash screen
    Splash();
  }

  private void addRead()
  {
    this.readCount += 1;
    nonPK = nonPK + 1;
    pagecount = (int) ((Math.ceil(nonPK/15.0)) - 1);		//APF0028
    this.counter = 0;
    String data = "";
    String t1 = "";
    String line4 = "";

    data = data + "   \n";
    data = data + "\n " + this.r1.trim();
    data = data + "\n " + this.text6.getText().trim();
    data = data + "\n " + this.temp.getText().trim() + " (c)";
    String type = "RDR";
    t1 = this.temp.getText().trim();
    if (t1.equals(""))
    {
      t1 = "0";
    }

    line4 = this.text6.getText().trim();
    if (line4.trim().equals(""))
    {
      line4 = "-";
    }

    this.dList.insertAtBack(data);
    this.tList.insertAtBack(type);

    this.fList.insertAtBack("step");
    this.fList.insertAtBack("read");
    this.fList.insertAtBack(this.r1.trim() + "," + t1);
    this.fList.insertAtBack(line4);

    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");

    this.dTextArea[0].setText("");
    this.dTextArea[1].setText("");
    this.dTextArea[2].setText("");
    this.dTextArea[3].setText("");
    this.dTextArea[4].setText("");
    this.dTextArea[5].setText("");
    this.dTextArea[6].setText("");
    this.dTextArea[7].setText("");
    this.dTextArea[8].setText("");
    this.dTextArea[9].setText("");
    this.dTextArea[10].setText("");
    this.dTextArea[11].setText("");
    this.dTextArea[12].setText("");
    this.dTextArea[13].setText("");
    this.dTextArea[14].setText("");

    this.dTextArea[0].setVisible(false);
    this.dTextArea[1].setVisible(false);
    this.dTextArea[2].setVisible(false);
    this.dTextArea[3].setVisible(false);
    this.dTextArea[4].setVisible(false);
    this.dTextArea[5].setVisible(false);
    this.dTextArea[6].setVisible(false);
    this.dTextArea[7].setVisible(false);
    this.dTextArea[8].setVisible(false);
    this.dTextArea[9].setVisible(false);
    this.dTextArea[10].setVisible(false);
    this.dTextArea[11].setVisible(false);
    this.dTextArea[12].setVisible(false);
    this.dTextArea[13].setVisible(false);
    this.dTextArea[14].setVisible(false);
    loadtext();
    this.text6.setText(" ");
    this.text6.setEditable(true);
    this.temp.setText(" ");
    for (int t = 0; t < this.tabcount; t++)
    {
        this.jTabbedPane1.setEnabledAt(t, true);
    }
    pkEnableDisable();

    //Return to Splash screen
    Splash();
  }

  private void addCherry()
  {
    this.counter = 0;
    nonPK = nonPK + 1;
    pagecount = (int) ((Math.ceil(nonPK/15.0)) - 1);		//APF0028
    String data = "";
    String t1 = "";
    String line4 = "";
    String line3 = "";

    data = data + "\n\n " + this.text7.getText().trim();
    data = data + "\n " + this.tr1.trim() + " to " + this.tr2.trim();
    String type = "CHR";

    line3 = this.text7.getText().trim() + "," + this.chltyp.trim();
    line4 = this.tr1.trim() + "," + this.tr2.trim();

    this.dList.insertAtBack(data);
    this.tList.insertAtBack(type);

    this.fList.insertAtBack("step");
    this.fList.insertAtBack("Cherry Picking");
    this.fList.insertAtBack(line3);
    this.fList.insertAtBack(line4);
    this.fList.insertAtBack(this.chso.trim());
    this.fList.insertAtBack(this.chde.trim());
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");

    this.dTextArea[0].setText("");
    this.dTextArea[1].setText("");
    this.dTextArea[2].setText("");
    this.dTextArea[3].setText("");
    this.dTextArea[4].setText("");
    this.dTextArea[5].setText("");
    this.dTextArea[6].setText("");
    this.dTextArea[7].setText("");
    this.dTextArea[8].setText("");
    this.dTextArea[9].setText("");
    this.dTextArea[10].setText("");
    this.dTextArea[11].setText("");
    this.dTextArea[12].setText("");
    this.dTextArea[13].setText("");
    this.dTextArea[14].setText("");

    this.dTextArea[0].setVisible(false);
    this.dTextArea[1].setVisible(false);
    this.dTextArea[2].setVisible(false);
    this.dTextArea[3].setVisible(false);
    this.dTextArea[4].setVisible(false);
    this.dTextArea[5].setVisible(false);
    this.dTextArea[6].setVisible(false);
    this.dTextArea[7].setVisible(false);
    this.dTextArea[8].setVisible(false);
    this.dTextArea[9].setVisible(false);
    this.dTextArea[10].setVisible(false);
    this.dTextArea[11].setVisible(false);
    this.dTextArea[12].setVisible(false);
    this.dTextArea[13].setVisible(false);
    this.dTextArea[14].setVisible(false);
    loadtext();
    this.text7.setText(" ");
    this.text7.setEditable(true);
    this.track1.setSelectedIndex(0);
    this.track2.setSelectedIndex(0);
    this.chsource.setSelectedIndex(0);
    this.chdest.setSelectedIndex(0);
    this.chliq.setSelectedIndex(0);
    
    for (int t = 0; t < this.tabcount; t++)
    {
        this.jTabbedPane1.setEnabledAt(t, true);
    }
    pkEnableDisable();

    //Return to Splash screen
    Splash();
  }

  private void addCherry02()
  {
    this.counter = 0;
    nonPK = nonPK + 1;
    pagecount = (int) ((Math.ceil(nonPK/15.0)) - 1);		//APF0028
    String data = "";
    String t1 = "";
    String line3 = "";
    String line4 = "";
    String line5 = "";
    String line6 = "";
    String line7 = "";
    String line8 = "";
    String line9 = "";
    String line10 = "";
    
//	Text writting to Step Icon.  NOTE: text actually gets written in loadtext() called below.
//	Get the file name from path and split into multiple lines to allow defined wrapping in icon 
	String fileName = cherry02dataPanel2.getWorkFile().trim().substring(cherry02dataPanel2.getWorkFile().trim().lastIndexOf("\\") + 1);
	if (fileName.length() > 12){fileName = new StringBuilder(fileName).insert(12, "...\n").toString();}		//CPF0032
	if (fileName.length() > 27)fileName = new StringBuilder(fileName).insert(27, "...\n").toString();		//CPF0032
	data = data + "\n\n" + fileName;
//  data = data + "\n " + ch2_plateloc.trim();
//  data = data + "\n " + ch2_liqtype.trim();
	
    String type = "CHR2";
    String ch2_scycle = this.cherry02dataPanel2.getSCycle();
    
    line3 = this.cherry02dataPanel2.getWorkFile().trim();
    line4 = ch2_plateloc;
    line5 = this.cherry02dataPanel2.getSPlateType().trim();
    line6 = this.cherry02dataPanel2.getDPlateType().trim();
    line7 = ch2_liqtype.trim();
    line8 = ch2_scycle + "," + ch2_smix  ;
    line9 = ch2_dcycle + "," + ch2_dmix  ;
    line10 = ch2_tubeloc;
    		

    this.dList.insertAtBack(data);
    this.tList.insertAtBack(type);

    this.fList.insertAtBack("step");
    this.fList.insertAtBack("CherryPicking02");
    this.fList.insertAtBack(line3);
    this.fList.insertAtBack(line4);
    this.fList.insertAtBack(line5);
    this.fList.insertAtBack(line6);
    this.fList.insertAtBack(line7);
    this.fList.insertAtBack(line8);
    this.fList.insertAtBack(line9);
    this.fList.insertAtBack(line10);

    this.dTextArea[0].setText("");
    this.dTextArea[1].setText("");
    this.dTextArea[2].setText("");
    this.dTextArea[3].setText("");
    this.dTextArea[4].setText("");
    this.dTextArea[5].setText("");
    this.dTextArea[6].setText("");
    this.dTextArea[7].setText("");
    this.dTextArea[8].setText("");
    this.dTextArea[9].setText("");
    this.dTextArea[10].setText("");
    this.dTextArea[11].setText("");
    this.dTextArea[12].setText("");
    this.dTextArea[13].setText("");
    this.dTextArea[14].setText("");

    this.dTextArea[0].setVisible(false);
    this.dTextArea[1].setVisible(false);
    this.dTextArea[2].setVisible(false);
    this.dTextArea[3].setVisible(false);
    this.dTextArea[4].setVisible(false);
    this.dTextArea[5].setVisible(false);
    this.dTextArea[6].setVisible(false);
    this.dTextArea[7].setVisible(false);
    this.dTextArea[8].setVisible(false);
    this.dTextArea[9].setVisible(false);
    this.dTextArea[10].setVisible(false);
    this.dTextArea[11].setVisible(false);
    this.dTextArea[12].setVisible(false);
    this.dTextArea[13].setVisible(false);
    this.dTextArea[14].setVisible(false);
    loadtext();

    this.cherry02dataPanel2.setText1(" ");
    this.cherry02dataPanel2.setText2(" ");
    this.cherry02dataPanel2.setText3(" ");
    this.cherry02dataPanel2.setText4(" ");
    this.cherry02dataPanel2.setSLabIndex(0);
    this.cherry02dataPanel2.setDLabIndex(0);
    this.cherry02dataPanel2.setSPlateIndex(0);
    this.cherry02dataPanel2.setDPlateIndex(0);
    this.cherry02dataPanel2.setBarIndex(0);
    this.cherry02dataPanel2.setLiqIndex(0);
    this.cherry02dataPanel2.setSCycle1(0);
    this.cherry02dataPanel2.setDCycle1(0);
    this.cherry02dataPanel2.setT1Enabled(false);
    this.cherry02dataPanel2.setT3Enabled(false);
    this.cherry02dataPanel2.setT4Enabled(false);
//  this.cherry02dataPanel2.setSCycleIndex(0);
//  this.cherry02dataPanel2.setDCycleIndex(0);

    for (int t = 0; t < this.tabcount; t++)
    {
        this.jTabbedPane1.setEnabledAt(t, true);
    }
    pkEnableDisable();

    //Return to Splash screen
    Splash();
  }
  
  
  private void addPK()
  {
    this.counter = 0;
    cntPK = cntPK + 1;
    String data = "";
    String t1 = "";
    String line3 = "";
    String line4 = "";
    String line5 = "";
    String line6 = "";
    String line7 = "";

    data = data + "\n " + this.pkdataPanel2x.getpkmtextwat().trim() + "\n " + this.text9.getText().trim();
    String type = "PKS";

    try
    {
    pksam = this.pkdataPanel2x.getpkmsamdil().trim();
    }
    catch (NullPointerException e) {
      pksam = "null" ;
    }
    try
    {
      pkstd = this.pkdataPanel2x.getpkmstddil().trim();
    }
    catch (NullPointerException e) {
      pkstd = "null" ;
    }
    
    try
    {
      pkmrddeck = this.pkdataPanel2x.getpkdeck1().trim();
    }
    catch (NullPointerException e) {
      pkmrddeck = "null" ;
    }
    try
    {
    	pkdillab2 = this.pkdataPanel2x.getpkdilsrc1().trim();
    }
    catch (NullPointerException e) {
      pkdillab2 = "null" ;
    }
    try
    {
        pkmrddil = this.pkdataPanel2x.getpkmmrddil().trim();
    }
    catch (NullPointerException e) {
      pkmrddil = "null" ;
    }
    try
    {
        pkdildil = this.pkdataPanel2x.getpkdil().trim();
    }
    catch (NullPointerException e) {
      pkdildil = "null" ;
    }
    try
    {
        pkdildeck = this.pkdataPanel2x.getpkdeck2().trim();
    }
    catch (NullPointerException e) {
      pkdildeck = "null" ;
    } 
    try
    {
        pkdillab3 = this.pkdataPanel2x.getpkdilres().trim();
    }
    catch (NullPointerException e) {
      pkdillab3 = "null" ;
    } 
    

    try
    {
    	pksamlab = this.pkdataPanel2x.getpkssrc().trim();
    }
    catch (NullPointerException e) {
      pksamlab = "null" ;
    }
    try
    {
    	pkstdlab = this.pkdataPanel2x.getpkstdsrc().trim();
    }
    catch (NullPointerException e) {
      pkstdlab = "null" ;
    }
    try
    {
    	pkmrdlab = this.pkdataPanel2x.getpkmrdsrc().trim();
    }
    catch (NullPointerException e) {
    	pkmrdlab = "null" ;
    }
    try
    {
    	pkdillab = this.pkdataPanel2x.getpkdilsrc2().trim();
    }
    catch (NullPointerException e) {
    	pkdillab = "null" ;
    }
    try
    {
    	pkpasslab = this.pkdataPanel2x.getpassaylab().trim();
    }
    catch (NullPointerException e) {
    	pkpasslab = "null" ;
    }
    try
    {
    	pkasslab = this.pkdataPanel2x.getassaylab().trim();
    }
    catch (NullPointerException e) {
    	pkasslab = "null" ;
    }
    if (!mrdon)
    {
//  	  pksam = "null";
//  	  pkstd = "null";
//	  pkmrddeck = "null";
      pkmrdlab = "null";
//  	  pkm_text_1.setText("");
//  	  pkm_text_ul.setText("");
//  	  pkdillab2 = "null";
      if (!dilon)
      {
      	  pkmrddil = "null";
    	  pkdildil = "null";
    	  pkdildeck = "null";
      	  pkdillab3 = "null";
      }
      if (!preassayon)
      {
    	  pkstdlab = "null";
//    	  TabWindow.this.pkMRDsource.setEnabled(false);
//    	  TabWindow.this.pkMRDdil.setEnabled(false);
      }
    }

    if (!dilon)
    {
  	  pkdildil = "null";
	  pkdildeck = "null";
  	  pkdillab3 = "null";
    }


	line3 = this.pkdataPanel2x.getpkmtextwat().trim() + "," + this.pkdataPanel2x.getpkmtext1().trim()
	+ ","  + this.pkdataPanel2x.getpkmtextul().trim() + ","  + this.pkdataPanel2x.getpkmpassayul().trim() + ","  
    + this.pkdataPanel2x.getpkmassayul().trim();

    line4 = this.pksam.trim() + "," + this.pkstd.trim()
    + "," + this.pkmrddil.trim() + "," + this.pkdildil.trim();

    if (mrdon)
    {
      pkck1 = "1";
    }
    else
    {
   	  pkmrdlab = "null";
      pkck1 = "0";
      if (!dilon)
      {
    	  pkmrdlab = "null";
      }
//      pksamlab = "null";
//      pkstdlab = "null";
    }

    if (dilon)
    {
      pkck2 = "1";
    }
    else
    {
      pkck2 = "0";
      pkdillab = "null";
    }
    if (preassayon)
    {
      pkck3 = "1";
    }
    else
    {
      pkck3 = "0";
      pkpasslab = "null";
//      pkstdlab = "null";
    }
    
    if (assayon)
    {
      pkck4 = "1";
    }
    else
    {
      pkck4 = "0";
      pkasslab = "null";
    }

    line5 = this.pksamlab.trim() + "," + this.pkstdlab.trim() + "," + this.pkmrdlab.trim() 
    + "," + this.pkdillab.trim() + ","  + this.pkpasslab.trim()+ "," + this.pkasslab.trim();
    		
    line6 = pkck1 +  ","  + pkck2 +  "," + pkck3 +  ","  + pkck4 ;

    line7 = pkdillab2 + "," + pkmrddeck +  "," + pkdillab3 + "," + pkdildeck;
    
    this.dList.insertAtBack(data);
    this.tList.insertAtBack(type);

    this.fList.insertAtBack("step");
    this.fList.insertAtBack("PK");
    this.fList.insertAtBack(line3);
    this.fList.insertAtBack(line4);
    this.fList.insertAtBack(line5);
    this.fList.insertAtBack(line6);
    this.fList.insertAtBack(line7);
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");

    this.dTextArea[0].setText("");
    this.dTextArea[1].setText("");
    this.dTextArea[2].setText("");
    this.dTextArea[3].setText("");
    this.dTextArea[4].setText("");
    this.dTextArea[5].setText("");
    this.dTextArea[6].setText("");
    this.dTextArea[7].setText("");
    this.dTextArea[8].setText("");
    this.dTextArea[9].setText("");
    this.dTextArea[10].setText("");
    this.dTextArea[11].setText("");
    this.dTextArea[12].setText("");
    this.dTextArea[13].setText("");
    this.dTextArea[14].setText("");

    this.dTextArea[0].setVisible(false);
    this.dTextArea[1].setVisible(false);
    this.dTextArea[2].setVisible(false);
    this.dTextArea[3].setVisible(false);
    this.dTextArea[4].setVisible(false);
    this.dTextArea[5].setVisible(false);
    this.dTextArea[6].setVisible(false);
    this.dTextArea[7].setVisible(false);
    this.dTextArea[8].setVisible(false);
    this.dTextArea[9].setVisible(false);
    this.dTextArea[10].setVisible(false);
    this.dTextArea[11].setVisible(false);
    this.dTextArea[12].setVisible(false);
    this.dTextArea[13].setVisible(false);
    this.dTextArea[14].setVisible(false);
    loadtext();

//      this.pkdataPanel2x.setpkmtextwat("");
//      this.pkdataPanel2x.setpkmtext1("");
//      this.pkdataPanel2x.setpkmtextul("");
      this.pkdataPanel2x.setmrdindex(0);
      this.pkdataPanel2x.setsamindex(0);
      this.pkdataPanel2x.setstdindex(0);
      this.pkdataPanel2x.setmrddilindex(0);
      this.pkdataPanel2x.setsamdilindex(0);
      this.pkdataPanel2x.setstddilindex(0);
      this.pkdataPanel2x.setdeck1index(0);
      this.pkdataPanel2x.setMRDon(false);
//      this.pkdataPanel2x.mrdon = false;

      this.pkdataPanel2x.setDILon(false);
//      this.pkdataPanel2x.dilon = false;
      this.pkdataPanel2x.setdilsrcindex(0);
      this.pkdataPanel2x.setdilresindex(0);
      this.pkdataPanel2x.setdeck2index(0);
      this.pkdataPanel2x.setpkmpassayul("");
      
      this.pkdataPanel2x.setPRAon(false);
//      this.pkdataPanel2x.preassayon = false;
      this.pkdataPanel2x.setpassaylabindex(0);

      this.pkdataPanel2x.setASSon(false);
//      this.pkdataPanel2x.assayon = false;
      this.pkdataPanel2x.setassaylabindex(0);
//    this.pkm_text_wat.setEditable(true);
//      this.pkm_text_1.setText("");
//    this.pkm_text_1.setEditable(true);
//    this.pkm_text_1.setEnabled(false);
//    this.pkm_text_ul.setText("");
//    this.pkm_text_ul.setEditable(true);
//    this.pkm_text_ul.setEnabled(false);
//    this.pkassay.setEnabled(false);
//    this.pkPassay.setEnabled(false);
//    this.pkMRDsource.setEnabled(false);
//    this.pkSsource.setEnabled(false);
//    this.pkDsource.setEnabled(false);
//    this.pkSTDsource.setEnabled(false);
//    this.pkstddil.setEnabled(false);
//    this.pksamdil.setEnabled(false);
//    this.pkMRDdil.setEnabled(false);
//    this.pkdil.setEnabled(false);
//    this.pkDsource.setEnabled(false);
//    this.pkdeck1.setEnabled(false);
//    this.pkdeck2.setEnabled(false);
//    this.pkdeck1.setVisible(false);
//    this.pkdeck2.setVisible(false);
//    this.pkDilsource.setEnabled(false);
//    this.pkDilreserve.setEnabled(false);
//	this.lblDiluent2.setVisible(false);
//   this.text11.setText("0");
//    this.text11.setEditable(true);
//    this.text12.setText("0");
//    this.text12.setEditable(true);
//    this.pklabel4.setVisible(false);
//    this.pkcheck1.setSelected(false);
//    this.pkcheck2.setSelected(false);
//    this.pkcheck3.setSelected(false);
//    this.pkcheck4.setSelected(false);

    for (int t = 0; t < this.tabcount; t++)
    {
        this.jTabbedPane1.setEnabledAt(t, true);
    }
    pkEnableDisable();

    //Reset PK Screen
    pkdataPanel2x.resetScreen();
    
    //Return to Splash screen
    Splash();
  }

  private void addPKsave()
  {
    this.counter = 0;
    cntPK = cntPK + 1;
    String data = "";
    String t1 = "";
    String line3 = "";
    String line4 = "";
    String line5 = "";
    String line6 = "";
    String line7 = "";

    data = data + "\n " + this.pkm_text_wat.getText().trim() + "\n " + this.text9.getText().trim();
    String type = "PKS";

    if (!pkcheck1.isSelected())
    {
  	  pksam = "null";
  	  pkstd = "null";
	  pkmrddeck = "null";
  	  pkm_text_1.setText("");
  	  pkm_text_ul.setText("");
  	  pkdillab2 = "null";
      if (!pkcheck2.isSelected())
      {
      	  pkmrddil = "null";
    	  pkdildil = "null";
    	  pkdildeck = "null";
      	  pkdillab3 = "null";
      }
    }

    if (!pkcheck2.isSelected())
    {
  	  pkdildil = "null";
	  pkdildeck = "null";
  	  pkdillab3 = "null";
    }

    
    
    line3 = this.pkm_text_wat.getText().trim() + "," + this.pkm_text_1.getText().trim()
	+ ","  + this.pkm_text_ul.getText().trim() + ","  + this.pkm_text_ul_preAssay.getText().trim() + ","  
    + this.pkm_text_ul_Assay.getText().trim();

    line4 = this.pksam.trim() + "," + this.pkstd.trim()
    + "," + this.pkmrddil.trim() + "," + this.pkdildil.trim();

    
    if (pkcheck1.isSelected())
    {
      pkck1 = "1";
    }
    else
    {
      pkck1 = "0";
      if (!pkcheck2.isSelected())
      {
    	  pkmrdlab = "null";
      }
      pksamlab = "null";
      pkstdlab = "null";
    }

    if (pkcheck2.isSelected())
    {
      pkck2 = "1";
    }
    else
    {
      pkck2 = "0";
      pkdillab = "null";
    }
    if (pkcheck3.isSelected())
    {
      pkck3 = "1";
    }
    else
    {
      pkck3 = "0";
      pkpasslab = "null";
//      pkstdlab = "null";
    }
    
    if (pkcheck4.isSelected())
    {
      pkck4 = "1";
    }
    else
    {
      pkck4 = "0";
      pkasslab = "null";
    }
    
    line5 = this.pksamlab.trim() + "," + this.pkstdlab.trim() + "," + this.pkmrdlab.trim() 
    + "," + this.pkdillab.trim() + ","  + this.pkpasslab.trim()+ "," + this.pkasslab.trim();
    		
    line6 = pkck1 +  ","  + pkck2 +  "," + pkck3 +  ","  + pkck4 ;

    line7 = pkdillab2 + "," + pkmrddeck +  "," + pkdillab3 + "," + pkdildeck;
    
    this.dList.insertAtBack(data);
    this.tList.insertAtBack(type);

    this.fList.insertAtBack("step");
    this.fList.insertAtBack("PK");
    this.fList.insertAtBack(line3);
    this.fList.insertAtBack(line4);
    this.fList.insertAtBack(line5);
    this.fList.insertAtBack(line6);
    this.fList.insertAtBack(line7);
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");

    this.dTextArea[0].setText("");
    this.dTextArea[1].setText("");
    this.dTextArea[2].setText("");
    this.dTextArea[3].setText("");
    this.dTextArea[4].setText("");
    this.dTextArea[5].setText("");
    this.dTextArea[6].setText("");
    this.dTextArea[7].setText("");
    this.dTextArea[8].setText("");
    this.dTextArea[9].setText("");
    this.dTextArea[10].setText("");
    this.dTextArea[11].setText("");
    this.dTextArea[12].setText("");
    this.dTextArea[13].setText("");
    this.dTextArea[14].setText("");

    this.dTextArea[0].setVisible(false);
    this.dTextArea[1].setVisible(false);
    this.dTextArea[2].setVisible(false);
    this.dTextArea[3].setVisible(false);
    this.dTextArea[4].setVisible(false);
    this.dTextArea[5].setVisible(false);
    this.dTextArea[6].setVisible(false);
    this.dTextArea[7].setVisible(false);
    this.dTextArea[8].setVisible(false);
    this.dTextArea[9].setVisible(false);
    this.dTextArea[10].setVisible(false);
    this.dTextArea[11].setVisible(false);
    this.dTextArea[12].setVisible(false);
    this.dTextArea[13].setVisible(false);
    this.dTextArea[14].setVisible(false);
    loadtext();
    this.pkm_text_wat.setText("");
    this.pkm_text_wat.setEditable(true);
    this.pkm_text_1.setText("");
    this.pkm_text_1.setEditable(true);
    this.pkm_text_1.setEnabled(false);
    this.pkm_text_ul.setText("");
    this.pkm_text_ul.setEditable(true);
    this.pkm_text_ul.setEnabled(false);
    this.pkassay.setEnabled(false);
    this.pkPassay.setEnabled(false);
    this.pkMRDsource.setEnabled(false);
    this.pkSsource.setEnabled(false);
    this.pkDsource.setEnabled(false);
    this.pkSTDsource.setEnabled(false);
    this.pkstddil.setEnabled(false);
    this.pksamdil.setEnabled(false);
    this.pkMRDdil.setEnabled(false);
    this.pkdil.setEnabled(false);
    this.pkDsource.setEnabled(false);
    this.pkdeck1.setEnabled(false);
    this.pkdeck2.setEnabled(false);
//    this.pkdeck1.setVisible(false);
//    this.pkdeck2.setVisible(false);
    this.pkDilsource.setEnabled(false);
    this.pkDilreserve.setEnabled(false);
	this.lblDiluent2.setVisible(false);
    this.text11.setText("0");
    this.text11.setEditable(true);
    this.text12.setText("0");
    this.text12.setEditable(true);
//    this.pklabel4.setVisible(false);
    this.pkcheck1.setSelected(false);
    this.pkcheck2.setSelected(false);
    this.pkcheck3.setSelected(false);
    this.pkcheck4.setSelected(false);

    for (int t = 0; t < this.tabcount; t++)
    {
        this.jTabbedPane1.setEnabledAt(t, true);
    }
    pkEnableDisable();

   //Return to Splash screen
    Splash();
  }

  private void addInc()
  {
    this.counter = 0;

    String line4 = "";
    String line5 = "";
    String data = "";
    data = data + "   \n\n";
    data = data + "\n " + this.text4.getText().trim() + " min";
    data = data + "\n " + this.itemp.getText().trim() + " (c)" + "\n";

    line4 = this.itemp.getText().trim();
    if (line4.trim().equals(""))
    {
      line4 = "0";
    }
    line5 = this.text4pm.getText().trim();
    if (line5.trim().equals(""))
    {
      line5 = "0";
    }

    String type = "INC";

    this.dList.insertAtBack(data);
    this.tList.insertAtBack(type);

    this.fList.insertAtBack("step");
    this.fList.insertAtBack("incubate");
    this.fList.insertAtBack(this.text4.getText().trim());
    this.fList.insertAtBack(line4);
    this.fList.insertAtBack(line5);

    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");

    this.text4.setText(" ");
    this.text4pm.setText("0");
    this.itemp.setText("");

    this.dTextArea[0].setText("");
    this.dTextArea[1].setText("");
    this.dTextArea[2].setText("");
    this.dTextArea[3].setText("");
    this.dTextArea[4].setText("");
    this.dTextArea[5].setText("");
    this.dTextArea[6].setText("");
    this.dTextArea[7].setText("");
    this.dTextArea[8].setText("");
    this.dTextArea[9].setText("");
    this.dTextArea[10].setText("");
    this.dTextArea[11].setText("");
    this.dTextArea[12].setText("");
    this.dTextArea[13].setText("");
    this.dTextArea[14].setText("");
    this.dTextArea[0].setVisible(false);
    this.dTextArea[1].setVisible(false);
    this.dTextArea[2].setVisible(false);
    this.dTextArea[3].setVisible(false);
    this.dTextArea[4].setVisible(false);
    this.dTextArea[5].setVisible(false);
    this.dTextArea[6].setVisible(false);
    this.dTextArea[7].setVisible(false);
    this.dTextArea[8].setVisible(false);
    this.dTextArea[9].setVisible(false);
    this.dTextArea[10].setVisible(false);
    this.dTextArea[11].setVisible(false);
    this.dTextArea[12].setVisible(false);
    this.dTextArea[13].setVisible(false);
    this.dTextArea[14].setVisible(false);
    loadtext();
    for (int t = 0; t < this.tabcount; t++)
    {
        this.jTabbedPane1.setEnabledAt(t, true);
    }
    pkEnableDisable();

    //Return to Splash screen
    Splash();
  }

  public void addCytomat()
  {
    this.counter = 0;
    nonPK = nonPK + 1;
    pagecount = (int) ((Math.ceil(nonPK/15.0)) - 1);		//APF0028
    String output = " ";
    String data = "";

    data = data + "   \n\n";
    data = data + "Cytomat " + this.cytloc + " to " + this.cytdeckloc + "\n\n";

    String type = "CYT";
    

    int line3 = cyt_fromto;
    int line6 = cyt_cover;

    this.dList.insertAtBack(data);
    this.tList.insertAtBack(type);

    this.fList.insertAtBack("step");
    this.fList.insertAtBack("Cytomat01");
    this.fList.insertAtBack(line3);
    this.fList.insertAtBack(this.cytloc);
    this.fList.insertAtBack(this.cytdeckloc);
    this.fList.insertAtBack(line6);
    this.fList.insertAtBack(this.cyt_plate);
    this.fList.insertAtBack(this.cyt_tsp);
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");

    this.dTextArea[0].setText("");
    this.dTextArea[1].setText("");
    this.dTextArea[2].setText("");
    this.dTextArea[3].setText("");
    this.dTextArea[4].setText("");
    this.dTextArea[5].setText("");
    this.dTextArea[6].setText("");
    this.dTextArea[7].setText("");
    this.dTextArea[8].setText("");
    this.dTextArea[9].setText("");
    this.dTextArea[10].setText("");
    this.dTextArea[11].setText("");
    this.dTextArea[12].setText("");
    this.dTextArea[13].setText("");
    this.dTextArea[14].setText("");

    this.dTextArea[0].setVisible(false);
    this.dTextArea[1].setVisible(false);
    this.dTextArea[2].setVisible(false);
    this.dTextArea[3].setVisible(false);
    this.dTextArea[4].setVisible(false);
    this.dTextArea[5].setVisible(false);
    this.dTextArea[6].setVisible(false);
    this.dTextArea[7].setVisible(false);
    this.dTextArea[8].setVisible(false);
    this.dTextArea[9].setVisible(false);
    this.dTextArea[10].setVisible(false);
    this.dTextArea[11].setVisible(false);
    this.dTextArea[12].setVisible(false);
    this.dTextArea[13].setVisible(false);
    this.dTextArea[14].setVisible(false);
    loadtext();
    this.saveButton.setEnabled(true);
    this.cytodatapanel2.setText1("");
    this.cytodatapanel2.setText2("");
    this.cytodatapanel2.setFTIndex(0);
    this.cytodatapanel2.setCoverIndex(0);
    this.cytodatapanel2.setPlateIndex(0);
    this.cytodatapanel2.setTSPIndex(1);

    for (int t = 0; t < this.tabcount; t++)
    {
        this.jTabbedPane1.setEnabledAt(t, true);
    }
    pkEnableDisable();

    //Return to Splash screen
    Splash();
  }

  public void addMovePlate2()
  {
    this.counter = 0;
    nonPK = nonPK + 1;
    pagecount = (int) ((Math.ceil(nonPK/15.0)) - 1);		//APF0028
    String output = " ";
    String data = "";

    data = data + "   \n\n";
    data = data + "Move Plate from " + this.startloc + " to " + this.endloc + "\n\n";

    String type = "MVP2";
    

    int line3 = mvp_type;
    String line6 = mvp_plate;

    this.dList.insertAtBack(data);
    this.tList.insertAtBack(type);

    this.fList.insertAtBack("step");
    this.fList.insertAtBack("MovePlate02");
    this.fList.insertAtBack(line3);
    this.fList.insertAtBack(this.startloc);
    this.fList.insertAtBack(this.endloc);
    this.fList.insertAtBack(line6);
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");

    this.dTextArea[0].setText("");
    this.dTextArea[1].setText("");
    this.dTextArea[2].setText("");
    this.dTextArea[3].setText("");
    this.dTextArea[4].setText("");
    this.dTextArea[5].setText("");
    this.dTextArea[6].setText("");
    this.dTextArea[7].setText("");
    this.dTextArea[8].setText("");
    this.dTextArea[9].setText("");
    this.dTextArea[10].setText("");
    this.dTextArea[11].setText("");
    this.dTextArea[12].setText("");
    this.dTextArea[13].setText("");
    this.dTextArea[14].setText("");

    this.dTextArea[0].setVisible(false);
    this.dTextArea[1].setVisible(false);
    this.dTextArea[2].setVisible(false);
    this.dTextArea[3].setVisible(false);
    this.dTextArea[4].setVisible(false);
    this.dTextArea[5].setVisible(false);
    this.dTextArea[6].setVisible(false);
    this.dTextArea[7].setVisible(false);
    this.dTextArea[8].setVisible(false);
    this.dTextArea[9].setVisible(false);
    this.dTextArea[10].setVisible(false);
    this.dTextArea[11].setVisible(false);
    this.dTextArea[12].setVisible(false);
    this.dTextArea[13].setVisible(false);
    this.dTextArea[14].setVisible(false);
    loadtext();
    this.saveButton.setEnabled(true);
    this.movplv2datapanel2.setText1("");
    this.movplv2datapanel2.setText2("");
    this.movplv2datapanel2.setTypeIndex(0);
    this.movplv2datapanel2.setPlateIndex(0);

    for (int t = 0; t < this.tabcount; t++)
    {
        this.jTabbedPane1.setEnabledAt(t, true);
    }
    pkEnableDisable();

    //Return to Splash screen
    Splash();
  }

  public void addLoad()
  {
    this.counter = 0;
    nonPK = nonPK + 1;
    pagecount = (int) ((Math.ceil(nonPK/15.0)) - 1);		//APF0028
    String output = " ";
    String data = "";

    data = data + "   \n\n";
    data = data + "Load  " + "\n\n";

    String type = "LOAD01";

    String line3 = load_plateindex + "," + this.load_plate ;
    String line4 = load_tubeindex + "," + this.load_tube ;
    String line5 = load_tipindex + "," + this.load_tip ;
    
    this.dList.insertAtBack(data);
    this.tList.insertAtBack(type);

    this.fList.insertAtBack("step");
    this.fList.insertAtBack("LOAD01");
    this.fList.insertAtBack(line3);
    this.fList.insertAtBack(line4);
    this.fList.insertAtBack(line5);
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");

    this.dTextArea[0].setText("");
    this.dTextArea[1].setText("");
    this.dTextArea[2].setText("");
    this.dTextArea[3].setText("");
    this.dTextArea[4].setText("");
    this.dTextArea[5].setText("");
    this.dTextArea[6].setText("");
    this.dTextArea[7].setText("");
    this.dTextArea[8].setText("");
    this.dTextArea[9].setText("");
    this.dTextArea[10].setText("");
    this.dTextArea[11].setText("");
    this.dTextArea[12].setText("");
    this.dTextArea[13].setText("");
    this.dTextArea[14].setText("");

    this.dTextArea[0].setVisible(false);
    this.dTextArea[1].setVisible(false);
    this.dTextArea[2].setVisible(false);
    this.dTextArea[3].setVisible(false);
    this.dTextArea[4].setVisible(false);
    this.dTextArea[5].setVisible(false);
    this.dTextArea[6].setVisible(false);
    this.dTextArea[7].setVisible(false);
    this.dTextArea[8].setVisible(false);
    this.dTextArea[9].setVisible(false);
    this.dTextArea[10].setVisible(false);
    this.dTextArea[11].setVisible(false);
    this.dTextArea[12].setVisible(false);
    this.dTextArea[13].setVisible(false);
    this.dTextArea[14].setVisible(false);
    loadtext();
    this.saveButton.setEnabled(true);
    this.loaddataPanel2.setPlateText("");
    this.loaddataPanel2.setTubeText("");
    this.loaddataPanel2.setTipText("");
    this.loaddataPanel2.setPlateIndex(0);
    this.loaddataPanel2.setTubeIndex(0);
    this.loaddataPanel2.setTipIndex(0);

    for (int t = 0; t < this.tabcount; t++)
    {
        this.jTabbedPane1.setEnabledAt(t, true);
    }
    pkEnableDisable();

    //Return to Splash screen
    Splash();
  }

  public void addDNANorm()
  {
    this.counter = 0;
    nonPK = nonPK + 1;
    pagecount = (int) ((Math.ceil(nonPK/15.0)) - 1);		//APF0028
    String output = " ";
    String data = "";

    data = data + "   \n\n";
    data = data + "DNA Normalization " + this.dna_con + " and " + this.dna_vol + "\n\n";

    String type = "DNA";
    

    String line3 = dna_qfile;
    int line4 = dna_procindex;
    String line5 = this.dna_con + "," + this.dna_vol;
    String line6 = this.dna_dplate + "," + this.dna_splate + "," + this.dna_nplate + "," + this.dna_plate;
    String line7 = dna_deck1 + "," + dna_deck2 + "," + dna_deck3 + "," + dna_deck4 ;
    
    this.dList.insertAtBack(data);
    this.tList.insertAtBack(type);

    this.fList.insertAtBack("step");
    this.fList.insertAtBack("CORDNA01");
    this.fList.insertAtBack(line3);
    this.fList.insertAtBack(line4);
    this.fList.insertAtBack(line5);
    this.fList.insertAtBack(line6);
    this.fList.insertAtBack(line7);
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");

    this.dTextArea[0].setText("");
    this.dTextArea[1].setText("");
    this.dTextArea[2].setText("");
    this.dTextArea[3].setText("");
    this.dTextArea[4].setText("");
    this.dTextArea[5].setText("");
    this.dTextArea[6].setText("");
    this.dTextArea[7].setText("");
    this.dTextArea[8].setText("");
    this.dTextArea[9].setText("");
    this.dTextArea[10].setText("");
    this.dTextArea[11].setText("");
    this.dTextArea[12].setText("");
    this.dTextArea[13].setText("");
    this.dTextArea[14].setText("");

    this.dTextArea[0].setVisible(false);
    this.dTextArea[1].setVisible(false);
    this.dTextArea[2].setVisible(false);
    this.dTextArea[3].setVisible(false);
    this.dTextArea[4].setVisible(false);
    this.dTextArea[5].setVisible(false);
    this.dTextArea[6].setVisible(false);
    this.dTextArea[7].setVisible(false);
    this.dTextArea[8].setVisible(false);
    this.dTextArea[9].setVisible(false);
    this.dTextArea[10].setVisible(false);
    this.dTextArea[11].setVisible(false);
    this.dTextArea[12].setVisible(false);
    this.dTextArea[13].setVisible(false);
    this.dTextArea[14].setVisible(false);
    loadtext();
    this.saveButton.setEnabled(true);
    this.dnadataPanel2.setText("Click here to select the Q file");
    this.dnadataPanel2.setText1("");
    this.dnadataPanel2.setText2("");
    this.dnadataPanel2.setProcIndex(0);
    this.dnadataPanel2.setSPlateIndex(0);
    this.dnadataPanel2.setNPlateIndex(0);
    this.dnadataPanel2.setDPlateIndex(0);
    this.dnadataPanel2.setPlateIndex(0);
    this.dnadataPanel2.setDeckIndex1(0);
    this.dnadataPanel2.setDeckIndex2(0);
    this.dnadataPanel2.setDeckIndex3(0);
    this.dnadataPanel2.setDeckIndex4(0);

    for (int t = 0; t < this.tabcount; t++)
    {
        this.jTabbedPane1.setEnabledAt(t, true);
    }
    pkEnableDisable();

    //Return to Splash screen
    Splash();
  }

  public void addLiconic()
  {
    this.counter = 0;
    nonPK = nonPK + 1;
    pagecount = (int) ((Math.ceil(nonPK/15.0)) - 1);		//APF0028
    String output = " ";
    String data = "";

    data = data + "   \n\n";
    data = data + "Liconic " + this.licloc + " to " + this.licdeckloc + "\n\n";

    String type = "LIC";

    int line3 = lic_fromto;
    int line6 = lic_cover;

    this.dList.insertAtBack(data);
    this.tList.insertAtBack(type);

    this.fList.insertAtBack("step");
    this.fList.insertAtBack("Liconic01");
    this.fList.insertAtBack(line3);
    this.fList.insertAtBack(this.licloc);
    this.fList.insertAtBack(this.licdeckloc);
    this.fList.insertAtBack(line6);
    this.fList.insertAtBack(this.lic_plate);
    this.fList.insertAtBack(this.lic_tsp);
    this.fList.insertAtBack("-");
    this.fList.insertAtBack("-");

    this.dTextArea[0].setText("");
    this.dTextArea[1].setText("");
    this.dTextArea[2].setText("");
    this.dTextArea[3].setText("");
    this.dTextArea[4].setText("");
    this.dTextArea[5].setText("");
    this.dTextArea[6].setText("");
    this.dTextArea[7].setText("");
    this.dTextArea[8].setText("");
    this.dTextArea[9].setText("");
    this.dTextArea[10].setText("");
    this.dTextArea[11].setText("");
    this.dTextArea[12].setText("");
    this.dTextArea[13].setText("");
    this.dTextArea[14].setText("");

    this.dTextArea[0].setVisible(false);
    this.dTextArea[1].setVisible(false);
    this.dTextArea[2].setVisible(false);
    this.dTextArea[3].setVisible(false);
    this.dTextArea[4].setVisible(false);
    this.dTextArea[5].setVisible(false);
    this.dTextArea[6].setVisible(false);
    this.dTextArea[7].setVisible(false);
    this.dTextArea[8].setVisible(false);
    this.dTextArea[9].setVisible(false);
    this.dTextArea[10].setVisible(false);
    this.dTextArea[11].setVisible(false);
    this.dTextArea[12].setVisible(false);
    this.dTextArea[13].setVisible(false);
    this.dTextArea[14].setVisible(false);
    loadtext();
    this.saveButton.setEnabled(true);
    this.licodatapanel2.setText1("");
    this.licodatapanel2.setText2("");
    this.licodatapanel2.setTSPIndex(1);
    this.licodatapanel2.setPlateIndex(0);

    for (int t = 0; t < this.tabcount; t++)
    {
        this.jTabbedPane1.setEnabledAt(t, true);
    }
    pkEnableDisable();

    //Return to Splash screen
    Splash();
  }

  
	  private void loadtext()
  {
    ListNode currentDList = this.dList.getFirstNode();
    ListNode currentTList = this.tList.getFirstNode();
    String tl = null;
    String stepNumbering;
    String indent;
    this.counter = 0;
    boolean done = false;
      
    while (currentDList != null)
    {
    
      stepNumbering = String.valueOf((this.counter + 1)+(15*(this.pagecount)));
      if ((this.pagecount > 0) && (!done))
      {
        for (int i = 0; i < this.pagecount * 15; i++)
        {
          if (currentDList != null)
          {
            currentDList = currentDList.next;
          }
          if (currentTList == null)
            continue;
          currentTList = currentTList.next;
        }

        done = true;
      }

      if ((this.counter < 15) && (currentTList != null))
      {
        tl = "";
        tl = currentTList.data.toString().trim();

        if (this.counter > 0)
        {
          this.menuItemCut.setEnabled(true);
        }
        
        if (Integer.parseInt(stepNumbering) < 10){indent = "         ";}else{indent = "         ";}
        this.dTextArea[this.counter].setText(indent + stepNumbering + currentDList.data.toString());   
        this.dTextArea[this.counter].setCaretPosition(0);

        if (tl.equals("REG")) {
          this.dTextArea[this.counter].setBackground(this.regColor);
          this.dTextArea[this.counter].setCaretPosition(0);
          this.dTextArea[this.counter].insertIcon(this.icons[7]);
        }
        if (tl.equals("INC")) {
          this.dTextArea[this.counter].setBackground(this.incColor);
          this.dTextArea[this.counter].setCaretPosition(0);
          this.dTextArea[this.counter].insertIcon(this.icons[25]);
        }
        if (tl.equals("SER")) {
            this.dTextArea[this.counter].setBackground(this.regColor);
            this.dTextArea[this.counter].setCaretPosition(0);
            this.dTextArea[this.counter].insertIcon(this.icons[27]);
        }
        if (tl.equals("BRW")) {
          this.dTextArea[this.counter].setBackground(this.brewColor);
        }
        if (tl.equals("WSH")) {
          this.dTextArea[this.counter].setBackground(this.wshColor);
          this.dTextArea[this.counter].setCaretPosition(0);
          this.dTextArea[this.counter].insertIcon(this.icons[24]);
        }
        if (tl.equals("RDR")) {
          this.dTextArea[this.counter].setBackground(this.readColor);
          this.dTextArea[this.counter].setCaretPosition(0);
          this.dTextArea[this.counter].insertIcon(this.icons[26]);
        }
        if (tl.equals("SPE")) {
          this.dTextArea[this.counter].setBackground(this.speColor);
          this.dTextArea[this.counter].setCaretPosition(0);
          this.dTextArea[this.counter].insertIcon(this.icons[28]);
        }
        if (tl.equals("MOV")) {
          this.dTextArea[this.counter].setBackground(this.movColor);
        }
        if (tl.equals("MOVPL")) {
          this.dTextArea[this.counter].setBackground(this.movplColor);
          this.dTextArea[this.counter].setCaretPosition(0);
          this.dTextArea[this.counter].insertIcon(this.icons[8]);
        }
        if (tl.equals("MVP2")) {
            this.dTextArea[this.counter].setBackground(this.movplColor);
            this.dTextArea[this.counter].setCaretPosition(0);
            this.dTextArea[this.counter].insertIcon(this.icons[8]);
        }
        if (tl.equals("PAU")) {
          this.dTextArea[this.counter].setBackground(this.pauColor);
          this.dTextArea[this.counter].setCaretPosition(0);
          this.dTextArea[this.counter].insertIcon(this.icons[20]);
        }
        if (tl.equals("CHR")) {
          this.dTextArea[this.counter].setBackground(this.pauColor);
          this.dTextArea[this.counter].setCaretPosition(0);
          this.dTextArea[this.counter].insertIcon(this.icons[19]);
        }
        if (tl.equals("CHR2")) {
            this.dTextArea[this.counter].setBackground(this.pauColor);
            this.dTextArea[this.counter].setCaretPosition(0);
            this.dTextArea[this.counter].insertIcon(this.icons[19]);
        }
        if (tl.equals("CYT")) {
            this.dTextArea[this.counter].setBackground(this.pauColor);
            this.dTextArea[this.counter].setCaretPosition(0);
            this.dTextArea[this.counter].insertIcon(this.icons[29]);
        }
        if (tl.equals("LIC")) {
            this.dTextArea[this.counter].setBackground(this.pauColor);
            this.dTextArea[this.counter].setCaretPosition(0);
            this.dTextArea[this.counter].insertIcon(this.icons[29]);
        }
        if (tl.equals("PKS")) {
        	this.dTextArea[this.counter].setText("\n\nPK SAMPLE PREP");
            this.dTextArea[this.counter].setBackground(this.pauColor);
            this.dTextArea[this.counter].setCaretPosition(0);
            this.dTextArea[this.counter].insertIcon(this.icons[23]);
            this.dTextArea[this.counter].setToolTipText("<html>"
                    + "Click to edit the protocol"
                    +"<br><br>"
                    +"PK STEP<br>"
                    +"LIMS FILE:"
                    + currentDList.data.toString()
                    +"<br><br>"
               + "</html>");
          }

        if (tl.equals("DNA")) {
            this.dTextArea[this.counter].setBackground(this.pauColor);
            this.dTextArea[this.counter].setCaretPosition(0);
            this.dTextArea[this.counter].insertIcon(this.icons[29]);
        }

        if (tl.equals("LOAD01")) {
        	
            this.dTextArea[this.counter].setBackground(this.pauColor);
            this.dTextArea[this.counter].setCaretPosition(0);
            this.dTextArea[this.counter].insertIcon(this.icons[32]);
        }

        this.dTextArea[this.counter].setEditable(false);
        this.dTextArea[this.counter].setFont(this.font);
        this.dTextArea[this.counter].setOpaque(false);
        this.dTextArea[this.counter].setBackground(new Color(0, 0, 0, 0));

        this.dTextArea[this.counter].setVisible(true);

        this.counter += 1;
      }

      if (currentDList != null)
      {
        currentDList = currentDList.next;
      }
      if (currentTList == null)
        continue;
      currentTList = currentTList.next;
    }
    
    double totalPages = Math.ceil(nonPK/15.0);																//APF0028
    double currentPage = pagecount+1;																		//APF0028
    if (currentPage > 1){arrowButton1.setEnabled(true);}else{arrowButton1.setEnabled(false);}				//APF0028
    if (totalPages > currentPage){arrowButton2.setEnabled(true);}else{arrowButton2.setEnabled(false);}		//APF0028
    
  }

  private void updText(String data)
  {
	String indent;
    if (this.stepnbr < 10){indent = "           ";}else{indent = "         ";}
    data = indent + stepnbr +  data;   //Insert step number into step graphic
    
    if (this.stepnbr == ((this.pagecount*15)+1) ){															//APF0027 Replaced previous logic
      this.dTextArea[0].setText(data);
      this.dTextArea[0].setCaretPosition(2);
    }
    if (this.stepnbr == ((this.pagecount*15)+2) ){															//APF0027
      this.dTextArea[1].setText(data);
    }
    if (this.stepnbr == ((this.pagecount*15)+3) ){															//APF0027
      this.dTextArea[2].setText(data);
    }
    if (this.stepnbr == ((this.pagecount*15)+4) ){															//APF0027
      this.dTextArea[3].setText(data);
    }
    if (this.stepnbr == ((this.pagecount*15)+5) ){															//APF0027
      this.dTextArea[4].setText(data);
    }
    if (this.stepnbr == ((this.pagecount*15)+6) ){															//APF0027
      this.dTextArea[5].setText(data);
    }
    if (this.stepnbr == ((this.pagecount*15)+7) ){															//APF0027
      this.dTextArea[6].setText(data);
    }
    if (this.stepnbr == ((this.pagecount*15)+8) ){															//APF0027
      this.dTextArea[7].setText(data);
    }
    if (this.stepnbr == ((this.pagecount*15)+9) ){															//APF0027
      this.dTextArea[8].setText(data);
    }
    if (this.stepnbr == ((this.pagecount*15)+10) ){															//APF0027
      this.dTextArea[9].setText(data);
    }
    if (this.stepnbr == ((this.pagecount*15)+11) ){															//APF0027
      this.dTextArea[10].setText(data);
    }
    if (this.stepnbr == ((this.pagecount*15)+12) ){															//APF0027
      this.dTextArea[11].setText(data);
    }
    if (this.stepnbr == ((this.pagecount*15)+13) ){															//APF0027
      this.dTextArea[12].setText(data);
    }
    if (this.stepnbr == ((this.pagecount*15)+14) ){															//APF0027
      this.dTextArea[13].setText(data);
    }
    if (this.stepnbr == ((this.pagecount*15)+15) ){															//APF0027
      this.dTextArea[14].setText(data);
    }
    
    double totalPages = Math.ceil(nonPK/15.0);																//APF0028
    double currentPage = pagecount+1;																		//APF0028
    if (currentPage > 1){arrowButton1.setEnabled(true);}else{arrowButton1.setEnabled(false);}				//APF0028
    if (totalPages > currentPage){arrowButton2.setEnabled(true);}else{arrowButton2.setEnabled(false);}		//APF0028

  }

  private void rmvAnyLink(int v)
  {
    ListNode currentFList = this.fList.getFirstNode();
    ListNode currentDList = this.dList.getFirstNode();
    ListNode currentTList = this.tList.getFirstNode();
    ListNode currentBList = this.bList.getFirstNode();
    ListNode currentEList = this.eList.getFirstNode();
    ListNode currentDBList = this.dbList.getFirstNode();
    ListNode currentDEList = this.deList.getFirstNode();
    ListNode currentTBList = this.tbList.getFirstNode();
    ListNode currentTEList = this.teList.getFirstNode();
    int val = 0;

    while (!this.bList.isEmpty())
    {
      this.bList.removeFromBack();
    }
    while (!this.eList.isEmpty())
    {
      this.eList.removeFromBack();
    }
    while (!this.dbList.isEmpty())
    {
      this.dbList.removeFromBack();
    }
    while (!this.deList.isEmpty())
    {
      this.deList.removeFromBack();
    }
    while (!this.tbList.isEmpty())
    {
      this.tbList.removeFromBack();
    }
    while (!this.teList.isEmpty())
    {
      this.teList.removeFromBack();
    }

    int count = 0;
    val = v;

    if (val > 0) {
      if (val == 1)
      {
        count = 0;
        while (currentFList != null)
        {
          count++;
          if (count > val * 10) {
            this.eList.insertAtBack(currentFList.data.toString());
          }
          currentFList = currentFList.next;
        }

        currentDList = this.dList.getFirstNode();

        count = 0;
        while (currentDList != null)
        {
          count++;
          if (count > val) {
            this.deList.insertAtBack(currentDList.data.toString());
          }
          currentDList = currentDList.next;
        }
        count = 0;
        if (currentTList != null)
        {
        String tl = "";
        tl = currentTList.data.toString().trim();

          if (tl.equals("REG")) {
      	    nonPK = nonPK - 1;
          }
          if (tl.equals("INC")) {
      	    nonPK = nonPK - 1;
          }
          if (tl.equals("SER")) {
      	    nonPK = nonPK - 1;
          }
          if (tl.equals("BRW")) {
      	    nonPK = nonPK - 1;
          }
          if (tl.equals("WSH")) {
      	  nonPK = nonPK - 1;
          }
          if (tl.equals("RDR")) {
      	    nonPK = nonPK - 1;
          }
          if (tl.equals("SPE")) {
        	  nonPK = nonPK - 1;
          }
          if (tl.equals("MOV")) {
        	  nonPK = nonPK - 1;
          }
          if (tl.equals("MOVPL")) {
        	  nonPK = nonPK - 1;
          }
          if (tl.equals("PAU")) {
        	  nonPK = nonPK - 1;
          }
          if (tl.equals("CHR")) {
        	  nonPK = nonPK - 1;
          }
          if (tl.equals("PKS")) {
        	  cntPK = cntPK - 1;
          }
          if (tl.equals("DNA")) {
        	  cntPK = cntPK - 1;
          }
          
          pagecount = (int) ((Math.ceil(nonPK/15.0)) - 1);		//APF0028
        
        }
        while (currentTList != null)
        {
          count++;
          if (count > val) {
            this.teList.insertAtBack(currentTList.data.toString());
          }
          currentTList = currentTList.next;
        }

      }

      if (val > 1)
      {
        count = 0;

        while (currentFList != null)
        {
          count++;
          if (count <= (val - 1) * 10) {
            this.bList.insertAtBack(currentFList.data.toString());
          }
          if (count > val * 10) {
            this.eList.insertAtBack(currentFList.data.toString());
          }
          currentFList = currentFList.next;
        }

        currentDList = this.dList.getFirstNode();

        count = 0;

        while (currentDList != null)
        {
          count++;
          if (count < val) {
            this.dbList.insertAtBack(currentDList.data.toString());
          }
          if (count > val) {
            this.deList.insertAtBack(currentDList.data.toString());
          }
          currentDList = currentDList.next;
        }

        currentTList = this.tList.getFirstNode();

        count = 0;

        while (currentTList != null)
        {
          count++;
          if (count < val) {
            this.tbList.insertAtBack(currentTList.data.toString());
          }
          if (count > val) {
            this.teList.insertAtBack(currentTList.data.toString());
          }

          if (count == val) {
          String tl = "";
          tl = currentTList.data.toString().trim();

          if (tl.equals("REG")) {
        	  nonPK = nonPK - 1;
          }
          if (tl.equals("INC")) {
        	  nonPK = nonPK - 1;
          }
          if (tl.equals("SER")) {
        	  nonPK = nonPK - 1;
          }
          if (tl.equals("BRW")) {
        	  nonPK = nonPK - 1;
          }
          if (tl.equals("WSH")) {
        	  nonPK = nonPK - 1;
          }
          if (tl.equals("RDR")) {
        	  nonPK = nonPK - 1;
          }
          if (tl.equals("SPE")) {
        	  nonPK = nonPK - 1;
          }
          if (tl.equals("MOV")) {
        	  nonPK = nonPK - 1;
          }
          if (tl.equals("MOVPL")) {
        	  nonPK = nonPK - 1;
          }
          if (tl.equals("PAU")) {
        	  nonPK = nonPK - 1;
          }
          if (tl.equals("CHR")) {
        	  nonPK = nonPK - 1;
          }
          if (tl.equals("PKS")) {
        	  cntPK = cntPK - 1;
            }
          if (tl.equals("DNA")) {
        	  cntPK = cntPK - 1;
          }
          
          pagecount = (int) ((Math.ceil(nonPK/15.0)) - 1);		//APF0028
          
          } // end of current type 
          
          currentTList = currentTList.next;
        }

      }

    }

    while (!this.fList.isEmpty())
    {
      this.fList.removeFromBack();
    }
    while (!this.dList.isEmpty())
    {
      this.dList.removeFromBack();
    }
    while (!this.tList.isEmpty())
    {
      this.tList.removeFromBack();
    }

    currentBList = this.bList.getFirstNode();
    while (currentBList != null)
    {
      this.fList.insertAtBack(currentBList.data.toString());
      currentBList = currentBList.next;
    }

    while (!this.bList.isEmpty())
    {
      this.bList.removeFromBack();
    }

    currentEList = this.eList.getFirstNode();
    while (currentEList != null)
    {
      this.fList.insertAtBack(currentEList.data.toString());
      currentEList = currentEList.next;
    }
    while (!this.eList.isEmpty())
    {
      this.eList.removeFromBack();
    }

    currentDBList = this.dbList.getFirstNode();
    while (currentDBList != null)
    {
      this.dList.insertAtBack(currentDBList.data.toString());
      currentDBList = currentDBList.next;
    }
    while (!this.dbList.isEmpty())
    {
      this.dbList.removeFromBack();
    }

    currentDEList = this.deList.getFirstNode();
    while (currentDEList != null)
    {
      this.dList.insertAtBack(currentDEList.data.toString());
      currentDEList = currentDEList.next;
    }
    while (!this.deList.isEmpty())
    {
      this.deList.removeFromBack();
    }

    currentTBList = this.tbList.getFirstNode();
    while (currentTBList != null)
    {
      this.tList.insertAtBack(currentTBList.data.toString());
      currentTBList = currentTBList.next;
    }
    while (!this.tbList.isEmpty())
    {
      this.tbList.removeFromBack();
    }

    currentTEList = this.teList.getFirstNode();
    while (currentTEList != null)
    {
      this.tList.insertAtBack(currentTEList.data.toString());
      currentTEList = currentTEList.next;
    }
    while (!this.teList.isEmpty())
    {
      this.teList.removeFromBack();
    }

    this.dTextArea[0].setText("");
    this.dTextArea[1].setText("");
    this.dTextArea[2].setText("");
    this.dTextArea[3].setText("");
    this.dTextArea[4].setText("");
    this.dTextArea[5].setText("");
    this.dTextArea[6].setText("");
    this.dTextArea[7].setText("");
    this.dTextArea[8].setText("");
    this.dTextArea[9].setText("");
    this.dTextArea[10].setText("");
    this.dTextArea[11].setText("");
    this.dTextArea[12].setText("");
    this.dTextArea[13].setText("");
    this.dTextArea[14].setText("");

    this.dTextArea[0].setVisible(false);
    this.dTextArea[1].setVisible(false);
    this.dTextArea[2].setVisible(false);
    this.dTextArea[3].setVisible(false);
    this.dTextArea[4].setVisible(false);
    this.dTextArea[5].setVisible(false);
    this.dTextArea[6].setVisible(false);
    this.dTextArea[7].setVisible(false);
    this.dTextArea[8].setVisible(false);
    this.dTextArea[9].setVisible(false);
    this.dTextArea[10].setVisible(false);
    this.dTextArea[11].setVisible(false);
    this.dTextArea[12].setVisible(false);
    this.dTextArea[13].setVisible(false);
    this.dTextArea[14].setVisible(false);
    loadtext();
    if (this.updButton1 != null)
    {
      this.updButton1.setEnabled(false);
      this.checkButton.setEnabled(false);
    }

    if (this.updButton2 != null)
    {
      this.updButton2.setEnabled(false);
    }

    if (this.updButton3 != null)
    {
      this.updButton3.setEnabled(false);
    }

    if (this.updButton4 != null)
    {
      this.updButton4.setEnabled(false);
    }

    if (this.updButton5 != null)
    {
      this.updButton5.setEnabled(false);
    }

    if (this.updButton6 != null)
    {
      this.updButton6.setEnabled(false);
    }

    if (this.updButton7 != null)
    {
      this.updButton7.setEnabled(false);
    }

    if (this.updButton8 != null)
    {
      this.updButton8.setEnabled(false);
    }

    if (this.updButton9 != null)
    {
      this.updButton9.setEnabled(false);
    }

    if (this.updButton10 != null)
    {
      this.updButton10.setEnabled(false);
    }
    if (this.btnUpdateStep != null)
    {
     this.btnUpdateStep.setEnabled(false);
    }

    if (this.addButton1 != null)
    {
      this.addButton1.setEnabled(true);
      this.addButton.setEnabled(true);
    }

    if (this.addButton2 != null)
    {
      this.addButton2.setEnabled(true);
    }

    if (this.addButton3 != null)
    {
      this.addButton3.setEnabled(true);
    }

    if (this.addButton4 != null)
    {
      this.addButton4.setEnabled(true);
    }

    if (this.addButton5 != null)
    {
      this.addButton5.setEnabled(true);
    }

    if (this.addButton6 != null)
    {
      this.addButton6.setEnabled(true);
    }

    if (this.addButton7 != null)
    {
      this.addButton7.setEnabled(true);
    }

    if (this.addButton8 != null)
    {
      this.addButton8.setEnabled(true);
    }

    if (this.addButton9 != null)
    {
      this.addButton9.setEnabled(true);
    }

    if (this.addButton10 != null)
    {
      this.addButton10.setEnabled(true);
    }

    if (this.btnAdd != null)
    {
        this.btnAdd.setEnabled(true);
    }

    this.rmvButton.setEnabled(false);
    this.saveButton.setEnabled(true);
    this.stepnum.setText("");
    this.update = false;
    this.addButton1.setEnabled(true);
    this.addButton.setEnabled(true);
    this.checkButton.setEnabled(false);
    for (int t = 0; t < this.tabcount; t++)
    {
        this.jTabbedPane1.setEnabledAt(t, true);
    }
    pkEnableDisable();
    int tindex = tabSearch(this.tabs, "Liquid Transfer");
    if (tindex > -1)
    {
      this.jTabbedPane1.setSelectedIndex(tindex);
    }
    
    double totalPages = Math.ceil(nonPK/15.0);																//APF0028
    double currentPage = pagecount+1;																		//APF0028
    if (currentPage > 1){arrowButton1.setEnabled(true);}else{arrowButton1.setEnabled(false);}				//APF0028
    if (totalPages > currentPage){arrowButton2.setEnabled(true);}else{arrowButton2.setEnabled(false);}		//APF0028
    
    resetScreens();
  }

  private String saveVASi()
  {
	  ///////////////////////////////////////////////////////////////////
	  //Executes VAS using an intermediate file in the System directory
	  
    String VASerror = "";
    
    File f = new File("C:\\Program Files\\Telos\\AutoPilot\\System\\VASi.dat");
	
    if (this.layoutFileSelected == "") {
      JOptionPane.showMessageDialog(null, "Please select valid Deck Layout File");
    }
    else {
            
    	String path = f.getAbsolutePath();
        try
        {
            BufferedWriter bw = new BufferedWriter(new FileWriter(f));
            bw.write(this.layoutFileSelected);
            bw.newLine();
            bw.write("start");
            bw.newLine();
            bw.write(this.hline3);
            bw.newLine();
            bw.write(this.hline4);
            bw.newLine();
            bw.write(this.hline5);
            bw.newLine();
            bw.write(this.hline6);
            bw.newLine();
            bw.write(this.hline7);
            bw.newLine();
            bw.write(this.hline8);
            bw.newLine();
            bw.write(this.hline9);
            bw.newLine();
            bw.write(this.security_auditTrailString);
            bw.newLine();

            ListNode currentFList = this.fList.getFirstNode();
            while (currentFList != null)
            {
              bw.write(currentFList.data.toString());
              bw.newLine();
              currentFList = currentFList.next;
            }

            bw.write("end");
            bw.newLine();
            bw.close();
        }
          catch (FileNotFoundException e1) {
            e1.printStackTrace();
        } catch (IOException e1) {
            e1.printStackTrace();
        }
          
        VASerror = VASengine.GO(path);

        if (VASerror.equals("")){
 
    	}
        else {
            JOptionPane.showMessageDialog(null, "VAS has determined that there is an error:\n" + VASerror + "\n");
    	}
        }
    Splash();
    return VASerror;
   
}
  
  private boolean saveFile(File f)
  {
	  
    String VASerror = "";
    if (this.layoutFileSelected == "") {
      JOptionPane.showMessageDialog(null, "Please select valid Deck Layout File");
    }
    else {
            
    	String path = f.getAbsolutePath();
    
        String extension = ".apr";
        String t1 = "";
        t1 = this.temp.getText().trim();
        if (t1.equals(""))
        {
          t1 = "0";
        }

        if (this.readCount == 0)
        {
          this.hline3 = "-";
          this.hline4 = "-";
        }
        else
        {
          this.hline3 = (this.r1.trim() + "," + t1);
          this.hline4 = this.text6.getText().trim();
          if (this.hline4.trim().equals(""))
          {
            this.hline4 = "-";
          }
        }

        if (!path.endsWith(extension))
        {
          path = path + extension;
        }
        f = new File(path);
        
        if (f.exists())
        {
        	if (saveAs == true){
        		int answer = JOptionPane.showConfirmDialog(
	            this, f + " exists. Overwrite?", "Overwrite?", 
	            2, 3);
	
        		if (answer != 0)
	          		{
        			return true;
	          		}
        		
            	if (security){
            		if (security_auditTrailString.contains("AT0") || security_auditTrailString.contains("AT1")){
            			security_auditTrailString = security_auditTrailString.replace("AT1", "AT0");	//1 indicates that protocol is valid	
            		}
    	      		security_Validated = false;
    	      		security_validationTitle = "Not Validated";
    	      		security_updateString = "SaveAs Protocol";
            	}	
        	}
        	
        }//end file exists

        if (security){
        	if (security_accessLevel.equals("1") && security_Validated){								//APN0051  Do Not Get Audit Trail input if this is a validated method being run by operator
        		//APN0051  Do Not Get Audit Trail input if this is a validated method being run by operator
        	}else{
        		JTextArea area = new JTextArea(5,20);
				area.setLineWrap(true);
				int okCxl = JOptionPane.showConfirmDialog(
						null, 
						area, 
						"Description of Modification", 
						JOptionPane.OK_CANCEL_OPTION, 
						JOptionPane.PLAIN_MESSAGE, 
						null);
	
				if (okCxl == JOptionPane.CANCEL_OPTION  ||  area.getText().equals("")) {
					JOptionPane.showMessageDialog(null, "You must provide a description of the current modification to continue."); 
					return false;
				}
			
				String auditInput = area.getText().replace("\n", "").replace("\r", "").replace(",", "").replace(";", "").replace("\"", "");  //APN0057 remove newlines //APN0059 and punctuation
	        	if (security_updateString.equals("")){security_updateString = "Save Protocol";}
	            //Update Audit Trail if security is activated
	      		auditUpdate(security_updateString, auditInput);
	      		security_updateString = "";							//Reset the update string.       		
        	}
        }
        
        
        try
        {
            BufferedWriter bw = new BufferedWriter(new FileWriter(f));
            bw.write(this.layoutFileSelected);
            bw.newLine();
            bw.write("start");
            bw.newLine();
            bw.write(this.hline3);
            bw.newLine();
            bw.write(this.hline4);
            bw.newLine();
            bw.write(this.hline5);
            bw.newLine();
            bw.write(this.hline6);
            bw.newLine();
            bw.write(this.hline7);
            bw.newLine();
            bw.write(this.hline8);
            bw.newLine();
            bw.write(this.hline9);
            bw.newLine();
            bw.write(this.security_auditTrailString);
            bw.newLine();

            ListNode currentFList = this.fList.getFirstNode();
            while (currentFList != null)
            {
              bw.write(currentFList.data.toString());
              bw.newLine();
              currentFList = currentFList.next;
            }

            bw.write("end");
            bw.newLine();
            bw.close();
        }
          catch (FileNotFoundException e1) {
            e1.printStackTrace();
        } catch (IOException e1) {
            e1.printStackTrace();
        }
          
           
        VASerror = VASengine.GO(path);
          
        }

    setTitle(APversion + " - " + String.valueOf(f) + "     " + security_validationTitle);
    this.aprActive = String.valueOf(f);
    Splash();
    return true;
}

  private void clearData()
  {
	  
	if (security){
		security_auditTrailString = "-";
		security_pKWorklist_Runtime = "";
	}
	
    setTitle(APversion);

    this.aprActive = "";

    this.layoutSelectButton.setIcon(this.icons[17]);
    this.layoutSelectButton.setToolTipText("Select the Deck Layout for this protocol");

    this.layoutFileSelected = "";
    
    arrowButton1.setEnabled(false);					//APF0028
    arrowButton2.setEnabled(false);
   
    arrayList_sdVolumesTransfer.clear();
    arrayList_sdVolumesDiluent.clear();
    
    while (!this.fList.isEmpty())
    {
      this.fList.removeFromBack();
    }

    while (!this.tList.isEmpty())
    {
      this.tList.removeFromBack();
    }

    while (!this.dList.isEmpty())
    {
      this.dList.removeFromBack();
    }

    while (!this.bList.isEmpty())
    {
      this.bList.removeFromBack();
    }
    while (!this.eList.isEmpty())
    {
      this.eList.removeFromBack();
    }
    while (!this.dbList.isEmpty())
    {
      this.dbList.removeFromBack();
    }
    while (!this.deList.isEmpty())
    {
      this.deList.removeFromBack();
    }
    while (!this.tbList.isEmpty())
    {
      this.tbList.removeFromBack();
    }
    while (!this.teList.isEmpty())
    {
      this.teList.removeFromBack();
    }

    this.dTextArea[0].setText("");
    this.dTextArea[1].setText("");
    this.dTextArea[2].setText("");
    this.dTextArea[3].setText("");
    this.dTextArea[4].setText("");
    this.dTextArea[5].setText("");
    this.dTextArea[6].setText("");
    this.dTextArea[7].setText("");
    this.dTextArea[8].setText("");
    this.dTextArea[9].setText("");
    this.dTextArea[10].setText("");
    this.dTextArea[11].setText("");
    this.dTextArea[12].setText("");
    this.dTextArea[13].setText("");
    this.dTextArea[14].setText("");

    this.dTextArea[0].setVisible(false);
    this.dTextArea[1].setVisible(false);
    this.dTextArea[2].setVisible(false);
    this.dTextArea[3].setVisible(false);
    this.dTextArea[4].setVisible(false);
    this.dTextArea[5].setVisible(false);
    this.dTextArea[6].setVisible(false);
    this.dTextArea[7].setVisible(false);
    this.dTextArea[8].setVisible(false);
    this.dTextArea[9].setVisible(false);
    this.dTextArea[10].setVisible(false);
    this.dTextArea[11].setVisible(false);
    this.dTextArea[12].setVisible(false);
    this.dTextArea[13].setVisible(false);
    this.dTextArea[14].setVisible(false);
    this.hline3 = "-";
    this.hline4 = "-";
    this.hline5 = "-";
    this.hline6 = "-";
    this.hline7 = "-";
    this.hline8 = "-";
    this.hline9 = "-";
    this.hline10 = "-";
    this.readCount = 0;
    for (int t = 0; t < this.tabcount; t++)
    {
        this.jTabbedPane1.setEnabledAt(t, true);
    }
    nonPK = 0;
    cntPK = 0;
    resetScreens();
  }

  private void resetScreens()
  {
	this.pausedataPanel2.setMinutes(" ");		//APN0057
	this.pausedataPanel2.setSeconds(" ");		//APN0057
	this.pausedataPanel2.setMessage(" ");		//APN0057
	this.pausedataPanel2.setTimed(false);		//APN0057
	    
    this.text2.setText("");
    this.text3.setText("");
    this.lchill.setSelected(false);
    this.lmix.setSelected(false);
    this.lmix2.setSelected(false);
    this.lshake.setSelected(false);
//    this.linc1button.setVisible(false);
//    this.linc2button.setVisible(false);
//    this.linc3button.setVisible(false);
//    this.linc4button.setVisible(false);
//    this.linc5button.setVisible(false);
//    this.linc6button.setVisible(false);
    this.menuItemPaste.setEnabled(false);
    this.menuItemCopy.setEnabled(true);
    this.menuItemCut.setEnabled(true);

    if (this.source != null)
    {
      this.source.setSelectedIndex(0);
      getSrc();
      this.source.revalidate();
      this.source.repaint();
    }
    if (this.dest != null)
    {
      this.dest.setSelectedIndex(0);
      getDest();
      this.dest.revalidate();
      this.dest.repaint();
    }
    if (this.splatetype != null)
    {
      this.splatetype.setSelectedIndex(0);
      this.splatetype.setVisible(true);
      this.splatetype.revalidate();
      this.splatetype.repaint();
    }
    if (this.dplatetype != null)
    {
      this.dplatetype.setSelectedIndex(0);
      this.dplatetype.setVisible(true);
      this.dplatetype.revalidate();
      this.dplatetype.repaint();
    }
    this.platebutton.setSelected(true);
    this.platebutton2.setSelected(true);
    this.brtubepanel.setFirstTube(" ");
    this.brtubepanel.setLastTube(" ");
    this.brtubepanel2.setFirstTube(" ");
    this.brtubepanel2.setLastTube(" ");
    this.panel96l.clearTray();
    this.panel96r.clearTray();
    this.liqPanel1.removeAll();
    this.liqPanel2.removeAll();
    this.s1inc.setVisible(false);
    this.s2inc.setVisible(false);
    this.sttext.setText("");
    this.dltext.setText("");
    if (this.liqPanel1 != null)
    {
      this.liqPanel1.add(getLiqPanel1Top(), "North");
      this.liqPanel1.add(this.panelbl, "Center");
      this.liqPanel1.add(getLiqPanel1Bot(), "South");
      this.liqPanel1.revalidate();
      this.liqPanel1.repaint();
    }
    if (this.liqPanel2 != null)
    {
      this.liqPanel2.add(getLiqPanel2Top(), "North");
      this.liqPanel2.add(this.panelbr, "Center");
      this.liqPanel2.add(getLiqPanel2Bot(), "South");
      this.liqPanel2.revalidate();
      this.liqPanel2.repaint();
    }

    if (this.source2 != null)
    {
      this.source2.setSelectedIndex(0);
      getSrc2();
      this.source2.revalidate();
      this.source2.repaint();
    }
    if (this.dest2 != null)
    {
      this.dest2.setSelectedIndex(0);
      getDest2();
      this.dest2.revalidate();
      this.dest2.repaint();
    }
    if (this.splatetype2 != null)
    {
      this.splatetype2.setSelectedIndex(0);
      this.splatetype2.revalidate();
      this.splatetype2.setVisible(true);
      this.splatetype2.repaint();
    }
    if (this.dplatetype2 != null)
    {
      this.dplatetype2.setSelectedIndex(0);
      this.dplatetype2.revalidate();
      this.dplatetype2.setVisible(true);
      this.dplatetype2.repaint();
    }
    if (this.source3 != null)
    {
      getSrc3();
      this.source3.revalidate();
      this.source3.repaint();
    }
    if (this.serPanel1 != null)
    {
      this.serPanel1.removeAll();
      this.s3inc.setSelected(false);
      this.s4inc.setSelected(false);
      this.s3inc.setVisible(false);
      this.s4inc.setVisible(false);
      this.platebutton3.setSelected(true);
      this.serPanel1.add(getSerPanel1Top(), "North");
      this.serPanel1.add(this.spanel96l, "Center");
      this.serPanel1.revalidate();
      this.serPanel1.repaint();
    }

    if (this.source5 != null)
    {
      this.source5.setSelectedIndex(0);
      getSrc5();
      this.source5.revalidate();
      this.source5.repaint();
    }
    if (this.splatetype3 != null)
    {
      this.splatetype3.setSelectedIndex(0);
      this.splatetype3.revalidate();
      this.splatetype3.setVisible(true);
      this.splatetype3.repaint();
    }
    if (this.dplatetype3 != null)
    {
      this.dplatetype3.setSelectedIndex(0);
      this.dplatetype3.revalidate();
      this.dplatetype3.setVisible(true);
      this.dplatetype3.repaint();
    }
    if (this.movePlatePanel != null)
    {
      this.movePlatePanel.revalidate();
      this.movePlatePanel.repaint();
    }

    this.brvol.setText("");
    this.bpl1 = "";
    this.bpl2 = "";
    this.brtext.setText("");
    this.brntext.setText("");

    if (this.brewPanel1 != null)
    {
      this.brmix.setSelected(false);
      this.brchill.setSelected(false);
      this.brtubepanel.setFirstTube(" ");
      this.brtubepanel.setLastTube(" ");
      this.source4.setSelectedIndex(0);
      this.dest3.setSelectedIndex(0);
      this.brresbutton1.setSelected(true);
      this.brresbutton2.setSelected(true);
      this.brewPanel1.removeAll();
      getSrc4();
      this.source4.revalidate();
      this.source4.repaint();
      this.brewPanel1.add(getBrewPanel1Top(), "North");
      this.brewPanel1.add(this.brrespanel1, "Center");
      this.brewPanel1.add(getBrewPanel1Bot(), "South");
      this.brewPanel1.revalidate();
      this.brewPanel1.repaint();
    }
    if (this.brewPanel2 != null)
    {
      this.brewPanel2.removeAll();
      this.brtubepanel2.setFirstTube(" ");
      this.brtubepanel2.setLastTube(" ");
      getDest3();
      this.dest3.revalidate();
      this.dest3.repaint();
      this.brewPanel2.add(getBrewPanel2Top(), "North");
      this.brewPanel2.add(this.brrespanel2, "Center");
      this.brewPanel2.add(getBrewPanel2Bot(), "South");
      this.brewPanel2.revalidate();
      this.brewPanel2.repaint();
    }

    if (this.spePanel != null)
    {
      this.time.setText("");
      this.pressure.setText("");
      this.assembutton.setSelected(true);
      this.disassembutton.setSelected(false);
      this.vacbutton.setSelected(false);
//      getFilter();
//      getCollection();
//      this.filter.setVisible(true);
//      this.collection.setVisible(true);
//      this.spelabel1.setVisible(true);
//      this.spelabel2.setVisible(true);
      this.time.setVisible(false);
      this.pressure.setVisible(false);
      this.spelabel3.setVisible(false);
      this.spelabel4.setVisible(false);
      this.spePanel.revalidate();
      this.spePanel.repaint();
    }

    if (this.incPanel != null)
    {
      this.text4.setText(" ");
      this.text4pm.setText("0");
      this.itemp.setText("");
      this.incPanel.revalidate();
      this.incPanel.repaint();
    }

    if (this.readPanel != null)
    {
      this.text6.setText(" ");
      this.text6.setEditable(true);
      this.temp.setText(" ");
      this.readPanel.revalidate();
      this.readPanel.repaint();
    }

    if (this.cherryPanel != null)
    {
      this.text7.setText(" ");
      this.text7.setEditable(true);
      this.cherryPanel.revalidate();
      this.cherryPanel.repaint();
    }
    if (this.pkPanel != null)
    {
    	// **ADD
    	this.pkdataPanel2x.setpkmtextwat("");
        this.pkdataPanel2x.setpkmtext1("");
        this.pkdataPanel2x.setpkmtextul("");
        this.pkdataPanel2x.setmrdindex(0);
        this.pkdataPanel2x.setsamindex(0);
        this.pkdataPanel2x.setstdindex(0);
        this.pkdataPanel2x.setmrddilindex(0);
        this.pkdataPanel2x.setsamdilindex(0);
        this.pkdataPanel2x.setstddilindex(0);
        this.pkdataPanel2x.setdeck1index(0);
        this.pkdataPanel2x.setMRDon(false);
        this.pkdataPanel2x.setDILon(false);
        this.pkdataPanel2x.setdilsrcindex(0);
        this.pkdataPanel2x.setdilresindex(0);
        this.pkdataPanel2x.setdeck2index(0);
        this.pkdataPanel2x.setpkmpassayul("");
        this.pkdataPanel2x.setPRAon(false);
        this.pkdataPanel2x.setpassaylabindex(0);
        this.pkdataPanel2x.setASSon(false);
        this.pkdataPanel2x.setassaylabindex(0);
        this.pkdataPanel2x.panel_3_reset();						//APN0053
        this.pkPanel.revalidate();
        this.pkPanel.repaint();    		
    }
    //Return to Splash screen
    Splash();

//    this.jTabbedPane1.setSelectedIndex(0);
  }

  private boolean loadFile()
  {
    
    int state = getJFileChooser().showOpenDialog(this);
    this.jFileChooser.setFileSelectionMode(0);
    
    if (state == 0)//"Open" Selected so .apr will be loaded and function will return True;
    {
    	
    	clearData();
    	nonPK=0;
    	
      while (!this.fList.isEmpty())
      {
        this.fList.removeFromBack();
      }
      while (!this.dList.isEmpty())
      {
        this.dList.removeFromBack();
      }
      while (!this.tList.isEmpty())
      {
        this.tList.removeFromBack();
      }

      while (!this.loadList.isEmpty())
      {
        this.loadList.removeFromBack();
      }

      int index1 = 0;
      int index2 = 0;
      int index3 = 0;
      int index4 = 0;
      int index5 = 0;
      int index6 = 0;
      int index7 = 0;
      int index8 = 0;
      int len = 0;
      int count = 0;
      int loadcount = 0;
      String steptype = null;
      String loaddata = null;
      String addsrc = null;
      String serdest = null;
      String reagent = null;
      String brewreg = null;
      String brewnew = null;
      String volume = null;
      String blt1 = null;
      String serreg = null;
      String inc3 = null;
      String inc4 = null;
      String read1 = null;
      String reader = null;
      String ressrc = null;
      String mp1 = null;
      String mp2 = null;
      String temp = null;
      String m1 = null;
      String c1 = null;
      String sh = null;
      String w1 = null;
      String b1 = null;
      String v1 = null;
      File f = getJFileChooser().getSelectedFile();

      if ((f == null) || (f.getName().equals("")))
      {
        JOptionPane.showMessageDialog(this, 
          "Invalid File Name", 
          "Invalid File Name ", 
          0);
      }
      else
      {
        try {
          String data = "";
          this.input = new FileInputStream(f);
          this.loadButton.setEnabled(false);
          BufferedReader bw = new BufferedReader(new FileReader(f));
          String data2 = "";

          while (!data2.equals("end")) {
            data = bw.readLine();
            count++;
            data2 = data;

            if (count == 1) {
              this.layoutFileSelected = data;

              this.layoutSelectButton.setIcon(this.icons[18]);
              this.layoutSelectButton.setToolTipText(this.layoutFileSelected);
            }
            if(count == 10){
            	security_auditTrailString = data2;
            }
            if ((count <= 10) || (data2.equals("end")))
              continue;
            this.loadList.insertAtBack(data2);
          }
        }
        catch (IOException e) {
          JOptionPane.showMessageDialog(this, 
            "Error Opening File", 
            "Error", 
            0);
        }
        try
        {
          this.input.close();
        } catch (IOException e) {
          JOptionPane.showMessageDialog(this, 
            "Error Closing File", 
            "Error", 
            0);
        }

        if (security){
        	if (security_auditTrailString.contains("AT1")){
        		security_Validated = true;
        		security_validationTitle = "Validated";
        	}else{
        		security_Validated = false;
        		security_validationTitle = "Not Validated";
        	}
        }else{
        	security_validationTitle = "";
        }
        
        setTitle(APversion + " - " + String.valueOf(f) + "     " + security_validationTitle);

        this.aprActive = String.valueOf(f);
        
        ListNode currentLoadList = this.loadList.getFirstNode();

        while (currentLoadList != null)
        {
          String listdata = currentLoadList.data.toString();
          loadcount++;
          this.fList.insertAtBack(listdata);

          if ((loadcount == 2) || (loadcount == 12) || (loadcount == 22) || 
            (loadcount == 32) || (loadcount == 42) || (loadcount == 52) || 
            (loadcount == 62) || (loadcount == 72) || (loadcount == 82) || 
            (loadcount == 92) || (loadcount == 102) || (loadcount == 112) || 
            (loadcount == 122) || (loadcount == 132) || (loadcount == 142) || 
            (loadcount == 152) || (loadcount == 162) || (loadcount == 172) || 
            (loadcount == 182) || (loadcount == 192) || (loadcount == 202) || 
            (loadcount == 212) || (loadcount == 222) || (loadcount == 232) || 
            (loadcount == 242) || (loadcount == 252) || (loadcount == 262) || 
            (loadcount == 272) || (loadcount == 282) || (loadcount == 292) || 
            (loadcount == 302) || (loadcount == 312) || (loadcount == 322) || 
            (loadcount == 332) || (loadcount == 342) || (loadcount == 352))
          {
            steptype = currentLoadList.data.toString();

            if (steptype.equals("add"))
            {
              this.tList.insertAtBack("REG");
              nonPK = nonPK + 1;
            }
            if (steptype.equals("serial"))
            {
              this.tList.insertAtBack("SER");
              nonPK = nonPK + 1;
            }
            if (steptype.equals("brew"))
            {
              this.tList.insertAtBack("BRW");
              nonPK = nonPK + 1;
            }
            if (steptype.equals("wash"))
            {
              this.tList.insertAtBack("WSH");
              nonPK = nonPK + 1;
            }
            if (steptype.equals("incubate"))
            {
              this.tList.insertAtBack("INC");
              nonPK = nonPK + 1;
            }
            if (steptype.equals("read"))
            {
              this.tList.insertAtBack("RDR");
              nonPK = nonPK + 1;
            }
            if (steptype.equals("SPE"))
            {
              this.tList.insertAtBack("SPE");
              nonPK = nonPK + 1;
            }
            if (steptype.equals("move"))
            {
              this.tList.insertAtBack("MOV");
              nonPK = nonPK + 1;
            }
            if (steptype.equals("move plate"))
            {
              this.tList.insertAtBack("MOVPL");
              nonPK = nonPK + 1;
            }
            if (steptype.equals("Cherry Picking"))
            {
              this.tList.insertAtBack("CHR");
              nonPK = nonPK + 1;
            }
            if (steptype.equals("PK"))
            {
              this.tList.insertAtBack("PKS");
              cntPK = cntPK + 1;
            }
            if (steptype.equals("Pause"))
            {
              this.tList.insertAtBack("PAU");
              nonPK = nonPK + 1;
            }
            if (steptype.equals("Cytomat01"))
            {
              this.tList.insertAtBack("CYT");
              nonPK = nonPK + 1;
            }
            if (steptype.equals("Liconic01"))
            {
              this.tList.insertAtBack("LIC");
              nonPK = nonPK + 1;
            }
            if (steptype.equals("MovePlate02"))
            {
              this.tList.insertAtBack("MVP2");
              nonPK = nonPK + 1;
            }
            if (steptype.equals("CherryPicking02"))
            {
              this.tList.insertAtBack("CHR2");
              nonPK = nonPK + 1;
            }
            if (steptype.equals("CORDNA01"))
            {
              this.tList.insertAtBack("DNA");
              nonPK = nonPK + 1;
            }
            if (steptype.equals("LOAD01"))
            {
              this.tList.insertAtBack("LOAD01");
              nonPK = nonPK + 1;
            }


          }

          if (loadcount > 2)
          {
            String m2;
            if (steptype.equals("add"))
            {
              if ((loadcount == 3) || (loadcount == 13) || (loadcount == 23) || 
                (loadcount == 33) || (loadcount == 43) || (loadcount == 53) || 
                (loadcount == 63) || (loadcount == 73) || (loadcount == 83) || 
                (loadcount == 93) || (loadcount == 103) || (loadcount == 113) || 
                (loadcount == 123) || (loadcount == 133) || (loadcount == 143) || 
                (loadcount == 153) || (loadcount == 163) || (loadcount == 173) || 
                (loadcount == 183) || (loadcount == 193) || (loadcount == 203) || 
                (loadcount == 213) || (loadcount == 223) || (loadcount == 233) || 
                (loadcount == 243) || (loadcount == 253) || (loadcount == 263) || 
                (loadcount == 273) || (loadcount == 283) || (loadcount == 293) || 
                (loadcount == 303) || (loadcount == 313) || (loadcount == 323) || 
                (loadcount == 333) || (loadcount == 343) || (loadcount == 353))
              {
                String line3 = currentLoadList.data.toString();
                index1 = line3.indexOf(',');
                len = line3.length();
                index2 = line3.indexOf(',', index1 + 1);
                index3 = line3.indexOf(',', index2 + 1);
                reagent = line3.substring(0, index1);
              }

              if ((loadcount == 4) || (loadcount == 14) || (loadcount == 24) || 
                (loadcount == 34) || (loadcount == 44) || (loadcount == 54) || 
                (loadcount == 64) || (loadcount == 74) || (loadcount == 84) || 
                (loadcount == 94) || (loadcount == 104) || (loadcount == 114) || 
                (loadcount == 124) || (loadcount == 134) || (loadcount == 144))
              {
                String line4 = currentLoadList.data.toString();
                index1 = line4.indexOf(',');
                len = line4.length();
                this.spl1u = line4.substring(0, index1);
                this.dpl1u = line4.substring(index1 + 1, len);
              }

              if ((loadcount == 5) || (loadcount == 15) || (loadcount == 25) || 
                (loadcount == 35) || (loadcount == 45) || (loadcount == 55) || 
                (loadcount == 65) || (loadcount == 75) || (loadcount == 85) || 
                (loadcount == 95) || (loadcount == 105) || (loadcount == 115) || 
                (loadcount == 125) || (loadcount == 135) || (loadcount == 145) || 
                (loadcount == 155) || (loadcount == 165) || (loadcount == 175) || 
                (loadcount == 185) || (loadcount == 195) || (loadcount == 205) || 
                (loadcount == 215) || (loadcount == 225) || (loadcount == 235) || 
                (loadcount == 245) || (loadcount == 255) || (loadcount == 265) || 
                (loadcount == 275) || (loadcount == 285) || (loadcount == 295) || 
                (loadcount == 305) || (loadcount == 315) || (loadcount == 325) || 
                (loadcount == 335) || (loadcount == 345) || (loadcount == 355))
              {
                String line5 = currentLoadList.data.toString();
                len = line5.length();
                index1 = line5.indexOf(',');
                index2 = line5.indexOf(',', index1 + 1);
                index3 = line5.indexOf(',', index2 + 1);
                index4 = line5.indexOf(',', index3 + 1);
                index5 = line5.indexOf(',', index4 + 1);
                addsrc = line5.substring(0, index1);
                this.mix1 = line5.substring(index1 + 1, index2);
                String asp1 = line5.substring(index2 + 1, index3);
                mp1 = line5.substring(index3 + 1, index4);
                c1 = line5.substring(index2 + 1, index2 + 2);

                v1 = line5.substring(index5 + 5, index5 + 6);
                this.rs1 = line5.substring(index5 + 7, len);
              }

              if ((loadcount == 6) || (loadcount == 16) || (loadcount == 26) || 
                (loadcount == 36) || (loadcount == 46) || (loadcount == 56) || 
                (loadcount == 66) || (loadcount == 76) || (loadcount == 86) || 
                (loadcount == 96) || (loadcount == 106) || (loadcount == 116) || 
                (loadcount == 126) || (loadcount == 136) || (loadcount == 146) || 
                (loadcount == 156) || (loadcount == 166) || (loadcount == 176) || 
                (loadcount == 186) || (loadcount == 196) || (loadcount == 206) || 
                (loadcount == 216) || (loadcount == 226) || (loadcount == 236) || 
                (loadcount == 246) || (loadcount == 256) || (loadcount == 266) || 
                (loadcount == 276) || (loadcount == 286) || (loadcount == 296) || 
                (loadcount == 306) || (loadcount == 316) || (loadcount == 326) || 
                (loadcount == 336) || (loadcount == 346) || (loadcount == 356))
              {
                String line6 = currentLoadList.data.toString();
                index1 = line6.indexOf(',');
                index2 = line6.indexOf(',', index1 + 1);
                index3 = line6.indexOf(',', index2 + 1);
                index4 = line6.indexOf(',', index3 + 1);
                index5 = line6.indexOf(',', index4 + 1);
                len = line6.length();
                String adddest = line6.substring(0, index1);
                m2 = line6.substring(index1 + 1, index1 + 2);
                mp2 = line6.substring(index3 + 1, index4);
                this.rs2 = line6.substring(index5 + 7, len);
              }

              if ((loadcount == 9) || (loadcount == 19) || (loadcount == 29) || 
                (loadcount == 39) || (loadcount == 49) || (loadcount == 59) || 
                (loadcount == 69) || (loadcount == 79) || (loadcount == 89) || 
                (loadcount == 99) || (loadcount == 109) || (loadcount == 119) || 
                (loadcount == 129) || (loadcount == 139) || (loadcount == 149) || 
                (loadcount == 159) || (loadcount == 169) || (loadcount == 179) || 
                (loadcount == 189) || (loadcount == 199) || (loadcount == 209) || 
                (loadcount == 219) || (loadcount == 229) || (loadcount == 239) || 
                (loadcount == 249) || (loadcount == 259) || (loadcount == 269) || 
                (loadcount == 279) || (loadcount == 289) || (loadcount == 299) || 
                (loadcount == 309) || (loadcount == 319) || (loadcount == 329) || 
                (loadcount == 339) || (loadcount == 349) || (loadcount == 359))
              {
                volume = currentLoadList.data.toString();
              }

              if ((loadcount == 10) || (loadcount == 20) || (loadcount == 30) || 
                (loadcount == 40) || (loadcount == 50) || (loadcount == 60) || 
                (loadcount == 70) || (loadcount == 80) || (loadcount == 90) || 
                (loadcount == 100) || (loadcount == 110) || (loadcount == 120) || 
                (loadcount == 130) || (loadcount == 140) || (loadcount == 150) || 
                (loadcount == 160) || (loadcount == 170) || (loadcount == 180) || 
                (loadcount == 190) || (loadcount == 200) || (loadcount == 210) || 
                (loadcount == 220) || (loadcount == 230) || (loadcount == 240) || 
                (loadcount == 250) || (loadcount == 260) || (loadcount == 270) || 
                (loadcount == 280) || (loadcount == 290) || (loadcount == 300) || 
                (loadcount == 310) || (loadcount == 320) || (loadcount == 330) || 
                (loadcount == 340) || (loadcount == 350) || (loadcount == 360))
              {
                loaddata = "";
                loaddata = loaddata + "   \n";
                loaddata = loaddata + "\n " + reagent;
                loaddata = loaddata + "\n " + volume + " ul" + "\n";
                if (this.spl1u.equals("Deck Position")) this.spl1u = "T";
                loaddata = loaddata + " Deck " + this.spl1u;
                loaddata = loaddata + " to " + this.dpl1u + "\n";
                loaddata = loaddata + " Multi: " + mp1;
                loaddata = loaddata + "-" + mp2;

                if (v1.equals("1"))
                {
                  loaddata = loaddata + " Recycle Tips\n";
                }

                this.dList.insertAtBack(loaddata);
                loaddata = "";
              }

            }

            if (steptype.equals("serial"))
            {
              if ((loadcount == 3) || (loadcount == 13) || (loadcount == 23) || 
                (loadcount == 33) || (loadcount == 43) || (loadcount == 53) || 
                (loadcount == 63) || (loadcount == 73) || (loadcount == 83) || 
                (loadcount == 93) || (loadcount == 103) || (loadcount == 113) || 
                (loadcount == 123) || (loadcount == 133) || (loadcount == 143) || 
                (loadcount == 153) || (loadcount == 163) || (loadcount == 173) || 
                (loadcount == 183) || (loadcount == 193) || (loadcount == 203) || 
                (loadcount == 213) || (loadcount == 223) || (loadcount == 233) || 
                (loadcount == 243) || (loadcount == 253) || (loadcount == 263) || 
                (loadcount == 273) || (loadcount == 283) || (loadcount == 293) || 
                (loadcount == 303) || (loadcount == 313) || (loadcount == 323) || 
                (loadcount == 333) || (loadcount == 343) || (loadcount == 353))
              {
                String line3 = currentLoadList.data.toString();
                index1 = line3.indexOf(',');
                len = line3.length();
                index2 = line3.indexOf(',', index1 + 1);
                index3 = line3.indexOf(',', index2 + 1);
                index4 = line3.indexOf(',', index3 + 1);
                this.sttext.setText(line3.substring(0, index1));
                this.slt1 = line3.substring(index1 + 1, index2);
                serreg = line3.substring(index2 + 1, index3);
                this.dlt1 = line3.substring(index3 + 1, index4);
                this.so2 = line3.substring(index4 + 1, len);
              }

              if ((loadcount == 4) || (loadcount == 14) || (loadcount == 24) || 
                (loadcount == 34) || (loadcount == 44) || (loadcount == 54) || 
                (loadcount == 64) || (loadcount == 74) || (loadcount == 84) || 
                (loadcount == 94) || (loadcount == 104) || (loadcount == 114) || 
                (loadcount == 124) || (loadcount == 134) || (loadcount == 144) || 
                (loadcount == 154) || (loadcount == 164) || (loadcount == 174) || 
                (loadcount == 184) || (loadcount == 194) || (loadcount == 204) || 
                (loadcount == 214) || (loadcount == 224) || (loadcount == 234) || 
                (loadcount == 244) || (loadcount == 254) || (loadcount == 264) || 
                (loadcount == 274) || (loadcount == 284) || (loadcount == 294) || 
                (loadcount == 304) || (loadcount == 314) || (loadcount == 324) || 
                (loadcount == 334) || (loadcount == 344) || (loadcount == 354))
              {
                String line4 = currentLoadList.data.toString();
                index1 = line4.indexOf(',');
                len = line4.length();
                this.spl2 = line4.substring(0, index1);
                this.dpl2 = line4.substring(index1 + 1, len);
              }
              String sersrc;
              if ((loadcount == 5) || (loadcount == 15) || (loadcount == 25) || 
                (loadcount == 35) || (loadcount == 45) || (loadcount == 55) || 
                (loadcount == 65) || (loadcount == 75) || (loadcount == 85) || 
                (loadcount == 95) || (loadcount == 105) || (loadcount == 115) || 
                (loadcount == 125) || (loadcount == 135) || (loadcount == 145) || 
                (loadcount == 155) || (loadcount == 165) || (loadcount == 175) || 
                (loadcount == 185) || (loadcount == 195) || (loadcount == 205) || 
                (loadcount == 215) || (loadcount == 225) || (loadcount == 235) || 
                (loadcount == 245) || (loadcount == 255) || (loadcount == 265) || 
                (loadcount == 275) || (loadcount == 285) || (loadcount == 295) || 
                (loadcount == 305) || (loadcount == 315) || (loadcount == 325) || 
                (loadcount == 335) || (loadcount == 345) || (loadcount == 355))
              {
                String line5 = currentLoadList.data.toString();
                index1 = line5.indexOf(',');
                index2 = line5.indexOf(',', index1 + 1);
                sersrc = line5.substring(0, index1);
                m2 = line5.substring(index1 + 5, index1 + 6);
              }

              if ((loadcount == 6) || (loadcount == 16) || (loadcount == 26) || 
                (loadcount == 36) || (loadcount == 46) || (loadcount == 56) || 
                (loadcount == 66) || (loadcount == 76) || (loadcount == 86) || 
                (loadcount == 96) || (loadcount == 106) || (loadcount == 116) || 
                (loadcount == 126) || (loadcount == 136) || (loadcount == 146) || 
                (loadcount == 156) || (loadcount == 166) || (loadcount == 176) || 
                (loadcount == 186) || (loadcount == 196) || (loadcount == 206) || 
                (loadcount == 216) || (loadcount == 226) || (loadcount == 236) || 
                (loadcount == 246) || (loadcount == 256) || (loadcount == 266) || 
                (loadcount == 276) || (loadcount == 286) || (loadcount == 296) || 
                (loadcount == 306) || (loadcount == 316) || (loadcount == 326) || 
                (loadcount == 336) || (loadcount == 346) || (loadcount == 356))
              {
                String line6 = currentLoadList.data.toString();
                index1 = line6.indexOf(',');
                index2 = line6.indexOf(',', index1 + 1);
                serdest = line6.substring(0, index1);
                sersrc = line6.substring(index2 + 1, index2 + 6);
              }

              if ((loadcount == 10) || (loadcount == 20) || (loadcount == 30) || 
                (loadcount == 40) || (loadcount == 50) || (loadcount == 60) || 
                (loadcount == 70) || (loadcount == 80) || (loadcount == 90) || 
                (loadcount == 100) || (loadcount == 110) || (loadcount == 120) || 
                (loadcount == 130) || (loadcount == 140) || (loadcount == 150) || 
                (loadcount == 160) || (loadcount == 170) || (loadcount == 180) || 
                (loadcount == 190) || (loadcount == 200) || (loadcount == 210) || 
                (loadcount == 220) || (loadcount == 230) || (loadcount == 240) || 
                (loadcount == 250) || (loadcount == 260) || (loadcount == 270) || 
                (loadcount == 280) || (loadcount == 290) || (loadcount == 300) || 
                (loadcount == 310) || (loadcount == 320) || (loadcount == 330) || 
                (loadcount == 340) || (loadcount == 350) || (loadcount == 360))
              {
                loaddata = "";
                loaddata = loaddata + "\nSERIAL DILUTION\n";
                loaddata = loaddata + "\n " + this.sttext.getText().trim() + "\n ";

                this.dList.insertAtBack(loaddata);
                loaddata = "";
              }
            }

            if (steptype.equals("brew"))
            {
              if ((loadcount == 3) || (loadcount == 13) || (loadcount == 23) || 
                (loadcount == 33) || (loadcount == 43) || (loadcount == 53) || 
                (loadcount == 63) || (loadcount == 73) || (loadcount == 83) || 
                (loadcount == 93) || (loadcount == 103) || (loadcount == 113) || 
                (loadcount == 123) || (loadcount == 133) || (loadcount == 143) || 
                (loadcount == 153) || (loadcount == 163) || (loadcount == 173) || 
                (loadcount == 183) || (loadcount == 193) || (loadcount == 203) || 
                (loadcount == 213) || (loadcount == 223) || (loadcount == 233) || 
                (loadcount == 243) || (loadcount == 253) || (loadcount == 263) || 
                (loadcount == 273) || (loadcount == 283) || (loadcount == 293) || 
                (loadcount == 303) || (loadcount == 313) || (loadcount == 323) || 
                (loadcount == 333) || (loadcount == 343) || (loadcount == 353))
              {
                String line3 = currentLoadList.data.toString();
                index1 = line3.indexOf(',');
                index2 = line3.indexOf(',', index1 + 1);
                index3 = line3.indexOf(',', index2 + 1);
                brewreg = line3.substring(0, index1);
                blt1 = line3.substring(index1 + 1, index2);
                brewnew = line3.substring(index2 + 1, index3);
              }

              if ((loadcount == 4) || (loadcount == 14) || (loadcount == 24) || 
                (loadcount == 34) || (loadcount == 44) || (loadcount == 54) || 
                (loadcount == 64) || (loadcount == 74) || (loadcount == 84) || 
                (loadcount == 94) || (loadcount == 104) || (loadcount == 114) || 
                (loadcount == 124) || (loadcount == 134) || (loadcount == 144) || 
                (loadcount == 154) || (loadcount == 164) || (loadcount == 174) || 
                (loadcount == 184) || (loadcount == 194) || (loadcount == 204) || 
                (loadcount == 214) || (loadcount == 224) || (loadcount == 234) || 
                (loadcount == 244) || (loadcount == 254) || (loadcount == 264) || 
                (loadcount == 274) || (loadcount == 284) || (loadcount == 294) || 
                (loadcount == 304) || (loadcount == 314) || (loadcount == 324) || 
                (loadcount == 334) || (loadcount == 344) || (loadcount == 354))
              {
                String line4 = currentLoadList.data.toString();
                index1 = line4.indexOf(',');
                len = line4.length();
                this.bpl1 = line4.substring(0, index1);
                this.bpl2 = line4.substring(index1 + 1, len);
              }
              String bsource;
              if ((loadcount == 5) || (loadcount == 15) || (loadcount == 25) || 
                (loadcount == 35) || (loadcount == 45) || (loadcount == 55) || 
                (loadcount == 65) || (loadcount == 75) || (loadcount == 85) || 
                (loadcount == 95) || (loadcount == 105) || (loadcount == 115) || 
                (loadcount == 125) || (loadcount == 135) || (loadcount == 145) || 
                (loadcount == 155) || (loadcount == 165) || (loadcount == 175) || 
                (loadcount == 185) || (loadcount == 195) || (loadcount == 205) || 
                (loadcount == 215) || (loadcount == 225) || (loadcount == 235) || 
                (loadcount == 245) || (loadcount == 255) || (loadcount == 265) || 
                (loadcount == 275) || (loadcount == 285) || (loadcount == 295) || 
                (loadcount == 305) || (loadcount == 315) || (loadcount == 325) || 
                (loadcount == 335) || (loadcount == 345) || (loadcount == 355))
              {
                String line5 = currentLoadList.data.toString();
                index1 = line5.indexOf(',');
                index2 = line5.indexOf(',', index1 + 1);
                bsource = line5.substring(0, index1);
                m1 = line5.substring(index1 + 1, index1 + 2);
                c1 = line5.substring(index2 + 1, index2 + 2);
              }

              if ((loadcount == 6) || (loadcount == 16) || (loadcount == 26) || 
                (loadcount == 36) || (loadcount == 46) || (loadcount == 56) || 
                (loadcount == 66) || (loadcount == 76) || (loadcount == 86) || 
                (loadcount == 96) || (loadcount == 106) || (loadcount == 116) || 
                (loadcount == 126) || (loadcount == 136) || (loadcount == 146) || 
                (loadcount == 156) || (loadcount == 166) || (loadcount == 176) || 
                (loadcount == 186) || (loadcount == 196) || (loadcount == 206) || 
                (loadcount == 216) || (loadcount == 226) || (loadcount == 236) || 
                (loadcount == 246) || (loadcount == 256) || (loadcount == 266) || 
                (loadcount == 276) || (loadcount == 286) || (loadcount == 296) || 
                (loadcount == 306) || (loadcount == 316) || (loadcount == 326) || 
                (loadcount == 336) || (loadcount == 346) || (loadcount == 356))
              {
                String line6 = currentLoadList.data.toString();
                index1 = line6.indexOf(',');
                index2 = line6.indexOf(',', index1 + 1);
                bsource = line6.substring(0, index1);
              }

              if ((loadcount == 10) || (loadcount == 20) || (loadcount == 30) || 
                (loadcount == 40) || (loadcount == 50) || (loadcount == 60) || 
                (loadcount == 70) || (loadcount == 80) || (loadcount == 90) || 
                (loadcount == 100) || (loadcount == 110) || (loadcount == 120) || 
                (loadcount == 130) || (loadcount == 140) || (loadcount == 150) || 
                (loadcount == 160) || (loadcount == 170) || (loadcount == 180) || 
                (loadcount == 190) || (loadcount == 200) || (loadcount == 210) || 
                (loadcount == 220) || (loadcount == 230) || (loadcount == 240) || 
                (loadcount == 250) || (loadcount == 260) || (loadcount == 270) || 
                (loadcount == 280) || (loadcount == 290) || (loadcount == 300) || 
                (loadcount == 310) || (loadcount == 320) || (loadcount == 330) || 
                (loadcount == 340) || (loadcount == 350) || (loadcount == 360))
              {
                loaddata = "";
                loaddata = loaddata + "\n BREW\n ";
                loaddata = loaddata + "\n " + brewnew + "\n";

                this.dList.insertAtBack(loaddata);
                loaddata = "";
              }
            }

            if (steptype.equals("wash"))
            {
              if ((loadcount == 4) || (loadcount == 14) || (loadcount == 24) || 
                (loadcount == 34) || (loadcount == 44) || (loadcount == 54) || 
                (loadcount == 64) || (loadcount == 74) || (loadcount == 84) || 
                (loadcount == 94) || (loadcount == 104) || (loadcount == 114) || 
                (loadcount == 124) || (loadcount == 134) || (loadcount == 144) || 
                (loadcount == 154) || (loadcount == 164) || (loadcount == 174) || 
                (loadcount == 184) || (loadcount == 194) || (loadcount == 204) || 
                (loadcount == 214) || (loadcount == 224) || (loadcount == 234) || 
                (loadcount == 244) || (loadcount == 254) || (loadcount == 264) || 
                (loadcount == 274) || (loadcount == 284) || (loadcount == 294) || 
                (loadcount == 304) || (loadcount == 314) || (loadcount == 324) || 
                (loadcount == 334) || (loadcount == 344) || (loadcount == 354))
              {
                w1 = currentLoadList.data.toString();
              }
              if ((loadcount == 10) || (loadcount == 20) || (loadcount == 30) || 
                (loadcount == 40) || (loadcount == 50) || (loadcount == 60) || 
                (loadcount == 70) || (loadcount == 80) || (loadcount == 90) || 
                (loadcount == 100) || (loadcount == 110) || (loadcount == 120) || 
                (loadcount == 130) || (loadcount == 140) || (loadcount == 150) || 
                (loadcount == 160) || (loadcount == 170) || (loadcount == 180) || 
                (loadcount == 190) || (loadcount == 200) || (loadcount == 210) || 
                (loadcount == 220) || (loadcount == 230) || (loadcount == 240) || 
                (loadcount == 250) || (loadcount == 260) || (loadcount == 270) || 
                (loadcount == 280) || (loadcount == 290) || (loadcount == 300) || 
                (loadcount == 310) || (loadcount == 320) || (loadcount == 330) || 
                (loadcount == 340) || (loadcount == 350) || (loadcount == 360))
              {
                loaddata = "";
                loaddata = loaddata + "\n WASH\n ";
                loaddata = loaddata + "\n " + w1 + "\n";

                this.dList.insertAtBack(loaddata);
                loaddata = "";
              }
            }

            if (steptype.equals("incubate"))
            {
              if ((loadcount == 3) || (loadcount == 13) || (loadcount == 23) || 
                (loadcount == 33) || (loadcount == 43) || (loadcount == 53) || 
                (loadcount == 63) || (loadcount == 73) || (loadcount == 83) || 
                (loadcount == 93) || (loadcount == 103) || (loadcount == 113) || 
                (loadcount == 123) || (loadcount == 133) || (loadcount == 143) || 
                (loadcount == 153) || (loadcount == 163) || (loadcount == 173) || 
                (loadcount == 183) || (loadcount == 193) || (loadcount == 203) || 
                (loadcount == 213) || (loadcount == 223) || (loadcount == 233) || 
                (loadcount == 243) || (loadcount == 253) || (loadcount == 263) || 
                (loadcount == 273) || (loadcount == 283) || (loadcount == 293) || 
                (loadcount == 303) || (loadcount == 313) || (loadcount == 323) || 
                (loadcount == 333) || (loadcount == 343) || (loadcount == 353))
              {
                inc3 = currentLoadList.data.toString();
              }
              if ((loadcount == 4) || (loadcount == 14) || (loadcount == 24) || 
                (loadcount == 34) || (loadcount == 44) || (loadcount == 54) || 
                (loadcount == 64) || (loadcount == 74) || (loadcount == 84) || 
                (loadcount == 94) || (loadcount == 104) || (loadcount == 114) || 
                (loadcount == 124) || (loadcount == 134) || (loadcount == 144) || 
                (loadcount == 154) || (loadcount == 164) || (loadcount == 174) || 
                (loadcount == 184) || (loadcount == 194) || (loadcount == 204) || 
                (loadcount == 214) || (loadcount == 224) || (loadcount == 234) || 
                (loadcount == 244) || (loadcount == 254) || (loadcount == 264) || 
                (loadcount == 274) || (loadcount == 284) || (loadcount == 294) || 
                (loadcount == 304) || (loadcount == 314) || (loadcount == 324) || 
                (loadcount == 334) || (loadcount == 344) || (loadcount == 354))
              {
                inc4 = currentLoadList.data.toString();
              }
              if ((loadcount == 10) || (loadcount == 20) || (loadcount == 30) || 
                (loadcount == 40) || (loadcount == 50) || (loadcount == 60) || 
                (loadcount == 70) || (loadcount == 80) || (loadcount == 90) || 
                (loadcount == 100) || (loadcount == 110) || (loadcount == 120) || 
                (loadcount == 130) || (loadcount == 140) || (loadcount == 150) || 
                (loadcount == 160) || (loadcount == 170) || (loadcount == 180) || 
                (loadcount == 190) || (loadcount == 200) || (loadcount == 210) || 
                (loadcount == 220) || (loadcount == 230) || (loadcount == 240) || 
                (loadcount == 250) || (loadcount == 260) || (loadcount == 270) || 
                (loadcount == 280) || (loadcount == 290) || (loadcount == 300) || 
                (loadcount == 310) || (loadcount == 320) || (loadcount == 330) || 
                (loadcount == 340) || (loadcount == 350) || (loadcount == 360))
              {
                loaddata = "";
                loaddata = loaddata + "\n INCUBATE\n ";

                loaddata = loaddata + "\n " + inc3 + " min";
                loaddata = loaddata + "\n " + inc4 + " (c)" + "\n";

                this.dList.insertAtBack(loaddata);
                loaddata = "";
              }
            }
            if (steptype.equals("read"))
            {
              if ((loadcount == 3) || (loadcount == 13) || (loadcount == 23) || 
                (loadcount == 33) || (loadcount == 43) || (loadcount == 53) || 
                (loadcount == 63) || (loadcount == 73) || (loadcount == 83) || 
                (loadcount == 93) || (loadcount == 103) || (loadcount == 113) || 
                (loadcount == 123) || (loadcount == 133) || (loadcount == 143) || 
                (loadcount == 153) || (loadcount == 163) || (loadcount == 173) || 
                (loadcount == 183) || (loadcount == 193) || (loadcount == 203) || 
                (loadcount == 213) || (loadcount == 223) || (loadcount == 233) || 
                (loadcount == 243) || (loadcount == 253) || (loadcount == 263) || 
                (loadcount == 273) || (loadcount == 283) || (loadcount == 293) || 
                (loadcount == 303) || (loadcount == 313) || (loadcount == 323) || 
                (loadcount == 333) || (loadcount == 343) || (loadcount == 353))
              {
                String line3 = currentLoadList.data.toString();
                index1 = line3.indexOf(',');
                index2 = line3.indexOf(',', index1 + 1);
                len = line3.length();
                read1 = line3.substring(0, index1);
                temp = line3.substring(index1 + 1, len);
              }

              if ((loadcount == 4) || (loadcount == 14) || (loadcount == 24) || 
                (loadcount == 34) || (loadcount == 44) || (loadcount == 54) || 
                (loadcount == 64) || (loadcount == 74) || (loadcount == 84) || 
                (loadcount == 94) || (loadcount == 104) || (loadcount == 114) || 
                (loadcount == 124) || (loadcount == 134) || (loadcount == 144) || 
                (loadcount == 154) || (loadcount == 164) || (loadcount == 174) || 
                (loadcount == 184) || (loadcount == 194) || (loadcount == 204) || 
                (loadcount == 214) || (loadcount == 224) || (loadcount == 234) || 
                (loadcount == 244) || (loadcount == 254) || (loadcount == 264) || 
                (loadcount == 274) || (loadcount == 284) || (loadcount == 294) || 
                (loadcount == 304) || (loadcount == 314) || (loadcount == 324) || 
                (loadcount == 334) || (loadcount == 344) || (loadcount == 354))
              {
                String line4 = currentLoadList.data.toString();
                reader = line4;
              }

              if ((loadcount == 10) || (loadcount == 20) || (loadcount == 30) || 
                (loadcount == 40) || (loadcount == 50) || (loadcount == 60) || 
                (loadcount == 70) || (loadcount == 80) || (loadcount == 90) || 
                (loadcount == 100) || (loadcount == 110) || (loadcount == 120) || 
                (loadcount == 130) || (loadcount == 140) || (loadcount == 150) || 
                (loadcount == 160) || (loadcount == 170) || (loadcount == 180) || 
                (loadcount == 190) || (loadcount == 200) || (loadcount == 210) || 
                (loadcount == 220) || (loadcount == 230) || (loadcount == 240) || 
                (loadcount == 250) || (loadcount == 260) || (loadcount == 270) || 
                (loadcount == 280) || (loadcount == 290) || (loadcount == 300) || 
                (loadcount == 310) || (loadcount == 320) || (loadcount == 330) || 
                (loadcount == 340) || (loadcount == 350) || (loadcount == 360))
              {
                loaddata = "";
                loaddata = loaddata + "\n READ\n ";
                loaddata = loaddata + "\n " + read1;
                loaddata = loaddata + "\n " + reader;
                loaddata = loaddata + "\n " + temp + " (c)";
                this.dList.insertAtBack(loaddata);
                loaddata = "";
              }
            }
            if (steptype.equals("SPE"))
            {
              if ((loadcount == 3) || (loadcount == 13) || (loadcount == 23) || 
                (loadcount == 33) || (loadcount == 43) || (loadcount == 53) || 
                (loadcount == 63) || (loadcount == 73) || (loadcount == 83) || 
                (loadcount == 93) || (loadcount == 103) || (loadcount == 113) || 
                (loadcount == 123) || (loadcount == 133) || (loadcount == 143) || 
                (loadcount == 153) || (loadcount == 163) || (loadcount == 173) || 
                (loadcount == 183) || (loadcount == 193) || (loadcount == 203) || 
                (loadcount == 213) || (loadcount == 223) || (loadcount == 233) || 
                (loadcount == 243) || (loadcount == 253) || (loadcount == 263) || 
                (loadcount == 273) || (loadcount == 283) || (loadcount == 293) || 
                (loadcount == 303) || (loadcount == 313) || (loadcount == 323) || 
                (loadcount == 333) || (loadcount == 343) || (loadcount == 353))
              {
                String line3 = currentLoadList.data.toString();
                index1 = line3.indexOf(',');
                b1 = line3.substring(0, index1);
              }
              if ((loadcount == 10) || (loadcount == 20) || (loadcount == 30) || 
                (loadcount == 40) || (loadcount == 50) || (loadcount == 60) || 
                (loadcount == 70) || (loadcount == 80) || (loadcount == 90) || 
                (loadcount == 100) || (loadcount == 110) || (loadcount == 120) || 
                (loadcount == 130) || (loadcount == 140) || (loadcount == 150) || 
                (loadcount == 160) || (loadcount == 170) || (loadcount == 180) || 
                (loadcount == 190) || (loadcount == 200) || (loadcount == 210) || 
                (loadcount == 220) || (loadcount == 230) || (loadcount == 240) || 
                (loadcount == 250) || (loadcount == 260) || (loadcount == 270) || 
                (loadcount == 280) || (loadcount == 290) || (loadcount == 300) || 
                (loadcount == 310) || (loadcount == 320) || (loadcount == 330) || 
                (loadcount == 340) || (loadcount == 350) || (loadcount == 360))
              {
                loaddata = "";
                loaddata = loaddata + "\n SPE\n ";
                if (b1.equals("1"))
                {
                  loaddata = loaddata + "\n Assemble";
                }
                if (b1.equals("2"))
                {
                  loaddata = loaddata + "\n Disassemble";
                }
                if (b1.equals("3"))
                {
                  loaddata = loaddata + "\n Apply Vac";
                }

                this.dList.insertAtBack(loaddata);
                loaddata = "";
              }
            }
            if (steptype.equals("move"))
            {
              if ((loadcount == 3) || (loadcount == 13) || (loadcount == 23) || 
                (loadcount == 33) || (loadcount == 43) || (loadcount == 53) || 
                (loadcount == 63) || (loadcount == 73) || (loadcount == 83) || 
                (loadcount == 93) || (loadcount == 103) || (loadcount == 113) || 
                (loadcount == 123) || (loadcount == 133) || (loadcount == 143) || 
                (loadcount == 153) || (loadcount == 163) || (loadcount == 173) || 
                (loadcount == 183) || (loadcount == 193) || (loadcount == 203) || 
                (loadcount == 213) || (loadcount == 223) || (loadcount == 233) || 
                (loadcount == 243) || (loadcount == 253) || (loadcount == 263) || 
                (loadcount == 273) || (loadcount == 283) || (loadcount == 293) || 
                (loadcount == 303) || (loadcount == 313) || (loadcount == 323) || 
                (loadcount == 333) || (loadcount == 343) || (loadcount == 353))
              {
                this.mov = currentLoadList.data.toString();
              }
              if ((loadcount == 10) || (loadcount == 20) || (loadcount == 30) || 
                (loadcount == 40) || (loadcount == 50) || (loadcount == 60) || 
                (loadcount == 70) || (loadcount == 80) || (loadcount == 90) || 
                (loadcount == 100) || (loadcount == 110) || (loadcount == 120) || 
                (loadcount == 130) || (loadcount == 140) || (loadcount == 150) || 
                (loadcount == 160) || (loadcount == 170) || (loadcount == 180) || 
                (loadcount == 190) || (loadcount == 200) || (loadcount == 210) || 
                (loadcount == 220) || (loadcount == 230) || (loadcount == 240) || 
                (loadcount == 250) || (loadcount == 260) || (loadcount == 270) || 
                (loadcount == 280) || (loadcount == 290) || (loadcount == 300) || 
                (loadcount == 310) || (loadcount == 320) || (loadcount == 330) || 
                (loadcount == 340) || (loadcount == 350) || (loadcount == 360))
              {
                loaddata = "";
                loaddata = loaddata + "\n MOVE\n ";
                loaddata = loaddata + "\n " + this.mov;
                this.dList.insertAtBack(loaddata);
                loaddata = "";
              }
            }
            if (steptype.equals("move plate"))
            {
              if ((loadcount == 4) || (loadcount == 14) || (loadcount == 24) || 
                (loadcount == 34) || (loadcount == 44) || (loadcount == 54) || 
                (loadcount == 64) || (loadcount == 74) || (loadcount == 84) || 
                (loadcount == 94) || (loadcount == 104) || (loadcount == 114) || 
                (loadcount == 124) || (loadcount == 134) || (loadcount == 144) || 
                (loadcount == 154) || (loadcount == 164) || (loadcount == 174) || 
                (loadcount == 184) || (loadcount == 194) || (loadcount == 204) || 
                (loadcount == 214) || (loadcount == 224) || (loadcount == 234) || 
                (loadcount == 244) || (loadcount == 254) || (loadcount == 264) || 
                (loadcount == 274) || (loadcount == 284) || (loadcount == 294) || 
                (loadcount == 304) || (loadcount == 314) || (loadcount == 324) || 
                (loadcount == 334) || (loadcount == 344) || (loadcount == 354))
              {
                String line4 = currentLoadList.data.toString();
                index1 = line4.indexOf(',');
                len = line4.length();
                this.spl3u = line4.substring(0, index1);
                this.dpl3u = line4.substring(index1 + 1, len);
              }
              if ((loadcount == 5) || (loadcount == 15) || (loadcount == 25) || 
                (loadcount == 35) || (loadcount == 45) || (loadcount == 55) || 
                (loadcount == 65) || (loadcount == 75) || (loadcount == 85) || 
                (loadcount == 95) || (loadcount == 105) || (loadcount == 115) || 
                (loadcount == 125) || (loadcount == 135) || (loadcount == 145) || 
                (loadcount == 155) || (loadcount == 165) || (loadcount == 175) || 
                (loadcount == 185) || (loadcount == 195) || (loadcount == 205) || 
                (loadcount == 215) || (loadcount == 225) || (loadcount == 235) || 
                (loadcount == 245) || (loadcount == 255) || (loadcount == 265) || 
                (loadcount == 275) || (loadcount == 285) || (loadcount == 295) || 
                (loadcount == 305) || (loadcount == 315) || (loadcount == 325) || 
                (loadcount == 335) || (loadcount == 345) || (loadcount == 355))
              {
                String line5 = currentLoadList.data.toString();
                len = line5.length();
                ressrc = line5.substring(0, len);
              }

              if ((loadcount == 10) || (loadcount == 20) || (loadcount == 30) || 
                (loadcount == 40) || (loadcount == 50) || (loadcount == 60) || 
                (loadcount == 70) || (loadcount == 80) || (loadcount == 90) || 
                (loadcount == 100) || (loadcount == 110) || (loadcount == 120) || 
                (loadcount == 130) || (loadcount == 140) || (loadcount == 150) || 
                (loadcount == 160) || (loadcount == 170) || (loadcount == 180) || 
                (loadcount == 190) || (loadcount == 200) || (loadcount == 210) || 
                (loadcount == 220) || (loadcount == 230) || (loadcount == 240) || 
                (loadcount == 250) || (loadcount == 260) || (loadcount == 270) || 
                (loadcount == 280) || (loadcount == 290) || (loadcount == 300) || 
                (loadcount == 310) || (loadcount == 320) || (loadcount == 330) || 
                (loadcount == 340) || (loadcount == 350) || (loadcount == 360))
              {
                loaddata = "";

                loaddata = loaddata + "   \n ";
                loaddata = loaddata + ressrc + "\n" + "Deck " + this.spl3u + " to " + this.dpl3u + "\n\n";

                this.dList.insertAtBack(loaddata);
                loaddata = "";
              }
            }

            if (steptype.equals("Cytomat01"))
            {
                if ((loadcount == 3) || (loadcount == 13) || (loadcount == 23) || 
                        (loadcount == 33) || (loadcount == 43) || (loadcount == 53) || 
                        (loadcount == 63) || (loadcount == 73) || (loadcount == 83) || 
                        (loadcount == 93) || (loadcount == 103) || (loadcount == 113) || 
                        (loadcount == 123) || (loadcount == 133) || (loadcount == 143) || 
                        (loadcount == 153) || (loadcount == 163) || (loadcount == 173) || 
                        (loadcount == 183) || (loadcount == 193) || (loadcount == 203) || 
                        (loadcount == 213) || (loadcount == 223) || (loadcount == 233) || 
                        (loadcount == 243) || (loadcount == 253) || (loadcount == 263) || 
                        (loadcount == 273) || (loadcount == 283) || (loadcount == 293) || 
                        (loadcount == 303) || (loadcount == 313) || (loadcount == 323) || 
                        (loadcount == 333) || (loadcount == 343) || (loadcount == 353))
                      {
                        this.cytfromto = currentLoadList.data.toString();
                        int num = Integer.parseInt(cytfromto);
                        TabWindow.this.cytodatapanel2.setFTIndex(Integer.parseInt(cytfromto));
//                        this.cytodatapanel2.revalidate();
//                        this.cytodatapanel2.repaint();
                      }

                if ((loadcount == 4) || (loadcount == 14) || (loadcount == 24) || 
                (loadcount == 34) || (loadcount == 44) || (loadcount == 54) || 
                (loadcount == 64) || (loadcount == 74) || (loadcount == 84) || 
                (loadcount == 94) || (loadcount == 104) || (loadcount == 114) || 
                (loadcount == 124) || (loadcount == 134) || (loadcount == 144) || 
                (loadcount == 154) || (loadcount == 164) || (loadcount == 174) || 
                (loadcount == 184) || (loadcount == 194) || (loadcount == 204) || 
                (loadcount == 214) || (loadcount == 224) || (loadcount == 234) || 
                (loadcount == 244) || (loadcount == 254) || (loadcount == 264) || 
                (loadcount == 274) || (loadcount == 284) || (loadcount == 294) || 
                (loadcount == 304) || (loadcount == 314) || (loadcount == 324) || 
                (loadcount == 334) || (loadcount == 344) || (loadcount == 354))
              {
                String line4 = currentLoadList.data.toString();
                cytloc = line4.trim();
              }
              if ((loadcount == 5) || (loadcount == 15) || (loadcount == 25) || 
                (loadcount == 35) || (loadcount == 45) || (loadcount == 55) || 
                (loadcount == 65) || (loadcount == 75) || (loadcount == 85) || 
                (loadcount == 95) || (loadcount == 105) || (loadcount == 115) || 
                (loadcount == 125) || (loadcount == 135) || (loadcount == 145) || 
                (loadcount == 155) || (loadcount == 165) || (loadcount == 175) || 
                (loadcount == 185) || (loadcount == 195) || (loadcount == 205) || 
                (loadcount == 215) || (loadcount == 225) || (loadcount == 235) || 
                (loadcount == 245) || (loadcount == 255) || (loadcount == 265) || 
                (loadcount == 275) || (loadcount == 285) || (loadcount == 295) || 
                (loadcount == 305) || (loadcount == 315) || (loadcount == 325) || 
                (loadcount == 335) || (loadcount == 345) || (loadcount == 355))
              {
                String line5 = currentLoadList.data.toString();
                cytdeckloc = line5.trim();
              }

              if ((loadcount == 6) || (loadcount == 16) || (loadcount == 26) || 
                      (loadcount == 36) || (loadcount == 46) || (loadcount == 56) || 
                      (loadcount == 66) || (loadcount == 76) || (loadcount == 86) || 
                      (loadcount == 96) || (loadcount == 106) || (loadcount == 116) || 
                      (loadcount == 126) || (loadcount == 136) || (loadcount == 146) || 
                      (loadcount == 156) || (loadcount == 166) || (loadcount == 176) || 
                      (loadcount == 186) || (loadcount == 196) || (loadcount == 206) || 
                      (loadcount == 216) || (loadcount == 226) || (loadcount == 236) || 
                      (loadcount == 246) || (loadcount == 256) || (loadcount == 266) || 
                      (loadcount == 276) || (loadcount == 286) || (loadcount == 296) || 
                      (loadcount == 306) || (loadcount == 316) || (loadcount == 326) || 
                      (loadcount == 336) || (loadcount == 346) || (loadcount == 356))
                    {
                      this.cytcover = currentLoadList.data.toString();
                      TabWindow.this.cytodatapanel2.setCoverIndex(Integer.parseInt(cytcover));
                    }

              if ((loadcount == 7) || (loadcount == 17) || (loadcount == 27) || 
                      (loadcount == 37) || (loadcount == 47) || (loadcount == 57) || 
                      (loadcount == 67) || (loadcount == 77) || (loadcount == 87) || 
                      (loadcount == 97) || (loadcount == 107) || (loadcount == 117) || 
                      (loadcount == 127) || (loadcount == 137) || (loadcount == 147) || 
                      (loadcount == 157) || (loadcount == 167) || (loadcount == 177) || 
                      (loadcount == 187) || (loadcount == 197) || (loadcount == 207) || 
                      (loadcount == 217) || (loadcount == 227) || (loadcount == 237) || 
                      (loadcount == 247) || (loadcount == 257) || (loadcount == 267) || 
                      (loadcount == 277) || (loadcount == 287) || (loadcount == 297) || 
                      (loadcount == 307) || (loadcount == 317) || (loadcount == 327) || 
                      (loadcount == 337) || (loadcount == 347) || (loadcount == 357))
                    {
                      String line7 = currentLoadList.data.toString();
                      cyt_plate = line7.trim();
    				  int pindex = pAllSearch(this.platesAll, this.cyt_plate);	
//                      int pindex = p96Search(this.plates96, this.cyt_plate);
//                      if (pindex == -1)
//                      {
//                        pindex = p384Search(this.plates384, this.cyt_plate);
//                        pindex += this.plates96.length;
//                      }
                      TabWindow.this.cytodatapanel2.setPlateIndex(pindex);
//                      this.cytodatapanel2.revalidate();
//                      this.cytodatapanel2.repaint();
                                     
                    }

              if ((loadcount == 8) || (loadcount == 18) || (loadcount == 28) || 
                      (loadcount == 38) || (loadcount == 48) || (loadcount == 58) || 
                      (loadcount == 68) || (loadcount == 78) || (loadcount == 88) || 
                      (loadcount == 98) || (loadcount == 108) || (loadcount == 118) || 
                      (loadcount == 128) || (loadcount == 138) || (loadcount == 148) || 
                      (loadcount == 158) || (loadcount == 168) || (loadcount == 178) || 
                      (loadcount == 188) || (loadcount == 198) || (loadcount == 208) || 
                      (loadcount == 218) || (loadcount == 228) || (loadcount == 238) || 
                      (loadcount == 248) || (loadcount == 258) || (loadcount == 268) || 
                      (loadcount == 278) || (loadcount == 288) || (loadcount == 298) || 
                      (loadcount == 308) || (loadcount == 318) || (loadcount == 328) || 
                      (loadcount == 338) || (loadcount == 348) || (loadcount == 358))
                    {
                      String line8 = currentLoadList.data.toString();
                      cyt_tsp = line8.trim();
                      int pindex = pSearch(this.platetypes, this.cyt_tsp);
                      TabWindow.this.cytodatapanel2.setTSPIndex(pindex);
                      this.cytodatapanel2.revalidate();
                      this.cytodatapanel2.repaint();
                                     
                    }

              if ((loadcount == 10) || (loadcount == 20) || (loadcount == 30) || 
                (loadcount == 40) || (loadcount == 50) || (loadcount == 60) || 
                (loadcount == 70) || (loadcount == 80) || (loadcount == 90) || 
                (loadcount == 100) || (loadcount == 110) || (loadcount == 120) || 
                (loadcount == 130) || (loadcount == 140) || (loadcount == 150) || 
                (loadcount == 160) || (loadcount == 170) || (loadcount == 180) || 
                (loadcount == 190) || (loadcount == 200) || (loadcount == 210) || 
                (loadcount == 220) || (loadcount == 230) || (loadcount == 240) || 
                (loadcount == 250) || (loadcount == 260) || (loadcount == 270) || 
                (loadcount == 280) || (loadcount == 290) || (loadcount == 300) || 
                (loadcount == 310) || (loadcount == 320) || (loadcount == 330) || 
                (loadcount == 340) || (loadcount == 350) || (loadcount == 360))
              {
                loaddata = "";

                loaddata = loaddata + "   \n\n";
                loaddata = loaddata + "Cytomat " + this.cytloc + " to " + this.cytdeckloc + "\n\n";

                this.dList.insertAtBack(loaddata);
                loaddata = "";
              }
            }

            if (steptype.equals("Liconic01"))
            {

                if ((loadcount == 3) || (loadcount == 13) || (loadcount == 23) || 
                        (loadcount == 33) || (loadcount == 43) || (loadcount == 53) || 
                        (loadcount == 63) || (loadcount == 73) || (loadcount == 83) || 
                        (loadcount == 93) || (loadcount == 103) || (loadcount == 113) || 
                        (loadcount == 123) || (loadcount == 133) || (loadcount == 143) || 
                        (loadcount == 153) || (loadcount == 163) || (loadcount == 173) || 
                        (loadcount == 183) || (loadcount == 193) || (loadcount == 203) || 
                        (loadcount == 213) || (loadcount == 223) || (loadcount == 233) || 
                        (loadcount == 243) || (loadcount == 253) || (loadcount == 263) || 
                        (loadcount == 273) || (loadcount == 283) || (loadcount == 293) || 
                        (loadcount == 303) || (loadcount == 313) || (loadcount == 323) || 
                        (loadcount == 333) || (loadcount == 343) || (loadcount == 353))
                      {
                        this.licfromto = currentLoadList.data.toString();
                        int num = Integer.parseInt(licfromto);
                        TabWindow.this.licodatapanel2.setFTIndex(Integer.parseInt(licfromto));
                      }

                if ((loadcount == 4) || (loadcount == 14) || (loadcount == 24) || 
                (loadcount == 34) || (loadcount == 44) || (loadcount == 54) || 
                (loadcount == 64) || (loadcount == 74) || (loadcount == 84) || 
                (loadcount == 94) || (loadcount == 104) || (loadcount == 114) || 
                (loadcount == 124) || (loadcount == 134) || (loadcount == 144) || 
                (loadcount == 154) || (loadcount == 164) || (loadcount == 174) || 
                (loadcount == 184) || (loadcount == 194) || (loadcount == 204) || 
                (loadcount == 214) || (loadcount == 224) || (loadcount == 234) || 
                (loadcount == 244) || (loadcount == 254) || (loadcount == 264) || 
                (loadcount == 274) || (loadcount == 284) || (loadcount == 294) || 
                (loadcount == 304) || (loadcount == 314) || (loadcount == 324) || 
                (loadcount == 334) || (loadcount == 344) || (loadcount == 354))
              {
                String line4 = currentLoadList.data.toString();
                licloc = line4.trim();
              }
              if ((loadcount == 5) || (loadcount == 15) || (loadcount == 25) || 
                (loadcount == 35) || (loadcount == 45) || (loadcount == 55) || 
                (loadcount == 65) || (loadcount == 75) || (loadcount == 85) || 
                (loadcount == 95) || (loadcount == 105) || (loadcount == 115) || 
                (loadcount == 125) || (loadcount == 135) || (loadcount == 145) || 
                (loadcount == 155) || (loadcount == 165) || (loadcount == 175) || 
                (loadcount == 185) || (loadcount == 195) || (loadcount == 205) || 
                (loadcount == 215) || (loadcount == 225) || (loadcount == 235) || 
                (loadcount == 245) || (loadcount == 255) || (loadcount == 265) || 
                (loadcount == 275) || (loadcount == 285) || (loadcount == 295) || 
                (loadcount == 305) || (loadcount == 315) || (loadcount == 325) || 
                (loadcount == 335) || (loadcount == 345) || (loadcount == 355))
              {
                String line5 = currentLoadList.data.toString();
                licdeckloc = line5.trim();
              }

              if ((loadcount == 6) || (loadcount == 16) || (loadcount == 26) || 
                      (loadcount == 36) || (loadcount == 46) || (loadcount == 56) || 
                      (loadcount == 66) || (loadcount == 76) || (loadcount == 86) || 
                      (loadcount == 96) || (loadcount == 106) || (loadcount == 116) || 
                      (loadcount == 126) || (loadcount == 136) || (loadcount == 146) || 
                      (loadcount == 156) || (loadcount == 166) || (loadcount == 176) || 
                      (loadcount == 186) || (loadcount == 196) || (loadcount == 206) || 
                      (loadcount == 216) || (loadcount == 226) || (loadcount == 236) || 
                      (loadcount == 246) || (loadcount == 256) || (loadcount == 266) || 
                      (loadcount == 276) || (loadcount == 286) || (loadcount == 296) || 
                      (loadcount == 306) || (loadcount == 316) || (loadcount == 326) || 
                      (loadcount == 336) || (loadcount == 346) || (loadcount == 356))
                    {
                      this.liccover = currentLoadList.data.toString();
                      TabWindow.this.licodatapanel2.setCoverIndex(Integer.parseInt(liccover));
                    }

              if ((loadcount == 7) || (loadcount == 17) || (loadcount == 27) || 
                      (loadcount == 37) || (loadcount == 47) || (loadcount == 57) || 
                      (loadcount == 67) || (loadcount == 77) || (loadcount == 87) || 
                      (loadcount == 97) || (loadcount == 107) || (loadcount == 117) || 
                      (loadcount == 127) || (loadcount == 137) || (loadcount == 147) || 
                      (loadcount == 157) || (loadcount == 167) || (loadcount == 177) || 
                      (loadcount == 187) || (loadcount == 197) || (loadcount == 207) || 
                      (loadcount == 217) || (loadcount == 227) || (loadcount == 237) || 
                      (loadcount == 247) || (loadcount == 257) || (loadcount == 267) || 
                      (loadcount == 277) || (loadcount == 287) || (loadcount == 297) || 
                      (loadcount == 307) || (loadcount == 317) || (loadcount == 327) || 
                      (loadcount == 337) || (loadcount == 347) || (loadcount == 357))
                    {
                      String line7 = currentLoadList.data.toString();
                      lic_plate = line7.trim();
    				  int pindex = pAllSearch(this.platesAll, this.lic_plate);	
                      TabWindow.this.licodatapanel2.setPlateIndex(pindex);
                    }

              if ((loadcount == 8) || (loadcount == 18) || (loadcount == 28) || 
                      (loadcount == 38) || (loadcount == 48) || (loadcount == 58) || 
                      (loadcount == 68) || (loadcount == 78) || (loadcount == 88) || 
                      (loadcount == 98) || (loadcount == 108) || (loadcount == 118) || 
                      (loadcount == 128) || (loadcount == 138) || (loadcount == 148) || 
                      (loadcount == 158) || (loadcount == 168) || (loadcount == 178) || 
                      (loadcount == 188) || (loadcount == 198) || (loadcount == 208) || 
                      (loadcount == 218) || (loadcount == 228) || (loadcount == 238) || 
                      (loadcount == 248) || (loadcount == 258) || (loadcount == 268) || 
                      (loadcount == 278) || (loadcount == 288) || (loadcount == 298) || 
                      (loadcount == 308) || (loadcount == 318) || (loadcount == 328) || 
                      (loadcount == 338) || (loadcount == 348) || (loadcount == 358))
                    {
                      String line8 = currentLoadList.data.toString();
                      lic_tsp = line8.trim();
                      int pindex = pSearch(this.platetypes, this.lic_tsp);
                      TabWindow.this.licodatapanel2.setTSPIndex(pindex);
                      this.licodatapanel2.revalidate();
                      this.licodatapanel2.repaint();
                    }

              if ((loadcount == 10) || (loadcount == 20) || (loadcount == 30) || 
                (loadcount == 40) || (loadcount == 50) || (loadcount == 60) || 
                (loadcount == 70) || (loadcount == 80) || (loadcount == 90) || 
                (loadcount == 100) || (loadcount == 110) || (loadcount == 120) || 
                (loadcount == 130) || (loadcount == 140) || (loadcount == 150) || 
                (loadcount == 160) || (loadcount == 170) || (loadcount == 180) || 
                (loadcount == 190) || (loadcount == 200) || (loadcount == 210) || 
                (loadcount == 220) || (loadcount == 230) || (loadcount == 240) || 
                (loadcount == 250) || (loadcount == 260) || (loadcount == 270) || 
                (loadcount == 280) || (loadcount == 290) || (loadcount == 300) || 
                (loadcount == 310) || (loadcount == 320) || (loadcount == 330) || 
                (loadcount == 340) || (loadcount == 350) || (loadcount == 360))
              {
                loaddata = "";

                loaddata = loaddata + "   \n\n";
                loaddata = loaddata + "Liconic " + this.licloc + " to " + this.licdeckloc + "\n\n";

                this.dList.insertAtBack(loaddata);
                loaddata = "";
              }
            }

            if (steptype.equals("MovePlate02"))
            {
                if ((loadcount == 3) || (loadcount == 13) || (loadcount == 23) || 
                        (loadcount == 33) || (loadcount == 43) || (loadcount == 53) || 
                        (loadcount == 63) || (loadcount == 73) || (loadcount == 83) || 
                        (loadcount == 93) || (loadcount == 103) || (loadcount == 113) || 
                        (loadcount == 123) || (loadcount == 133) || (loadcount == 143) || 
                        (loadcount == 153) || (loadcount == 163) || (loadcount == 173) || 
                        (loadcount == 183) || (loadcount == 193) || (loadcount == 203) || 
                        (loadcount == 213) || (loadcount == 223) || (loadcount == 233) || 
                        (loadcount == 243) || (loadcount == 253) || (loadcount == 263) || 
                        (loadcount == 273) || (loadcount == 283) || (loadcount == 293) || 
                        (loadcount == 303) || (loadcount == 313) || (loadcount == 323) || 
                        (loadcount == 333) || (loadcount == 343) || (loadcount == 353))
                      {
                        this.movetype = currentLoadList.data.toString();
                        int num = Integer.parseInt(movetype);
                        TabWindow.this.movplv2datapanel2.setTypeIndex(Integer.parseInt(movetype));
                      }

                if ((loadcount == 4) || (loadcount == 14) || (loadcount == 24) || 
                (loadcount == 34) || (loadcount == 44) || (loadcount == 54) || 
                (loadcount == 64) || (loadcount == 74) || (loadcount == 84) || 
                (loadcount == 94) || (loadcount == 104) || (loadcount == 114) || 
                (loadcount == 124) || (loadcount == 134) || (loadcount == 144) || 
                (loadcount == 154) || (loadcount == 164) || (loadcount == 174) || 
                (loadcount == 184) || (loadcount == 194) || (loadcount == 204) || 
                (loadcount == 214) || (loadcount == 224) || (loadcount == 234) || 
                (loadcount == 244) || (loadcount == 254) || (loadcount == 264) || 
                (loadcount == 274) || (loadcount == 284) || (loadcount == 294) || 
                (loadcount == 304) || (loadcount == 314) || (loadcount == 324) || 
                (loadcount == 334) || (loadcount == 344) || (loadcount == 354))
              {
                String line4 = currentLoadList.data.toString();
                startloc = line4.trim();
              }
              if ((loadcount == 5) || (loadcount == 15) || (loadcount == 25) || 
                (loadcount == 35) || (loadcount == 45) || (loadcount == 55) || 
                (loadcount == 65) || (loadcount == 75) || (loadcount == 85) || 
                (loadcount == 95) || (loadcount == 105) || (loadcount == 115) || 
                (loadcount == 125) || (loadcount == 135) || (loadcount == 145) || 
                (loadcount == 155) || (loadcount == 165) || (loadcount == 175) || 
                (loadcount == 185) || (loadcount == 195) || (loadcount == 205) || 
                (loadcount == 215) || (loadcount == 225) || (loadcount == 235) || 
                (loadcount == 245) || (loadcount == 255) || (loadcount == 265) || 
                (loadcount == 275) || (loadcount == 285) || (loadcount == 295) || 
                (loadcount == 305) || (loadcount == 315) || (loadcount == 325) || 
                (loadcount == 335) || (loadcount == 345) || (loadcount == 355))
              {
                String line5 = currentLoadList.data.toString();
                endloc = line5.trim();
              }

              if ((loadcount == 6) || (loadcount == 16) || (loadcount == 26) || 
                      (loadcount == 36) || (loadcount == 46) || (loadcount == 56) || 
                      (loadcount == 66) || (loadcount == 76) || (loadcount == 86) || 
                      (loadcount == 96) || (loadcount == 106) || (loadcount == 116) || 
                      (loadcount == 126) || (loadcount == 136) || (loadcount == 146) || 
                      (loadcount == 156) || (loadcount == 166) || (loadcount == 176) || 
                      (loadcount == 186) || (loadcount == 196) || (loadcount == 206) || 
                      (loadcount == 216) || (loadcount == 226) || (loadcount == 236) || 
                      (loadcount == 246) || (loadcount == 256) || (loadcount == 266) || 
                      (loadcount == 276) || (loadcount == 286) || (loadcount == 296) || 
                      (loadcount == 306) || (loadcount == 316) || (loadcount == 326) || 
                      (loadcount == 336) || (loadcount == 346) || (loadcount == 356))
                    {
                  String line6 = currentLoadList.data.toString();
                  mvp_plate = line6.trim();

				  int pindex = pAllSearch(this.platesAll, this.mvp_plate);	
//				  int pindex = p96Search(this.plates96, this.mvp_plate);
//                  if (pindex == -1)
//                  {
//                    pindex = p384Search(this.plates384, this.mvp_plate);
//                    pindex += this.plates96.length;
//                  }
                  TabWindow.this.movplv2datapanel2.setPlateIndex(pindex);
                  this.movplv2datapanel2.revalidate();
                  this.movplv2datapanel2.repaint();
                    }


              if ((loadcount == 10) || (loadcount == 20) || (loadcount == 30) || 
                (loadcount == 40) || (loadcount == 50) || (loadcount == 60) || 
                (loadcount == 70) || (loadcount == 80) || (loadcount == 90) || 
                (loadcount == 100) || (loadcount == 110) || (loadcount == 120) || 
                (loadcount == 130) || (loadcount == 140) || (loadcount == 150) || 
                (loadcount == 160) || (loadcount == 170) || (loadcount == 180) || 
                (loadcount == 190) || (loadcount == 200) || (loadcount == 210) || 
                (loadcount == 220) || (loadcount == 230) || (loadcount == 240) || 
                (loadcount == 250) || (loadcount == 260) || (loadcount == 270) || 
                (loadcount == 280) || (loadcount == 290) || (loadcount == 300) || 
                (loadcount == 310) || (loadcount == 320) || (loadcount == 330) || 
                (loadcount == 340) || (loadcount == 350) || (loadcount == 360))
              {
                loaddata = "";

                loaddata = loaddata + "   \n\n";
                loaddata = loaddata + "Move Plate from " + this.startloc + " to " + this.endloc + "\n\n";

                this.dList.insertAtBack(loaddata);
                loaddata = "";
              }
            }

            if (steptype.equals("Cherry Picking"))
            {
              if ((loadcount == 3) || (loadcount == 13) || (loadcount == 23) || 
                (loadcount == 33) || (loadcount == 43) || (loadcount == 53) || 
                (loadcount == 63) || (loadcount == 73) || (loadcount == 83) || 
                (loadcount == 93) || (loadcount == 103) || (loadcount == 113) || 
                (loadcount == 123) || (loadcount == 133) || (loadcount == 143) || 
                (loadcount == 153) || (loadcount == 163) || (loadcount == 173) || 
                (loadcount == 183) || (loadcount == 193) || (loadcount == 203) || 
                (loadcount == 213) || (loadcount == 223) || (loadcount == 233) || 
                (loadcount == 243) || (loadcount == 253) || (loadcount == 263) || 
                (loadcount == 273) || (loadcount == 283) || (loadcount == 293) || 
                (loadcount == 303) || (loadcount == 313) || (loadcount == 323) || 
                (loadcount == 333) || (loadcount == 343) || (loadcount == 353))
              {
                String line3 = currentLoadList.data.toString();
                index1 = line3.indexOf(',');
                this.text7.setText(line3.substring(0, index1));
                len = line3.length();
                this.chltyp = line3.substring(index1 + 1, len);
              }

              if ((loadcount == 4) || (loadcount == 14) || (loadcount == 24) || 
                (loadcount == 34) || (loadcount == 44) || (loadcount == 54) || 
                (loadcount == 64) || (loadcount == 74) || (loadcount == 84) || 
                (loadcount == 94) || (loadcount == 104) || (loadcount == 114) || 
                (loadcount == 124) || (loadcount == 134) || (loadcount == 144) || 
                (loadcount == 154) || (loadcount == 164) || (loadcount == 174) || 
                (loadcount == 184) || (loadcount == 194) || (loadcount == 204) || 
                (loadcount == 214) || (loadcount == 224) || (loadcount == 234) || 
                (loadcount == 244) || (loadcount == 254) || (loadcount == 264) || 
                (loadcount == 274) || (loadcount == 284) || (loadcount == 294) || 
                (loadcount == 304) || (loadcount == 314) || (loadcount == 324) || 
                (loadcount == 334) || (loadcount == 344) || (loadcount == 354))
              {
                String line4 = currentLoadList.data.toString();
                index1 = line4.indexOf(',');
                len = line4.length();
                this.tr1 = line4.substring(0, index1);
                this.tr2 = line4.substring(index1 + 1, len);
              }

              if ((loadcount == 5) || (loadcount == 15) || (loadcount == 25) || 
                (loadcount == 35) || (loadcount == 45) || (loadcount == 55) || 
                (loadcount == 65) || (loadcount == 75) || (loadcount == 85) || 
                (loadcount == 95) || (loadcount == 105) || (loadcount == 115) || 
                (loadcount == 125) || (loadcount == 135) || (loadcount == 145) || 
                (loadcount == 155) || (loadcount == 165) || (loadcount == 175) || 
                (loadcount == 185) || (loadcount == 195) || (loadcount == 205) || 
                (loadcount == 215) || (loadcount == 225) || (loadcount == 235) || 
                (loadcount == 245) || (loadcount == 255) || (loadcount == 265) || 
                (loadcount == 275) || (loadcount == 285) || (loadcount == 295) || 
                (loadcount == 305) || (loadcount == 315) || (loadcount == 325) || 
                (loadcount == 335) || (loadcount == 345) || (loadcount == 355))
              {
                String line5 = currentLoadList.data.toString();
                len = line5.length();
                this.chso = line5.substring(0, len);
              }

              if ((loadcount == 6) || (loadcount == 16) || (loadcount == 26) || 
                (loadcount == 36) || (loadcount == 46) || (loadcount == 56) || 
                (loadcount == 66) || (loadcount == 76) || (loadcount == 86) || 
                (loadcount == 96) || (loadcount == 106) || (loadcount == 116) || 
                (loadcount == 126) || (loadcount == 136) || (loadcount == 146) || 
                (loadcount == 156) || (loadcount == 166) || (loadcount == 176) || 
                (loadcount == 186) || (loadcount == 196) || (loadcount == 206) || 
                (loadcount == 216) || (loadcount == 226) || (loadcount == 236) || 
                (loadcount == 246) || (loadcount == 256) || (loadcount == 266) || 
                (loadcount == 276) || (loadcount == 286) || (loadcount == 296) || 
                (loadcount == 306) || (loadcount == 316) || (loadcount == 326) || 
                (loadcount == 336) || (loadcount == 346) || (loadcount == 356))
              {
                String line6 = currentLoadList.data.toString();
                len = line6.length();
                this.chde = line6.substring(0, len);
              }

              if ((loadcount == 10) || (loadcount == 20) || (loadcount == 30) || 
                (loadcount == 40) || (loadcount == 50) || (loadcount == 60) || 
                (loadcount == 70) || (loadcount == 80) || (loadcount == 90) || 
                (loadcount == 100) || (loadcount == 110) || (loadcount == 120) || 
                (loadcount == 130) || (loadcount == 140) || (loadcount == 150) || 
                (loadcount == 160) || (loadcount == 170) || (loadcount == 180) || 
                (loadcount == 190) || (loadcount == 200) || (loadcount == 210) || 
                (loadcount == 220) || (loadcount == 230) || (loadcount == 240) || 
                (loadcount == 250) || (loadcount == 260) || (loadcount == 270) || 
                (loadcount == 280) || (loadcount == 290) || (loadcount == 300) || 
                (loadcount == 310) || (loadcount == 320) || (loadcount == 330) || 
                (loadcount == 340) || (loadcount == 350) || (loadcount == 360))
              {
                loaddata = "";

                loaddata = loaddata + "\n\n " + this.text7.getText().trim();
                loaddata = loaddata + "\n " + this.tr1.trim() + " to " + this.tr2.trim();

                this.dList.insertAtBack(loaddata);
                loaddata = "";
              }
            }

            if (steptype.equals("CherryPicking02"))
            {
              if ((loadcount == 3) || (loadcount == 13) || (loadcount == 23) || 
                (loadcount == 33) || (loadcount == 43) || (loadcount == 53) || 
                (loadcount == 63) || (loadcount == 73) || (loadcount == 83) || 
                (loadcount == 93) || (loadcount == 103) || (loadcount == 113) || 
                (loadcount == 123) || (loadcount == 133) || (loadcount == 143) || 
                (loadcount == 153) || (loadcount == 163) || (loadcount == 173) || 
                (loadcount == 183) || (loadcount == 193) || (loadcount == 203) || 
                (loadcount == 213) || (loadcount == 223) || (loadcount == 233) || 
                (loadcount == 243) || (loadcount == 253) || (loadcount == 263) || 
                (loadcount == 273) || (loadcount == 283) || (loadcount == 293) || 
                (loadcount == 303) || (loadcount == 313) || (loadcount == 323) || 
                (loadcount == 333) || (loadcount == 343) || (loadcount == 353))
              {
                String line3 = currentLoadList.data.toString();
                this.cherry02dataPanel2.setText2(line3);

              }

              if ((loadcount == 4) || (loadcount == 14) || (loadcount == 24) || 
                (loadcount == 34) || (loadcount == 44) || (loadcount == 54) || 
                (loadcount == 64) || (loadcount == 74) || (loadcount == 84) || 
                (loadcount == 94) || (loadcount == 104) || (loadcount == 114) || 
                (loadcount == 124) || (loadcount == 134) || (loadcount == 144) || 
                (loadcount == 154) || (loadcount == 164) || (loadcount == 174) || 
                (loadcount == 184) || (loadcount == 194) || (loadcount == 204) || 
                (loadcount == 214) || (loadcount == 224) || (loadcount == 234) || 
                (loadcount == 244) || (loadcount == 254) || (loadcount == 264) || 
                (loadcount == 274) || (loadcount == 284) || (loadcount == 294) || 
                (loadcount == 304) || (loadcount == 314) || (loadcount == 324) || 
                (loadcount == 334) || (loadcount == 344) || (loadcount == 354))
              {
                String line4 = currentLoadList.data.toString();
                ch2_plateloc = line4;
              }

              if ((loadcount == 5) || (loadcount == 15) || (loadcount == 25) || 
                (loadcount == 35) || (loadcount == 45) || (loadcount == 55) || 
                (loadcount == 65) || (loadcount == 75) || (loadcount == 85) || 
                (loadcount == 95) || (loadcount == 105) || (loadcount == 115) || 
                (loadcount == 125) || (loadcount == 135) || (loadcount == 145) || 
                (loadcount == 155) || (loadcount == 165) || (loadcount == 175) || 
                (loadcount == 185) || (loadcount == 195) || (loadcount == 205) || 
                (loadcount == 215) || (loadcount == 225) || (loadcount == 235) || 
                (loadcount == 245) || (loadcount == 255) || (loadcount == 265) || 
                (loadcount == 275) || (loadcount == 285) || (loadcount == 295) || 
                (loadcount == 305) || (loadcount == 315) || (loadcount == 325) || 
                (loadcount == 335) || (loadcount == 345) || (loadcount == 355))
              {

                String line5 = currentLoadList.data.toString();
                ch2_splate = line5.trim();
                int sindex = pAllSearch(this.platesAll, ch2_splate);	//AP-0001
                if (sindex == -1)
                {
                 cherry02dataPanel2.setSLabIndex(1);
                 sindex = tAllSearch(this.tubesAll, ch2_splate);	//AP-0001

                }
                else
                {
                 cherry02dataPanel2.setSLabIndex(2);	
                }
                TabWindow.this.cherry02dataPanel2.setSPlateIndex(sindex);
              }

              if ((loadcount == 6) || (loadcount == 16) || (loadcount == 26) || 
                (loadcount == 36) || (loadcount == 46) || (loadcount == 56) || 
                (loadcount == 66) || (loadcount == 76) || (loadcount == 86) || 
                (loadcount == 96) || (loadcount == 106) || (loadcount == 116) || 
                (loadcount == 126) || (loadcount == 136) || (loadcount == 146) || 
                (loadcount == 156) || (loadcount == 166) || (loadcount == 176) || 
                (loadcount == 186) || (loadcount == 196) || (loadcount == 206) || 
                (loadcount == 216) || (loadcount == 226) || (loadcount == 236) || 
                (loadcount == 246) || (loadcount == 256) || (loadcount == 266) || 
                (loadcount == 276) || (loadcount == 286) || (loadcount == 296) || 
                (loadcount == 306) || (loadcount == 316) || (loadcount == 326) || 
                (loadcount == 336) || (loadcount == 346) || (loadcount == 356))
              {
                String line6 = currentLoadList.data.toString();
                ch2_dplate = line6.trim();
                int dindex = pAllSearch(this.platesAll, ch2_dplate);	//AP-0001
                if (dindex == -1)
                {
                 cherry02dataPanel2.setDLabIndex(1);
                 dindex = tAllSearch(this.tubesAll, ch2_dplate);	//AP-0001
                }
                else
                {
                 cherry02dataPanel2.setDLabIndex(2);	
                }
                TabWindow.this.cherry02dataPanel2.setDPlateIndex(dindex);   //AP-0001
              }

              if ((loadcount == 7) || (loadcount == 17) || (loadcount == 27) || 
                      (loadcount == 37) || (loadcount == 47) || (loadcount == 57) || 
                      (loadcount == 67) || (loadcount == 77) || (loadcount == 87) || 
                      (loadcount == 97) || (loadcount == 107) || (loadcount == 117) || 
                      (loadcount == 127) || (loadcount == 137) || (loadcount == 147) || 
                      (loadcount == 157) || (loadcount == 167) || (loadcount == 177) || 
                      (loadcount == 187) || (loadcount == 197) || (loadcount == 207) || 
                      (loadcount == 217) || (loadcount == 227) || (loadcount == 237) || 
                      (loadcount == 247) || (loadcount == 257) || (loadcount == 267) || 
                      (loadcount == 277) || (loadcount == 287) || (loadcount == 297) || 
                      (loadcount == 307) || (loadcount == 317) || (loadcount == 327) || 
                      (loadcount == 337) || (loadcount == 347) || (loadcount == 357))
                    {
                      String line7 = currentLoadList.data.toString();
                      len = line7.length();
                      this.ch2_liqtype = line7;
                      int lindex = lSearch(this.liqtypes, this.ch2_liqtype);
                      this.cherry02dataPanel2.setLiqIndex(lindex);
                    }

              if ((loadcount == 8) || (loadcount == 18) || (loadcount == 28) || 
                      (loadcount == 38) || (loadcount == 48) || (loadcount == 58) || 
                      (loadcount == 68) || (loadcount == 78) || (loadcount == 88) || 
                      (loadcount == 98) || (loadcount == 108) || (loadcount == 118) || 
                      (loadcount == 128) || (loadcount == 138) || (loadcount == 148) || 
                      (loadcount == 158) || (loadcount == 168) || (loadcount == 178) || 
                      (loadcount == 188) || (loadcount == 198) || (loadcount == 208) || 
                      (loadcount == 218) || (loadcount == 228) || (loadcount == 238) || 
                      (loadcount == 248) || (loadcount == 258) || (loadcount == 268) || 
                      (loadcount == 278) || (loadcount == 288) || (loadcount == 298) || 
                      (loadcount == 308) || (loadcount == 318) || (loadcount == 328) || 
                      (loadcount == 338) || (loadcount == 348) || (loadcount == 358))
                    {
            	  	  String line8 = currentLoadList.data.toString();
            	  	  index1 = line8.indexOf(',');
            	  	  len = line8.length();
            	  	  TabWindow.this.ch2_scycle = line8.substring(0, index1);
            	  	  TabWindow.this.ch2_smix = line8.substring(index1 + 1, len);
            	  	  TabWindow.this.cherry02dataPanel2.setSCycle1(Integer.parseInt(ch2_scycle));
//            	  	  TabWindow.this.cherry02dataPanel2.setSCycleIndex(Integer.parseInt(ch2_scycle));
            	  	  TabWindow.this.cherry02dataPanel2.setText3(ch2_smix);

                    }

              if ((loadcount == 9) || (loadcount == 19) || (loadcount == 29) || 
                      (loadcount == 39) || (loadcount == 49) || (loadcount == 59) || 
                      (loadcount == 69) || (loadcount == 79) || (loadcount == 89) || 
                      (loadcount == 99) || (loadcount == 109) || (loadcount == 119) || 
                      (loadcount == 129) || (loadcount == 139) || (loadcount == 149) || 
                      (loadcount == 159) || (loadcount == 169) || (loadcount == 179) || 
                      (loadcount == 189) || (loadcount == 199) || (loadcount == 209) || 
                      (loadcount == 219) || (loadcount == 229) || (loadcount == 239) || 
                      (loadcount == 249) || (loadcount == 259) || (loadcount == 269) || 
                      (loadcount == 279) || (loadcount == 289) || (loadcount == 299) || 
                      (loadcount == 309) || (loadcount == 319) || (loadcount == 329) || 
                      (loadcount == 339) || (loadcount == 349) || (loadcount == 359))
                    {
            	  	  String line9 = currentLoadList.data.toString();
            	  	  index1 = line9.indexOf(',');
            	  	  len = line9.length();
            	  	  TabWindow.this.ch2_dcycle = line9.substring(0, index1);
            	  	  TabWindow.this.ch2_dmix = line9.substring(index1 + 1, len);
//            	  	  TabWindow.this.cherry02dataPanel2.setDCycleIndex(Integer.parseInt(ch2_dcycle));
            	  	  TabWindow.this.cherry02dataPanel2.setDCycle1(Integer.parseInt(ch2_dcycle));
            	  	  TabWindow.this.cherry02dataPanel2.setText4(ch2_dmix);
                      TabWindow.this.cherry02dataPanel2.revalidate();
                      TabWindow.this.cherry02dataPanel2.repaint();
                    }

              if ((loadcount == 10) || (loadcount == 20) || (loadcount == 30) || 
                (loadcount == 40) || (loadcount == 50) || (loadcount == 60) || 
                (loadcount == 70) || (loadcount == 80) || (loadcount == 90) || 
                (loadcount == 100) || (loadcount == 110) || (loadcount == 120) || 
                (loadcount == 130) || (loadcount == 140) || (loadcount == 150) || 
                (loadcount == 160) || (loadcount == 170) || (loadcount == 180) || 
                (loadcount == 190) || (loadcount == 200) || (loadcount == 210) || 
                (loadcount == 220) || (loadcount == 230) || (loadcount == 240) || 
                (loadcount == 250) || (loadcount == 260) || (loadcount == 270) || 
                (loadcount == 280) || (loadcount == 290) || (loadcount == 300) || 
                (loadcount == 310) || (loadcount == 320) || (loadcount == 330) || 
                (loadcount == 340) || (loadcount == 350) || (loadcount == 360))
              {
                  String line10 = currentLoadList.data.toString();
                  ch2_tubeloc = line10;
                  if (ch2_plateloc.equals("null") && ch2_tubeloc.equals("null"))
                  {
                  	this.ch2_plateloc = "";
                  	this.ch2_tubeloc = "";
                  	this.cherry02dataPanel2.setT1Enabled(false);
                  	this.cherry02dataPanel2.setT1TEnabled(false);
                  	this.cherry02dataPanel2.setBarIndex(0);
                  }
                  else if (ch2_plateloc.equals("null"))
                  {
                   	this.ch2_plateloc = "";
                  	this.cherry02dataPanel2.setT1Enabled(false);
                   	this.cherry02dataPanel2.setT1TEnabled(true);
                  	this.cherry02dataPanel2.setBarIndex(1);
                  }
                  else if (ch2_tubeloc.equals("null"))
                  {
                	this.ch2_tubeloc = "";
                   	this.cherry02dataPanel2.setT1TEnabled(false);
                   	this.cherry02dataPanel2.setT1Enabled(true);
                  	this.cherry02dataPanel2.setBarIndex(1);
                  }
                  else 
                  {
                   	this.cherry02dataPanel2.setT1TEnabled(true);
                   	this.cherry02dataPanel2.setT1Enabled(true);
                  	this.cherry02dataPanel2.setBarIndex(1);
                  }
 					this.cherry02dataPanel2.setText1(ch2_plateloc);
  					this.cherry02dataPanel2.setText1T(ch2_tubeloc);

  					loaddata = "";

            	String fileName = cherry02dataPanel2.getWorkFile().trim().substring(cherry02dataPanel2.getWorkFile().trim().lastIndexOf("\\") + 1);
            	
        		if (fileName.length() > 12){fileName = new StringBuilder(fileName).insert(12, "...\n").toString();}		//CPF0032
        		if (fileName.length() > 27)fileName = new StringBuilder(fileName).insert(27, "...\n").toString();		//CPF0032
        		
                loaddata = loaddata + "\n\n" + fileName;																//CPF0032
                //loaddata = loaddata + "\n " + this.tr1.trim() + " to " + this.tr2.trim();

                this.dList.insertAtBack(loaddata);
                loaddata = "";
              }
            }

            if (steptype.equals("CORDNA01"))
            {
                if ((loadcount == 3) || (loadcount == 13) || (loadcount == 23) || 
                        (loadcount == 33) || (loadcount == 43) || (loadcount == 53) || 
                        (loadcount == 63) || (loadcount == 73) || (loadcount == 83) || 
                        (loadcount == 93) || (loadcount == 103) || (loadcount == 113) || 
                        (loadcount == 123) || (loadcount == 133) || (loadcount == 143) || 
                        (loadcount == 153) || (loadcount == 163) || (loadcount == 173) || 
                        (loadcount == 183) || (loadcount == 193) || (loadcount == 203) || 
                        (loadcount == 213) || (loadcount == 223) || (loadcount == 233) || 
                        (loadcount == 243) || (loadcount == 253) || (loadcount == 263) || 
                        (loadcount == 273) || (loadcount == 283) || (loadcount == 293) || 
                        (loadcount == 303) || (loadcount == 313) || (loadcount == 323) || 
                        (loadcount == 333) || (loadcount == 343) || (loadcount == 353))
                      {
                		String line3 = currentLoadList.data.toString();
                		dna_qfile = line3.trim();
                        TabWindow.this.dnadataPanel2.setText(dna_qfile);
                      }

                if ((loadcount == 4) || (loadcount == 14) || (loadcount == 24) || 
                (loadcount == 34) || (loadcount == 44) || (loadcount == 54) || 
                (loadcount == 64) || (loadcount == 74) || (loadcount == 84) || 
                (loadcount == 94) || (loadcount == 104) || (loadcount == 114) || 
                (loadcount == 124) || (loadcount == 134) || (loadcount == 144) || 
                (loadcount == 154) || (loadcount == 164) || (loadcount == 174) || 
                (loadcount == 184) || (loadcount == 194) || (loadcount == 204) || 
                (loadcount == 214) || (loadcount == 224) || (loadcount == 234) || 
                (loadcount == 244) || (loadcount == 254) || (loadcount == 264) || 
                (loadcount == 274) || (loadcount == 284) || (loadcount == 294) || 
                (loadcount == 304) || (loadcount == 314) || (loadcount == 324) || 
                (loadcount == 334) || (loadcount == 344) || (loadcount == 354))
              {
                    this.proc = currentLoadList.data.toString();
                    int num = Integer.parseInt(proc);
                    TabWindow.this.dnadataPanel2.setProcIndex(Integer.parseInt(proc));
              }
              if ((loadcount == 5) || (loadcount == 15) || (loadcount == 25) || 
                (loadcount == 35) || (loadcount == 45) || (loadcount == 55) || 
                (loadcount == 65) || (loadcount == 75) || (loadcount == 85) || 
                (loadcount == 95) || (loadcount == 105) || (loadcount == 115) || 
                (loadcount == 125) || (loadcount == 135) || (loadcount == 145) || 
                (loadcount == 155) || (loadcount == 165) || (loadcount == 175) || 
                (loadcount == 185) || (loadcount == 195) || (loadcount == 205) || 
                (loadcount == 215) || (loadcount == 225) || (loadcount == 235) || 
                (loadcount == 245) || (loadcount == 255) || (loadcount == 265) || 
                (loadcount == 275) || (loadcount == 285) || (loadcount == 295) || 
                (loadcount == 305) || (loadcount == 315) || (loadcount == 325) || 
                (loadcount == 335) || (loadcount == 345) || (loadcount == 355))
              {
                String line5 = currentLoadList.data.toString();
                index1 = line5.indexOf(',');
                len = line5.length();
                this.dna_con = line5.substring(0, index1);
                this.dna_vol = line5.substring(index1 + 1, len);
                TabWindow.this.dnadataPanel2.setText1(dna_con);
                TabWindow.this.dnadataPanel2.setText2(dna_vol);
              }

              if ((loadcount == 6) || (loadcount == 16) || (loadcount == 26) || 
                      (loadcount == 36) || (loadcount == 46) || (loadcount == 56) || 
                      (loadcount == 66) || (loadcount == 76) || (loadcount == 86) || 
                      (loadcount == 96) || (loadcount == 106) || (loadcount == 116) || 
                      (loadcount == 126) || (loadcount == 136) || (loadcount == 146) || 
                      (loadcount == 156) || (loadcount == 166) || (loadcount == 176) || 
                      (loadcount == 186) || (loadcount == 196) || (loadcount == 206) || 
                      (loadcount == 216) || (loadcount == 226) || (loadcount == 236) || 
                      (loadcount == 246) || (loadcount == 256) || (loadcount == 266) || 
                      (loadcount == 276) || (loadcount == 286) || (loadcount == 296) || 
                      (loadcount == 306) || (loadcount == 316) || (loadcount == 326) || 
                      (loadcount == 336) || (loadcount == 346) || (loadcount == 356))
                    {
                  String line6 = currentLoadList.data.toString();
                  index1 = line6.indexOf(',');
                  index2 = line6.indexOf(',', index1 + 1);
                  index3 = line6.indexOf(',', index2 + 1);
                  len = line6.length();
                  this.dna_dplate = line6.substring(0, index1);
                  this.dna_splate = line6.substring(index1 + 1, index2);
                  this.dna_nplate = line6.substring(index2 + 1, index3);
                  this.dna_plate = line6.substring(index3 + 1, len);
                  int dindex = rAllSearch(this.reservoirsAll, dna_dplate);	//AP-0001
                  int sindex = tAllSearch(this.tubesAll, dna_splate);	//AP-0001
                  int nindex = pPATSearch(this.platesTubes, dna_nplate);	//AP-0001
                  int pindex = pPATSearch(this.platesTubes, dna_plate);	//AP-0001
                  TabWindow.this.dnadataPanel2.setDPlateIndex(dindex);
                  TabWindow.this.dnadataPanel2.setSPlateIndex(sindex);
                  TabWindow.this.dnadataPanel2.setNPlateIndex(nindex);
                  TabWindow.this.dnadataPanel2.setPlateIndex(pindex);
                    }

              if ((loadcount == 7) || (loadcount == 17) || (loadcount == 27) || 
                      (loadcount == 37) || (loadcount == 47) || (loadcount == 57) || 
                      (loadcount == 67) || (loadcount == 77) || (loadcount == 87) || 
                      (loadcount == 97) || (loadcount == 107) || (loadcount == 117) || 
                      (loadcount == 127) || (loadcount == 137) || (loadcount == 147) || 
                      (loadcount == 157) || (loadcount == 167) || (loadcount == 177) || 
                      (loadcount == 187) || (loadcount == 197) || (loadcount == 207) || 
                      (loadcount == 217) || (loadcount == 227) || (loadcount == 237) || 
                      (loadcount == 247) || (loadcount == 257) || (loadcount == 267) || 
                      (loadcount == 277) || (loadcount == 287) || (loadcount == 297) || 
                      (loadcount == 307) || (loadcount == 317) || (loadcount == 327) || 
                      (loadcount == 337) || (loadcount == 347) || (loadcount == 357))
                    {
                  String line7 = currentLoadList.data.toString();
                  index1 = line7.indexOf(',');
                  index2 = line7.indexOf(',', index1 + 1);
                  index3 = line7.indexOf(',', index2 + 1);
                  len = line7.length();
                  this.dna_deck1 = line7.substring(0, index1);
                  this.dna_deck2 = line7.substring(index1 + 1, index2);
                  this.dna_deck3 = line7.substring(index2 + 1, index3);
                  this.dna_deck4 = line7.substring(index3 + 1, len);
                  TabWindow.this.dnadataPanel2.setDeckIndex1(Integer.parseInt(dna_deck1));
                  TabWindow.this.dnadataPanel2.setDeckIndex2(Integer.parseInt(dna_deck2));
                  TabWindow.this.dnadataPanel2.setDeckIndex3(Integer.parseInt(dna_deck3));
                  TabWindow.this.dnadataPanel2.setDeckIndex4(Integer.parseInt(dna_deck4));
                    }

              if ((loadcount == 10) || (loadcount == 20) || (loadcount == 30) || 
                (loadcount == 40) || (loadcount == 50) || (loadcount == 60) || 
                (loadcount == 70) || (loadcount == 80) || (loadcount == 90) || 
                (loadcount == 100) || (loadcount == 110) || (loadcount == 120) || 
                (loadcount == 130) || (loadcount == 140) || (loadcount == 150) || 
                (loadcount == 160) || (loadcount == 170) || (loadcount == 180) || 
                (loadcount == 190) || (loadcount == 200) || (loadcount == 210) || 
                (loadcount == 220) || (loadcount == 230) || (loadcount == 240) || 
                (loadcount == 250) || (loadcount == 260) || (loadcount == 270) || 
                (loadcount == 280) || (loadcount == 290) || (loadcount == 300) || 
                (loadcount == 310) || (loadcount == 320) || (loadcount == 330) || 
                (loadcount == 340) || (loadcount == 350) || (loadcount == 360))
              {
                loaddata = "";

                loaddata = loaddata + "   \n\n";
                loaddata = loaddata + "DNA Normalization " + this.dna_con + " and " + this.dna_vol + "\n\n";

                this.dList.insertAtBack(loaddata);
                loaddata = "";
              }
            }

            if (steptype.equals("LOAD01"))
            {
				if ((loadcount == 3) || (loadcount == 13) || (loadcount == 23) || 
                        (loadcount == 33) || (loadcount == 43) || (loadcount == 53) || 
                        (loadcount == 63) || (loadcount == 73) || (loadcount == 83) || 
                        (loadcount == 93) || (loadcount == 103) || (loadcount == 113) || 
                        (loadcount == 123) || (loadcount == 133) || (loadcount == 143) || 
                        (loadcount == 153) || (loadcount == 163) || (loadcount == 173) || 
                        (loadcount == 183) || (loadcount == 193) || (loadcount == 203) || 
                        (loadcount == 213) || (loadcount == 223) || (loadcount == 233) || 
                        (loadcount == 243) || (loadcount == 253) || (loadcount == 263) || 
                        (loadcount == 273) || (loadcount == 283) || (loadcount == 293) || 
                        (loadcount == 303) || (loadcount == 313) || (loadcount == 323) || 
                        (loadcount == 333) || (loadcount == 343) || (loadcount == 353))
                      {
                		String line3 = currentLoadList.data.toString();
                        index1 = line3.indexOf(',');
                        len = line3.length();
                        this.load_plindex = line3.substring(0, index1);
                        int num = Integer.parseInt(load_plindex);
                        this.load_plate = line3.substring(index1 + 1, len);
                        TabWindow.this.loaddataPanel2.setPlateIndex(num);
                        TabWindow.this.loaddataPanel2.setPlateText(load_plate);
                      }

                if ((loadcount == 4) || (loadcount == 14) || (loadcount == 24) || 
                (loadcount == 34) || (loadcount == 44) || (loadcount == 54) || 
                (loadcount == 64) || (loadcount == 74) || (loadcount == 84) || 
                (loadcount == 94) || (loadcount == 104) || (loadcount == 114) || 
                (loadcount == 124) || (loadcount == 134) || (loadcount == 144) || 
                (loadcount == 154) || (loadcount == 164) || (loadcount == 174) || 
                (loadcount == 184) || (loadcount == 194) || (loadcount == 204) || 
                (loadcount == 214) || (loadcount == 224) || (loadcount == 234) || 
                (loadcount == 244) || (loadcount == 254) || (loadcount == 264) || 
                (loadcount == 274) || (loadcount == 284) || (loadcount == 294) || 
                (loadcount == 304) || (loadcount == 314) || (loadcount == 324) || 
                (loadcount == 334) || (loadcount == 344) || (loadcount == 354))
              {
            		String line4 = currentLoadList.data.toString();
                    index1 = line4.indexOf(',');
                    len = line4.length();
                    this.load_tuindex = line4.substring(0, index1);
                    int num = Integer.parseInt(load_tuindex);
                    this.load_tube = line4.substring(index1 + 1, len);
                    TabWindow.this.loaddataPanel2.setTubeIndex(num);
                    TabWindow.this.loaddataPanel2.setTubeText(load_tube);
              }
              if ((loadcount == 5) || (loadcount == 15) || (loadcount == 25) || 
                (loadcount == 35) || (loadcount == 45) || (loadcount == 55) || 
                (loadcount == 65) || (loadcount == 75) || (loadcount == 85) || 
                (loadcount == 95) || (loadcount == 105) || (loadcount == 115) || 
                (loadcount == 125) || (loadcount == 135) || (loadcount == 145) || 
                (loadcount == 155) || (loadcount == 165) || (loadcount == 175) || 
                (loadcount == 185) || (loadcount == 195) || (loadcount == 205) || 
                (loadcount == 215) || (loadcount == 225) || (loadcount == 235) || 
                (loadcount == 245) || (loadcount == 255) || (loadcount == 265) || 
                (loadcount == 275) || (loadcount == 285) || (loadcount == 295) || 
                (loadcount == 305) || (loadcount == 315) || (loadcount == 325) || 
                (loadcount == 335) || (loadcount == 345) || (loadcount == 355))
              {
          		String line5 = currentLoadList.data.toString();
                index1 = line5.indexOf(',');
                len = line5.length();
                this.load_tiindex = line5.substring(0, index1);
                int num = Integer.parseInt(load_tiindex);
                this.load_tip = line5.substring(index1 + 1, len);
                TabWindow.this.loaddataPanel2.setTipIndex(num);
                TabWindow.this.loaddataPanel2.setTipText(load_tube);
              }


              if ((loadcount == 10) || (loadcount == 20) || (loadcount == 30) || 
                (loadcount == 40) || (loadcount == 50) || (loadcount == 60) || 
                (loadcount == 70) || (loadcount == 80) || (loadcount == 90) || 
                (loadcount == 100) || (loadcount == 110) || (loadcount == 120) || 
                (loadcount == 130) || (loadcount == 140) || (loadcount == 150) || 
                (loadcount == 160) || (loadcount == 170) || (loadcount == 180) || 
                (loadcount == 190) || (loadcount == 200) || (loadcount == 210) || 
                (loadcount == 220) || (loadcount == 230) || (loadcount == 240) || 
                (loadcount == 250) || (loadcount == 260) || (loadcount == 270) || 
                (loadcount == 280) || (loadcount == 290) || (loadcount == 300) || 
                (loadcount == 310) || (loadcount == 320) || (loadcount == 330) || 
                (loadcount == 340) || (loadcount == 350) || (loadcount == 360))
              {
                loaddata = "";

                loaddata = loaddata + "   \n\n";
                loaddata = loaddata + "Load " + "\n\n";

                this.dList.insertAtBack(loaddata);
                loaddata = "";
              }
            }

            if (steptype.equals("PK"))
            {
              if ((loadcount == 3) || (loadcount == 13) || (loadcount == 23) || 
                (loadcount == 33) || (loadcount == 43) || (loadcount == 53) || 
                (loadcount == 63) || (loadcount == 73) || (loadcount == 83) || 
                (loadcount == 93) || (loadcount == 103) || (loadcount == 113) || 
                (loadcount == 123) || (loadcount == 133) || (loadcount == 143) || 
                (loadcount == 153) || (loadcount == 163) || (loadcount == 173) || 
                (loadcount == 183) || (loadcount == 193) || (loadcount == 203) || 
                (loadcount == 213) || (loadcount == 223) || (loadcount == 233) || 
                (loadcount == 243) || (loadcount == 253) || (loadcount == 263) || 
                (loadcount == 273) || (loadcount == 283) || (loadcount == 293) || 
                (loadcount == 303) || (loadcount == 313) || (loadcount == 323) || 
                (loadcount == 333) || (loadcount == 343) || (loadcount == 353))
              {
                String line3 = currentLoadList.data.toString();
                len = line3.length();
                index1 = line3.indexOf(',');
                index2 = line3.indexOf(',', index1 + 1);
                index3 = line3.indexOf(',', index2 + 1);
                index4 = line3.indexOf(',', index3 + 1);

				this.pkdataPanel2x.setpkmtextwat(line3.substring(0, index1));
				this.pkdataPanel2x.setpkmtext1(line3.substring(index1 + 1, index2));
				this.pkdataPanel2x.setpkmtextul(line3.substring(index2 + 1, index3));
				this.pkdataPanel2x.setpkmpassayul(line3.substring(index3 + 1, index4));
				this.pkdataPanel2x.setpkmassayul(line3.substring(index4 + 1, len));
				
//				this.pkm_text_wat.setText(line3.substring(0, index1));
//				this.pkm_text_1.setText(line3.substring(index1 + 1, index2));
//				this.pkm_text_ul.setText(line3.substring(index2 + 1, index3));
//				this.pkm_text_ul_preAssay.setText(line3.substring(index3 + 1, index4));
//				this.pkm_text_ul_Assay.setText(line3.substring(index4 + 1, len));
              }

              if ((loadcount == 4) || (loadcount == 14) || (loadcount == 24) || 
                      (loadcount == 34) || (loadcount == 44) || (loadcount == 54) || 
                      (loadcount == 64) || (loadcount == 74) || (loadcount == 84) || 
                      (loadcount == 94) || (loadcount == 104) || (loadcount == 114) || 
                      (loadcount == 124) || (loadcount == 134) || (loadcount == 144) || 
                      (loadcount == 154) || (loadcount == 164) || (loadcount == 174) || 
                      (loadcount == 184) || (loadcount == 194) || (loadcount == 204) || 
                      (loadcount == 214) || (loadcount == 224) || (loadcount == 234) || 
                      (loadcount == 244) || (loadcount == 254) || (loadcount == 264) || 
                      (loadcount == 274) || (loadcount == 284) || (loadcount == 294) || 
                      (loadcount == 304) || (loadcount == 314) || (loadcount == 324) || 
                      (loadcount == 334) || (loadcount == 344) || (loadcount == 354))
                    {
            	  
                  String line4 = currentLoadList.data.toString();
                  len = line4.length();
                  index1 = line4.indexOf(',');
                  index2 = line4.indexOf(',', index1 + 1);
                  index3 = line4.indexOf(',', index2 + 1);
                  index4 = line4.indexOf(',', index3 + 1);

    				this.pksam = line4.substring(0, index1);
      				this.pkstd = line4.substring(index1 + 1, index2);
      				this.pkmrddil = line4.substring(index2 + 1, index3);
      				this.pkdildil = line4.substring(index3 + 1, len);

                    }
              
              
              if ((loadcount == 5) || (loadcount == 15) || (loadcount == 25) || 
                (loadcount == 35) || (loadcount == 45) || (loadcount == 55) || 
                (loadcount == 65) || (loadcount == 75) || (loadcount == 85) || 
                (loadcount == 95) || (loadcount == 105) || (loadcount == 115) || 
                (loadcount == 125) || (loadcount == 135) || (loadcount == 145) || 
                (loadcount == 155) || (loadcount == 165) || (loadcount == 175) || 
                (loadcount == 185) || (loadcount == 195) || (loadcount == 205) || 
                (loadcount == 215) || (loadcount == 225) || (loadcount == 235) || 
                (loadcount == 245) || (loadcount == 255) || (loadcount == 265) || 
                (loadcount == 275) || (loadcount == 285) || (loadcount == 295) || 
                (loadcount == 305) || (loadcount == 315) || (loadcount == 325) || 
                (loadcount == 335) || (loadcount == 345) || (loadcount == 355))
              {
                String line5 = currentLoadList.data.toString();
                len = line5.length();
                index1 = line5.indexOf(',');
                index2 = line5.indexOf(',', index1 + 1);
                index3 = line5.indexOf(',', index2 + 1);
                index4 = line5.indexOf(',', index3 + 1);
                index5 = line5.indexOf(',', index4 + 1);
                this.pksamlab = line5.substring(0, index1);
                this.pkstdlab = line5.substring(index1 + 1, index2);
                this.pkmrdlab = line5.substring(index2 + 1, index3);
                this.pkdillab = line5.substring(index3 + 1, index4);
                this.pkpasslab = line5.substring(index4 + 1, index5);
                this.pkasslab = line5.substring(index5 + 1, len);
                int sindex = p96tubeSearch(this.plates96tubes, this.pksamlab);
                if (sindex == -1)
                {
                  sindex = 0 ;
                }
                this.pkdataPanel2x.setsamindex(sindex);
//                this.pkSsource.setSelectedIndex(sindex);

                sindex = p96Search(this.plates96, pkdillab);
                if (sindex == -1)
                {
                  sindex = 0 ;
                }
                this.pkdataPanel2x.setdilsrcindex2(sindex + 1);
//                this.pkDsource.setSelectedIndex(sindex);
                
                sindex = p96Search(this.plates96, pkpasslab);
                if (sindex == -1)
                {
                  sindex = 0 ;
                }
                this.pkdataPanel2x.setpassaylabindex(sindex + 1);
//                this.pkPassay.setSelectedIndex(sindex);
                
                sindex = p96tubeSearch(this.plates96tubes, pkstdlab);
                if (sindex == -1)
                {
                  sindex = 0 ;
                }
                this.pkdataPanel2x.setstdindex(sindex);
//                this.pkSTDsource.setSelectedIndex(sindex);
                
                sindex = p96Search(this.plates96, pkasslab);
                if (sindex == -1)
                {
                  sindex = 0 ;
                }
                this.pkdataPanel2x.setassaylabindex(sindex + 1);
              }

              if ((loadcount == 6) || (loadcount == 16) || (loadcount == 26) || 
                (loadcount == 36) || (loadcount == 46) || (loadcount == 56) || 
                (loadcount == 66) || (loadcount == 76) || (loadcount == 86) || 
                (loadcount == 96) || (loadcount == 106) || (loadcount == 116) || 
                (loadcount == 126) || (loadcount == 136) || (loadcount == 146) || 
                (loadcount == 156) || (loadcount == 166) || (loadcount == 176) || 
                (loadcount == 186) || (loadcount == 196) || (loadcount == 206) || 
                (loadcount == 216) || (loadcount == 226) || (loadcount == 236) || 
                (loadcount == 246) || (loadcount == 256) || (loadcount == 266) || 
                (loadcount == 276) || (loadcount == 286) || (loadcount == 296) || 
                (loadcount == 306) || (loadcount == 316) || (loadcount == 326) || 
                (loadcount == 336) || (loadcount == 346) || (loadcount == 356))
              {
                String line6 = currentLoadList.data.toString();
                len = line6.length();
                index1 = line6.indexOf(',');
                index2 = line6.indexOf(',', index1 + 1);
                index3 = line6.indexOf(',', index2 + 1);
                index4 = line6.indexOf(',', index3 + 1);
                
                this.pkck1 = line6.substring(0, index1);
                this.pkck2 = line6.substring(index1 + 1, index2);
                this.pkck3 = line6.substring(index2 + 1, index3);
                this.pkck4 = line6.substring(index3 + 1, len);
                if (pkck1.equals("1"))
                {
                    this.pkdataPanel2x.setMRDon(true);
//                	pkdataPanel2x.mrdon = true;
//                  pkcheck1.setSelected(true);
                }
                else
                {
                    this.pkdataPanel2x.setMRDon(false);
//                	pkdataPanel2x.mrdon = false;
                  if (!pkck2.equals("1"))
                  {
                      this.pkdataPanel2x.setDILon(false);
//                  	pkdataPanel2x.dilon = false;
                  }
                }
                if (pkck2.equals("1"))
                {
                  this.pkdataPanel2x.setDILon(true);
//                  pkdataPanel2x.dilon = true;
                }
                else
                {
                  this.pkdataPanel2x.setDILon(false);
//                  pkdataPanel2x.dilon = true;
                  if (!pkck1.equals("1"))
                  {
        	    	  // TabWindow.this.pkMRDsource.setEnabled(false);
        	    	  // TabWindow.this.pkMRDdil.setEnabled(false);
                  }
                }
                if (pkck3.equals("1"))
                {
                  this.pkdataPanel2x.setPRAon(true);
//                  pkdataPanel2x.preassayon = true;
                }
                else
                {
                    this.pkdataPanel2x.setPRAon(false);
//                  pkdataPanel2x.preassayon = false;
//                  if (!pkcheck2.isSelected())
//                  {
//                   pkcheck3.setEnabled(true);
//                  }
                }

                if (pkck4.equals("1"))
                {
                    this.pkdataPanel2x.setASSon(true);
//                  pkdataPanel2x.assayon = true;
//                  pkcheck4.setSelected(true);
//                  pkcheck4.setEnabled(true);
//    	    	  TabWindow.this.pkassay.setEnabled(true);
//    	    	  TabWindow.this.pkm_text_ul_Assay.setEnabled(true);
                }
                else
                {
                    this.pkdataPanel2x.setASSon(false);
//                  pkdataPanel2x.assayon = true;
//                  pkcheck4.setSelected(false);
                  if (!preassayon)
                  {
//        	    	  TabWindow.this.pkPassay.setEnabled(false);
//        	    	  TabWindow.this.pkm_text_ul_preAssay.setEnabled(false);
//        	    	  pkcheck4.setEnabled(false);
                      this.pkdataPanel2x.setASSon(false);
//                      pkdataPanel2x.assayon = false;
                  }
//    	    	  TabWindow.this.pkassay.setEnabled(false);
//    	    	  TabWindow.this.pkm_text_ul_Assay.setEnabled(false);
                }
                int sindex = p96Search(this.plates96, pkmrdlab);
                if (sindex == -1)
                {
                  sindex = 0 ;
                }
                  this.pkdataPanel2x.setmrdindex(sindex + 1);
//                this.pkMRDsource.setSelectedIndex(sindex);
                  int lindex = lSearch(this.liqtypes, this.pksam);
                  if (lindex  <  0)
                  {
                  	lindex = 0;
                  }
                  this.pkdataPanel2x.setsamdilindex(lindex);
                  
//                  this.pksamdil.setSelectedIndex(lindex);
  				lindex = lSearch(this.liqtypes, this.pkstd);
                  if (lindex  <  0)
                  {
                  	lindex = 0;
                  }
                  this.pkdataPanel2x.setstddilindex(lindex);
//                  this.pkstddil.setSelectedIndex(lindex);
                  lindex = lSearch(this.liqtypes, this.pkmrddil);
                  if (lindex  <  0)
                  {
                  	lindex = 0;
                  }
                  this.pkdataPanel2x.setmrddilindex(lindex);
//                  this.pkMRDdil.setSelectedIndex(lindex);
                  lindex = lSearch(this.liqtypes, this.pkdildil);
                  if (lindex  <  0)
                  {
                  	lindex = 0;
                  }
                  this.pkdataPanel2x.setdilindex(lindex);
              }

              if ((loadcount == 7) || (loadcount == 17) || (loadcount == 27) || 
                      (loadcount == 37) || (loadcount == 47) || (loadcount == 57) || 
                      (loadcount == 67) || (loadcount == 77) || (loadcount == 87) || 
                      (loadcount == 97) || (loadcount == 107) || (loadcount == 117) || 
                      (loadcount == 127) || (loadcount == 137) || (loadcount == 147) || 
                      (loadcount == 157) || (loadcount == 167) || (loadcount == 177) || 
                      (loadcount == 187) || (loadcount == 197) || (loadcount == 207) || 
                      (loadcount == 217) || (loadcount == 227) || (loadcount == 237) || 
                      (loadcount == 247) || (loadcount == 257) || (loadcount == 267) || 
                      (loadcount == 277) || (loadcount == 287) || (loadcount == 297) || 
                      (loadcount == 307) || (loadcount == 317) || (loadcount == 327) || 
                      (loadcount == 337) || (loadcount == 347) || (loadcount == 357))
                    {
                      String line7 = currentLoadList.data.toString();
                      len = line7.length();
                      index1 = line7.indexOf(',');
                      index2 = line7.indexOf(',', index1 + 1);
                      index3 = line7.indexOf(',', index2 + 1);
                      
                      this.pkl1u = line7.substring(0, index1);
                      this.pkd1u = line7.substring(index1 + 1, index2);
                      this.pkl2u = line7.substring(index2 + 1, index3);
                      this.pkd2u = line7.substring(index3 + 1, len);
    				  int rindex1 = rSearch(this.reservoirs, this.pkl1u);
                      if (rindex1 >= 0)
                      {
                    	pkdataPanel2x.setdilsrcindex(rindex1 + 1);
//                    	this.pkDilsource.setEnabled(true);
//                        this.pkDilsource.setSelectedIndex(rindex1);
                      }
    				  int rindex2 = rSearch(this.reservoirs, this.pkl2u);
                      if (rindex2 >= 0)
                      {
                      	pkdataPanel2x.setdilresindex(rindex2 + 1);
//                        this.pkDilreserve.setSelectedIndex(rindex2);
//                        this.pkDilreserve.setEnabled(true);
                      }
    	               
                      int pindex1 = pSearch(this.platetypes, this.pkd1u);
                      int pindex2 = pSearch(this.platetypes, this.pkd2u);
                      if (pindex1 >= 0)
                      {
                          pkdataPanel2x.setdeck1index(pindex1);
//                        this.pkDilsource.setEnabled(true);
//                        this.pkdeck1.setEnabled(true);
//                        this.pkdeck1.setSelectedIndex(pindex1);
                      }
                      if (pindex2 >= 0)
                      {
                        pkdataPanel2x.setdeck2index(pindex2);
//                        this.pkdeck2.setVisible(true);
//                        this.pkdeck2.setEnabled(true);
//                        this.pkDilreserve.setEnabled(true);
//                    	this.lblDiluent2.setVisible(true);
//                        this.pkdeck2.setSelectedIndex(pindex2);
                      }
                    }

              if ((loadcount == 10) || (loadcount == 20) || (loadcount == 30) || 
                (loadcount == 40) || (loadcount == 50) || (loadcount == 60) || 
                (loadcount == 70) || (loadcount == 80) || (loadcount == 90) || 
                (loadcount == 100) || (loadcount == 110) || (loadcount == 120) || 
                (loadcount == 130) || (loadcount == 140) || (loadcount == 150) || 
                (loadcount == 160) || (loadcount == 170) || (loadcount == 180) || 
                (loadcount == 190) || (loadcount == 200) || (loadcount == 210) || 
                (loadcount == 220) || (loadcount == 230) || (loadcount == 240) || 
                (loadcount == 250) || (loadcount == 260) || (loadcount == 270) || 
                (loadcount == 280) || (loadcount == 290) || (loadcount == 300) || 
                (loadcount == 310) || (loadcount == 320) || (loadcount == 330) || 
                (loadcount == 340) || (loadcount == 350) || (loadcount == 360))
              {
                loaddata = "";

                loaddata = loaddata + "\n " + this.pkm_text_wat.getText().trim();

                this.dList.insertAtBack(loaddata);
                loaddata = "";
                pkdataPanel2x.resetPanel3();
              }
            }

            if (steptype.equals("Pause"))
            {
                 if ((loadcount == 3) || (loadcount == 13) || (loadcount == 23) || 
                     (loadcount == 33) || (loadcount == 43) || (loadcount == 53) || 
                     (loadcount == 63) || (loadcount == 73) || (loadcount == 83) || 
                     (loadcount == 93) || (loadcount == 103) || (loadcount == 113) || 
                     (loadcount == 123) || (loadcount == 133) || (loadcount == 143) || 
                     (loadcount == 153) || (loadcount == 163) || (loadcount == 173) || 
                     (loadcount == 183) || (loadcount == 193) || (loadcount == 203) || 
                     (loadcount == 213) || (loadcount == 223) || (loadcount == 233) || 
                     (loadcount == 243) || (loadcount == 253) || (loadcount == 263) || 
                     (loadcount == 273) || (loadcount == 283) || (loadcount == 293) || 
                     (loadcount == 303) || (loadcount == 313) || (loadcount == 323) || 
                     (loadcount == 333) || (loadcount == 343) || (loadcount == 353))
              {
                	 String line3 = currentLoadList.data.toString();
                     len = line3.length();
                     if (len > 1)
                     {
                     index1 = line3.indexOf(',');
                     index2 = line3.indexOf(',', index1 + 1);
                     if (line3.substring(0, index1).equals("1"))
                     {
                    	 this.pausedataPanel2.setTimed(true);
                     }
                     else
                     {
                    	 this.pausedataPanel2.setTimed(false);
                     }
                     
                     	this.pausedataPanel2.setMinutes(line3.substring(index1 + 1, index2));
                     	this.pausedataPanel2.setSeconds(line3.substring(index2 + 1, len));
                     }
                     else
                     {
                    	 this.pausedataPanel2.setTimed(false);
            			 this.pausedataPanel2.setMinutes("");
            			 this.pausedataPanel2.setSeconds("");
                    	 
                     }
              }
                 
              if ((loadcount == 4) || (loadcount == 14) || (loadcount == 24) || 
                  (loadcount == 34) || (loadcount == 44) || (loadcount == 54) || 
                  (loadcount == 64) || (loadcount == 74) || (loadcount == 84) || 
                  (loadcount == 94) || (loadcount == 104) || (loadcount == 114) || 
                  (loadcount == 124) || (loadcount == 134) || (loadcount == 144) || 
                  (loadcount == 154) || (loadcount == 164) || (loadcount == 174) || 
                  (loadcount == 184) || (loadcount == 194) || (loadcount == 204) || 
                  (loadcount == 214) || (loadcount == 224) || (loadcount == 234) || 
                  (loadcount == 244) || (loadcount == 254) || (loadcount == 264) || 
                  (loadcount == 274) || (loadcount == 284) || (loadcount == 294) || 
                  (loadcount == 304) || (loadcount == 314) || (loadcount == 324) || 
                  (loadcount == 334) || (loadcount == 344) || (loadcount == 354))
                 {
               	  
                  String line4 = currentLoadList.data.toString();
                  len = line4.length();
                  this.pausedataPanel2.setMessage(line4.trim());
                 }
                 

               if ((loadcount == 10) || (loadcount == 20) || (loadcount == 30) || 
                (loadcount == 40) || (loadcount == 50) || (loadcount == 60) || 
                (loadcount == 70) || (loadcount == 80) || (loadcount == 90) || 
                (loadcount == 100) || (loadcount == 110) || (loadcount == 120) || 
                (loadcount == 130) || (loadcount == 140) || (loadcount == 150) || 
                (loadcount == 160) || (loadcount == 170) || (loadcount == 180) || 
                (loadcount == 190) || (loadcount == 200) || (loadcount == 210) || 
                (loadcount == 220) || (loadcount == 230) || (loadcount == 240) || 
                (loadcount == 250) || (loadcount == 260) || (loadcount == 270) || 
                (loadcount == 280) || (loadcount == 290) || (loadcount == 300) || 
                (loadcount == 310) || (loadcount == 320) || (loadcount == 330) || 
                (loadcount == 340) || (loadcount == 350) || (loadcount == 360))
              {
                loaddata = "";
                loaddata = loaddata + "\n\n PAUSE\n";
                this.dList.insertAtBack(loaddata);
                loaddata = "";
              }
            }
          }
          if (currentLoadList == null)
            continue;
          currentLoadList = currentLoadList.next;
        }

        this.dTextArea[0].setText("");
        this.dTextArea[1].setText("");
        this.dTextArea[2].setText("");
        this.dTextArea[3].setText("");
        this.dTextArea[4].setText("");
        this.dTextArea[5].setText("");
        this.dTextArea[6].setText("");
        this.dTextArea[7].setText("");
        this.dTextArea[8].setText("");
        this.dTextArea[9].setText("");
        this.dTextArea[10].setText("");
        this.dTextArea[11].setText("");
        this.dTextArea[12].setText("");
        this.dTextArea[13].setText("");
        this.dTextArea[14].setText("");

        this.dTextArea[0].setVisible(false);
        this.dTextArea[1].setVisible(false);
        this.dTextArea[2].setVisible(false);
        this.dTextArea[3].setVisible(false);
        this.dTextArea[4].setVisible(false);
        this.dTextArea[5].setVisible(false);
        this.dTextArea[6].setVisible(false);
        this.dTextArea[7].setVisible(false);
        this.dTextArea[8].setVisible(false);
        this.dTextArea[9].setVisible(false);
        this.dTextArea[10].setVisible(false);
        this.dTextArea[11].setVisible(false);
        this.dTextArea[12].setVisible(false);
        this.dTextArea[13].setVisible(false);
        this.dTextArea[14].setVisible(false);
        TabWindow.this.pagecount = 0;		//APF0028 Added this line to reset the page number if opening an existing protocol
        loadtext();
        for (int t = 0; t < this.tabcount; t++)
        {
            this.jTabbedPane1.setEnabledAt(t, true);
        }
        pkEnableDisable();

      }
      return true;
    }else{return false;}//Operator chose Cancel
  }

  private void editFile(int v)
  {
    ListNode currentFList = this.fList.getFirstNode();
    ListNode currentDList = this.dList.getFirstNode();
    ListNode currentTList = this.tList.getFirstNode();
    String step = this.stepnum.getText().trim();
    int index1 = 0;
    int index2 = 0;
    int index3 = 0;
    int index4 = 0;
    int index5 = 0;
    int index6 = 0;
    int index7 = 0;
    int index8 = 0;
    int sindex = 0;
    int dindex = 0;
    int lindex = 0;
    int pindex1 = 0;
    int pindex2 = 0;
    int tindex1 = 0;
    int tindex2 = 0;
    int rindex1 = 0;
    int rindex2 = 0;
    int len = 0;
    int count = 0;
    int val = 0;
    String steptype = "";
    String bpl1 = null;
    String bpl2 = null;
    boolean b24l = false;
    boolean b24r = false;
    boolean b48l = false;
    boolean b48r = false;
    boolean b96l = false;
    boolean b96r = false;
    boolean b384l = false;
    boolean b384r = false;
    boolean bresl = false;
    boolean bresr = false;
    boolean b384s = false;
    this.saveButton.setEnabled(false);
    this.panel24l.clearTray();
    this.panel24r.clearTray();
    this.panel48l.clearTray();
    this.panel48r.clearTray();
    this.panel96l.clearTray();
    this.panel96r.clearTray();
    val = v;
    if (val > 0) {
      if (val == 1) {
        for (int counter = 1; counter <= 10; counter++)
        {
          if (counter == 2) {
            steptype = currentFList.data.toString();
          }
    	  
          if (steptype.equals("add")) {
            this.updButton1.setEnabled(true);
            this.checkButton.setEnabled(true);
            this.addButton1.setEnabled(false);
            this.addButton.setEnabled(false);
            this.rmvButton.setEnabled(true);
            this.arrowButton1.setEnabled(false);
            this.arrowButton2.setEnabled(false);
            for (int t = 0; t < this.tabcount; t++) {	
            	this.jTabbedPane1.setEnabledAt(t, false);
            }

            if (counter == 3) {
              String line3 = currentFList.data.toString();
              index1 = line3.indexOf(',');
              len = line3.length();
              index2 = line3.indexOf(',', index1 + 1);
              index3 = line3.indexOf(',', index2 + 1);
              this.text2.setText(line3.substring(0, index1));
              String lt1 = line3.substring(index1 + 1, index2);
              lindex = lSearch(this.liqtypes, lt1);
              this.liqtype.setSelectedIndex(lindex);
            }

            if (counter == 4) {
              String line4 = currentFList.data.toString();
              index1 = line4.indexOf(',');
              len = line4.length();
              this.spl1u = line4.substring(0, index1);
              this.dpl1u = line4.substring(index1 + 1, len);
              pindex1 = pSearch(this.platetypes, this.spl1u);
              pindex2 = pSearch(this.platetypes, this.dpl1u);
              if (pindex1 >= 0)
              {
                this.platebutton.setSelected(true);
                this.splatetype.setVisible(true);
                this.splatetype.setSelectedIndex(pindex1);
              }
              if (pindex2 >= 0)
              {
                this.platebutton2.setSelected(true);
                this.dplatetype.setSelectedIndex(pindex2);
                this.dplatetype.revalidate();
                this.dplatetype.repaint();
              }

            }

            if (counter == 5) {
              String line5 = currentFList.data.toString();
              index1 = line5.indexOf(',');
              index2 = line5.indexOf(',', index1 + 1);
              index3 = line5.indexOf(',', index2 + 1);
              index4 = line5.indexOf(',', index3 + 1);
              index5 = line5.indexOf(',', index4 + 1);
              len = line5.length();
              String addsrc = line5.substring(0, index1);
              sindex = p24Search(this.plates24, addsrc);
              if (sindex >= 0)
              {
                b24l = true;
                sindex = pAllSearch(this.platesAll, addsrc);
              }
              else
              {
                b24l = false;
              }
              if (sindex == -1)
              {
               sindex = p48Search(this.plates48, addsrc);
               if (sindex >= 0)
               {
                 b48l = true;
                 sindex = pAllSearch(this.platesAll, addsrc);
               }
               else
               {
                 b48l = false;
               }
              }
              if (sindex == -1)
              {
               sindex = p96Search(this.plates96, addsrc);
               if (sindex >= 0)
               {
                 b96l = true;
                 sindex = pAllSearch(this.platesAll, addsrc);
               }
               else
               {
                 b96l = false;
               }
              }
              if (sindex == -1)
              {
                sindex = p384Search(this.plates384, addsrc);
                if (sindex >= 0)
                {
                  b384l = true;
                  sindex = pAllSearch(this.platesAll, addsrc);
                }
                else
                {
                  b384l = false;
                }
              }
              if (sindex == -1)
              {
                sindex = rSearch(this.reservoirs, addsrc);
                if (sindex >= 0)
                {
                  bresl = true;
                }
              }
              if (sindex == -1)
              {
                  sindex = tSearch(this.tubes, addsrc) +1;																	//APN0045	Added  +1
                  String graphicFile = "C:/Program Files/TELOS/AutoPilot/Graphics/" + addsrc + ".png";						//APN0045	Added  +1
          			if (new File(graphicFile).exists()) {																	//APN0045	Added  +1
          				tubepanel.label1.setText("");																		//APN0045	Added  +1
          				tubepanel.label1.setIcon(new ImageIcon(graphicFile));												//APN0045	Added  +1
          			}else{																									//APN0045	Added  +1
          				tubepanel.label1.setIcon(new ImageIcon(""));														//APN0045	Added  +1
          				tubepanel.label1.setText("NO IMAGE OF " + addsrc + " AVAILABLE");									//APN0045	Added  +1
          			} 																										//APN0045	Added  +1
                }	
              
              String mix1 = line5.substring(index1 + 1, index2);
              String asp1 = line5.substring(index2 + 1, index3);
              String mp1 = line5.substring(index3 + 1, index4);
              String sh1 = line5.substring(index4 + 1, index5);
              String c1 = line5.substring(index5 + 1, index5 + 2);
              String i3 = line5.substring(index5 + 3, index5 + 4);
              String v1 = line5.substring(index5 + 5, index5 + 6);
              if (bresl)
              {
                this.rs1 = line5.substring(index5 + 7, len);
                
              }
              this.advwin.setASPMode(asp1);
              if (mp1.equals("Variable"))
              {
                this.advwin.setMPA(13);
              }
              else
              {
                this.advwin.setMPA(Integer.parseInt(mp1));
              }
              this.lmixt1.setText(mix1);
              if (c1.equals("1"))
              {
                this.advwin.setChill(true);
              }
              if (c1.equals("0"))
              {
                this.advwin.setChill(false);
              }
              if (sh1.equals("1"))
              {
                this.advwin.setShake(true);
              }
              if (sh1.equals("0"))
              {
                this.advwin.setShake(false);
              }
              if (v1.equals("1"))
              {
                this.advwin.setVStart(true);
              }
              if (v1.equals("0"))
              {
                this.advwin.setVStart(false);
              }
//              if (i3.equals("1"))
//              {
//                this.linc1button.setSelected(true);
//              }
//              if (i3.equals("2"))
//              {
//                this.linc2button.setSelected(true);
//              }
//              if (i3.equals("3"))
//              {
//                this.linc3button.setSelected(true);
//              }
            }

            if (counter == 6) {
              String line6 = currentFList.data.toString();
              index1 = line6.indexOf(',');
              index2 = line6.indexOf(',', index1 + 1);
              index3 = line6.indexOf(',', index2 + 1);
              index4 = line6.indexOf(',', index3 + 1);
              index5 = line6.indexOf(',', index4 + 1);
              len = line6.length();
              String adddest = line6.substring(0, index1);
              String mix2 = line6.substring(index1 + 1, index2);

              String asp2 = line6.substring(index2 + 1, index3);
              String mp2 = line6.substring(index3 + 1, index4);
              String sh2 = line6.substring(index4 + 1, index5);
              String c2 = line6.substring(index5 + 1, index5 + 2);
              String i4 = line6.substring(index5 + 3, index5 + 4);
              String v2 = line6.substring(index5 + 5, index5 + 6);

              dindex = p24Search(this.plates24, adddest);
              if (dindex >= 0)
              {
                b24r = true;
                dindex = pAllSearch(this.platesAll, adddest);
              }
              else
              {
                b24r = false;
              }
              if (dindex == -1)
              {
                dindex = p48Search(this.plates48, adddest);
                if (dindex >= 0)
                {
                  b48r = true;
                  dindex = pAllSearch(this.platesAll, adddest);
                }
                else
                {
                  b48r = false;
                }
              }
              if (dindex == -1)
              {
                dindex = p96Search(this.plates96, adddest);
                if (dindex >= 0)
                {
                  b96r = true;
                  dindex = pAllSearch(this.platesAll, adddest);
                }
                else
                {
                  b96r = false;
                }
              }
              if (dindex == -1)
              {
                dindex = p384Search(this.plates384, adddest);
                if (dindex >= 0)
                {
                  b384r = true;
                  dindex = pAllSearch(this.platesAll, adddest);
                }
                else
                {
                  b384r = false;
                }
              }
              if (dindex == -1)
              {
                dindex = rSearch(this.reservoirs, adddest);
                if (dindex >= 0)
                {
                  bresr = true;
                }
              }
              if (dindex == -1)
              {
                  dindex = tSearch(this.tubes, adddest) +1;																	//APN0045	Added  +1																//APN0045	Added  +1
                  String graphicFile = "C:/Program Files/TELOS/AutoPilot/Graphics/" + adddest + ".png";						//APN0045	Added  +1
          			if (new File(graphicFile).exists()) {																	//APN0045	Added  +1
          				tubepanel2.label1.setText("");																		//APN0045	Added  +1
          				tubepanel2.label1.setIcon(new ImageIcon(graphicFile));												//APN0045	Added  +1
          			}else{																									//APN0045	Added  +1
          				tubepanel2.label1.setIcon(new ImageIcon(""));														//APN0045	Added  +1
          				tubepanel2.label1.setText("NO IMAGE OF " + adddest + " AVAILABLE");									//APN0045	Added  +1
          			} 																										//APN0045	Added  +1 
                }
              if (bresl)
              {
                this.rs2 = line6.substring(index5 + 7, len);
              }
              this.advwind.setASPMode(asp2);
              if (mp2.equals("Variable"))
              {
                this.advwind.setMPA(13);
              }
              else
              {
                this.advwind.setMPA(Integer.parseInt(mp2));
              }
              this.lmixt2.setText(mix2);

              if (sh2.equals("1"))
              {
                this.advwind.setShake(true);
              }
              if (sh2.equals("0"))
              {
                this.advwind.setShake(false);
              }
              if (i4.equals("1"))
              {
                this.linc4button.setSelected(true);
              }
              if (i4.equals("2"))
              {
                this.linc5button.setSelected(true);
              }
              if (i4.equals("3"))
              {
                this.linc6button.setSelected(true);
              }
              if (c2.equals("1"))
              {
                this.advwind.setChill(true);
              }
              if (c2.equals("0"))
              {
                this.advwind.setChill(false);
              }
              if (v2.equals("1"))
              {
                this.advwind.setVStart(true);
              }
              if (v2.equals("0"))
              {
                this.advwind.setVStart(false);
              }
            }

            if (counter == 7) {
              if (this.spl1u.equals("Tube"))
              {
                String line7 = currentFList.data.toString();
                index1 = line7.indexOf(',');
                index2 = line7.indexOf(',', index1 + 1);
                index3 = line7.indexOf(',', index2 + 1);
                String tube1 = line7.substring(0, index1);
                String tube2 = line7.substring(index1 + 1, index2);
                String i1 = line7.substring(index2 + 1, index2 + 2);
                this.tubepanel.setFirstTube(tube1);
                this.tubepanel.setLastTube(tube2);
                this.tubepanel.setIncButton(i1);
                this.tubebutton.setSelected(true);
                this.liqPanel1.removeAll();
                getSrc();
                this.source.setSelectedIndex(sindex);
                this.source.revalidate();
                this.source.repaint();
                this.splatetype.setVisible(false);
                this.liqPanel1.add(getLiqPanel1Top(), "North");
                this.liqPanel1.add(this.tubepanel, "Center");
                this.liqPanel1.add(getLiqPanel1Bot(), "South");
                this.liqPanel1.revalidate();
                this.liqPanel1.repaint();
              }
              if (bresl)
              {
                this.liqPanel1.removeAll();
                this.resbutton.setSelected(true);
                getSrc();
                this.source.setSelectedIndex(sindex  +1);	//APN0047 added +1

                Object selectedItem = source.getSelectedItem()		;														//APN0047
                String graphicFile = "C:/Program Files/TELOS/AutoPilot/Graphics/" + String.valueOf(selectedItem) + ".png";	//APN0047
        		if (new File(graphicFile).exists()) {																		//APN0047
        			ReservoirPanel1.label1.setText("");																		//APN0047
        			
                    ImageIcon imageIcon = new ImageIcon(graphicFile); // load the image to a imageIcon
        			Image image = imageIcon.getImage(); // transform it 
        			Image newimg = image.getScaledInstance(360, 220,  java.awt.Image.SCALE_SMOOTH); // scale it the smooth way  
        			imageIcon = new ImageIcon(newimg);  // transform it back
        			
        			ReservoirPanel1.label1.setIcon(imageIcon);																//APN0047
        		}else{																										//APN0047
        			ReservoirPanel1.label1.setIcon(new ImageIcon(""));														//APN0047
        			if (dindex == 0){																						//APN0047
        				ReservoirPanel1.label1.setText("SELECT RESERVOIR TYPE ABOVE");										//APN0047
        			}else{																									//APN0047
        				ReservoirPanel1.label1.setText("NO IMAGE OF " + String.valueOf(selectedItem) + " AVAILABLE");		//APN0047
        			}																										//APN0047
        		}	
                
                this.source.revalidate();
                this.source.repaint();
                this.splatetype.setVisible(true);
                
                TabWindow.this.liqPanel1.removeAll();															//APN0047
                TabWindow.this.liqPanel1.add(TabWindow.this.getLiqPanel1Top(), "North");						//APN0047
                TabWindow.this.liqPanel1.add(TabWindow.this.ReservoirPanel1, "Center");							//APN0047		
                TabWindow.this.liqPanel1.add(TabWindow.this.getLiqPanel1Bot(), "South");						//APN0047
                TabWindow.this.liqPanel1.revalidate();															//APN0047
                TabWindow.this.liqPanel1.repaint();																//APN0047

              }
              if ((!this.spl1u.equals("Tube")) && (!bresl))
              {
                String seq = currentFList.data.toString();
                if (b24l)
                {
                  this.liqPanel1.removeAll();
                  getSrc();
                  this.source.setSelectedIndex(sindex);
                  this.source.revalidate();
                  this.splatetype.setVisible(true);
                  this.panel24l.setButtons(seq);
                  this.liqPanel1.add(getLiqPanel1Top(), "North");
                  this.liqPanel1.add(this.panel24l, "Center");
                  this.liqPanel1.add(getLiqPanel1Bot(), "South");
                  this.liqPanel1top.revalidate();
                  this.liqPanel1top.repaint();
                  this.liqPanel1.revalidate();
                  this.liqPanel1.repaint();
                }
                if (b48l)
                {
                  this.liqPanel1.removeAll();
                  getSrc();
//                  this.source.removeAllItems();
//                  for (int i = 0; i < this.p48count; i++)
//                    this.source.addItem(this.plates48[i]);
                  this.source.setSelectedIndex(sindex);
                  this.source.revalidate();
                  this.splatetype.setVisible(true);
                  this.panel48l.setButtons(seq);
//                  this.button48l2.setSelected(true);
                  this.liqPanel1.add(getLiqPanel1Top(), "North");
                  this.liqPanel1.add(this.panel48l, "Center");
                  this.liqPanel1.add(getLiqPanel1Bot(), "South");
                  this.liqPanel1top.revalidate();
                  this.liqPanel1top.repaint();
                  this.liqPanel1.revalidate();
                  this.liqPanel1.repaint();
                }
                if (b96l)
                {
                  this.liqPanel1.removeAll();
                  getSrc();
//                  this.source.removeAllItems();
//                  for (int i = 0; i < this.p96count; i++)
//                    this.source.addItem(this.plates96[i]);
                  this.source.setSelectedIndex(sindex);
                  this.source.revalidate();
                  this.splatetype.setVisible(true);
                  this.panel96l.setButtons(seq);
//                  this.button96l2.setSelected(true);
                  this.liqPanel1.add(getLiqPanel1Top(), "North");
                  this.liqPanel1.add(this.panel96l, "Center");
                  this.liqPanel1.add(getLiqPanel1Bot(), "South");
                  this.liqPanel1top.revalidate();
                  this.liqPanel1top.repaint();
                  this.liqPanel1.revalidate();
                  this.liqPanel1.repaint();
                }
                if (b384l)
                {
                  this.panel384l.setButtons(seq);
//                  this.button384l2.setSelected(true);
//                  this.liqPanel1.removeAll();
                  getSrc();
//                  this.source.removeAllItems();
//                  for (int i = 0; i < this.p384count; i++)
//                    this.source.addItem(this.plates384[i]);
                  this.source.setSelectedIndex(sindex);
                  this.source.revalidate();
                  this.splatetype.setVisible(true);
                  this.panel384l.setButtons(seq);
//                  this.button384l2.setSelected(true);
                  this.liqPanel1.add(getLiqPanel1Top(), "North");
                  this.liqPanel1.add(this.panel384l, "Center");
                  this.liqPanel1.add(getLiqPanel1Bot(), "South");
                  this.liqPanel1.revalidate();
                  this.liqPanel1.repaint();
                }
              }
            }

            if (counter == 8) {
              if (this.dpl1u.equals("Tube"))
              {
                String line8 = currentFList.data.toString();
                index1 = line8.indexOf(',');
                index2 = line8.indexOf(',', index1 + 1);
                index3 = line8.indexOf(',', index2 + 1);
                String tube3 = line8.substring(0, index1);
                String tube4 = line8.substring(index1 + 1, index2);
                String i2 = line8.substring(index2 + 1, index2 + 2);
                this.tubepanel2.setFirstTube(tube3);
                this.tubepanel2.setLastTube(tube4);
                this.tubepanel2.setIncButton(i2);
                this.tubebutton2.setSelected(true);
                this.liqPanel2.removeAll();
                getDest();
                this.dest.setSelectedIndex(dindex);
                this.dest.revalidate();
                this.dest.repaint();
                this.dplatetype.setVisible(false);
                this.liqPanel2.add(getLiqPanel2Top(), "North");
                this.liqPanel2.add(this.tubepanel2, "Center");
                this.liqPanel2.add(getLiqPanel2Bot(), "South");
                this.liqPanel2.revalidate();
                this.liqPanel2.repaint();
                this.liqPanel2top.revalidate();
                this.liqPanel2top.repaint();
              }
              if (bresr)
              {
                this.liqPanel2.removeAll();
                this.resbutton2.setSelected(true);
                getDest();
                this.dest.setSelectedIndex(dindex  +1);	//APN0047 added +1
                Object selectedItem = dest.getSelectedItem();																//APN0047
                String graphicFile = "C:/Program Files/TELOS/AutoPilot/Graphics/" + String.valueOf(selectedItem) + ".png";	//APN0047
//                this.resTextArea2[(Integer.parseInt(this.rs2) - 1)].setBackground(Color.blue);							//APN0047 Commented Out
        		if (new File(graphicFile).exists()) {																		//APN0047
        			ReservoirPanel2.label1.setText("");																		//APN0047
        			ReservoirPanel2.label1.setIcon(new ImageIcon(graphicFile));												//APN0047
        		}else{																										//APN0047
        			ReservoirPanel2.label1.setIcon(new ImageIcon(""));														//APN0047
        			if (dindex == 0){																						//APN0047
        				ReservoirPanel2.label1.setText("SELECT RESERVOIR TYPE ABOVE");										//APN0047
        			}else{																									//APN0047
        				ReservoirPanel2.label1.setText("NO IMAGE OF " + String.valueOf(selectedItem) + " AVAILABLE");		//APN0047
        			}																										//APN0047
        		}																											//APN0047

                this.dest.revalidate();
                this.dplatetype.setVisible(true);
                
                TabWindow.this.liqPanel2.removeAll();															//APN0047
                TabWindow.this.liqPanel2.add(TabWindow.this.getLiqPanel2Top(), "North");						//APN0047
                TabWindow.this.liqPanel2.add(TabWindow.this.ReservoirPanel2, "Center");							//APN0047		
                TabWindow.this.liqPanel2.add(TabWindow.this.getLiqPanel2Bot(), "South");						//APN0047
                TabWindow.this.liqPanel2.revalidate();															//APN0047
                TabWindow.this.liqPanel2.repaint();																//APN0047

              }
              if ((!this.dpl1u.equals("Tube")) && (!bresr))
              {
                String seq = currentFList.data.toString();
                if (b24r)
                {
                  this.liqPanel2.removeAll();
                  getDest();
                  this.dest.setSelectedIndex(dindex);
                  this.dest.revalidate();
                  this.dplatetype.setVisible(true);
                  this.panel24r.setButtons(seq);
                  this.liqPanel2.add(getLiqPanel2Top(), "North");
                  this.liqPanel2.add(this.panel24r, "Center");
                  this.liqPanel2.add(getLiqPanel2Bot(), "South");
                  this.liqPanel2.revalidate();
                  this.liqPanel2.repaint();
                  this.liqPanel2top.revalidate();
                  this.liqPanel2top.repaint();
                }
                if (b48r)
                {
                  this.liqPanel2.removeAll();
                  getDest();
                  this.dest.setSelectedIndex(dindex);
                  this.dest.revalidate();
                  this.dplatetype.setVisible(true);
                  this.panel48r.setButtons(seq);
                  this.liqPanel2.add(getLiqPanel2Top(), "North");
                  this.liqPanel2.add(this.panel48r, "Center");
                  this.liqPanel2.add(getLiqPanel2Bot(), "South");
                  this.liqPanel2.revalidate();
                  this.liqPanel2.repaint();
                  this.liqPanel2top.revalidate();
                  this.liqPanel2top.repaint();
                }
                if (b96r)
                {
                  this.liqPanel2.removeAll();
                  getDest();
//                  this.dest.removeAllItems();
//                  for (int i = 0; i < this.p96count; i++)
//                    this.dest.addItem(this.plates96[i]);
                  this.dest.setSelectedIndex(dindex);
                  this.dest.revalidate();
                  this.dplatetype.setVisible(true);
                  this.panel96r.setButtons(seq);
//                  this.button96l.setSelected(true);
                  this.liqPanel2.add(getLiqPanel2Top(), "North");
                  this.liqPanel2.add(this.panel96r, "Center");
                  this.liqPanel2.add(getLiqPanel2Bot(), "South");
                  this.liqPanel2.revalidate();
                  this.liqPanel2.repaint();
                  this.liqPanel2top.revalidate();
                  this.liqPanel2top.repaint();
                }
                if (b384r)
                {
                  this.liqPanel2.removeAll();
                  getDest();
//                  this.dest.removeAllItems();
//                  for (int i = 0; i < this.p384count; i++)
//                    this.dest.addItem(this.plates384[i]);
                  this.dest.setSelectedIndex(dindex);
                  this.dest.revalidate();
                  this.dplatetype.setVisible(true);
                  this.panel384r.setButtons(seq);
//                  this.button384l.setSelected(true);
                  this.liqPanel2.add(getLiqPanel2Top(), "North");
                  this.liqPanel2.add(this.panel384r, "Center");
                  this.liqPanel2.add(getLiqPanel2Bot(), "South");
                  this.liqPanel2.revalidate();
                  this.liqPanel2.repaint();
                }
              }

            }

            if (counter == 9) {
              this.text3.setText(currentFList.data.toString());
            }

            if (counter == 10) {
              String line10 = currentFList.data.toString();
              String q1 = line10.substring(0, 1);
              String q2 = line10.substring(2, 3);
              String q3 = line10.substring(4, 5);
              String q4 = line10.substring(6, 7);
              String q5 = line10.substring(8, 9);
              String q6 = line10.substring(10, 11);
              String q7 = line10.substring(12, 13);
              String q8 = line10.substring(14, 15);
              this.panel384l.setQ1(q1);
              this.panel384l.setQ2(q2);
              this.panel384l.setQ3(q3);
              this.panel384l.setQ4(q4);
              this.panel384r.setQ1(q5);
              this.panel384r.setQ2(q6);
              this.panel384r.setQ3(q7);
              this.panel384r.setQ4(q8);
            }

            int tindex = tabSearch(this.tabs, "Liquid Transfer");
            if (tindex > -1)
            {
              this.jTabbedPane1.setSelectedIndex(tindex);
            }

          }

          if (steptype.equals("serial")) {
            this.updButton6.setEnabled(true);
            this.checkButton.setEnabled(true);
            this.addButton6.setEnabled(false);
            this.rmvButton.setEnabled(true);
            this.arrowButton1.setEnabled(false);
            this.arrowButton2.setEnabled(false);
            for (int t = 0; t < this.tabcount; t++) {
              this.jTabbedPane1.setEnabledAt(t, false);
            }

            if (counter == 3) {
              String line3 = currentFList.data.toString();
              index1 = line3.indexOf(',');
              len = line3.length();
              index2 = line3.indexOf(',', index1 + 1);
              index3 = line3.indexOf(',', index2 + 1);
              index4 = line3.indexOf(',', index3 + 1);
              this.sttext.setText(line3.substring(0, index1));
              this.slt1 = line3.substring(index1 + 1, index2);
//            this.dltext.setText(line3.substring(index2 + 1, index3));
              this.spl4u = line3.substring(index2 + 1, index3);
              pindex1 = pSearch(this.platetypes, this.spl4u);
              if (pindex1 >= 0)
              {
                this.splatetype4.setSelectedIndex(pindex1);
              }

              this.dlt1 = line3.substring(index3 + 1, index4);
              this.so2 = line3.substring(index4 + 1, len);
              lindex = lSearch(this.liqtypes, this.slt1);
              this.sliqtype.setSelectedIndex(lindex);
              rindex1 = rSearch(this.reservoirs, this.so2);

              if (rindex1 >= 0)
              {
                this.source2.setSelectedIndex(rindex1);
                this.source.revalidate();
                this.source.repaint();
                this.serPanel1top.revalidate();
                this.serPanel1top.repaint();
              }

            }

            if (counter == 4) {
              String line4 = currentFList.data.toString();
              index1 = line4.indexOf(',');
              len = line4.length();
              this.spl2u = line4.substring(0, index1);
              this.dpl2u = line4.substring(index1 + 1, len);
              pindex1 = pSearch(this.platetypes, this.spl2u);
              pindex2 = pSearch(this.platetypes, this.dpl2u);

              if (pindex1 >= 0)
              {
                this.platebutton.setSelected(true);
                this.splatetype2.setVisible(true);
                this.splatetype2.setSelectedIndex(pindex1);
              }

              if (pindex2 >= 0)
              {
                this.platebutton2.setSelected(true);
                this.dplatetype2.setVisible(true);
                this.dplatetype2.setSelectedIndex(pindex2);
              }

            }

            if (counter == 5) {
              String line5 = currentFList.data.toString();
              index1 = line5.indexOf(',');
              index2 = line5.indexOf(',', index1 + 1);
              String sersrc = line5.substring(0, index1);
              String i3 = line5.substring(index1 + 5, index1 + 6);
              sindex = tSearch(this.tubes, sersrc);
              if (sindex >= 0)
              {
                this.source3.setSelectedIndex(sindex);
              }
              if (sindex == -1)
              {
                sindex = p96Search(this.plates96, sersrc);
              }
              if (sindex >= 0)
              {
                this.source3.setSelectedIndex(sindex);
              }
              if (sindex == -1)
              {
                sindex = rSearch(this.reservoirs, sersrc);
              }
              if (sindex >= 0)
              {
                this.source3.setSelectedIndex(sindex);
              }

              if (i3.equals("1"))
              {
                this.sinc1button.setSelected(true);
              }
              if (i3.equals("2"))
              {
                this.sinc2button.setSelected(true);
              }
              if (i3.equals("3"))
              {
                this.sinc3button.setSelected(true);
              }
            }

            if (counter == 6) {
              String line6 = currentFList.data.toString();
              index1 = line6.indexOf(',');
              index2 = line6.indexOf(',', index1 + 1);
              String serdest = line6.substring(0, index1);
              String i4 = line6.substring(index2 + 1, index2 + 6);
              dindex = p96Search(this.plates96, serdest);
              if (dindex >= 0)
              {
                this.dest2.setSelectedIndex(dindex);
              }
              if (dindex == -1)
              {
                dindex = p384Search(this.plates384, serdest);
                if (dindex >= 0)
                {
                  this.dest2.setSelectedIndex(dindex);
                  b384s = true;
                }
              }
            }

            if (counter == 7) {
              if (this.spl2u.equals("Tube"))
              {
                String line7 = currentFList.data.toString();
                index1 = line7.indexOf(',');
                index2 = line7.indexOf(',', index1 + 1);
                index3 = line7.indexOf(',', index2 + 1);
                String tube1 = line7.substring(0, index1);
                String tube2 = line7.substring(index1 + 1, index2);
                String i1 = line7.substring(index2 + 1, index2 + 2);
                this.stubepanel.setFirstTube(tube1);
                this.stubepanel.setLastTube(tube2);
                this.stubepanel.setIncButton(i1);
                this.tubebutton3.setSelected(true);
                this.serPanel1.removeAll();
                getSrc3();
                this.source3.setSelectedIndex(sindex);
                this.source3.revalidate();
                this.source3.repaint();
                this.splatetype2.setVisible(false);
                this.serPanel1.add(getSerPanel1Top(), "North");
                this.serPanel1.add(this.stubepanel, "Center");
                this.serPanel1.revalidate();
                this.serPanel1.repaint();
              }
              if (!this.spl2u.equals("Tube"))
              {
                String seq = currentFList.data.toString();
                this.serPanel1.removeAll();
                this.platebutton3.setSelected(true);
                getSrc3();
                this.source3.setSelectedIndex(sindex);
                this.source3.revalidate();
                this.source3.repaint();
                this.splatetype2.setVisible(true);
                this.splatetype2.revalidate();
                this.splatetype2.repaint();
                this.serPanel1.add(getSerPanel1Top(), "North");
                this.serPanel1.add(this.spanel96l, "Center");
                this.spanel96l.setButtons(seq);
                this.serPanel1.revalidate();
                this.serPanel1.repaint();
                this.serPanel1top.revalidate();
                this.serPanel1top.repaint();
              }

            }

            if (counter == 8) {
              String seq = currentFList.data.toString();
              if (!b384s)
              {
                this.serPanel2.removeAll();
                this.dest2.removeAllItems();
                for (int i = 0; i < this.p96count; i++)
                  this.dest2.addItem(this.plates96[i]);
                this.dest2.setSelectedIndex(dindex);
                this.dest2.revalidate();
                this.dplatetype2.setVisible(true);
                this.spanel96r.setButtons(seq);
                this.button96s.setSelected(true);
                this.serPanel2.add(getSerPanel2Top(), "North");
                this.serPanel2.add(this.spanel96r, "Center");
                this.serPanel2top.revalidate();
                this.serPanel2top.repaint();
                this.serPanel2.revalidate();
                this.serPanel2.repaint();
              }
              else
              {
                this.serPanel2.removeAll();
                this.dest2.removeAllItems();
                for (int i = 0; i < this.p384count; i++)
                {
                  this.dest2.addItem(this.plates384[i]);
                }
                this.dest2.revalidate();
                this.spanel384r.setButtons(seq);
                this.button384s.setSelected(true);
                this.serPanel2.add(getSerPanel2Top(), "North");
                this.serPanel2.add(this.spanel384r, "Center");
                this.serPanel2.revalidate();
                this.serPanel2.repaint();
              }
            }

            if (counter == 9) {
              String transfer = currentFList.data.toString();
              Collections.addAll(arrayList_sdVolumesTransfer, transfer.split(","));
              //this.dilwin.setTransfer(transfer);  //sdDev
            }

            if (counter == 10) {
              String dilution = currentFList.data.toString();
              Collections.addAll(arrayList_sdVolumesDiluent, dilution.split(","));
              //this.dilwin.setDilution(dilution);  //sdDev
            }

            int tindex = tabSearch(this.tabs, "Serial Dilution");
            
            if (tindex > -1)
            {
              this.jTabbedPane1.setSelectedIndex(tindex);
            }

          }

          if (steptype.equals("brew")) {
            this.updButton7.setEnabled(true);
            this.checkButton.setEnabled(true);
            this.addButton7.setEnabled(false);
            this.rmvButton.setEnabled(true);
            this.arrowButton1.setEnabled(false);
            this.arrowButton2.setEnabled(false);
            for (int t = 0; t < this.tabcount; t++)
              this.jTabbedPane1.setEnabledAt(t, false);
            if (counter == 3) {
              String line3 = currentFList.data.toString();
              index1 = line3.indexOf(',');
              index2 = line3.indexOf(',', index1 + 1);
              index3 = line3.indexOf(',', index2 + 1);
              this.brtext.setText(line3.substring(0, index1));
              String blt = line3.substring(index1 + 1, index2);
              lindex = lSearch(this.liqtypes, blt);
              this.bliqtype.setSelectedIndex(lindex);
              this.brntext.setText(line3.substring(index2 + 1, index3));
            }
            if (counter == 4) {
              String line4 = currentFList.data.toString();
              index1 = line4.indexOf(',');
              len = line4.length();
              bpl1 = line4.substring(0, index1);
              bpl2 = line4.substring(index1 + 1, len);
              if (bpl1.equals("Reservoir"))
              {
                this.brresbutton1.setSelected(true);
                this.brtubebutton1.setSelected(false);
                this.brewPanel1.removeAll();
                getSrc4();
                this.source4.revalidate();
                this.source4.repaint();
                this.brewPanel1.add(getBrewPanel1Top(), "North");
                this.brewPanel1.add(this.brrespanel1, "Center");
                this.brewPanel1.add(getBrewPanel1Bot(), "South");
                this.brewPanel1.revalidate();
                this.brewPanel1.repaint();
              }

              if (bpl1.equals("Tube"))
              {
                this.brtubebutton1.setSelected(true);
                this.brresbutton1.setSelected(false);
                this.brewPanel1.removeAll();
                getSrc4();
                this.source4.revalidate();
                this.source4.repaint();
                this.brewPanel1.add(getBrewPanel1Top(), "North");
                this.brewPanel1.add(this.brtubepanel, "Center");
                this.brewPanel1.add(getBrewPanel1Bot(), "South");
                this.brewPanel1.revalidate();
                this.brewPanel1.repaint();
              }
              if (bpl2.equals("Reservoir"))
              {
                this.brtubebutton2.setSelected(false);
                this.brresbutton2.setSelected(true);
                this.brewPanel2.removeAll();
                getDest3();
                this.brresbutton2.setSelected(true);
                this.dest3.revalidate();
                this.dest3.repaint();
                this.brewPanel2.add(getBrewPanel2Top(), "North");
                this.brewPanel2.add(this.brrespanel2, "Center");
                this.brewPanel2.add(getBrewPanel2Bot(), "South");
                this.brewPanel2.revalidate();
                this.brewPanel2.repaint();
              }
              if (bpl2.equals("Tube"))
              {
                this.brtubebutton2.setSelected(true);
                this.brresbutton2.setSelected(false);
                this.brewPanel2.removeAll();
                getDest3();
                this.dest3.revalidate();
                this.dest3.repaint();
                this.brewPanel2.add(getBrewPanel2Top(), "North");
                this.brewPanel2.add(this.brtubepanel2, "Center");
                this.brewPanel2.add(getBrewPanel2Bot(), "South");
                this.brewPanel2.revalidate();
                this.brewPanel2.repaint();
              }

            }

            if (counter == 5) {
              String line5 = currentFList.data.toString();
              index1 = line5.indexOf(',');
              index2 = line5.indexOf(',', index1 + 1);
              String bsource = line5.substring(0, index1);
              String m1 = line5.substring(index1 + 1, index1 + 2);
              String c1 = line5.substring(index2 + 1, index2 + 2);
              int bindex1 = 0;
              bindex1 = rSearch(this.reservoirs, bsource);
              if (bindex1 > 0)
              {
                this.source4.setSelectedIndex(bindex1);
              }
              if (bindex1 == -1)
              {
                bindex1 = tSearch(this.tubes, bsource);
              }
              if (bindex1 > 0)
              {
                this.source4.setSelectedIndex(bindex1);
              }
              if (m1.equals("1"))
              {
                this.brmix.setSelected(true);
              }
              if (m1.equals("0"))
              {
                this.brmix.setSelected(false);
              }
              if (c1.equals("1"))
              {
                this.brchill.setSelected(true);
              }
              if (c1.equals("0"))
              {
                this.brchill.setSelected(false);
              }
            }

            if (counter == 6) {
              String line6 = currentFList.data.toString();
              index1 = line6.indexOf(',');
              index2 = line6.indexOf(',', index1 + 1);
              String bdest = line6.substring(0, index1);
              int bindex2 = 0;
              bindex2 = rSearch(this.reservoirs, bdest);
              if (bindex2 > 0)
              {
                this.dest3.setSelectedIndex(bindex2);
              }
              if (bindex2 == -1)
              {
                bindex2 = tSearch(this.tubes, bdest);
              }
              if (bindex2 > 0)
              {
                this.dest3.setSelectedIndex(bindex2);
              }
            }
            if ((counter == 7) && (bpl1.equals("Tube"))) {
              String line7 = currentFList.data.toString();
              index1 = line7.indexOf(',');
              index2 = line7.indexOf(',', index1 + 1);
              index3 = line7.indexOf(',', index2 + 1);
              String tube1 = line7.substring(0, index1);
              String tube2 = line7.substring(index1 + 1, index2);
              String i1 = line7.substring(index2 + 1, index2 + 2);
              this.brtubepanel.setFirstTube(tube1);
              this.brtubepanel.setLastTube(tube2);
              this.brtubepanel.setIncButton(i1);
            }
            if ((counter == 8) && (bpl2.equals("Tube"))) {
              String line8 = currentFList.data.toString();
              index1 = line8.indexOf(',');
              index2 = line8.indexOf(',', index1 + 1);
              index3 = line8.indexOf(',', index2 + 1);
              String tube3 = line8.substring(0, index1);
              String tube4 = line8.substring(index1 + 1, index2);
              String i2 = line8.substring(index2 + 1, index2 + 2);
              this.brtubepanel2.setFirstTube(tube3);
              this.brtubepanel2.setLastTube(tube4);
              this.brtubepanel2.setIncButton(i2);
            }

            if (counter == 9) {
              this.brvol.setText(currentFList.data.toString());
            }

            int tindex = tabSearch(this.tabs, "Brew");
            if (tindex > -1)
            {
              this.jTabbedPane1.setSelectedIndex(tindex);
            }

          }

          if (steptype.equals("wash")) {
            this.updButton3.setEnabled(true);
            this.checkButton.setEnabled(true);
            this.addButton3.setEnabled(false);
            this.rmvButton.setEnabled(true);
            this.arrowButton1.setEnabled(false);
            this.arrowButton2.setEnabled(false);
            for (int t = 0; t < this.tabcount; t++) {
              this.jTabbedPane1.setEnabledAt(t, false);
            }

            if (counter == 4) {
              this.wmethod.setText(currentFList.data.toString());
            }
            int tindex = tabSearch(this.tabs, "Wash");
            if (tindex > -1)
            {
              this.jTabbedPane1.setSelectedIndex(tindex);
            }

          }

          if (steptype.equals("SPE")) {
            this.updButton5.setEnabled(true);
            this.checkButton.setEnabled(true);
            this.addButton5.setEnabled(false);
            this.rmvButton.setEnabled(true);
            this.arrowButton1.setEnabled(false);
            this.arrowButton2.setEnabled(false);
            for (int t = 0; t < this.tabcount; t++) {
              this.jTabbedPane1.setEnabledAt(t, false);
            }

            if (counter == 3) {
              String line3 = currentFList.data.toString();
              index1 = line3.indexOf(',');
              len = line3.length();
              index2 = line3.indexOf(',', index1 + 1);
              index3 = line3.indexOf(',', index2 + 1);
              index4 = line3.indexOf(',', index3 + 1);
              String b1 = line3.substring(0, index1);
              String fil = line3.substring(index1 + 1, index2);
              String col = line3.substring(index2 + 1, index3);
              String ti = line3.substring(index3 + 1, index4);
              String pre = line3.substring(index4 + 1, len);
//              int findex1 = 0;
//              findex1 = fSearch(this.filters, fil);
//              int cindex = 0;
//              cindex = p96Search(this.plates96, col);

              if (b1.equals("1"))
              {
                this.assembutton.setSelected(true);
              }
              if (b1.equals("2"))
              {
                this.disassembutton.setSelected(true);
              }
              if (b1.equals("3"))
              {
                this.vacbutton.setSelected(true);
                this.filter.setVisible(false);
                this.collection.setVisible(false);
                this.spelabel1.setVisible(false);
                this.spelabel2.setVisible(false);
                this.time.setVisible(true);
                this.pressure.setVisible(true);
                this.pressure.setText(pre);
                this.time.setText(ti);
                this.spelabel3.setVisible(true);
                this.spelabel4.setVisible(true);
                this.spePanel.revalidate();
                this.spePanel.repaint();
              }

//              if (findex1 > 0)
//              {
//                this.filter.setSelectedIndex(findex1);
//              }
//
//              if (col.equals("None"))
//              {
//                this.collection.setSelectedIndex(0);
//              }
//              if (cindex >= 0)
//              {
//                this.collection.setSelectedIndex(cindex + 1);
//              }

            }

            int tindex = tabSearch(this.tabs, "SPE");
            if (tindex > -1)
            {
              this.jTabbedPane1.setSelectedIndex(tindex);
            }

          }

          if (steptype.equals("Cherry Picking")) {
            this.updButton11.setEnabled(true);
            this.checkButton.setEnabled(true);
            this.addButton.setEnabled(false);
            this.addButton11.setEnabled(false);
            this.rmvButton.setEnabled(true);
            this.arrowButton1.setEnabled(false);
            this.arrowButton2.setEnabled(false);
            for (int t = 0; t < this.tabcount; t++) {
              this.jTabbedPane1.setEnabledAt(t, false);
            }
            if (counter == 3) {
              String line3 = currentFList.data.toString();
              index1 = line3.indexOf(',');
              this.text7.setText(line3.substring(0, index1));
              len = line3.length();
              this.chltyp = line3.substring(index1 + 1, len);
              lindex = lSearch(this.liqtypes, this.chltyp);
              this.chliq.setSelectedIndex(lindex);
            }

            if (counter == 4) {
              String line4 = currentFList.data.toString();
              index1 = line4.indexOf(',');
              len = line4.length();
              this.tr1 = line4.substring(0, index1);
              this.tr2 = line4.substring(index1 + 1, len);
              this.track1.setSelectedIndex(Integer.parseInt(this.tr1) - 1);
              this.track2.setSelectedIndex(Integer.parseInt(this.tr2) - 1);
            }

            if (counter == 5) {
              String line5 = currentFList.data.toString();
              len = line5.length();
              String chso = line5.substring(0, len);
              sindex = pAllSearch(this.platesAll, chso);	//AP-0001
              this.chsource.setSelectedIndex(sindex);		//AP-0001
              
//              sindex = p96Search(this.plates96, chso);	//AP-0001 commented out and ready for deletion
//              if (sindex == -1)
//              {
//                sindex = p384Search(this.plates384, chso);
//                sindex += this.plates96.length;
//              }
//              this.chsource.setSelectedIndex(sindex);
              
            }

            if (counter == 6) {
              String line6 = currentFList.data.toString();
              len = line6.length();
              this.chde = line6.substring(0, len);
              dindex = pAllSearch(this.platesAll, this.chde);	//AP-0001
              this.chdest.setSelectedIndex(dindex);				//AP-0001
              
//              dindex = p96Search(this.plates96, this.chde);	//AP-0001 commented out and ready for deletion
//              if (dindex == -1)
//              {
//                dindex = p384Search(this.plates384, this.chde);
//                dindex += this.plates96.length;
//              }
//              this.chdest.setSelectedIndex(dindex);
              
            }

            int tindex = tabSearch(this.tabs, "Cherry Picking");
            if (tindex > -1)
            {
              this.jTabbedPane1.setSelectedIndex(tindex);
            }
          }
          
          if (steptype.equals("CherryPicking02")) {
              this.cherry02_btnUpdateStep.setEnabled(true);
              this.checkButton.setEnabled(true);
              this.addButton.setEnabled(false);
              this.cherry02_btnAddStep.setEnabled(false);
              this.rmvButton.setEnabled(true);
              this.arrowButton1.setEnabled(false);
              this.arrowButton2.setEnabled(false);
              for (int t = 0; t < this.tabcount; t++) {
                this.jTabbedPane1.setEnabledAt(t, false);
              }

			  if (counter == 3) {
                String line3 = currentFList.data.toString();
                this.cherry02dataPanel2.setText2(line3);
              }

              if (counter == 4) {
                String line4 = currentFList.data.toString();
                ch2_plateloc = line4;
              }

              if (counter == 5) {
                String line5 = currentFList.data.toString();
                len = line5.length();
                String chso = line5.substring(0, len);
                sindex = pAllSearch(this.platesAll, chso);	//AP-0001
                if (sindex == -1)
                {
                 cherry02dataPanel2.setSLabIndex(1);
                 sindex = tAllSearch(this.tubesAll, chso);	//AP-0001

                }
                else
                {
                 cherry02dataPanel2.setSLabIndex(2);	
                }
                this.cherry02dataPanel2.setSPlateIndex(sindex);		//AP-0001
                
              }

              if (counter == 6) {
                String line6 = currentFList.data.toString();
                len = line6.length();
                this.chde = line6.substring(0, len);
                dindex = pAllSearch(this.platesAll, this.chde);	//AP-0001
                if (dindex == -1)
                {
                 cherry02dataPanel2.setDLabIndex(1);
                 dindex = tAllSearch(this.tubesAll, this.chde);	//AP-0001
                }
                else
                {
                 cherry02dataPanel2.setDLabIndex(2);	
                }
                this.cherry02dataPanel2.setDPlateIndex(dindex);				//AP-0001
                
              }

              if (counter == 7) {
                  String line7 = currentFList.data.toString();
                  this.ch2_liqtype = line7;
                  lindex = lSearch(this.liqtypes, this.ch2_liqtype);
                  this.cherry02dataPanel2.setLiqIndex(lindex);
                }

              if (counter == 8) {
                  String line8 = currentFList.data.toString();
                  len = line8.length();
                  index1 = line8.indexOf(',');
                  this.ch2_scycle = line8.substring(0, index1);
                  this.ch2_smix = line8.substring(index1 + 1, len);
                  TabWindow.this.cherry02dataPanel2.setSCycle1(Integer.parseInt(ch2_scycle));
//                  TabWindow.this.cherry02dataPanel2.setSCycleIndex(Integer.parseInt(ch2_scycle));
                  TabWindow.this.cherry02dataPanel2.setText3(ch2_smix);
                }

              if (counter == 9) {
                  String line9 = currentFList.data.toString();
                  len = line9.length();
                  index1 = line9.indexOf(',');
                  this.ch2_dcycle = line9.substring(0, index1);
                  this.ch2_dmix = line9.substring(index1 + 1, len);
//                TabWindow.this.cherry02dataPanel2.setDCycleIndex(Integer.parseInt(ch2_dcycle));
                  TabWindow.this.cherry02dataPanel2.setDCycle1(Integer.parseInt(ch2_dcycle));
                TabWindow.this.cherry02dataPanel2.setText4(ch2_dmix);
                }

              if (counter == 10) {
                  String line10 = currentFList.data.toString();
                  ch2_tubeloc = line10;
                  if (ch2_plateloc.equals("null") && ch2_tubeloc.equals("null"))
                  {
                  	this.ch2_plateloc = "";
                  	this.ch2_tubeloc = "";
                  	this.cherry02dataPanel2.setT1Enabled(false);
                  	this.cherry02dataPanel2.setT1TEnabled(false);
                  	this.cherry02dataPanel2.setBarIndex(0);
                  }
                  else if (ch2_plateloc.equals("null"))
                  {
                   	this.ch2_plateloc = "";
                  	this.cherry02dataPanel2.setT1Enabled(false);
                   	this.cherry02dataPanel2.setT1TEnabled(true);
                  	this.cherry02dataPanel2.setBarIndex(1);
                  }
                  else if (ch2_tubeloc.equals("null"))
                  {
                	this.ch2_tubeloc = "";
                   	this.cherry02dataPanel2.setT1TEnabled(false);
                   	this.cherry02dataPanel2.setT1Enabled(true);
                  	this.cherry02dataPanel2.setBarIndex(1);
                  }
                  else 
                  {
                   	this.cherry02dataPanel2.setT1TEnabled(true);
                   	this.cherry02dataPanel2.setT1Enabled(true);
                  	this.cherry02dataPanel2.setBarIndex(1);
                  }
 					this.cherry02dataPanel2.setText1(ch2_plateloc);
  					this.cherry02dataPanel2.setText1T(ch2_tubeloc);
                }

              int tindex = tabSearch(this.tabs, "CherryPick02");
              if (tindex > -1)
              {
                this.jTabbedPane1.setSelectedIndex(tindex);
              }
            }

          if (steptype.equals("PK")) {
              this.btnUpdateStep.setEnabled(true);
              this.checkButton.setEnabled(true);
              this.addButton.setEnabled(false);
              this.btnAdd.setEnabled(false);
              this.rmvButton.setEnabled(true);
              this.arrowButton1.setEnabled(false);
              this.arrowButton2.setEnabled(false);
              for (int t = 0; t < this.tabcount; t++) {
                this.jTabbedPane1.setEnabledAt(t, false);
              }
              if (counter == 3) {
                String line3 = currentFList.data.toString();
                index1 = line3.indexOf(',');
                len = line3.length();
                index1 = line3.indexOf(',');
                index2 = line3.indexOf(',', index1 + 1);
                index3 = line3.indexOf(',', index2 + 1);
                index4 = line3.indexOf(',', index3 + 1);
                index5 = line3.indexOf(',', index4 + 1);

				this.pkdataPanel2x.setpkmtextwat(line3.substring(0, index1));
				this.pkdataPanel2x.setpkmtext1(line3.substring(index1 + 1, index2));
				this.pkdataPanel2x.setpkmtextul(line3.substring(index2 + 1, index3));
				this.pkdataPanel2x.setpkmpassayul(line3.substring(index3 + 1, index4));
				this.pkdataPanel2x.setpkmassayul(line3.substring(index4 + 1, len));
				
//              text = line3.substring(0, index1);
//				this.pkm_text_wat.setText(line3.substring(0, index1));
//				this.pkm_text_1.setText(line3.substring(index1 + 1, index2));
//				this.pkm_text_ul.setText(line3.substring(index2 + 1, index3));
//				this.pkm_text_ul_preAssay.setText(line3.substring(index3 + 1, index4));
//				this.pkm_text_ul_Assay.setText(line3.substring(index4 + 1, len));
              }

              if (counter == 4) {
            	  String line4 = currentFList.data.toString();
                  len = line4.length();
                  index1 = line4.indexOf(',');
                  index2 = line4.indexOf(',', index1 + 1);
                  index3 = line4.indexOf(',', index2 + 1);
                  index4 = line4.indexOf(',', index3 + 1);

                  this.pksam = line4.substring(0, index1);
  				  this.pkstd = line4.substring(index1 + 1, index2);
  				  this.pkmrddil = line4.substring(index2 + 1, index3);
  			      this.pkdildil = line4.substring(index3 + 1, len);
  				
              }

              if (counter == 5) {
                String line5 = currentFList.data.toString();
                len = line5.length();
                index1 = line5.indexOf(',');
                index2 = line5.indexOf(',', index1 + 1);
                index3 = line5.indexOf(',', index2 + 1);
                index4 = line5.indexOf(',', index3 + 1);
                index5 = line5.indexOf(',', index4 + 1);
                
                this.pksamlab = line5.substring(0, index1);
                this.pkstdlab = line5.substring(index1 + 1, index2);
                this.pkmrdlab = line5.substring(index2 + 1, index3);
                this.pkdillab = line5.substring(index3 + 1, index4);
                this.pkpasslab = line5.substring(index4 + 1, index5);
                this.pkasslab = line5.substring(index5 + 1, len);
                
                
                sindex = p96tubeSearch(this.plates96tubes, this.pksamlab);
                if (sindex == -1)
                {
                  sindex = 0 ;
                }
                this.pkdataPanel2x.setsamindex(sindex);
//                this.pkSsource.setSelectedIndex(sindex);

                sindex = p96Search(this.plates96, pkdillab);
                if (sindex == -1)
                {
                  sindex = 0 ;
                }
                this.pkdataPanel2x.setdilsrcindex2(sindex + 1);
//                this.pkDsource.setSelectedIndex(sindex);
                
                sindex = p96Search(this.plates96, pkpasslab);
                if (sindex == -1)
                {
                  sindex = 0 ;
                }
                this.pkdataPanel2x.setpassaylabindex(sindex + 1);
                
                sindex = p96tubeSearch(this.plates96tubes, pkstdlab);
                if (sindex == -1)
                {
                  sindex = 0 ;
                }
                this.pkdataPanel2x.setstdindex(sindex);
                
                sindex = p96Search(this.plates96, pkasslab);
                if (sindex == -1)
                {
                  sindex = 0 ;
                }
                this.pkdataPanel2x.setassaylabindex(sindex + 1);
                
              }

              if (counter == 6) {
                String line6 = currentFList.data.toString();
                len = line6.length();
                index1 = line6.indexOf(',');
                index2 = line6.indexOf(',', index1 + 1);
                index3 = line6.indexOf(',', index2 + 1);
                index4 = line6.indexOf(',', index3 + 1);
                
                this.pkck1 = line6.substring(0, index1);
                this.pkck2 = line6.substring(index1 + 1, index2);
                this.pkck3 = line6.substring(index2 + 1, index3);
                this.pkck4 = line6.substring(index3 + 1, len);
                if (pkck1.equals("1"))
                {
                    this.pkdataPanel2x.setMRDon(true);
//                	pkdataPanel2x.mrdon = true;
//                  pkcheck1.setSelected(true);
                }
                else
                {
                    this.pkdataPanel2x.setMRDon(false);
//                	pkdataPanel2x.mrdon = false;
                  if (!pkck2.equals("1"))
                  {
                      this.pkdataPanel2x.setDILon(false);
//                  	pkdataPanel2x.dilon = false;
                  }
                }
                if (pkck2.equals("1"))
                {
                  this.pkdataPanel2x.setDILon(true);
//                  pkdataPanel2x.dilon = true;
                }
                else
                {
                  this.pkdataPanel2x.setDILon(false);
//                  pkdataPanel2x.dilon = true;
                  if (!pkck1.equals("1"))
                  {
        	    	  // TabWindow.this.pkMRDsource.setEnabled(false);
        	    	  // TabWindow.this.pkMRDdil.setEnabled(false);
                  }
                }
                if (pkck3.equals("1"))
                {
                  this.pkdataPanel2x.setPRAon(true);
//                  pkdataPanel2x.preassayon = true;
                }
                else
                {
                    this.pkdataPanel2x.setPRAon(false);
//                  pkdataPanel2x.preassayon = false;
//                  if (!pkcheck2.isSelected())
//                  {
//                   pkcheck3.setEnabled(true);
//                  }
                }

                if (pkck4.equals("1"))
                {
                    this.pkdataPanel2x.setASSon(true);
//                  pkdataPanel2x.assayon = true;
//                  pkcheck4.setSelected(true);
//                  pkcheck4.setEnabled(true);
//    	    	  TabWindow.this.pkassay.setEnabled(true);
//    	    	  TabWindow.this.pkm_text_ul_Assay.setEnabled(true);
                }
                else
                {
                    this.pkdataPanel2x.setASSon(false);
//                  pkdataPanel2x.assayon = true;
//                  pkcheck4.setSelected(false);
                  if (!preassayon)
                  {
//        	    	  TabWindow.this.pkPassay.setEnabled(false);
//        	    	  TabWindow.this.pkm_text_ul_preAssay.setEnabled(false);
//        	    	  pkcheck4.setEnabled(false);
                      this.pkdataPanel2x.setASSon(false);
//                      pkdataPanel2x.assayon = false;
                  }
//    	    	  TabWindow.this.pkassay.setEnabled(false);
//    	    	  TabWindow.this.pkm_text_ul_Assay.setEnabled(false);
                }
                sindex = p96Search(this.plates96, pkmrdlab);
                if (sindex == -1)
                {
                  sindex = 0 ;
                }
                this.pkdataPanel2x.setmrdindex(sindex + 1);

                sindex = p96tubeSearch(this.plates96tubes, pkstdlab);
                if (sindex == -1)
                {
                  sindex = 0 ;
                }
                this.pkdataPanel2x.setstdindex(sindex);
                
                lindex = lSearch(this.liqtypes, this.pkmrddil);
			    if (lindex  <  0)
			    {
              	 lindex = 0;
			    }
			    this.pkdataPanel2x.setmrddilindex(lindex);
			      lindex = lSearch(this.liqtypes, this.pksam);
			      if (lindex  <  0)
			      {
			    	  lindex = 0;
			      }
			      this.pkdataPanel2x.setsamdilindex(lindex);
              
			      lindex = lSearch(this.liqtypes, this.pkstd);
			      if (lindex  <  0)
			      {
              	lindex = 0;
			      }
			      this.pkdataPanel2x.setstddilindex(lindex);

			      lindex = lSearch(this.liqtypes, this.pkmrddil);
			      if (lindex  <  0)
			      {
              	lindex = 0;
			      }
			      this.pkdataPanel2x.setmrddilindex(lindex);

			      lindex = lSearch(this.liqtypes, this.pkdildil);
			      if (lindex  <  0)
			      {
              	lindex = 0;
			      }
			      this.pkdataPanel2x.setdilindex(lindex);
              }

              if (counter == 7) {
                  String line7 = currentFList.data.toString();
                  len = line7.length();
                  index1 = line7.indexOf(',');
                  index2 = line7.indexOf(',', index1 + 1);
                  index3 = line7.indexOf(',', index2 + 1);
                  
                  this.pkl1u = line7.substring(0, index1);
                  this.pkd1u = line7.substring(index1 + 1, index2);
                  this.pkl2u = line7.substring(index2 + 1, index3);
                  this.pkd2u = line7.substring(index3 + 1, len);

				  rindex1 = rSearch(this.reservoirs, this.pkl1u);
                  if (rindex1 >= 0)
                  {
                	pkdataPanel2x.setdilsrcindex(rindex1 + 1);
//                	this.pkDilsource.setEnabled(true);
//                    this.pkDilsource.setSelectedIndex(rindex1);
                  }
				  rindex2 = rSearch(this.reservoirs, this.pkl2u);
                  if (rindex2 >= 0)
                  {
                  	pkdataPanel2x.setdilresindex(rindex2 + 1);
//                    this.pkDilreserve.setSelectedIndex(rindex2);
//                    this.pkDilreserve.setEnabled(true);
                  }
	               
                  pindex1 = pSearch(this.platetypes, this.pkd1u);
                  pindex2 = pSearch(this.platetypes, this.pkd2u);
                  if (pindex1 >= 0)
                  {
                      pkdataPanel2x.setdeck1index(pindex1);
//                    this.pkDilsource.setEnabled(true);
//                    this.pkdeck1.setEnabled(true);
//                    this.pkdeck1.setSelectedIndex(pindex1);
                  }
                  if (pindex2 >= 0)
                  {
                    pkdataPanel2x.setdeck2index(pindex2);
//                    this.pkdeck2.setVisible(true);
//                    this.pkdeck2.setEnabled(true);
//                    this.pkDilreserve.setEnabled(true);
//                	this.lblDiluent2.setVisible(true);
//                    this.pkdeck2.setSelectedIndex(pindex2);
                  }
             }
              
              int tindex = tabSearch(this.tabs, "PK");
              if (tindex > -1)
              {
                this.jTabbedPane1.setSelectedIndex(tindex);
              }
              
              VAStools.setJLabelImage(PanelPKB2.lblPkBlank, "TelosLogo.png", -1, "", 199, 74);							//APN0047
            }

          if (steptype.equals("move plate")) {
            this.updButton10.setEnabled(true);
            this.checkButton.setEnabled(true);
            this.addButton.setEnabled(false);
            this.addButton10.setEnabled(false);
            this.rmvButton.setEnabled(true);
            this.arrowButton1.setEnabled(false);
            this.arrowButton2.setEnabled(false);
            for (int t = 0; t < this.tabcount; t++) {
              this.jTabbedPane1.setEnabledAt(t, false);
            }
            if (counter == 4) {
              String line4 = currentFList.data.toString();
              index1 = line4.indexOf(',');
              len = line4.length();
              this.spl3u = line4.substring(0, index1);
              this.dpl3u = line4.substring(index1 + 1, len);
              pindex1 = pSearch(this.platetypes, this.spl3u);
              pindex2 = pSearch(this.platetypes, this.dpl3u);
              if (pindex1 >= 0)
              {
                this.splatetype3.setSelectedIndex(pindex1);
                this.splatetype3.revalidate();
                this.splatetype3.repaint();
              }
              if (pindex2 >= 0)
              {
                this.dplatetype3.setSelectedIndex(pindex2);
                this.dplatetype3.revalidate();
                this.dplatetype3.repaint();
              }

            }

            if (counter == 5) {
              String line5 = currentFList.data.toString();
              len = line5.length();
              String ressrc = line5.substring(0, len);
              sindex = p96Search(this.plates96, ressrc);
              if (sindex == -1)
              {
                sindex = p384Search(this.plates384, ressrc);
                sindex += this.plates96.length;
              }
              this.source5.setSelectedIndex(sindex);
            }

            int tindex = tabSearch(this.tabs, "Move Plate");
            if (tindex > -1)
            {
              this.jTabbedPane1.setSelectedIndex(tindex);
            }

          }

          if (steptype.equals("move")) {
            this.updButton8.setEnabled(true);
            this.checkButton.setEnabled(true);
            this.addButton8.setEnabled(false);
            this.rmvButton.setEnabled(true);
            this.arrowButton1.setEnabled(false);
            this.arrowButton2.setEnabled(false);
            for (int t = 0; t < this.tabcount; t++) {
              this.jTabbedPane1.setEnabledAt(t, false);
            }

            int tindex = tabSearch(this.tabs, "Move");
            if (tindex > -1)
            {
              this.jTabbedPane1.setSelectedIndex(tindex);
            }

          }

          if (steptype.equals("Cytomat01")) {
              this.cyt_btnUpdateStep.setEnabled(true);
              this.checkButton.setEnabled(true);
              this.addButton.setEnabled(false);
              this.cyt_btnAddStep.setEnabled(false);
              this.rmvButton.setEnabled(true);
              this.arrowButton1.setEnabled(false);
              this.arrowButton2.setEnabled(false);
              for (int t = 0; t < this.tabcount; t++) {
                this.jTabbedPane1.setEnabledAt(t, false);
              }
              if (counter == 3) {
                String line3 = currentFList.data.toString();
                cytfromto = line3.trim();
                TabWindow.this.cytodatapanel2.setFTIndex(Integer.parseInt(cytfromto));
              }

              if (counter == 4) {
                  String line4 = currentFList.data.toString();
                  cytloc = line4.trim();
                  cytodatapanel2.setText1(cytloc);
                }

              if (counter == 5) {
                  String line5 = currentFList.data.toString();
                  cytdeckloc = line5.trim();
                  cytodatapanel2.setText2(cytdeckloc);
                                  }

              if (counter == 6) {
                  String line6 = currentFList.data.toString();
                  cytcover = line6.trim();
                  TabWindow.this.cytodatapanel2.setCoverIndex(Integer.parseInt(cytcover));
//                  this.cytodatapanel2.revalidate();
//                  this.cytodatapanel2.repaint();
                }

              if (counter == 7) {
                  String line7 = currentFList.data.toString();
                  cyt_plate = line7.trim();
				  int pindex = pAllSearch(this.platesAll, this.cyt_plate);	
//                  int pindex = p96Search(this.plates96, this.cyt_plate);
//                  if (pindex == -1)
//                  {
//                    pindex = p384Search(this.plates384, this.cyt_plate);
//                    pindex += this.plates96.length;
//                  }
                  cytodatapanel2.setPlateIndex(pindex);
//                  this.cytodatapanel2.revalidate();
//                  this.cytodatapanel2.repaint();
                }

              if (counter == 8) {
                  String line8 = currentFList.data.toString();
                  cyt_tsp = line8.trim();
                  int pindex = pSearch(this.platetypes, this.cyt_tsp);
                  cytodatapanel2.setTSPIndex(pindex);
                  this.cytodatapanel2.revalidate();
                  this.cytodatapanel2.repaint();
                }


              int tindex = tabSearch(this.tabs, "Cytomat01");
              if (tindex > -1)
              {
                this.jTabbedPane1.setSelectedIndex(tindex);
              }

            }

          if (steptype.equals("Liconic01")) {
              this.lic_btnUpdateStep.setEnabled(true);
              this.checkButton.setEnabled(true);
              this.addButton.setEnabled(false);
              this.lic_btnAddStep.setEnabled(false);
              this.rmvButton.setEnabled(true);
              this.arrowButton1.setEnabled(false);
              this.arrowButton2.setEnabled(false);
              for (int t = 0; t < this.tabcount; t++) {
                this.jTabbedPane1.setEnabledAt(t, false);
              }
              if (counter == 3) {
                  String line3 = currentFList.data.toString();
                  licfromto = line3.trim();
                  TabWindow.this.licodatapanel2.setFTIndex(Integer.parseInt(licfromto));
                }

                if (counter == 4) {
                    String line4 = currentFList.data.toString();
                    licloc = line4.trim();
                    licodatapanel2.setText1(licloc);
                  }

                if (counter == 5) {
                    String line5 = currentFList.data.toString();
                    licdeckloc = line5.trim();
                    licodatapanel2.setText2(licdeckloc);
                  }

                if (counter == 6) {
                    String line6 = currentFList.data.toString();
                    liccover = line6.trim();
                    TabWindow.this.licodatapanel2.setCoverIndex(Integer.parseInt(liccover));
                  }

                if (counter == 7) {
                    String line7 = currentFList.data.toString();
                    lic_plate = line7.trim();
  				  	int pindex = pAllSearch(this.platesAll, this.lic_plate);	
  				  	TabWindow.this.licodatapanel2.setPlateIndex(pindex);
                  }

                if (counter == 8) {
                    String line8 = currentFList.data.toString();
                    lic_tsp = line8.trim();
                    int pindex = pSearch(this.platetypes, this.lic_tsp);
                    licodatapanel2.setTSPIndex(pindex);
                    this.licodatapanel2.revalidate();
                    this.licodatapanel2.repaint();
                  }

              int tindex = tabSearch(this.tabs, "Liconic01");
              if (tindex > -1)
              {
                this.jTabbedPane1.setSelectedIndex(tindex);
              }

            }

          if (steptype.equals("MovePlate02")) {
              this.movplv2_btnUpdateStep.setEnabled(true);
              this.checkButton.setEnabled(true);
              this.addButton.setEnabled(false);
              this.movplv2_btnAddStep.setEnabled(false);
              this.rmvButton.setEnabled(true);
              this.arrowButton1.setEnabled(false);
              this.arrowButton2.setEnabled(false);
              for (int t = 0; t < this.tabcount; t++) {
                this.jTabbedPane1.setEnabledAt(t, false);
              }
              if (counter == 3) {
                String line3 = currentFList.data.toString();
                movetype = line3.trim();
                TabWindow.this.movplv2datapanel2.setTypeIndex(Integer.parseInt(movetype));
              }

              if (counter == 4) {
                  String line4 = currentFList.data.toString();
                  startloc = line4.trim();
                  movplv2datapanel2.setText1(startloc);
                }

              if (counter == 5) {
                  String line5 = currentFList.data.toString();
                  endloc = line5.trim();
                  movplv2datapanel2.setText2(endloc);
                                  }

              if (counter == 6) {
                  String line6 = currentFList.data.toString();
                  mvp_plate = line6.trim();
				  int pindex = pAllSearch(this.platesAll, this.mvp_plate);	
//                  int pindex = p96Search(this.plates96, this.mvp_plate);
//                  if (pindex == -1)
//                  {
//                    pindex = p384Search(this.plates384, this.mvp_plate);
//                    pindex += this.plates96.length;
//                  }
                  movplv2datapanel2.setPlateIndex(pindex);
                  this.movplv2datapanel2.revalidate();
                  this.movplv2datapanel2.repaint();
                }


              int tindex = tabSearch(this.tabs, "MovePlate02");
              if (tindex > -1)
              {
                this.jTabbedPane1.setSelectedIndex(tindex);
              }

            }


          if (steptype.equals("Pause")) {
            this.updButton9.setEnabled(true);
            this.checkButton.setEnabled(true);
            this.addButton9.setEnabled(false);
            this.rmvButton.setEnabled(true);
            this.arrowButton1.setEnabled(false);
            this.arrowButton2.setEnabled(false);
            for (int t = 0; t < this.tabcount; t++) {
              this.jTabbedPane1.setEnabledAt(t, false);
            }

            if (counter == 3) {
                String line3 = currentFList.data.toString();
                index1 = line3.indexOf(',');
                len = line3.length();

                if (len > 1)
                {
                index1 = line3.indexOf(',');
                index2 = line3.indexOf(',', index1 + 1);
                if (line3.substring(0, index1).equals("1"))
                {
               	 this.pausedataPanel2.setTimed(true);
                }
                else
                {
               	 this.pausedataPanel2.setTimed(false);
                }
                
                	this.pausedataPanel2.setMinutes(line3.substring(index1 + 1, index2));
                	this.pausedataPanel2.setSeconds(line3.substring(index2 + 1, len));
                }
                else
                {
               	 this.pausedataPanel2.setTimed(false);
       			 this.pausedataPanel2.setMinutes("");
       			 this.pausedataPanel2.setSeconds("");
               	 
                }
              }

            if (counter == 4) {
               String line4 = currentFList.data.toString();
               len = line4.length();
               this.pausedataPanel2.setMessage(line4.trim());
              }

            int tindex = tabSearch(this.tabs, "Pause");
            if (tindex > -1)
            {
              this.jTabbedPane1.setSelectedIndex(tindex);
            }

          }

          if (steptype.equals("read")) {
            this.updButton4.setEnabled(true);
            this.checkButton.setEnabled(true);
            this.addButton4.setEnabled(false);
            this.rmvButton.setEnabled(true);
            this.arrowButton1.setEnabled(false);
            this.arrowButton2.setEnabled(false);
            for (int t = 0; t < this.tabcount; t++) {
              this.jTabbedPane1.setEnabledAt(t, false);
            }

            if (counter == 3) {
              String line3 = currentFList.data.toString();
              index1 = line3.indexOf(',');
              String read = line3.substring(0, index1);
              if (read.equals("LMax384II"))
              {
                this.reader.setSelectedIndex(0);
              }
              if (read.equals("M5"))
              {
                this.reader.setSelectedIndex(1);
              }
              if (read.equals("Plus384"))
              {
                this.reader.setSelectedIndex(2);
              }
              if (read.equals("MSD"))
              {
                this.reader.setSelectedIndex(3);
              }
            }

            if (counter == 4) {
              this.text6.setText(currentFList.data.toString());
            }

            int tindex = tabSearch(this.tabs, "Read");
            if (tindex > -1)
            {
              this.jTabbedPane1.setSelectedIndex(tindex);
            }
          }

          if (steptype.equals("incubate")) {
            this.updButton2.setEnabled(true);
            this.checkButton.setEnabled(true);
            this.addButton2.setEnabled(false);
            this.rmvButton.setEnabled(true);
            this.arrowButton1.setEnabled(false);
            this.arrowButton2.setEnabled(false);
            for (int t = 0; t < this.tabcount; t++) {
              this.jTabbedPane1.setEnabledAt(t, false);
            }

            if (counter == 3) {
              this.text4.setText(currentFList.data.toString());
            }
            if (counter == 4) {
              this.itemp.setText(currentFList.data.toString());
            }
            if (counter == 5) {
              this.text4pm.setText(currentFList.data.toString());
            }

            int tindex = tabSearch(this.tabs, "Incubate");
            if (tindex > -1)
            {
              this.jTabbedPane1.setSelectedIndex(tindex);
            }
          }

          if (steptype.equals("Cytomat01")) {
              this.cyt_btnUpdateStep.setEnabled(true);
              this.checkButton.setEnabled(true);
              this.addButton.setEnabled(false);
              this.cyt_btnAddStep.setEnabled(false);
              this.rmvButton.setEnabled(true);
              this.arrowButton1.setEnabled(false);
              this.arrowButton2.setEnabled(false);
              for (int t = 0; t < this.tabcount; t++) {
                this.jTabbedPane1.setEnabledAt(t, false);
              }
              if (counter == 3) {
                String line3 = currentFList.data.toString();
                cytfromto = line3.trim();
                TabWindow.this.cytodatapanel2.setFTIndex(Integer.parseInt(cytfromto));
              }

              if (counter == 4) {
                  String line4 = currentFList.data.toString();
                  cytloc = line4.trim();
                  cytodatapanel2.setText1(cytloc);
                }

              if (counter == 5) {
                  String line5 = currentFList.data.toString();
                  cytdeckloc = line5.trim();
                  cytodatapanel2.setText2(cytdeckloc);
                }

              if (counter == 6) {
                  String line6 = currentFList.data.toString();
                  cytcover = line6.trim();
                  TabWindow.this.cytodatapanel2.setCoverIndex(Integer.parseInt(cytcover));
                }

              if (counter == 7) {
                  String line7 = currentFList.data.toString();
                  cyt_plate = line7.trim();
				  int pindex = pAllSearch(this.platesAll, this.cyt_plate);	
//                  int pindex = p96Search(this.plates96, this.cyt_plate);
//                  if (pindex == -1)
//                  {
//                    pindex = p384Search(this.plates384, this.cyt_plate);
//                    pindex += this.plates96.length;
//                  }
                  cytodatapanel2.setPlateIndex(pindex);
                }

              if (counter == 8) {
                  String line8 = currentFList.data.toString();
                  cyt_tsp = line8.trim();
                  int pindex = pSearch(this.platetypes, this.cyt_tsp);
                  cytodatapanel2.setTSPIndex(pindex);
                  this.cytodatapanel2.revalidate();
                  this.cytodatapanel2.repaint();
                }

              int tindex = tabSearch(this.tabs, "Cytomat01");
              if (tindex > -1)
              {
                this.jTabbedPane1.setSelectedIndex(tindex);
              }
            }

          if (steptype.equals("Liconic01")) {
              this.lic_btnUpdateStep.setEnabled(true);
              this.checkButton.setEnabled(true);
              this.addButton.setEnabled(false);
              this.lic_btnAddStep.setEnabled(false);
              this.rmvButton.setEnabled(true);
              this.arrowButton1.setEnabled(false);
              this.arrowButton2.setEnabled(false);
              for (int t = 0; t < this.tabcount; t++) {
                this.jTabbedPane1.setEnabledAt(t, false);
              }

              if (counter == 3) {
                  String line3 = currentFList.data.toString();
                  licfromto = line3.trim();
                  TabWindow.this.licodatapanel2.setFTIndex(Integer.parseInt(licfromto));
                }

              if (counter == 4) {
                  String line4 = currentFList.data.toString();
                  licloc = line4.trim();
                  licodatapanel2.setText1(licloc);
                }

              if (counter == 5) {
                  String line5 = currentFList.data.toString();
                  licdeckloc = line5.trim();
                  licodatapanel2.setText2(licdeckloc);
                }

              if (counter == 6) {
                  String line6 = currentFList.data.toString();
                  liccover = line6.trim();
                  TabWindow.this.licodatapanel2.setCoverIndex(Integer.parseInt(liccover));
                }

              if (counter == 7) {
                  String line7 = currentFList.data.toString();
                  lic_plate = line7.trim();
                  int pindex = pAllSearch(this.platesAll, this.lic_plate);	
                  TabWindow.this.licodatapanel2.setPlateIndex(pindex);
                }

              if (counter == 8) {
                  String line8 = currentFList.data.toString();
                  lic_tsp = line8.trim();
                  int pindex = pSearch(this.platetypes, this.lic_tsp);
                  licodatapanel2.setTSPIndex(pindex);
                  this.licodatapanel2.revalidate();
                  this.licodatapanel2.repaint();
                }

              int tindex = tabSearch(this.tabs, "Liconic01");
              if (tindex > -1)
              {
                this.jTabbedPane1.setSelectedIndex(tindex);
              }
            }

          
          if (steptype.equals("MovePlate02")) {
              this.movplv2_btnUpdateStep.setEnabled(true);
              this.checkButton.setEnabled(true);
              this.addButton.setEnabled(false);
              this.movplv2_btnAddStep.setEnabled(false);
              this.rmvButton.setEnabled(true);
              this.arrowButton1.setEnabled(false);
              this.arrowButton2.setEnabled(false);
              for (int t = 0; t < this.tabcount; t++) {
                this.jTabbedPane1.setEnabledAt(t, false);
              }
              if (counter == 3) {
                String line3 = currentFList.data.toString();
                movetype = line3.trim();
                TabWindow.this.movplv2datapanel2.setTypeIndex(Integer.parseInt(movetype));
                
              }

              if (counter == 4) {
                  String line4 = currentFList.data.toString();
                  startloc = line4.trim();
                  movplv2datapanel2.setText1(startloc);
                }

              if (counter == 5) {
                  String line5 = currentFList.data.toString();
                  endloc = line5.trim();
                  movplv2datapanel2.setText2(endloc);
                                  }

              if (counter == 6) {
                  String line6 = currentFList.data.toString();
                  mvp_plate = line6.trim();
				  int pindex = pAllSearch(this.platesAll, this.mvp_plate);	
//                  int pindex = p96Search(this.plates96, this.mvp_plate);
//                  if (pindex == -1)
//                  {
//                    pindex = p384Search(this.plates384, this.mvp_plate);
//                    pindex += this.plates96.length;
//                  }
                  movplv2datapanel2.setPlateIndex(pindex);
                  this.movplv2datapanel2.revalidate();
                  this.movplv2datapanel2.repaint();
                }


              int tindex = tabSearch(this.tabs, "MovePlate02");
              if (tindex > -1)
              {
                this.jTabbedPane1.setSelectedIndex(tindex);
              }
            }

          if (steptype.equals("CORDNA01")) {
              this.dna_btnUpdateStep.setEnabled(true);
              this.checkButton.setEnabled(true);
              this.addButton.setEnabled(false);
              this.dna_btnAddStep.setEnabled(false);
              this.rmvButton.setEnabled(true);
              this.arrowButton1.setEnabled(false);
              this.arrowButton2.setEnabled(false);
              for (int t = 0; t < this.tabcount; t++) {
                this.jTabbedPane1.setEnabledAt(t, false);
              }
              if (counter == 3) {
                String line3 = currentFList.data.toString();
                dna_qfile = line3.trim();
                TabWindow.this.dnadataPanel2.setText(dna_qfile);
        }

              if (counter == 4) {
                  String line4 = currentFList.data.toString();
                  proc = line4.trim();
                  TabWindow.this.dnadataPanel2.setProcIndex(Integer.parseInt(proc));
                }

              if (counter == 5) {
                  String line5 = currentFList.data.toString();
                  index1 = line5.indexOf(',');
                  len = line5.length();
                  this.dna_con = line5.substring(0, index1);
                  this.dna_vol = line5.substring(index1 + 1, len);
                  TabWindow.this.dnadataPanel2.setText1(dna_con);
                  TabWindow.this.dnadataPanel2.setText2(dna_vol);
                }

              if (counter == 6) {
                  String line6 = currentFList.data.toString();
                  index1 = line6.indexOf(',');
                  index2 = line6.indexOf(',', index1 + 1);
                  index3 = line6.indexOf(',', index2 + 1);
                  len = line6.length();
                  this.dna_dplate = line6.substring(0, index1);
                  this.dna_splate = line6.substring(index1 + 1, index2);
                  this.dna_nplate = line6.substring(index2 + 1, index3);
                  this.dna_plate = line6.substring(index3 + 1, len);
                  dindex = rAllSearch(this.reservoirsAll, dna_dplate);	//AP-0001
                  sindex = tAllSearch(this.tubesAll, dna_splate);	//AP-0001
                  int nindex = pPATSearch(this.platesTubes, dna_nplate);	//AP-0001
                  int pindex = pPATSearch(this.platesTubes, dna_plate);	//AP-0001
                  TabWindow.this.dnadataPanel2.setDPlateIndex(dindex);
                  TabWindow.this.dnadataPanel2.setSPlateIndex(sindex);
                  TabWindow.this.dnadataPanel2.setNPlateIndex(nindex);
                  TabWindow.this.dnadataPanel2.setPlateIndex(pindex);
                }

              if (counter == 7) {
                  String line7 = currentFList.data.toString();
                  index1 = line7.indexOf(',');
                  index2 = line7.indexOf(',', index1 + 1);
                  index3 = line7.indexOf(',', index2 + 1);
                  len = line7.length();
                  this.dna_deck1 = line7.substring(0, index1);
                  this.dna_deck2 = line7.substring(index1 + 1, index2);
                  this.dna_deck3 = line7.substring(index2 + 1, index3);
                  this.dna_deck4 = line7.substring(index3 + 1, len);
                  TabWindow.this.dnadataPanel2.setDeckIndex1(Integer.parseInt(dna_deck1));
                  TabWindow.this.dnadataPanel2.setDeckIndex2(Integer.parseInt(dna_deck2));
                  TabWindow.this.dnadataPanel2.setDeckIndex3(Integer.parseInt(dna_deck3));
                  TabWindow.this.dnadataPanel2.setDeckIndex4(Integer.parseInt(dna_deck4));
                }

              int tindex = tabSearch(this.tabs, "CORDNA01");
              if (tindex > -1)
              {
                this.jTabbedPane1.setSelectedIndex(tindex);
              }

            }

          if (steptype.equals("LOAD01")) {

              this.load_btnUpdateStep.setEnabled(true);
              this.checkButton.setEnabled(true);
              this.addButton.setEnabled(false);
              this.load_btnAddStep.setEnabled(false);
              this.rmvButton.setEnabled(true);
              this.arrowButton1.setEnabled(false);
              this.arrowButton2.setEnabled(false);
              for (int t = 0; t < this.tabcount; t++) {
                this.jTabbedPane1.setEnabledAt(t, false);
              }
              if (counter == 3) {
                String line3 = currentFList.data.toString();
                index1 = line3.indexOf(',');
                len = line3.length();
                this.load_plindex = line3.substring(0, index1);
                int num = Integer.parseInt(load_plindex);
                this.load_plate = line3.substring(index1 + 1, len);
                TabWindow.this.loaddataPanel2.setPlateIndex(num);
                TabWindow.this.loaddataPanel2.setPlateText(load_plate);
        }

              if (counter == 4) {
                  String line4 = currentFList.data.toString();
                  index1 = line4.indexOf(',');
                  len = line4.length();
                  this.load_tuindex = line4.substring(0, index1);
                  int num = Integer.parseInt(load_tuindex);
                  this.load_tube = line4.substring(index1 + 1, len);
                  TabWindow.this.loaddataPanel2.setTubeIndex(num);
                  TabWindow.this.loaddataPanel2.setTubeText(load_tube);
                }

              if (counter == 5) {
                  String line5 = currentFList.data.toString();
                  index1 = line5.indexOf(',');
                  len = line5.length();
                  this.load_tiindex = line5.substring(0, index1);
                  int num = Integer.parseInt(load_tiindex);
                  this.load_tip = line5.substring(index1 + 1, len);
                  TabWindow.this.loaddataPanel2.setTipIndex(num);
                  TabWindow.this.loaddataPanel2.setTipText(load_tip);
                }


              int tindex = tabSearch(this.tabs, "LOAD01");
              if (tindex > -1)
              {
                this.jTabbedPane1.setSelectedIndex(tindex);
              }

            }


          if (currentFList == null)
            continue;
          currentFList = currentFList.next;
        }

        while (!this.bList.isEmpty())
        {
          this.bList.removeFromBack();
        }
        while (!this.eList.isEmpty())
        {
          this.eList.removeFromBack();
        }
        while (!this.dbList.isEmpty())
        {
          this.dbList.removeFromBack();
        }
        while (!this.deList.isEmpty())
        {
          this.deList.removeFromBack();
        }

        currentFList = this.fList.getFirstNode();

        count = 0;
        while (currentFList != null)
        {
          count++;
          if (count > val * 10) {
            this.eList.insertAtBack(currentFList.data.toString());
          }
          currentFList = currentFList.next;
        }

        currentDList = this.dList.getFirstNode();

        count = 0;
        while (currentDList != null)
        {
          count++;
          if (count > val) {
            this.deList.insertAtBack(currentDList.data.toString());
          }
          currentDList = currentDList.next;
        }

        count = 0;
        while (currentTList != null)
        {
          count++;
          if (count > val) {
            this.teList.insertAtBack(currentTList.data.toString());
          }
          currentTList = currentTList.next;
        }

      }

      if (val > 1) {
        for (int x = 1; x <= (val - 1) * 10; x++)
        {
          String list = currentFList.data.toString();
          currentFList = currentFList.next;
        }
        for (int counter2 = 1; counter2 <= 10; counter2++)
        {
          if ((counter2 == 2) && (currentFList != null)) {
            steptype = currentFList.data.toString();
            
          }
          if (steptype.equals("add")) {
            this.updButton1.setEnabled(true);
            this.checkButton.setEnabled(true);
            this.addButton1.setEnabled(false);
            this.addButton.setEnabled(false);
            this.rmvButton.setEnabled(true);
            this.arrowButton1.setEnabled(false);
            this.arrowButton2.setEnabled(false);
            for (int t = 0; t < this.tabcount; t++) {
              this.jTabbedPane1.setEnabledAt(t, false);
            }

            if ((counter2 == 3) && (currentFList != null)) {
              String line3 = currentFList.data.toString();
              index1 = line3.indexOf(',');
              len = line3.length();
              index2 = line3.indexOf(',', index1 + 1);
              index3 = line3.indexOf(',', index2 + 1);
              this.text2.setText(line3.substring(0, index1));
              String lt1 = line3.substring(index1 + 1, index2);
              lindex = lSearch(this.liqtypes, lt1);
              this.liqtype.setSelectedIndex(lindex);
            }
            if ((counter2 == 4) && (currentFList != null)) {
              String line4 = currentFList.data.toString();
              index1 = line4.indexOf(',');
              len = line4.length();
              this.spl1u = line4.substring(0, index1);
              this.dpl1u = line4.substring(index1 + 1, len);
              pindex1 = pSearch(this.platetypes, this.spl1u);
              pindex2 = pSearch(this.platetypes, this.dpl1u);
              if (pindex1 >= 0)
              {
                this.platebutton.setSelected(true);
                this.splatetype.setVisible(true);
                this.splatetype.setSelectedIndex(pindex1);
              }
              if (pindex2 >= 0)
              {
                this.platebutton2.setSelected(true);
                this.dplatetype.setVisible(true);
                this.dplatetype.setSelectedIndex(pindex2);
              }

            }

            if ((counter2 == 5) && (currentFList != null)) {
              String line5 = currentFList.data.toString();
              index1 = line5.indexOf(',');
              index2 = line5.indexOf(',', index1 + 1);
              index3 = line5.indexOf(',', index2 + 1);
              index4 = line5.indexOf(',', index3 + 1);
              index5 = line5.indexOf(',', index4 + 1);
              len = line5.length();
              String addsrc = line5.substring(0, index1);
              sindex = p24Search(this.plates24, addsrc);
              if (sindex >= 0)
              {
                b24l = true;
               sindex = pAllSearch(this.platesAll, addsrc);
              }
              else
              {
                b24l = false;
              }
              if (sindex == -1)
              {
               sindex = p48Search(this.plates48, addsrc);
               if (sindex >= 0)
               {
                 b48l = true;
                 sindex = pAllSearch(this.platesAll, addsrc);
               }
               else
               {
                 b48l = false;
               }
              }
              if (sindex == -1)
              {
               sindex = p96Search(this.plates96, addsrc);
               if (sindex >= 0)
               {
                 b96l = true;
                 sindex = pAllSearch(this.platesAll, addsrc);
               }
               else
               {
                 b96l = false;
               }
              }
              if (sindex == -1)
              {
                sindex = p384Search(this.plates384, addsrc);
                if (sindex >= 0)
                {
                  b384l = true;
                  sindex = pAllSearch(this.platesAll, addsrc);
               }
                else
                {
                  b384l = false;
                }
              }
              if (sindex == -1)
              {
                sindex = rSearch(this.reservoirs, addsrc);
                if (sindex >= 0)
                {
                  bresl = true;
                }
              }
              if (sindex == -1)
//              {
// 
//                sindex = tSearch(this.tubes, addsrc);
//              }
              
              {
                  sindex = tSearch(this.tubes, addsrc) +1;																	//APF0050	Added  +1
                  String graphicFile = "C:/Program Files/TELOS/AutoPilot/Graphics/" + addsrc + ".png";						//APF0050	Added  +1
          			if (new File(graphicFile).exists()) {																	//APF0050	Added  +1
          				tubepanel.label1.setText("");																		//APF0050	Added  +1
          				tubepanel.label1.setIcon(new ImageIcon(graphicFile));												//APF0050	Added  +1
          			}else{																									//APF0050	Added  +1
          				tubepanel.label1.setIcon(new ImageIcon(""));														//APF0050	Added  +1
          				tubepanel.label1.setText("NO IMAGE OF " + addsrc + " AVAILABLE");									//APF0050	Added  +1
          			} 																										//APF0050	Added  +1
                }
              
              String mix1 = line5.substring(index1 + 1, index2);
              String asp1 = line5.substring(index2 + 1, index3);
              String mp1 = line5.substring(index3 + 1, index4);
              String sh1 = line5.substring(index4 + 1, index5);
              String c1 = line5.substring(index5 + 1, index5 + 2);
              String i3 = line5.substring(index5 + 3, index5 + 4);
              String v1 = line5.substring(index5 + 5, index5 + 6);
              if (bresl)
              {
                this.rs1 = line5.substring(index5 + 7, len);
              }
              this.advwin.setASPMode(asp1);
              if (mp1.equals("Variable"))
              {
                this.advwin.setMPA(13);
              }
              else
              {
                this.advwin.setMPA(Integer.parseInt(mp1));
              }
              this.lmixt1.setText(mix1);
              if (c1.equals("1"))
              {
                this.advwin.setChill(true);
              }
              if (c1.equals("0"))
              {
                this.advwin.setChill(false);
              }
              if (sh1.equals("1"))
              {
                this.advwin.setShake(true);
              }
              if (sh1.equals("0"))
              {
                this.advwin.setShake(false);
              }
              if (v1.equals("1"))
              {
                this.advwin.setVStart(true);
              }
              if (v1.equals("0"))
              {
                this.advwin.setVStart(false);
              }
//              if (i3.equals("1"))
//              {
//                this.linc1button.setSelected(true);
//              }
//              if (i3.equals("2"))
//              {
//                this.linc2button.setSelected(true);
//              }
//              if (i3.equals("3"))
//              {
//                this.linc3button.setSelected(true);
//              }
            }

            if ((counter2 == 6) && (currentFList != null)) {
              String line6 = currentFList.data.toString();
              index1 = line6.indexOf(',');
              index2 = line6.indexOf(',', index1 + 1);
              index3 = line6.indexOf(',', index2 + 1);
              index4 = line6.indexOf(',', index3 + 1);
              index5 = line6.indexOf(',', index4 + 1);
              len = line6.length();
              String adddest = line6.substring(0, index1);
              String mix2 = line6.substring(index1 + 1, index2);

              String asp2 = line6.substring(index2 + 1, index3);
              String mp2 = line6.substring(index3 + 1, index4);
              String sh2 = line6.substring(index4 + 1, index5);
              String c2 = line6.substring(index5 + 1, index5 + 2);
              String i4 = line6.substring(index5 + 3, index5 + 4);
              String v2 = line6.substring(index5 + 5, index5 + 6);

              dindex = p24Search(this.plates24, adddest);
              if (dindex >= 0)
              {
                b24r = true;
                dindex = pAllSearch(this.platesAll, adddest);
              }
              else
              {
                b24r = false;
              }
              if (dindex == -1)
              {
                dindex = p48Search(this.plates48, adddest);
                if (dindex >= 0)
                {
                  b48r = true;
                  dindex = pAllSearch(this.platesAll, adddest);
                }
                else
                {
                  b48r = false;
                }
              }
              if (dindex == -1)
              {
                dindex = p96Search(this.plates96, adddest);
                if (dindex >= 0)
                {
                  b96r = true;
                  dindex = pAllSearch(this.platesAll, adddest);
                }
                else
                {
                  b96r = false;
                }
              }
              if (dindex == -1)
              {
                dindex = p384Search(this.plates384, adddest);
                if (dindex >= 0)
                {
                  b384r = true;
                  dindex = pAllSearch(this.platesAll, adddest);
                }
                else
                {
                  b384r = false;
                }
              }
              if (dindex == -1)
              {
                dindex = rSearch(this.reservoirs, adddest);
                if (dindex >= 0)
                {
                  bresr = true;
                }
              }
              if (dindex == -1)
//              {
//                dindex = tSearch(this.tubes, adddest);
//              }
              {
                  dindex = tSearch(this.tubes, adddest) +1;																	//APF0050	Added  +1																//APN0045	Added  +1
                  String graphicFile = "C:/Program Files/TELOS/AutoPilot/Graphics/" + adddest + ".png";						//APF0050	Added  +1
          			if (new File(graphicFile).exists()) {																	//APF0050	Added  +1
          				tubepanel2.label1.setText("");																		//APF0050	Added  +1
          				tubepanel2.label1.setIcon(new ImageIcon(graphicFile));												//APF0050	Added  +1
          			}else{																									//APF0050	Added  +1
          				tubepanel2.label1.setIcon(new ImageIcon(""));														//APF0050	Added  +1
          				tubepanel2.label1.setText("NO IMAGE OF " + adddest + " AVAILABLE");									//APF0050	Added  +1
          			} 																										//APF0050	Added  +1
                }
              if (bresr)
              {
                this.rs2 = line6.substring(index5 + 7, len);
              }
              this.advwind.setASPMode(asp2);
              if (mp2.equals("Variable"))
              {
                this.advwind.setMPA(13);
              }
              else
              {
                this.advwind.setMPA(Integer.parseInt(mp2));
              }
              this.lmixt2.setText(mix2);

              if (sh2.equals("1"))
              {
                this.advwind.setShake(true);
              }
              if (sh2.equals("0"))
              {
                this.advwind.setShake(false);
              }
              if (i4.equals("1"))
              {
                this.linc4button.setSelected(true);
              }
              if (i4.equals("2"))
              {
                this.linc5button.setSelected(true);
              }
              if (i4.equals("3"))
              {
                this.linc6button.setSelected(true);
              }
              if (c2.equals("1"))
              {
                this.advwind.setChill(true);
              }
              if (c2.equals("0"))
              {
                this.advwind.setChill(false);
              }
              if (v2.equals("1"))
              {
                this.advwind.setVStart(true);
              }
              if (v2.equals("0"))
              {
                this.advwind.setVStart(false);
              }

            }

            if ((counter2 == 7) && (currentFList != null)) {
              if (this.spl1u.equals("Tube"))
              {
                String line7 = currentFList.data.toString();
                index1 = line7.indexOf(',');
                index2 = line7.indexOf(',', index1 + 1);
                index3 = line7.indexOf(',', index2 + 1);
                String tube1 = line7.substring(0, index1);
                String tube2 = line7.substring(index1 + 1, index2);
                String i1 = line7.substring(index2 + 1, index2 + 2);
                this.tubepanel.setFirstTube(tube1);
                this.tubepanel.setLastTube(tube2);
                this.tubepanel.setIncButton(i1);
                this.tubebutton.setSelected(true);
                this.liqPanel1.removeAll();
                getSrc();
                this.source.setSelectedIndex(sindex);
                this.source.revalidate();
                this.source.repaint();
                this.splatetype.setVisible(false);
                this.liqPanel1.add(getLiqPanel1Top(), "North");
                this.liqPanel1.add(this.tubepanel, "Center");
                this.liqPanel1.add(getLiqPanel1Bot(), "South");
                this.liqPanel1.revalidate();
                this.liqPanel1.repaint();
              }
              if (bresl)
              {
                this.liqPanel1.removeAll();
                this.resbutton.setSelected(true);
                getSrc();
//                this.resTextArea[(Integer.parseInt(this.rs1) - 1)].setBorder(this.redline);								//APN0047 Commented out
                this.source.setSelectedIndex(sindex  +1);	//APN0047 added +1

                Object selectedItem = source.getSelectedItem();																//APN0047
                String graphicFile = "C:/Program Files/TELOS/AutoPilot/Graphics/" + String.valueOf(selectedItem) + ".png";	//APN0047
                
                ImageIcon imageIcon = new ImageIcon(graphicFile); // load the image to a imageIcon
    			Image image = imageIcon.getImage(); // transform it 
    			Image newimg = image.getScaledInstance(360, 220,  java.awt.Image.SCALE_SMOOTH); // scale it the smooth way  
    			imageIcon = new ImageIcon(newimg);  // transform it back	    
    			
    			
        		if (new File(graphicFile).exists()) {																		//APN0047
        			ReservoirPanel1.label1.setText("");																		//APN0047
        			ReservoirPanel1.label1.setIcon(imageIcon);												//APN0047
        		}else{																										//APN0047
        			ReservoirPanel1.label1.setIcon(new ImageIcon(""));														//APN0047
        			if (dindex == 0){																						//APN0047
        				ReservoirPanel1.label1.setText("SELECT RESERVOIR TYPE ABOVE");										//APN0047
        			}else{																									//APN0047
        				ReservoirPanel1.label1.setText("NO IMAGE OF " + String.valueOf(selectedItem) + " AVAILABLE");		//APN0047
        			}																										//APN0047
        		}																											//APN0047

                this.source.revalidate();
                
                TabWindow.this.liqPanel1.removeAll();															//APN0047
                TabWindow.this.liqPanel1.add(TabWindow.this.getLiqPanel1Top(), "North");						//APN0047
                TabWindow.this.liqPanel1.add(TabWindow.this.ReservoirPanel1, "Center");							//APN0047		
                TabWindow.this.liqPanel1.add(TabWindow.this.getLiqPanel1Bot(), "South");						//APN0047
                TabWindow.this.liqPanel1.revalidate();															//APN0047
                TabWindow.this.liqPanel1.repaint();																//APN0047
              }
              
              if ((!this.spl1u.equals("Tube")) && (!bresl))
              {
                String seq = currentFList.data.toString();
                if (b24l)
                {
                  this.liqPanel1.removeAll();
                  getSrc();
                  if (pindex1 >= 0)
                  {
                    this.platebutton.setSelected(true);
                    this.splatetype.setVisible(true);
                    this.splatetype.setSelectedIndex(pindex1);
                  }
                  this.source.setSelectedIndex(sindex);
                  this.source.revalidate();

                  this.panel24l.setButtons(seq);
                  this.liqPanel1.add(getLiqPanel1Top(), "North");
                  this.liqPanel1.add(this.panel24l, "Center");
                  this.liqPanel1.add(getLiqPanel1Bot(), "South");
                  this.liqPanel1top.revalidate();
                  this.liqPanel1top.repaint();
                  this.liqPanel1.revalidate();
                  this.liqPanel1.repaint();
                }
                if (b48l)
                {
                  this.liqPanel1.removeAll();
                  getSrc();
                  if (pindex1 >= 0)
                  {
                    this.platebutton.setSelected(true);
                    this.splatetype.setVisible(true);
                    this.splatetype.setSelectedIndex(pindex1);
                  }
                  this.source.setSelectedIndex(sindex);
                  this.source.revalidate();

                  this.panel48l.setButtons(seq);
                  this.liqPanel1.add(getLiqPanel1Top(), "North");
                  this.liqPanel1.add(this.panel48l, "Center");
                  this.liqPanel1.add(getLiqPanel1Bot(), "South");
                  this.liqPanel1top.revalidate();
                  this.liqPanel1top.repaint();
                  this.liqPanel1.revalidate();
                  this.liqPanel1.repaint();
                }
                if (b96l)
                {
                  this.liqPanel1.removeAll();
                  getSrc();
                  if (pindex1 >= 0)
                  {
                    this.platebutton.setSelected(true);
                    this.splatetype.setVisible(true);
                    this.splatetype.setSelectedIndex(pindex1);
                  }
                  this.source.setSelectedIndex(sindex);
                  this.source.revalidate();

                  this.panel96l.setButtons(seq);
                  this.liqPanel1.add(getLiqPanel1Top(), "North");
                  this.liqPanel1.add(this.panel96l, "Center");
                  this.liqPanel1.add(getLiqPanel1Bot(), "South");
                  this.liqPanel1top.revalidate();
                  this.liqPanel1top.repaint();
                  this.liqPanel1.revalidate();
                  this.liqPanel1.repaint();
                }
                if (b384l)
                {
                  this.panel384l.setButtons(seq);
//                  this.button384l2.setSelected(true);
                  this.liqPanel1.removeAll();
                  getSrc();
//                  this.source.removeAllItems();
//                  for (int i = 0; i < this.p384count; i++)
//                    this.source.addItem(this.plates384[i]);
                  if (pindex1 >= 0)
                  {
                    this.platebutton.setSelected(true);
                    this.splatetype.setVisible(true);
                    this.splatetype.setSelectedIndex(pindex1);
                  }
                  this.source.setSelectedIndex(sindex);
                  this.source.revalidate();
                  this.splatetype.setVisible(true);
                  this.panel384l.setButtons(seq);
                  this.button384l2.setSelected(true);
                  this.liqPanel1.add(getLiqPanel1Top(), "North");
                  this.liqPanel1.add(this.panel384l, "Center");
                  this.liqPanel1.add(getLiqPanel1Bot(), "South");
                  this.liqPanel1.revalidate();
                  this.liqPanel1.repaint();
                }
              }

            }

            if ((counter2 == 8) && (currentFList != null)) {
              if (this.dpl1u.equals("Tube"))
              {
                String line8 = currentFList.data.toString();
                index1 = line8.indexOf(',');
                index2 = line8.indexOf(',', index1 + 1);
                index3 = line8.indexOf(',', index2 + 1);
                String tube3 = line8.substring(0, index1);
                String tube4 = line8.substring(index1 + 1, index2);
                String i2 = line8.substring(index2 + 1, index2 + 2);
                this.tubepanel2.setFirstTube(tube3);
                this.tubepanel2.setLastTube(tube4);
                this.tubepanel2.setIncButton(i2);
                this.tubebutton2.setSelected(true);
                this.liqPanel2.removeAll();
                getDest();
                this.dest.setSelectedIndex(dindex);
                this.dest.revalidate();
                this.dest.repaint();
                this.dplatetype.setVisible(false);
                this.liqPanel2.add(getLiqPanel2Top(), "North");
                this.liqPanel2.add(this.tubepanel2, "Center");
                this.liqPanel2.add(getLiqPanel2Bot(), "South");
                this.liqPanel2.revalidate();
                this.liqPanel2.repaint();
                this.liqPanel2top.revalidate();
                this.liqPanel2top.repaint();
              }
              if (bresr)
              {

                  this.liqPanel2.removeAll();
                  this.resbutton2.setSelected(true);
                  getDest();
                  this.dest.setSelectedIndex(dindex  +1);	//APN0047 added +1
                  Object selectedItem = dest.getSelectedItem();																//APN0047
                  String graphicFile = "C:/Program Files/TELOS/AutoPilot/Graphics/" + String.valueOf(selectedItem) + ".png";	//APN0047
//                  this.resTextArea2[(Integer.parseInt(this.rs2) - 1)].setBackground(Color.blue);							//APN0047 Commented Out
          		if (new File(graphicFile).exists()) {																		//APN0047
          			ReservoirPanel2.label1.setText("");																		//APN0047
          			
                    ImageIcon imageIcon = new ImageIcon(graphicFile); // load the image to a imageIcon
        			Image image = imageIcon.getImage(); // transform it 
        			Image newimg = image.getScaledInstance(360, 220,  java.awt.Image.SCALE_SMOOTH); // scale it the smooth way  
        			imageIcon = new ImageIcon(newimg);  // transform it back
        			
          			ReservoirPanel2.label1.setIcon(imageIcon);																//APN0047
          		}else{																										//APN0047
          			ReservoirPanel2.label1.setIcon(new ImageIcon(""));														//APN0047
          			if (dindex == 0){																						//APN0047
          				ReservoirPanel2.label1.setText("SELECT RESERVOIR TYPE ABOVE");										//APN0047
          			}else{																									//APN0047
          				ReservoirPanel2.label1.setText("NO IMAGE OF " + String.valueOf(selectedItem) + " AVAILABLE");		//APN0047
          			}																										//APN0047
          		}																											//APN0047

                  this.dest.revalidate();
                  this.dplatetype.setVisible(true);
                  
                  TabWindow.this.liqPanel2.removeAll();															//APN0047
                  TabWindow.this.liqPanel2.add(TabWindow.this.getLiqPanel2Top(), "North");						//APN0047
                  TabWindow.this.liqPanel2.add(TabWindow.this.ReservoirPanel2, "Center");							//APN0047		
                  TabWindow.this.liqPanel2.add(TabWindow.this.getLiqPanel2Bot(), "South");						//APN0047
                  TabWindow.this.liqPanel2.revalidate();															//APN0047
                  TabWindow.this.liqPanel2.repaint();																//APN0047

                
                
                
              }
              if ((!this.dpl1u.equals("Tube")) && (!bresr))
              {
                String seq = currentFList.data.toString();
                if (b24r)
                {
                  getDest();
                  if (pindex2 >= 0)
                  {
                    this.platebutton2.setSelected(true);
                    this.dplatetype.setSelectedIndex(pindex2);
                  }
                  this.dest.setSelectedIndex(dindex);
                  this.dest.revalidate();
                  this.dplatetype.setVisible(true);
                  this.panel24r.setButtons(seq);
                  this.liqPanel2.add(getLiqPanel2Top(), "North");
                  this.liqPanel2.add(this.panel24r, "Center");
                  this.liqPanel2.add(getLiqPanel2Bot(), "South");
                  this.liqPanel2.revalidate();
                  this.liqPanel2.repaint();
                  this.liqPanel2top.revalidate();
                  this.liqPanel2top.repaint();
                }
                if (b48r)
                {
//                  this.liqPanel2.removeAll();
                  getDest();
//                  this.dest.removeAllItems();

//                  for (int i = 0; i < this.p48count; i++)
//                    this.dest.addItem(this.plates48[i]);
                  if (pindex2 >= 0)
                  {
                    this.platebutton2.setSelected(true);
                    this.dplatetype.setSelectedIndex(pindex2);
                  }
                  this.dest.setSelectedIndex(dindex);
                  this.dest.revalidate();
                  this.dplatetype.setVisible(true);
                  this.panel48r.setButtons(seq);
//                this.button48l.setSelected(true);
                  this.liqPanel2.add(getLiqPanel2Top(), "North");
                  this.liqPanel2.add(this.panel48r, "Center");
                  this.liqPanel2.add(getLiqPanel2Bot(), "South");
                  this.liqPanel2.revalidate();
                  this.liqPanel2.repaint();
                  this.liqPanel2top.revalidate();
                  this.liqPanel2top.repaint();
                }
                if (b96r)
                {
//                  this.liqPanel2.removeAll();
                  getDest();
//                  this.dest.removeAllItems();

//                  for (int i = 0; i < this.p96count; i++)
//                    this.dest.addItem(this.plates96[i]);
                  if (pindex2 >= 0)
                  {
                    this.platebutton2.setSelected(true);
                    this.dplatetype.setSelectedIndex(pindex2);
                  }
                  this.dest.setSelectedIndex(dindex);
                  this.dest.revalidate();
                  this.dplatetype.setVisible(true);
                  this.panel96r.setButtons(seq);
//                  this.button96l.setSelected(true);
                  this.liqPanel2.add(getLiqPanel2Top(), "North");
                  this.liqPanel2.add(this.panel96r, "Center");
                  this.liqPanel2.add(getLiqPanel2Bot(), "South");
                  this.liqPanel2.revalidate();
                  this.liqPanel2.repaint();
                  this.liqPanel2top.revalidate();
                  this.liqPanel2top.repaint();
                }
                if (b384r)
                {
//                  this.liqPanel2.removeAll();
                  getDest();
//                  this.dest.removeAllItems();
//                  for (int i = 0; i < this.p384count; i++)
//                    this.dest.addItem(this.plates384[i]);
                  this.dest.setSelectedIndex(dindex);
                  this.dest.revalidate();
                  this.dplatetype.setVisible(true);
                  this.panel384r.setButtons(seq);
//                  this.button384l.setSelected(true);
                  this.liqPanel2.add(getLiqPanel2Top(), "North");
                  this.liqPanel2.add(this.panel384r, "Center");
                  this.liqPanel2.add(getLiqPanel2Bot(), "South");
                  this.liqPanel2.revalidate();
                  this.liqPanel2.repaint();
                }
              }

            }

            if ((counter2 == 9) && (currentFList != null)) {
              this.text3.setText(currentFList.data.toString());
            }

            if ((counter2 == 10) && (currentFList != null)) {
              String line10 = currentFList.data.toString();
              String q1 = line10.substring(0, 1);
              String q2 = line10.substring(2, 3);
              String q3 = line10.substring(4, 5);
              String q4 = line10.substring(6, 7);
              String q5 = line10.substring(8, 9);
              String q6 = line10.substring(10, 11);
              String q7 = line10.substring(12, 13);
              String q8 = line10.substring(14, 15);
              this.panel384l.setQ1(q1);
              this.panel384l.setQ2(q2);
              this.panel384l.setQ3(q3);
              this.panel384l.setQ4(q4);
              this.panel384r.setQ1(q5);
              this.panel384r.setQ2(q6);
              this.panel384r.setQ3(q7);
              this.panel384r.setQ4(q8);
            }

            int tindex = tabSearch(this.tabs, "Liquid Transfer");
            if (tindex > -1)
            {
              this.jTabbedPane1.setSelectedIndex(tindex);
            }

          }

          if (steptype.equals("serial")) {
            this.updButton6.setEnabled(true);
            this.checkButton.setEnabled(true);
            this.addButton6.setEnabled(false);
            this.rmvButton.setEnabled(true);
            this.arrowButton1.setEnabled(false);
            this.arrowButton2.setEnabled(false);
            for (int t = 0; t < this.tabcount; t++) {
              this.jTabbedPane1.setEnabledAt(t, false);
            }

            if ((counter2 == 3) && (currentFList != null)) {
              String line3 = currentFList.data.toString();
              index1 = line3.indexOf(',');
              len = line3.length();
              index2 = line3.indexOf(',', index1 + 1);
              index3 = line3.indexOf(',', index2 + 1);
              index4 = line3.indexOf(',', index3 + 1);
              this.sttext.setText(line3.substring(0, index1));
              this.slt1 = line3.substring(index1 + 1, index2);
//            this.dltext.setText(line3.substring(index2 + 1, index3));
              this.spl4u = line3.substring(index2 + 1, index3);
              pindex1 = pSearch(this.platetypes, this.spl4u);
              if (pindex1 >= 0)
              {
                this.splatetype4.setSelectedIndex(pindex1);
              }
              this.dlt1 = line3.substring(index3 + 1, index4);
              this.so2 = line3.substring(index4 + 1, len);
              lindex = lSearch(this.liqtypes, this.slt1);
              this.sliqtype.setSelectedIndex(lindex);
              
              rindex1 = rSearch(this.reservoirs, this.so2);			//APF0023  Added this line
              if (rindex1 >= 0)										//APF0023  Added this line
              {														//APF0023  Added this line
                this.source2.setSelectedIndex(rindex1);				//APF0023  Added this line
                this.source.revalidate();							//APF0023  Added this line
                this.source.repaint();								//APF0023  Added this line
                this.serPanel1top.revalidate();						//APF0023  Added this line
                this.serPanel1top.repaint();						//APF0023  Added this line
              }														//APF0023  Added this line
              
              //////////////Removed manual index settings			//APF0023  removed several if() s.

            }

            if ((counter2 == 4) && (currentFList != null)) {
              String line4 = currentFList.data.toString();
              index1 = line4.indexOf(',');
              len = line4.length();
              this.spl2u = line4.substring(0, index1);
              this.dpl2u = line4.substring(index1 + 1, len);
              pindex1 = pSearch(this.platetypes, this.spl2u);
              pindex2 = pSearch(this.platetypes, this.dpl2u);

              if (pindex1 >= 0)
              {
                this.platebutton.setSelected(true);

                this.splatetype2.setVisible(true);
                this.splatetype2.setSelectedIndex(pindex1);
                this.splatetype2.revalidate();
                this.splatetype2.repaint();
              }

              if (pindex2 >= 0)
              {
                this.dplatetype2.setSelectedIndex(pindex2);
                this.dplatetype2.revalidate();
                this.dplatetype2.repaint();
              }

            }

            if ((counter2 == 5) && (currentFList != null)) {
              String line5 = currentFList.data.toString();
              index1 = line5.indexOf(',');
              index2 = line5.indexOf(',', index1 + 1);
              String sersrc = line5.substring(0, index1);
              String i3 = line5.substring(index1 + 5, index1 + 6);
              sindex = tSearch(this.tubes, sersrc);
              if (sindex == -1)
              {
                sindex = p96Search(this.plates96, sersrc);
              }
              if (sindex == -1)
              {
                sindex = rSearch(this.reservoirs, sersrc);
              }
              if (sindex == -1)
              {
                sindex = p384Search(this.plates384, sersrc);
                if (sindex >= 0)
                {
                  this.source3.setSelectedIndex(sindex);
                  b384s = true;
                }
              }

              if (i3.equals("1"))
              {
                this.sinc1button.setSelected(true);
              }
              if (i3.equals("2"))
              {
                this.sinc2button.setSelected(true);
              }
              if (i3.equals("3"))
              {
                this.sinc3button.setSelected(true);
              }
            }

            if ((counter2 == 6) && (currentFList != null)) {
              String line6 = currentFList.data.toString();
              index1 = line6.indexOf(',');
              index2 = line6.indexOf(',', index1 + 1);
              String serdest = line6.substring(0, index1);
              String i4 = line6.substring(index2 + 1, index2 + 6);
              dindex = p96Search(this.plates96, serdest);
              if (dindex >= 0)
              {
                this.dest2.setSelectedIndex(dindex);
              }
              if (dindex == -1)
              {
                dindex = p384Search(this.plates384, serdest);
                if (dindex >= 0)
                {
                  this.dest2.setSelectedIndex(dindex);
                  b384s = true;
                }

              }

              if (i4.equals("1"))
              {
                this.sinc4button.setSelected(true);
              }
              if (i4.equals("2"))
              {
                this.sinc5button.setSelected(true);
              }
              if (i4.equals("3"))
              {
                this.sinc6button.setSelected(true);
              }
            }

            if ((counter2 == 7) && (currentFList != null)) {
              if (this.spl2u.equals("Tube"))
              {
                String line7 = currentFList.data.toString();
                index1 = line7.indexOf(',');
                index2 = line7.indexOf(',', index1 + 1);
                index3 = line7.indexOf(',', index2 + 1);
                String tube1 = line7.substring(0, index1);
                String tube2 = line7.substring(index1 + 1, index2);
                String i1 = line7.substring(index2 + 1, index2 + 2);
                this.stubepanel.setFirstTube(tube1);
                this.stubepanel.setLastTube(tube2);
                this.stubepanel.setIncButton(i1);
                this.tubebutton3.setSelected(true);
                this.serPanel1.removeAll();
                getSrc3();
                this.source3.setSelectedIndex(sindex);
                this.source3.revalidate();
                this.source3.repaint();
                this.splatetype2.setVisible(false);
                this.serPanel1.add(getSerPanel1Top(), "North");
                this.serPanel1.add(this.stubepanel, "Center");
                this.serPanel1.revalidate();
                this.serPanel1.repaint();
              }
              if (!this.spl2u.equals("Tube"))
              {
                String seq = currentFList.data.toString();
                this.serPanel1.removeAll();
                this.platebutton3.setSelected(true);
                getSrc3();
                this.source3.setSelectedIndex(sindex);
                this.source3.revalidate();
                this.source3.repaint();
                this.splatetype2.setVisible(true);
                this.splatetype2.revalidate();
                this.splatetype2.repaint();
                this.serPanel1.add(getSerPanel1Top(), "North");
                this.serPanel1.add(this.spanel96l, "Center");
                this.spanel96l.setButtons(seq);
                this.serPanel1.revalidate();
                this.serPanel1.repaint();
                this.serPanel1top.revalidate();
                this.serPanel1top.repaint();
              }

            }

            if ((counter2 == 8) && (currentFList != null)) {
              String seq = currentFList.data.toString();
              if (!b384s)
              {
                this.serPanel2.removeAll();
                this.dest2.removeAllItems();
                for (int i = 0; i < this.p96count; i++)
                  this.dest2.addItem(this.plates96[i]);
                this.dest2.setSelectedIndex(dindex);
                this.dest2.revalidate();
                this.dplatetype2.setVisible(true);
                this.spanel96r.setButtons(seq);
                this.button96s.setSelected(true);
                this.serPanel2.add(getSerPanel2Top(), "North");
                this.serPanel2.add(this.spanel96r, "Center");
                this.serPanel2top.revalidate();
                this.serPanel2top.repaint();
                this.serPanel2.revalidate();
                this.serPanel2.repaint();
              }
              else
              {
                this.serPanel2.removeAll();
                this.dest2.removeAllItems();
                for (int i = 0; i < this.p384count; i++)
                {
                  this.dest2.addItem(this.plates384[i]);
                }
                this.dest2.revalidate();
                this.spanel384r.setButtons(seq);
                this.button384s.setSelected(true);
                this.serPanel2.add(getSerPanel2Top(), "North");
                this.serPanel2.add(this.spanel384r, "Center");
                this.serPanel2.revalidate();
                this.serPanel2.repaint();
              }
            }

            if (counter2 == 9) {	//SD2.1.1
                String transfer = currentFList.data.toString();
                Collections.addAll(arrayList_sdVolumesTransfer, transfer.split(","));
            }

            if (counter2 == 10) {	//SD2.1.1
                String dilution = currentFList.data.toString();
                Collections.addAll(arrayList_sdVolumesDiluent, dilution.split(","));
            }
              
            if ((counter2 != 9) || 
              (counter2 == 10));
            int tindex = tabSearch(this.tabs, "Serial Dilution");
            if (tindex > -1)
            {
              this.jTabbedPane1.setSelectedIndex(tindex);
            }

          }

          if (steptype.equals("brew")) {
            this.updButton7.setEnabled(true);
            this.checkButton.setEnabled(true);
            this.addButton7.setEnabled(false);
            this.rmvButton.setEnabled(true);
            this.arrowButton1.setEnabled(false);
            this.arrowButton2.setEnabled(false);
            for (int t = 0; t < this.tabcount; t++) {
              this.jTabbedPane1.setEnabledAt(t, false);
            }
            if ((counter2 == 3) && (currentFList != null)) {
              String line3 = currentFList.data.toString();
              index1 = line3.indexOf(',');
              index2 = line3.indexOf(',', index1 + 1);
              index3 = line3.indexOf(',', index2 + 1);
              this.brtext.setText(line3.substring(0, index1));
              String blt = line3.substring(index1 + 1, index2);
              lindex = lSearch(this.liqtypes, blt);
              this.liqtype.setSelectedIndex(lindex);
              this.brntext.setText(line3.substring(index2 + 1, index3));
            }
            if ((counter2 == 4) && (currentFList != null)) {
              String line4 = currentFList.data.toString();
              index1 = line4.indexOf(',');
              len = line4.length();
              bpl1 = line4.substring(0, index1);
              bpl2 = line4.substring(index1 + 1, len);
              if (bpl1.equals("Reservoir"))
              {
                this.brresbutton1.setSelected(true);
                this.brtubebutton1.setSelected(false);
                this.brewPanel1.removeAll();
                getSrc4();
                this.source4.revalidate();
                this.source4.repaint();
                this.brewPanel1.add(getBrewPanel1Top(), "North");
                this.brewPanel1.add(this.brrespanel1, "Center");
                this.brewPanel1.add(getBrewPanel1Bot(), "South");
                this.brewPanel1.revalidate();
                this.brewPanel1.repaint();
              }
              if (bpl1.equals("Tube"))
              {
                this.brtubebutton1.setSelected(true);
                this.brresbutton1.setSelected(false);
                this.brewPanel1.removeAll();
                getSrc4();
                this.source4.revalidate();
                this.source4.repaint();
                this.brewPanel1.add(getBrewPanel1Top(), "North");
                this.brewPanel1.add(this.brtubepanel, "Center");
                this.brewPanel1.add(getBrewPanel1Bot(), "South");
                this.brewPanel1.revalidate();
                this.brewPanel1.repaint();
              }
              if (bpl2.equals("Reservoir"))
              {
                this.brtubebutton2.setSelected(false);
                this.brresbutton2.setSelected(true);
                this.brewPanel2.removeAll();
                getDest3();
                this.brresbutton2.setSelected(true);
                this.dest3.revalidate();
                this.dest3.repaint();
                this.brewPanel2.add(getBrewPanel2Top(), "North");
                this.brewPanel2.add(this.brrespanel2, "Center");
                this.brewPanel2.add(getBrewPanel2Bot(), "South");
                this.brewPanel2.revalidate();
                this.brewPanel2.repaint();
              }
              if (bpl2.equals("Tube"))
              {
                this.brtubebutton2.setSelected(true);
                this.brresbutton2.setSelected(false);
                this.brewPanel2.removeAll();
                getDest3();
                this.dest3.revalidate();
                this.dest3.repaint();
                this.brewPanel2.add(getBrewPanel2Top(), "North");
                this.brewPanel2.add(this.brtubepanel2, "Center");
                this.brewPanel2.add(getBrewPanel2Bot(), "South");
                this.brewPanel2.revalidate();
                this.brewPanel2.repaint();
              }
            }
            if ((counter2 == 5) && (currentFList != null)) {
              String line5 = currentFList.data.toString();
              index1 = line5.indexOf(',');
              index2 = line5.indexOf(',', index1 + 1);
              String bsource = line5.substring(0, index1);
              String m1 = line5.substring(index1 + 1, index1 + 2);
              String c1 = line5.substring(index2 + 1, index2 + 2);
              int bindex1 = 0;
              bindex1 = rSearch(this.reservoirs, bsource);
              if (bindex1 > 0)
              {
                this.source4.setSelectedIndex(bindex1);
              }
              if (bindex1 == -1)
              {
                bindex1 = tSearch(this.tubes, bsource);
              }
              if (bindex1 > 0)
              {
                this.source4.setSelectedIndex(bindex1);
              }

              if (m1.equals("1"))
              {
                this.brmix.setSelected(true);
              }
              if (m1.equals("0"))
              {
                this.brmix.setSelected(false);
              }
              if (c1.equals("1"))
              {
                this.brchill.setSelected(true);
              }
              if (c1.equals("0"))
              {
                this.brchill.setSelected(false);
              }
            }

            if ((counter2 == 6) && (currentFList != null)) {
              String line6 = currentFList.data.toString();
              index1 = line6.indexOf(',');
              index2 = line6.indexOf(',', index1 + 1);
              String bdest = line6.substring(0, index1);
              int bindex2 = 0;
              bindex2 = rSearch(this.reservoirs, bdest);
              if (bindex2 > 0)
              {
                this.dest3.setSelectedIndex(bindex2);
              }
              if (bindex2 == -1)
              {
                bindex2 = tSearch(this.tubes, bdest);
              }
              if (bindex2 > 0)
              {
                this.dest3.setSelectedIndex(bindex2);
              }
            }
            if ((counter2 == 7) && (currentFList != null) && (bpl1.equals("Tube"))) {
              String line7 = currentFList.data.toString();
              index1 = line7.indexOf(',');
              index2 = line7.indexOf(',', index1 + 1);
              index3 = line7.indexOf(',', index2 + 1);
              String tube1 = line7.substring(0, index1);
              String tube2 = line7.substring(index1 + 1, index2);
              String i1 = line7.substring(index2 + 1, index2 + 2);
              this.brtubepanel.setFirstTube(tube1);
              this.brtubepanel.setLastTube(tube2);
              this.brtubepanel.setIncButton(i1);
            }
            if ((counter2 == 8) && (currentFList != null) && (bpl2.equals("Tube"))) {
              String line8 = currentFList.data.toString();
              index1 = line8.indexOf(',');
              index2 = line8.indexOf(',', index1 + 1);
              index3 = line8.indexOf(',', index2 + 1);
              String tube3 = line8.substring(0, index1);
              String tube4 = line8.substring(index1 + 1, index2);
              String i2 = line8.substring(index2 + 1, index2 + 2);
              this.brtubepanel2.setFirstTube(tube3);
              this.brtubepanel2.setLastTube(tube4);
              this.brtubepanel2.setIncButton(i2);
            }
            if ((counter2 == 9) && (currentFList != null)) {
              this.brvol.setText(currentFList.data.toString());
            }

            int tindex = tabSearch(this.tabs, "Brew");
            if (tindex > -1)
            {
              this.jTabbedPane1.setSelectedIndex(tindex);
            }

          }

          if (steptype.equals("wash")) {
            this.updButton3.setEnabled(true);
            this.checkButton.setEnabled(true);
            this.addButton3.setEnabled(false);
            this.rmvButton.setEnabled(true);
            this.arrowButton1.setEnabled(false);
            this.arrowButton2.setEnabled(false);
            for (int t = 0; t < this.tabcount; t++) {
              this.jTabbedPane1.setEnabledAt(t, false);
            }

            if ((counter2 == 4) && (currentFList != null)) {
              this.wmethod.setText(currentFList.data.toString());
            }
            int tindex = tabSearch(this.tabs, "Wash");
            if (tindex > -1)
            {
              this.jTabbedPane1.setSelectedIndex(tindex);
            }

          }

          if (steptype.equals("incubate")) {
        	  
            this.updButton2.setEnabled(true);
            this.checkButton.setEnabled(true);
            this.addButton2.setEnabled(false);
            this.rmvButton.setEnabled(true);
            this.arrowButton1.setEnabled(false);
            this.arrowButton2.setEnabled(false);
            for (int t = 0; t < this.tabcount; t++) {
              this.jTabbedPane1.setEnabledAt(t, false);
            }

            if ((counter2 == 3) && (currentFList != null)) {
              this.text4.setText(currentFList.data.toString());
            }
            if ((counter2 == 4) && (currentFList != null)) {
              this.itemp.setText(currentFList.data.toString());
            }
            if ((counter2 == 5) && (currentFList != null)) {
              this.text4pm.setText(currentFList.data.toString());
            }
            int tindex = tabSearch(this.tabs, "Incubate");
            if (tindex > -1)
            {
              this.jTabbedPane1.setSelectedIndex(tindex);
            }

          }

          if (steptype.equals("read")) {
            this.updButton4.setEnabled(true);
            this.checkButton.setEnabled(true);
            this.addButton4.setEnabled(false);
            this.rmvButton.setEnabled(true);
            this.arrowButton1.setEnabled(false);
            this.arrowButton2.setEnabled(false);
            for (int t = 0; t < this.tabcount; t++) {
              this.jTabbedPane1.setEnabledAt(t, false);
            }

            if ((counter2 == 3) && (currentFList != null)) {
              String line3 = currentFList.data.toString();
              index1 = line3.indexOf(',');
              String read = line3.substring(0, index1);
              if (read.equals("LMax384II"))
              {
                this.reader.setSelectedIndex(0);
              }
              if (read.equals("M5"))
              {
                this.reader.setSelectedIndex(1);
              }
              if (read.equals("Plus384"))
              {
                this.reader.setSelectedIndex(2);
              }
              if (read.equals("MSD"))
              {
                this.reader.setSelectedIndex(3);
              }
            }

            if ((counter2 == 4) && (currentFList != null)) {
              this.text6.setText(currentFList.data.toString());
            }

            int tindex = tabSearch(this.tabs, "Read");
            if (tindex > -1)
            {
              this.jTabbedPane1.setSelectedIndex(tindex);
            }

          }

          if (steptype.equals("SPE")) {
            this.updButton5.setEnabled(true);
            this.checkButton.setEnabled(true);
            this.addButton5.setEnabled(false);
            this.rmvButton.setEnabled(true);
            this.arrowButton1.setEnabled(false);
            this.arrowButton2.setEnabled(false);
            for (int t = 0; t < this.tabcount; t++) {
              this.jTabbedPane1.setEnabledAt(t, false);
            }

            if ((counter2 == 3) && (currentFList != null)) {
              String line3 = currentFList.data.toString();
              index1 = line3.indexOf(',');
              len = line3.length();
              index2 = line3.indexOf(',', index1 + 1);
              index3 = line3.indexOf(',', index2 + 1);
              index4 = line3.indexOf(',', index3 + 1);
              String b1 = line3.substring(0, index1);
              String fil = line3.substring(index1 + 1, index2);
              String col = line3.substring(index2 + 1, index3);
              String ti = line3.substring(index3 + 1, index4);
              String pre = line3.substring(index4 + 1, len);

//              int findex1 = 0;
//              findex1 = fSearch(this.filters, fil);
//              int cindex = 0;
//              cindex = p96Search(this.plates96, col);
//
              if (b1.equals("1"))
              {
                this.filter.setVisible(false);
                this.collection.setVisible(false);
                this.spelabel1.setVisible(false);
                this.spelabel2.setVisible(false);
                this.assembutton.setSelected(true);
              }
              if (b1.equals("2"))
              {
                this.filter.setVisible(false);
                this.collection.setVisible(false);
                this.spelabel1.setVisible(false);
                this.spelabel2.setVisible(false);
                this.disassembutton.setSelected(true);
              }
              if (b1.equals("3"))
              {
                this.vacbutton.setSelected(true);
                this.filter.setVisible(false);
                this.collection.setVisible(false);
                this.spelabel1.setVisible(false);
                this.spelabel2.setVisible(false);
                this.time.setVisible(true);
                this.pressure.setVisible(true);
                this.pressure.setText(pre);
                this.time.setText(ti);
                this.spelabel3.setVisible(true);
                this.spelabel4.setVisible(true);
                this.spePanel.revalidate();
                this.spePanel.repaint();
              }

//              if (findex1 > 0)
//              {
//                this.filter.setSelectedIndex(findex1);
//              }
//
//              if (col.equals("None"))
//              {
//                this.collection.setSelectedIndex(0);
//              }
//              if (cindex >= 0)
//              {
//                this.collection.setSelectedIndex(cindex + 1);
//              }

            }

            int tindex = tabSearch(this.tabs, "SPE");
            if (tindex > -1)
            {
              this.jTabbedPane1.setSelectedIndex(tindex);
            }

          }

          if (steptype.equals("Pause")) {
            this.updButton9.setEnabled(true);
            this.checkButton.setEnabled(true);
            this.addButton9.setEnabled(false);
            this.rmvButton.setEnabled(true);
            this.arrowButton1.setEnabled(false);
            this.arrowButton2.setEnabled(false);
            for (int t = 0; t < this.tabcount; t++) {
              this.jTabbedPane1.setEnabledAt(t, false);
            }
            if (counter2 == 3) {
                String line3 = currentFList.data.toString();
                index1 = line3.indexOf(',');
                len = line3.length();
                if (len > 1)
                {
                index1 = line3.indexOf(',');
                index2 = line3.indexOf(',', index1 + 1);
                if (line3.substring(0, index1).equals("1"))
                {
               	 this.pausedataPanel2.setTimed(true);
                }
                else
                {
               	 this.pausedataPanel2.setTimed(false);
                }
                
                	this.pausedataPanel2.setMinutes(line3.substring(index1 + 1, index2));
                	this.pausedataPanel2.setSeconds(line3.substring(index2 + 1, len));
                }
                else
                {
               	 this.pausedataPanel2.setTimed(false);
       			 this.pausedataPanel2.setMinutes("");
       			 this.pausedataPanel2.setSeconds("");
               	 
                }
              }

            if (counter2 == 4) {
               String line4 = currentFList.data.toString();
               len = line4.length();
               this.pausedataPanel2.setMessage(line4.trim());
              }

            int tindex = tabSearch(this.tabs, "Pause");
            if (tindex > -1)
            {
              this.jTabbedPane1.setSelectedIndex(tindex);
            }

          }

          if (steptype.equals("move plate")) {
            this.updButton10.setEnabled(true);
            this.checkButton.setEnabled(true);
            this.addButton.setEnabled(false);
            this.addButton10.setEnabled(false);

            this.rmvButton.setEnabled(true);
            this.arrowButton1.setEnabled(false);
            this.arrowButton2.setEnabled(false);
            for (int t = 0; t < this.tabcount; t++) {
              this.jTabbedPane1.setEnabledAt(t, false);
            }
            if (counter2 == 4) {
              String line4 = currentFList.data.toString();
              index1 = line4.indexOf(',');
              len = line4.length();
              this.spl3u = line4.substring(0, index1);
              this.dpl3u = line4.substring(index1 + 1, len);
              pindex1 = pSearch(this.platetypes, this.spl3u);
              pindex2 = pSearch(this.platetypes, this.dpl3u);
              if (pindex1 >= 0)
              {
                this.splatetype3.setSelectedIndex(pindex1);
                this.splatetype3.revalidate();
                this.splatetype3.repaint();
              }
              if (pindex2 >= 0)
              {
                this.dplatetype3.setSelectedIndex(pindex2);
                this.dplatetype3.revalidate();
                this.dplatetype3.repaint();
              }

            }

            if (counter2 == 5) {
              String line5 = currentFList.data.toString();
              len = line5.length();
              String ressrc = line5.substring(0, len);
              sindex = p96Search(this.plates96, ressrc);
              if (sindex == -1)
              {
                sindex = p384Search(this.plates384, ressrc);
                sindex += this.plates96.length;
              }
              this.source5.setSelectedIndex(sindex);
            }

            int tindex = tabSearch(this.tabs, "Move Plate");
            if (tindex > -1)
            {
              this.jTabbedPane1.setSelectedIndex(tindex);
            }

          }

          if (steptype.equals("Cherry Picking")) {
            this.updButton11.setEnabled(true);
            this.checkButton.setEnabled(true);
            this.addButton.setEnabled(false);
            this.addButton11.setEnabled(false);
            this.rmvButton.setEnabled(true);
            this.arrowButton1.setEnabled(false);
            this.arrowButton2.setEnabled(false);
            for (int t = 0; t < this.tabcount; t++) {
              this.jTabbedPane1.setEnabledAt(t, false);
            }
            if (counter2 == 3) {
              String line3 = currentFList.data.toString();
              index1 = line3.indexOf(',');
              this.text7.setText(line3.substring(0, index1));
              len = line3.length();
              this.chltyp = line3.substring(index1 + 1, len);
              lindex = lSearch(this.liqtypes, this.chltyp);
              this.chliq.setSelectedIndex(lindex);
            }

            if (counter2 == 4) {
              String line4 = currentFList.data.toString();
              index1 = line4.indexOf(',');
              len = line4.length();
              this.tr1 = line4.substring(0, index1);
              this.tr2 = line4.substring(index1 + 1, len);
              this.track1.setSelectedIndex(Integer.parseInt(this.tr1) - 1);
              this.track2.setSelectedIndex(Integer.parseInt(this.tr2) - 1);
            }

            if (counter2 == 5) {
              String line5 = currentFList.data.toString();
              len = line5.length();
              String chso = line5.substring(0, len);
              
              sindex = pAllSearch(this.platesAll, chso);		//AP-0001
              this.chsource.setSelectedIndex(sindex);			//AP-0001
              
//              sindex = p96Search(this.plates96, chso);	//AP-0001 Commented out and ready for deletion
//              if (sindex == -1)
//              {
//                sindex = p384Search(this.plates384, chso);
//                sindex += this.plates96.length;
//              }
//              this.chsource.setSelectedIndex(sindex);
              
            }

            if (counter2 == 6) {
              String line6 = currentFList.data.toString();
              len = line6.length();
              String chde = line6.substring(0, len);
              
              sindex = pAllSearch(this.platesAll, chde);	//AP-0001
              this.chdest.setSelectedIndex(sindex);			//AP-0001
              
//              sindex = p96Search(this.plates96, chde);	//AP-0001 Commented out and ready for deletion
//              if (sindex == -1)
//              {
//                sindex = p384Search(this.plates384, chde);
//                sindex += this.plates96.length;
//              }
//              this.chdest.setSelectedIndex(sindex);
              
            }

            int tindex = tabSearch(this.tabs, "Cherry Picking");
            if (tindex > -1)
            {
              this.jTabbedPane1.setSelectedIndex(tindex);
            }

          }

          if (steptype.equals("CherryPicking02")) {
              this.cherry02_btnUpdateStep.setEnabled(true);
              this.checkButton.setEnabled(true);
              this.addButton.setEnabled(false);
              this.cherry02_btnAddStep.setEnabled(false);
              this.rmvButton.setEnabled(true);
              this.arrowButton1.setEnabled(false);
              this.arrowButton2.setEnabled(false);
              for (int t = 0; t < this.tabcount; t++) {
                this.jTabbedPane1.setEnabledAt(t, false);
              }

			  if (counter2 == 3) {
                String line3 = currentFList.data.toString();
                this.cherry02dataPanel2.setText2(line3);
			  }

              if (counter2 == 4) {
                String line4 = currentFList.data.toString();
                ch2_plateloc = line4;
              }

              if (counter2 == 5) {
                String line5 = currentFList.data.toString();
                len = line5.length();
                String ch2_splate = line5.substring(0, len);
                sindex = pAllSearch(this.platesAll, ch2_splate);		//AP-0001
                if (sindex == -1)
                {
                 cherry02dataPanel2.setSLabIndex(1);
                 sindex = tAllSearch(this.tubesAll, ch2_splate);	//AP-0001
                }
                else
                {
                 cherry02dataPanel2.setSLabIndex(2);	
                }
                this.cherry02dataPanel2.setSPlateIndex(sindex);			//AP-0001
                
              }

              if (counter2 == 6) {
                String line6 = currentFList.data.toString();
                len = line6.length();
                String ch2_dplate = line6.substring(0, len);
                dindex = pAllSearch(this.platesAll, ch2_dplate);	//AP-0001
                if (dindex == -1)
                {
                 cherry02dataPanel2.setDLabIndex(1);
                 dindex = tAllSearch(this.tubesAll, ch2_dplate);	//AP-0001
                }
                else
                {
                 cherry02dataPanel2.setDLabIndex(2);	
                }
                this.cherry02dataPanel2.setDPlateIndex(dindex);			//AP-0001
                
              }

              if (counter2 == 7) {
                  String line7 = currentFList.data.toString();
                  this.ch2_liqtype = line7;
                  lindex = lSearch(this.liqtypes, this.ch2_liqtype);
                  this.cherry02dataPanel2.setLiqIndex(lindex);
                }

              if (counter2 == 8) {
                  String line8 = currentFList.data.toString();
                  len = line8.length();
                  index1 = line8.indexOf(',');
                  this.ch2_scycle = line8.substring(0, index1);
                  this.ch2_smix = line8.substring(index1 + 1, len);
                  TabWindow.this.cherry02dataPanel2.setSCycle1(Integer.parseInt(ch2_scycle));
//                  TabWindow.this.cherry02dataPanel2.setSCycleIndex(Integer.parseInt(ch2_scycle));
                  TabWindow.this.cherry02dataPanel2.setText3(ch2_smix);
                }

              if (counter2 == 9) {
                  String line9 = currentFList.data.toString();
                  len = line9.length();
                  index1 = line9.indexOf(',');
                  this.ch2_dcycle = line9.substring(0, index1);
                  this.ch2_dmix = line9.substring(index1 + 1, len);
                  TabWindow.this.cherry02dataPanel2.setDCycle1(Integer.parseInt(ch2_dcycle));
                  TabWindow.this.cherry02dataPanel2.setText4(ch2_dmix);
                }

              if (counter2 == 10) {
                  String line10 = currentFList.data.toString();
                  ch2_tubeloc = line10;
                  if (ch2_plateloc.equals("null") && ch2_tubeloc.equals("null"))
                  {
                  	this.ch2_plateloc = "";
                  	this.ch2_tubeloc = "";
                  	this.cherry02dataPanel2.setT1Enabled(false);
                  	this.cherry02dataPanel2.setT1TEnabled(false);
                  	this.cherry02dataPanel2.setBarIndex(0);
                  }
                  else if (ch2_plateloc.equals("null"))
                  {
                   	this.ch2_plateloc = "";
                  	this.cherry02dataPanel2.setT1Enabled(false);
                   	this.cherry02dataPanel2.setT1TEnabled(true);
                  	this.cherry02dataPanel2.setBarIndex(1);
                  }
                  else if (ch2_tubeloc.equals("null"))
                  {
                	this.ch2_tubeloc = "";
                   	this.cherry02dataPanel2.setT1TEnabled(false);
                   	this.cherry02dataPanel2.setT1Enabled(true);
                  	this.cherry02dataPanel2.setBarIndex(1);
                  }
                  else 
                  {
                   	this.cherry02dataPanel2.setT1TEnabled(true);
                   	this.cherry02dataPanel2.setT1Enabled(true);
                  	this.cherry02dataPanel2.setBarIndex(1);
                  }
 					this.cherry02dataPanel2.setText1(ch2_plateloc);
  					this.cherry02dataPanel2.setText1T(ch2_tubeloc);
                }

              int tindex = tabSearch(this.tabs, "CherryPick02");
              if (tindex > -1)
              {
                this.jTabbedPane1.setSelectedIndex(tindex);
              }

            }

          
          if (steptype.equals("PK")) {
              this.btnUpdateStep.setEnabled(true);
              this.checkButton.setEnabled(true);
              this.addButton.setEnabled(false);
              this.btnAdd.setEnabled(false);
              this.rmvButton.setEnabled(true);
              this.arrowButton1.setEnabled(false);
              this.arrowButton2.setEnabled(false);
              for (int t = 0; t < this.tabcount; t++) {
                this.jTabbedPane1.setEnabledAt(t, false);
              }
              if (counter2 == 3) {
                String line3 = currentFList.data.toString();
                len = line3.length();
                index1 = line3.indexOf(',');
                index2 = line3.indexOf(',', index1 + 1);
                index3 = line3.indexOf(',', index2 + 1);
                index4 = line3.indexOf(',', index3 + 1);
                index5 = line3.indexOf(',', index4 + 1);
//                index6 = line3.indexOf(',', index5 + 1);
//                index7 = line3.indexOf(',', index6 + 1);
//                index8 = line3.indexOf(',', index7 + 1);
                text = line3.substring(0, index1);
				this.pkm_text_wat.setText(line3.substring(0, index1));
				this.pkm_text_1.setText(line3.substring(index1 + 1, index2));
				this.pkm_text_ul.setText(line3.substring(index2 + 1, index3));
				this.pkm_text_ul_preAssay.setText(line3.substring(index3 + 1, index4));
				this.pkm_text_ul_Assay.setText(line3.substring(index4 + 1, len));
				

              }

              if (counter == 4) {
            	  String line4 = currentFList.data.toString();
                  len = line4.length();
                  index1 = line4.indexOf(',');
                  index2 = line4.indexOf(',', index1 + 1);
                  index3 = line4.indexOf(',', index2 + 1);
                  index4 = line4.indexOf(',', index3 + 1);
//                  index5 = line4.indexOf(',', index4 + 1);
//                  index6 = line4.indexOf(',', index5 + 1);
//                  index7 = line4.indexOf(',', index6 + 1);
//                  index8 = line4.indexOf(',', index7 + 1);

  				this.pksam = line4.substring(0, index1);
  				this.pkstd = line4.substring(index1 + 1, index2);
  				this.pkmrddil = line4.substring(index2 + 1, index3);
  				this.pkdildil = line4.substring(index3 + 1, len);
  				
                lindex = lSearch(this.liqtypes, this.pksam);
                if (lindex  <  0)
                {
                	lindex = 0;
                }
                this.pksamdil.setSelectedIndex(lindex);
                lindex = lSearch(this.liqtypes, this.pkstd);
                if (lindex  <  0)
                {
                	lindex = 0;
                }
                this.pkstddil.setSelectedIndex(lindex);
                lindex = lSearch(this.liqtypes, this.pkmrddil);
                if (lindex  <  0)
                {
                	lindex = 0;
                }
                this.pkMRDdil.setSelectedIndex(lindex);
                lindex = lSearch(this.liqtypes, this.pkdildil);
                if (lindex  <  0)
                {
                	lindex = 0;
                }
                this.pkdil.setSelectedIndex(lindex);
              }
              
              if (counter2 == 5) {
                String line5 = currentFList.data.toString();
                len = line5.length();
                index1 = line5.indexOf(',');
                index2 = line5.indexOf(',', index1 + 1);
                index3 = line5.indexOf(',', index2 + 1);
                index4 = line5.indexOf(',', index3 + 1);
                index5 = line5.indexOf(',', index4 + 1);
                this.pksamlab = line5.substring(0, index1);
                this.pkstdlab = line5.substring(index1 + 1, index2);
                this.pkmrdlab = line5.substring(index2 + 1, index3);
                this.pkdillab = line5.substring(index3 + 1, index4);
                this.pkpasslab = line5.substring(index4 + 1, index5);
                this.pkasslab = line5.substring(index5 + 1, len);
                sindex = p96Search(this.plates96, pkmrdlab);
                if (sindex == -1)
                {
                  sindex = 0 ;
                }
                this.pkMRDsource.setSelectedIndex(sindex);
                
                sindex = p96Search(this.plates96, this.pksamlab);
                if (sindex == -1)
                {
                  sindex = tSearch(this.tubes, pksamlab);
                  if (sindex == -1)
                  {
                    sindex = 0 ;
                  }
                  else
                  {
                  sindex += this.plates96.length;
                  }
                }
                
                this.pkSsource.setSelectedIndex(sindex);

                sindex = p96Search(this.plates96, pkdillab);
                if (sindex == -1)
                {
                  sindex = 0 ;
                }
                this.pkDsource.setSelectedIndex(sindex);
                
                sindex = p96Search(this.plates96, pkpasslab);
                if (sindex == -1)
                {
                  sindex = 0 ;
                }
                this.pkPassay.setSelectedIndex(sindex);
                
                sindex = p96Search(this.plates96, pkstdlab);
                if (sindex == -1)
                {
                  sindex = tSearch(this.tubes, pkstdlab);
                  if (sindex == -1)
                  {
                    sindex = 0 ;
                  }
                  else
                  {
                  sindex += this.plates96.length;
                  }
                }
                
                this.pkSTDsource.setSelectedIndex(sindex);
                
                sindex = p96Search(this.plates96, pkasslab);
                if (sindex == -1)
                {
                  sindex = 0 ;
                }
                this.pkassay.setSelectedIndex(sindex);
              }

              if (counter2 == 6) {
                String line6 = currentFList.data.toString();
                len = line6.length();
                index1 = line6.indexOf(',');
                index2 = line6.indexOf(',', index1 + 1);
                index3 = line6.indexOf(',', index2 + 1);
                index4 = line6.indexOf(',', index3 + 1);
                
                this.pkck1 = line6.substring(0, index1);
                this.pkck2 = line6.substring(index1 + 1, index2);
                this.pkck3 = line6.substring(index2 + 1, index3);
                this.pkck4 = line6.substring(index3 + 1, len);
                if (pkck1.equals("1"))
                {
                  pkcheck1.setSelected(true);
    	    	  TabWindow.this.pkm_text_1.setEnabled(true);
    	    	  TabWindow.this.pkm_text_ul.setEnabled(true);
    	    	  TabWindow.this.pksamdil.setEnabled(true);
    	    	  TabWindow.this.pkstddil.setEnabled(true);
    	    	  TabWindow.this.pkMRDdil.setEnabled(true);
    	    	  TabWindow.this.pkSTDsource.setEnabled(true);
    	    	  TabWindow.this.pkSsource.setEnabled(true);
    	    	  TabWindow.this.pkMRDsource.setEnabled(true);
    	    	  TabWindow.this.pkSsource.setEnabled(true);
                }
                else
                {
                  pkcheck1.setSelected(false);
                  if (!pkck2.equals("1"))
                  {
        	    	  TabWindow.this.pkMRDsource.setEnabled(false);
        	    	  TabWindow.this.pkdil.setEnabled(false);
                  }
    	    	  TabWindow.this.pkSsource.setEnabled(false);
    	    	  TabWindow.this.pkm_text_1.setEnabled(false);
    	    	  TabWindow.this.pkm_text_ul.setEnabled(false);
                }

                if (pkck2.equals("1"))
                {
                  pkcheck2.setSelected(true);
    	    	  TabWindow.this.pkDsource.setEnabled(true);
    	    	  TabWindow.this.pkdil.setEnabled(true);
    	    	  TabWindow.this.pkMRDsource.setEnabled(true);
    	    	  TabWindow.this.pkMRDdil.setEnabled(true);
                }
                else
                {
                  pkcheck2.setSelected(false);
    	    	  TabWindow.this.pkdil.setEnabled(false);
    	    	  TabWindow.this.pkDsource.setEnabled(false);
                  if (!pkck1.equals("1"))
                  {
        	    	  TabWindow.this.pkMRDsource.setEnabled(false);
        	    	  TabWindow.this.pkMRDdil.setEnabled(false);
                  }
                }
                if (pkck3.equals("1"))
                {
                  pkcheck3.setSelected(true);
                  pkcheck3.setEnabled(true);
    	    	  TabWindow.this.pkPassay.setEnabled(true);
    	    	  TabWindow.this.pkm_text_ul_preAssay.setEnabled(true);
                }
                else
                {
                  pkcheck3.setSelected(false);
                  if (!pkcheck2.isSelected())
                  {
                   pkcheck3.setEnabled(true);
                  }

                }

                if (pkck4.equals("1"))
                {
                  pkcheck4.setSelected(true);
                  pkcheck4.setEnabled(true);
    	    	  TabWindow.this.pkassay.setEnabled(true);
    	    	  TabWindow.this.pkm_text_ul_Assay.setEnabled(true);
                }
                else
                {
                  pkcheck4.setSelected(false);
                  if (!pkcheck3.isSelected())
                  {
        	    	  TabWindow.this.pkPassay.setEnabled(false);
        	    	  TabWindow.this.pkm_text_ul_preAssay.setEnabled(false);
        	    	  pkcheck4.setEnabled(false);
                  }
    	    	  TabWindow.this.pkassay.setEnabled(false);
    	    	  TabWindow.this.pkm_text_ul_Assay.setEnabled(false);
                }

              }

              int tindex = tabSearch(this.tabs, "PK");
              if (tindex > -1)
              {
                this.jTabbedPane1.setSelectedIndex(tindex);
              }
            }

          if (steptype.equals("move")) {
            this.updButton8.setEnabled(true);
            this.checkButton.setEnabled(true);
            this.addButton8.setEnabled(false);
            this.rmvButton.setEnabled(true);
            this.arrowButton1.setEnabled(false);
            this.arrowButton2.setEnabled(false);
            for (int t = 0; t < this.tabcount; t++) {
              this.jTabbedPane1.setEnabledAt(t, false);
            }

            int tindex = tabSearch(this.tabs, "Move");
            if (tindex > -1)
            {
              this.jTabbedPane1.setSelectedIndex(tindex);
            }

          }

          if (steptype.equals("Cytomat01")) {
              this.cyt_btnUpdateStep.setEnabled(true);
              this.checkButton.setEnabled(true);
              this.addButton.setEnabled(false);
              this.cyt_btnAddStep.setEnabled(false);
              this.rmvButton.setEnabled(true);
              this.arrowButton1.setEnabled(false);
              this.arrowButton2.setEnabled(false);
              for (int t = 0; t < this.tabcount; t++) {
                this.jTabbedPane1.setEnabledAt(t, false);
              }
              if (counter2 == 3) {
                String line3 = currentFList.data.toString();
                cytfromto = line3.trim();
                TabWindow.this.cytodatapanel2.setFTIndex(Integer.parseInt(cytfromto));
              }

              if (counter2 == 4) {
                  String line4 = currentFList.data.toString();
                  cytloc = line4.trim();
                  cytodatapanel2.setText1(cytloc);
                }

              if (counter2 == 5) {
                  String line5 = currentFList.data.toString();
                  cytdeckloc = line5.trim();
                  cytodatapanel2.setText2(cytdeckloc);
                                  }

              if (counter2 == 6) {
                  String line6 = currentFList.data.toString();
                  cytcover = line6.trim();
                  TabWindow.this.cytodatapanel2.setCoverIndex(Integer.parseInt(cytcover));
                }

              if (counter2 == 7) {
                  String line7 = currentFList.data.toString();
                  cyt_plate = line7.trim();
				  int pindex = pAllSearch(this.platesAll, this.cyt_plate);	
//                  int pindex = p96Search(this.plates96, this.cyt_plate);
//                  if (pindex == -1)
//                  {
//                    pindex = p384Search(this.plates384, this.cyt_plate);
//                    pindex += this.plates96.length;
//                  }
                  cytodatapanel2.setPlateIndex(pindex);
                }

              if (counter2 == 8) {
                  String line8 = currentFList.data.toString();
                  cyt_tsp = line8.trim();
                  int pindex = pSearch(this.platetypes, this.cyt_tsp);
                  cytodatapanel2.setTSPIndex(pindex);
                  this.cytodatapanel2.revalidate();
                  this.cytodatapanel2.repaint();
                }


              int tindex = tabSearch(this.tabs, "Cytomat01");
              if (tindex > -1)
              {
                this.jTabbedPane1.setSelectedIndex(tindex);
              }

            }

          if (steptype.equals("Liconic01")) {
              this.lic_btnUpdateStep.setEnabled(true);
              this.checkButton.setEnabled(true);
              this.addButton.setEnabled(false);
              this.lic_btnAddStep.setEnabled(false);
              this.rmvButton.setEnabled(true);
              this.arrowButton1.setEnabled(false);
              this.arrowButton2.setEnabled(false);
              for (int t = 0; t < this.tabcount; t++) {
                this.jTabbedPane1.setEnabledAt(t, false);
              }
 
              if (counter2 == 3) {
                  String line3 = currentFList.data.toString();
                  licfromto = line3.trim();
                  TabWindow.this.licodatapanel2.setFTIndex(Integer.parseInt(licfromto));
                }

              if (counter2 == 4) {
                  String line4 = currentFList.data.toString();
                  licloc = line4.trim();
                  licodatapanel2.setText1(licloc);
                }

              if (counter2 == 5) {
                  String line5 = currentFList.data.toString();
                  licdeckloc = line5.trim();
                  licodatapanel2.setText2(licdeckloc);
                }

              if (counter2 == 6) {
                  String line6 = currentFList.data.toString();
                  liccover = line6.trim();
                  TabWindow.this.licodatapanel2.setCoverIndex(Integer.parseInt(liccover));
                }

              if (counter2 == 7) {
                  String line7 = currentFList.data.toString();
                  lic_plate = line7.trim();
                  int pindex = pAllSearch(this.platesAll, this.lic_plate);	
                  TabWindow.this.licodatapanel2.setPlateIndex(pindex);
                }

              if (counter2 == 8) {
                  String line8 = currentFList.data.toString();
                  lic_tsp = line8.trim();
                  int pindex = pSearch(this.platetypes, this.lic_tsp);
                  licodatapanel2.setTSPIndex(pindex);
                  this.licodatapanel2.revalidate();
                  this.licodatapanel2.repaint();
                }

              int tindex = tabSearch(this.tabs, "Liconic01");
              if (tindex > -1)
              {
                this.jTabbedPane1.setSelectedIndex(tindex);
              }

            }

          if (steptype.equals("MovePlate02")) {
              this.movplv2_btnUpdateStep.setEnabled(true);
              this.checkButton.setEnabled(true);
              this.addButton.setEnabled(false);
              this.movplv2_btnAddStep.setEnabled(false);
              this.rmvButton.setEnabled(true);
              this.arrowButton1.setEnabled(false);
              this.arrowButton2.setEnabled(false);
              for (int t = 0; t < this.tabcount; t++) {
                this.jTabbedPane1.setEnabledAt(t, false);
              }
              if (counter2 == 3) {
                String line3 = currentFList.data.toString();
                movetype = line3.trim();
                TabWindow.this.movplv2datapanel2.setTypeIndex(Integer.parseInt(movetype));
              }

              if (counter2 == 4) {
                  String line4 = currentFList.data.toString();
                  startloc = line4.trim();
                  movplv2datapanel2.setText1(startloc);
                }

              if (counter2 == 5) {
                  String line5 = currentFList.data.toString();
                  endloc = line5.trim();
                  movplv2datapanel2.setText2(endloc);
                                  }

              if (counter2 == 6) {
                  String line6 = currentFList.data.toString();
                  mvp_plate = line6.trim();
				  int pindex = pAllSearch(this.platesAll, this.mvp_plate);	
//                  int pindex = p96Search(this.plates96, this.mvp_plate);
//                  if (pindex == -1)
//                  {
//                    pindex = p384Search(this.plates384, this.mvp_plate);
//                    pindex += this.plates96.length;
//                  }
                  movplv2datapanel2.setPlateIndex(pindex);
                  this.movplv2datapanel2.revalidate();
                  this.movplv2datapanel2.repaint();
                }


              int tindex = tabSearch(this.tabs, "MovePlate02");
              if (tindex > -1)
              {
                this.jTabbedPane1.setSelectedIndex(tindex);
              }

            }
          
          if (steptype.equals("CORDNA01")) {
              this.dna_btnUpdateStep.setEnabled(true);
              this.checkButton.setEnabled(true);
              this.addButton.setEnabled(false);
              this.dna_btnAddStep.setEnabled(false);
              this.rmvButton.setEnabled(true);
              this.arrowButton1.setEnabled(false);
              this.arrowButton2.setEnabled(false);
              for (int t = 0; t < this.tabcount; t++) {
                this.jTabbedPane1.setEnabledAt(t, false);
              }
              if (counter2 == 3) {
                String line3 = currentFList.data.toString();
                dna_qfile = line3.trim();
                TabWindow.this.dnadataPanel2.setText(dna_qfile);
        }

              if (counter2 == 4) {
                  String line4 = currentFList.data.toString();
                  proc = line4.trim();
                  TabWindow.this.dnadataPanel2.setProcIndex(Integer.parseInt(proc));
                }

              if (counter2 == 5) {
                  String line5 = currentFList.data.toString();
                  index1 = line5.indexOf(',');
                  len = line5.length();
                  this.dna_con = line5.substring(0, index1);
                  this.dna_vol = line5.substring(index1 + 1, len);
                  TabWindow.this.dnadataPanel2.setText1(dna_con);
                  TabWindow.this.dnadataPanel2.setText2(dna_vol);
                }

              if (counter2 == 6) {
                  String line6 = currentFList.data.toString();
                  index1 = line6.indexOf(',');
                  index2 = line6.indexOf(',', index1 + 1);
                  index3 = line6.indexOf(',', index2 + 1);
                  len = line6.length();
                  this.dna_dplate = line6.substring(0, index1);
                  this.dna_splate = line6.substring(index1 + 1, index2);
                  this.dna_nplate = line6.substring(index2 + 1, index3);
                  this.dna_plate = line6.substring(index3 + 1, len);
                  dindex = rAllSearch(this.reservoirsAll, dna_dplate);	//AP-0001
                  sindex = tAllSearch(this.tubesAll, dna_splate);	//AP-0001
                  int nindex = pPATSearch(this.platesTubes, dna_nplate);	//AP-0001
                  int pindex = pPATSearch(this.platesTubes, dna_plate);	//AP-0001
                  TabWindow.this.dnadataPanel2.setDPlateIndex(dindex);
                  TabWindow.this.dnadataPanel2.setSPlateIndex(sindex);
                  TabWindow.this.dnadataPanel2.setNPlateIndex(nindex);
                  TabWindow.this.dnadataPanel2.setPlateIndex(pindex);
                }

              if (counter2 == 7) {
                  String line7 = currentFList.data.toString();
                  index1 = line7.indexOf(',');
                  index2 = line7.indexOf(',', index1 + 1);
                  index3 = line7.indexOf(',', index2 + 1);
                  len = line7.length();
                  this.dna_deck1 = line7.substring(0, index1);
                  this.dna_deck2 = line7.substring(index1 + 1, index2);
                  this.dna_deck3 = line7.substring(index2 + 1, index3);
                  this.dna_deck4 = line7.substring(index3 + 1, len);
                  TabWindow.this.dnadataPanel2.setDeckIndex1(Integer.parseInt(dna_deck1));
                  TabWindow.this.dnadataPanel2.setDeckIndex2(Integer.parseInt(dna_deck2));
                  TabWindow.this.dnadataPanel2.setDeckIndex3(Integer.parseInt(dna_deck3));
                  TabWindow.this.dnadataPanel2.setDeckIndex4(Integer.parseInt(dna_deck4));
              }

              int tindex = tabSearch(this.tabs, "CORDNA01");
              if (tindex > -1)
              {
                this.jTabbedPane1.setSelectedIndex(tindex);
              }

            }

          if (steptype.equals("LOAD01")) {
              this.load_btnUpdateStep.setEnabled(true);
              this.checkButton.setEnabled(true);
              this.addButton.setEnabled(false);
              this.load_btnAddStep.setEnabled(false);
              this.rmvButton.setEnabled(true);
              this.arrowButton1.setEnabled(false);
              this.arrowButton2.setEnabled(false);
              for (int t = 0; t < this.tabcount; t++) {
                this.jTabbedPane1.setEnabledAt(t, false);
              }
              if (counter2 == 3) {
                  String line3 = currentFList.data.toString();
                  index1 = line3.indexOf(',');
                  len = line3.length();
                  this.load_plindex = line3.substring(0, index1);
                  int num = Integer.parseInt(load_plindex);
                  this.load_plate = line3.substring(index1 + 1, len);
                  TabWindow.this.loaddataPanel2.setPlateIndex(num);
                  TabWindow.this.loaddataPanel2.setPlateText(load_plate);
        }

              if (counter2 == 4) {
                  String line4 = currentFList.data.toString();
                  index1 = line4.indexOf(',');
                  len = line4.length();
                  this.load_tuindex = line4.substring(0, index1);
                  int num = Integer.parseInt(load_tuindex);
                  this.load_tube = line4.substring(index1 + 1, len);
                  TabWindow.this.loaddataPanel2.setTubeIndex(num);
                  TabWindow.this.loaddataPanel2.setTubeText(load_tube);
                }

              if (counter2 == 5) {
                  String line5 = currentFList.data.toString();
                  index1 = line5.indexOf(',');
                  len = line5.length();
                  this.load_tiindex = line5.substring(0, index1);
                  int num = Integer.parseInt(load_tiindex);
                  this.load_tip = line5.substring(index1 + 1, len);
                  TabWindow.this.loaddataPanel2.setTipIndex(num);
                  TabWindow.this.loaddataPanel2.setTipText(load_tip);
                }

              int tindex = tabSearch(this.tabs, "LOAD01");
              if (tindex > -1)
              {
                this.jTabbedPane1.setSelectedIndex(tindex);
              }

            }

          if (currentFList == null)
            continue;
          currentFList = currentFList.next;
        }

        while (!this.bList.isEmpty())
        {
          this.bList.removeFromBack();
        }
        while (!this.eList.isEmpty())
        {
          this.eList.removeFromBack();
        }
        while (!this.dbList.isEmpty())
        {
          this.dbList.removeFromBack();
        }
        while (!this.deList.isEmpty())
        {
          this.deList.removeFromBack();
        }

        currentFList = this.fList.getFirstNode();

        count = 0;

        while (currentFList != null)
        {
          count++;
          if (count <= (val - 1) * 10) {
            this.bList.insertAtBack(currentFList.data.toString());
          }
          currentFList = currentFList.next;
        }

        currentFList = this.fList.getFirstNode();

        count = 0;
        while (currentFList != null)
        {
          count++;
          if (count > val * 10) {
            this.eList.insertAtBack(currentFList.data.toString());
          }
          currentFList = currentFList.next;
        }

        currentDList = this.dList.getFirstNode();

        count = 0;

        while (currentDList != null)
        {
          count++;
          if (count < val) {
            this.dbList.insertAtBack(currentDList.data.toString());
          }
          if (count > val) {
            this.deList.insertAtBack(currentDList.data.toString());
          }
          currentDList = currentDList.next;
        }

        currentTList = this.tList.getFirstNode();

        count = 0;

        while (currentTList != null)
        {
          count++;
          if (count < val) {
            this.tbList.insertAtBack(currentTList.data.toString());
          }
          if (count > val) {
            this.teList.insertAtBack(currentTList.data.toString());
          }
          currentTList = currentTList.next;
        }
      }
    }
  }

  private void insertStep(int v)
  {
    ListNode currentFList = this.fList.getFirstNode();
    ListNode currentDList = this.dList.getFirstNode();
    ListNode currentTList = this.tList.getFirstNode();
    ListNode currentBList = this.bList.getFirstNode();
    ListNode currentEList = this.eList.getFirstNode();
    ListNode currentDBList = this.dbList.getFirstNode();
    ListNode currentDEList = this.deList.getFirstNode();
    ListNode currentTBList = this.tbList.getFirstNode();
    ListNode currentTEList = this.teList.getFirstNode();
    int val = 0;
    String data = "";
    String type = "";
    nonPK ++;
    
    while (!this.bList.isEmpty())
    {
      this.bList.removeFromBack();
    }
    while (!this.eList.isEmpty())
    {
      this.eList.removeFromBack();
    }
    while (!this.dbList.isEmpty())
    {
      this.dbList.removeFromBack();
    }
    while (!this.deList.isEmpty())
    {
      this.deList.removeFromBack();
    }
    while (!this.tbList.isEmpty())
    {
      this.tbList.removeFromBack();
    }
    while (!this.teList.isEmpty())
    {
      this.teList.removeFromBack();
    }

    int count = 0;
    val = v;

    if (val > 0)
    {
      count = 0;

      while (currentFList != null)
      {
        count++;
        if (count <= val * 10) {
          this.bList.insertAtBack(currentFList.data.toString());
        }
        if (count > val * 10) {
          this.eList.insertAtBack(currentFList.data.toString());
        }
        currentFList = currentFList.next;
      }

      currentDList = this.dList.getFirstNode();

      count = 0;

      while (currentDList != null)
      {
        count++;
        if (count <= val) {
          this.dbList.insertAtBack(currentDList.data.toString());
        }
        if (count > val) {
          this.deList.insertAtBack(currentDList.data.toString());
        }
        currentDList = currentDList.next;
      }

      currentTList = this.tList.getFirstNode();

      count = 0;

      while (currentTList != null)
      {
        count++;
        if (count <= val) {
          this.tbList.insertAtBack(currentTList.data.toString());
        }
        if (count > val) {
          this.teList.insertAtBack(currentTList.data.toString());
        }
        currentTList = currentTList.next;
      }

    }

    while (!this.fList.isEmpty())
    {
      this.fList.removeFromBack();
    }
    while (!this.dList.isEmpty())
    {
      this.dList.removeFromBack();
    }
    while (!this.tList.isEmpty())
    {
      this.tList.removeFromBack();
    }

    currentBList = this.bList.getFirstNode();
    while (currentBList != null)
    {
      this.fList.insertAtBack(currentBList.data.toString());
      currentBList = currentBList.next;
    }

    while (!this.bList.isEmpty())
    {
      this.bList.removeFromBack();
    }

    int tabindex = this.jTabbedPane1.getSelectedIndex();
    String tabtext = this.jTabbedPane1.getTitleAt(tabindex);
    if (tabtext.trim().equals("Liquid Transfer")) {
      this.counter = 0;
      data = "";

      String c1 = "0";
      String c2 = "0";
      String sh1 = "0";
      String sh2 = "0";
      String tpwa = "0";
      String v1 = "0";
      String v2 = "0";
      String line3 = "";
      String line4 = "";
      String q1 = "0";
      String q2 = "0";
      String q3 = "0";
      String q4 = "0";
      String q5 = "0";
      String q6 = "0";
      String q7 = "0";
      String q8 = "0";
      String i1 = "0";
      String i2 = "0";
      String i3 = "0";
      String i4 = "0";
      String tube1 = "";
      String tube2 = "";
      String tube3 = "";
      String tube4 = "";
      String resnum = "";
      String asp1 = "";
      String mp1 = "";
      String asp2 = "";
      String mp2 = "";

      type = "REG";
      data = data + "   \n";
      data = data + "\n " + this.text2.getText().trim();
      data = data + "\n " + this.text3.getText().trim() + " ul" + "\n";
      if (this.spl1.equals("Deck Position")) this.spl1 = "T";
      data = data + " Deck " + this.spl1;
      data = data + " to " + this.dpl1 + "\n";

      line3 = line3 + this.text2.getText().trim();
      line3 = line3 + "," + this.lt1.trim() + ",-,-,-";

      asp1 = this.advwin.getMode();
      mp1 = this.advwin.getMPA();
      data = data + " Multi: " + mp1;

      asp2 = this.advwind.getMode();
      mp2 = this.advwind.getMPA();
      data = data + "-" + mp2;

      if (this.tubepanel.incIsSelected())
      {
        i1 = "1";
      }
      if (this.tubepanel2.incIsSelected())
      {
        i2 = "1";
      }

//      if (this.linc1button.isSelected())
//      {
//        i3 = "1";
//      }
//      if (this.linc2button.isSelected())
//      {
//        i3 = "2";
//      }
//      if (this.linc3button.isSelected())
//     {
//        i3 = "3";
//      }
//      if (this.linc4button.isSelected())
//      {
//        i4 = "1";
//      }
//      if (this.linc5button.isSelected())
//      {
//        i4 = "2";
//      }
//      if (this.linc6button.isSelected())
//      {
//        i4 = "3";
//      }
      if (this.advwin.getChill() > 0)
      {
        c1 = "1";
      }
      if (this.advwin.getTPWA() > 0)
      {
        tpwa = "1";
      }
      if (this.advwin.getShake() > 0)
      {
        sh1 = "1";
      }
      if (this.advwin.getVStart() > 0)
      {
        data = data + " Recycle Tips\n";
        v1 = "1";
      }

      if (this.advwind.getChill() > 0)
      {
        c2 = "1";
      }
      if (this.advwind.getShake() > 0)
      {
        sh2 = "1";
      }
      if (this.advwind.getVStart() > 0)
      {
        data = data + " Recycle Tips\n";
        v2 = "1";
      }

      String sourceseq = " ";
      String destseq = "0";
      tube1 = this.tubepanel.getFirstTube();
      tube2 = this.tubepanel.getLastTube();
      tube3 = this.tubepanel2.getFirstTube();
      tube4 = this.tubepanel2.getLastTube();
      resnum = this.res24Xpanel.getReservoir();

      if ((this.s1.trim().endsWith("Tube")) && (this.de.trim().endsWith("Tube")))
      {
        sourceseq = tube1 + "," + tube2 + "," + i1;
        destseq = tube3 + "," + tube4 + "," + i2;
        this.spl1 = "Tube";
        this.dpl1 = "Tube";
      }

      if ((this.s1.trim().endsWith("_1X")) && (this.de.trim().endsWith("Tube")))
      {
        sourceseq = "1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,52,53,54,55,56,57,58,59,60,61,62,63,64,65,66,67,68,69,70,71,72,73,74,75,76,77,78,79,80,81,82,83,84,85,86,87,88,89,90,91,92,93,94,95,96";

        destseq = tube3 + "," + tube4 + "," + i2;

        this.dpl1 = "Tube";
      }
      if ((this.s1.trim().endsWith("_2X")) && (this.de.trim().endsWith("Tube")))
      {
        sourceseq = "1,2,3,4,5,6,7,8";
        destseq = tube3 + "," + tube4 + "," + i2;

        this.dpl1 = "Tube";
      }
      if ((this.s1.trim().endsWith("_4X")) && (this.de.trim().endsWith("Tube")))
      {
        sourceseq = "1,2,3,4,5,6,7,8";
        destseq = tube3 + "," + tube4 + "," + i2;

        this.dpl1 = "Tube";
      }
      if ((this.s1.trim().endsWith("_5X")) && (this.de.trim().endsWith("Tube")))
      {
        sourceseq = "1,2,3,4,5,6,7,8";
        destseq = tube3 + "," + tube4 + "," + i2;

        this.dpl1 = "Tube";
      }
      if ((this.s1.trim().equals("Reservoir_24X")) && (this.de.trim().endsWith("Tube")))
      {
        sourceseq = "1,2,3,4,5,6,7,8";
        destseq = tube3 + "," + tube4 + "," + i2;

        this.dpl1 = "Tube";
      }
      if ((this.s1.trim().endsWith("_1X")) && (this.de.trim().endsWith("_96")))
      {
        this.panel96r.loadTray();
        sourceseq = "1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,52,53,54,55,56,57,58,59,60,61,62,63,64,65,66,67,68,69,70,71,72,73,74,75,76,77,78,79,80,81,82,83,84,85,86,87,88,89,90,91,92,93,94,95,96";

        destseq = this.panel96r.getTray().trim();
      }

      if ((this.s1.trim().endsWith("_2X")) && (this.de.trim().endsWith("_96")))
      {
        this.panel96r.loadTray();
        sourceseq = "1,2,3,4,5,6,7,8";
        destseq = this.panel96r.getTray().trim();
      }

      if ((this.s1.trim().endsWith("_4X")) && (this.de.trim().endsWith("_96")))
      {
        this.panel96r.loadTray();
        sourceseq = "1,2,3,4,5,6,7,8";
        destseq = this.panel96r.getTray().trim();
      }

      if ((this.s1.trim().endsWith("_5X")) && (this.de.trim().endsWith("_96")))
      {
        this.panel96r.loadTray();
        sourceseq = "1,2,3,4,5,6,7,8";
        destseq = this.panel96r.getTray().trim();
      }

      if ((this.s1.trim().equals("Reservoir_24X")) && (this.de.trim().endsWith("_96")) && (this.resbutton.isSelected()))
      {
        this.panel96r.loadTray();
        sourceseq = "1,2,3,4,5,6,7,8";
        destseq = this.panel96r.getTray().trim();
      }

      if ((this.s1.trim().endsWith("_96")) && (this.de.trim().endsWith("_96")))
      {
        this.panel96l.loadTray();
        this.panel96r.loadTray();
        sourceseq = this.panel96l.getTray().trim();
        destseq = this.panel96r.getTray().trim();
      }

      if ((this.s1.trim().endsWith("Tube")) && (this.de.trim().endsWith("_96")))
      {
        this.panel96r.loadTray();
        sourceseq = tube1 + "," + tube2 + "," + i1;
        destseq = this.panel96r.getTray().trim();
        this.spl1 = "Tube";
      }
      if ((this.s1.trim().endsWith("Set_24X")) && (this.de.trim().endsWith("_96")))
      {
        this.panel96r.loadTray();
        sourceseq = resnum;
        destseq = this.panel96r.getTray().trim();
        this.spl1 = "Set";
      }

      if ((this.s1.trim().endsWith("_96")) && (this.de.trim().equals("Set_24X")))
      {
        this.panel96l.loadTray();
        sourceseq = this.panel96l.getTray().trim();
        destseq = resnum;
        this.dpl1 = "Set";
      }

      if ((this.s1.trim().endsWith("Tube")) && (this.de.trim().endsWith("_384")))
      {
        q5 = this.panel384r.getQ1();
        q6 = this.panel384r.getQ2();
        q7 = this.panel384r.getQ3();
        q8 = this.panel384r.getQ4();
        this.panel384r.loadTray();
        sourceseq = tube1 + "," + tube2 + "," + i1;
        destseq = this.panel384r.getTray().trim();
        this.spl1 = "Tube";
      }

      if ((this.s1.trim().endsWith("_96")) && (this.de.trim().endsWith("Tube")))
      {
        this.panel96l.loadTray();
        sourceseq = this.panel96l.getTray().trim();
        destseq = tube3 + "," + tube4 + "," + i2;
        this.dpl1 = "Tube";
      }

      if ((this.s1.trim().endsWith("_1X")) && (this.de.trim().endsWith("_384")))
      {
        q5 = this.panel384r.getQ1();
        q6 = this.panel384r.getQ2();
        q7 = this.panel384r.getQ3();
        q8 = this.panel384r.getQ4();
        this.panel384r.loadTray();
        sourceseq = "1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,52,53,54,55,56,57,58,59,60,61,62,63,64,65,66,67,68,69,70,71,72,73,74,75,76,77,78,79,80,81,82,83,84,85,86,87,88,89,90,91,92,93,94,95,96";

        destseq = this.panel384r.getTray().trim();
      }

      if ((this.s1.trim().endsWith("_2X")) && (this.de.trim().endsWith("_384")))
      {
        q5 = this.panel384r.getQ1();
        q6 = this.panel384r.getQ2();
        q7 = this.panel384r.getQ3();
        q8 = this.panel384r.getQ4();
        this.panel384r.loadTray();
        sourceseq = "1,2,3,4,5,6,7,8";
        destseq = this.panel384r.getTray().trim();
      }

      if ((this.s1.trim().endsWith("_4X")) && (this.de.trim().endsWith("_384")))
      {
        q5 = this.panel384r.getQ1();
        q6 = this.panel384r.getQ2();
        q7 = this.panel384r.getQ3();
        q8 = this.panel384r.getQ4();
        this.panel384r.loadTray();
        sourceseq = "1,2,3,4,5,6,7,8";
        destseq = this.panel384r.getTray().trim();
      }

      if ((this.s1.trim().endsWith("_5X")) && (this.de.trim().endsWith("_384")))
      {
        q5 = this.panel384r.getQ1();
        q6 = this.panel384r.getQ2();
        q7 = this.panel384r.getQ3();
        q8 = this.panel384r.getQ4();
        this.panel384r.loadTray();
        sourceseq = "1,2,3,4,5,6,7,8";
        destseq = this.panel384r.getTray().trim();
      }

      if ((this.s1.trim().equals("Reservoir_24X")) && (this.de.trim().endsWith("_384")) && (this.resbutton.isSelected()))
      {
        q5 = this.panel384r.getQ1();
        q6 = this.panel384r.getQ2();
        q7 = this.panel384r.getQ3();
        q8 = this.panel384r.getQ4();
        this.panel384r.loadTray();
        sourceseq = "1,2,3,4,5,6,7,8";
        destseq = this.panel384r.getTray().trim();
      }

      if ((this.s1.trim().equals("Set_24X")) && (this.de.trim().endsWith("_384")))
      {
        q5 = this.panel384r.getQ1();
        q6 = this.panel384r.getQ2();
        q7 = this.panel384r.getQ3();
        q8 = this.panel384r.getQ4();
        sourceseq = resnum;
        this.panel384r.loadTray();
        destseq = this.panel384r.getTray().trim();
        this.spl1 = "Set";
      }

      if ((this.s1.trim().endsWith("_96")) && (this.de.trim().endsWith("_384")))
      {
        this.panel96l.loadTray();

        q5 = this.panel384r.getQ1();
        q6 = this.panel384r.getQ2();
        q7 = this.panel384r.getQ3();
        q8 = this.panel384r.getQ4();
        this.panel384r.loadTray();
        sourceseq = this.panel96l.getTray().trim();
        destseq = this.panel384r.getTray().trim();
      }
      if ((this.s1.trim().endsWith("_384")) && (this.de.trim().endsWith("_96")))
      {
        q1 = this.panel384l.getQ1();
        q2 = this.panel384l.getQ2();
        q3 = this.panel384l.getQ3();
        q4 = this.panel384l.getQ4();
        this.panel384l.loadTray();
        this.panel96r.loadTray();
        sourceseq = this.panel384l.getTray().trim();
        destseq = this.panel96r.getTray().trim();
      }
      if ((this.s1.trim().endsWith("_384")) && (this.de.trim().endsWith("_384")))
      {
        q1 = this.panel384l.getQ1();
        q2 = this.panel384l.getQ2();
        q3 = this.panel384l.getQ3();
        q4 = this.panel384l.getQ4();
        this.panel384l.loadTray();

        q5 = this.panel384r.getQ1();
        q6 = this.panel384r.getQ2();
        q7 = this.panel384r.getQ3();
        q8 = this.panel384r.getQ4();
        this.panel384r.loadTray();
        sourceseq = this.panel384l.getTray().trim();
        destseq = this.panel384r.getTray().trim();
      }

      if ((this.s1.trim().endsWith("_1X")) && (this.de.trim().equals("Set_24X")))
      {
        sourceseq = "1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,52,53,54,55,56,57,58,59,60,61,62,63,64,65,66,67,68,69,70,71,72,73,74,75,76,77,78,79,80,81,82,83,84,85,86,87,88,89,90,91,92,93,94,95,96";

        destseq = resnum;

        this.spl2 = "Set";
      }

      if ((this.s1.trim().endsWith("_4X")) && (this.de.trim().equals("Set_24X")))
      {
        sourceseq = "1,2,3,4,5,6,7,8";
        destseq = resnum;

        this.spl2 = "Set";
      }
      if ((this.s1.trim().endsWith("_5X")) && (this.de.trim().equals("Set_24X")))
      {
        sourceseq = "1,2,3,4,5,6,7,8";
        destseq = resnum;

        this.spl2 = "Set";
      }
      if ((this.s1.trim().equals("Reservoir_24X")) && (this.de.trim().equals("Set_24X")))
      {
        sourceseq = "1,2,3,4,5,6,7,8";
        destseq = resnum;

        this.dpl1 = "Set";
      }

      line4 = line4 + this.spl1.trim() + "," + this.dpl1.trim();

      if (this.s1.trim().equals("Set_24X"))
      {
        this.s1 = "Reservoir_24X";
      }
      if (this.de.trim().equals("Set_24X"))
      {
        this.de = "Reservoir_24X";
      }

      if (sourceseq.endsWith(","))
        sourceseq = sourceseq.substring(0, sourceseq.length() - 1);
      if (destseq.endsWith(",")) {
        destseq = destseq.substring(0, destseq.length() - 1);
      }
      this.fList.insertAtBack("step");
      this.fList.insertAtBack("add");
      this.fList.insertAtBack(line3);
      this.fList.insertAtBack(line4);
      this.fList.insertAtBack(this.s1.trim() + "," + this.mix1 + "," + asp1 + "," + mp1 + "," + sh1 + "," + c1 + "," + tpwa + "," + v1 + "," + this.rs1);
      this.fList.insertAtBack(this.de.trim() + "," + this.mix2 + "," + asp2 + "," + mp2 + "," + sh2 + "," + c2 + "," + i4 + "," + v2 + "," + this.rs2);
      this.fList.insertAtBack(sourceseq);
      this.fList.insertAtBack(destseq);
      this.fList.insertAtBack(this.text3.getText().trim());
      this.fList.insertAtBack(q1 + "," + q2 + "," + q3 + "," + q4 + "," + q5 + "," + q6 + "," + q7 + "," + q8);
    }

    if (tabtext.trim().equals("Move Plate"))
    {
      data = "";

      data = data + "   \n ";
      data = data + this.so5 + "\n" + "Deck " + this.spl3 + " to " + this.dpl3 + "\n\n";

      type = "MOVPL";

      this.fList.insertAtBack("step");
      this.fList.insertAtBack("move plate");
      this.fList.insertAtBack("-");
      this.fList.insertAtBack(this.spl3 + "," + this.dpl3);
      this.fList.insertAtBack(this.so5);
      this.fList.insertAtBack("-");
      this.fList.insertAtBack("-");
      this.fList.insertAtBack("-");
      this.fList.insertAtBack("-");
      this.fList.insertAtBack("-");
    }

    currentEList = this.eList.getFirstNode();
    while (currentEList != null)
    {
      this.fList.insertAtBack(currentEList.data.toString());
      currentEList = currentEList.next;
    }
    while (!this.eList.isEmpty())
    {
      this.eList.removeFromBack();
    }

    currentDBList = this.dbList.getFirstNode();
    while (currentDBList != null)
    {
      this.dList.insertAtBack(currentDBList.data.toString());
      currentDBList = currentDBList.next;
    }
    while (!this.dbList.isEmpty())
    {
      this.dbList.removeFromBack();
    }

    this.dList.insertAtBack(data);
    data = "";

    currentDEList = this.deList.getFirstNode();
    while (currentDEList != null)
    {
      this.dList.insertAtBack(currentDEList.data.toString());
      currentDEList = currentDEList.next;
    }
    while (!this.deList.isEmpty())
    {
      this.deList.removeFromBack();
    }

    currentTBList = this.tbList.getFirstNode();
    while (currentTBList != null)
    {
      this.tList.insertAtBack(currentTBList.data.toString());
      currentTBList = currentTBList.next;
    }
    while (!this.tbList.isEmpty())
    {
      this.tbList.removeFromBack();
    }

    this.tList.insertAtBack(type);
    type = "";

    currentTEList = this.teList.getFirstNode();
    while (currentTEList != null)
    {
      this.tList.insertAtBack(currentTEList.data.toString());
      currentTEList = currentTEList.next;
    }
    while (!this.teList.isEmpty())
    {
      this.teList.removeFromBack();
    }

    this.dTextArea[0].setText("");
    this.dTextArea[1].setText("");
    this.dTextArea[2].setText("");
    this.dTextArea[3].setText("");
    this.dTextArea[4].setText("");
    this.dTextArea[5].setText("");
    this.dTextArea[6].setText("");
    this.dTextArea[7].setText("");
    this.dTextArea[8].setText("");
    this.dTextArea[9].setText("");
    this.dTextArea[10].setText("");
    this.dTextArea[11].setText("");
    this.dTextArea[12].setText("");
    this.dTextArea[13].setText("");
    this.dTextArea[14].setText("");

    this.dTextArea[0].setVisible(false);
    this.dTextArea[1].setVisible(false);
    this.dTextArea[2].setVisible(false);
    this.dTextArea[3].setVisible(false);
    this.dTextArea[4].setVisible(false);
    this.dTextArea[5].setVisible(false);
    this.dTextArea[6].setVisible(false);
    this.dTextArea[7].setVisible(false);
    this.dTextArea[8].setVisible(false);
    this.dTextArea[9].setVisible(false);
    this.dTextArea[10].setVisible(false);
    this.dTextArea[11].setVisible(false);
    this.dTextArea[12].setVisible(false);
    this.dTextArea[13].setVisible(false);
    this.dTextArea[14].setVisible(false);
    loadtext();
    if (this.updButton1 != null)
    {
      this.updButton1.setEnabled(false);
      this.checkButton.setEnabled(false);
    }

    if (this.updButton2 != null)
    {
      this.updButton2.setEnabled(false);
    }

    if (this.updButton3 != null)
    {
      this.updButton3.setEnabled(false);
    }

    if (this.updButton4 != null)
    {
      this.updButton4.setEnabled(false);
    }

    if (this.updButton5 != null)
    {
      this.updButton5.setEnabled(false);
    }

    if (this.updButton6 != null)
    {
      this.updButton6.setEnabled(false);
    }

    if (this.updButton7 != null)
    {
      this.updButton7.setEnabled(false);
    }

    if (this.updButton8 != null)
    {
      this.updButton8.setEnabled(false);
    }

    if (this.updButton9 != null)
    {
      this.updButton9.setEnabled(false);
    }

    if (this.updButton10 != null)
    {
      this.updButton10.setEnabled(false);
    }

    if (this.addButton1 != null)
    {
      this.addButton1.setEnabled(true);
      this.addButton.setEnabled(true);
    }

    if (this.addButton2 != null)
    {
      this.addButton2.setEnabled(true);
    }

    if (this.addButton3 != null)
    {
      this.addButton3.setEnabled(true);
    }

    if (this.addButton4 != null)
    {
      this.addButton4.setEnabled(true);
    }

    if (this.addButton5 != null)
    {
      this.addButton5.setEnabled(true);
    }

    if (this.addButton6 != null)
    {
      this.addButton6.setEnabled(true);
    }

    if (this.addButton7 != null)
    {
      this.addButton7.setEnabled(true);
    }

    if (this.addButton8 != null)
    {
      this.addButton8.setEnabled(true);
    }

    if (this.addButton9 != null)
    {
      this.addButton9.setEnabled(true);
    }

    if (this.addButton10 != null)
    {
      this.addButton10.setEnabled(true);
    }

    this.rmvButton.setEnabled(false);
    this.saveButton.setEnabled(true);
    this.stepnum.setText("");
    this.update = false;
    this.addButton1.setEnabled(true);
    this.addButton.setEnabled(true);
    this.checkButton.setEnabled(false);
    for (int t = 0; t < this.tabcount; t++)
        this.jTabbedPane1.setEnabledAt(t, true);
    pkEnableDisable();
      resetScreens();
      //Return to Splash screen
      Splash();

  }

  private void cutStep(int v)
  {
    ListNode currentFList = this.fList.getFirstNode();
    ListNode currentDList = this.dList.getFirstNode();
    ListNode currentTList = this.tList.getFirstNode();
    ListNode currentBList = this.bList.getFirstNode();
    ListNode currentEList = this.eList.getFirstNode();
    ListNode currentDBList = this.dbList.getFirstNode();
    ListNode currentDEList = this.deList.getFirstNode();
    ListNode currentTBList = this.tbList.getFirstNode();
    ListNode currentTEList = this.teList.getFirstNode();

    ListNode currentCFList = this.cfList.getFirstNode();
    ListNode currentCDList = this.cdList.getFirstNode();
    ListNode currentCTList = this.ctList.getFirstNode();

    while (!this.cfList.isEmpty())
    {
      this.cfList.removeFromBack();
    }
    while (!this.cdList.isEmpty())
    {
      this.cdList.removeFromBack();
    }
    while (!this.ctList.isEmpty())
    {
      this.ctList.removeFromBack();
    }

    int val = 0;

    while (!this.bList.isEmpty())
    {
      this.bList.removeFromBack();
    }
    while (!this.eList.isEmpty())
    {
      this.eList.removeFromBack();
    }
    while (!this.dbList.isEmpty())
    {
      this.dbList.removeFromBack();
    }
    while (!this.deList.isEmpty())
    {
      this.deList.removeFromBack();
    }
    while (!this.tbList.isEmpty())
    {
      this.tbList.removeFromBack();
    }
    while (!this.teList.isEmpty())
    {
      this.teList.removeFromBack();
    }

    int count = 0;
    val = v;

    if (val > 0) {
      if (val == 1)
      {
        count = 0;
        while (currentFList != null)
        {
          count++;
          if (count > val * 10) {
            this.eList.insertAtBack(currentFList.data.toString());
          }
          if ((count >= 1) && (count <= 10)) {
            this.cfList.insertAtBack(currentFList.data.toString());
          }
          currentFList = currentFList.next;
        }

        currentDList = this.dList.getFirstNode();

        count = 0;
        while (currentDList != null)
        {
          count++;
          if (count > val) {
            this.deList.insertAtBack(currentDList.data.toString());
          }
          if (count == val) {
            this.cdList.insertAtBack(currentDList.data.toString());
          }
          currentDList = currentDList.next;
        }
        count = 0;
        while (currentTList != null)
        {
          count++;
          if (count > val) {
            this.teList.insertAtBack(currentTList.data.toString());
          }
          if (count == val) {
            this.ctList.insertAtBack(currentTList.data.toString());
          }
          currentTList = currentTList.next;
        }

      }

      if (val > 1)
      {
        count = 0;

        while (currentFList != null)
        {
          count++;
          if ((count > (val - 1) * 10) && (count <= val * 10)) {
            this.cfList.insertAtBack(currentFList.data.toString());
          }
          if (count <= (val - 1) * 10) {
            this.bList.insertAtBack(currentFList.data.toString());
          }
          if (count > val * 10) {
            this.eList.insertAtBack(currentFList.data.toString());
          }
          currentFList = currentFList.next;
        }

        currentDList = this.dList.getFirstNode();

        count = 0;

        while (currentDList != null)
        {
          count++;
          if (count == val) {
            this.cdList.insertAtBack(currentDList.data.toString());
          }
          if (count < val) {
            this.dbList.insertAtBack(currentDList.data.toString());
          }
          if (count > val) {
            this.deList.insertAtBack(currentDList.data.toString());
          }
          currentDList = currentDList.next;
        }

        currentTList = this.tList.getFirstNode();

        count = 0;

        while (currentTList != null)
        {
          count++;
          if (count == val) {
        	// insert here nonPK
              String tl = "";
              tl = currentTList.data.toString().trim();

                if (tl.equals("REG")) {
            	    nonPK = nonPK - 1;
                }
                if (tl.equals("INC")) {
            	    nonPK = nonPK - 1;
                }
                if (tl.equals("SER")) {
            	    nonPK = nonPK - 1;
                }
                if (tl.equals("BRW")) {
            	    nonPK = nonPK - 1;
                }
                if (tl.equals("WSH")) {
            	  nonPK = nonPK - 1;
                }
                if (tl.equals("RDR")) {
            	    nonPK = nonPK - 1;
                }
                if (tl.equals("SPE")) {
              	  nonPK = nonPK - 1;
                }
                if (tl.equals("MOV")) {
              	  nonPK = nonPK - 1;
                }
                if (tl.equals("MOVPL")) {
              	  nonPK = nonPK - 1;
                }
                if (tl.equals("PAU")) {
              	  nonPK = nonPK - 1;
                }
                if (tl.equals("CHR")) {
              	  nonPK = nonPK - 1;
                }
                if (tl.equals("PKS")) {
              	  cntPK = cntPK - 1;
                }
            pagecount = (int) ((Math.ceil(nonPK/15.0)) - 1);		//APF0028
            
            this.ctList.insertAtBack(currentTList.data.toString());
          }
          if (count < val) {
            this.tbList.insertAtBack(currentTList.data.toString());
          }
          if (count > val) {
            this.teList.insertAtBack(currentTList.data.toString());
          }
          currentTList = currentTList.next;
        }

      }

    }

    while (!this.fList.isEmpty())
    {
      this.fList.removeFromBack();
    }
    while (!this.dList.isEmpty())
    {
      this.dList.removeFromBack();
    }
    while (!this.tList.isEmpty())
    {
      this.tList.removeFromBack();
    }

    currentBList = this.bList.getFirstNode();
    while (currentBList != null)
    {
      this.fList.insertAtBack(currentBList.data.toString());
      currentBList = currentBList.next;
    }

    while (!this.bList.isEmpty())
    {
      this.bList.removeFromBack();
    }

    currentEList = this.eList.getFirstNode();
    while (currentEList != null)
    {
      this.fList.insertAtBack(currentEList.data.toString());
      currentEList = currentEList.next;
    }
    while (!this.eList.isEmpty())
    {
      this.eList.removeFromBack();
    }

    currentDBList = this.dbList.getFirstNode();
    while (currentDBList != null)
    {
      this.dList.insertAtBack(currentDBList.data.toString());
      currentDBList = currentDBList.next;
    }
    while (!this.dbList.isEmpty())
    {
      this.dbList.removeFromBack();
    }

    currentDEList = this.deList.getFirstNode();
    while (currentDEList != null)
    {
      this.dList.insertAtBack(currentDEList.data.toString());
      currentDEList = currentDEList.next;
    }
    while (!this.deList.isEmpty())
    {
      this.deList.removeFromBack();
    }

    currentTBList = this.tbList.getFirstNode();
    while (currentTBList != null)
    {
      this.tList.insertAtBack(currentTBList.data.toString());
      currentTBList = currentTBList.next;
    }
    while (!this.tbList.isEmpty())
    {
      this.tbList.removeFromBack();
    }

    currentTEList = this.teList.getFirstNode();
    while (currentTEList != null)
    {
      this.tList.insertAtBack(currentTEList.data.toString());
      currentTEList = currentTEList.next;
    }
    while (!this.teList.isEmpty())
    {
      this.teList.removeFromBack();
    }

    this.dTextArea[0].setText("");
    this.dTextArea[1].setText("");
    this.dTextArea[2].setText("");
    this.dTextArea[3].setText("");
    this.dTextArea[4].setText("");
    this.dTextArea[5].setText("");
    this.dTextArea[6].setText("");
    this.dTextArea[7].setText("");
    this.dTextArea[8].setText("");
    this.dTextArea[9].setText("");
    this.dTextArea[10].setText("");
    this.dTextArea[11].setText("");
    this.dTextArea[12].setText("");
    this.dTextArea[13].setText("");
    this.dTextArea[14].setText("");

    this.dTextArea[0].setVisible(false);
    this.dTextArea[1].setVisible(false);
    this.dTextArea[2].setVisible(false);
    this.dTextArea[3].setVisible(false);
    this.dTextArea[4].setVisible(false);
    this.dTextArea[5].setVisible(false);
    this.dTextArea[6].setVisible(false);
    this.dTextArea[7].setVisible(false);
    this.dTextArea[8].setVisible(false);
    this.dTextArea[9].setVisible(false);
    this.dTextArea[10].setVisible(false);
    this.dTextArea[11].setVisible(false);
    this.dTextArea[12].setVisible(false);
    this.dTextArea[13].setVisible(false);
    this.dTextArea[14].setVisible(false);
    loadtext();
    if (this.updButton1 != null)
    {
      this.updButton1.setEnabled(false);
      this.checkButton.setEnabled(false);
    }

    if (this.updButton2 != null)
    {
      this.updButton2.setEnabled(false);
    }

    if (this.updButton3 != null)
    {
      this.updButton3.setEnabled(false);
    }

    if (this.updButton4 != null)
    {
      this.updButton4.setEnabled(false);
    }

    if (this.updButton5 != null)
    {
      this.updButton5.setEnabled(false);
    }

    if (this.updButton6 != null)
    {
      this.updButton6.setEnabled(false);
    }

    if (this.updButton7 != null)
    {
      this.updButton7.setEnabled(false);
    }

    if (this.updButton8 != null)
    {
      this.updButton8.setEnabled(false);
    }

    if (this.updButton9 != null)
    {
      this.updButton9.setEnabled(false);
    }

    if (this.updButton10 != null)
    {
      this.updButton10.setEnabled(false);
    }

    if (this.addButton1 != null)
    {
      this.addButton1.setEnabled(true);
      this.addButton.setEnabled(true);
    }

    if (this.addButton2 != null)
    {
      this.addButton2.setEnabled(true);
    }

    if (this.addButton3 != null)
    {
      this.addButton3.setEnabled(true);
    }

    if (this.addButton4 != null)
    {
      this.addButton4.setEnabled(true);
    }

    if (this.addButton5 != null)
    {
      this.addButton5.setEnabled(true);
    }

    if (this.addButton6 != null)
    {
      this.addButton6.setEnabled(true);
    }

    if (this.addButton7 != null)
    {
      this.addButton7.setEnabled(true);
    }

    if (this.addButton8 != null)
    {
      this.addButton8.setEnabled(true);
    }

    if (this.addButton9 != null)
    {
      this.addButton9.setEnabled(true);
    }

    if (this.addButton10 != null)
    {
      this.addButton10.setEnabled(true);
    }

    this.rmvButton.setEnabled(false);
    this.saveButton.setEnabled(true);
    this.stepnum.setText("");
    this.update = false;
    this.addButton1.setEnabled(true);
    this.addButton.setEnabled(true);
    this.checkButton.setEnabled(false);
    for (int t = 0; t < this.tabcount; t++)
      this.jTabbedPane1.setEnabledAt(t, true);
    pkEnableDisable();
    int tindex = tabSearch(this.tabs, "Liquid Transfer");
    if (tindex > -1)
    {
      this.jTabbedPane1.setSelectedIndex(tindex);
    }
    resetScreens();
  }

  private void copyStep(int v)
  {
    int val = 0;
    String data = "";
    String type = "";

    ListNode currentFList = this.fList.getFirstNode();
    ListNode currentDList = this.dList.getFirstNode();
    ListNode currentTList = this.tList.getFirstNode();

    ListNode currentCFList = this.cfList.getFirstNode();
    ListNode currentCDList = this.cdList.getFirstNode();
    ListNode currentCTList = this.ctList.getFirstNode();

    while (!this.cfList.isEmpty())
    {
      this.cfList.removeFromBack();
    }
    while (!this.cdList.isEmpty())
    {
      this.cdList.removeFromBack();
    }
    while (!this.ctList.isEmpty())
    {
      this.ctList.removeFromBack();
    }

    int count = 0;
    val = v;

    if (val > 0)
    {
      count = 0;

      while (currentFList != null)
      {
        count++;
        if (val == 1)
        {
          if ((count >= 1) && (count <= 10)) {
            this.cfList.insertAtBack(currentFList.data.toString());
          }
        }
        if (val > 1)
        {
          if ((count > (val - 1) * 10) && (count <= val * 10)) {
            this.cfList.insertAtBack(currentFList.data.toString());
          }
        }

        currentFList = currentFList.next;
      }

      currentDList = this.dList.getFirstNode();

      count = 0;

      while (currentDList != null)
      {
        count++;
        if (count == val) {
          this.cdList.insertAtBack(currentDList.data.toString());
        }
        currentDList = currentDList.next;
      }

      currentTList = this.tList.getFirstNode();

      count = 0;

      while (currentTList != null)
      {
        count++;
        if (count == val) {
          this.ctList.insertAtBack(currentTList.data.toString());
        }
        currentTList = currentTList.next;
      }
    }
  }

  private void pasteStep(int v)
  {
    ListNode currentFList = this.fList.getFirstNode();
    ListNode currentDList = this.dList.getFirstNode();
    ListNode currentTList = this.tList.getFirstNode();
    ListNode currentBList = this.bList.getFirstNode();
    ListNode currentEList = this.eList.getFirstNode();
    ListNode currentDBList = this.dbList.getFirstNode();
    ListNode currentDEList = this.deList.getFirstNode();
    ListNode currentTBList = this.tbList.getFirstNode();
    ListNode currentTEList = this.teList.getFirstNode();

    ListNode currentCFList = this.cfList.getFirstNode();
    ListNode currentCDList = this.cdList.getFirstNode();
    ListNode currentCTList = this.ctList.getFirstNode();

    int val = 0;
    String data = "";
    String type = "";
    nonPK ++;	
    pagecount = (int) ((Math.ceil(nonPK/15.0)) - 1);		//APF0028
    
    while (!this.bList.isEmpty())
    {
      this.bList.removeFromBack();
    }
    while (!this.eList.isEmpty())
    {
      this.eList.removeFromBack();
    }
    while (!this.dbList.isEmpty())
    {
      this.dbList.removeFromBack();
    }
    while (!this.deList.isEmpty())
    {
      this.deList.removeFromBack();
    }
    while (!this.tbList.isEmpty())
    {
      this.tbList.removeFromBack();
    }
    while (!this.teList.isEmpty())
    {
      this.teList.removeFromBack();
    }

    int count = 0;
    val = v;

    if (val > 0)
    {
      count = 0;

      while (currentFList != null)
      {
        count++;
        if (count <= val * 10) {
          this.bList.insertAtBack(currentFList.data.toString());
        }
        if (count > val * 10) {
          this.eList.insertAtBack(currentFList.data.toString());
        }
        currentFList = currentFList.next;
      }

      currentDList = this.dList.getFirstNode();

      count = 0;

      while (currentDList != null)
      {
        count++;
        if (count <= val) {
          this.dbList.insertAtBack(currentDList.data.toString());
        }
        if (count > val) {
          this.deList.insertAtBack(currentDList.data.toString());
        }
        currentDList = currentDList.next;
      }

      currentTList = this.tList.getFirstNode();

      count = 0;

      while (currentTList != null)
      {
        count++;
        if (count <= val) {
          this.tbList.insertAtBack(currentTList.data.toString());
        }
        if (count > val) {
          this.teList.insertAtBack(currentTList.data.toString());
        }
        currentTList = currentTList.next;
      }

    }

    while (!this.fList.isEmpty())
    {
      this.fList.removeFromBack();
    }
    while (!this.dList.isEmpty())
    {
      this.dList.removeFromBack();
    }
    while (!this.tList.isEmpty())
    {
      this.tList.removeFromBack();
    }

    currentBList = this.bList.getFirstNode();
    while (currentBList != null)
    {
      this.fList.insertAtBack(currentBList.data.toString());
      currentBList = currentBList.next;
    }

    while (!this.bList.isEmpty())
    {
      this.bList.removeFromBack();
    }

    currentCFList = this.cfList.getFirstNode();
    while (currentCFList != null)
    {
      this.fList.insertAtBack(currentCFList.data.toString());
      currentCFList = currentCFList.next;
    }

    while (!this.cfList.isEmpty())
    {
      this.cfList.removeFromBack();
    }

    currentEList = this.eList.getFirstNode();
    while (currentEList != null)
    {
      this.fList.insertAtBack(currentEList.data.toString());
      currentEList = currentEList.next;
    }
    while (!this.eList.isEmpty())
    {
      this.eList.removeFromBack();
    }

    currentDBList = this.dbList.getFirstNode();
    while (currentDBList != null)
    {
      this.dList.insertAtBack(currentDBList.data.toString());
      currentDBList = currentDBList.next;
    }
    while (!this.dbList.isEmpty())
    {
      this.dbList.removeFromBack();
    }

    currentCDList = this.cdList.getFirstNode();
    while (currentCDList != null)
    {
      this.dList.insertAtBack(currentCDList.data.toString());
      currentCDList = currentCDList.next;
    }
    while (!this.cdList.isEmpty())
    {
      this.cdList.removeFromBack();
    }

    currentDEList = this.deList.getFirstNode();
    while (currentDEList != null)
    {
      this.dList.insertAtBack(currentDEList.data.toString());
      currentDEList = currentDEList.next;
    }
    while (!this.deList.isEmpty())
    {
      this.deList.removeFromBack();
    }

    currentTBList = this.tbList.getFirstNode();
    while (currentTBList != null)
    {
      this.tList.insertAtBack(currentTBList.data.toString());
      currentTBList = currentTBList.next;
    }
    while (!this.tbList.isEmpty())
    {
      this.tbList.removeFromBack();
    }

    currentCTList = this.ctList.getFirstNode();
    while (currentCTList != null)
    {
      this.tList.insertAtBack(currentCTList.data.toString());
      currentCTList = currentCTList.next;
    }
    while (!this.ctList.isEmpty())
    {
      this.ctList.removeFromBack();
    }

    currentTEList = this.teList.getFirstNode();
    while (currentTEList != null)
    {
      this.tList.insertAtBack(currentTEList.data.toString());
      currentTEList = currentTEList.next;
    }
    while (!this.teList.isEmpty())
    {
      this.teList.removeFromBack();
    }

    this.dTextArea[0].setText("");
    this.dTextArea[1].setText("");
    this.dTextArea[2].setText("");
    this.dTextArea[3].setText("");
    this.dTextArea[4].setText("");
    this.dTextArea[5].setText("");
    this.dTextArea[6].setText("");
    this.dTextArea[7].setText("");
    this.dTextArea[8].setText("");
    this.dTextArea[9].setText("");
    this.dTextArea[10].setText("");
    this.dTextArea[11].setText("");
    this.dTextArea[12].setText("");
    this.dTextArea[13].setText("");
    this.dTextArea[14].setText("");

    this.dTextArea[0].setVisible(false);
    this.dTextArea[1].setVisible(false);
    this.dTextArea[2].setVisible(false);
    this.dTextArea[3].setVisible(false);
    this.dTextArea[4].setVisible(false);
    this.dTextArea[5].setVisible(false);
    this.dTextArea[6].setVisible(false);
    this.dTextArea[7].setVisible(false);
    this.dTextArea[8].setVisible(false);
    this.dTextArea[9].setVisible(false);
    this.dTextArea[10].setVisible(false);
    this.dTextArea[11].setVisible(false);
    this.dTextArea[12].setVisible(false);
    this.dTextArea[13].setVisible(false);
    this.dTextArea[14].setVisible(false);
    loadtext();
    if (this.updButton1 != null)
    {
      this.updButton1.setEnabled(false);
      this.checkButton.setEnabled(false);
    }

    if (this.updButton2 != null)
    {
      this.updButton2.setEnabled(false);
    }

    if (this.updButton3 != null)
    {
      this.updButton3.setEnabled(false);
    }

    if (this.updButton4 != null)
    {
      this.updButton4.setEnabled(false);
    }

    if (this.updButton5 != null)
    {
      this.updButton5.setEnabled(false);
    }

    if (this.updButton6 != null)
    {
      this.updButton6.setEnabled(false);
    }

    if (this.updButton7 != null)
    {
      this.updButton7.setEnabled(false);
    }

    if (this.updButton8 != null)
    {
      this.updButton8.setEnabled(false);
    }

    if (this.updButton9 != null)
    {
      this.updButton9.setEnabled(false);
    }

    if (this.updButton10 != null)
    {
      this.updButton10.setEnabled(false);
    }

    if (this.addButton1 != null)
    {
      this.addButton1.setEnabled(true);
      this.addButton.setEnabled(true);
    }

    if (this.addButton2 != null)
    {
      this.addButton2.setEnabled(true);
    }

    if (this.addButton3 != null)
    {
      this.addButton3.setEnabled(true);
    }

    if (this.addButton4 != null)
    {
      this.addButton4.setEnabled(true);
    }

    if (this.addButton5 != null)
    {
      this.addButton5.setEnabled(true);
    }

    if (this.addButton6 != null)
    {
      this.addButton6.setEnabled(true);
    }

    if (this.addButton7 != null)
    {
      this.addButton7.setEnabled(true);
    }

    if (this.addButton8 != null)
    {
      this.addButton8.setEnabled(true);
    }

    if (this.addButton9 != null)
    {
      this.addButton9.setEnabled(true);
    }

    if (this.addButton10 != null)
    {
      this.addButton10.setEnabled(true);
    }

    this.rmvButton.setEnabled(false);
    this.saveButton.setEnabled(true);
    this.stepnum.setText("");
    this.update = false;
    this.addButton1.setEnabled(true);
    this.addButton.setEnabled(true);
    this.checkButton.setEnabled(false);
    for (int t = 0; t < this.tabcount; t++)
      this.jTabbedPane1.setEnabledAt(t, true);
    pkEnableDisable();
    resetScreens();
    //Return to Splash screen
    Splash();

  }

  private void loadCFG()
  {
    String filename = "C:/Program Files/TELOS/AutoPilot/Config/TelosCFG.cfg";
    File f = new File(filename);
    int index = 0;
    int len = 0;
    int count2 = 0;
    int next = 0;

//    File directoryOfLabware = new File("C:\\Program Files\\HAMILTON\\LabWare\\APcustom"); //List of all files 
    
    File directoryOfLabware = new File(VASengine.BitPath+"HAMILTON\\LabWare\\APcustom");
    FilenameFilter FilterRCK = new FilenameFilter()
    {
      public boolean accept(File dir, String name)
      {
        String lowercaseName = name.toLowerCase();

        return (lowercaseName.contains(".rck")) || (lowercaseName.contains(".ctr"));
      }
    };

    if (directoryOfLabware.exists())
    {
      this.arrayLabware = directoryOfLabware.list(FilterRCK);

      for (int i = 0; i < this.arrayLabware.length; i++)
      {
        if (this.arrayLabware[i].contains("_24.rck"))							//AP-0001
        {
          this.arrayPlates24.add(this.arrayLabware[i].replace(".rck", ""));
          this.p24count += 1;
        }
          
        if (this.arrayLabware[i].contains("_48.rck"))							//AP-0001
        {
          this.arrayPlates48.add(this.arrayLabware[i].replace(".rck", ""));
          this.p48count += 1;
        }
          
        if (this.arrayLabware[i].contains("_96.rck"))
        {
          this.arrayPlates96.add(this.arrayLabware[i].replace(".rck", ""));
          this.p96count += 1;
        }

        if (this.arrayLabware[i].contains("_384.rck"))
        {
          this.arrayPlates384.add(this.arrayLabware[i].replace(".rck", ""));
          this.p384count += 1;
        }

        if (this.arrayLabware[i].contains("X.rck"))
        {
          this.arrayReservoirs.add(this.arrayLabware[i].replace(".rck", ""));
          this.rcount += 1;
        }

        if (!this.arrayLabware[i].contains("Tube.ctr"))
          continue;
        this.arrayTubes.add(this.arrayLabware[i].replace(".ctr", ""));
        this.tcount += 1;
      }

      this.plates24 = new String[this.p24count];	//AP-0001
      this.arrayPlates24.toArray(this.plates24);	//AP-0001
      this.plates48 = new String[this.p48count];	//AP-0001
      this.arrayPlates48.toArray(this.plates48);	//AP-0001
      this.plates96 = new String[this.p96count];
      this.arrayPlates96.toArray(this.plates96);
      this.plates384 = new String[this.p384count];
      this.platesAll = new String[this.p24count + this.p48count + this.p96count + this.p384count + 1];
      this.plates96tubes = new String[this.p96count + this.tcount + 1];
      this.arrayPlates384.toArray(this.plates384);
      pALLcount = 0;									//AP-0001
      platesAll[0] = "Plate type" ;
	  pALLcount ++;
      
      for (int n = 0; n < this.plates24.length; n++)	//AP-0001
      {
    	  platesAll[pALLcount] = plates24[n];
    	  pALLcount ++;
      }
      for (int n = 0; n < this.plates48.length; n++)	//AP-0001
      {
    	  platesAll[pALLcount] = plates48[n];
    	  pALLcount ++;
      }
      for (int n = 0; n < this.plates96.length; n++)
      {
    	  platesAll[pALLcount] = plates96[n];
    	  pALLcount ++;
      }
      for (int n = 0; n < this.plates384.length; n++)
      {
    	  platesAll[pALLcount] = plates384[n];
    	  pALLcount ++;
      }
      
//    this.arrayPlates384.toArray(this.plates384);

      this.reservoirs = new String[this.rcount];
      this.arrayReservoirs.toArray(this.reservoirs);
      this.tubes = new String[this.tcount];
      this.arrayTubes.toArray(this.tubes);
      this.platesTubes = new String[this.p24count + this.p48count + this.p96count + this.p384count + tcount + 1];
      patcount = 0;									//AP-0001
      platesTubes[0] = "Select labware type" ;
	  patcount ++;
      
      for (int n = 0; n < this.plates24.length; n++)	//AP-0001
      {
    	  platesTubes[patcount] = plates24[n];
    	  patcount ++;
      }
      for (int n = 0; n < this.plates48.length; n++)	//AP-0001
      {
    	  platesTubes[patcount] = plates48[n];
    	  patcount ++;
      }
      for (int n = 0; n < this.plates96.length; n++)
      {
    	  platesTubes[patcount] = plates96[n];
    	  patcount ++;
      }
      for (int n = 0; n < this.plates384.length; n++)
      {
    	  platesTubes[patcount] = plates384[n];
    	  patcount ++;
      }
      for (int n = 0; n < this.tubes.length; n++)
      {
    	  platesTubes[patcount] = tubes[n];
    	  patcount ++;
      }
      pat96count = 0;									//AP-0001
      plates96tubes[0] = "Select labware type" ;
	  pat96count ++;
      
      for (int n = 0; n < this.plates96.length; n++)
      {
    	  plates96tubes[pat96count] = plates96[n];
    	  pat96count ++;
      }
      for (int n = 0; n < this.tubes.length; n++)
      {
    	  plates96tubes[pat96count] = tubes[n];
    	  pat96count ++;
      }
      this.tubesAll = new String[tcount + 1];
      tallcount = 0;									//AP-0001
      tubesAll[0] = "Select tube type" ;				//APN0045
	  tallcount ++;
      
      for (int n = 0; n < this.tubes.length; n++)
      {
    	  tubesAll[tallcount] = tubes[n];
    	  tallcount ++;
      }

      this.reservoirsAll = new String[this.rcount + 1];
      rallcount = 0;									//AP-0001
      reservoirsAll[0] = "Select Reservoir type" ;		////APN0045
	  rallcount ++;
      
      for (int n = 0; n < this.reservoirs.length; n++)
      {
    	  reservoirsAll[rallcount] = reservoirs[n];
    	  rallcount ++;
      }

      if (this.rcount > 0)
      {
        this.showres = true;
      }
      else
      {
        this.showres = false;
      }
      if (this.tcount > 0)
      {
        this.showtube = true;
      }
      else
      {
        this.showtube = false;
      }
    }else
    {
    	JOptionPane.showMessageDialog(null, "The AutoPilot labware directory could not be found");
    	ExitApp();
    }

    try
    {
      String data = "";
      this.input = new FileInputStream(f);
      BufferedReader bw = new BufferedReader(new FileReader(f));

      data = bw.readLine();
      len = data.length();

      while ((index >= 0) && (index < len)) {
        if (this.tabcount == 0)
        {
          index = data.indexOf(',');
        }
        else
        {
          index = data.indexOf(',', index + 1);
        }
        this.tabcount += 1;
      }
            
      this.tabs = new String[this.tabcount];
      index = 0;

      this.tabs = data.split(",");
      
//      while ((index >= 0) && (index < len) && (count2 < this.tabcount)) {
//        if (count2 == 0)
//        {
//          if (data.contains(",")){index = data.indexOf(',');}else {index = data.length();}  //If only one record then there won't be a ","
//          this.tabs[count2] = data.substring(0, index);  
//        }
//        else
//        {
//          next = index;
//          index = data.indexOf(',', index + 1);
//          if (index > 0)
//          {
//            this.tabs[count2] = data.substring(next + 1, index);
//          }
//          else
//          {
//            this.tabs[count2] = data.substring(next + 1, len);
//          }
//        }
//        count2++;
//      }

      data = bw.readLine();

      data = bw.readLine();

      data = bw.readLine();
      len = data.length();
      index = 0;

      while ((index >= 0) && (index < len)) {
        if (this.ptcount == 0)
        {
          index = data.indexOf(',');
        }
        else
        {
          index = data.indexOf(',', index + 1);
        }
        this.ptcount += 1;
      }
      this.platetypes = new String[this.ptcount];
      index = 0;
      count2 = 0;
      while ((index >= 0) && (index < len) && (count2 < this.ptcount)) {
        if (count2 == 0)
        {
          index = data.indexOf(',');
          this.platetypes[count2] = data.substring(0, index);
        }
        else
        {
          next = index;
          index = data.indexOf(',', index + 1);
          if (index > 0)
          {
            this.platetypes[count2] = data.substring(next + 1, index);
          }
          else
          {
            this.platetypes[count2] = data.substring(next + 1, len);
          }
        }
        count2++;
      }

      this.deckpositions = new String[this.ptcount];
      deckpositions[0] = "Pos";
      for (int x = 0;x < ptcount - 1; x++)
      {
    	  deckpositions[x+1] = platetypes[x+1];
      }
      data = bw.readLine();
      len = data.length();
      index = 0;

      while ((index >= 0) && (index < len)) {
        if (this.fcount == 0)
        {
          index = data.indexOf(',');
        }
        else
        {
          index = data.indexOf(',', index + 1);
        }
        this.fcount += 1;
      }
      this.filters = new String[this.fcount];
      index = 0;
      count2 = 0;
      while ((index >= 0) && (index < len) && (count2 < this.fcount)) {
        if (count2 == 0)
        {
          if (data.contains(",")){index = data.indexOf(',');}else {index = data.length();}  //If only one record then there won't be a ","
          this.filters[count2] = data.substring(0, index);
        }
        else
        {
          next = index;
          index = data.indexOf(',', index + 1);
          if (index > 0)
          {
            this.filters[count2] = data.substring(next + 1, index);
          }
          else
          {
            this.filters[count2] = data.substring(next + 1, len);
          }
        }
        count2++;
      }

      data = bw.readLine();

      data = bw.readLine();

      data = bw.readLine();
      len = data.length();
      index = 0;

/////////////////////////////////////////////////////////////////////////////////////////  APF0019
//      while ((index >= 0) && (index < len)) {
//        if (this.lcount == 0)
//        {
//          index = data.indexOf(',');
//        }
//        else
//        {
//          index = data.indexOf(',', index + 1);
//        }
//        this.lcount += 1;
//      }
//      this.liqtypes = new String[this.lcount];
//      index = 0;
//      count2 = 0;
//      while ((index >= 0) && (index < len) && (count2 < this.lcount)) {
//        if (count2 == 0)
//        {
//          if (data.contains(",")){index = data.indexOf(',');}else {index = data.length();}  //If only one record then there won't be a ","
//          this.liqtypes[count2] = data.substring(0, index);
//        }
//        else
//        {
//          next = index;
//          index = data.indexOf(',', index + 1);
//          if (index > 0)
//          {
//            this.liqtypes[count2] = data.substring(next + 1, index);
//          }
//          else
//          {
//            this.liqtypes[count2] = data.substring(next + 1, len);
//          }
//        }
//        count2++;
//      }      
      this.liqtypes = VAStools.AP_LiquidClassesFromHam(VASengine.BitPath + "\\HAMILTON\\Config\\ML_STARLiquids.mdb");	//APF0019
      this.lcount = liqtypes.length;																					//APF0019 
//////////////////////////////////////////////////////////////////////////////////////// END APF0019
      
      
      //CHILL - obsolete
      data = bw.readLine();
      len = data.length();
      index = 0;
      index = 0;
      count2 = 0;
      do {
        if (count2 == 0)
        {
          index = data.indexOf(',');
          String c = data.substring(0, index);
          if (c.equals("chill"))
          {
            this.showchill = true;
          }
        }
        if (count2 == 1)
        {
          next = index;
          index = data.indexOf(',', index + 1);
          if (index > 0)
          {
            String sh = data.substring(next + 1, index);
            if (sh.equals("shake"))
            {
              this.showshake = true;
            }
          }
          else
          {
            String sh = data.substring(next + 1, len);
            if (sh.equals("shake"))
            {
              this.showshake = true;
            }
          }
        }
        count2++;

        if (index < 0) break; 
      }while (index < len);
      
      data = bw.readLine();																			//Tips
      data = bw.readLine();																			//Pipetting Hardware
      if (bw.readLine().split(",")[1].equals("1")){security = true;}else{security = false;} 		//Security  APN0049          
      
    }
    catch (IOException e)
    {
      JOptionPane.showMessageDialog(this, 
        "Error Opening File", 
        "Error", 
        0);
    }
    try
    {
      this.input.close();
    } catch (IOException e) {
      JOptionPane.showMessageDialog(this, 
        "Error Closing File", 
        "Error", 
        0);
    }
    
    this.tabcount += 1; //account for addition of "Splash" tab which is not included in config file
  }

  public void buttonSetEnable1(){
	  if (security){
		  loadButton.setEnabled(true);
		  saveButton.setEnabled(true);
		  saveAsButton.setEnabled(true);
		  newButton.setEnabled(true);
		  layoutButton.setEnabled(true);
		  layoutSelectButton.setEnabled(true);	
		  
		  if (nonPK > 0 || cntPK > 0)
		  {
			  runButton.setEnabled(true);
		  }
		  pkEnableDisable(); 
		  
		  if (security_accessLevel.equals("1") && security_Validated == true){			//APN0051
			  saveButton.setEnabled(false);												//APN0051
			  saveAsButton.setEnabled(false);											//APN0051
			  layoutSelectButton.setEnabled(false);										//APN0051
		  }																				//APN0051
		  
		  if (security_accessLevel.equals("3")){					//admin
			  securityButton.setEnabled(true);
			  securityAccountButton.setEnabled(true);
		  }else if(security_accessLevel.equals("2")){				//developer
			  securityButton.setEnabled(true);
		  }else{
			  //no additional functionality required for operator	//operator
		  }
	  }else{
		  loadButton.setEnabled(true);
		  saveButton.setEnabled(true);
		  saveAsButton.setEnabled(true);
		  newButton.setEnabled(true);
		  //addButton.setEnabled(true);
		  //checkButton.setEnabled(true);
		  //rmvButton.setEnabled(true);
		  layoutButton.setEnabled(true);
		  layoutSelectButton.setEnabled(true);	
		  
		  if (nonPK > 0 || cntPK > 0)
		  {
			  runButton.setEnabled(true);
		  }
		  
		  pkEnableDisable();  
	  }
	  

  }
  
  public void buttonSetDisable1(){
	  loadButton.setEnabled(false);
	  saveButton.setEnabled(false);
	  saveAsButton.setEnabled(false);
	  newButton.setEnabled(false);
	  addButton.setEnabled(false);
	  checkButton.setEnabled(false);
	  rmvButton.setEnabled(false);
	  layoutButton.setEnabled(false);
	  layoutSelectButton.setEnabled(false);
	  runButton.setEnabled(false);
	  if (security){securityButton.setEnabled(false);securityAccountButton.setEnabled(false);}
  }
  
  public void buttonSetEnable2(){
	  if (security){
		  loadButton.setEnabled(true);
		  saveButton.setEnabled(true);
		  saveAsButton.setEnabled(true);
		  newButton.setEnabled(true);
		  layoutButton.setEnabled(true);
		  layoutSelectButton.setEnabled(true);
		  runButton.setEnabled(true);
		  if (security_accessLevel.equals("3")){						//admin
			  securityButton.setEnabled(true);
			  securityAccountButton.setEnabled(true);
		  }else if (security_accessLevel.equals("2")){					//developer
			  securityButton.setEnabled(true);
		  }else{
			  //no additional functionality required for operator		//operator
		  }
	  }else{
		  loadButton.setEnabled(true);
		  saveButton.setEnabled(true);
		  saveAsButton.setEnabled(true);
		  newButton.setEnabled(true);
		  //addButton.setEnabled(true);
		  //checkButton.setEnabled(true);
		  //rmvButton.setEnabled(true);
		  layoutButton.setEnabled(true);
		  layoutSelectButton.setEnabled(true);
		  runButton.setEnabled(true);		  
	  }

  }
  
  public void buttonSetDisable2(){
	  loadButton.setEnabled(false);
	  saveButton.setEnabled(false);
	  saveAsButton.setEnabled(false);
	  newButton.setEnabled(false);
	  addButton.setEnabled(false);
	  checkButton.setEnabled(false);
	  //rmvButton.setEnabled(false);
	  layoutButton.setEnabled(false);
	  layoutSelectButton.setEnabled(false);
	  runButton.setEnabled(false);
	  if (security){securityButton.setEnabled(false);securityAccountButton.setEnabled(false);}
  }
  public void buttonSetEnable3(){
	  if (security){
		  loadButton.setEnabled(true);
		  saveButton.setEnabled(true);
		  saveAsButton.setEnabled(true);
		  newButton.setEnabled(true);
		  layoutButton.setEnabled(true);
		  layoutSelectButton.setEnabled(true);
		  runButton.setEnabled(true); 
		  if (security_accessLevel.equals("3")){					//admin
			  securityButton.setEnabled(true);
			  securityAccountButton.setEnabled(true);
		  }else if (security_accessLevel.equals("2")){				//developer
			  securityButton.setEnabled(true);
		  }else{
			//no additional functionality required for operator		//operator
		  }
	  }else{
		  loadButton.setEnabled(true);
		  saveButton.setEnabled(true);
		  saveAsButton.setEnabled(true);
		  newButton.setEnabled(true);
		  //addButton.setEnabled(true);
		  //checkButton.setEnabled(true);
		  //rmvButton.setEnabled(true);
		  layoutButton.setEnabled(true);
		  layoutSelectButton.setEnabled(true);
		  runButton.setEnabled(true);  
	  }
	  

  }
  public void buttonSetDisable3(){
	  //loadButton.setEnabled(false);
	  saveButton.setEnabled(false);
	  saveAsButton.setEnabled(false);
	  newButton.setEnabled(false);
	  addButton.setEnabled(false);
	  checkButton.setEnabled(false);
	  rmvButton.setEnabled(false);
	  layoutButton.setEnabled(false);
	  //layoutSelectButton.setEnabled(false);
	  runButton.setEnabled(false);
	  if (security){securityButton.setEnabled(false);securityAccountButton.setEnabled(false);}
  }
  
  public void buttonSetDisable4(){
	  loadButton.setEnabled(false);
	  saveButton.setEnabled(false);
	  saveAsButton.setEnabled(false);
	  newButton.setEnabled(false);
	  addButton.setEnabled(false);
	  checkButton.setEnabled(false);
	  rmvButton.setEnabled(false);
	  layoutButton.setEnabled(false);
	  layoutSelectButton.setEnabled(false);
	  LogOffButton.setEnabled(false);
	  securityButton.setEnabled(false);
	  securityAccountButton.setEnabled(false);
	  runButton.setEnabled(false);	  
  }
  
  public static String[] returnltypes()
  {
    return liqtypes;
  }

  public static String[] return96plates()
  {
    return plates96;
  }
 
  public int p24Search(String[] array, String key)
  {
    for (int n = 0; n < this.plates24.length; n++)
      if (array[n].equals(key))
        return n;
    return -1;
  }
  public int p48Search(String[] array, String key)
  {
    for (int n = 0; n < this.plates48.length; n++)
      if (array[n].equals(key))
        return n;
    return -1;
  }

  public int p96Search(String[] array, String key)
  {
    for (int n = 0; n < this.plates96.length; n++)
      if (array[n].equals(key))
        return n;
    return -1;
  }

  public int p384Search(String[] array, String key)
  {
    for (int n = 0; n < this.plates384.length; n++)
      if (array[n].equals(key))
        return n;
    return -1;
  }
  public int pAllSearch(String[] array, String key)
  {
    for (int n = 0; n < this.platesAll.length; n++)
      if (array[n].equals(key))
        return n;
    return -1;
  }
  
  public int p96tubeSearch(String[] array, String key)
  {
    for (int n = 0; n < this.plates96tubes.length; n++)
      if (array[n].equals(key))
        return n;
    return -1;
  }
  
  public int pPATSearch(String[] array, String key)
  {
    for (int n = 0; n < this.platesTubes.length; n++)
      if (array[n].equals(key))
        return n;
    return -1;
  }
  

  public int rSearch(String[] array, String key)
  {
    for (int n = 0; n < this.reservoirs.length; n++)
      if (array[n].equals(key))
        return n;
    return -1;
  }

  public int tSearch(String[] array, String key)
  {
    for (int n = 0; n < this.tubes.length; n++)
      if (array[n].equals(key))
        return n;
    return -1;
  }

  public int tAllSearch(String[] array, String key)
  {
    for (int n = 0; n < this.tubesAll.length; n++)
      if (array[n].equals(key))
        return n;
    return -1;
  }

  public int rAllSearch(String[] array, String key)
  {
    for (int n = 0; n < this.reservoirsAll.length; n++)
      if (array[n].equals(key))
        return n;
    return -1;
  }

  public int lSearch(String[] array, String key)
  {
    for (int n = 0; n < this.liqtypes.length; n++)
      if (array[n].equals(key))
        return n;
    return -1;
  }

  public int pSearch(String[] array, String key)
  {
    for (int n = 0; n < this.platetypes.length; n++)
      if (array[n].equals(key))
        return n;
    return -1;
  }

  public int fSearch(String[] array, String key)
  {
    for (int n = 0; n < this.filters.length; n++)
      if (array[n].equals(key))
        return n;
    return -1;
  }

  public int tabSearch(String[] array, String key)
  {
    for (int n = 0; n < this.tabs.length; n++){
    	
    	if (array[n].equals(key)){
    		 return n;
    	}
    }

    return -1;
  }
  
  //Method to force exit of application
  public static void ExitApp()
  {
	  //JOptionPane.showMessageDialog(null, Thread.currentThread().getStackTrace());
	  loginWindow.setAlwaysOnTop(false);
	  JOptionPane.showMessageDialog(null, "AutoPilot will abort!");
	  System.exit(1);
  }
  
  //Method to force exit of application
  public void pkEnableDisable()
  {
	  try {
		if (PKmod){
			  for (int t = 0; t < this.tabcount; t++)
			  {
			      this.jTabbedPane1.setEnabledAt(t, true);
			  }
			  if (nonPK > 0 || cntPK > 0)
			  {
			  this.jTabbedPane1.setEnabledAt(jTabbedPane1.indexOfTab("PK  "), false);	
			  }
			  
			  if (nonPK == 0 && cntPK < 1)
			  {
			      this.jTabbedPane1.setEnabledAt(jTabbedPane1.indexOfTab("PK  "), true);	
			  }
			  
			  if (cntPK > 0)
			  {
			      for (int t = 0; t < this.tabcount; t++)
			      {
			      this.jTabbedPane1.setEnabledAt(t, false);
			      }
			  }
		  }
	} catch (Exception e) {
		// TODO Auto-generated catch block
		e.printStackTrace();
	}
  }
  
  private String removeWhiteSpace(String s)
  {
	  String stringIn = s;
	  String stringTrim = stringIn.trim();
	  String stringOut = stringTrim.replaceAll("\\s", "");
	  return stringOut;
  }
  
  private void auditUpdate(String action, String reason)
  {
	  String version = "";
      if (security){	  
    	  if (security_auditTrailString.equals("-")){
    		  security_auditTrailString = "AT0";	//Create new string if this is the first time string has been updated.  0 = not validated
    		  version = "1";
    	  }else{
    		  
    		  version = String.valueOf((Integer.valueOf(security_auditTrailString.split(";")[security_auditTrailString.split(";").length-1])) + 1);
    	  }

    	  //DateTime	User	Action	ReasonForMod	Version
	 	  String writeString = VAStools.dateTime() + ";" + security_userLoggedIn + ";" + action + ";" + reason + ";" + version; 
		  security_auditTrailString = security_auditTrailString + "," + writeString;		//DateTime;Author;Action  
      }
  }

  private String cvtToCsv(String S){
	  	boolean valid = true;
	  	String Working = S;
		String Final = "";
		String Range = "";
		
		Working = Working.replaceAll("\\s","");	//Remove White Spaces
		if (Working.contains("--") || Working.contains(",,") || Working.equals("") || Working.equals(null)){valid = false;}	//Confirm no double ",," or "--", or "" or null
		else{
			Scanner scanner = new Scanner(Working);        
		    String validationResult = scanner.findInLine("[^0-9,\\-]+");	//Only allow 0-9 AND , AND -
		    if (validationResult != null) {valid = false;}
			scanner.close();
	
			String[] arraySplitComma = Working.split(",");	
			for (int a = 0; a < arraySplitComma.length; a++){
				
				if (arraySplitComma[a].contains("-")){																			//Dash detected
					Range = "";
					try {
						String[] arraySplitDash = arraySplitComma[a].split("-");
						
						if (arraySplitDash[0].equals("") || arraySplitDash[1].equals("")){valid = false;}						//dash must be flanked by numbers
						else if (Integer.parseInt(arraySplitDash[0]) > Integer.parseInt(arraySplitDash[1])){valid = false;}		//first number of range must be less than second number
						else {
							for (int i = Integer.parseInt(arraySplitDash[0]); i < Integer.parseInt(arraySplitDash[1])+1; i++){	//Convert to comma seperated string
								Range = Range + String.valueOf(i) + ",";
							}
							Final = Final + Range;
						}
					} catch (NumberFormatException e) {
						// TODO Auto-generated catch block
						valid = false;
						e.printStackTrace();
					}
				}else{Final = Final + arraySplitComma[a] + ",";}
				
			}
			try {
				Final = Final.substring(0, Final.length() - 1);
			} catch (Exception e) {
				valid = false;
				e.printStackTrace();
			}			
		}

		if(valid){
			return Final;
		}else {
			return "fail";
		}
  }
  
  class DirectoryRestrictedFileSystemView extends FileSystemView
  {
      private final File[] rootDirectories;

      DirectoryRestrictedFileSystemView(File rootDirectory)
      {
          this.rootDirectories = new File[] {rootDirectory};
      }

      DirectoryRestrictedFileSystemView(File[] rootDirectories)
      {
          this.rootDirectories = rootDirectories;
      }

      @Override
      public File createNewFolder(File containingDir) throws IOException
      {       
          throw new UnsupportedOperationException("Unable to create directory");
      }

      @Override
      public File[] getRoots()
      {
          return rootDirectories;
      }

      @Override
      public boolean isRoot(File file)
      {
          for (File root : rootDirectories) {
              if (root.equals(file)) {
                  return true;
              }
          }
          return false;
      }
      
      @Override
      public File getHomeDirectory()
      {
        return rootDirectories[0];
      }
  }
  
  private JFileChooser getJFileChooser5()
  {
	  jFileChooser5 = null;
	  File[] array_file = new File[directorySelect.arrayList_directoryData.size()];

	  for (int i=0; i < directorySelect.arrayList_directoryData.size(); i++){
		  array_file[i] = new File(directorySelect.arrayList_directoryData.get(i));
	  }
	  
	  if (this.jFileChooser5 == null) {
    	
      FileSystemView fsv = new DirectoryRestrictedFileSystemView(array_file);
 
      
      if (security){
    	  this.jFileChooser5 = new JFileChooser(fsv.getHomeDirectory(),fsv);
      }else{
    	  this.jFileChooser5 = new JFileChooser();
      }
     
      jFileChooser5.setDialogTitle("Select Worklist");
      this.jFileChooser5.setApproveButtonText("Select");
      this.jFileChooser5.setMultiSelectionEnabled(false);
      this.jFileChooser5.setAcceptAllFileFilterUsed(false);
      FileFilter filter3 = new textFileFilter(".txt or .pdf or .xls", new String[] { "TXT","PDF","XLS" });
      this.jFileChooser5.setFileFilter(filter3);
      
      jFileChooser5.approveSelection();
    }
    return this.jFileChooser5;
  }
  
}
